/**
  @file body.h

  @brief Relationships between parameters associated with individual bodies.

  @author Rory Barnes ([RoryBarnes](https://github.com/RoryBarnes/))

  @date May 7 2014

*/

#include <math.h>

// Mass-Radius identifiers
#define REIDHAWLEY 0
#define GORDASVECH99 1
#define BAYLESSOROSZ06 2
#define SOTIN07 3

// Array indices and constants for
// the Proxima Centauri stellar evolution grid
#define PROXIMACEN_T 1
#define PROXIMACEN_L 2
#define PROXIMACEN_R 3
#define PROXIMACEN_ERROR -4
#define PROXIMACEN_ALEN 500
#define PROXIMACEN_MLEN 2
// This fudge gives the correct present-day luminosity
// for Proxima Centauri for a stellar mass of 0.123MSUN
// according to the data in Demory et al. (2009)
#define PROXIMACEN_FUDGE 1.02910909837

// Array sizes for Proxima Cen b radius grid
#define PROXCENBCOMPLEN 7
#define PROXCENBTIMELEN 50

// Array sizes for the Lopez et al. (2012) radius grid
#define MASSLEN 6
#define COMPLEN 12
#define FLUXLEN 3
#define METLEN 2
#define TIMELEN 50

// Constants/array lens for Baraffe model
#define STELLAR_T 1  // Effective Temperature
#define STELLAR_L 2  // Luminosity
#define STELLAR_R 3  // Radius
#define STELLAR_RG 4 // Radius of Gyration
#define STELLAR_ERR_LINEAR 1
#define STELLAR_ERR_NONE 0
#define STELLAR_ERR_OUTOFBOUNDS_LO -2
#define STELLAR_ERR_OUTOFBOUNDS_HI -3
#define STELLAR_ERR_ISNAN -4
#define STELLAR_ERR_FILE -6
#define STELLAR_ERR_BADORDER -7
#define STELLAR_BAR_MLEN 25
#define STELLAR_BAR_ALEN 502

/* @cond DOXYGEN_OVERRIDE */

int fiSign(double);
double fdFreqToPer(double);
double fdTimescale(double, double);
double fdTimescaleMulti(double, double *, int);
double fdPerToFreq(double);
double fdRotVel(double, double);
double fdDPerDt(double, double);

double fdBodyPotEnergy(double, double);
double fdRotAngMom(double, double, double, double);
double fdRotKinEnergy(double, double, double, double);

double fdDensityMassToRadius(double, double);
double fdMassFromRadiusDensity(double, double);
double fdMassToRad(double, int);
double fdRadToMass(double, int);
double fdRadiusFreqToRotVel(double, double);
double fdRadiusRotVelToFreq(double, double);
double fdDensityMassToRadius(double, double);
double fdMassFromRadiusDensity(double, double);
double fdSphereDensity(double, double);

double fdMassToRad_BaylessOrosz06(double);
double fdMassToRad_GordaSvech99(double);
double fdMassToRad_ReidHawley(double);
double fdMassToRad_Sotin07(double);
double fdRadToMass_BaylessOrosz06(double);
double fdRadToMass_GordaSvech99(double);
double fdRadToMass_ReidHawley(double);
double fdRadToMass_Sotin07(double);
double fdMassToRad_LehmerCatling17(double);
double fdLehmerRadius(BODY *, int, int);
double fdLehmerPres(double, double, double);
double fdThermalTemp(BODY *, int);
double fdEqH2AtmosphereSoundSpeed(double, double, double);

double fdImK2Total(BODY *, int);
double fdImK2Man(BODY *, int);
double fdK2Man(BODY *, int);
double fdTidalQMan(BODY *, int);
double fdImK2ManThermint(BODY *, int);
void AssignTidalProperties(BODY *, EVOLVE *, int);
double fdHflowSecMan(BODY *, EVOLVE *, int);

void BodyCopy(BODY *, BODY *, EVOLVE *);

void CalcXYZobl(BODY *, int);
double CalcDynEllipEq(BODY *, int);

void fdHabitableZoneKopparapu2013(BODY *, int, double *);

double fdEffectiveTemperature(BODY*,int);

// RB: Move

// Proxima Centauri properties
double fdProximaCenStellar(int, double, double, int *);

// Proxima Centauri b radius evolution
double fdProximaCenBRadius(double, double, double);

// Lopez et al. (2012) radius evolution
double fdLopezRadius(double, double, double, double, int);

// Proxima Cen b radius evolution grid
static double const daProxCenBComp[] = {0.00001,     0.000100100, 0.000200000,
                                        0.000500000, 0.00100000,  0.00200000,
                                        0.00500000,  0.0100000};
static double const daProxCenBAge[]  = {
      1.0000000E+07, 1.1513954E+07, 1.3257114E+07, 1.5264180E+07,
      1.7575106E+07, 2.0235896E+07, 2.3299518E+07, 2.6826958E+07,
      3.0888436E+07, 3.5564803E+07, 4.0949151E+07, 4.7148664E+07,
      5.4286754E+07, 6.2505519E+07, 7.1968567E+07, 8.2864277E+07,
      9.5409548E+07, 1.0985411E+08, 1.2648552E+08, 1.4563485E+08,
      1.6768329E+08, 1.9306977E+08, 2.2229965E+08, 2.5595479E+08,
      2.9470517E+08, 3.3932218E+08, 3.9069399E+08, 4.4984327E+08,
      5.1794747E+08, 5.9636233E+08, 6.8664885E+08, 7.9060432E+08,
      9.1029818E+08, 1.0481131E+09, 1.2067926E+09, 1.3894955E+09,
      1.5998587E+09, 1.8420700E+09, 2.1209509E+09, 2.4420531E+09,
      2.8117687E+09, 3.2374575E+09, 3.7275937E+09, 4.2919343E+09,
      4.9417134E+09, 5.6898660E+09, 6.5512856E+09, 7.5431201E+09,
      8.6851137E+09, 1.0000000E+10};
static double const daProxCenBRadius[PROXCENBCOMPLEN][PROXCENBTIMELEN] = {
      {1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000,
       1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000,
       1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000,
       1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000,
       1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000,
       1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000,
       1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000, 1.074000,
       1.074000},
      {1.463086, 1.417403, 1.459464, 1.454848, 1.449649, 1.399296, 1.433978,
       1.413519, 1.417165, 1.428509, 1.402747, 1.384987, 1.403590, 1.375102,
       1.385195, 1.374890, 1.362673, 1.323761, 1.306699, 1.277714, 1.252569,
       1.230352, 1.220339, 1.221584, 1.222095, 1.222095, 1.222095, 1.222095,
       1.222111, 1.223893, 1.223893, 1.223893, 1.224062, 1.225732, 1.225732,
       1.227059, 1.227619, 1.229901, 1.231516, 1.232917, 1.235364, 1.238102,
       1.241533, 1.247181, 1.253112, 1.258545, 1.259453, 1.260718, 1.262341,
       1.264332},
      {1.340164, 1.337498, 1.346076, 1.344281, 1.344740, 1.354108, 1.371647,
       1.379671, 1.396774, 1.393320, 1.387313, 1.356807, 1.358361, 1.363181,
       1.349616, 1.333890, 1.340613, 1.321234, 1.316514, 1.303926, 1.284028,
       1.274702, 1.275090, 1.257792, 1.244128, 1.234062, 1.231587, 1.231587,
       1.231600, 1.233385, 1.233385, 1.233385, 1.233553, 1.235224, 1.235224,
       1.236552, 1.237112, 1.239395, 1.241010, 1.242411, 1.244860, 1.247598,
       1.251031, 1.256684, 1.262615, 1.268051, 1.268973, 1.270258, 1.271907,
       1.273931},
      {1.333884, 1.324986, 1.335872, 1.332229, 1.335282, 1.341235, 1.352728,
       1.365189, 1.401351, 1.427044, 1.409745, 1.390667, 1.395495, 1.394515,
       1.394435, 1.374629, 1.380264, 1.378815, 1.374714, 1.373205, 1.354925,
       1.358028, 1.342092, 1.338170, 1.339958, 1.335360, 1.325887, 1.304005,
       1.299493, 1.287097, 1.274994, 1.261897, 1.262066, 1.263739, 1.263739,
       1.265068, 1.265627, 1.267915, 1.269531, 1.270935, 1.273386, 1.276127,
       1.279564, 1.285222, 1.291162, 1.296610, 1.297574, 1.298918, 1.300645,
       1.302760},
      {1.413774, 1.411315, 1.410963, 1.410291, 1.410811, 1.412933, 1.422476,
       1.434454, 1.455379, 1.467372, 1.454690, 1.446973, 1.456963, 1.452898,
       1.448635, 1.453498, 1.458382, 1.449156, 1.465357, 1.436860, 1.454029,
       1.448281, 1.437643, 1.445156, 1.431543, 1.429250, 1.414306, 1.408134,
       1.407909, 1.402028, 1.386891, 1.373207, 1.359784, 1.347110, 1.336101,
       1.326304, 1.313143, 1.315432, 1.317051, 1.318459, 1.320916, 1.323662,
       1.327106, 1.332775, 1.338726, 1.344193, 1.345230, 1.346672, 1.348525,
       1.350797},
      {1.662329, 1.623717, 1.593079, 1.571521, 1.557122, 1.554048, 1.555934,
       1.591067, 1.615452, 1.651527, 1.621438, 1.597991, 1.603272, 1.610057,
       1.608260, 1.616519, 1.604277, 1.589421, 1.609819, 1.605975, 1.620978,
       1.623501, 1.629322, 1.629510, 1.640833, 1.615868, 1.626902, 1.621330,
       1.609979, 1.604379, 1.604895, 1.588252, 1.597051, 1.579589, 1.580114,
       1.562533, 1.570851, 1.564879, 1.560128, 1.545140, 1.527490, 1.535571,
       1.521252, 1.523939, 1.522123, 1.520980, 1.508153, 1.509398, 1.502326,
       1.466334},
      {2.827404, 2.769879, 2.742116, 2.682535, 2.628528, 2.582153, 2.540170,
       2.517673, 2.518111, 2.496462, 2.421273, 2.342483, 2.278526, 2.232106,
       2.193505, 2.158516, 2.159635, 2.119929, 2.126924, 2.096071, 2.085289,
       2.060263, 2.051307, 2.028710, 2.009123, 1.995419, 1.979485, 1.960599,
       1.940499, 1.929413, 1.913216, 1.904773, 1.879795, 1.865157, 1.852126,
       1.837503, 1.824437, 1.821542, 1.809304, 1.793153, 1.777415, 1.757947,
       1.738880, 1.724441, 1.709415, 1.694036, 1.675065, 1.659353, 1.640337,
       1.618860}};

// Lopez et al. (2012) radius evolution grid
static double const daLopezMass[] = {1., 1.8, 3.3, 6., 11., 20.};
static double const daLopezComp[] = {
      0.000100100, 0.000200000, 0.000500000, 0.00100000, 0.00200000, 0.00500000,
      0.0100000,   0.0200000,   0.0500000,   0.100000,   0.200000,   0.500000};
static double const daLopezFlux[]  = {0.1, 1., 10.};
static double const daLopezMetal[] = {1, 50};
static double const daLopezAge[]   = {
      1.0000000E+07, 1.1513954E+07, 1.3257114E+07, 1.5264180E+07,
      1.7575106E+07, 2.0235896E+07, 2.3299518E+07, 2.6826958E+07,
      3.0888436E+07, 3.5564803E+07, 4.0949151E+07, 4.7148664E+07,
      5.4286754E+07, 6.2505519E+07, 7.1968567E+07, 8.2864277E+07,
      9.5409548E+07, 1.0985411E+08, 1.2648552E+08, 1.4563485E+08,
      1.6768329E+08, 1.9306977E+08, 2.2229965E+08, 2.5595479E+08,
      2.9470517E+08, 3.3932218E+08, 3.9069399E+08, 4.4984327E+08,
      5.1794747E+08, 5.9636233E+08, 6.8664885E+08, 7.9060432E+08,
      9.1029818E+08, 1.0481131E+09, 1.2067926E+09, 1.3894955E+09,
      1.5998587E+09, 1.8420700E+09, 2.1209509E+09, 2.4420531E+09,
      2.8117687E+09, 3.2374575E+09, 3.7275937E+09, 4.2919343E+09,
      4.9417134E+09, 5.6898660E+09, 6.5512856E+09, 7.5431201E+09,
      8.6851137E+09, 1.0000000E+10};
static double const daLopezRadius[MASSLEN][COMPLEN][FLUXLEN][METLEN][TIMELEN] =
      {{{{{1.445836, 1.385246, 1.430933, 1.400800, 1.414283, 1.441308, 1.416683,
           1.412709, 1.414727, 1.355974, 1.381037, 1.347044, 1.316127, 1.325111,
           1.310354, 1.290528, 1.234411, 1.210126, 1.178034, 1.149377, 1.120278,
           1.084456, 1.056138, 1.056323, 1.056398, 1.056398, 1.056398, 1.056398,
           1.056401, 1.056663, 1.056663, 1.056663, 1.056688, 1.056934, 1.056934,
           1.057130, 1.057213, 1.057549, 1.057786, 1.057994, 1.058355, 1.058758,
           1.059264, 1.060098, 1.060972, 1.061944, 1.063555, 1.065799, 1.068682,
           1.072218},
          {1.446733, 1.415381, 1.433767, 1.403187, 1.422971, 1.416618, 1.409167,
           1.400464, 1.366839, 1.354972, 1.365957, 1.336716, 1.320932, 1.301908,
           1.295460, 1.261211, 1.245574, 1.208695, 1.183815, 1.143053, 1.115556,
           1.073906, 1.056138, 1.056322, 1.056398, 1.056398, 1.056398, 1.056398,
           1.056402, 1.056663, 1.056663, 1.056663, 1.056687, 1.056934, 1.056934,
           1.057130, 1.057213, 1.057550, 1.057786, 1.057994, 1.058354, 1.058759,
           1.059264, 1.060097, 1.060971, 1.061944, 1.063556, 1.065800, 1.068682,
           1.072219}},
         {{1.441204, 1.405420, 1.439753, 1.422199, 1.433833, 1.401446, 1.426167,
           1.395167, 1.381877, 1.394364, 1.373727, 1.344842, 1.321353, 1.323422,
           1.317669, 1.297941, 1.253455, 1.225514, 1.204690, 1.166268, 1.129633,
           1.126133, 1.102200, 1.102334, 1.102389, 1.102389, 1.102389, 1.102389,
           1.102392, 1.102582, 1.102582, 1.102582, 1.102602, 1.102781, 1.102781,
           1.102924, 1.102984, 1.103228, 1.103401, 1.103552, 1.103814, 1.104110,
           1.104478, 1.105086, 1.105723, 1.106432, 1.107606, 1.109244, 1.111345,
           1.113923},
          {1.448756, 1.418062, 1.430631, 1.409362, 1.427162, 1.422809, 1.405228,
           1.394127, 1.381716, 1.344149, 1.350221, 1.340538, 1.323599, 1.312994,
           1.290104, 1.278357, 1.254783, 1.217682, 1.190539, 1.153483, 1.153430,
           1.123142, 1.102200, 1.102335, 1.102389, 1.102389, 1.102389, 1.102389,
           1.102393, 1.102582, 1.102582, 1.102582, 1.102600, 1.102781, 1.102781,
           1.102923, 1.102982, 1.103229, 1.103401, 1.103553, 1.103816, 1.104109,
           1.104478, 1.105086, 1.105723, 1.106433, 1.107607, 1.109243, 1.111344,
           1.113922}},
         {{1.454119, 1.396634, 1.459584, 1.406911, 1.409822, 1.427953, 1.420558,
           1.377651, 1.386389, 1.381209, 1.357336, 1.352541, 1.318090, 1.339110,
           1.364362, 1.344174, 1.315733, 1.288806, 1.277806, 1.229660, 1.205651,
           1.182393, 1.175173, 1.176514, 1.177061, 1.177061, 1.177061, 1.177061,
           1.177079, 1.178993, 1.178993, 1.178993, 1.179174, 1.180970, 1.180970,
           1.182395, 1.182997, 1.185451, 1.187185, 1.188690, 1.191323, 1.194264,
           1.197950, 1.204023, 1.210396, 1.216234, 1.217199, 1.218540, 1.220266,
           1.222384},
          {1.448557, 1.393896, 1.441714, 1.436572, 1.430770, 1.374686, 1.424563,
           1.393142, 1.379153, 1.389949, 1.376695, 1.337956, 1.364094, 1.338005,
           1.352433, 1.350212, 1.335755, 1.291508, 1.270629, 1.241909, 1.209630,
           1.187717, 1.175173, 1.176512, 1.177061, 1.177061, 1.177061, 1.177061,
           1.177081, 1.178993, 1.178993, 1.178993, 1.179175, 1.180970, 1.180970,
           1.182396, 1.182997, 1.185451, 1.187185, 1.188692, 1.191322, 1.194263,
           1.197951, 1.204022, 1.210396, 1.216233, 1.217198, 1.218542, 1.220266,
           1.222381}}},
        {{{1.250829, 1.265759, 1.265252, 1.299140, 1.278582, 1.270103, 1.287215,
           1.283894, 1.249912, 1.275734, 1.259933, 1.260464, 1.249243, 1.235320,
           1.234898, 1.223572, 1.215308, 1.198074, 1.182327, 1.184213, 1.163851,
           1.142504, 1.133669, 1.113676, 1.090774, 1.068521, 1.065405, 1.065405,
           1.065408, 1.065674, 1.065674, 1.065674, 1.065701, 1.065950, 1.065950,
           1.066150, 1.066234, 1.066576, 1.066819, 1.067029, 1.067398, 1.067809,
           1.068323, 1.069171, 1.070061, 1.071052, 1.072693, 1.074979, 1.077914,
           1.081518},
          {1.253413, 1.251410, 1.251639, 1.263134, 1.257946, 1.263517, 1.268779,
           1.259174, 1.255489, 1.237226, 1.257098, 1.243946, 1.214987, 1.236036,
           1.202411, 1.213728, 1.203934, 1.189852, 1.173997, 1.162157, 1.141176,
           1.132625, 1.116521, 1.096706, 1.078970, 1.065405, 1.065405, 1.065405,
           1.065407, 1.065674, 1.065674, 1.065674, 1.065701, 1.065950, 1.065950,
           1.066151, 1.066235, 1.066576, 1.066819, 1.067029, 1.067397, 1.067809,
           1.068324, 1.069171, 1.070062, 1.071053, 1.072693, 1.074979, 1.077915,
           1.081516}},
         {{1.329659, 1.332976, 1.320571, 1.299735, 1.343461, 1.339672, 1.353848,
           1.332617, 1.350116, 1.334095, 1.369848, 1.328048, 1.296603, 1.325537,
           1.301410, 1.276506, 1.272913, 1.258612, 1.273842, 1.228945, 1.220710,
           1.198458, 1.175988, 1.145775, 1.166283, 1.161227, 1.130378, 1.112156,
           1.112157, 1.112349, 1.112349, 1.112349, 1.112368, 1.112547, 1.112547,
           1.112690, 1.112750, 1.112995, 1.113167, 1.113318, 1.113582, 1.113876,
           1.114243, 1.114850, 1.115488, 1.116196, 1.117369, 1.119004, 1.121105,
           1.123679},
          {1.253070, 1.261416, 1.235982, 1.251338, 1.274869, 1.249977, 1.250716,
           1.239113, 1.276512, 1.261321, 1.270515, 1.254722, 1.219431, 1.241252,
           1.240191, 1.198599, 1.210267, 1.205850, 1.176134, 1.181441, 1.156025,
           1.168974, 1.163074, 1.145093, 1.124400, 1.112156, 1.112156, 1.112156,
           1.112158, 1.112349, 1.112349, 1.112349, 1.112368, 1.112547, 1.112547,
           1.112690, 1.112750, 1.112996, 1.113167, 1.113319, 1.113581, 1.113875,
           1.114243, 1.114851, 1.115488, 1.116196, 1.117369, 1.119005, 1.121104,
           1.123680}},
         {{1.402342, 1.402369, 1.393643, 1.378934, 1.381433, 1.387532, 1.410252,
           1.379381, 1.375181, 1.395584, 1.430238, 1.389739, 1.438064, 1.434052,
           1.426391, 1.416022, 1.417480, 1.386433, 1.384914, 1.356734, 1.337197,
           1.316333, 1.300556, 1.277833, 1.239449, 1.233635, 1.210949, 1.193618,
           1.186789, 1.188700, 1.188700, 1.188700, 1.188882, 1.190675, 1.190675,
           1.192100, 1.192700, 1.195154, 1.196886, 1.198390, 1.201020, 1.203959,
           1.207642, 1.213710, 1.220079, 1.225915, 1.226895, 1.228263, 1.230017,
           1.232171},
          {1.299673, 1.300995, 1.308160, 1.306936, 1.306513, 1.316942, 1.340514,
           1.341000, 1.360909, 1.358684, 1.349384, 1.318791, 1.317446, 1.330527,
           1.310108, 1.289468, 1.298318, 1.276248, 1.271791, 1.264005, 1.234574,
           1.230433, 1.229335, 1.210580, 1.194862, 1.186770, 1.186770, 1.186770,
           1.186783, 1.188700, 1.188700, 1.188700, 1.188881, 1.190675, 1.190675,
           1.192101, 1.192702, 1.195152, 1.196886, 1.198390, 1.201019, 1.203959,
           1.207643, 1.213711, 1.220078, 1.225912, 1.226894, 1.228261, 1.230016,
           1.232171}}},
        {{{1.210339, 1.215959, 1.219503, 1.218501, 1.225280, 1.224650, 1.217566,
           1.219064, 1.238864, 1.258862, 1.237037, 1.247692, 1.246828, 1.229778,
           1.244333, 1.230269, 1.227552, 1.221753, 1.226308, 1.196432, 1.204061,
           1.202850, 1.194097, 1.174139, 1.179367, 1.165709, 1.150176, 1.132381,
           1.124687, 1.104471, 1.092741, 1.092741, 1.092767, 1.093031, 1.093031,
           1.093242, 1.093331, 1.093691, 1.093946, 1.094168, 1.094556, 1.094989,
           1.095533, 1.096426, 1.097366, 1.098410, 1.100138, 1.102549, 1.105645,
           1.109442},
          {1.237099, 1.230949, 1.236237, 1.231701, 1.235842, 1.236340, 1.247160,
           1.248114, 1.253684, 1.258589, 1.275894, 1.255198, 1.256399, 1.245114,
           1.264303, 1.247105, 1.255998, 1.250833, 1.223811, 1.226365, 1.220954,
           1.222575, 1.203469, 1.201089, 1.186553, 1.179738, 1.174061, 1.153489,
           1.135506, 1.124660, 1.102113, 1.092741, 1.092767, 1.093031, 1.093031,
           1.093242, 1.093331, 1.093691, 1.093946, 1.094168, 1.094556, 1.094989,
           1.095532, 1.096427, 1.097365, 1.098411, 1.100139, 1.102550, 1.105644,
           1.109441}},
         {{1.211861, 1.212021, 1.218905, 1.214511, 1.218017, 1.217320, 1.216565,
           1.230490, 1.251588, 1.221370, 1.240333, 1.228140, 1.237760, 1.223958,
           1.237103, 1.238344, 1.235559, 1.237774, 1.218439, 1.215991, 1.234850,
           1.223669, 1.214905, 1.210517, 1.200529, 1.196708, 1.184518, 1.175390,
           1.158051, 1.141686, 1.141686, 1.141686, 1.141704, 1.141882, 1.141882,
           1.142025, 1.142085, 1.142329, 1.142501, 1.142653, 1.142914, 1.143208,
           1.143576, 1.144180, 1.144815, 1.145522, 1.146694, 1.148324, 1.150420,
           1.152991},
          {1.235942, 1.244393, 1.230880, 1.246866, 1.251345, 1.232373, 1.222377,
           1.248882, 1.269167, 1.273159, 1.227101, 1.253135, 1.276949, 1.253513,
           1.239183, 1.223933, 1.258080, 1.247364, 1.213945, 1.235427, 1.217575,
           1.231131, 1.238756, 1.239025, 1.220878, 1.225986, 1.210632, 1.197780,
           1.177928, 1.166335, 1.141686, 1.141686, 1.141704, 1.141882, 1.141882,
           1.142024, 1.142085, 1.142328, 1.142501, 1.142652, 1.142913, 1.143207,
           1.143575, 1.144180, 1.144815, 1.145524, 1.146695, 1.148325, 1.150419,
           1.152988}},
         {{1.266857, 1.264102, 1.268449, 1.275951, 1.278836, 1.285446, 1.297079,
           1.305272, 1.342433, 1.368606, 1.335559, 1.341884, 1.335537, 1.325780,
           1.323221, 1.308540, 1.318469, 1.302303, 1.303864, 1.311131, 1.303288,
           1.303428, 1.296672, 1.282612, 1.277982, 1.262815, 1.255546, 1.241973,
           1.232231, 1.224661, 1.217857, 1.217857, 1.218037, 1.219827, 1.219827,
           1.221247, 1.221850, 1.224294, 1.226022, 1.227524, 1.230146, 1.233079,
           1.236756, 1.242808, 1.249160, 1.254989, 1.256021, 1.257459, 1.259307,
           1.261575},
          {1.303539, 1.289854, 1.305751, 1.301848, 1.306012, 1.312447, 1.327965,
           1.339849, 1.380307, 1.402112, 1.389947, 1.363314, 1.368072, 1.370904,
           1.367766, 1.349440, 1.358154, 1.352174, 1.348461, 1.343532, 1.323794,
           1.324476, 1.308251, 1.305400, 1.305006, 1.298884, 1.289764, 1.268981,
           1.260357, 1.246393, 1.233431, 1.217857, 1.218037, 1.219827, 1.219827,
           1.221249, 1.221847, 1.224293, 1.226022, 1.227524, 1.230146, 1.233078,
           1.236754, 1.242807, 1.249160, 1.254987, 1.256020, 1.257458, 1.259307,
           1.261572}}},
        {{{1.212106, 1.216531, 1.212732, 1.219557, 1.213809, 1.214782, 1.219159,
           1.225199, 1.232771, 1.236492, 1.255068, 1.235828, 1.259035, 1.256340,
           1.245998, 1.243868, 1.241496, 1.228685, 1.230554, 1.242635, 1.238310,
           1.223306, 1.218344, 1.203896, 1.205714, 1.207501, 1.191433, 1.184386,
           1.162312, 1.152766, 1.156449, 1.137839, 1.137869, 1.138157, 1.138157,
           1.138384, 1.138480, 1.138873, 1.139149, 1.139391, 1.139811, 1.140282,
           1.140871, 1.141843, 1.142861, 1.143996, 1.145872, 1.148491, 1.151849,
           1.155970},
          {1.257804, 1.263522, 1.269852, 1.256067, 1.260960, 1.270528, 1.272707,
           1.271555, 1.280359, 1.278897, 1.281543, 1.284290, 1.289178, 1.275764,
           1.277367, 1.275302, 1.267811, 1.274996, 1.284778, 1.280909, 1.273916,
           1.249424, 1.244490, 1.248019, 1.231295, 1.233634, 1.235097, 1.207402,
           1.214162, 1.194895, 1.176136, 1.175056, 1.159583, 1.146452, 1.138157,
           1.138384, 1.138481, 1.138872, 1.139149, 1.139391, 1.139811, 1.140282,
           1.140872, 1.141842, 1.142861, 1.143994, 1.145873, 1.148490, 1.151850,
           1.155973}},
         {{1.263465, 1.267228, 1.264763, 1.263010, 1.260870, 1.267381, 1.270136,
           1.272690, 1.271180, 1.284538, 1.277006, 1.279692, 1.275450, 1.279979,
           1.269720, 1.277403, 1.267397, 1.259412, 1.268727, 1.262197, 1.249786,
           1.252066, 1.254621, 1.232180, 1.240407, 1.236003, 1.225303, 1.218113,
           1.204922, 1.190566, 1.190566, 1.190566, 1.190585, 1.190764, 1.190764,
           1.190906, 1.190965, 1.191208, 1.191379, 1.191529, 1.191790, 1.192083,
           1.192449, 1.193052, 1.193684, 1.194389, 1.195555, 1.197179, 1.199267,
           1.201826},
          {1.333540, 1.332362, 1.335601, 1.324425, 1.338016, 1.327671, 1.330102,
           1.330467, 1.334519, 1.334030, 1.343625, 1.336645, 1.330801, 1.338525,
           1.337309, 1.348036, 1.348691, 1.346635, 1.343497, 1.341972, 1.338816,
           1.325354, 1.319073, 1.323058, 1.311285, 1.302051, 1.304115, 1.300843,
           1.272724, 1.268949, 1.248888, 1.243044, 1.232905, 1.220218, 1.199463,
           1.190906, 1.190965, 1.191209, 1.191379, 1.191530, 1.191791, 1.192084,
           1.192449, 1.193052, 1.193684, 1.194388, 1.195556, 1.197180, 1.199267,
           1.201826}},
         {{1.301253, 1.300235, 1.305321, 1.302154, 1.307258, 1.310227, 1.316445,
           1.334211, 1.346286, 1.366943, 1.342477, 1.360086, 1.371504, 1.348950,
           1.360575, 1.358488, 1.346153, 1.353389, 1.351515, 1.363703, 1.350590,
           1.356752, 1.327024, 1.327741, 1.332356, 1.326198, 1.300546, 1.315694,
           1.294026, 1.277114, 1.275740, 1.266438, 1.266617, 1.268400, 1.268400,
           1.269816, 1.270413, 1.272850, 1.274571, 1.276066, 1.278678, 1.281601,
           1.285262, 1.291292, 1.297619, 1.303434, 1.304552, 1.306111, 1.308111,
           1.310567},
          {1.390091, 1.387269, 1.386225, 1.387320, 1.387400, 1.390664, 1.398617,
           1.412451, 1.435022, 1.452822, 1.435628, 1.423784, 1.436723, 1.427587,
           1.431535, 1.429931, 1.453533, 1.440314, 1.449892, 1.424276, 1.447015,
           1.426442, 1.417461, 1.425844, 1.420654, 1.410284, 1.401874, 1.380633,
           1.385646, 1.379427, 1.364412, 1.346428, 1.328359, 1.322530, 1.305913,
           1.290532, 1.270414, 1.272849, 1.274571, 1.276068, 1.278680, 1.281600,
           1.285262, 1.291290, 1.297618, 1.303434, 1.304553, 1.306110, 1.308110,
           1.310563}}},
        {{{1.298642, 1.291593, 1.287135, 1.282907, 1.290959, 1.286816, 1.296666,
           1.287226, 1.297413, 1.301064, 1.303204, 1.292129, 1.299481, 1.312053,
           1.317103, 1.308571, 1.314404, 1.318153, 1.310436, 1.344844, 1.327164,
           1.327530, 1.324253, 1.325446, 1.337031, 1.338500, 1.330843, 1.327977,
           1.319810, 1.302528, 1.317053, 1.295248, 1.293694, 1.278855, 1.289218,
           1.272003, 1.256683, 1.259698, 1.245676, 1.237299, 1.226554, 1.230890,
           1.217259, 1.207082, 1.207246, 1.199740, 1.204739, 1.195256, 1.198912,
           1.203397},
          {1.426226, 1.409133, 1.395514, 1.385327, 1.378258, 1.381263, 1.384580,
           1.389793, 1.386535, 1.378984, 1.388138, 1.388885, 1.375862, 1.386944,
           1.371370, 1.402675, 1.400538, 1.401027, 1.398571, 1.393168, 1.363064,
           1.373462, 1.338327, 1.353894, 1.356275, 1.355405, 1.369861, 1.365610,
           1.329494, 1.340769, 1.325269, 1.328442, 1.323872, 1.293609, 1.316798,
           1.287056, 1.310503, 1.280541, 1.274337, 1.248920, 1.253209, 1.220337,
           1.232107, 1.222655, 1.221701, 1.207682, 1.192408, 1.195258, 1.198913,
           1.203397}},
         {{1.346225, 1.340288, 1.337300, 1.344175, 1.340264, 1.342522, 1.340229,
           1.346095, 1.348089, 1.347188, 1.351407, 1.346713, 1.350224, 1.348216,
           1.362513, 1.363375, 1.375054, 1.375434, 1.404740, 1.379528, 1.387565,
           1.402191, 1.383950, 1.413782, 1.412422, 1.373180, 1.380406, 1.374253,
           1.392603, 1.354590, 1.345139, 1.372817, 1.370866, 1.338036, 1.321670,
           1.317843, 1.315546, 1.301531, 1.305375, 1.293429, 1.286412, 1.283814,
           1.276562, 1.269191, 1.272418, 1.263251, 1.258674, 1.267131, 1.252048,
           1.252973},
          {1.489321, 1.470088, 1.447684, 1.431788, 1.422178, 1.431134, 1.430469,
           1.426399, 1.440582, 1.431214, 1.436787, 1.430543, 1.430974, 1.447071,
           1.431681, 1.437536, 1.433304, 1.452675, 1.439318, 1.442225, 1.440098,
           1.445799, 1.442943, 1.439267, 1.438826, 1.439890, 1.442275, 1.447024,
           1.452121, 1.448941, 1.456062, 1.451299, 1.426605, 1.441896, 1.425644,
           1.421815, 1.380169, 1.397061, 1.367420, 1.372480, 1.365777, 1.361313,
           1.341077, 1.334490, 1.328386, 1.300951, 1.307550, 1.310789, 1.312151,
           1.294244}},
         {{1.380342, 1.374018, 1.376169, 1.376212, 1.372563, 1.386382, 1.389363,
           1.402527, 1.430343, 1.437010, 1.407803, 1.411995, 1.424205, 1.427239,
           1.429259, 1.432747, 1.442874, 1.451516, 1.454294, 1.450622, 1.484207,
           1.479490, 1.478587, 1.457110, 1.470358, 1.436583, 1.475715, 1.437005,
           1.468186, 1.476033, 1.473816, 1.477117, 1.421639, 1.422957, 1.457725,
           1.445620, 1.441082, 1.426519, 1.459483, 1.455337, 1.408501, 1.445629,
           1.437707, 1.425178, 1.406361, 1.414492, 1.434260, 1.407782, 1.425660,
           1.399618},
          {1.637994, 1.597982, 1.566695, 1.544714, 1.531565, 1.530354, 1.532133,
           1.570537, 1.597093, 1.636029, 1.606177, 1.579723, 1.581542, 1.593006,
           1.585888, 1.606355, 1.589706, 1.570563, 1.598802, 1.589959, 1.609957,
           1.619486, 1.614835, 1.599782, 1.616188, 1.598152, 1.592322, 1.602430,
           1.580052, 1.591987, 1.585516, 1.561876, 1.565889, 1.556981, 1.554383,
           1.541521, 1.542132, 1.536756, 1.528319, 1.518004, 1.498102, 1.497593,
           1.493495, 1.488247, 1.483510, 1.482196, 1.476199, 1.472893, 1.464681,
           1.426923}}},
        {{{2.007394, 1.972758, 1.937943, 1.929532, 1.892921, 1.855020, 1.816598,
           1.776477, 1.734862, 1.691520, 1.656041, 1.631291, 1.621620, 1.630465,
           1.622359, 1.626641, 1.618242, 1.622847, 1.615776, 1.618165, 1.605790,
           1.598282, 1.596647, 1.587227, 1.586028, 1.576627, 1.573639, 1.566207,
           1.574588, 1.550863, 1.547157, 1.539935, 1.532126, 1.523084, 1.515065,
           1.509354, 1.497796, 1.487064, 1.478213, 1.462658, 1.452461, 1.440538,
           1.423321, 1.417017, 1.407246, 1.393393, 1.379671, 1.370670, 1.364862,
           1.369914},
          {2.367849, 2.316762, 2.295056, 2.249379, 2.196492, 2.141597, 2.087477,
           2.030741, 1.973077, 1.914734, 1.875575, 1.851339, 1.817701, 1.787540,
           1.759058, 1.770383, 1.739653, 1.747155, 1.719942, 1.709399, 1.694944,
           1.681634, 1.669489, 1.650314, 1.641429, 1.626741, 1.621616, 1.608950,
           1.594564, 1.582924, 1.569233, 1.554816, 1.541582, 1.529341, 1.513993,
           1.503319, 1.484649, 1.470212, 1.454214, 1.437581, 1.420113, 1.402644,
           1.382117, 1.370612, 1.356254, 1.340207, 1.324755, 1.321517, 1.312533,
           1.294340}},
         {{1.982794, 1.950088, 1.941460, 1.907540, 1.872364, 1.836690, 1.800924,
           1.764621, 1.728789, 1.694817, 1.676881, 1.660610, 1.645413, 1.664683,
           1.647091, 1.670137, 1.680694, 1.677016, 1.681691, 1.681652, 1.677921,
           1.678975, 1.674575, 1.675430, 1.668050, 1.661682, 1.660307, 1.658246,
           1.649603, 1.645785, 1.640545, 1.635068, 1.630991, 1.624096, 1.618179,
           1.606047, 1.597642, 1.589828, 1.577073, 1.564638, 1.550922, 1.537055,
           1.536335, 1.522515, 1.508096, 1.498158, 1.486549, 1.481428, 1.463319,
           1.451720},
          {2.528737, 2.475230, 2.419230, 2.360801, 2.301850, 2.263887, 2.234604,
           2.178372, 2.124108, 2.066620, 1.998447, 1.933365, 1.883859, 1.849603,
           1.867398, 1.844718, 1.846625, 1.816487, 1.804116, 1.795798, 1.768706,
           1.769759, 1.746795, 1.742446, 1.723561, 1.714242, 1.693817, 1.680363,
           1.667235, 1.663143, 1.648421, 1.668158, 1.681774, 1.675350, 1.668816,
           1.650341, 1.631483, 1.619588, 1.601577, 1.581664, 1.563348, 1.541642,
           1.522711, 1.504886, 1.482728, 1.462699, 1.447866, 1.432008, 1.418710,
           1.427374}},
         {{2.056579, 2.019423, 2.001939, 1.974774, 1.942956, 1.917533, 1.896892,
           1.889229, 1.898794, 1.889066, 1.825891, 1.800395, 1.787630, 1.792502,
           1.795836, 1.792708, 1.791351, 1.792922, 1.795208, 1.797324, 1.795829,
           1.796545, 1.796271, 1.800805, 1.799932, 1.799451, 1.799874, 1.799544,
           1.799474, 1.802140, 1.801532, 1.800594, 1.797028, 1.795360, 1.785687,
           1.785023, 1.781326, 1.776265, 1.769097, 1.756601, 1.756373, 1.751241,
           1.720246, 1.718727, 1.709617, 1.708254, 1.697026, 1.686122, 1.675274,
           1.621363},
          {2.852414, 2.779039, 2.755135, 2.685822, 2.618970, 2.560810, 2.511036,
           2.479174, 2.472183, 2.443085, 2.368333, 2.280438, 2.204180, 2.152598,
           2.113060, 2.077565, 2.096233, 2.052361, 2.077126, 2.046006, 2.044817,
           2.021894, 2.022011, 2.000441, 1.982421, 1.973557, 1.961320, 1.943778,
           1.923696, 1.914992, 1.899050, 1.895715, 1.866687, 1.851224, 1.839793,
           1.824988, 1.812987, 1.811480, 1.796912, 1.779665, 1.763239, 1.741511,
           1.720655, 1.704638, 1.688842, 1.671648, 1.651368, 1.635007, 1.614300,
           1.586490}}},
        {{{2.910610, 2.843465, 2.774609, 2.721995, 2.662878, 2.606195, 2.553474,
           2.502396, 2.450246, 2.399778, 2.352682, 2.309593, 2.264376, 2.223442,
           2.184540, 2.145877, 2.108951, 2.074855, 2.041258, 2.010281, 1.980171,
           1.952549, 1.925697, 1.900542, 1.881402, 1.869589, 1.854820, 1.836964,
           1.819409, 1.803084, 1.786966, 1.771691, 1.757187, 1.743082, 1.728540,
           1.713819, 1.698864, 1.683278, 1.666676, 1.649167, 1.632352, 1.615093,
           1.597190, 1.581006, 1.563898, 1.548149, 1.533604, 1.521964, 1.511520,
           1.503858},
          {3.400023, 3.328866, 3.257999, 3.191260, 3.125488, 3.057430, 2.991616,
           2.925637, 2.860258, 2.796366, 2.733960, 2.669468, 2.605130, 2.545110,
           2.485719, 2.426254, 2.369397, 2.317384, 2.265119, 2.214958, 2.168970,
           2.123915, 2.080890, 2.040273, 2.003911, 1.967700, 1.933711, 1.901883,
           1.878602, 1.863552, 1.839664, 1.815063, 1.791243, 1.768328, 1.745451,
           1.722631, 1.699183, 1.675983, 1.652063, 1.628229, 1.603584, 1.578785,
           1.554370, 1.531519, 1.509087, 1.488631, 1.470196, 1.455278, 1.442398,
           1.432898}},
         {{2.858393, 2.798103, 2.737344, 2.686685, 2.632511, 2.580356, 2.534133,
           2.488673, 2.446825, 2.402425, 2.352190, 2.304109, 2.263515, 2.223119,
           2.185914, 2.149698, 2.114780, 2.081546, 2.050279, 2.022418, 1.994436,
           1.967872, 1.942747, 1.920832, 1.900336, 1.918239, 1.946444, 1.943844,
           1.933194, 1.920373, 1.906101, 1.892521, 1.879447, 1.866912, 1.854037,
           1.840695, 1.826449, 1.811695, 1.796362, 1.779379, 1.762501, 1.744862,
           1.727346, 1.710381, 1.693067, 1.677001, 1.662467, 1.650689, 1.641146,
           1.632261},
          {3.566467, 3.488055, 3.412712, 3.336887, 3.264896, 3.199507, 3.135043,
           3.074990, 3.022439, 2.966371, 2.891085, 2.820045, 2.756352, 2.691956,
           2.629528, 2.568792, 2.513029, 2.456226, 2.402028, 2.351192, 2.303492,
           2.256253, 2.211536, 2.171333, 2.131505, 2.093750, 2.057832, 2.025406,
           1.993723, 1.964368, 1.936072, 1.909350, 1.915685, 1.949178, 1.937851,
           1.917423, 1.892627, 1.869938, 1.846253, 1.819511, 1.792571, 1.763763,
           1.735127, 1.706820, 1.677986, 1.651161, 1.626080, 1.603580, 1.584490,
           1.567776}},
         {{2.956871, 2.882379, 2.825636, 2.771139, 2.722000, 2.686454, 2.663075,
           2.661725, 2.695579, 2.697874, 2.596861, 2.519798, 2.474614, 2.437961,
           2.402629, 2.367993, 2.336683, 2.307212, 2.283034, 2.259088, 2.236325,
           2.213804, 2.193179, 2.177940, 2.162638, 2.146635, 2.131814, 2.118216,
           2.107918, 2.108260, 2.103818, 2.100695, 2.097183, 2.091773, 2.081954,
           2.073109, 2.060557, 2.049878, 2.036290, 2.020992, 2.006025, 1.991384,
           1.975581, 1.963316, 1.950316, 1.936395, 1.915223, 1.894575, 1.872723,
           1.852318},
          {4.063686, 3.942579, 3.838427, 3.741183, 3.653720, 3.590884, 3.546577,
           3.537466, 3.573999, 3.564250, 3.400275, 3.274867, 3.197770, 3.130756,
           3.069138, 3.004046, 2.941070, 2.878883, 2.825440, 2.770339, 2.713200,
           2.659032, 2.607273, 2.563894, 2.518817, 2.473453, 2.432853, 2.393432,
           2.355583, 2.326334, 2.292412, 2.261148, 2.230185, 2.206013, 2.175303,
           2.150009, 2.123645, 2.115482, 2.096828, 2.067458, 2.037732, 2.008608,
           1.980349, 1.955095, 1.930835, 1.904556, 1.874153, 1.845607, 1.820939,
           1.797698}}},
        {{{3.955489, 3.847070, 3.741294, 3.640398, 3.544689, 3.450833, 3.361254,
           3.274910, 3.194681, 3.114756, 3.038556, 2.967320, 2.898122, 2.831914,
           2.767964, 2.708509, 2.652240, 2.597407, 2.545455, 2.497820, 2.451660,
           2.407578, 2.366090, 2.327548, 2.290889, 2.264869, 2.246607, 2.220688,
           2.192816, 2.167833, 2.141966, 2.117510, 2.094188, 2.072001, 2.049259,
           2.028213, 2.005297, 1.982309, 1.958692, 1.934399, 1.910194, 1.885858,
           1.860949, 1.836121, 1.811391, 1.787488, 1.765947, 1.747821, 1.731930,
           1.718469},
          {4.379959, 4.266512, 4.156056, 4.048554, 3.947141, 3.847203, 3.750353,
           3.659028, 3.573937, 3.492868, 3.412690, 3.333658, 3.257678, 3.185481,
           3.113774, 3.043045, 2.974252, 2.905527, 2.837599, 2.770314, 2.706099,
           2.643509, 2.581842, 2.523394, 2.468142, 2.414075, 2.363283, 2.316240,
           2.274548, 2.249848, 2.217051, 2.180146, 2.143971, 2.109679, 2.075745,
           2.043098, 2.011567, 1.979639, 1.947200, 1.914745, 1.883248, 1.851234,
           1.819017, 1.787967, 1.756549, 1.726434, 1.698409, 1.673934, 1.653226,
           1.637502}},
         {{3.896824, 3.792357, 3.692076, 3.597267, 3.506416, 3.419415, 3.337092,
           3.262488, 3.194651, 3.122642, 3.038995, 2.964732, 2.896761, 2.832844,
           2.771422, 2.714742, 2.660146, 2.607902, 2.558967, 2.514204, 2.470455,
           2.428643, 2.389560, 2.353805, 2.320418, 2.306543, 2.335536, 2.362280,
           2.349883, 2.329969, 2.308366, 2.285366, 2.263535, 2.243132, 2.222175,
           2.200932, 2.179186, 2.158516, 2.135523, 2.111080, 2.085826, 2.059541,
           2.032929, 2.007255, 1.981783, 1.956698, 1.934056, 1.914871, 1.896813,
           1.880388},
          {4.605658, 4.482486, 4.362402, 4.248080, 4.138736, 4.036523, 3.944775,
           3.859215, 3.784758, 3.709158, 3.614284, 3.525288, 3.444353, 3.367526,
           3.293976, 3.224652, 3.154970, 3.086817, 3.021216, 2.956686, 2.891274,
           2.827299, 2.764682, 2.706756, 2.649150, 2.594132, 2.541931, 2.492783,
           2.444768, 2.400445, 2.358165, 2.318365, 2.315055, 2.360699, 2.350665,
           2.319429, 2.282768, 2.246438, 2.209176, 2.170643, 2.131369, 2.090627,
           2.049370, 2.009443, 1.970307, 1.931827, 1.896432, 1.864505, 1.836605,
           1.811842}},
         {{4.035762, 3.919674, 3.812384, 3.715411, 3.629720, 3.565473, 3.523557,
           3.515584, 3.555260, 3.552165, 3.390424, 3.267504, 3.194915, 3.131251,
           3.072309, 3.017421, 2.964216, 2.915194, 2.875034, 2.836137, 2.796907,
           2.760034, 2.725937, 2.700039, 2.673612, 2.647550, 2.623554, 2.601082,
           2.583492, 2.581070, 2.573102, 2.566944, 2.556119, 2.544572, 2.526172,
           2.509442, 2.489621, 2.471562, 2.449979, 2.426065, 2.402613, 2.378046,
           2.354780, 2.334585, 2.313883, 2.291440, 2.257966, 2.223337, 2.188793,
           2.155066},
          {5.240438, 5.067803, 4.908165, 4.768656, 4.646987, 4.557514, 4.496001,
           4.490649, 4.545988, 4.531215, 4.294400, 4.118022, 4.010149, 3.913547,
           3.821314, 3.734144, 3.653169, 3.573518, 3.504660, 3.436434, 3.369770,
           3.305221, 3.244540, 3.193630, 3.141867, 3.090652, 3.041053, 2.990613,
           2.942308, 2.903526, 2.858159, 2.812755, 2.767848, 2.730279, 2.685326,
           2.646458, 2.605581, 2.589270, 2.559292, 2.514889, 2.469747, 2.423744,
           2.378318, 2.339501, 2.300727, 2.261762, 2.215500, 2.172245, 2.132723,
           2.097564}}},
        {{{6.703874, 6.503014, 6.313403, 6.135281, 5.951303, 5.761250, 5.580227,
           5.406346, 5.236298, 5.077627, 4.924253, 4.776062, 4.634167, 4.495834,
           4.363454, 4.236464, 4.114121, 4.000350, 3.889525, 3.784194, 3.684226,
           3.590866, 3.502917, 3.417775, 3.338194, 3.263238, 3.199861, 3.160011,
           3.118385, 3.065179, 3.012195, 2.961709, 2.912614, 2.866767, 2.821168,
           2.777606, 2.735583, 2.692832, 2.649300, 2.604989, 2.560343, 2.516362,
           2.470639, 2.426456, 2.382554, 2.340920, 2.302239, 2.266959, 2.234647,
           2.207405},
          {6.523621, 6.338278, 6.163428, 5.985804, 5.802927, 5.627149, 5.462113,
           5.304730, 5.158926, 5.022834, 4.887040, 4.752246, 4.621054, 4.492712,
           4.369375, 4.250751, 4.135707, 4.027649, 3.924050, 3.823728, 3.728268,
           3.637250, 3.553964, 3.471833, 3.394593, 3.322062, 3.253542, 3.196680,
           3.159184, 3.120061, 3.065198, 3.009280, 2.953068, 2.897403, 2.841026,
           2.785671, 2.731601, 2.676669, 2.621362, 2.565371, 2.511064, 2.455354,
           2.400281, 2.347977, 2.296663, 2.247146, 2.202303, 2.164227, 2.130444,
           2.102407}},
         {{6.509508, 6.329252, 6.159265, 5.985457, 5.806857, 5.636251, 5.475501,
           5.323441, 5.186090, 5.041615, 4.871299, 4.718673, 4.580818, 4.449605,
           4.324939, 4.204176, 4.090673, 3.984387, 3.881492, 3.783708, 3.690361,
           3.604441, 3.521957, 3.444205, 3.371240, 3.301982, 3.239507, 3.286700,
           3.364778, 3.342413, 3.297966, 3.253641, 3.208505, 3.163574, 3.118115,
           3.073215, 3.027933, 2.983591, 2.938102, 2.889940, 2.841655, 2.792479,
           2.743716, 2.695277, 2.647953, 2.600889, 2.554615, 2.512758, 2.473898,
           2.439304},
          {6.871400, 6.668324, 6.477360, 6.299354, 6.134339, 5.968551, 5.804546,
           5.658151, 5.530188, 5.395166, 5.225878, 5.070160, 4.928230, 4.792690,
           4.662702, 4.536662, 4.416386, 4.301740, 4.192207, 4.088994, 3.991853,
           3.896750, 3.807125, 3.723152, 3.643853, 3.570806, 3.498488, 3.430635,
           3.367177, 3.308291, 3.253129, 3.273114, 3.346446, 3.345207, 3.293001,
           3.238713, 3.181465, 3.123357, 3.061894, 2.996360, 2.929019, 2.858292,
           2.786809, 2.717778, 2.648598, 2.581966, 2.520832, 2.464427, 2.413818,
           2.369332}},
         {{6.600362, 6.382588, 6.181322, 6.002807, 5.840936, 5.720830, 5.638162,
           5.624787, 5.700176, 5.685802, 5.365154, 5.125172, 4.982354, 4.856276,
           4.738337, 4.629739, 4.526628, 4.429552, 4.348431, 4.266011, 4.185366,
           4.110688, 4.037842, 3.979298, 3.918895, 3.858261, 3.800787, 3.748813,
           3.716348, 3.705614, 3.689530, 3.657592, 3.620148, 3.585899, 3.545979,
           3.510284, 3.469581, 3.433029, 3.391816, 3.347508, 3.304409, 3.260781,
           3.216299, 3.172720, 3.124943, 3.072956, 3.005055, 2.939574, 2.877303,
           2.819285},
          {7.831215, 7.571415, 7.335350, 7.129547, 6.949279, 6.816475, 6.713455,
           6.694146, 6.785356, 6.752864, 6.334905, 6.026159, 5.839979, 5.676749,
           5.521214, 5.371854, 5.228744, 5.092742, 4.975472, 4.855925, 4.738550,
           4.628962, 4.523979, 4.435798, 4.345844, 4.256242, 4.173492, 4.094186,
           4.017874, 3.959200, 3.890536, 3.824525, 3.763908, 3.734510, 3.706139,
           3.671047, 3.612925, 3.557959, 3.498155, 3.434021, 3.371801, 3.308504,
           3.247259, 3.190410, 3.134672, 3.076489, 3.002663, 2.931971, 2.865667,
           2.805296}}},
        {{{10.978984, 10.635671, 10.298009, 9.962430, 9.631692, 9.310571,
           8.993513,  8.684437,  8.385771,  8.094556, 7.814060, 7.542950,
           7.281229,  7.032614,  6.788501,  6.560075, 6.342302, 6.138189,
           5.948247,  5.764993,  5.580120,  5.401970, 5.233905, 5.078507,
           4.930707,  4.789481,  4.656305,  4.532658, 4.458214, 4.406961,
           4.312976,  4.216256,  4.122214,  4.033274, 3.944736, 3.861158,
           3.780254,  3.701627,  3.622517,  3.543211, 3.464908, 3.388524,
           3.312486,  3.238659,  3.164745,  3.091229, 3.024637, 2.962401,
           2.907093,  2.859859},
          {9.292233, 8.994800, 8.707110, 8.429830, 8.158233, 7.894024, 7.639423,
           7.394466, 7.163880, 6.943271, 6.727297, 6.517205, 6.314736, 6.123515,
           5.943732, 5.768762, 5.591387, 5.419099, 5.254557, 5.103285, 4.957746,
           4.818711, 4.685114, 4.560741, 4.474160, 4.418781, 4.328891, 4.229468,
           4.133195, 4.044480, 3.957362, 3.875238, 3.798462, 3.727332, 3.655481,
           3.587171, 3.519479, 3.454312, 3.389776, 3.325058, 3.258919, 3.192811,
           3.126755, 3.061680, 2.994440, 2.926156, 2.862998, 2.807647, 2.758376,
           2.716601}},
         {{10.445367, 10.140562, 9.836663, 9.539822, 9.252148, 8.969021,
           8.696696,  8.440648,  8.199147, 7.938699, 7.629182, 7.352930,
           7.109330,  6.872123,  6.650491, 6.438937, 6.240444, 6.055735,
           5.881560,  5.706256,  5.533308, 5.368929, 5.216179, 5.073381,
           4.937973,  4.806482,  4.684445, 4.570823, 4.676199, 4.838400,
           4.789879,  4.706728,  4.613352, 4.518718, 4.423050, 4.330452,
           4.239658,  4.152652,  4.064660, 3.976635, 3.889491, 3.801783,
           3.713974,  3.625433,  3.537320, 3.450981, 3.369731, 3.297029,
           3.230317,  3.170761},
          {9.839453, 9.527785, 9.225637, 8.927615, 8.642493, 8.375736, 8.123124,
           7.895951, 7.696641, 7.490486, 7.233482, 6.990598, 6.772794, 6.567080,
           6.371681, 6.187670, 6.014986, 5.847727, 5.681565, 5.518927, 5.363208,
           5.218259, 5.080716, 4.952197, 4.827428, 4.709464, 4.599153, 4.644871,
           4.812321, 4.789771, 4.690033, 4.580938, 4.472430, 4.369343, 4.270773,
           4.179100, 4.088982, 4.002079, 3.916842, 3.832001, 3.748117, 3.666576,
           3.582971, 3.501453, 3.421104, 3.343635, 3.270470, 3.202088, 3.139758,
           3.083232}},
         {{10.105882, 9.774837, 9.461830, 9.185660, 8.940344, 8.761958,
           8.636776,  8.627671, 8.772529, 8.750696, 8.201443, 7.794881,
           7.558898,  7.355210, 7.163380, 6.980163, 6.796171, 6.613543,
           6.462940,  6.310242, 6.160126, 6.017178, 5.881153, 5.767654,
           5.648664,  5.532983, 5.424202, 5.350662, 5.314159, 5.304125,
           5.231741,  5.153256, 5.074777, 5.004596, 4.927008, 4.856015,
           4.781059,  4.712241, 4.634344, 4.551638, 4.467469, 4.380145,
           4.292425,  4.212804, 4.134215, 4.051213, 3.948210, 3.850019,
           3.758176,  3.670753},
          {11.171884, 10.782806, 10.411620, 10.083204, 9.792616, 9.577019,
           9.428992,  9.413790,  9.561078,  9.517521,  8.898650, 8.440010,
           8.155302,  7.905140,  7.668993,  7.448961,  7.239698, 7.041015,
           6.861206,  6.669370,  6.482982,  6.307482,  6.142929, 6.002272,
           5.858804,  5.714606,  5.577774,  5.449640,  5.353608, 5.326159,
           5.277732,  5.179574,  5.077015,  4.983017,  4.882229, 4.789432,
           4.694907,  4.609889,  4.522267,  4.431403,  4.344169, 4.257090,
           4.172440,  4.098634,  4.024085,  3.948026,  3.852952, 3.765058,
           3.682837,  3.606844}}},
        {{{18.155575, 17.613051, 17.083181, 16.574365, 16.076662, 15.573585,
           15.082297, 14.593342, 14.107857, 13.637955, 13.171635, 12.717771,
           12.279867, 11.845979, 11.428777, 11.024595, 10.630515, 10.248277,
           9.879222,  9.531268,  9.192628,  8.868734,  8.557079,  8.261488,
           7.985060,  7.715503,  7.460953,  7.218827,  6.989653,  6.862262,
           6.782826,  6.610572,  6.421870,  6.242790,  6.063676,  5.894955,
           5.731807,  5.575846,  5.423424,  5.268924,  5.113105,  4.956415,
           4.807660,  4.666740,  4.529700,  4.398224,  4.278791,  4.171829,
           4.077434,  3.995637},
          {13.546229, 13.078700, 12.624286, 12.181986, 11.747483, 11.333259,
           10.934353, 10.554587, 10.193992, 9.855442,  9.531321,  9.206776,
           8.892320,  8.587884,  8.296789,  8.025434,  7.757952,  7.504985,
           7.265064,  7.037577,  6.881111,  6.790147,  6.633300,  6.445360,
           6.259920,  6.081525,  5.912217,  5.755748,  5.609150,  5.476456,
           5.345439,  5.220043,  5.096275,  4.979682,  4.866147,  4.761666,
           4.658674,  4.561228,  4.463908,  4.367958,  4.273684,  4.180345,
           4.088730,  4.001204,  3.915530,  3.832307,  3.760082,  3.694226,
           3.637551,  3.590278}},
         {{16.890564, 16.459027, 16.029871, 15.600302, 15.177992, 14.760278,
           14.345901, 13.946838, 13.564195, 13.126090, 12.612893, 12.151056,
           11.737029, 11.342757, 10.956605, 10.584550, 10.220568, 9.873397,
           9.549330,  9.228431,  8.921750,  8.625744,  8.345347,  8.085256,
           7.829457,  7.586755,  7.356373,  7.137421,  7.155558,  7.524724,
           7.670883,  7.514547,  7.307052,  7.089935,  6.870153,  6.662507,
           6.466674,  6.277946,  6.092914,  5.910595,  5.726976,  5.546343,
           5.376362,  5.212761,  5.055534,  4.905359,  4.764886,  4.636772,
           4.520814,  4.416285},
          {14.293220, 13.816591, 13.353613, 12.900982, 12.469071, 12.057190,
           11.671013, 11.325589, 11.020180, 10.710471, 10.321512, 9.957682,
           9.633408,  9.319673,  9.018534,  8.727832,  8.449137,  8.188899,
           7.937329,  7.695898,  7.465250,  7.243938,  7.034487,  7.321821,
           7.701436,  7.535897,  7.294721,  7.050170,  6.817505,  6.609907,
           6.420088,  6.245077,  6.079690,  5.925730,  5.774460,  5.629895,
           5.493603,  5.365911,  5.241356,  5.118576,  4.998020,  4.880251,
           4.763076,  4.649535,  4.538425,  4.430654,  4.329326,  4.235677,
           4.152163,  4.074142}},
         {{15.773386, 15.293756, 14.836503, 14.427722, 14.063711, 13.797027,
           13.610434, 13.600805, 13.822161, 13.786327, 12.912173, 12.253740,
           11.858168, 11.510295, 11.176066, 10.854397, 10.545269, 10.246693,
           9.992083,  9.735676,  9.475108,  9.226356,  8.989667,  8.789443,
           8.582911,  8.390844,  8.340981,  8.317061,  8.196370,  8.036511,
           7.855509,  7.677530,  7.503473,  7.342246,  7.176477,  7.026319,
           6.873473,  6.736072,  6.586813,  6.438154,  6.290861,  6.145058,
           6.006251,  5.882556,  5.761197,  5.636973,  5.483911,  5.337779,
           5.198554,  5.066993},
          {16.104066, 15.544834, 15.023601, 14.550114, 14.128526, 13.816120,
           13.592904, 13.557381, 13.755384, 13.692526, 12.796801, 12.116633,
           11.697924, 11.327225, 10.968382, 10.624381, 10.290316, 9.973869,
           9.702112,  9.420327,  9.140575,  8.874262,  8.618816,  8.402294,
           8.339913,  8.311813,  8.119490,  7.900022,  7.688401,  7.496687,
           7.302443,  7.121169,  6.952740,  6.803150,  6.649415,  6.503669,
           6.358592,  6.224297,  6.088844,  5.955910,  5.830778,  5.708535,
           5.589996,  5.485718,  5.383964,  5.279784,  5.148539,  5.025174,
           4.909498,  4.801824}}},
        {{{90.526686, 84.447123, 78.196543, 72.035289, 66.148292, 60.510406,
           54.937212, 49.228140, 43.850065, 38.210750, 33.036717, 30.149574,
           28.247478, 26.772126, 25.534848, 24.431407, 23.382702, 22.393489,
           21.455628, 20.552038, 19.698933, 18.876484, 18.091334, 17.339102,
           16.622435, 15.936572, 15.289911, 14.808020, 14.792929, 14.484265,
           13.901321, 13.318781, 12.766859, 12.253283, 11.750276, 11.297384,
           10.854886, 10.444741, 10.053867, 9.672786,  9.318062,  8.974224,
           8.653853,  8.348205,  8.063177,  7.789341,  7.546448,  7.334216,
           7.146149,  6.983368},
          {23.730703, 22.667346, 21.666425, 20.708436, 19.815338, 18.971709,
           18.188778, 17.467580, 16.808376, 16.198459, 15.613721, 15.043651,
           14.779422, 14.751487, 14.303486, 13.747857, 13.213837, 12.709849,
           12.244085, 11.800803, 11.385338, 10.992882, 10.619732, 10.278162,
           9.958073,  9.643720,  9.348363,  9.072490,  8.806287,  8.566300,
           8.323583,  8.095973,  7.877374,  7.677238,  7.474665,  7.289536,
           7.107649,  6.936213,  6.767454,  6.603242,  6.448180,  6.295653,
           6.147109,  6.014272,  5.884432,  5.757567,  5.649863,  5.556086,
           5.480562,  5.417107}},
         {{60.462457, 56.493232, 52.352756, 48.217989, 44.077402, 39.649459,
           35.211933, 32.284794, 29.866674, 28.035876, 26.553742, 25.423648,
           24.454855, 23.558481, 22.716226, 21.906317, 21.114699, 20.355643,
           19.625578, 18.906471, 18.211673, 17.538264, 16.890425, 16.267846,
           15.669218, 15.093064, 16.651650, 18.652690, 18.132809, 17.282624,
           16.428850, 15.612749, 14.866263, 14.186265, 13.559801, 12.975461,
           12.435191, 11.940383, 11.465435, 11.022158, 10.597333, 10.202074,
           9.817167,  9.458013,  9.112956,  8.784646,  8.484061,  8.200740,
           7.941852,  7.708358},
          {25.285747, 24.198479, 23.175360, 22.216316, 21.318496, 20.485824,
           19.724767, 19.033687, 18.428063, 17.841386, 17.166730, 16.522423,
           15.930070, 15.373046, 15.433674, 17.880044, 18.351251, 17.355341,
           16.388819, 15.516305, 14.751424, 14.079845, 13.476216, 12.928735,
           12.436024, 11.991170, 11.572287, 11.185846, 10.830199, 10.495389,
           10.183261, 9.883573,  9.602928,  9.338172,  9.083570,  8.840534,
           8.607877,  8.386934,  8.168968,  7.960536,  7.756914,  7.561745,
           7.368256,  7.185386,  7.007817,  6.839170,  6.678881,  6.531785,
           6.395351,  6.272626}},
         {{28.900866, 27.852085, 26.948929, 26.167147, 25.470824, 24.903645,
           24.458853, 24.233842, 24.321955, 24.172590, 23.000977, 22.066821,
           21.455357, 20.901676, 20.365306, 19.845165, 19.338800, 18.843655,
           18.406566, 17.954394, 17.535543, 17.819126, 18.161124, 17.852089,
           17.373185, 16.868955, 16.371475, 15.886089, 15.414853, 14.964782,
           14.536222, 14.127069, 13.731772, 13.359053, 12.992099, 12.648414,
           12.308131, 11.988700, 11.676377, 11.366611, 11.074303, 10.787758,
           10.514335, 10.271209, 10.035954, 9.801060,  9.524302,  9.255729,
           8.997898,  8.752271},
          {27.949152, 26.741756, 25.674203, 24.727904, 23.883234, 23.192015,
           22.641596, 22.350811, 22.394501, 22.190251, 20.943697, 19.938387,
           19.264824, 18.650129, 18.057525, 17.483235, 17.838421, 18.091219,
           17.505526, 16.845731, 16.213525, 15.611574, 15.051732, 14.534115,
           14.056080, 13.603265, 13.176542, 12.776173, 12.399847, 12.053980,
           11.715844, 11.394911, 11.088893, 10.811249, 10.528201, 10.270588,
           10.014436, 9.783581,  9.552159,  9.326965,  9.117502,  8.913825,
           8.720782,  8.555188,  8.396308,  8.235139,  8.028905,  7.833237,
           7.647582,  7.470901}}}},
       {{{{1.471320, 1.440949, 1.470571, 1.446786, 1.450790, 1.478450, 1.461054,
           1.477729, 1.446681, 1.433662, 1.428776, 1.443908, 1.410463, 1.418274,
           1.390573, 1.385243, 1.360477, 1.343328, 1.303833, 1.289764, 1.256417,
           1.234385, 1.214769, 1.214913, 1.214972, 1.214972, 1.214972, 1.214972,
           1.214974, 1.215183, 1.215183, 1.215183, 1.215201, 1.215396, 1.215396,
           1.215548, 1.215613, 1.215878, 1.216066, 1.216228, 1.216512, 1.216830,
           1.217229, 1.217885, 1.218572, 1.219339, 1.220607, 1.222373, 1.224642,
           1.227428},
          {1.465690, 1.465408, 1.488526, 1.484988, 1.481104, 1.442493, 1.457213,
           1.448673, 1.426864, 1.464731, 1.422955, 1.440845, 1.409416, 1.415127,
           1.379209, 1.383691, 1.359361, 1.337573, 1.301946, 1.282705, 1.259921,
           1.232695, 1.214769, 1.214914, 1.214972, 1.214972, 1.214972, 1.214972,
           1.214975, 1.215183, 1.215183, 1.215183, 1.215202, 1.215396, 1.215396,
           1.215548, 1.215614, 1.215879, 1.216066, 1.216228, 1.216514, 1.216831,
           1.217227, 1.217885, 1.218572, 1.219338, 1.220606, 1.222374, 1.224643,
           1.227426}},
         {{1.490477, 1.456695, 1.486072, 1.455405, 1.463296, 1.444853, 1.473253,
           1.435661, 1.424403, 1.453109, 1.418265, 1.414174, 1.415213, 1.390829,
           1.372620, 1.357402, 1.340659, 1.321343, 1.303940, 1.282788, 1.284282,
           1.259581, 1.251059, 1.251165, 1.251207, 1.251207, 1.251207, 1.251207,
           1.251209, 1.251361, 1.251361, 1.251361, 1.251376, 1.251518, 1.251518,
           1.251631, 1.251680, 1.251873, 1.252013, 1.252132, 1.252340, 1.252573,
           1.252866, 1.253348, 1.253855, 1.254419, 1.255350, 1.256649, 1.258317,
           1.260366},
          {1.488129, 1.471347, 1.503864, 1.481016, 1.477464, 1.493149, 1.480196,
           1.480511, 1.465541, 1.458823, 1.430039, 1.434807, 1.434374, 1.419914,
           1.396780, 1.380817, 1.363422, 1.360255, 1.316156, 1.299192, 1.298479,
           1.272214, 1.251059, 1.251163, 1.251207, 1.251207, 1.251207, 1.251207,
           1.251209, 1.251361, 1.251361, 1.251361, 1.251375, 1.251518, 1.251518,
           1.251630, 1.251680, 1.251873, 1.252013, 1.252132, 1.252340, 1.252573,
           1.252866, 1.253347, 1.253853, 1.254418, 1.255349, 1.256649, 1.258318,
           1.260366}},
         {{1.483041, 1.471337, 1.482335, 1.465139, 1.463218, 1.484457, 1.458671,
           1.470412, 1.479945, 1.485491, 1.483960, 1.455737, 1.463976, 1.455317,
           1.463528, 1.429251, 1.425273, 1.400196, 1.366493, 1.355609, 1.335926,
           1.316947, 1.308997, 1.310061, 1.310496, 1.310496, 1.310496, 1.310496,
           1.310512, 1.312030, 1.312030, 1.312030, 1.312174, 1.313599, 1.313599,
           1.314730, 1.315209, 1.317158, 1.318535, 1.319730, 1.321819, 1.324153,
           1.327080, 1.331902, 1.336961, 1.341600, 1.342398, 1.343507, 1.344934,
           1.346684},
          {1.491607, 1.463547, 1.494307, 1.490723, 1.486708, 1.447605, 1.452458,
           1.453517, 1.491780, 1.504201, 1.453887, 1.477306, 1.481119, 1.447922,
           1.449505, 1.423332, 1.415513, 1.387073, 1.377503, 1.347999, 1.336857,
           1.314042, 1.308997, 1.310060, 1.310496, 1.310496, 1.310496, 1.310496,
           1.310502, 1.312030, 1.312030, 1.312030, 1.312173, 1.313599, 1.313599,
           1.314730, 1.315211, 1.317156, 1.318535, 1.319729, 1.321818, 1.324155,
           1.327083, 1.331902, 1.336963, 1.341601, 1.342398, 1.343509, 1.344934,
           1.346679}}},
        {{{1.430436, 1.438573, 1.442940, 1.438078, 1.433359, 1.419870, 1.461371,
           1.460871, 1.449488, 1.446297, 1.431569, 1.416704, 1.412511, 1.436652,
           1.429340, 1.413720, 1.401129, 1.393225, 1.385688, 1.375623, 1.365093,
           1.349925, 1.327375, 1.319367, 1.292348, 1.282105, 1.265381, 1.241044,
           1.223308, 1.223518, 1.223518, 1.223518, 1.223539, 1.223736, 1.223736,
           1.223892, 1.223959, 1.224228, 1.224418, 1.224582, 1.224871, 1.225195,
           1.225598, 1.226264, 1.226964, 1.227742, 1.229030, 1.230826, 1.233130,
           1.235959},
          {1.364300, 1.362671, 1.361648, 1.374950, 1.379529, 1.391633, 1.378143,
           1.376050, 1.373678, 1.380620, 1.368022, 1.363985, 1.368791, 1.351166,
           1.357658, 1.344427, 1.340691, 1.335625, 1.317170, 1.309137, 1.302920,
           1.288205, 1.283477, 1.265312, 1.250875, 1.231290, 1.223306, 1.223306,
           1.223308, 1.223518, 1.223518, 1.223518, 1.223538, 1.223736, 1.223736,
           1.223892, 1.223959, 1.224228, 1.224418, 1.224582, 1.224872, 1.225195,
           1.225597, 1.226265, 1.226964, 1.227742, 1.229030, 1.230825, 1.233132,
           1.235959}},
         {{1.422807, 1.434243, 1.431175, 1.421659, 1.409535, 1.437927, 1.461312,
           1.446809, 1.470909, 1.464266, 1.467452, 1.446007, 1.457507, 1.421562,
           1.422624, 1.429771, 1.414147, 1.416989, 1.403594, 1.399581, 1.386515,
           1.367567, 1.343402, 1.338349, 1.318754, 1.316834, 1.301622, 1.281142,
           1.260077, 1.260230, 1.260230, 1.260230, 1.260243, 1.260385, 1.260385,
           1.260500, 1.260548, 1.260743, 1.260880, 1.261000, 1.261210, 1.261444,
           1.261739, 1.262222, 1.262729, 1.263296, 1.264230, 1.265535, 1.267208,
           1.269263},
          {1.363407, 1.375192, 1.376261, 1.379656, 1.381826, 1.376737, 1.396298,
           1.383464, 1.382493, 1.385337, 1.388037, 1.373450, 1.378935, 1.359363,
           1.361027, 1.354118, 1.355098, 1.337898, 1.337771, 1.317696, 1.313121,
           1.309394, 1.317008, 1.304075, 1.286993, 1.271593, 1.260075, 1.260075,
           1.260076, 1.260230, 1.260230, 1.260230, 1.260242, 1.260385, 1.260385,
           1.260499, 1.260548, 1.260742, 1.260880, 1.261000, 1.261210, 1.261444,
           1.261738, 1.262222, 1.262729, 1.263295, 1.264230, 1.265534, 1.267209,
           1.269264}},
         {{1.496859, 1.484919, 1.474987, 1.481255, 1.493839, 1.479327, 1.496945,
           1.523698, 1.523960, 1.533018, 1.519822, 1.487419, 1.482028, 1.502187,
           1.501302, 1.478674, 1.457303, 1.475520, 1.463779, 1.427687, 1.423726,
           1.416016, 1.401537, 1.379478, 1.364381, 1.358392, 1.344805, 1.328923,
           1.319566, 1.321100, 1.321100, 1.321100, 1.321243, 1.322673, 1.322673,
           1.323809, 1.324290, 1.326243, 1.327623, 1.328823, 1.330919, 1.333264,
           1.336201, 1.341038, 1.346115, 1.350769, 1.351574, 1.352696, 1.354137,
           1.355905},
          {1.419645, 1.409152, 1.420504, 1.417589, 1.419778, 1.427064, 1.432760,
           1.455581, 1.467177, 1.461310, 1.461765, 1.431430, 1.438675, 1.427279,
           1.427168, 1.421088, 1.423638, 1.409541, 1.404303, 1.382290, 1.381103,
           1.361599, 1.364904, 1.350468, 1.340836, 1.326893, 1.319560, 1.319560,
           1.319574, 1.321100, 1.321100, 1.321100, 1.321243, 1.322673, 1.322673,
           1.323809, 1.324288, 1.326243, 1.327623, 1.328824, 1.330919, 1.333261,
           1.336201, 1.341038, 1.346114, 1.350769, 1.351573, 1.352697, 1.354136,
           1.355904}}},
        {{{1.326302, 1.325207, 1.325190, 1.330575, 1.332465, 1.331134, 1.319096,
           1.335634, 1.347205, 1.346155, 1.343695, 1.334471, 1.331897, 1.339175,
           1.326174, 1.332404, 1.344877, 1.334920, 1.323946, 1.328054, 1.316646,
           1.311594, 1.312011, 1.309564, 1.289913, 1.284788, 1.277195, 1.266302,
           1.258193, 1.248570, 1.248570, 1.248570, 1.248590, 1.248795, 1.248795,
           1.248960, 1.249029, 1.249311, 1.249510, 1.249682, 1.249984, 1.250322,
           1.250746, 1.251443, 1.252177, 1.252990, 1.254340, 1.256219, 1.258633,
           1.261596},
          {1.349385, 1.347635, 1.350552, 1.345405, 1.347103, 1.351995, 1.354802,
           1.345884, 1.346831, 1.349549, 1.360583, 1.341004, 1.340865, 1.345555,
           1.339152, 1.345407, 1.339523, 1.338084, 1.324613, 1.340397, 1.309631,
           1.330114, 1.317040, 1.306555, 1.309091, 1.302420, 1.278998, 1.275548,
           1.258001, 1.248570, 1.248570, 1.248570, 1.248589, 1.248795, 1.248795,
           1.248959, 1.249028, 1.249309, 1.249510, 1.249683, 1.249985, 1.250323,
           1.250747, 1.251443, 1.252177, 1.252992, 1.254340, 1.256220, 1.258634,
           1.261596}},
         {{1.361097, 1.360312, 1.360167, 1.354152, 1.363301, 1.357436, 1.359150,
           1.370303, 1.369219, 1.361210, 1.368687, 1.366614, 1.366158, 1.365324,
           1.355625, 1.365830, 1.360865, 1.359073, 1.353157, 1.353651, 1.344920,
           1.339221, 1.326239, 1.336975, 1.324043, 1.313972, 1.311058, 1.296178,
           1.286718, 1.286872, 1.286872, 1.286872, 1.286887, 1.287030, 1.287030,
           1.287146, 1.287194, 1.287391, 1.287531, 1.287652, 1.287863, 1.288099,
           1.288397, 1.288885, 1.289397, 1.289970, 1.290913, 1.292230, 1.293922,
           1.295996},
          {1.353022, 1.356314, 1.349877, 1.356086, 1.354384, 1.353170, 1.354475,
           1.360838, 1.364150, 1.364258, 1.377774, 1.365731, 1.377390, 1.378347,
           1.357819, 1.374085, 1.354995, 1.358437, 1.356971, 1.369972, 1.371606,
           1.369394, 1.366610, 1.362376, 1.356190, 1.346303, 1.338378, 1.328283,
           1.317659, 1.306252, 1.293387, 1.286872, 1.286886, 1.287030, 1.287030,
           1.287145, 1.287194, 1.287392, 1.287531, 1.287653, 1.287864, 1.288101,
           1.288396, 1.288885, 1.289397, 1.289969, 1.290914, 1.292230, 1.293921,
           1.295997}},
         {{1.390374, 1.388585, 1.388080, 1.394459, 1.397255, 1.401241, 1.407033,
           1.408570, 1.429703, 1.445579, 1.433434, 1.422862, 1.422007, 1.424972,
           1.421807, 1.423879, 1.423270, 1.421030, 1.417509, 1.415813, 1.413868,
           1.407006, 1.401611, 1.396613, 1.388668, 1.384111, 1.372277, 1.367510,
           1.357538, 1.354629, 1.348345, 1.348345, 1.348492, 1.349936, 1.349936,
           1.351083, 1.351566, 1.353542, 1.354937, 1.356148, 1.358265, 1.360631,
           1.363597, 1.368483, 1.373609, 1.378313, 1.379142, 1.380301, 1.381789,
           1.383613},
          {1.393449, 1.393950, 1.394997, 1.391866, 1.392739, 1.397744, 1.401338,
           1.414930, 1.442661, 1.475984, 1.448607, 1.444360, 1.449326, 1.440864,
           1.446785, 1.424073, 1.423664, 1.431110, 1.426247, 1.431453, 1.416033,
           1.423889, 1.408520, 1.402495, 1.408568, 1.406961, 1.396795, 1.372757,
           1.376316, 1.366998, 1.356582, 1.348345, 1.348492, 1.349936, 1.349936,
           1.351082, 1.351566, 1.353542, 1.354937, 1.356148, 1.358264, 1.360631,
           1.363597, 1.368482, 1.373610, 1.378313, 1.379144, 1.380302, 1.381789,
           1.383611}}},
        {{{1.323223, 1.323543, 1.325542, 1.326167, 1.327655, 1.324772, 1.329053,
           1.331947, 1.336911, 1.346274, 1.330946, 1.337986, 1.361937, 1.334331,
           1.343736, 1.338060, 1.335109, 1.339845, 1.326483, 1.342412, 1.330120,
           1.324878, 1.339142, 1.316929, 1.307715, 1.305114, 1.312948, 1.306819,
           1.290067, 1.290304, 1.290304, 1.290304, 1.290325, 1.290548, 1.290548,
           1.290725, 1.290797, 1.291102, 1.291316, 1.291502, 1.291826, 1.292189,
           1.292645, 1.293396, 1.294183, 1.295057, 1.296509, 1.298530, 1.301123,
           1.304309},
          {1.369318, 1.365450, 1.370612, 1.360400, 1.362457, 1.355549, 1.377581,
           1.381173, 1.386492, 1.380321, 1.386081, 1.353848, 1.372563, 1.373033,
           1.363518, 1.382289, 1.374833, 1.376785, 1.364889, 1.362621, 1.362190,
           1.350507, 1.358346, 1.346014, 1.348551, 1.344849, 1.338084, 1.326580,
           1.329601, 1.317790, 1.309290, 1.301059, 1.290325, 1.290548, 1.290548,
           1.290723, 1.290797, 1.291100, 1.291316, 1.291500, 1.291827, 1.292189,
           1.292646, 1.293396, 1.294181, 1.295058, 1.296509, 1.298528, 1.301125,
           1.304308}},
         {{1.363137, 1.366299, 1.365074, 1.368705, 1.358999, 1.364673, 1.366300,
           1.363582, 1.363190, 1.377416, 1.369098, 1.387900, 1.373807, 1.369210,
           1.384453, 1.365520, 1.368530, 1.363896, 1.356756, 1.353428, 1.359709,
           1.349135, 1.354153, 1.349772, 1.346867, 1.342423, 1.331105, 1.331105,
           1.331106, 1.331263, 1.331263, 1.331263, 1.331277, 1.331424, 1.331424,
           1.331540, 1.331588, 1.331791, 1.331933, 1.332055, 1.332270, 1.332509,
           1.332812, 1.333307, 1.333829, 1.334410, 1.335371, 1.336707, 1.338427,
           1.340537},
          {1.419354, 1.416992, 1.421361, 1.419500, 1.414798, 1.421489, 1.415966,
           1.414487, 1.419969, 1.426166, 1.427716, 1.435870, 1.419215, 1.442241,
           1.440173, 1.431730, 1.415267, 1.417488, 1.411135, 1.411070, 1.424274,
           1.423911, 1.399367, 1.414897, 1.402090, 1.391920, 1.404739, 1.385429,
           1.386769, 1.377722, 1.360065, 1.360554, 1.352648, 1.331424, 1.331424,
           1.331539, 1.331590, 1.331790, 1.331933, 1.332055, 1.332270, 1.332509,
           1.332812, 1.333308, 1.333830, 1.334410, 1.335368, 1.336709, 1.338426,
           1.340533}},
         {{1.397977, 1.397261, 1.399142, 1.395266, 1.398859, 1.403789, 1.411537,
           1.418719, 1.438120, 1.446316, 1.444454, 1.441469, 1.441097, 1.443720,
           1.436788, 1.435512, 1.433376, 1.435838, 1.426825, 1.429728, 1.422093,
           1.421104, 1.422928, 1.418773, 1.410909, 1.411675, 1.406384, 1.401074,
           1.392183, 1.393742, 1.393742, 1.393742, 1.393890, 1.395359, 1.395359,
           1.396524, 1.397015, 1.399023, 1.400439, 1.401672, 1.403823, 1.406229,
           1.409243, 1.414210, 1.419419, 1.424203, 1.425076, 1.426295, 1.427857,
           1.429775},
          {1.460263, 1.458515, 1.459522, 1.455382, 1.456767, 1.456647, 1.469309,
           1.477644, 1.495340, 1.495933, 1.492108, 1.492492, 1.496694, 1.502583,
           1.482203, 1.499760, 1.467901, 1.466512, 1.495714, 1.461562, 1.467797,
           1.491150, 1.477260, 1.483065, 1.452917, 1.466481, 1.438710, 1.462116,
           1.451609, 1.446392, 1.431016, 1.425773, 1.421471, 1.395359, 1.395359,
           1.396524, 1.397017, 1.399022, 1.400439, 1.401671, 1.403824, 1.406227,
           1.409243, 1.414208, 1.419420, 1.424202, 1.425078, 1.426293, 1.427857,
           1.429775}}},
        {{{1.437216, 1.432222, 1.425592, 1.422001, 1.418427, 1.425916, 1.419681,
           1.423542, 1.429930, 1.433657, 1.429572, 1.436394, 1.431778, 1.426034,
           1.429869, 1.440350, 1.438929, 1.441104, 1.440212, 1.444257, 1.458212,
           1.456377, 1.443915, 1.452631, 1.432437, 1.433639, 1.422878, 1.443882,
           1.417643, 1.422886, 1.436969, 1.430003, 1.424499, 1.405908, 1.409285,
           1.392567, 1.407574, 1.383484, 1.389738, 1.387330, 1.382270, 1.364607,
           1.358441, 1.358043, 1.355190, 1.365605, 1.356915, 1.351690, 1.353901,
           1.348643},
          {1.559235, 1.536422, 1.521659, 1.508585, 1.499444, 1.494733, 1.504597,
           1.500652, 1.511507, 1.504983, 1.502211, 1.506744, 1.503363, 1.500873,
           1.507936, 1.495202, 1.510284, 1.510865, 1.513441, 1.512422, 1.513061,
           1.500402, 1.504618, 1.506515, 1.508627, 1.474203, 1.488431, 1.496747,
           1.487201, 1.483645, 1.472104, 1.481782, 1.469777, 1.451688, 1.459584,
           1.443856, 1.437435, 1.435728, 1.408139, 1.414254, 1.415427, 1.392710,
           1.400714, 1.393416, 1.376327, 1.388904, 1.381690, 1.383293, 1.381004,
           1.345994}},
         {{1.481235, 1.473629, 1.467494, 1.464053, 1.460704, 1.468057, 1.459847,
           1.455021, 1.464002, 1.471849, 1.467425, 1.477591, 1.470628, 1.468532,
           1.471587, 1.486308, 1.488048, 1.491462, 1.484230, 1.480889, 1.492912,
           1.482929, 1.484608, 1.477582, 1.478587, 1.487530, 1.489626, 1.479033,
           1.471920, 1.468214, 1.467310, 1.456069, 1.468994, 1.449795, 1.458200,
           1.440168, 1.439528, 1.438645, 1.437883, 1.434799, 1.427546, 1.417918,
           1.411675, 1.414232, 1.410952, 1.408083, 1.398550, 1.397010, 1.401314,
           1.408632},
          {1.607458, 1.579084, 1.556212, 1.538551, 1.544625, 1.556494, 1.556068,
           1.552831, 1.560303, 1.548488, 1.558714, 1.550680, 1.552963, 1.558200,
           1.552751, 1.557769, 1.558154, 1.550058, 1.554762, 1.558121, 1.543080,
           1.556751, 1.555113, 1.549895, 1.540106, 1.558434, 1.548659, 1.552300,
           1.553937, 1.558029, 1.551292, 1.549833, 1.541551, 1.540871, 1.529858,
           1.529790, 1.524299, 1.510999, 1.499341, 1.497087, 1.490773, 1.468718,
           1.473216, 1.473696, 1.462370, 1.457029, 1.443983, 1.453458, 1.450090,
           1.412363}},
         {{1.507744, 1.498728, 1.492658, 1.489635, 1.487284, 1.497084, 1.499063,
           1.517564, 1.524859, 1.542099, 1.536435, 1.524441, 1.531411, 1.541715,
           1.538760, 1.540032, 1.541874, 1.547011, 1.554916, 1.560166, 1.578680,
           1.581422, 1.552690, 1.559161, 1.572792, 1.581648, 1.577349, 1.549635,
           1.578375, 1.571844, 1.551341, 1.581205, 1.557146, 1.563029, 1.550241,
           1.568040, 1.557175, 1.572182, 1.554461, 1.563792, 1.542547, 1.547418,
           1.528797, 1.544678, 1.546331, 1.539457, 1.550930, 1.534496, 1.533835,
           1.523058},
          {1.710097, 1.674235, 1.644870, 1.624142, 1.607290, 1.600557, 1.602653,
           1.631366, 1.651489, 1.681950, 1.651395, 1.633851, 1.645926, 1.643528,
           1.652175, 1.636472, 1.632880, 1.626438, 1.631446, 1.637413, 1.642611,
           1.631383, 1.657758, 1.687865, 1.689209, 1.650643, 1.694782, 1.658431,
           1.668726, 1.628704, 1.642936, 1.640026, 1.658221, 1.623969, 1.630622,
           1.603779, 1.627225, 1.620082, 1.622569, 1.598406, 1.585179, 1.610120,
           1.575738, 1.594002, 1.597919, 1.597113, 1.570878, 1.581056, 1.576222,
           1.543696}}},
        {{{2.056754, 2.030751, 2.004203, 1.976870, 1.964618, 1.943284, 1.915249,
           1.886499, 1.857120, 1.826993, 1.796166, 1.763535, 1.737141, 1.718352,
           1.731592, 1.716519, 1.709602, 1.711601, 1.701029, 1.707183, 1.700416,
           1.702165, 1.693234, 1.694989, 1.685817, 1.680457, 1.676092, 1.671100,
           1.665631, 1.658268, 1.653376, 1.646070, 1.640050, 1.632645, 1.626118,
           1.617898, 1.609647, 1.600885, 1.591570, 1.581966, 1.571549, 1.557300,
           1.550224, 1.540477, 1.528707, 1.520361, 1.509503, 1.500221, 1.495556,
           1.482531},
          {2.435432, 2.399067, 2.357507, 2.314255, 2.269764, 2.236437, 2.213006,
           2.168094, 2.123158, 2.078513, 2.032547, 1.984952, 1.936957, 1.895555,
           1.865316, 1.876739, 1.849473, 1.827827, 1.828851, 1.798612, 1.805162,
           1.783192, 1.777293, 1.756525, 1.751193, 1.731412, 1.716341, 1.707400,
           1.700398, 1.688423, 1.674162, 1.662466, 1.648624, 1.641031, 1.622239,
           1.610515, 1.596218, 1.582503, 1.568365, 1.555134, 1.540678, 1.526463,
           1.511908, 1.497832, 1.482482, 1.470031, 1.459180, 1.449923, 1.443310,
           1.436995}},
         {{2.033871, 2.008741, 1.998757, 1.975710, 1.950667, 1.924505, 1.898064,
           1.871643, 1.845867, 1.817998, 1.796722, 1.782361, 1.755276, 1.736557,
           1.722171, 1.727748, 1.720275, 1.723982, 1.724630, 1.738082, 1.738171,
           1.742615, 1.742100, 1.740389, 1.736232, 1.735904, 1.731775, 1.729189,
           1.723530, 1.719704, 1.715496, 1.714850, 1.705018, 1.701540, 1.692889,
           1.686092, 1.680803, 1.670173, 1.661738, 1.652879, 1.642582, 1.633548,
           1.624290, 1.614447, 1.605705, 1.597429, 1.584169, 1.580520, 1.578751,
           1.573249},
          {2.546486, 2.503263, 2.484078, 2.442799, 2.409827, 2.384209, 2.347123,
           2.327270, 2.295351, 2.272641, 2.237642, 2.209373, 2.174438, 2.146805,
           2.116065, 2.087731, 2.058830, 2.030877, 2.004603, 1.978088, 1.953671,
           1.928419, 1.905630, 1.881007, 1.858582, 1.837005, 1.818650, 1.797176,
           1.777845, 1.760661, 1.743842, 1.727325, 1.726764, 1.742707, 1.736077,
           1.724583, 1.711312, 1.696804, 1.681325, 1.663913, 1.650391, 1.632632,
           1.616411, 1.600564, 1.585443, 1.568886, 1.556698, 1.545009, 1.534757,
           1.521628}},
         {{2.076137, 2.047890, 2.038398, 2.013155, 1.988553, 1.968411, 1.952369,
           1.945821, 1.952470, 1.948945, 1.913723, 1.875462, 1.847748, 1.830142,
           1.826029, 1.824303, 1.827024, 1.824663, 1.829617, 1.827852, 1.828419,
           1.828146, 1.827819, 1.830377, 1.830369, 1.831113, 1.830166, 1.830722,
           1.830048, 1.831728, 1.831518, 1.830140, 1.828119, 1.827371, 1.823010,
           1.819519, 1.814884, 1.810966, 1.805606, 1.800281, 1.797189, 1.788379,
           1.782428, 1.776134, 1.766888, 1.763748, 1.754646, 1.745001, 1.738309,
           1.717036},
          {2.778311, 2.751899, 2.716560, 2.676082, 2.647291, 2.624050, 2.597360,
           2.593244, 2.608266, 2.601239, 2.525192, 2.464276, 2.424463, 2.388178,
           2.351417, 2.317419, 2.284092, 2.252563, 2.224677, 2.194346, 2.164733,
           2.135581, 2.108815, 2.084202, 2.061539, 2.038332, 2.015142, 1.993619,
           1.973482, 1.957721, 1.941022, 1.922553, 1.905525, 1.892507, 1.876336,
           1.862068, 1.846912, 1.841292, 1.833629, 1.819630, 1.805242, 1.790211,
           1.774654, 1.763314, 1.749800, 1.737984, 1.721581, 1.707144, 1.691446,
           1.682402}}},
        {{{2.770366, 2.717588, 2.666757, 2.615539, 2.568098, 2.524656, 2.480006,
           2.437452, 2.397652, 2.359071, 2.321806, 2.285931, 2.251762, 2.218807,
           2.187855, 2.158517, 2.129466, 2.101219, 2.074008, 2.049374, 2.025005,
           2.002805, 1.980724, 1.960317, 1.940434, 1.922046, 1.910503, 1.899827,
           1.885997, 1.872611, 1.859024, 1.845669, 1.832703, 1.820427, 1.807650,
           1.795773, 1.783452, 1.771116, 1.758092, 1.744523, 1.730413, 1.715761,
           1.700257, 1.685853, 1.671661, 1.658359, 1.646592, 1.636394, 1.628175,
           1.622061},
          {3.318878, 3.240870, 3.165355, 3.092810, 3.021711, 2.952761, 2.888464,
           2.825728, 2.768309, 2.715069, 2.661153, 2.608705, 2.555510, 2.508387,
           2.461446, 2.415238, 2.370960, 2.328432, 2.287573, 2.246900, 2.207159,
           2.169572, 2.133421, 2.097776, 2.063655, 2.031208, 2.001812, 1.971894,
           1.943853, 1.919233, 1.903961, 1.883025, 1.861040, 1.839753, 1.818474,
           1.797780, 1.776931, 1.757500, 1.737616, 1.717299, 1.697144, 1.676794,
           1.656634, 1.637976, 1.619273, 1.602142, 1.586695, 1.573609, 1.563836,
           1.555986}},
         {{2.745054, 2.694664, 2.644812, 2.597284, 2.551740, 2.509690, 2.468357,
           2.429587, 2.393711, 2.358038, 2.317703, 2.279520, 2.245327, 2.213662,
           2.183850, 2.154095, 2.125417, 2.099445, 2.074158, 2.049931, 2.027534,
           2.006833, 1.986173, 1.967237, 1.949500, 1.932713, 1.939455, 1.955634,
           1.953345, 1.945735, 1.935925, 1.925488, 1.914963, 1.904682, 1.893899,
           1.883294, 1.871868, 1.860411, 1.848616, 1.835781, 1.822362, 1.808692,
           1.794505, 1.781267, 1.767490, 1.755080, 1.744555, 1.736179, 1.729943,
           1.725333},
          {3.488165, 3.405537, 3.323018, 3.242300, 3.165364, 3.093519, 3.025160,
           2.962159, 2.907616, 2.852122, 2.789863, 2.729767, 2.676361, 2.623271,
           2.572896, 2.526636, 2.479803, 2.434479, 2.390301, 2.348887, 2.307679,
           2.267740, 2.229624, 2.193650, 2.158577, 2.126265, 2.094056, 2.063988,
           2.035077, 2.010391, 1.984364, 1.959772, 1.936200, 1.942690, 1.955207,
           1.942872, 1.923978, 1.904235, 1.882764, 1.860629, 1.837992, 1.815135,
           1.792375, 1.768913, 1.747144, 1.727303, 1.709011, 1.693104, 1.678830,
           1.666431}},
         {{2.807851, 2.749821, 2.694528, 2.649216, 2.603472, 2.569437, 2.541743,
           2.533705, 2.545820, 2.541800, 2.465275, 2.406102, 2.369520, 2.339428,
           2.310489, 2.282563, 2.256698, 2.233471, 2.213668, 2.194093, 2.176177,
           2.159518, 2.143637, 2.132287, 2.120854, 2.109183, 2.098576, 2.089012,
           2.080377, 2.078175, 2.073884, 2.070519, 2.068088, 2.066019, 2.059322,
           2.053787, 2.046144, 2.039770, 2.031358, 2.021654, 2.012431, 2.002839,
           1.992696, 1.984782, 1.976016, 1.966614, 1.951377, 1.936632, 1.923391,
           1.911152},
          {3.672086, 3.580732, 3.490907, 3.538237, 3.500549, 3.440830, 3.389510,
           3.363547, 3.370818, 3.346873, 3.214953, 3.110577, 3.039100, 2.974270,
           2.913087, 2.852352, 2.794022, 2.739022, 2.689490, 2.639437, 2.592443,
           2.547992, 2.504923, 2.467043, 2.431199, 2.394606, 2.360897, 2.327479,
           2.298039, 2.272933, 2.244987, 2.217949, 2.193474, 2.173294, 2.148586,
           2.126892, 2.103590, 2.088071, 2.077814, 2.057770, 2.035742, 2.014087,
           1.992675, 1.973860, 1.956006, 1.937146, 1.914526, 1.892957, 1.873106,
           1.856862}}},
        {{{3.639941, 3.551577, 3.465221, 3.383233, 3.305572, 3.229285, 3.156671,
           3.088430, 3.021657, 2.958056, 2.898283, 2.841246, 2.785513, 2.733039,
           2.683521, 2.634853, 2.587838, 2.543443, 2.502003, 2.462374, 2.423949,
           2.387807, 2.353226, 2.321074, 2.290364, 2.261217, 2.235728, 2.218385,
           2.199851, 2.178832, 2.158550, 2.138221, 2.118537, 2.100148, 2.081719,
           2.063942, 2.046047, 2.028050, 2.009551, 1.990945, 1.971317, 1.951047,
           1.930436, 1.910480, 1.891076, 1.872468, 1.855898, 1.841878, 1.829809,
           1.820131},
          {4.326247, 4.226168, 4.130031, 4.032423, 3.935547, 3.841909, 3.750310,
           3.660557, 3.576715, 3.495789, 3.414862, 3.337018, 3.258135, 3.180910,
           3.106299, 3.032401, 2.960751, 2.892348, 2.825622, 2.761115, 2.700857,
           2.642011, 2.585252, 2.532106, 2.482627, 2.434737, 2.389217, 2.345626,
           2.305156, 2.265847, 2.232156, 2.208819, 2.177696, 2.147523, 2.116276,
           2.086204, 2.056655, 2.027729, 1.999296, 1.970612, 1.942070, 1.913363,
           1.884776, 1.857206, 1.831465, 1.807085, 1.785550, 1.767037, 1.751693,
           1.739260}},
         {{3.599289, 3.513257, 3.429821, 3.352474, 3.276582, 3.204621, 3.137397,
           3.075112, 3.016614, 2.956890, 2.892256, 2.831563, 2.776861, 2.726297,
           2.677209, 2.630209, 2.584977, 2.542564, 2.503430, 2.465492, 2.429133,
           2.395021, 2.362575, 2.333104, 2.304980, 2.278243, 2.253187, 2.263476,
           2.285171, 2.277405, 2.263255, 2.248401, 2.232288, 2.216394, 2.200239,
           2.184046, 2.167311, 2.150425, 2.132882, 2.114107, 2.095046, 2.076022,
           2.056399, 2.037449, 2.019147, 2.001119, 1.984973, 1.971344, 1.960447,
           1.951874},
          {4.551855, 4.442926, 4.337405, 4.237619, 4.142987, 4.049960, 3.960046,
           3.877486, 3.803126, 3.724956, 3.630221, 3.539225, 3.452410, 3.369920,
           3.287397, 3.207195, 3.130645, 3.056177, 2.984793, 2.917136, 2.852313,
           2.789297, 2.730875, 2.675273, 2.622276, 2.571463, 2.524400, 2.480408,
           2.437775, 2.398521, 2.360297, 2.324687, 2.289710, 2.256410, 2.271134,
           2.279678, 2.257637, 2.230399, 2.200522, 2.169079, 2.136972, 2.103807,
           2.071159, 2.038637, 2.007115, 1.976476, 1.948108, 1.923103, 1.901881,
           1.883503}},
         {{3.687271, 3.591291, 3.503087, 3.421290, 3.347797, 3.290453, 3.245513,
           3.225213, 3.237368, 3.224429, 3.107033, 3.015803, 2.959166, 2.908644,
           2.860959, 2.815662, 2.773622, 2.734848, 2.703043, 2.670938, 2.639525,
           2.610691, 2.584109, 2.563188, 2.542614, 2.522491, 2.503386, 2.485768,
           2.469334, 2.459078, 2.447710, 2.439929, 2.434285, 2.432328, 2.421888,
           2.412709, 2.400462, 2.390582, 2.378454, 2.363650, 2.348548, 2.332138,
           2.315317, 2.301083, 2.286375, 2.270365, 2.248556, 2.227719, 2.207850,
           2.189298},
          {4.780752, 4.775604, 4.839072, 4.752818, 4.648761, 4.562685, 4.497070,
           4.467980, 4.487332, 4.452819, 4.246990, 4.086411, 3.978128, 3.878615,
           3.781499, 3.689965, 3.599075, 3.511617, 3.434243, 3.357464, 3.280925,
           3.207978, 3.141711, 3.081258, 3.021738, 2.965179, 2.910253, 2.858134,
           2.808272, 2.767583, 2.724214, 2.682191, 2.642299, 2.608803, 2.571170,
           2.539396, 2.505091, 2.475682, 2.454286, 2.432078, 2.401864, 2.369502,
           2.336146, 2.306009, 2.276553, 2.247978, 2.213889, 2.182159, 2.152865,
           2.126317}}},
        {{{5.987286, 5.802512, 5.625291, 5.457058, 5.293017, 5.138448, 4.989769,
           4.846903, 4.709543, 4.576399, 4.448321, 4.326974, 4.211236, 4.099867,
           3.993239, 3.894167, 3.797627, 3.706337, 3.620438, 3.537862, 3.460244,
           3.386708, 3.317329, 3.250314, 3.188216, 3.129245, 3.074207, 3.020438,
           2.969814, 2.935889, 2.905604, 2.866612, 2.826542, 2.788086, 2.749542,
           2.712958, 2.677078, 2.642534, 2.606689, 2.570556, 2.534849, 2.500339,
           2.467323, 2.434534, 2.402375, 2.371109, 2.342740, 2.317280, 2.296038,
           2.277738},
          {6.534541, 6.364361, 6.200190, 6.041954, 5.884536, 5.727872, 5.579988,
           5.436052, 5.297128, 5.166592, 5.039241, 4.911603, 4.787948, 4.667427,
           4.549752, 4.436026, 4.327475, 4.223506, 4.121676, 4.020030, 3.920915,
           3.821766, 3.725173, 3.632162, 3.541450, 3.454164, 3.370179, 3.288304,
           3.209998, 3.135219, 3.065224, 2.996951, 2.942846, 2.901919, 2.847357,
           2.792590, 2.738260, 2.686629, 2.636930, 2.586449, 2.537519, 2.490273,
           2.445300, 2.401602, 2.358636, 2.316762, 2.279212, 2.245250, 2.216377,
           2.191951}},
         {{5.875741, 5.698550, 5.533586, 5.372499, 5.218237, 5.076158, 4.938288,
           4.810687, 4.691753, 4.567734, 4.428109, 4.302243, 4.188147, 4.080256,
           3.978156, 3.881552, 3.788399, 3.700640, 3.619181, 3.540675, 3.466375,
           3.396754, 3.330463, 3.267686, 3.209476, 3.153807, 3.102241, 3.052056,
           3.004718, 2.999029, 3.032603, 3.031599, 3.004374, 2.972755, 2.939093,
           2.904879, 2.870614, 2.836631, 2.803465, 2.769019, 2.734103, 2.699097,
           2.663995, 2.630386, 2.598363, 2.566506, 2.537366, 2.512339, 2.491322,
           2.472050},
          {6.930589, 6.742763, 6.559763, 6.382443, 6.210835, 6.049268, 5.894693,
           5.750272, 5.627313, 5.498851, 5.340952, 5.195516, 5.063412, 4.935912,
           4.813816, 4.695662, 4.581079, 4.470936, 4.367289, 4.268297, 4.171502,
           4.074902, 3.979335, 3.886183, 3.792860, 3.701892, 3.614613, 3.529288,
           3.446970, 3.370766, 3.294962, 3.222992, 3.154047, 3.089975, 3.025892,
           2.998972, 3.043756, 3.010291, 2.959965, 2.904317, 2.847564, 2.791749,
           2.734733, 2.679301, 2.625956, 2.575484, 2.527508, 2.483800, 2.444271,
           2.409655}},
         {{6.020279, 5.820211, 5.635779, 5.476201, 5.329253, 5.213400, 5.123295,
           5.082983, 5.104283, 5.065927, 4.819854, 4.629669, 4.507396, 4.400620,
           4.298854, 4.202370, 4.111991, 4.027650, 3.954241, 3.882038, 3.811522,
           3.747262, 3.685954, 3.635101, 3.585022, 3.535882, 3.490319, 3.448558,
           3.408797, 3.379917, 3.344838, 3.312494, 3.295171, 3.286339, 3.271737,
           3.250709, 3.223954, 3.199284, 3.170892, 3.139602, 3.109156, 3.078178,
           3.047651, 3.021794, 2.994611, 2.965569, 2.926095, 2.889156, 2.853019,
           2.816221},
          {8.136379, 7.840681, 7.554564, 7.299039, 7.073903, 6.900886, 6.774121,
           6.723380, 6.762100, 6.703511, 6.327013, 6.044272, 5.868398, 5.711868,
           5.561480, 5.417902, 5.279572, 5.146933, 5.028473, 4.909746, 4.792744,
           4.680399, 4.574167, 4.481971, 4.391578, 4.300104, 4.213790, 4.131006,
           4.049918, 3.979522, 3.900697, 3.823533, 3.750911, 3.685755, 3.613467,
           3.548584, 3.481388, 3.421480, 3.358098, 3.313832, 3.268257, 3.207100,
           3.145389, 3.089476, 3.036199, 2.982022, 2.919537, 2.863421, 2.812592,
           2.765499}}},
        {{{9.720677, 9.413441, 9.123161, 8.831715, 8.531210, 8.246660, 7.971404,
           7.706102, 7.452040, 7.206594, 6.972416, 6.745875, 6.530653, 6.326086,
           6.130115, 5.940491, 5.759429, 5.587737, 5.425810, 5.270703, 5.121273,
           4.978468, 4.845240, 4.715156, 4.589946, 4.471597, 4.359576, 4.252249,
           4.151748, 4.057045, 3.967222, 3.907365, 3.853081, 3.783641, 3.711014,
           3.640923, 3.572498, 3.507858, 3.445483, 3.384285, 3.324008, 3.265315,
           3.207926, 3.154072, 3.100183, 3.048251, 3.001131, 2.959380, 2.922209,
           2.891462},
          {9.577579, 9.304796, 9.037866, 8.765567, 8.488701, 8.226751, 7.974510,
           7.735039, 7.510277, 7.296493, 7.088417, 6.881180, 6.680043, 6.487128,
           6.302871, 6.124953, 5.952474, 5.786186, 5.625676, 5.471508, 5.322065,
           5.177759, 5.039134, 4.909104, 4.786104, 4.664868, 4.548546, 4.439455,
           4.334992, 4.235503, 4.140325, 4.046801, 3.955061, 3.896986, 3.827994,
           3.744881, 3.657868, 3.573630, 3.490985, 3.411214, 3.332069, 3.255603,
           3.182021, 3.112958, 3.045477, 2.981746, 2.923903, 2.873705, 2.829323,
           2.791989}},
         {{9.431574, 9.153768, 8.875576, 8.590433, 8.318314, 8.058932, 7.810083,
           7.577230, 7.359644, 7.132984, 6.872234, 6.639454, 6.431529, 6.235711,
           6.046085, 5.864730, 5.692817, 5.530100, 5.376493, 5.228709, 5.086374,
           4.953506, 4.828459, 4.706636, 4.589655, 4.479918, 4.375368, 4.276218,
           4.183073, 4.096907, 4.014408, 4.020936, 4.095368, 4.064111, 4.007379,
           3.943243, 3.877480, 3.812509, 3.748069, 3.684495, 3.622594, 3.560183,
           3.499350, 3.440994, 3.384809, 3.331823, 3.281885, 3.237572, 3.196318,
           3.158650},
          {10.111389, 9.806401, 9.515425, 9.241727, 8.973993, 8.708856,
           8.454848,  8.227391, 8.026329, 7.820435, 7.570251, 7.336547,
           7.125194,  6.920951, 6.724910, 6.534935, 6.351746, 6.173443,
           6.001200,  5.834603, 5.674770, 5.522355, 5.376003, 5.237189,
           5.103872,  4.977907, 4.861103, 4.745956, 4.635597, 4.532901,
           4.435036,  4.340348, 4.250067, 4.164642, 4.079039, 3.993789,
           4.050452,  4.081330, 4.016932, 3.929793, 3.836832, 3.742041,
           3.649293,  3.557437, 3.469109, 3.385175, 3.307724, 3.235781,
           3.171057,  3.113139}},
         {{9.520707, 9.168619, 8.844236, 8.554390, 8.295324, 8.096759, 7.948403,
           7.886411, 7.936514, 7.865484, 7.409692, 7.062057, 6.836916, 6.636768,
           6.448763, 6.271176, 6.102293, 5.943786, 5.807574, 5.673984, 5.546085,
           5.423085, 5.305377, 5.206541, 5.109066, 5.013603, 4.922680, 4.837166,
           4.757587, 4.695655, 4.624165, 4.555219, 4.498935, 4.473935, 4.446327,
           4.402131, 4.349248, 4.300203, 4.246240, 4.190392, 4.137398, 4.084468,
           4.032927, 3.989430, 3.945117, 3.898066, 3.830891, 3.763886, 3.698517,
           3.633912},
          {11.896656, 11.392986, 10.945216, 10.566372, 10.248519, 10.004575,
           9.813623,  9.747384,  9.834585,  9.743170,  9.126576,  8.668616,
           8.385639,  8.128043,  7.879651,  7.642277,  7.413921,  7.199717,
           7.008516,  6.814430,  6.623744,  6.443977,  6.273279,  6.123215,
           5.973104,  5.825820,  5.687372,  5.558113,  5.431685,  5.324790,
           5.207961,  5.098289,  4.993914,  4.904534,  4.805294,  4.718790,
           4.629633,  4.553571,  4.504903,  4.449249,  4.372447,  4.290146,
           4.208985,  4.134807,  4.059787,  3.983508,  3.890515,  3.802687,
           3.721283,  3.645485}}},
        {{{16.876550, 16.362172, 15.856645, 15.362422, 14.867690, 14.379122,
           13.896396, 13.422554, 12.963398, 12.513385, 12.073779, 11.646783,
           11.231458, 10.831098, 10.447442, 10.071682, 9.712430,  9.369011,
           9.034119,  8.721755,  8.423324,  8.140429,  7.870162,  7.606124,
           7.346365,  7.102376,  6.873312,  6.654451,  6.445775,  6.250209,
           6.065912,  5.893356,  5.735607,  5.644885,  5.536131,  5.401134,
           5.265803,  5.136517,  5.012817,  4.891691,  4.776152,  4.664921,
           4.556622,  4.454660,  4.356729,  4.262423,  4.177091,  4.098915,
           4.029029,  3.964494},
          {14.322385, 13.859041, 13.410428, 12.980354, 12.564016, 12.160691,
           11.774485, 11.404624, 11.056484, 10.728270, 10.405048, 10.076021,
           9.755161,  9.444280,  9.137722,  8.844578,  8.563377,  8.292804,
           8.035188,  7.785736,  7.539307,  7.297365,  7.069533,  6.853761,
           6.647453,  6.449572,  6.262897,  6.086052,  5.919327,  5.760045,
           5.661809,  5.566664,  5.434494,  5.304410,  5.176729,  5.057051,
           4.941889,  4.832466,  4.727690,  4.624552,  4.524779,  4.428453,
           4.334850,  4.245525,  4.158810,  4.072719,  3.993449,  3.918938,
           3.851934,  3.792848}},
         {{16.036242, 15.571089, 15.113038, 14.660556, 14.222879, 13.793046,
           13.377299, 12.981891, 12.600387, 12.179653, 11.687520, 11.248269,
           10.855747, 10.483829, 10.117965, 9.771442,  9.439734,  9.116622,
           8.817396,  8.529606,  8.255877,  7.995574,  7.742130,  7.494640,
           7.256473,  7.035143,  6.824303,  6.623572,  6.432544,  6.255175,
           6.085438,  5.927126,  5.823142,  5.966423,  6.020464,  5.914196,
           5.781488,  5.644187,  5.508714,  5.377166,  5.249277,  5.129789,
           5.013373,  4.902104,  4.795924,  4.691941,  4.592674,  4.500596,
           4.416463,  4.340885},
          {15.123104, 14.636119, 14.169038, 13.716781, 13.290915, 12.894663,
           12.520887, 12.179990, 11.873194, 11.548972, 11.145345, 10.768037,
           10.425940, 10.088363, 9.768365,  9.459311,  9.155657,  8.870229,
           8.598916,  8.338971,  8.091320,  7.850635,  7.613762,  7.384201,
           7.167435,  6.962454,  6.766035,  6.578009,  6.399135,  6.232854,
           6.072783,  5.922779,  5.821573,  5.967171,  6.012109,  5.884888,
           5.734390,  5.583618,  5.440628,  5.301439,  5.170830,  5.046213,
           4.925112,  4.809860,  4.699826,  4.594012,  4.495104,  4.404039,
           4.321840,  4.246758}},
         {{15.431833, 14.872586, 14.359498, 13.903464, 13.494933, 13.183423,
           12.948916, 12.877853, 13.007467, 12.908250, 12.078034, 11.455045,
           11.066537, 10.718765, 10.391709, 10.081308, 9.788568,  9.513524,
           9.273975,  9.027297,  8.776866,  8.535925,  8.310418,  8.119287,
           7.923797,  7.730104,  7.543932,  7.368425,  7.202439,  7.068472,
           6.918913,  6.775979,  6.673372,  6.628549,  6.558085,  6.454171,
           6.338331,  6.233535,  6.125606,  6.017087,  5.918341,  5.818064,
           5.717967,  5.626606,  5.533198,  5.435539,  5.313967,  5.196256,
           5.083141,  4.975466},
          {17.542212, 16.844221, 16.211803, 15.656655, 15.164214, 14.798683,
           14.538607, 14.463698, 14.631999, 14.506940, 13.542716, 12.817961,
           12.363580, 11.953909, 11.562708, 11.191336, 10.834656, 10.494127,
           10.197013, 9.898755,  9.607005,  9.327735,  9.054327,  8.806538,
           8.552458,  8.306166,  8.074503,  7.852667,  7.640609,  7.462948,
           7.269277,  7.084997,  6.912727,  6.765992,  6.661366,  6.601442,
           6.464656,  6.326698,  6.185275,  6.043980,  5.914348,  5.787745,
           5.668182,  5.562421,  5.460233,  5.358403,  5.235929,  5.120960,
           5.013781,  4.914612}}},
        {{{36.196020, 35.087955, 34.029887, 33.033120, 32.042657, 31.052685,
           30.063425, 29.100974, 28.158405, 27.215733, 26.289978, 25.387854,
           24.498039, 23.631167, 22.791155, 21.968932, 21.170419, 20.391995,
           19.641035, 18.913982, 18.211774, 17.538113, 16.881795, 16.253814,
           15.652710, 15.073252, 14.517238, 13.981136, 13.467574, 12.973903,
           12.499650, 12.045678, 11.617438, 11.252903, 11.090009, 10.839853,
           10.464269, 10.093383, 9.728664,  9.374826,  9.040237,  8.720234,
           8.418567,  8.138551,  7.870226,  7.620774,  7.394620,  7.191160,
           7.013081,  6.853511},
          {24.893909, 24.012999, 23.158125, 22.320552, 21.511378, 20.728373,
           19.978780, 19.271597, 18.601216, 17.971342, 17.359562, 16.749735,
           16.162928, 15.598686, 15.053762, 14.530054, 14.024224, 13.538904,
           13.073725, 12.628843, 12.204309, 11.801610, 11.415728, 11.173616,
           11.026307, 10.726529, 10.377538, 10.039957, 9.718264,  9.420103,
           9.127081,  8.854139,  8.593226,  8.352380,  8.117881,  7.896216,
           7.686697,  7.491297,  7.300937,  7.114760,  6.943323,  6.775305,
           6.616790,  6.468295,  6.327773,  6.192281,  6.073086,  5.968537,
           5.878645,  5.801776}},
         {{32.977815, 32.159172, 31.343623, 30.524853, 29.718699, 28.937654,
           28.166426, 27.397238, 26.635965, 25.745804, 24.688075, 23.766604,
           22.947222, 22.155681, 21.386959, 20.638456, 19.916001, 19.217628,
           18.547138, 17.897225, 17.265202, 16.655783, 16.072902, 15.516129,
           14.976220, 14.454348, 13.952914, 13.473030, 13.010663, 12.572972,
           12.147892, 11.747312, 11.532218, 12.221224, 12.566548, 12.207460,
           11.742510, 11.280464, 10.829563, 10.402395, 10.007203, 9.631943,
           9.280687,  8.953722,  8.649429,  8.362932,  8.101364,  7.859429,
           7.642469,  7.443936},
          {26.270980, 25.361795, 24.469569, 23.612848, 22.792932, 22.017348,
           21.289609, 20.626731, 20.045578, 19.457510, 18.737874, 18.064803,
           17.451324, 16.859878, 16.295255, 15.752908, 15.229455, 14.726855,
           14.245967, 13.782555, 13.336073, 12.904385, 12.490556, 12.098280,
           11.724316, 11.517304, 12.199783, 12.552432, 12.158001, 11.680984,
           11.223538, 10.795052, 10.399386, 10.043145, 9.708599,  9.397859,
           9.111716,  8.841271,  8.591487,  8.350709,  8.125209,  7.907874,
           7.704483,  7.507928,  7.323917,  7.147575,  6.985212,  6.834274,
           6.697010,  6.572205}},
         {{29.821165, 28.842568, 27.928776, 27.117462, 26.390147, 25.846322,
           25.453251, 25.375999, 25.703011, 25.546149, 23.877226, 22.623387,
           21.850394, 21.164599, 20.506861, 19.873368, 19.266866, 18.686705,
           18.178744, 17.661138, 17.142588, 16.643158, 16.163366, 15.746489,
           15.316967, 14.888813, 14.478443, 14.087413, 13.709756, 13.540172,
           13.462208, 13.305621, 12.975714, 12.628916, 12.272426, 11.936853,
           11.603254, 11.297866, 10.997583, 10.704612, 10.431856, 10.167847,
           9.924233,  9.706883,  9.501863,  9.295102,  9.049303,  8.813755,
           8.592145,  8.380336},
          {29.699182, 28.573234, 27.525595, 26.591873, 25.763097, 25.131544,
           24.677543, 24.553773, 24.835325, 24.652424, 23.014477, 21.775031,
           20.996728, 20.298052, 19.626907, 18.983769, 18.365573, 17.771195,
           17.246766, 16.713099, 16.181431, 15.671123, 15.178836, 14.747688,
           14.309099, 13.874542, 13.518488, 13.438322, 13.297310, 12.936990,
           12.532750, 12.142583, 11.766471, 11.425813, 11.085942, 10.779476,
           10.478480, 10.208426, 9.945435,  9.688612,  9.456215,  9.234513,
           9.026480,  8.845564,  8.678118,  8.508473,  8.301600,  8.106697,
           7.925388,  7.755302}}}},
       {{{{1.577472, 1.559461, 1.585009, 1.570833, 1.566897, 1.578707, 1.562984,
           1.558948, 1.547341, 1.559006, 1.547955, 1.536006, 1.527862, 1.512831,
           1.503524, 1.497147, 1.493162, 1.480606, 1.458481, 1.441762, 1.425814,
           1.399262, 1.387899, 1.388012, 1.388058, 1.388058, 1.388058, 1.388058,
           1.388060, 1.388222, 1.388222, 1.388222, 1.388238, 1.388393, 1.388393,
           1.388513, 1.388564, 1.388773, 1.388921, 1.389049, 1.389272, 1.389522,
           1.389835, 1.390351, 1.390892, 1.391496, 1.392494, 1.393886, 1.395673,
           1.397863},
          {1.633163, 1.632294, 1.620560, 1.660339, 1.671177, 1.668215, 1.663578,
           1.660905, 1.648379, 1.635165, 1.636914, 1.628944, 1.579647, 1.596998,
           1.581892, 1.591633, 1.556967, 1.551689, 1.538942, 1.503993, 1.492625,
           1.474268, 1.440536, 1.414385, 1.388058, 1.388058, 1.388058, 1.388058,
           1.388060, 1.388222, 1.388222, 1.388222, 1.388238, 1.388393, 1.388393,
           1.388513, 1.388564, 1.388772, 1.388921, 1.389049, 1.389272, 1.389522,
           1.389835, 1.390351, 1.390893, 1.391497, 1.392494, 1.393885, 1.395672,
           1.397866}},
         {{1.564283, 1.566880, 1.575980, 1.569434, 1.571889, 1.553758, 1.561639,
           1.554270, 1.561965, 1.549638, 1.543723, 1.531272, 1.538059, 1.523641,
           1.506351, 1.506459, 1.492736, 1.466606, 1.455558, 1.459984, 1.445507,
           1.424178, 1.416472, 1.416555, 1.416590, 1.416590, 1.416590, 1.416590,
           1.416590, 1.416712, 1.416712, 1.416712, 1.416722, 1.416833, 1.416833,
           1.416923, 1.416961, 1.417114, 1.417222, 1.417317, 1.417483, 1.417667,
           1.417896, 1.418277, 1.418674, 1.419120, 1.419854, 1.420880, 1.422194,
           1.423804},
          {1.618647, 1.607913, 1.618416, 1.639663, 1.614230, 1.620401, 1.629211,
           1.595601, 1.596835, 1.614293, 1.589134, 1.591521, 1.588607, 1.584898,
           1.556774, 1.545658, 1.529590, 1.518684, 1.506541, 1.485176, 1.476879,
           1.446476, 1.442903, 1.422165, 1.416590, 1.416590, 1.416590, 1.416590,
           1.416590, 1.416712, 1.416712, 1.416712, 1.416721, 1.416833, 1.416833,
           1.416924, 1.416961, 1.417116, 1.417222, 1.417316, 1.417482, 1.417666,
           1.417896, 1.418276, 1.418676, 1.419120, 1.419854, 1.420880, 1.422194,
           1.423809}},
         {{1.592620, 1.599704, 1.584973, 1.601783, 1.604145, 1.593995, 1.599424,
           1.610180, 1.598130, 1.609325, 1.599463, 1.590843, 1.587924, 1.564325,
           1.563927, 1.557700, 1.545389, 1.526701, 1.512279, 1.499769, 1.483970,
           1.470196, 1.462139, 1.462978, 1.463321, 1.463321, 1.463321, 1.463321,
           1.463335, 1.464530, 1.464530, 1.464530, 1.464642, 1.465765, 1.465765,
           1.466658, 1.467034, 1.468569, 1.469655, 1.470597, 1.472245, 1.474084,
           1.476392, 1.480194, 1.484181, 1.487840, 1.488496, 1.489410, 1.490582,
           1.492023},
          {1.602552, 1.605003, 1.605284, 1.602761, 1.599390, 1.595785, 1.598254,
           1.607755, 1.628155, 1.615020, 1.591811, 1.613604, 1.610666, 1.573912,
           1.564694, 1.551321, 1.544086, 1.541192, 1.522259, 1.508470, 1.496811,
           1.478909, 1.462139, 1.462976, 1.463321, 1.463321, 1.463321, 1.463321,
           1.463332, 1.464530, 1.464530, 1.464530, 1.464644, 1.465765, 1.465765,
           1.466658, 1.467036, 1.468569, 1.469655, 1.470597, 1.472244, 1.474084,
           1.476392, 1.480193, 1.484181, 1.487840, 1.488495, 1.489410, 1.490583,
           1.492024}}},
        {{{1.558770, 1.557331, 1.557825, 1.561076, 1.561445, 1.561997, 1.540608,
           1.555764, 1.562641, 1.550775, 1.546803, 1.555765, 1.542976, 1.550419,
           1.544952, 1.532453, 1.527906, 1.529277, 1.523277, 1.514845, 1.507060,
           1.497942, 1.481364, 1.469921, 1.462767, 1.448526, 1.438504, 1.417293,
           1.402848, 1.396085, 1.396085, 1.396085, 1.396102, 1.396257, 1.396257,
           1.396380, 1.396432, 1.396644, 1.396793, 1.396924, 1.397150, 1.397404,
           1.397724, 1.398246, 1.398797, 1.399408, 1.400423, 1.401835, 1.403648,
           1.405875},
          {1.515642, 1.518731, 1.522759, 1.533998, 1.516174, 1.535783, 1.532368,
           1.523009, 1.531608, 1.528247, 1.517498, 1.521793, 1.527976, 1.525631,
           1.511931, 1.505877, 1.508883, 1.506236, 1.486312, 1.492357, 1.480224,
           1.476255, 1.462439, 1.450001, 1.446663, 1.425495, 1.409891, 1.395920,
           1.395920, 1.396085, 1.396085, 1.396085, 1.396103, 1.396257, 1.396257,
           1.396381, 1.396432, 1.396644, 1.396793, 1.396923, 1.397151, 1.397405,
           1.397722, 1.398247, 1.398797, 1.399409, 1.400423, 1.401834, 1.403648,
           1.405875}},
         {{1.550821, 1.557895, 1.557321, 1.560867, 1.559349, 1.556605, 1.539375,
           1.547009, 1.540330, 1.571310, 1.567819, 1.532254, 1.539345, 1.533276,
           1.523101, 1.548089, 1.523326, 1.509198, 1.529369, 1.504951, 1.496716,
           1.495897, 1.477473, 1.476763, 1.466348, 1.465557, 1.457875, 1.439941,
           1.424858, 1.424980, 1.424980, 1.424980, 1.424989, 1.425102, 1.425102,
           1.425194, 1.425230, 1.425386, 1.425493, 1.425590, 1.425756, 1.425940,
           1.426171, 1.426556, 1.426956, 1.427401, 1.428142, 1.429173, 1.430496,
           1.432117},
          {1.522414, 1.529715, 1.526507, 1.527640, 1.523829, 1.529233, 1.540309,
           1.537960, 1.545396, 1.541466, 1.538168, 1.541152, 1.533825, 1.519604,
           1.515953, 1.511813, 1.521865, 1.513448, 1.509676, 1.496211, 1.487272,
           1.472573, 1.468538, 1.472998, 1.468213, 1.459427, 1.444817, 1.428699,
           1.424859, 1.424980, 1.424980, 1.424980, 1.424990, 1.425102, 1.425102,
           1.425192, 1.425230, 1.425386, 1.425493, 1.425589, 1.425755, 1.425940,
           1.426171, 1.426556, 1.426956, 1.427403, 1.428141, 1.429172, 1.430495,
           1.432119}},
         {{1.586270, 1.584515, 1.587504, 1.586456, 1.581791, 1.587603, 1.595109,
           1.604918, 1.609280, 1.624598, 1.613252, 1.597281, 1.600215, 1.592562,
           1.592653, 1.584446, 1.579467, 1.574134, 1.568760, 1.565931, 1.552381,
           1.539569, 1.542185, 1.528140, 1.517989, 1.514367, 1.500868, 1.485545,
           1.474851, 1.473090, 1.473090, 1.473090, 1.473202, 1.474332, 1.474332,
           1.475230, 1.475611, 1.477154, 1.478245, 1.479194, 1.480850, 1.482702,
           1.485025, 1.488849, 1.492862, 1.496542, 1.497199, 1.498116, 1.499293,
           1.500734},
          {1.560917, 1.573266, 1.571751, 1.566193, 1.553547, 1.569005, 1.575859,
           1.593039, 1.584676, 1.610836, 1.584253, 1.579808, 1.576679, 1.566110,
           1.565691, 1.581076, 1.572671, 1.565554, 1.552728, 1.553933, 1.544858,
           1.521978, 1.522686, 1.514609, 1.512075, 1.494459, 1.488356, 1.476890,
           1.471877, 1.473090, 1.473090, 1.473090, 1.473204, 1.474332, 1.474332,
           1.475231, 1.475611, 1.477154, 1.478245, 1.479194, 1.480852, 1.482703,
           1.485024, 1.488847, 1.492861, 1.496542, 1.497199, 1.498117, 1.499292,
           1.500737}}},
        {{{1.473885, 1.466265, 1.469701, 1.466283, 1.465797, 1.470556, 1.473974,
           1.481297, 1.486714, 1.486493, 1.480110, 1.473726, 1.474620, 1.471986,
           1.476851, 1.472364, 1.475507, 1.474129, 1.468791, 1.459153, 1.465474,
           1.461657, 1.457658, 1.453024, 1.447753, 1.438259, 1.434293, 1.422839,
           1.419526, 1.419698, 1.419698, 1.419698, 1.419714, 1.419876, 1.419876,
           1.420005, 1.420060, 1.420281, 1.420437, 1.420573, 1.420811, 1.421075,
           1.421407, 1.421956, 1.422530, 1.423168, 1.424229, 1.425704, 1.427598,
           1.429925},
          {1.490602, 1.495140, 1.490760, 1.494215, 1.494693, 1.496901, 1.499725,
           1.492548, 1.498681, 1.502806, 1.503747, 1.508504, 1.500686, 1.507235,
           1.507097, 1.506155, 1.493718, 1.502645, 1.504973, 1.497627, 1.494410,
           1.490717, 1.479908, 1.480655, 1.476513, 1.475099, 1.463790, 1.455597,
           1.441465, 1.442572, 1.429918, 1.419698, 1.419716, 1.419876, 1.419876,
           1.420005, 1.420060, 1.420281, 1.420437, 1.420572, 1.420811, 1.421075,
           1.421407, 1.421957, 1.422531, 1.423170, 1.424229, 1.425703, 1.427599,
           1.429923}},
         {{1.496584, 1.498835, 1.497036, 1.496177, 1.490849, 1.496141, 1.500016,
           1.500690, 1.503434, 1.511507, 1.514572, 1.514054, 1.517205, 1.511746,
           1.505996, 1.508720, 1.501375, 1.499325, 1.501645, 1.499495, 1.491609,
           1.496208, 1.488166, 1.480829, 1.476294, 1.474699, 1.465525, 1.462925,
           1.452983, 1.449807, 1.449807, 1.449807, 1.449819, 1.449935, 1.449935,
           1.450025, 1.450065, 1.450222, 1.450333, 1.450429, 1.450597, 1.450786,
           1.451023, 1.451411, 1.451822, 1.452275, 1.453029, 1.454078, 1.455426,
           1.457079},
          {1.516423, 1.520126, 1.518154, 1.521618, 1.514661, 1.522351, 1.514743,
           1.521267, 1.518911, 1.521276, 1.515459, 1.522954, 1.516000, 1.522949,
           1.522889, 1.525526, 1.526289, 1.512076, 1.520521, 1.521706, 1.516564,
           1.514691, 1.511734, 1.505916, 1.501834, 1.491089, 1.477574, 1.482549,
           1.472547, 1.466657, 1.456562, 1.449807, 1.449818, 1.449935, 1.449935,
           1.450027, 1.450065, 1.450222, 1.450333, 1.450430, 1.450597, 1.450787,
           1.451023, 1.451413, 1.451820, 1.452276, 1.453028, 1.454077, 1.455426,
           1.457079}},
         {{1.524008, 1.518630, 1.519631, 1.518045, 1.516497, 1.523678, 1.529351,
           1.541406, 1.555305, 1.563834, 1.554973, 1.549703, 1.551845, 1.543069,
           1.542131, 1.546605, 1.542711, 1.541447, 1.535216, 1.534790, 1.533384,
           1.533488, 1.522402, 1.520482, 1.522847, 1.517075, 1.509032, 1.506716,
           1.497563, 1.498795, 1.498795, 1.498795, 1.498910, 1.500063, 1.500063,
           1.500974, 1.501360, 1.502935, 1.504044, 1.505011, 1.506696, 1.508583,
           1.510947, 1.514841, 1.518925, 1.522675, 1.523337, 1.524261, 1.525449,
           1.526906},
          {1.544867, 1.545495, 1.545622, 1.546649, 1.547227, 1.548945, 1.553610,
           1.560444, 1.569622, 1.580282, 1.573525, 1.576737, 1.567007, 1.566311,
           1.566041, 1.565870, 1.563605, 1.565212, 1.555501, 1.558416, 1.555632,
           1.549509, 1.559850, 1.556096, 1.541266, 1.536273, 1.534912, 1.524922,
           1.521773, 1.518552, 1.504718, 1.498795, 1.498911, 1.500063, 1.500063,
           1.500975, 1.501361, 1.502933, 1.504044, 1.505012, 1.506698, 1.508582,
           1.510946, 1.514839, 1.518925, 1.522674, 1.523335, 1.524263, 1.525449,
           1.526905}}},
        {{{1.482834, 1.479903, 1.486543, 1.486096, 1.482117, 1.486589, 1.485025,
           1.483320, 1.485500, 1.486108, 1.491501, 1.497552, 1.498239, 1.492612,
           1.507272, 1.490626, 1.492383, 1.488077, 1.495532, 1.489154, 1.486064,
           1.492043, 1.483875, 1.481027, 1.474888, 1.476522, 1.472842, 1.460939,
           1.458857, 1.459044, 1.459044, 1.459044, 1.459060, 1.459233, 1.459233,
           1.459372, 1.459428, 1.459665, 1.459832, 1.459980, 1.460233, 1.460516,
           1.460873, 1.461461, 1.462076, 1.462761, 1.463896, 1.465478, 1.467508,
           1.469999},
          {1.526207, 1.526968, 1.525172, 1.522341, 1.525720, 1.530139, 1.526428,
           1.525544, 1.532824, 1.538667, 1.539295, 1.527002, 1.533739, 1.545902,
           1.545996, 1.547544, 1.528063, 1.548675, 1.546430, 1.520527, 1.537182,
           1.539564, 1.530100, 1.517311, 1.510667, 1.516296, 1.515169, 1.518694,
           1.502474, 1.506641, 1.485601, 1.476976, 1.471316, 1.478042, 1.459233,
           1.459370, 1.459429, 1.459666, 1.459832, 1.459978, 1.460233, 1.460517,
           1.460873, 1.461460, 1.462075, 1.462764, 1.463895, 1.465476, 1.467508,
           1.469999}},
         {{1.515191, 1.515112, 1.511318, 1.511179, 1.515058, 1.519540, 1.520574,
           1.523279, 1.525726, 1.517189, 1.518134, 1.523024, 1.530386, 1.533318,
           1.522330, 1.514291, 1.517529, 1.521823, 1.518242, 1.522310, 1.514314,
           1.517261, 1.513153, 1.509121, 1.508898, 1.510002, 1.507808, 1.491051,
           1.491051, 1.491179, 1.491179, 1.491179, 1.491191, 1.491310, 1.491310,
           1.491405, 1.491444, 1.491607, 1.491719, 1.491819, 1.491993, 1.492189,
           1.492430, 1.492832, 1.493252, 1.493720, 1.494497, 1.495576, 1.496963,
           1.498662},
          {1.564004, 1.562952, 1.566774, 1.563965, 1.563850, 1.560234, 1.571574,
           1.571005, 1.569365, 1.570892, 1.571369, 1.566644, 1.571968, 1.570368,
           1.584367, 1.571196, 1.563414, 1.566006, 1.571799, 1.573445, 1.569513,
           1.564172, 1.562970, 1.555414, 1.548717, 1.552569, 1.549934, 1.545857,
           1.533307, 1.532168, 1.521363, 1.523028, 1.506956, 1.509285, 1.503893,
           1.491404, 1.491444, 1.491606, 1.491719, 1.491821, 1.491992, 1.492188,
           1.492430, 1.492832, 1.493252, 1.493722, 1.494495, 1.495575, 1.496965,
           1.498666}},
         {{1.543449, 1.542413, 1.541409, 1.541501, 1.540381, 1.543070, 1.551084,
           1.560281, 1.574324, 1.585233, 1.576697, 1.569950, 1.570758, 1.569164,
           1.567830, 1.571571, 1.574208, 1.560185, 1.568930, 1.566559, 1.570233,
           1.561186, 1.558121, 1.564114, 1.557375, 1.550444, 1.548786, 1.542093,
           1.540358, 1.541625, 1.541625, 1.541625, 1.541745, 1.542929, 1.542929,
           1.543870, 1.544267, 1.545889, 1.547031, 1.548026, 1.549763, 1.551706,
           1.554139, 1.558150, 1.562356, 1.566215, 1.566890, 1.567827, 1.569033,
           1.570511},
          {1.604722, 1.598312, 1.600231, 1.597287, 1.603903, 1.603179, 1.610070,
           1.608134, 1.626105, 1.630272, 1.625151, 1.622193, 1.623939, 1.626242,
           1.641158, 1.629017, 1.625085, 1.627218, 1.631801, 1.624844, 1.637039,
           1.624300, 1.612566, 1.607918, 1.606852, 1.615100, 1.607623, 1.610228,
           1.599532, 1.591772, 1.586907, 1.585350, 1.582441, 1.570384, 1.565515,
           1.555945, 1.550593, 1.545888, 1.547031, 1.548027, 1.549763, 1.551705,
           1.554142, 1.558151, 1.562357, 1.566216, 1.566887, 1.567828, 1.569033,
           1.570513}}},
        {{{1.631444, 1.617430, 1.606214, 1.597788, 1.592090, 1.589345, 1.586793,
           1.594115, 1.588614, 1.593382, 1.599802, 1.595371, 1.598363, 1.596839,
           1.587060, 1.585008, 1.586032, 1.587780, 1.596789, 1.604480, 1.606750,
           1.608058, 1.615048, 1.622837, 1.626337, 1.621019, 1.618560, 1.620919,
           1.616716, 1.613041, 1.602682, 1.598558, 1.601130, 1.605520, 1.600688,
           1.592364, 1.579325, 1.592064, 1.585289, 1.577813, 1.575728, 1.569043,
           1.567568, 1.564944, 1.551863, 1.558200, 1.552098, 1.545532, 1.550285,
           1.546525},
          {1.781000, 1.749786, 1.723863, 1.702331, 1.685349, 1.672189, 1.665057,
           1.661976, 1.668697, 1.666432, 1.663192, 1.667091, 1.662880, 1.668880,
           1.661830, 1.646339, 1.649520, 1.665168, 1.669665, 1.666116, 1.663431,
           1.661499, 1.651521, 1.652072, 1.666749, 1.650020, 1.662056, 1.641878,
           1.642857, 1.651762, 1.654815, 1.640120, 1.642176, 1.634381, 1.624073,
           1.621410, 1.612974, 1.617942, 1.608889, 1.587524, 1.604047, 1.587885,
           1.576141, 1.585722, 1.585108, 1.570441, 1.567023, 1.543402, 1.544671,
           1.526870}},
         {{1.657630, 1.645101, 1.634809, 1.626991, 1.621760, 1.619269, 1.616241,
           1.623460, 1.621592, 1.628765, 1.626292, 1.624804, 1.624691, 1.626091,
           1.634216, 1.629899, 1.629011, 1.626932, 1.635776, 1.642750, 1.644031,
           1.651891, 1.643552, 1.640448, 1.657951, 1.647708, 1.640056, 1.645650,
           1.635986, 1.631473, 1.647644, 1.631968, 1.625745, 1.634167, 1.613334,
           1.611211, 1.612020, 1.619729, 1.610099, 1.594000, 1.607405, 1.605950,
           1.604976, 1.598861, 1.586164, 1.593905, 1.590197, 1.591481, 1.581687,
           1.564550},
          {1.821970, 1.785639, 1.760274, 1.742098, 1.724389, 1.707875, 1.693072,
           1.680811, 1.690404, 1.686151, 1.679774, 1.688385, 1.680001, 1.692517,
           1.681615, 1.676445, 1.678454, 1.685969, 1.678583, 1.685396, 1.689548,
           1.679759, 1.688733, 1.693673, 1.703586, 1.684884, 1.680504, 1.678071,
           1.680076, 1.682484, 1.685744, 1.683022, 1.684108, 1.685107, 1.687485,
           1.682698, 1.683459, 1.677189, 1.673029, 1.664256, 1.643441, 1.656051,
           1.650168, 1.630866, 1.639938, 1.637267, 1.630754, 1.613829, 1.593552,
           1.592762}},
         {{1.672899, 1.660495, 1.650610, 1.643789, 1.640062, 1.640680, 1.651085,
           1.656534, 1.676239, 1.678612, 1.664971, 1.663895, 1.667537, 1.671732,
           1.668271, 1.675888, 1.666164, 1.670085, 1.668075, 1.672633, 1.677967,
           1.686305, 1.699680, 1.698244, 1.691390, 1.695926, 1.711192, 1.706914,
           1.701077, 1.687549, 1.708921, 1.691025, 1.707372, 1.691375, 1.704558,
           1.705531, 1.695414, 1.703266, 1.693443, 1.691748, 1.683841, 1.697232,
           1.680477, 1.675610, 1.677610, 1.680184, 1.691274, 1.691995, 1.676434,
           1.669097},
          {1.925828, 1.884862, 1.846006, 1.814709, 1.790403, 1.774336, 1.765562,
           1.767315, 1.781751, 1.806779, 1.785272, 1.768126, 1.767853, 1.774115,
           1.776835, 1.775537, 1.756899, 1.748915, 1.761397, 1.772645, 1.781563,
           1.769126, 1.764173, 1.770081, 1.786341, 1.787201, 1.759065, 1.768072,
           1.765256, 1.790110, 1.791514, 1.762262, 1.768220, 1.772978, 1.764958,
           1.768719, 1.746094, 1.754802, 1.751710, 1.750439, 1.749043, 1.752364,
           1.719696, 1.719133, 1.739486, 1.731613, 1.736796, 1.725093, 1.700950,
           1.667576}}},
        {{{2.129471, 2.110282, 2.094639, 2.083914, 2.063764, 2.043190, 2.022295,
           2.001224, 1.979434, 1.957066, 1.934620, 1.922301, 1.911625, 1.888911,
           1.868190, 1.853907, 1.839970, 1.848208, 1.835488, 1.830122, 1.829305,
           1.827517, 1.823122, 1.818024, 1.817894, 1.813468, 1.810313, 1.803152,
           1.798794, 1.794585, 1.795209, 1.783812, 1.777389, 1.772091, 1.765525,
           1.758111, 1.751711, 1.744311, 1.736252, 1.727840, 1.719087, 1.709000,
           1.700063, 1.691238, 1.681907, 1.672514, 1.664023, 1.656358, 1.649980,
           1.645743},
          {2.449688, 2.432744, 2.401407, 2.382793, 2.356937, 2.334092, 2.308485,
           2.287121, 2.262780, 2.242060, 2.219850, 2.195983, 2.171512, 2.147869,
           2.125553, 2.102428, 2.078194, 2.055505, 2.033105, 2.012168, 1.990199,
           1.969454, 1.950151, 1.931403, 1.912697, 1.895639, 1.879303, 1.863663,
           1.847604, 1.833239, 1.821566, 1.812388, 1.799360, 1.787946, 1.776668,
           1.763182, 1.750578, 1.739137, 1.727104, 1.715312, 1.703349, 1.692302,
           1.679910, 1.667480, 1.656251, 1.645314, 1.635541, 1.626762, 1.619703,
           1.614048}},
         {{2.119130, 2.100242, 2.084865, 2.074576, 2.055187, 2.035654, 2.016345,
           1.996849, 1.977745, 1.956957, 1.932574, 1.919042, 1.908611, 1.886020,
           1.866076, 1.852507, 1.841534, 1.847100, 1.835318, 1.833699, 1.833277,
           1.833371, 1.836821, 1.845453, 1.845177, 1.845652, 1.844462, 1.840348,
           1.837573, 1.834709, 1.835432, 1.826249, 1.822400, 1.817314, 1.811943,
           1.807099, 1.801764, 1.796285, 1.786755, 1.779123, 1.771160, 1.762858,
           1.755320, 1.746818, 1.739783, 1.732651, 1.726362, 1.720416, 1.716859,
           1.714506},
          {2.537384, 2.516740, 2.483379, 2.461361, 2.432659, 2.406146, 2.379833,
           2.358414, 2.336193, 2.313913, 2.287022, 2.259777, 2.233383, 2.207919,
           2.183212, 2.157821, 2.133429, 2.110098, 2.088160, 2.064626, 2.041668,
           2.021772, 2.001958, 1.982366, 1.963313, 1.946273, 1.929229, 1.913939,
           1.898299, 1.883788, 1.869298, 1.854873, 1.841192, 1.835164, 1.846970,
           1.841064, 1.831514, 1.820227, 1.807620, 1.794662, 1.781409, 1.768093,
           1.755307, 1.742297, 1.729133, 1.717755, 1.705845, 1.695639, 1.686910,
           1.679406}},
         {{2.145974, 2.125064, 2.105941, 2.098282, 2.080136, 2.065684, 2.053795,
           2.048935, 2.053830, 2.049210, 2.008154, 1.980322, 1.970049, 1.951480,
           1.932880, 1.920583, 1.911935, 1.918917, 1.913259, 1.911755, 1.913066,
           1.911449, 1.912136, 1.912152, 1.913160, 1.912093, 1.912178, 1.911299,
           1.909657, 1.910348, 1.908319, 1.907055, 1.904791, 1.903824, 1.901203,
           1.899998, 1.896390, 1.894821, 1.890914, 1.885856, 1.882091, 1.877388,
           1.873139, 1.870342, 1.867540, 1.863438, 1.855869, 1.849333, 1.842470,
           1.836075},
          {2.647104, 2.609070, 2.582720, 2.597175, 2.579697, 2.562484, 2.543479,
           2.537124, 2.542982, 2.536020, 2.480174, 2.435334, 2.403794, 2.372210,
           2.343192, 2.314826, 2.286317, 2.259240, 2.235837, 2.214202, 2.189378,
           2.166069, 2.145335, 2.127080, 2.108999, 2.090273, 2.072622, 2.056308,
           2.039164, 2.026507, 2.010980, 1.997099, 1.982782, 1.971958, 1.958391,
           1.946804, 1.935012, 1.925485, 1.919904, 1.910068, 1.899279, 1.886391,
           1.873086, 1.863901, 1.854117, 1.844900, 1.831881, 1.817825, 1.809083,
           1.800228}}},
        {{{2.659998, 2.619472, 2.583416, 2.547580, 2.512922, 2.478908, 2.447080,
           2.415684, 2.385863, 2.356333, 2.329142, 2.301938, 2.276751, 2.252173,
           2.228242, 2.206472, 2.185342, 2.164329, 2.144324, 2.124969, 2.107608,
           2.090274, 2.074077, 2.057821, 2.042909, 2.028129, 2.014041, 2.002891,
           1.994925, 1.985508, 1.975021, 1.964260, 1.953684, 1.943190, 1.932280,
           1.921918, 1.910663, 1.900105, 1.888529, 1.877193, 1.865404, 1.853137,
           1.840547, 1.828001, 1.815844, 1.804005, 1.793104, 1.783709, 1.775545,
           1.770200},
          {3.159468, 3.093995, 3.032078, 2.973160, 2.916605, 2.863279, 2.812583,
           2.764875, 2.720804, 2.678946, 2.635977, 2.592821, 2.552155, 2.512030,
           2.472716, 2.435664, 2.397978, 2.361920, 2.327610, 2.294280, 2.262033,
           2.231158, 2.202787, 2.174610, 2.147773, 2.122022, 2.098663, 2.075829,
           2.053235, 2.031403, 2.010765, 1.996320, 1.980550, 1.962877, 1.944715,
           1.926623, 1.908584, 1.890937, 1.873798, 1.856639, 1.839999, 1.822736,
           1.807083, 1.792377, 1.777117, 1.763044, 1.750353, 1.738732, 1.729903,
           1.722460}},
         {{2.640273, 2.603546, 2.568138, 2.532537, 2.499516, 2.467120, 2.437347,
           2.407728, 2.381407, 2.353992, 2.323902, 2.295454, 2.270722, 2.246102,
           2.222857, 2.202221, 2.180622, 2.160174, 2.140759, 2.123025, 2.105453,
           2.088996, 2.073645, 2.058731, 2.044400, 2.030744, 2.018179, 2.017214,
           2.026665, 2.027794, 2.022244, 2.014902, 2.006790, 1.998577, 1.989567,
           1.980930, 1.971577, 1.962424, 1.952553, 1.942350, 1.931610, 1.920684,
           1.909759, 1.898607, 1.887786, 1.877224, 1.867665, 1.861275, 1.856429,
           1.852618},
          {3.301708, 3.230535, 3.162363, 3.096319, 3.035556, 2.978216, 2.924411,
           2.875635, 2.831525, 2.786622, 2.735937, 2.687977, 2.641522, 2.597540,
           2.557240, 2.516393, 2.476724, 2.439832, 2.403099, 2.368312, 2.336152,
           2.304600, 2.273740, 2.245154, 2.217781, 2.193282, 2.168201, 2.143768,
           2.120854, 2.098469, 2.078311, 2.057664, 2.037711, 2.018904, 2.021834,
           2.026999, 2.014360, 1.998456, 1.981571, 1.963750, 1.945018, 1.926723,
           1.907583, 1.889366, 1.870813, 1.855109, 1.839989, 1.826617, 1.815172,
           1.804573}},
         {{2.677618, 2.637057, 2.597198, 2.561930, 2.529522, 2.503062, 2.482291,
           2.473020, 2.478866, 2.472584, 2.419800, 2.377796, 2.350377, 2.326604,
           2.304598, 2.282738, 2.262286, 2.243430, 2.227178, 2.210727, 2.195035,
           2.181482, 2.168159, 2.157999, 2.147628, 2.138016, 2.129043, 2.121019,
           2.116644, 2.115804, 2.113598, 2.111619, 2.108179, 2.105713, 2.101552,
           2.098238, 2.093338, 2.089199, 2.083654, 2.076995, 2.070393, 2.063671,
           2.057250, 2.052302, 2.047489, 2.041271, 2.030415, 2.020185, 2.009897,
           2.000551},
          {3.469165, 3.386715, 3.309000, 3.236865, 3.168283, 3.102621, 3.135049,
           3.128993, 3.130095, 3.110584, 3.020036, 2.942408, 2.885613, 2.834143,
           2.783398, 2.734943, 2.692153, 2.647963, 2.610175, 2.573978, 2.535953,
           2.501796, 2.468692, 2.440203, 2.411327, 2.382226, 2.354869, 2.329613,
           2.304929, 2.284561, 2.262385, 2.240699, 2.221134, 2.204330, 2.185336,
           2.168539, 2.150337, 2.134775, 2.123930, 2.112244, 2.096024, 2.079246,
           2.062154, 2.047394, 2.033327, 2.018558, 2.000655, 1.984489, 1.970153,
           1.957336}}},
        {{{3.372831, 3.304443, 3.237699, 3.175525, 3.114982, 3.057097, 3.001023,
           2.947103, 2.896353, 2.848159, 2.801849, 2.757964, 2.717137, 2.677313,
           2.639599, 2.604578, 2.570716, 2.537895, 2.506717, 2.477473, 2.450095,
           2.423066, 2.396987, 2.372154, 2.348589, 2.326930, 2.305346, 2.284876,
           2.268841, 2.257148, 2.241238, 2.225125, 2.209509, 2.194105, 2.178638,
           2.163587, 2.148333, 2.133286, 2.118059, 2.102471, 2.086856, 2.070495,
           2.053829, 2.037297, 2.020683, 2.004239, 1.989135, 1.976602, 1.966019,
           1.957464},
          {4.126014, 4.022614, 3.926900, 3.835790, 3.749740, 3.667102, 3.587621,
           3.510871, 3.437339, 3.367551, 3.298283, 3.229090, 3.162637, 3.096548,
           3.033491, 2.971027, 2.911116, 2.854680, 2.799746, 2.747667, 2.697778,
           2.649892, 2.605186, 2.562394, 2.521247, 2.482105, 2.445088, 2.408567,
           2.373565, 2.340787, 2.309082, 2.278811, 2.258575, 2.233393, 2.207464,
           2.181470, 2.155873, 2.131070, 2.106841, 2.082645, 2.058470, 2.034544,
           2.010949, 1.988205, 1.966585, 1.945654, 1.926527, 1.909565, 1.895943,
           1.884996}},
         {{3.340493, 3.274268, 3.210825, 3.150594, 3.091725, 3.037260, 2.983904,
           2.934129, 2.888571, 2.842593, 2.792079, 2.747008, 2.705720, 2.666649,
           2.630162, 2.595609, 2.562202, 2.530203, 2.500393, 2.472500, 2.445627,
           2.419678, 2.395082, 2.372012, 2.350602, 2.330182, 2.310312, 2.291637,
           2.291076, 2.306073, 2.301304, 2.290892, 2.279596, 2.267591, 2.255012,
           2.242427, 2.229506, 2.216460, 2.203053, 2.188901, 2.174827, 2.160185,
           2.145064, 2.130053, 2.115189, 2.100838, 2.088143, 2.077538, 2.069123,
           2.062571},
          {4.384216, 4.268321, 4.158666, 4.052879, 3.953826, 3.862093, 3.774967,
           3.691912, 3.616566, 3.542284, 3.459015, 3.380360, 3.306518, 3.235422,
           3.167980, 3.101585, 3.039217, 2.977745, 2.919816, 2.865513, 2.812866,
           2.762914, 2.715871, 2.670721, 2.627992, 2.586815, 2.546526, 2.508048,
           2.472079, 2.437949, 2.404761, 2.373274, 2.344130, 2.315604, 2.289907,
           2.304043, 2.296490, 2.276599, 2.253217, 2.228484, 2.203169, 2.177745,
           2.152285, 2.126770, 2.101677, 2.077224, 2.054317, 2.034228, 2.017193,
           2.002542}},
         {{3.384414, 3.312638, 3.245441, 3.183693, 3.127532, 3.080039, 3.042332,
           3.022129, 3.022875, 3.006315, 2.924149, 2.857627, 2.812966, 2.773822,
           2.736886, 2.701670, 2.668415, 2.637395, 2.611376, 2.585757, 2.561146,
           2.539023, 2.518784, 2.502554, 2.487346, 2.472392, 2.458614, 2.446151,
           2.434983, 2.428450, 2.422423, 2.416951, 2.413964, 2.413034, 2.406001,
           2.400198, 2.392107, 2.385578, 2.377414, 2.367617, 2.358186, 2.348252,
           2.338365, 2.330490, 2.321707, 2.311268, 2.295713, 2.281001, 2.266766,
           2.253226},
          {4.678534, 4.549770, 4.424448, 4.304515, 4.190379, 4.079726, 3.974091,
           4.016470, 4.087383, 4.068447, 3.920729, 3.792740, 3.697383, 3.610313,
           3.526804, 3.447458, 3.373970, 3.303780, 3.242109, 3.180178, 3.120055,
           3.063012, 3.008241, 2.958331, 2.910075, 2.862076, 2.816434, 2.774083,
           2.733705, 2.698830, 2.662413, 2.628405, 2.596308, 2.568596, 2.539185,
           2.513182, 2.487409, 2.463845, 2.440250, 2.424671, 2.402874, 2.378805,
           2.354082, 2.331912, 2.310164, 2.288946, 2.264023, 2.240539, 2.218981,
           2.199490}}},
        {{{5.455127, 5.290320, 5.133335, 4.984487, 4.844668, 4.709186, 4.581777,
           4.459526, 4.343826, 4.235396, 4.130252, 4.031845, 3.939541, 3.851250,
           3.767525, 3.687066, 3.610890, 3.540032, 3.472004, 3.406565, 3.344961,
           3.285930, 3.229506, 3.176929, 3.126286, 3.077508, 3.031689, 2.988742,
           2.947598, 2.908441, 2.875625, 2.852244, 2.823289, 2.792763, 2.762297,
           2.732850, 2.704144, 2.677013, 2.649663, 2.622309, 2.594866, 2.567443,
           2.540550, 2.514507, 2.488178, 2.463074, 2.440611, 2.420738, 2.403776,
           2.390169},
          {6.507519, 6.334114, 6.164153, 6.004227, 5.849453, 5.701551, 5.558236,
           5.418039, 5.281886, 5.150289, 5.018772, 4.889032, 4.760342, 4.633763,
           4.511867, 4.391883, 4.279853, 4.169212, 4.063817, 3.963634, 3.866360,
           3.771883, 3.679689, 3.590907, 3.508292, 3.427178, 3.350809, 3.277942,
           3.208442, 3.142759, 3.078141, 3.016955, 2.959716, 2.904319, 2.862042,
           2.822650, 2.775516, 2.729342, 2.684546, 2.640948, 2.598016, 2.556203,
           2.516429, 2.477983, 2.440942, 2.405593, 2.374431, 2.346610, 2.323285,
           2.303864}},
         {{5.368544, 5.211573, 5.061524, 4.921169, 4.788251, 4.660378, 4.541398,
           4.428082, 4.325816, 4.220849, 4.108338, 4.007217, 3.915426, 3.828521,
           3.746742, 3.667932, 3.593180, 3.525151, 3.458776, 3.395778, 3.336489,
           3.279672, 3.226124, 3.176640, 3.128278, 3.082501, 3.039472, 2.999733,
           2.961755, 2.926387, 2.904195, 2.918459, 2.923706, 2.905570, 2.882379,
           2.858017, 2.832975, 2.808233, 2.783516, 2.758481, 2.733223, 2.707344,
           2.681396, 2.656293, 2.632649, 2.610141, 2.589422, 2.571693, 2.556921,
           2.544795},
          {6.931629, 6.732017, 6.542390, 6.363876, 6.190106, 6.033380, 5.888187,
           5.754447, 5.635230, 5.512098, 5.362012, 5.215594, 5.076592, 4.943372,
           4.813559, 4.685037, 4.563027, 4.442042, 4.327127, 4.213540, 4.103934,
           3.999446, 3.898900, 3.803530, 3.711451, 3.623443, 3.541677, 3.462744,
           3.388109, 3.319252, 3.252361, 3.189862, 3.129004, 3.071262, 3.015878,
           2.963614, 2.912779, 2.918922, 2.916098, 2.877647, 2.834268, 2.789582,
           2.745208, 2.701082, 2.657835, 2.617154, 2.578693, 2.543846, 2.512434,
           2.484818}},
         {{5.456880, 5.283227, 5.123359, 4.977396, 4.846610, 4.737367, 4.648096,
           4.596791, 4.585537, 4.537895, 4.355995, 4.211747, 4.112540, 4.023224,
           3.938261, 3.858523, 3.786016, 3.716165, 3.655979, 3.597975, 3.541308,
           3.488460, 3.439880, 3.399738, 3.359309, 3.320644, 3.285465, 3.253384,
           3.224845, 3.203834, 3.178536, 3.154373, 3.132555, 3.119764, 3.105641,
           3.099280, 3.085703, 3.072332, 3.055262, 3.036454, 3.018507, 2.999714,
           2.980440, 2.963707, 2.946677, 2.927546, 2.900307, 2.873812, 2.848971,
           2.825989},
          {7.375239, 7.172820, 6.968608, 6.764055, 6.572290, 6.393399, 6.221938,
           6.302156, 6.604217, 6.626446, 6.326426, 6.072238, 5.893647, 5.729802,
           5.570854, 5.416573, 5.265292, 5.116153, 4.981910, 4.845971, 4.714752,
           4.587149, 4.465448, 4.357008, 4.250180, 4.146132, 4.048076, 3.954186,
           3.866347, 3.792630, 3.714167, 3.640773, 3.571505, 3.510065, 3.446699,
           3.389460, 3.331309, 3.281291, 3.232185, 3.181699, 3.140710, 3.108841,
           3.066221, 3.025656, 2.985460, 2.945718, 2.899226, 2.855646, 2.816754,
           2.781960}}},
        {{{8.926530, 8.605853, 8.299822, 8.007425, 7.733231, 7.471510, 7.227902,
           6.998840, 6.768279, 6.543639, 6.331638, 6.130706, 5.942982, 5.765261,
           5.596052, 5.433759, 5.280651, 5.138172, 5.001517, 4.870815, 4.747615,
           4.632844, 4.522819, 4.417212, 4.319119, 4.226622, 4.138573, 4.055066,
           3.976811, 3.901022, 3.829850, 3.763239, 3.700422, 3.655299, 3.612130,
           3.561043, 3.508018, 3.456643, 3.406134, 3.356741, 3.309235, 3.261123,
           3.214495, 3.170176, 3.128890, 3.088936, 3.053275, 3.021835, 2.994308,
           2.970308},
          {9.602156, 9.321937, 9.045514, 8.777934, 8.523078, 8.280678, 8.050715,
           7.838284, 7.639121, 7.451990, 7.269553, 7.091759, 6.910675, 6.722439,
           6.539271, 6.361535, 6.188926, 6.023192, 5.863483, 5.711777, 5.561563,
           5.411613, 5.264657, 5.121511, 4.979701, 4.840735, 4.708063, 4.579838,
           4.455693, 4.336499, 4.225071, 4.117931, 4.016040, 3.918093, 3.824172,
           3.735985, 3.662981, 3.604594, 3.532166, 3.458779, 3.388063, 3.321480,
           3.255520, 3.192666, 3.132942, 3.075297, 3.023199, 2.977233, 2.937325,
           2.902629}},
         {{8.720802, 8.419539, 8.130758, 7.858208, 7.599801, 7.358096, 7.136876,
           6.925223, 6.714645, 6.503763, 6.274602, 6.068592, 5.883380, 5.711312,
           5.545640, 5.389389, 5.242650, 5.104008, 4.971991, 4.847077, 4.729388,
           4.618212, 4.512192, 4.412546, 4.319810, 4.231084, 4.148218, 4.069555,
           3.995716, 3.925581, 3.858895, 3.796590, 3.738233, 3.724399, 3.763792,
           3.737052, 3.696879, 3.652255, 3.606134, 3.559543, 3.513578, 3.468743,
           3.423593, 3.380855, 3.340088, 3.301490, 3.267130, 3.236312, 3.210068,
           3.188085},
          {10.272098, 9.956925, 9.655772, 9.376860, 9.106772, 8.846327,
           8.606056,  8.391475, 8.204064, 8.008566, 7.766450, 7.538678,
           7.332761,  7.140394, 6.948192, 6.750985, 6.562875, 6.382097,
           6.209803,  6.045665, 5.888291, 5.739774, 5.594026, 5.450494,
           5.309230,  5.171841, 5.035971, 4.902387, 4.773642, 4.652778,
           4.533181,  4.416889, 4.307103, 4.201789, 4.100116, 4.004548,
           3.910622,  3.822010, 3.738836, 3.745532, 3.737705, 3.672883,
           3.600670,  3.528973, 3.460182, 3.392551, 3.329817, 3.273388,
           3.220900,  3.173511}},
         {{8.959590, 8.597423, 8.265601, 7.973045, 7.719695, 7.511269, 7.328822,
           7.217735, 7.190292, 7.079186, 6.703603, 6.415880, 6.216627, 6.036282,
           5.867040, 5.710589, 5.563217, 5.423045, 5.299709, 5.179951, 5.064579,
           4.955934, 4.853798, 4.767403, 4.684715, 4.602750, 4.527404, 4.456206,
           4.390845, 4.337992, 4.279376, 4.224801, 4.174502, 4.135297, 4.090142,
           4.059286, 4.034482, 4.019339, 3.988515, 3.954026, 3.919341, 3.883639,
           3.848375, 3.818341, 3.788244, 3.756982, 3.713031, 3.670679, 3.630486,
           3.592380},
          {10.998798, 10.657155, 10.329732, 10.011366, 9.707783, 9.781427,
           10.165344, 10.193417, 10.262643, 10.116416, 9.439600, 8.934468,
           8.605083,  8.319980,  8.060440,  7.806915,  7.556641, 7.325678,
           7.120393,  6.918617,  6.725328,  6.549121,  6.384060, 6.234055,
           6.082341,  5.933064,  5.791886,  5.657323,  5.523724, 5.403914,
           5.273401,  5.147352,  5.024230,  4.912866,  4.795232, 4.692281,
           4.586858,  4.494106,  4.401172,  4.307981,  4.222931, 4.143240,
           4.081806,  4.020519,  3.951828,  3.882312,  3.803863, 3.731955,
           3.664128,  3.602361}}},
        {{{15.891049, 15.356836, 14.823518, 14.305512, 13.798564, 13.299736,
           12.816125, 12.346296, 11.889697, 11.446336, 11.019388, 10.608935,
           10.215882, 9.843581,  9.488671,  9.148696,  8.823917,  8.516727,
           8.227729,  7.949041,  7.687076,  7.442268,  7.207743,  6.990429,
           6.785570,  6.596343,  6.415544,  6.236037,  6.061123,  5.898905,
           5.745265,  5.595929,  5.455209,  5.322367,  5.197559,  5.089905,
           5.014493,  4.916621,  4.814363,  4.714068,  4.619722,  4.526857,
           4.439391,  4.357294,  4.279473,  4.206892,  4.139552,  4.079762,
           4.025923,  3.978882},
          {15.083006, 14.644414, 14.219236, 13.804971, 13.397849, 12.996436,
           12.603294, 12.218281, 11.846736, 11.486464, 11.128626, 10.775455,
           10.426015, 10.090653, 9.768776,  9.458080,  9.157095,  8.866854,
           8.589603,  8.328362,  8.077183,  7.833940,  7.606776,  7.389412,
           7.180012,  6.981029,  6.790089,  6.610295,  6.435838,  6.263351,
           6.094117,  5.935690,  5.786023,  5.640953,  5.496914,  5.355916,
           5.218690,  5.094243,  5.007521,  4.887382,  4.764248,  4.644707,
           4.528585,  4.419844,  4.317012,  4.221467,  4.133673,  4.055349,
           3.985092,  3.924462}},
         {{15.359120, 14.854109, 14.358303, 13.875665, 13.401235, 12.947068,
           12.509429, 12.092423, 11.698309, 11.275579, 10.800720, 10.374005,
           9.994603,  9.636913,  9.294523,  8.966608,  8.657111,  8.367695,
           8.092486,  7.828866,  7.582834,  7.346568,  7.127543,  6.923799,
           6.733435,  6.554320,  6.380130,  6.206980,  6.045013,  5.895774,
           5.751393,  5.612000,  5.481954,  5.360161,  5.244570,  5.161256,
           5.216501,  5.205210,  5.122696,  5.030140,  4.936289,  4.846782,
           4.756836,  4.674396,  4.595954,  4.521967,  4.455007,  4.395145,
           4.342900,  4.297963},
          {16.091258, 15.666294, 15.210060, 14.757259, 14.313424, 13.888206,
           13.477441, 13.100437, 12.764897, 12.406820, 11.959229, 11.540811,
           11.160724, 10.796789, 10.444667, 10.109334, 9.789337,  9.481898,
           9.188441,  8.905972,  8.636297,  8.381543,  8.134047,  7.894419,
           7.667818,  7.448702,  7.237700,  7.039137,  6.850188,  6.675829,
           6.505850,  6.338951,  6.174349,  6.021993,  5.876980,  5.737982,
           5.601337,  5.468271,  5.335510,  5.206608,  5.190856,  5.218404,
           5.114692,  4.992603,  4.870210,  4.747896,  4.636461,  4.533616,
           4.440653,  4.355235}},
         {{15.983357, 15.231830, 14.548557, 13.954108, 13.425437, 13.007566,
           12.675068, 12.496410, 12.492478, 12.282550, 11.464330, 10.840094,
           10.423482, 10.051292, 9.704679,  9.378871,  9.068923,  8.778452,
           8.525900,  8.275056,  8.035631,  7.810360,  7.603009,  7.426534,
           7.248690,  7.071323,  6.901751,  6.746031,  6.603376,  6.486405,
           6.358385,  6.236855,  6.124566,  6.031682,  5.929775,  5.844030,
           5.755126,  5.709094,  5.662723,  5.587670,  5.512858,  5.438366,
           5.367064,  5.307078,  5.248995,  5.188283,  5.110123,  5.035873,
           4.964841,  4.894093},
          {17.143727, 16.639411, 16.155117, 17.443215, 17.481086, 17.000005,
           16.561843, 16.353924, 16.415613, 16.104273, 14.828010, 13.905570,
           13.322660, 12.816825, 12.344394, 11.903044, 11.494073, 11.103553,
           10.757007, 10.408528, 10.067107, 9.744809,  9.438478,  9.164547,
           8.891994,  8.628889,  8.378177,  8.141036,  7.917452,  7.729030,
           7.532669,  7.344079,  7.160922,  6.995516,  6.821231,  6.672013,
           6.520191,  6.386790,  6.250359,  6.116287,  5.995753,  5.881105,
           5.789240,  5.708351,  5.605673,  5.497951,  5.371692,  5.252942,
           5.139726,  5.034308}}},
        {{{33.873305, 33.023092, 32.156387, 31.320858, 30.503003, 29.689342,
           28.881251, 28.086487, 27.296170, 26.505229, 25.727135, 24.939223,
           24.164461, 23.396533, 22.639652, 21.888554, 21.130889, 20.382704,
           19.638667, 18.899289, 18.173300, 17.459168, 16.761431, 16.080057,
           15.424190, 14.790863, 14.179554, 13.598437, 13.037886, 12.511119,
           12.012078, 11.539789, 11.094701, 10.676980, 10.281909, 9.910111,
           9.560950,  9.229669,  9.037225,  8.828535,  8.559708,  8.300173,
           8.051733,  7.822136,  7.606067,  7.404085,  7.220574,  7.054976,
           6.910138,  6.776441},
          {27.783638, 27.026682, 26.278405, 25.534253, 24.787678, 24.059173,
           23.347341, 22.659411, 21.996479, 21.342991, 20.694077, 20.036759,
           19.369094, 18.700670, 18.040190, 17.384201, 16.741485, 16.110835,
           15.501275, 14.913548, 14.344304, 13.801440, 13.281817, 12.783270,
           12.311914, 11.864824, 11.442348, 11.041418, 10.662072, 10.302338,
           9.961166,  9.638194,  9.334721,  9.103961,  8.938217,  8.697114,
           8.445479,  8.213104,  7.987848,  7.772356,  7.572150,  7.382384,
           7.200930,  7.032998,  6.875249,  6.724307,  6.589275,  6.467711,
           6.360184,  6.265013}},
         {{32.165018, 31.387900, 30.633777, 29.903137, 29.183191, 28.501394,
           27.838777, 27.204249, 26.597087, 25.823374, 24.815522, 23.938252,
           23.165063, 22.414430, 21.662520, 20.920913, 20.188568, 19.461442,
           18.751520, 18.053980, 17.365271, 16.697504, 16.048401, 15.428014,
           14.824627, 14.243483, 13.693318, 13.162401, 12.662312, 12.193372,
           11.743982, 11.318319, 10.916345, 10.544827, 10.187237, 9.851055,
           9.535642,  9.363597,  9.609916,  9.566406,  9.305247,  9.034586,
           8.766650,  8.516280,  8.279762,  8.059853,  7.856616,  7.669560,
           7.497266,  7.341434},
          {29.134078, 28.321976, 27.536780, 26.778282, 26.050134, 25.347099,
           24.680553, 24.078033, 23.551686, 22.965373, 22.166748, 21.403742,
           20.701315, 20.008308, 19.317561, 18.637085, 17.971532, 17.317219,
           16.687838, 16.073766, 15.480432, 14.908177, 14.357867, 13.840238,
           13.341434, 12.863869, 12.411398, 11.982485, 11.576597, 11.196958,
           10.833139, 10.488557, 10.160359, 9.851216,  9.557410,  9.327960,
           9.550190,  9.587980,  9.328109,  9.044590,  8.766842,  8.506483,
           8.260657,  8.033366,  7.819310,  7.621768,  7.439287,  7.273974,
           7.122380,  6.984283}},
         {{32.715961, 31.368933, 30.186719, 29.178155, 28.306835, 27.687177,
           27.264930, 27.220044, 27.628445, 27.421001, 25.448376, 23.979722,
           23.059033, 22.228584, 21.429169, 20.654637, 19.912198, 19.198933,
           18.568083, 17.932846, 17.303097, 16.703937, 16.138334, 15.637742,
           15.134790, 14.641647, 14.173859, 13.730085, 13.309271, 12.955971,
           12.575472, 12.214369, 11.874526, 11.584789, 11.272865, 11.004867,
           10.779613, 10.659934, 10.488530, 10.258899, 10.035195, 9.823573,
           9.621708,  9.443118,  9.267129,  9.090007,  8.875832,  8.668516,
           8.472049,  8.284503},
          {36.573338, 34.681034, 33.084621, 31.791619, 30.721513, 29.982806,
           29.482971, 29.435281, 29.905318, 29.650665, 27.420793, 25.791129,
           24.791244, 23.896657, 23.028450, 22.185647, 21.371840, 20.582395,
           19.879115, 19.167388, 18.463942, 17.790082, 17.141261, 16.566164,
           15.992140, 15.420427, 14.878299, 14.363452, 13.874948, 13.459543,
           13.015537, 12.594724, 12.198943, 11.856186, 11.489715, 11.172741,
           10.852251, 10.699816, 10.511281, 10.236406, 9.966597,  9.713699,
           9.475187,  9.266494,  9.073010,  8.884489,  8.663236,  8.458989,
           8.269129,  8.094236}}}},
       {{{{1.742289, 1.744374, 1.753638, 1.761988, 1.749485, 1.749651, 1.740983,
           1.738878, 1.737935, 1.732267, 1.729209, 1.725258, 1.725542, 1.711200,
           1.709211, 1.702958, 1.693590, 1.683086, 1.671034, 1.658011, 1.642579,
           1.626996, 1.615903, 1.615982, 1.616016, 1.616016, 1.616016, 1.616016,
           1.616017, 1.616133, 1.616133, 1.616133, 1.616144, 1.616254, 1.616254,
           1.616341, 1.616378, 1.616527, 1.616630, 1.616723, 1.616880, 1.617058,
           1.617283, 1.617653, 1.618037, 1.618467, 1.619181, 1.620171, 1.621447,
           1.623010},
          {1.837866, 1.837043, 1.819752, 1.814417, 1.822248, 1.804062, 1.810196,
           1.815766, 1.784166, 1.802715, 1.796782, 1.784694, 1.782656, 1.791276,
           1.768995, 1.763416, 1.765596, 1.731815, 1.730971, 1.715400, 1.700478,
           1.694058, 1.663875, 1.648339, 1.624970, 1.616016, 1.616016, 1.616016,
           1.616016, 1.616133, 1.616133, 1.616133, 1.616145, 1.616254, 1.616254,
           1.616341, 1.616378, 1.616527, 1.616630, 1.616723, 1.616880, 1.617058,
           1.617285, 1.617651, 1.618039, 1.618467, 1.619179, 1.620173, 1.621447,
           1.623008}},
         {{1.748901, 1.737592, 1.748395, 1.741605, 1.740711, 1.742600, 1.733224,
           1.737186, 1.734015, 1.716436, 1.725827, 1.728187, 1.708732, 1.709469,
           1.708588, 1.688737, 1.680072, 1.673640, 1.672842, 1.668796, 1.652797,
           1.637854, 1.636277, 1.636337, 1.636363, 1.636363, 1.636363, 1.636363,
           1.636364, 1.636449, 1.636449, 1.636449, 1.636457, 1.636536, 1.636536,
           1.636599, 1.636626, 1.636735, 1.636812, 1.636882, 1.636999, 1.637130,
           1.637295, 1.637566, 1.637849, 1.638164, 1.638690, 1.639420, 1.640355,
           1.641504},
          {1.843742, 1.835490, 1.841222, 1.832519, 1.827808, 1.830527, 1.808546,
           1.808919, 1.821146, 1.797939, 1.803182, 1.796477, 1.811166, 1.795302,
           1.788208, 1.790363, 1.779063, 1.769808, 1.748118, 1.738892, 1.715536,
           1.706733, 1.685492, 1.676946, 1.652579, 1.636363, 1.636363, 1.636363,
           1.636363, 1.636449, 1.636449, 1.636449, 1.636456, 1.636536, 1.636536,
           1.636601, 1.636626, 1.636737, 1.636812, 1.636880, 1.636997, 1.637128,
           1.637293, 1.637565, 1.637848, 1.638166, 1.638688, 1.639417, 1.640356,
           1.641508}},
         {{1.757768, 1.755812, 1.754934, 1.747598, 1.747850, 1.760075, 1.757656,
           1.763891, 1.762658, 1.768192, 1.752325, 1.757699, 1.738826, 1.741604,
           1.738600, 1.730586, 1.719829, 1.711621, 1.700563, 1.691781, 1.682131,
           1.668823, 1.668823, 1.669420, 1.669663, 1.669663, 1.669663, 1.669663,
           1.669674, 1.670524, 1.670524, 1.670524, 1.670608, 1.671409, 1.671409,
           1.672044, 1.672312, 1.673408, 1.674178, 1.674852, 1.676024, 1.677336,
           1.678982, 1.681690, 1.684530, 1.687143, 1.687638, 1.688331, 1.689219,
           1.690307},
          {1.843738, 1.841505, 1.846849, 1.841526, 1.832632, 1.834534, 1.824498,
           1.821947, 1.816473, 1.827867, 1.811612, 1.825610, 1.812021, 1.797111,
           1.810049, 1.803581, 1.809100, 1.800935, 1.776643, 1.765626, 1.753484,
           1.743357, 1.719027, 1.704854, 1.688916, 1.669663, 1.669663, 1.669663,
           1.669671, 1.670524, 1.670524, 1.670524, 1.670607, 1.671409, 1.671409,
           1.672045, 1.672312, 1.673406, 1.674178, 1.674850, 1.676025, 1.677336,
           1.678980, 1.681689, 1.684532, 1.687143, 1.687638, 1.688331, 1.689220,
           1.690309}}},
        {{{1.733296, 1.735243, 1.735381, 1.732386, 1.731466, 1.734979, 1.731084,
           1.723082, 1.739046, 1.728808, 1.735100, 1.721569, 1.723984, 1.715047,
           1.712223, 1.724601, 1.708300, 1.704257, 1.710851, 1.698315, 1.689323,
           1.687922, 1.677981, 1.670831, 1.660263, 1.654555, 1.644033, 1.630357,
           1.623421, 1.623539, 1.623539, 1.623539, 1.623550, 1.623660, 1.623660,
           1.623747, 1.623785, 1.623937, 1.624042, 1.624135, 1.624299, 1.624478,
           1.624705, 1.625079, 1.625472, 1.625908, 1.626632, 1.627641, 1.628934,
           1.630521},
          {1.728775, 1.730797, 1.726867, 1.730260, 1.734024, 1.747043, 1.736029,
           1.737967, 1.751413, 1.741484, 1.739607, 1.741537, 1.728385, 1.726147,
           1.738803, 1.732427, 1.731804, 1.720341, 1.709211, 1.705834, 1.704713,
           1.703962, 1.690517, 1.678631, 1.678538, 1.669398, 1.653956, 1.643811,
           1.625684, 1.623539, 1.623539, 1.623539, 1.623549, 1.623660, 1.623660,
           1.623749, 1.623785, 1.623937, 1.624042, 1.624135, 1.624299, 1.624478,
           1.624706, 1.625080, 1.625473, 1.625908, 1.626633, 1.627640, 1.628932,
           1.630520}},
         {{1.731265, 1.728946, 1.731270, 1.730903, 1.734496, 1.743071, 1.720624,
           1.730324, 1.757678, 1.740298, 1.737068, 1.750282, 1.730870, 1.731378,
           1.721540, 1.724431, 1.720925, 1.711507, 1.707287, 1.708394, 1.697232,
           1.690488, 1.688298, 1.690699, 1.688405, 1.680253, 1.671282, 1.652903,
           1.644068, 1.644154, 1.644154, 1.644154, 1.644165, 1.644244, 1.644244,
           1.644308, 1.644335, 1.644445, 1.644526, 1.644592, 1.644709, 1.644841,
           1.645009, 1.645283, 1.645567, 1.645888, 1.646415, 1.647151, 1.648099,
           1.649259},
          {1.728377, 1.732942, 1.729999, 1.732427, 1.732375, 1.737855, 1.733128,
           1.739091, 1.741440, 1.733624, 1.730730, 1.736110, 1.729480, 1.731538,
           1.721619, 1.725622, 1.720419, 1.716155, 1.707086, 1.702107, 1.696452,
           1.690040, 1.682833, 1.686710, 1.685087, 1.674290, 1.664865, 1.653201,
           1.644070, 1.644154, 1.644154, 1.644154, 1.644165, 1.644244, 1.644244,
           1.644310, 1.644335, 1.644447, 1.644526, 1.644591, 1.644712, 1.644844,
           1.645008, 1.645282, 1.645570, 1.645889, 1.646415, 1.647153, 1.648098,
           1.649255}},
         {{1.755704, 1.750938, 1.753309, 1.755331, 1.753418, 1.747163, 1.746178,
           1.765409, 1.775071, 1.787702, 1.779151, 1.765563, 1.770493, 1.768174,
           1.753535, 1.762424, 1.758914, 1.751953, 1.746518, 1.746129, 1.740456,
           1.733047, 1.722183, 1.719513, 1.716793, 1.706356, 1.695461, 1.686557,
           1.677672, 1.678530, 1.678530, 1.678530, 1.678613, 1.679422, 1.679422,
           1.680060, 1.680333, 1.681435, 1.682216, 1.682893, 1.684078, 1.685400,
           1.687059, 1.689791, 1.692657, 1.695291, 1.695783, 1.696469, 1.697350,
           1.698431},
          {1.751005, 1.753943, 1.754114, 1.751918, 1.751044, 1.759034, 1.767473,
           1.768217, 1.782330, 1.788591, 1.778646, 1.767848, 1.771561, 1.767885,
           1.765297, 1.762346, 1.755554, 1.755147, 1.754533, 1.746775, 1.744222,
           1.735222, 1.728420, 1.721460, 1.716945, 1.713319, 1.703301, 1.691328,
           1.680547, 1.678530, 1.678530, 1.678530, 1.678612, 1.679422, 1.679422,
           1.680063, 1.680333, 1.681437, 1.682216, 1.682892, 1.684077, 1.685400,
           1.687060, 1.689792, 1.692658, 1.695290, 1.695783, 1.696468, 1.697349,
           1.698428}}},
        {{{1.679250, 1.678391, 1.676114, 1.676606, 1.677837, 1.676017, 1.677215,
           1.679730, 1.688971, 1.694348, 1.678709, 1.684269, 1.679674, 1.681718,
           1.693469, 1.690384, 1.689189, 1.689017, 1.671898, 1.674680, 1.676057,
           1.678989, 1.663236, 1.669916, 1.669150, 1.663739, 1.658116, 1.645660,
           1.645662, 1.645783, 1.645783, 1.645783, 1.645796, 1.645910, 1.645910,
           1.646003, 1.646040, 1.646200, 1.646311, 1.646408, 1.646578, 1.646767,
           1.647006, 1.647396, 1.647806, 1.648264, 1.649019, 1.650072, 1.651426,
           1.653089},
          {1.700105, 1.700635, 1.701230, 1.700334, 1.703650, 1.702439, 1.705288,
           1.708647, 1.706981, 1.709977, 1.699541, 1.714225, 1.715418, 1.719538,
           1.714618, 1.717646, 1.700591, 1.699144, 1.712972, 1.710874, 1.693527,
           1.705586, 1.697471, 1.688183, 1.682698, 1.677960, 1.673558, 1.679347,
           1.669596, 1.664575, 1.656317, 1.645783, 1.645794, 1.645910, 1.645910,
           1.646002, 1.646041, 1.646198, 1.646311, 1.646408, 1.646576, 1.646765,
           1.647006, 1.647396, 1.647806, 1.648263, 1.649019, 1.650074, 1.651426,
           1.653083}},
         {{1.698859, 1.695176, 1.696896, 1.699695, 1.700722, 1.699064, 1.693074,
           1.697257, 1.707575, 1.710017, 1.707134, 1.707847, 1.702009, 1.707910,
           1.700150, 1.699546, 1.698628, 1.699262, 1.698230, 1.695527, 1.696070,
           1.691944, 1.692047, 1.686211, 1.680103, 1.676575, 1.672718, 1.667219,
           1.667219, 1.667307, 1.667307, 1.667307, 1.667317, 1.667398, 1.667398,
           1.667467, 1.667493, 1.667606, 1.667688, 1.667755, 1.667878, 1.668012,
           1.668185, 1.668463, 1.668758, 1.669084, 1.669628, 1.670384, 1.671352,
           1.672542},
          {1.718715, 1.721146, 1.719876, 1.719014, 1.720641, 1.720926, 1.722911,
           1.722985, 1.721086, 1.724645, 1.732009, 1.731508, 1.729564, 1.727722,
           1.729560, 1.732064, 1.730880, 1.732624, 1.727681, 1.730432, 1.725839,
           1.730198, 1.723531, 1.715634, 1.708738, 1.712331, 1.707074, 1.700826,
           1.693427, 1.689896, 1.683904, 1.676965, 1.667317, 1.667398, 1.667398,
           1.667464, 1.667493, 1.667607, 1.667688, 1.667757, 1.667876, 1.668013,
           1.668183, 1.668463, 1.668759, 1.669086, 1.669627, 1.670381, 1.671351,
           1.672539}},
         {{1.716343, 1.714240, 1.715720, 1.715995, 1.716011, 1.718981, 1.722052,
           1.728990, 1.739867, 1.742846, 1.738378, 1.736669, 1.736447, 1.735249,
           1.731204, 1.728760, 1.732528, 1.727821, 1.727149, 1.724764, 1.723661,
           1.724497, 1.717306, 1.717179, 1.714492, 1.713488, 1.707723, 1.701684,
           1.701688, 1.702577, 1.702577, 1.702577, 1.702660, 1.703490, 1.703490,
           1.704146, 1.704424, 1.705557, 1.706358, 1.707052, 1.708268, 1.709624,
           1.711327, 1.714130, 1.717071, 1.719770, 1.720245, 1.720913, 1.721770,
           1.722820},
          {1.735289, 1.736917, 1.739624, 1.735460, 1.735613, 1.734897, 1.736678,
           1.741123, 1.755766, 1.767550, 1.764980, 1.762279, 1.760810, 1.761850,
           1.760061, 1.760865, 1.759082, 1.761796, 1.756900, 1.759939, 1.753394,
           1.751063, 1.744303, 1.750370, 1.747026, 1.733374, 1.731470, 1.730459,
           1.725542, 1.723087, 1.717703, 1.709228, 1.702660, 1.703490, 1.703490,
           1.704147, 1.704423, 1.705556, 1.706358, 1.707051, 1.708265, 1.709624,
           1.711326, 1.714129, 1.717072, 1.719767, 1.720246, 1.720912, 1.721771,
           1.722821}}},
        {{{1.704214, 1.702529, 1.704916, 1.703446, 1.704823, 1.702490, 1.703633,
           1.707279, 1.708119, 1.710023, 1.711452, 1.716061, 1.706621, 1.717616,
           1.720326, 1.719587, 1.717509, 1.712421, 1.717885, 1.709639, 1.708763,
           1.716836, 1.712277, 1.708366, 1.700396, 1.703652, 1.704790, 1.697396,
           1.694683, 1.688404, 1.682844, 1.682844, 1.682856, 1.682981, 1.682981,
           1.683076, 1.683117, 1.683288, 1.683409, 1.683511, 1.683696, 1.683897,
           1.684152, 1.684573, 1.685013, 1.685505, 1.686315, 1.687449, 1.688901,
           1.690685},
          {1.754163, 1.758665, 1.755094, 1.753985, 1.757613, 1.755814, 1.753506,
           1.754147, 1.753350, 1.751839, 1.754301, 1.755067, 1.749889, 1.753617,
           1.754184, 1.754304, 1.771731, 1.760542, 1.760092, 1.754613, 1.754394,
           1.762508, 1.748686, 1.751037, 1.756658, 1.755459, 1.745793, 1.748416,
           1.748084, 1.745451, 1.734453, 1.731178, 1.730715, 1.719540, 1.717734,
           1.713307, 1.705135, 1.693589, 1.683409, 1.683512, 1.683694, 1.683898,
           1.684153, 1.684572, 1.685013, 1.685504, 1.686316, 1.687447, 1.688900,
           1.690681}},
         {{1.726104, 1.724389, 1.726434, 1.725562, 1.728067, 1.725267, 1.726119,
           1.730298, 1.731729, 1.726022, 1.727654, 1.731593, 1.732183, 1.733695,
           1.730915, 1.727517, 1.732614, 1.722341, 1.727919, 1.727041, 1.727852,
           1.723869, 1.723975, 1.723133, 1.717594, 1.717537, 1.709327, 1.705788,
           1.705789, 1.705882, 1.705882, 1.705882, 1.705890, 1.705976, 1.705976,
           1.706045, 1.706073, 1.706191, 1.706275, 1.706348, 1.706473, 1.706617,
           1.706791, 1.707086, 1.707392, 1.707735, 1.708298, 1.709084, 1.710096,
           1.711334},
          {1.786170, 1.783705, 1.781261, 1.785793, 1.781898, 1.782101, 1.776422,
           1.782653, 1.782159, 1.783360, 1.784493, 1.788663, 1.786100, 1.791457,
           1.779755, 1.788428, 1.789316, 1.791530, 1.786273, 1.789692, 1.788548,
           1.785664, 1.785063, 1.781257, 1.787490, 1.780365, 1.774869, 1.776023,
           1.771458, 1.765875, 1.755036, 1.765146, 1.746075, 1.742137, 1.738656,
           1.730698, 1.727578, 1.716691, 1.715237, 1.706349, 1.706474, 1.706615,
           1.706792, 1.707084, 1.707391, 1.707732, 1.708296, 1.709084, 1.710096,
           1.711336}},
         {{1.744883, 1.742407, 1.744393, 1.742555, 1.743178, 1.747345, 1.750503,
           1.755213, 1.763719, 1.771091, 1.768543, 1.766361, 1.762448, 1.767703,
           1.767463, 1.762999, 1.768285, 1.766990, 1.761863, 1.766113, 1.765053,
           1.765767, 1.763965, 1.756400, 1.761103, 1.752461, 1.756966, 1.752262,
           1.747867, 1.742634, 1.742634, 1.742634, 1.742723, 1.743588, 1.743588,
           1.744273, 1.744563, 1.745743, 1.746577, 1.747300, 1.748566, 1.749981,
           1.751756, 1.754675, 1.757742, 1.760547, 1.761004, 1.761639, 1.762457,
           1.763454},
          {1.808817, 1.805721, 1.802730, 1.802188, 1.799368, 1.805783, 1.809490,
           1.810535, 1.818578, 1.833212, 1.820812, 1.822832, 1.819188, 1.814607,
           1.810084, 1.817752, 1.814330, 1.832786, 1.825665, 1.828229, 1.820531,
           1.826872, 1.824113, 1.828278, 1.822079, 1.813770, 1.814918, 1.799719,
           1.819565, 1.805978, 1.804636, 1.800532, 1.782601, 1.779062, 1.785373,
           1.768131, 1.774036, 1.767429, 1.761821, 1.757913, 1.748566, 1.749981,
           1.751755, 1.754675, 1.757742, 1.760549, 1.761003, 1.761639, 1.762456,
           1.763459}}},
        {{{1.857076, 1.844762, 1.832299, 1.822122, 1.814237, 1.808751, 1.805691,
           1.803862, 1.803602, 1.810353, 1.807244, 1.809083, 1.804026, 1.795887,
           1.798906, 1.809595, 1.800466, 1.805028, 1.799144, 1.812198, 1.816028,
           1.812996, 1.819398, 1.809426, 1.823825, 1.821452, 1.830263, 1.820746,
           1.810078, 1.810443, 1.815373, 1.810624, 1.815086, 1.804957, 1.802871,
           1.809608, 1.799330, 1.794068, 1.801057, 1.793050, 1.786729, 1.785551,
           1.782513, 1.771617, 1.775725, 1.778553, 1.774137, 1.778018, 1.770249,
           1.743981},
          {2.011154, 1.983871, 1.973640, 1.948287, 1.927050, 1.909625, 1.895812,
           1.885464, 1.878494, 1.874416, 1.869762, 1.876838, 1.871608, 1.878175,
           1.871832, 1.868293, 1.865293, 1.873465, 1.864159, 1.864772, 1.874048,
           1.859120, 1.849442, 1.852861, 1.861525, 1.864387, 1.862109, 1.847152,
           1.853817, 1.851106, 1.851188, 1.837265, 1.841090, 1.841153, 1.834965,
           1.828551, 1.833030, 1.826211, 1.823998, 1.839466, 1.809409, 1.824802,
           1.819346, 1.815981, 1.802276, 1.776387, 1.792932, 1.765403, 1.766785,
           1.767998}},
         {{1.869625, 1.860421, 1.849722, 1.840460, 1.833123, 1.827986, 1.825115,
           1.823129, 1.829857, 1.827954, 1.825171, 1.829726, 1.822315, 1.834197,
           1.830627, 1.819158, 1.828304, 1.837486, 1.831738, 1.831976, 1.837795,
           1.842470, 1.840897, 1.843389, 1.840808, 1.855337, 1.831109, 1.843691,
           1.839572, 1.835349, 1.833177, 1.834760, 1.828502, 1.824120, 1.822820,
           1.836029, 1.816093, 1.823591, 1.816966, 1.821318, 1.804830, 1.808082,
           1.798421, 1.802527, 1.804491, 1.804652, 1.790543, 1.802723, 1.792240,
           1.793688},
          {2.052682, 2.026084, 1.997493, 1.969321, 1.945654, 1.926278, 1.910911,
           1.899600, 1.892398, 1.888246, 1.895886, 1.889262, 1.886721, 1.890609,
           1.889392, 1.885419, 1.890692, 1.879072, 1.877233, 1.869494, 1.887058,
           1.871181, 1.880840, 1.869335, 1.869483, 1.878467, 1.871019, 1.869970,
           1.872290, 1.872816, 1.869685, 1.868930, 1.869786, 1.867290, 1.866798,
           1.868983, 1.869109, 1.868741, 1.863656, 1.858707, 1.862036, 1.856496,
           1.853400, 1.856618, 1.834760, 1.820124, 1.807728, 1.815323, 1.803987,
           1.783410}},
         {{1.884800, 1.872275, 1.861161, 1.852434, 1.846310, 1.843461, 1.843972,
           1.847599, 1.862575, 1.865397, 1.858108, 1.858531, 1.857320, 1.859155,
           1.860021, 1.854247, 1.860881, 1.858297, 1.859188, 1.866320, 1.856423,
           1.863175, 1.860848, 1.883098, 1.896649, 1.900706, 1.882671, 1.880649,
           1.877150, 1.876717, 1.896854, 1.874871, 1.859402, 1.868100, 1.882756,
           1.874209, 1.868317, 1.865939, 1.868705, 1.865009, 1.864106, 1.853385,
           1.872259, 1.863740, 1.863531, 1.863047, 1.862077, 1.877313, 1.854706,
           1.846554},
          {2.108116, 2.080659, 2.051225, 2.021124, 2.002254, 1.992646, 1.977700,
           1.970811, 1.960258, 1.961177, 1.960915, 1.949669, 1.944304, 1.952685,
           1.946200, 1.954486, 1.952438, 1.938869, 1.956406, 1.941650, 1.944058,
           1.932113, 1.953716, 1.948065, 1.953661, 1.949433, 1.951560, 1.948536,
           1.933772, 1.931548, 1.955716, 1.955529, 1.947607, 1.923496, 1.943704,
           1.931880, 1.922272, 1.913918, 1.934822, 1.920020, 1.926904, 1.914179,
           1.918804, 1.901487, 1.908674, 1.906833, 1.901438, 1.887906, 1.878312,
           1.860144}}},
        {{{2.230603, 2.217415, 2.204297, 2.197930, 2.185561, 2.171353, 2.156869,
           2.142160, 2.127084, 2.111576, 2.095757, 2.079484, 2.067463, 2.059946,
           2.046968, 2.031189, 2.019905, 2.009485, 2.015535, 2.006264, 2.000858,
           2.000945, 1.997652, 1.995602, 1.990478, 1.990130, 1.986422, 1.984703,
           1.980411, 1.976822, 1.973149, 1.968739, 1.965064, 1.960130, 1.954969,
           1.950583, 1.946822, 1.939676, 1.932960, 1.926966, 1.920456, 1.913042,
           1.905677, 1.897828, 1.890212, 1.882668, 1.875417, 1.867449, 1.863334,
           1.858769},
          {2.516564, 2.499963, 2.473369, 2.452954, 2.431607, 2.411871, 2.389390,
           2.370298, 2.350086, 2.331713, 2.311637, 2.292056, 2.273062, 2.254227,
           2.236453, 2.218920, 2.202135, 2.185630, 2.169925, 2.152654, 2.137500,
           2.121103, 2.106648, 2.092087, 2.077184, 2.063743, 2.051295, 2.038236,
           2.025698, 2.013272, 2.002931, 1.994134, 1.986459, 1.977990, 1.968764,
           1.960067, 1.950725, 1.941973, 1.932707, 1.924248, 1.913894, 1.906009,
           1.896474, 1.887251, 1.877804, 1.869138, 1.859410, 1.852965, 1.846352,
           1.840783}},
         {{2.224487, 2.211761, 2.198654, 2.188414, 2.180469, 2.166871, 2.153147,
           2.139591, 2.126097, 2.111508, 2.094396, 2.077927, 2.061382, 2.050437,
           2.042640, 2.030071, 2.019281, 2.009169, 2.015198, 2.006194, 2.000030,
           2.001357, 1.999127, 1.999213, 2.002158, 2.002875, 2.005570, 2.004786,
           2.002841, 2.000379, 1.998090, 1.994753, 1.992043, 1.988201, 1.986190,
           1.979955, 1.976450, 1.971352, 1.966364, 1.960702, 1.955090, 1.949275,
           1.943239, 1.938027, 1.930577, 1.924060, 1.918796, 1.914383, 1.910739,
           1.908320},
          {2.561735, 2.544476, 2.517294, 2.498921, 2.476235, 2.453756, 2.431940,
           2.412716, 2.393525, 2.374788, 2.354142, 2.333560, 2.312757, 2.294107,
           2.274621, 2.257009, 2.239517, 2.221838, 2.206492, 2.189701, 2.173385,
           2.157376, 2.142275, 2.127687, 2.112917, 2.099311, 2.084713, 2.073491,
           2.061164, 2.049891, 2.038260, 2.028126, 2.017863, 2.008188, 1.999481,
           2.003894, 2.003885, 1.997379, 1.989682, 1.981016, 1.971771, 1.962113,
           1.952467, 1.943369, 1.932949, 1.923099, 1.913156, 1.904053, 1.896140,
           1.889202}},
         {{2.240647, 2.226176, 2.211606, 2.200863, 2.193838, 2.183135, 2.174576,
           2.170332, 2.172513, 2.168666, 2.141740, 2.118766, 2.101836, 2.090797,
           2.083261, 2.072466, 2.062843, 2.053723, 2.056552, 2.053899, 2.050633,
           2.052067, 2.049560, 2.051338, 2.050421, 2.050394, 2.048322, 2.047004,
           2.045901, 2.044768, 2.043617, 2.041162, 2.039742, 2.038365, 2.036270,
           2.034626, 2.031134, 2.030911, 2.029832, 2.026183, 2.023963, 2.021375,
           2.018464, 2.016927, 2.015292, 2.013151, 2.008541, 2.005000, 2.000088,
           1.995503},
          {2.617035, 2.598156, 2.570375, 2.549185, 2.525316, 2.500531, 2.497515,
           2.501095, 2.503078, 2.496363, 2.464439, 2.436174, 2.414462, 2.395222,
           2.375765, 2.356355, 2.337133, 2.318722, 2.301475, 2.283681, 2.267011,
           2.250202, 2.233271, 2.219298, 2.205846, 2.192114, 2.178919, 2.166197,
           2.154181, 2.144615, 2.133763, 2.123666, 2.112854, 2.104248, 2.094166,
           2.085677, 2.076045, 2.068202, 2.060937, 2.055660, 2.048251, 2.039115,
           2.030786, 2.022525, 2.015304, 2.007733, 1.997667, 1.988803, 1.980331,
           1.971927}}},
        {{{2.601681, 2.577492, 2.555430, 2.532871, 2.510043, 2.487503, 2.467116,
           2.446800, 2.427562, 2.408120, 2.389097, 2.370816, 2.352834, 2.336337,
           2.319943, 2.303901, 2.289148, 2.274666, 2.260289, 2.246639, 2.233978,
           2.221539, 2.209733, 2.198379, 2.187880, 2.177550, 2.167758, 2.158191,
           2.149772, 2.143359, 2.137727, 2.130062, 2.122608, 2.115008, 2.106977,
           2.099390, 2.091151, 2.082882, 2.074365, 2.065317, 2.056192, 2.046810,
           2.036967, 2.027244, 2.017543, 2.007762, 1.999033, 1.991745, 1.984505,
           1.978359},
          {3.031152, 2.978836, 2.928811, 2.882356, 2.837782, 2.795569, 2.755431,
           2.719435, 2.684168, 2.651340, 2.619547, 2.590427, 2.562107, 2.534997,
           2.508092, 2.482078, 2.457167, 2.433895, 2.410450, 2.387118, 2.363917,
           2.341433, 2.319573, 2.298358, 2.278929, 2.259078, 2.240205, 2.222354,
           2.205050, 2.189412, 2.173896, 2.159023, 2.146372, 2.137198, 2.125281,
           2.112997, 2.100392, 2.088297, 2.075805, 2.063725, 2.051507, 2.039535,
           2.027252, 2.015805, 2.003941, 1.993403, 1.983224, 1.973582, 1.965146,
           1.957933}},
         {{2.589607, 2.566736, 2.544880, 2.523163, 2.501189, 2.479675, 2.460195,
           2.441498, 2.423840, 2.405159, 2.384738, 2.366165, 2.348534, 2.331577,
           2.315655, 2.299832, 2.285415, 2.270570, 2.257291, 2.243961, 2.231238,
           2.219152, 2.208167, 2.197316, 2.186898, 2.177144, 2.167938, 2.158979,
           2.154294, 2.158682, 2.162023, 2.158055, 2.152763, 2.147120, 2.141023,
           2.134682, 2.127961, 2.121267, 2.114289, 2.106757, 2.099059, 2.090963,
           2.082827, 2.074824, 2.066803, 2.058995, 2.052034, 2.046191, 2.041714,
           2.038179},
          {3.115012, 3.059054, 3.005932, 2.954847, 2.906195, 2.862128, 2.819205,
           2.781766, 2.747987, 2.715746, 2.680445, 2.646886, 2.615516, 2.588117,
           2.560965, 2.534166, 2.508648, 2.483320, 2.459364, 2.435967, 2.411942,
           2.388995, 2.366474, 2.344764, 2.324708, 2.304713, 2.286424, 2.267967,
           2.250628, 2.234955, 2.219325, 2.204802, 2.190303, 2.176870, 2.163145,
           2.154653, 2.163163, 2.156054, 2.145350, 2.134074, 2.121649, 2.109066,
           2.095970, 2.083230, 2.070594, 2.057608, 2.045754, 2.034183, 2.024257,
           2.015535}},
         {{2.609840, 2.584053, 2.560659, 2.538602, 2.517406, 2.499645, 2.486897,
           2.479620, 2.480683, 2.475290, 2.441485, 2.413767, 2.395424, 2.378235,
           2.362584, 2.347810, 2.333329, 2.319682, 2.308350, 2.296496, 2.284710,
           2.273680, 2.263678, 2.255034, 2.246305, 2.237702, 2.230264, 2.223406,
           2.219193, 2.218370, 2.216721, 2.212784, 2.209181, 2.206624, 2.202933,
           2.200277, 2.196866, 2.194535, 2.191348, 2.187805, 2.184496, 2.180660,
           2.176465, 2.173434, 2.170165, 2.166177, 2.159643, 2.153432, 2.147167,
           2.141082},
          {3.214757, 3.152577, 3.093200, 3.038989, 2.986494, 2.936878, 2.891032,
           2.848865, 2.813593, 2.853808, 2.822370, 2.783423, 2.750726, 2.721736,
           2.692108, 2.664518, 2.637152, 2.610423, 2.585025, 2.559374, 2.533939,
           2.509994, 2.486641, 2.465237, 2.443719, 2.422789, 2.402912, 2.384296,
           2.366348, 2.351722, 2.336134, 2.321435, 2.307180, 2.295134, 2.281617,
           2.269466, 2.256688, 2.245253, 2.233247, 2.225369, 2.216620, 2.205164,
           2.193129, 2.182472, 2.171217, 2.160581, 2.147266, 2.134548, 2.124215,
           2.114525}}},
        {{{3.075421, 3.038254, 3.001167, 2.965210, 2.930311, 2.897908, 2.865380,
           2.833723, 2.803019, 2.773659, 2.745510, 2.719015, 2.692567, 2.667156,
           2.642682, 2.619130, 2.596657, 2.575877, 2.555042, 2.535146, 2.516296,
           2.498249, 2.481092, 2.464636, 2.449002, 2.434055, 2.420525, 2.406700,
           2.393205, 2.380653, 2.371248, 2.362443, 2.351564, 2.340723, 2.329943,
           2.319010, 2.307921, 2.296838, 2.285435, 2.273964, 2.262213, 2.249691,
           2.236966, 2.224394, 2.211791, 2.199327, 2.187703, 2.177067, 2.167570,
           2.159525},
          {3.680860, 3.606293, 3.535052, 3.466577, 3.402676, 3.341015, 3.283268,
           3.230207, 3.180133, 3.132664, 3.086673, 3.043373, 3.000609, 2.959566,
           2.920654, 2.884344, 2.848347, 2.812803, 2.777875, 2.743895, 2.711026,
           2.678236, 2.646390, 2.615620, 2.586610, 2.558527, 2.531091, 2.505058,
           2.480272, 2.456668, 2.434308, 2.413752, 2.393081, 2.374296, 2.360750,
           2.342787, 2.324838, 2.307091, 2.289530, 2.272546, 2.255531, 2.238503,
           2.221692, 2.205494, 2.189618, 2.174265, 2.160111, 2.147346, 2.136068,
           2.126326}},
         {{3.061158, 3.024098, 2.987902, 2.952666, 2.919220, 2.887674, 2.856385,
           2.826490, 2.798500, 2.770234, 2.740181, 2.711804, 2.685467, 2.660325,
           2.636033, 2.612912, 2.591202, 2.570270, 2.549849, 2.530526, 2.512055,
           2.494477, 2.477714, 2.461966, 2.447015, 2.432896, 2.419824, 2.406700,
           2.394151, 2.386334, 2.393254, 2.394957, 2.388475, 2.380681, 2.372366,
           2.363757, 2.354815, 2.345732, 2.336298, 2.326495, 2.316624, 2.306546,
           2.295709, 2.285034, 2.274379, 2.263995, 2.254470, 2.246387, 2.239864,
           2.234744},
          {3.832226, 3.748573, 3.667831, 3.593140, 3.521879, 3.453922, 3.391747,
           3.331902, 3.280488, 3.235138, 3.184530, 3.134860, 3.088286, 3.045565,
           3.003674, 2.963523, 2.925259, 2.888289, 2.851093, 2.814659, 2.779368,
           2.745696, 2.713162, 2.681530, 2.651109, 2.621897, 2.594442, 2.568343,
           2.542860, 2.519099, 2.496384, 2.474655, 2.453682, 2.433624, 2.414248,
           2.394805, 2.388326, 2.393779, 2.379782, 2.363801, 2.346815, 2.329407,
           2.312458, 2.294880, 2.277409, 2.260238, 2.243897, 2.228719, 2.214937,
           2.202612}},
         {{3.084479, 3.044294, 3.005571, 2.970625, 2.938116, 2.910081, 2.886881,
           2.873085, 2.871465, 2.861607, 2.812499, 2.772561, 2.744868, 2.719680,
           2.695688, 2.673271, 2.651310, 2.630138, 2.611687, 2.593291, 2.575341,
           2.558486, 2.543568, 2.531068, 2.518569, 2.506994, 2.495964, 2.485899,
           2.476841, 2.471732, 2.466519, 2.464616, 2.460900, 2.457689, 2.452813,
           2.448998, 2.444371, 2.440932, 2.436799, 2.431894, 2.426826, 2.421221,
           2.415446, 2.410837, 2.405981, 2.400431, 2.392033, 2.383165, 2.374262,
           2.365781},
          {4.011370, 3.916151, 3.825032, 3.740664, 3.659635, 3.585669, 3.515556,
           3.449528, 3.390245, 3.333208, 3.332854, 3.323871, 3.283424, 3.239546,
           3.195530, 3.153465, 3.110464, 3.068016, 3.028951, 2.990655, 2.951751,
           2.914171, 2.878329, 2.846250, 2.814884, 2.783639, 2.753958, 2.725620,
           2.698940, 2.676666, 2.652262, 2.628676, 2.606293, 2.586628, 2.565541,
           2.547900, 2.529572, 2.514346, 2.497435, 2.481197, 2.471337, 2.455967,
           2.440001, 2.425031, 2.410085, 2.395130, 2.377293, 2.360552, 2.345128,
           2.331101}}},
        {{{4.347040, 4.264849, 4.185738, 4.110418, 4.036078, 3.965050, 3.899047,
           3.833153, 3.770315, 3.709633, 3.653600, 3.598869, 3.546983, 3.496944,
           3.450742, 3.404954, 3.360955, 3.319144, 3.280011, 3.243056, 3.207127,
           3.172718, 3.139524, 3.107862, 3.078631, 3.049610, 3.021338, 2.994163,
           2.967923, 2.942884, 2.918779, 2.896909, 2.878133, 2.863444, 2.843811,
           2.824064, 2.804173, 2.784587, 2.764859, 2.744933, 2.725900, 2.705950,
           2.686268, 2.667205, 2.648380, 2.629924, 2.612745, 2.597319, 2.583349,
           2.571819},
          {5.397559, 5.257254, 5.128694, 5.003137, 4.886063, 4.776147, 4.672169,
           4.576793, 4.486296, 4.403273, 4.322889, 4.241929, 4.163988, 4.088519,
           4.014131, 3.941791, 3.871320, 3.799799, 3.730293, 3.663789, 3.598652,
           3.535808, 3.475634, 3.417757, 3.361289, 3.307481, 3.257602, 3.208800,
           3.162294, 3.117933, 3.076798, 3.035644, 2.995582, 2.956714, 2.919152,
           2.885197, 2.859818, 2.827519, 2.794935, 2.762525, 2.731638, 2.700409,
           2.670035, 2.641138, 2.613419, 2.587245, 2.562966, 2.541958, 2.522931,
           2.506628}},
         {{4.310473, 4.230813, 4.154785, 4.080615, 4.008618, 3.941358, 3.877628,
           3.815357, 3.757020, 3.699040, 3.638396, 3.582028, 3.530609, 3.482126,
           3.435702, 3.390395, 3.347356, 3.306369, 3.269100, 3.232113, 3.196991,
           3.163083, 3.130847, 3.100797, 3.072124, 3.043839, 3.016822, 2.991194,
           2.966724, 2.943843, 2.921655, 2.902967, 2.900913, 2.912697, 2.901454,
           2.887193, 2.871781, 2.855942, 2.839718, 2.823002, 2.806173, 2.790001,
           2.772785, 2.756024, 2.739681, 2.723800, 2.709303, 2.696808, 2.686528,
           2.678075},
          {5.756046, 5.595485, 5.443116, 5.297976, 5.163722, 5.035107, 4.915713,
           4.806584, 4.703483, 4.608935, 4.516808, 4.432733, 4.351027, 4.268244,
           4.186616, 4.106359, 4.025960, 3.949081, 3.875402, 3.802041, 3.731442,
           3.664975, 3.600257, 3.538923, 3.480901, 3.425074, 3.370992, 3.319775,
           3.272542, 3.227163, 3.183176, 3.140147, 3.099137, 3.059605, 3.020224,
           2.982699, 2.946522, 2.912514, 2.906721, 2.903142, 2.876721, 2.847612,
           2.817559, 2.788581, 2.758977, 2.730343, 2.703285, 2.678284, 2.655663,
           2.635217}},
         {{4.324899, 4.244732, 4.165115, 4.090776, 4.023474, 3.964309, 3.913751,
           3.879456, 3.867548, 3.839337, 3.747718, 3.672380, 3.619418, 3.570172,
           3.522680, 3.477511, 3.434760, 3.395425, 3.359242, 3.323530, 3.288567,
           3.255683, 3.225162, 3.198354, 3.171288, 3.145764, 3.122765, 3.102329,
           3.083868, 3.070076, 3.054695, 3.041351, 3.029951, 3.022664, 3.017376,
           3.014881, 3.006800, 3.000105, 2.991478, 2.981638, 2.972200, 2.962210,
           2.952183, 2.944084, 2.935674, 2.926284, 2.911870, 2.897269, 2.882707,
           2.868706},
          {6.186031, 5.998164, 5.817634, 5.649772, 5.491490, 5.344020, 5.206118,
           5.078543, 4.959036, 4.850610, 4.743859, 4.641309, 4.543996, 4.515064,
           4.520956, 4.453191, 4.371628, 4.288411, 4.207320, 4.128255, 4.050792,
           3.974463, 3.901550, 3.836430, 3.772049, 3.709978, 3.651076, 3.595907,
           3.541732, 3.493620, 3.443982, 3.397485, 3.352306, 3.311803, 3.269613,
           3.232291, 3.194037, 3.159763, 3.126436, 3.094826, 3.066019, 3.040621,
           3.020232, 2.995227, 2.970012, 2.944467, 2.915081, 2.887779, 2.862159,
           2.838919}}},
        {{{6.557691, 6.381236, 6.218239, 6.060051, 5.908598, 5.766962, 5.629159,
           5.499495, 5.372305, 5.251966, 5.138880, 5.027931, 4.921733, 4.822728,
           4.726444, 4.634498, 4.550399, 4.466796, 4.387950, 4.313906, 4.244971,
           4.177225, 4.113108, 4.052879, 3.995087, 3.939211, 3.885780, 3.834235,
           3.786784, 3.739197, 3.693802, 3.650235, 3.608324, 3.568705, 3.533367,
           3.508858, 3.475919, 3.442868, 3.409872, 3.377159, 3.346279, 3.314903,
           3.284233, 3.254431, 3.225598, 3.197733, 3.171976, 3.148531, 3.129060,
           3.111485},
          {7.966688, 7.764917, 7.575165, 7.386890, 7.201630, 7.023771, 6.854149,
           6.691342, 6.542407, 6.395502, 6.252206, 6.106248, 5.959102, 5.819536,
           5.681250, 5.547166, 5.417484, 5.289360, 5.170275, 5.051707, 4.937038,
           4.829058, 4.723574, 4.622640, 4.528951, 4.436027, 4.348153, 4.265748,
           4.184510, 4.105656, 4.030130, 3.954844, 3.882361, 3.813446, 3.746334,
           3.680829, 3.617853, 3.558296, 3.515780, 3.464935, 3.413336, 3.362861,
           3.314927, 3.268483, 3.223983, 3.182011, 3.143583, 3.109865, 3.078717,
           3.051440}},
         {{6.452147, 6.288825, 6.134894, 5.984237, 5.842979, 5.707778, 5.578235,
           5.457388, 5.339622, 5.225481, 5.103447, 4.990538, 4.887245, 4.788523,
           4.694314, 4.606277, 4.521394, 4.439916, 4.363512, 4.292495, 4.223475,
           4.157517, 4.094842, 4.037071, 3.980341, 3.926300, 3.875116, 3.827190,
           3.781256, 3.737184, 3.694792, 3.654358, 3.615767, 3.580153, 3.556673,
           3.581666, 3.564432, 3.540631, 3.514722, 3.487833, 3.460760, 3.434729,
           3.407668, 3.381366, 3.355618, 3.330818, 3.308269, 3.288740, 3.272185,
           3.258204},
          {8.534289, 8.303288, 8.085374, 7.878008, 7.679749, 7.492983, 7.307934,
           7.131515, 6.961620, 6.795556, 6.630866, 6.466028, 6.302999, 6.153761,
           6.004724, 5.857123, 5.712297, 5.572585, 5.439606, 5.308861, 5.187911,
           5.068241, 4.953271, 4.846244, 4.742327, 4.642812, 4.551382, 4.460220,
           4.372918, 4.290586, 4.208796, 4.129398, 4.054226, 3.981059, 3.910230,
           3.842904, 3.779293, 3.716972, 3.656774, 3.598447, 3.557169, 3.571618,
           3.531062, 3.486221, 3.441440, 3.396598, 3.353731, 3.313602, 3.276892,
           3.243803}},
         {{6.235395, 6.210642, 6.085377, 5.952003, 5.821387, 5.709233, 5.608877,
           5.536958, 5.504086, 5.439134, 5.263906, 5.120911, 5.014144, 4.915885,
           4.824676, 4.735013, 4.649922, 4.569301, 4.499143, 4.428682, 4.360283,
           4.295517, 4.235523, 4.180889, 4.128446, 4.080117, 4.035826, 3.995654,
           3.956503, 3.925349, 3.892132, 3.861439, 3.833543, 3.811783, 3.787267,
           3.768947, 3.750797, 3.742206, 3.732376, 3.715407, 3.698628, 3.681510,
           3.664805, 3.651188, 3.637240, 3.621305, 3.598633, 3.575255, 3.552490,
           3.530590},
          {9.184410, 8.919331, 8.666222, 8.424717, 8.197829, 7.987335, 7.785385,
           7.595113, 7.409480, 7.226808, 7.044986, 6.863458, 6.682146, 6.508953,
           6.337374, 6.283106, 6.310692, 6.195239, 6.059015, 5.914684, 5.772962,
           5.632177, 5.501516, 5.379543, 5.260351, 5.145161, 5.033247, 4.925257,
           4.824489, 4.731173, 4.636348, 4.547632, 4.463420, 4.387181, 4.308965,
           4.238865, 4.168131, 4.107065, 4.048710, 3.990533, 3.935988, 3.883576,
           3.834356, 3.795793, 3.759739, 3.717316, 3.668582, 3.623108, 3.581823,
           3.542761}}},
        {{{11.575054, 11.183417, 10.794384, 10.406021, 10.041949, 9.700594,
           9.374409,  9.067947,  8.778834,  8.502920,  8.249209,  8.004475,
           7.772368,  7.552421,  7.347123,  7.150694,  6.964913,  6.790268,
           6.623856,  6.465020,  6.315234,  6.175227,  6.038610,  5.909726,
           5.788286,  5.669907,  5.557734,  5.452299,  5.350523,  5.253424,
           5.160537,  5.072629,  4.989517,  4.908803,  4.830804,  4.756014,
           4.685738,  4.628538,  4.576706,  4.515952,  4.455631,  4.397816,
           4.340734,  4.285321,  4.231771,  4.180397,  4.132590,  4.089714,
           4.052396,  4.018549},
          {12.399348, 12.054531, 11.730298, 11.426552, 11.128139, 10.829509,
           10.530093, 10.246878, 9.989249,  9.744461,  9.502696,  9.263984,
           9.036869,  8.817739,  8.605372,  8.402914,  8.209723,  8.020999,
           7.839171,  7.666089,  7.494206,  7.324956,  7.150537,  6.976460,
           6.805102,  6.635511,  6.466982,  6.303720,  6.145404,  5.989551,
           5.841889,  5.697062,  5.558012,  5.427245,  5.299292,  5.177200,
           5.061717,  4.951562,  4.845265,  4.743248,  4.652283,  4.582005,
           4.498588,  4.417747,  4.340657,  4.264905,  4.194491,  4.129751,
           4.072606,  4.020156}},
         {{11.284960, 10.918342, 10.541977, 10.180951, 9.847489, 9.533492,
           9.232874,  8.952730,  8.689269,  8.429106,  8.158712, 7.909165,
           7.683089,  7.469967,  7.270868,  7.079295,  6.897808, 6.729628,
           6.567684,  6.413348,  6.269701,  6.131225,  5.999152, 5.876691,
           5.757483,  5.644462,  5.536395,  5.436597,  5.338754, 5.246880,
           5.158846,  5.076641,  4.997947,  4.923383,  4.850690, 4.781783,
           4.716586,  4.685427,  4.698693,  4.659598,  4.610776, 4.559445,
           4.509537,  4.459308,  4.410603,  4.364179,  4.321862, 4.284208,
           4.251645,  4.223478},
          {13.412915, 13.000341, 12.605730, 12.226613, 11.878223, 11.550525,
           11.239447, 10.940344, 10.642541, 10.361362, 10.090122, 9.832494,
           9.602284,  9.361366,  9.128030,  8.903056,  8.685856,  8.478043,
           8.283642,  8.090584,  7.900816,  7.717008,  7.534598,  7.357629,
           7.177485,  6.998974,  6.824715,  6.653320,  6.484960,  6.324334,
           6.168349,  6.015548,  5.871328,  5.731560,  5.596708,  5.470290,
           5.347456,  5.230175,  5.117867,  5.011572,  4.908195,  4.808485,
           4.714487,  4.698735,  4.661824,  4.588705,  4.516319,  4.445165,
           4.378776,  4.317477}},
         {{10.616569, 10.257465, 9.924970, 9.909895, 9.714937, 9.470268,
           9.245911,  9.079602,  8.993147, 8.832229, 8.437905, 8.116389,
           7.882164,  7.667144,  7.464705, 7.274908, 7.095388, 6.924268,
           6.769791,  6.624265,  6.479702, 6.343460, 6.218625, 6.105023,
           5.995201,  5.888895,  5.791238, 5.696929, 5.609106, 5.535177,
           5.458220,  5.387205,  5.320782, 5.264250, 5.203669, 5.151381,
           5.098668,  5.055358,  5.012821, 4.977387, 4.950525, 4.917496,
           4.882651,  4.852313,  4.821484, 4.789902, 4.748903, 4.707692,
           4.668197,  4.630391},
          {14.563684, 14.073130, 13.601173, 13.153981, 12.740724, 12.349360,
           11.986167, 11.652348, 11.345198, 11.056698, 10.759684, 10.458548,
           10.167439, 9.897679,  10.130321, 10.072883, 9.852089,  9.598049,
           9.351880,  9.104696,  8.861344,  8.624151,  8.397706,  8.192613,
           7.992757,  7.800263,  7.614574,  7.432775,  7.256225,  7.092794,
           6.922246,  6.757342,  6.599729,  6.453103,  6.305488,  6.168201,
           6.030768,  5.905690,  5.785101,  5.664114,  5.553161,  5.449354,
           5.352167,  5.267826,  5.188469,  5.113003,  5.020613,  4.934183,
           4.853987,  4.782568}}},
        {{{30.874990, 29.567316, 28.300896, 27.043482, 25.841342, 24.696210,
           23.585083, 22.516330, 21.511880, 20.557482, 19.656011, 18.812954,
           18.017534, 17.268345, 16.572658, 15.901506, 15.282093, 14.694888,
           14.148852, 13.636400, 13.151808, 12.695993, 12.265243, 11.857871,
           11.475421, 11.112683, 10.773180, 10.453364, 10.152437, 9.863576,
           9.591416,  9.337857,  9.089237,  8.851739,  8.630810,  8.418282,
           8.216083,  8.024551,  7.839443,  7.658852,  7.492064,  7.377982,
           7.234811,  7.089902,  6.953108,  6.823606,  6.704257,  6.598394,
           6.499172,  6.409681},
          {25.910814, 25.006523, 24.136984, 23.294363, 22.494767, 21.738100,
           21.033058, 20.354069, 19.695570, 19.071562, 18.462065, 17.855920,
           17.263513, 16.702643, 16.150607, 15.623788, 15.117591, 14.630338,
           14.167838, 13.727241, 13.303449, 12.898363, 12.513319, 12.141439,
           11.788008, 11.451460, 11.127534, 10.819934, 10.526049, 10.247112,
           9.972392,  9.705664,  9.453207,  9.206995,  8.970979,  8.746636,
           8.530919,  8.326896,  8.131276,  7.943109,  7.757997,  7.578701,
           7.433809,  7.306014,  7.145479,  6.991434,  6.846231,  6.711539,
           6.591057,  6.478070}},
         {{28.898445, 27.756809, 26.618522, 25.532892, 24.520031, 23.519498,
           22.551755, 21.644930, 20.786111, 19.893720, 18.944299, 18.105728,
           17.359178, 16.669063, 16.005014, 15.391921, 14.810546, 14.270058,
           13.765558, 13.285811, 12.833168, 12.406591, 12.002169, 11.624668,
           11.265327, 10.925375, 10.604612, 10.307934, 10.022475, 9.753440,
           9.501489,  9.258066,  9.026553,  8.809899,  8.601275,  8.406633,
           8.220300,  8.044084,  7.874408,  7.708910,  7.569273,  7.597448,
           7.541236,  7.414766,  7.290021,  7.169207,  7.055665,  6.952363,
           6.862048,  6.779823},
          {28.134170, 27.111643, 26.138060, 25.214740, 24.368531, 23.534431,
           22.735462, 22.004774, 21.359847, 20.692279, 19.897314, 19.162072,
           18.498575, 17.863160, 17.256399, 16.678429, 16.113449, 15.589822,
           15.088407, 14.610292, 14.155648, 13.722052, 13.306626, 12.913023,
           12.538507, 12.176473, 11.832254, 11.500625, 11.178165, 10.872357,
           10.575571, 10.293923, 10.019016, 9.756646,  9.508091,  9.267836,
           9.037724,  8.821520,  8.613773,  8.416516,  8.228649,  8.048288,
           7.874479,  7.707030,  7.585669,  7.622638,  7.513325,  7.372257,
           7.233882,  7.100708}},
         {{25.747391, 26.359873, 25.429465, 24.359512, 23.348175, 22.497208,
           21.788599, 21.326720, 21.141781, 20.631648, 19.206303, 18.099282,
           17.342710, 16.661743, 16.036904, 15.454898, 14.911351, 14.398248,
           13.945434, 13.499579, 13.072897, 12.672394, 12.292394, 11.956616,
           11.628743, 11.315974, 11.016176, 10.732566, 10.469354, 10.245645,
           10.011486, 9.791981,  9.590615,  9.416643,  9.235964,  9.078582,
           8.923501,  8.791681,  8.658886,  8.524629,  8.405112,  8.299523,
           8.218121,  8.136978,  8.043332,  7.950551,  7.836163,  7.727779,
           7.625317,  7.528271},
          {30.475541, 29.367280, 28.303081, 27.248048, 26.226774, 25.534201,
           26.928891, 27.103877, 27.242215, 26.680203, 24.594179, 23.023886,
           21.977831, 21.061185, 20.217509, 19.440482, 18.710088, 18.020140,
           17.418519, 16.831075, 16.272089, 15.744731, 15.241513, 14.787275,
           14.335423, 13.896936, 13.473849, 13.072966, 12.691647, 12.354192,
           12.005696, 11.673430, 11.363392, 11.083458, 10.792295, 10.535448,
           10.280455, 10.054103, 9.828534,  9.613843,  9.416411,  9.229199,
           9.056404,  8.910088,  8.765066,  8.620002,  8.435335,  8.266560,
           8.106390,  7.959683}}}},
       {{{{1.950134, 1.937528, 1.949896, 1.938549, 1.943638, 1.935009, 1.936613,
           1.935872, 1.933083, 1.929606, 1.931849, 1.921587, 1.918024, 1.922778,
           1.909186, 1.911521, 1.899599, 1.894858, 1.884799, 1.876165, 1.868699,
           1.859099, 1.859099, 1.859156, 1.859181, 1.859181, 1.859181, 1.859181,
           1.859182, 1.859266, 1.859266, 1.859266, 1.859275, 1.859352, 1.859352,
           1.859415, 1.859442, 1.859546, 1.859622, 1.859689, 1.859805, 1.859934,
           1.860094, 1.860360, 1.860637, 1.860947, 1.861459, 1.862172, 1.863093,
           1.864214},
          {2.043566, 2.029447, 2.024128, 2.018759, 2.029011, 2.017775, 2.019776,
           2.008499, 2.006668, 2.013905, 2.002051, 2.008340, 1.995946, 2.000849,
           1.987778, 1.982703, 1.979070, 1.970042, 1.962204, 1.953180, 1.948671,
           1.936093, 1.921726, 1.904023, 1.888908, 1.867364, 1.859181, 1.859181,
           1.859182, 1.859266, 1.859266, 1.859266, 1.859273, 1.859352, 1.859352,
           1.859415, 1.859442, 1.859547, 1.859622, 1.859690, 1.859806, 1.859931,
           1.860094, 1.860357, 1.860638, 1.860946, 1.861459, 1.862172, 1.863090,
           1.864215}},
         {{1.939341, 1.937818, 1.951484, 1.939905, 1.941262, 1.943927, 1.943141,
           1.934479, 1.938613, 1.936406, 1.935001, 1.924743, 1.921174, 1.918710,
           1.920917, 1.914932, 1.910239, 1.909018, 1.898795, 1.891947, 1.883118,
           1.873796, 1.873796, 1.873839, 1.873858, 1.873858, 1.873858, 1.873858,
           1.873858, 1.873920, 1.873920, 1.873920, 1.873926, 1.873986, 1.873986,
           1.874031, 1.874051, 1.874131, 1.874187, 1.874236, 1.874321, 1.874419,
           1.874538, 1.874736, 1.874945, 1.875176, 1.875557, 1.876092, 1.876778,
           1.877620},
          {2.041653, 2.044241, 2.033377, 2.024614, 2.014813, 2.022154, 2.017702,
           2.013969, 2.014005, 2.004072, 2.009643, 2.000250, 1.994986, 1.998679,
           1.986592, 1.985641, 1.983201, 1.975952, 1.967654, 1.958097, 1.943320,
           1.934562, 1.916213, 1.905459, 1.898980, 1.880623, 1.873858, 1.873858,
           1.873858, 1.873920, 1.873920, 1.873920, 1.873923, 1.873986, 1.873986,
           1.874031, 1.874051, 1.874130, 1.874187, 1.874236, 1.874322, 1.874417,
           1.874539, 1.874737, 1.874944, 1.875174, 1.875561, 1.876095, 1.876780,
           1.877622}},
         {{1.957308, 1.952939, 1.959563, 1.954526, 1.959744, 1.962616, 1.962551,
           1.956156, 1.960867, 1.966581, 1.961335, 1.957631, 1.950055, 1.944687,
           1.937865, 1.939841, 1.930380, 1.928549, 1.922345, 1.913605, 1.906069,
           1.897614, 1.897614, 1.898051, 1.898227, 1.898227, 1.898227, 1.898227,
           1.898230, 1.898860, 1.898860, 1.898860, 1.898918, 1.899507, 1.899507,
           1.899970, 1.900168, 1.900969, 1.901535, 1.902026, 1.902884, 1.903843,
           1.905046, 1.907030, 1.909109, 1.911021, 1.911387, 1.911902, 1.912559,
           1.913366},
          {2.047342, 2.036697, 2.033638, 2.038753, 2.032414, 2.012802, 2.024492,
           2.014948, 2.022177, 2.024362, 2.019467, 2.019703, 2.019152, 2.016218,
           2.021207, 2.016758, 2.007795, 2.005504, 2.008142, 1.981524, 1.984779,
           1.973236, 1.949309, 1.935720, 1.922915, 1.910318, 1.898227, 1.898227,
           1.898234, 1.898860, 1.898860, 1.898860, 1.898918, 1.899507, 1.899507,
           1.899969, 1.900168, 1.900968, 1.901535, 1.902025, 1.902883, 1.903842,
           1.905047, 1.907030, 1.909111, 1.911020, 1.911389, 1.911899, 1.912559,
           1.913365}}},
        {{{1.941921, 1.939843, 1.939725, 1.936168, 1.937603, 1.942646, 1.943195,
           1.929107, 1.941306, 1.930186, 1.933899, 1.943334, 1.939013, 1.936895,
           1.938011, 1.920809, 1.928232, 1.930054, 1.916583, 1.922685, 1.908446,
           1.914356, 1.901200, 1.903575, 1.889874, 1.885858, 1.872242, 1.866384,
           1.866384, 1.866470, 1.866470, 1.866470, 1.866479, 1.866560, 1.866560,
           1.866624, 1.866650, 1.866759, 1.866835, 1.866902, 1.867019, 1.867151,
           1.867312, 1.867583, 1.867867, 1.868182, 1.868704, 1.869431, 1.870366,
           1.871505},
          {1.944940, 1.944937, 1.942370, 1.942832, 1.937837, 1.943714, 1.946650,
           1.951225, 1.950885, 1.951781, 1.948845, 1.952792, 1.935945, 1.939098,
           1.942644, 1.940482, 1.931447, 1.940418, 1.923750, 1.930060, 1.924210,
           1.923851, 1.916640, 1.913733, 1.900162, 1.899388, 1.885750, 1.880815,
           1.866386, 1.866470, 1.866470, 1.866470, 1.866480, 1.866560, 1.866560,
           1.866622, 1.866650, 1.866757, 1.866835, 1.866902, 1.867020, 1.867149,
           1.867314, 1.867582, 1.867865, 1.868182, 1.868704, 1.869431, 1.870366,
           1.871512}},
         {{1.939995, 1.944605, 1.943931, 1.943659, 1.939305, 1.939221, 1.941080,
           1.942550, 1.940830, 1.940472, 1.936663, 1.938609, 1.932816, 1.935553,
           1.928936, 1.936008, 1.931909, 1.926402, 1.923282, 1.919958, 1.919655,
           1.922231, 1.921057, 1.913101, 1.906688, 1.897679, 1.893327, 1.883803,
           1.881308, 1.881370, 1.881370, 1.881370, 1.881376, 1.881436, 1.881436,
           1.881484, 1.881502, 1.881585, 1.881639, 1.881690, 1.881776, 1.881873,
           1.881993, 1.882192, 1.882404, 1.882634, 1.883024, 1.883561, 1.884250,
           1.885100},
          {1.946109, 1.946087, 1.945766, 1.944650, 1.943880, 1.946085, 1.939717,
           1.952409, 1.948911, 1.944160, 1.949043, 1.944534, 1.943085, 1.942995,
           1.943625, 1.942391, 1.937622, 1.928757, 1.929182, 1.925614, 1.918802,
           1.917461, 1.920611, 1.921058, 1.912677, 1.908005, 1.898986, 1.889590,
           1.881308, 1.881370, 1.881370, 1.881370, 1.881378, 1.881436, 1.881436,
           1.881482, 1.881502, 1.881583, 1.881639, 1.881688, 1.881776, 1.881872,
           1.881995, 1.882194, 1.882404, 1.882636, 1.883023, 1.883560, 1.884251,
           1.885098}},
         {{1.954899, 1.956315, 1.955380, 1.953275, 1.948217, 1.953240, 1.951569,
           1.961444, 1.972250, 1.972899, 1.969729, 1.960417, 1.963562, 1.955479,
           1.958194, 1.953667, 1.952678, 1.953103, 1.946486, 1.947008, 1.943155,
           1.939997, 1.935819, 1.931245, 1.925796, 1.920036, 1.913298, 1.905858,
           1.905862, 1.906494, 1.906494, 1.906494, 1.906550, 1.907143, 1.907143,
           1.907612, 1.907809, 1.908615, 1.909184, 1.909682, 1.910546, 1.911512,
           1.912725, 1.914721, 1.916818, 1.918741, 1.919100, 1.919601, 1.920247,
           1.921038},
          {1.960931, 1.958267, 1.957793, 1.960181, 1.966276, 1.964954, 1.965236,
           1.970419, 1.984963, 1.975774, 1.967685, 1.973816, 1.961134, 1.959897,
           1.972720, 1.968915, 1.964888, 1.953473, 1.958950, 1.958832, 1.950567,
           1.946016, 1.941859, 1.943086, 1.936273, 1.928566, 1.918222, 1.913313,
           1.905864, 1.906494, 1.906494, 1.906494, 1.906551, 1.907143, 1.907143,
           1.907611, 1.907809, 1.908614, 1.909184, 1.909681, 1.910544, 1.911511,
           1.912724, 1.914722, 1.916815, 1.918741, 1.919100, 1.919603, 1.920249,
           1.921038}}},
        {{{1.909118, 1.909804, 1.908627, 1.909790, 1.910290, 1.912036, 1.908631,
           1.910761, 1.918007, 1.915648, 1.918426, 1.915623, 1.916721, 1.916078,
           1.914403, 1.914493, 1.912372, 1.910586, 1.911218, 1.909806, 1.908169,
           1.904889, 1.904261, 1.900063, 1.899369, 1.896394, 1.893065, 1.888004,
           1.888006, 1.888096, 1.888096, 1.888096, 1.888101, 1.888187, 1.888187,
           1.888251, 1.888282, 1.888396, 1.888477, 1.888549, 1.888671, 1.888810,
           1.888981, 1.889267, 1.889565, 1.889896, 1.890445, 1.891209, 1.892192,
           1.893401},
          {1.935062, 1.936988, 1.935029, 1.938057, 1.935139, 1.933765, 1.934111,
           1.937406, 1.935484, 1.937787, 1.936065, 1.934845, 1.941843, 1.951478,
           1.946551, 1.941305, 1.940401, 1.948165, 1.942578, 1.944686, 1.938967,
           1.942075, 1.939902, 1.933335, 1.926811, 1.923930, 1.923515, 1.917089,
           1.917879, 1.908687, 1.904252, 1.901149, 1.897204, 1.888187, 1.888187,
           1.888252, 1.888282, 1.888396, 1.888477, 1.888549, 1.888671, 1.888810,
           1.888980, 1.889266, 1.889561, 1.889896, 1.890444, 1.891210, 1.892194,
           1.893397}},
         {{1.923090, 1.923451, 1.923721, 1.923863, 1.924810, 1.926035, 1.926782,
           1.928195, 1.925427, 1.926326, 1.931914, 1.930429, 1.934092, 1.933441,
           1.927891, 1.931836, 1.929500, 1.921962, 1.926811, 1.921729, 1.919934,
           1.921371, 1.921288, 1.916847, 1.916034, 1.913766, 1.903902, 1.903666,
           1.903667, 1.903733, 1.903733, 1.903733, 1.903738, 1.903795, 1.903795,
           1.903844, 1.903867, 1.903948, 1.904005, 1.904057, 1.904146, 1.904244,
           1.904369, 1.904571, 1.904785, 1.905025, 1.905417, 1.905969, 1.906673,
           1.907539},
          {1.948787, 1.950964, 1.947406, 1.950376, 1.946005, 1.947292, 1.944930,
           1.946564, 1.943448, 1.941955, 1.947161, 1.950638, 1.959333, 1.951941,
           1.957068, 1.958332, 1.953926, 1.947451, 1.944488, 1.950168, 1.948616,
           1.940417, 1.946832, 1.938743, 1.932960, 1.936791, 1.933488, 1.929792,
           1.923150, 1.917791, 1.917868, 1.907916, 1.903736, 1.903795, 1.903795,
           1.903846, 1.903867, 1.903948, 1.904005, 1.904058, 1.904145, 1.904244,
           1.904369, 1.904571, 1.904785, 1.905024, 1.905418, 1.905969, 1.906672,
           1.907540}},
         {{1.934569, 1.935797, 1.935741, 1.936251, 1.935746, 1.937106, 1.940950,
           1.946467, 1.951436, 1.952325, 1.953166, 1.948886, 1.954134, 1.953542,
           1.943021, 1.952084, 1.943117, 1.950169, 1.941532, 1.940204, 1.939943,
           1.945000, 1.943155, 1.935792, 1.939222, 1.931416, 1.928748, 1.928748,
           1.928753, 1.929397, 1.929397, 1.929397, 1.929456, 1.930062, 1.930062,
           1.930541, 1.930740, 1.931567, 1.932148, 1.932653, 1.933537, 1.934525,
           1.935766, 1.937802, 1.939943, 1.941907, 1.942244, 1.942717, 1.943322,
           1.944067},
          {1.960721, 1.962507, 1.958528, 1.958667, 1.959660, 1.960482, 1.961223,
           1.960129, 1.967934, 1.976892, 1.975081, 1.971595, 1.979007, 1.971580,
           1.975764, 1.970110, 1.971242, 1.975161, 1.972349, 1.965552, 1.965698,
           1.960569, 1.968129, 1.968611, 1.956934, 1.956609, 1.956958, 1.954540,
           1.946191, 1.943466, 1.938687, 1.935851, 1.929455, 1.930062, 1.930062,
           1.930540, 1.930740, 1.931566, 1.932148, 1.932652, 1.933536, 1.934526,
           1.935763, 1.937803, 1.939944, 1.941906, 1.942243, 1.942717, 1.943323,
           1.944066}}},
        {{{1.943936, 1.943334, 1.945032, 1.944153, 1.942997, 1.943334, 1.940708,
           1.941214, 1.940688, 1.942975, 1.939936, 1.940429, 1.941628, 1.944829,
           1.955544, 1.955703, 1.949097, 1.953555, 1.953823, 1.942820, 1.953585,
           1.950352, 1.945818, 1.946238, 1.937337, 1.935979, 1.933226, 1.938308,
           1.936708, 1.934964, 1.924129, 1.924133, 1.924144, 1.924235, 1.924235,
           1.924308, 1.924336, 1.924459, 1.924547, 1.924624, 1.924758, 1.924907,
           1.925096, 1.925403, 1.925727, 1.926084, 1.926680, 1.927509, 1.928577,
           1.929877},
          {2.010864, 2.010611, 2.006559, 2.002112, 2.003233, 2.000890, 2.001116,
           1.997135, 1.997294, 1.999988, 1.997265, 2.003740, 2.006278, 2.001515,
           2.003192, 2.009017, 2.007279, 1.997120, 2.010733, 2.008200, 2.005767,
           2.008736, 2.007509, 1.998049, 2.005068, 2.002358, 1.993586, 1.989929,
           1.988643, 1.986359, 1.982352, 1.979621, 1.979516, 1.978164, 1.972193,
           1.970263, 1.962494, 1.955247, 1.949232, 1.943674, 1.939074, 1.934816,
           1.933403, 1.926709, 1.925725, 1.926086, 1.926681, 1.927512, 1.928576,
           1.929877}},
         {{1.959748, 1.957211, 1.959299, 1.958034, 1.959427, 1.957774, 1.960069,
           1.959569, 1.961091, 1.960958, 1.960188, 1.961607, 1.963400, 1.963290,
           1.964234, 1.970965, 1.968858, 1.961465, 1.969205, 1.963184, 1.962492,
           1.965424, 1.961689, 1.959443, 1.957821, 1.956360, 1.953873, 1.954902,
           1.947238, 1.945126, 1.946572, 1.941002, 1.941008, 1.941070, 1.941070,
           1.941120, 1.941138, 1.941225, 1.941283, 1.941337, 1.941430, 1.941531,
           1.941660, 1.941869, 1.942093, 1.942340, 1.942748, 1.943316, 1.944047,
           1.944949},
          {2.018856, 2.016925, 2.020420, 2.019448, 2.018666, 2.018123, 2.018268,
           2.017189, 2.019828, 2.015504, 2.019174, 2.020385, 2.017976, 2.016197,
           2.010146, 2.013006, 2.016414, 2.019341, 2.017911, 2.015661, 2.019666,
           2.009162, 2.006191, 2.020980, 2.012549, 2.007504, 2.010851, 1.997080,
           2.011788, 2.005344, 2.001075, 1.999129, 1.995031, 1.993930, 1.981169,
           1.979747, 1.975475, 1.977819, 1.970692, 1.962252, 1.958476, 1.955142,
           1.946756, 1.941873, 1.942091, 1.942338, 1.942749, 1.943317, 1.944046,
           1.944944}},
         {{1.973245, 1.970471, 1.972046, 1.971046, 1.971695, 1.972555, 1.974132,
           1.980542, 1.985172, 1.991094, 1.985603, 1.986329, 1.988274, 1.990348,
           1.990835, 1.986324, 1.991028, 1.986548, 1.985156, 1.985219, 1.986040,
           1.983879, 1.984353, 1.986775, 1.985801, 1.979803, 1.977365, 1.978762,
           1.976744, 1.973443, 1.967574, 1.967574, 1.967634, 1.968261, 1.968261,
           1.968756, 1.968965, 1.969820, 1.970419, 1.970944, 1.971860, 1.972881,
           1.974166, 1.976277, 1.978493, 1.980519, 1.980820, 1.981242, 1.981783,
           1.982449},
          {2.039989, 2.033693, 2.032635, 2.033522, 2.032888, 2.034297, 2.035899,
           2.033423, 2.046985, 2.054619, 2.048504, 2.040754, 2.035092, 2.038116,
           2.036558, 2.040300, 2.045064, 2.046055, 2.043343, 2.045767, 2.047930,
           2.042611, 2.050643, 2.043759, 2.036805, 2.035952, 2.028047, 2.042097,
           2.040680, 2.021933, 2.028601, 2.017145, 2.039447, 2.024300, 2.018377,
           2.020269, 2.019351, 1.999283, 2.010335, 2.001692, 1.994694, 1.996589,
           1.987674, 1.981983, 1.985534, 1.980519, 1.980823, 1.981242, 1.981783,
           1.982446}}},
        {{{2.096397, 2.081813, 2.071217, 2.063865, 2.056341, 2.049157, 2.042596,
           2.037054, 2.035145, 2.040454, 2.037379, 2.041192, 2.037683, 2.036239,
           2.042102, 2.040399, 2.036927, 2.042113, 2.035762, 2.033146, 2.033141,
           2.042993, 2.046593, 2.044156, 2.051758, 2.047923, 2.044866, 2.052900,
           2.056903, 2.045993, 2.041881, 2.038991, 2.047884, 2.047785, 2.039197,
           2.032797, 2.037530, 2.031905, 2.025438, 2.027050, 2.020607, 2.033826,
           2.020307, 2.016816, 2.019099, 2.023327, 2.018717, 2.022864, 2.016945,
           2.010789},
          {2.271409, 2.250659, 2.229210, 2.205529, 2.181615, 2.162072, 2.150858,
           2.137033, 2.119382, 2.110229, 2.104722, 2.111253, 2.106542, 2.102141,
           2.107070, 2.101780, 2.105448, 2.102509, 2.094966, 2.100542, 2.094862,
           2.084202, 2.093502, 2.093862, 2.091495, 2.085898, 2.077736, 2.071905,
           2.061610, 2.077012, 2.074103, 2.071976, 2.070406, 2.070257, 2.069296,
           2.066419, 2.060326, 2.062266, 2.041782, 2.059038, 2.046498, 2.051395,
           2.040245, 2.023859, 2.027894, 2.016938, 2.002603, 2.008118, 2.001691,
           1.988521}},
         {{2.095038, 2.083526, 2.081892, 2.076566, 2.068975, 2.062517, 2.057650,
           2.054627, 2.053566, 2.051823, 2.055832, 2.053100, 2.057702, 2.056393,
           2.053050, 2.056721, 2.060168, 2.059167, 2.064545, 2.062827, 2.058856,
           2.062940, 2.064687, 2.068533, 2.071251, 2.067931, 2.064882, 2.066125,
           2.067835, 2.066037, 2.062232, 2.057834, 2.068275, 2.069318, 2.067648,
           2.049140, 2.048433, 2.058893, 2.058039, 2.043614, 2.053997, 2.048441,
           2.040408, 2.041377, 2.039603, 2.040426, 2.035530, 2.034796, 2.033182,
           2.028998},
          {2.285889, 2.265262, 2.244146, 2.220911, 2.196434, 2.183044, 2.170233,
           2.154682, 2.142665, 2.128552, 2.114826, 2.110985, 2.117268, 2.112508,
           2.118343, 2.110053, 2.110042, 2.112240, 2.106869, 2.112112, 2.110624,
           2.105057, 2.099287, 2.102269, 2.100221, 2.097757, 2.090945, 2.092017,
           2.088532, 2.091476, 2.083843, 2.082485, 2.084657, 2.085727, 2.082800,
           2.082366, 2.081413, 2.082202, 2.081584, 2.082106, 2.075314, 2.075944,
           2.076980, 2.065130, 2.058998, 2.054214, 2.048863, 2.036905, 2.029721,
           2.005189}},
         {{2.102917, 2.096994, 2.090731, 2.084553, 2.078310, 2.073039, 2.069122,
           2.067929, 2.077326, 2.080064, 2.073302, 2.074514, 2.072171, 2.076564,
           2.073983, 2.074272, 2.081488, 2.074498, 2.077116, 2.078227, 2.086356,
           2.087359, 2.087859, 2.085143, 2.089171, 2.091631, 2.089240, 2.089094,
           2.088453, 2.082974, 2.082473, 2.094559, 2.080826, 2.086194, 2.086372,
           2.078395, 2.084541, 2.076480, 2.076136, 2.087267, 2.071540, 2.072855,
           2.075396, 2.073085, 2.081042, 2.074559, 2.073465, 2.070276, 2.074820,
           2.072419},
          {2.318878, 2.299267, 2.278010, 2.256134, 2.232169, 2.225870, 2.207609,
           2.195412, 2.190348, 2.185142, 2.170506, 2.151018, 2.157226, 2.152509,
           2.159323, 2.154727, 2.150503, 2.155080, 2.151555, 2.152484, 2.146146,
           2.146987, 2.149639, 2.152157, 2.144222, 2.151482, 2.151429, 2.151051,
           2.147116, 2.131256, 2.127350, 2.138631, 2.135555, 2.131327, 2.120716,
           2.122004, 2.142922, 2.121857, 2.111503, 2.120200, 2.116671, 2.121519,
           2.134429, 2.118929, 2.106301, 2.113374, 2.092479, 2.082409, 2.079961,
           2.054617}}},
        {{{2.385538, 2.376145, 2.366465, 2.362639, 2.352846, 2.342750, 2.332465,
           2.321956, 2.311305, 2.301045, 2.289900, 2.278528, 2.271288, 2.265807,
           2.254939, 2.243120, 2.231124, 2.221831, 2.217130, 2.218079, 2.210788,
           2.209364, 2.207082, 2.203096, 2.201860, 2.197772, 2.196908, 2.194331,
           2.193480, 2.189911, 2.187400, 2.184486, 2.181317, 2.178084, 2.174232,
           2.170894, 2.166491, 2.162774, 2.157555, 2.152673, 2.147234, 2.141606,
           2.135560, 2.129674, 2.123710, 2.117422, 2.112922, 2.106316, 2.101259,
           2.097419},
          {2.692219, 2.673668, 2.648584, 2.627985, 2.606839, 2.586764, 2.564211,
           2.544762, 2.524012, 2.505132, 2.486251, 2.467405, 2.448941, 2.430596,
           2.413610, 2.396967, 2.379500, 2.364706, 2.349467, 2.335348, 2.321693,
           2.308346, 2.296051, 2.283687, 2.272102, 2.260787, 2.249519, 2.238956,
           2.228984, 2.219522, 2.210160, 2.201589, 2.194416, 2.188868, 2.181677,
           2.174257, 2.166942, 2.159856, 2.152748, 2.145670, 2.138172, 2.131320,
           2.123443, 2.115177, 2.108995, 2.101975, 2.095390, 2.089643, 2.084463,
           2.080025}},
         {{2.381066, 2.371818, 2.362320, 2.357602, 2.349080, 2.339388, 2.329565,
           2.319827, 2.310685, 2.300506, 2.288332, 2.276410, 2.267412, 2.261805,
           2.252903, 2.241186, 2.229466, 2.220612, 2.212476, 2.217179, 2.210074,
           2.207852, 2.206038, 2.202721, 2.202157, 2.200324, 2.201397, 2.207242,
           2.206672, 2.205045, 2.203036, 2.201183, 2.200166, 2.196352, 2.193292,
           2.190317, 2.186933, 2.183756, 2.179468, 2.175095, 2.170769, 2.166064,
           2.161067, 2.155870, 2.151064, 2.146391, 2.141857, 2.138323, 2.135284,
           2.132093},
          {2.716275, 2.698803, 2.671200, 2.652113, 2.630151, 2.609690, 2.587191,
           2.567708, 2.546725, 2.530424, 2.510766, 2.491250, 2.474033, 2.455557,
           2.438823, 2.421782, 2.405566, 2.390493, 2.375387, 2.361324, 2.347700,
           2.334529, 2.321410, 2.310373, 2.299311, 2.287862, 2.276967, 2.265949,
           2.256460, 2.247259, 2.237827, 2.229180, 2.220634, 2.212922, 2.204558,
           2.200960, 2.206256, 2.201806, 2.195423, 2.189107, 2.181926, 2.174607,
           2.166933, 2.159102, 2.151418, 2.143840, 2.136344, 2.129470, 2.123172,
           2.118259}},
         {{2.393940, 2.383714, 2.373350, 2.365708, 2.360346, 2.352412, 2.345894,
           2.343083, 2.344268, 2.341010, 2.321275, 2.304598, 2.292363, 2.284414,
           2.278777, 2.269281, 2.258046, 2.249949, 2.242972, 2.247831, 2.241442,
           2.238653, 2.238716, 2.237395, 2.237672, 2.237393, 2.237332, 2.234725,
           2.234006, 2.232628, 2.230880, 2.229309, 2.227448, 2.226556, 2.224467,
           2.222756, 2.220995, 2.219595, 2.217685, 2.216337, 2.213930, 2.212027,
           2.210507, 2.209684, 2.208405, 2.206762, 2.203409, 2.199987, 2.196749,
           2.194023},
          {2.743704, 2.725653, 2.698038, 2.678148, 2.656097, 2.635045, 2.612903,
           2.592597, 2.571731, 2.552637, 2.561635, 2.549414, 2.534669, 2.519384,
           2.502865, 2.487268, 2.471534, 2.455995, 2.442303, 2.428905, 2.414916,
           2.401812, 2.389690, 2.378541, 2.366608, 2.355507, 2.345559, 2.335102,
           2.325013, 2.316238, 2.307005, 2.298094, 2.289754, 2.282181, 2.274390,
           2.267060, 2.260032, 2.253554, 2.246759, 2.242680, 2.237858, 2.231200,
           2.224866, 2.217578, 2.212843, 2.206647, 2.198779, 2.191372, 2.184580,
           2.178557}}},
        {{{2.672383, 2.657204, 2.641590, 2.627406, 2.611994, 2.596785, 2.582920,
           2.569193, 2.555616, 2.541838, 2.528259, 2.515253, 2.502441, 2.490395,
           2.478403, 2.466741, 2.455443, 2.444802, 2.434296, 2.424131, 2.414269,
           2.405068, 2.396029, 2.387812, 2.379921, 2.372195, 2.364559, 2.357296,
           2.350240, 2.343851, 2.337811, 2.334894, 2.329528, 2.323930, 2.318178,
           2.312357, 2.306360, 2.300299, 2.293839, 2.287226, 2.280127, 2.272901,
           2.265093, 2.257583, 2.249786, 2.242271, 2.234757, 2.227874, 2.222171,
           2.217184},
          {3.162531, 3.111603, 3.063957, 3.017499, 2.973561, 2.933767, 2.897310,
           2.861076, 2.826612, 2.794607, 2.763255, 2.735490, 2.707479, 2.680081,
           2.654593, 2.630258, 2.606893, 2.584673, 2.563903, 2.543698, 2.524245,
           2.505718, 2.488278, 2.471327, 2.455485, 2.440471, 2.425386, 2.411003,
           2.397223, 2.384878, 2.372455, 2.360260, 2.348554, 2.337834, 2.330679,
           2.321303, 2.311486, 2.301952, 2.292333, 2.282514, 2.272963, 2.263137,
           2.253817, 2.244320, 2.235219, 2.226015, 2.217943, 2.210622, 2.204179,
           2.198373}},
         {{2.665905, 2.651213, 2.635692, 2.621860, 2.606817, 2.592112, 2.578955,
           2.566260, 2.553648, 2.540514, 2.525804, 2.512158, 2.499519, 2.487203,
           2.475240, 2.464060, 2.452842, 2.442039, 2.431531, 2.421938, 2.412234,
           2.402950, 2.394123, 2.386855, 2.378764, 2.370979, 2.363732, 2.356824,
           2.350149, 2.343941, 2.346425, 2.351106, 2.348070, 2.344130, 2.339625,
           2.334935, 2.330093, 2.324958, 2.319580, 2.313922, 2.307969, 2.301798,
           2.295506, 2.289378, 2.282981, 2.276934, 2.271322, 2.266640, 2.262596,
           2.259071},
          {3.210347, 3.156373, 3.106152, 3.058779, 3.012978, 2.969949, 2.929400,
           2.892288, 2.859288, 2.825809, 2.795689, 2.766541, 2.740443, 2.714038,
           2.687792, 2.664141, 2.640808, 2.618246, 2.597235, 2.577145, 2.558853,
           2.540287, 2.522478, 2.506055, 2.489606, 2.474563, 2.459571, 2.445020,
           2.431371, 2.418054, 2.405563, 2.393779, 2.382999, 2.371965, 2.361060,
           2.350587, 2.342207, 2.350570, 2.343748, 2.335212, 2.326155, 2.316724,
           2.306705, 2.296949, 2.287185, 2.277073, 2.267946, 2.259131, 2.250802,
           2.243682}},
         {{2.680967, 2.664569, 2.648247, 2.634226, 2.619518, 2.607669, 2.598842,
           2.593208, 2.594477, 2.590204, 2.566283, 2.546749, 2.533259, 2.521174,
           2.509148, 2.498032, 2.487125, 2.476431, 2.467054, 2.457987, 2.448633,
           2.439864, 2.432019, 2.425141, 2.417782, 2.410523, 2.403803, 2.397703,
           2.391817, 2.388710, 2.388065, 2.384400, 2.380765, 2.377962, 2.374520,
           2.371561, 2.368307, 2.365868, 2.363142, 2.360264, 2.357993, 2.355811,
           2.353700, 2.352332, 2.350571, 2.348287, 2.343876, 2.339831, 2.335936,
           2.332173},
          {3.265419, 3.208859, 3.154850, 3.104947, 3.058012, 3.013002, 2.970999,
           2.931733, 2.899182, 2.865938, 2.833357, 2.802727, 2.795142, 2.789378,
           2.769220, 2.746593, 2.724268, 2.702492, 2.682783, 2.662988, 2.643535,
           2.626026, 2.608345, 2.592561, 2.576280, 2.560016, 2.544757, 2.529514,
           2.515976, 2.503817, 2.490863, 2.478567, 2.466698, 2.456779, 2.445898,
           2.437011, 2.426864, 2.417630, 2.408310, 2.398632, 2.393019, 2.384347,
           2.374991, 2.366275, 2.358513, 2.350958, 2.341239, 2.332167, 2.323670,
           2.315420}}},
        {{{3.055077, 3.030727, 3.007042, 2.983736, 2.961046, 2.938684, 2.916958,
           2.896177, 2.875991, 2.855578, 2.835935, 2.816744, 2.798035, 2.779993,
           2.762423, 2.745434, 2.728905, 2.713452, 2.698760, 2.684065, 2.669923,
           2.656461, 2.643563, 2.631296, 2.619573, 2.608443, 2.597801, 2.587588,
           2.577859, 2.568529, 2.559653, 2.551571, 2.547232, 2.539844, 2.531957,
           2.524605, 2.516883, 2.508687, 2.500207, 2.491378, 2.482239, 2.472760,
           2.462983, 2.453165, 2.443200, 2.433410, 2.424041, 2.415399, 2.407601,
           2.400752},
          {3.693005, 3.626290, 3.560722, 3.498948, 3.442374, 3.387057, 3.335014,
           3.286938, 3.241269, 3.197775, 3.155579, 3.114962, 3.077346, 3.040480,
           3.005631, 2.972396, 2.940873, 2.910851, 2.883295, 2.855912, 2.830015,
           2.805110, 2.781433, 2.758846, 2.736994, 2.715770, 2.695759, 2.675617,
           2.656176, 2.637509, 2.619600, 2.602463, 2.585942, 2.570176, 2.555025,
           2.545433, 2.532337, 2.520210, 2.507062, 2.493961, 2.481042, 2.468196,
           2.455434, 2.443008, 2.430909, 2.418983, 2.407866, 2.397695, 2.388556,
           2.380551}},
         {{3.045957, 3.022151, 2.998820, 2.975976, 2.953870, 2.932191, 2.911292,
           2.892086, 2.873073, 2.853404, 2.831973, 2.811795, 2.793123, 2.775081,
           2.757642, 2.740747, 2.724440, 2.709773, 2.694798, 2.680367, 2.666499,
           2.653309, 2.640710, 2.628826, 2.617419, 2.606596, 2.596228, 2.586414,
           2.577081, 2.568344, 2.559889, 2.562293, 2.567715, 2.563311, 2.557535,
           2.551895, 2.545731, 2.538929, 2.531813, 2.524359, 2.516695, 2.508714,
           2.500554, 2.492410, 2.484351, 2.476377, 2.468990, 2.462661, 2.457292,
           2.453006},
          {3.780552, 3.707205, 3.639140, 3.572334, 3.509788, 3.452865, 3.397544,
           3.346062, 3.298127, 3.253946, 3.210356, 3.167882, 3.129174, 3.092588,
           3.057322, 3.023119, 2.990648, 2.959888, 2.930651, 2.903180, 2.877511,
           2.852237, 2.828278, 2.805383, 2.783057, 2.761108, 2.739769, 2.719354,
           2.700342, 2.681580, 2.663482, 2.646155, 2.629634, 2.613931, 2.598696,
           2.584105, 2.569874, 2.556576, 2.567256, 2.557625, 2.547031, 2.534606,
           2.521741, 2.508781, 2.495782, 2.483008, 2.470610, 2.458988, 2.447976,
           2.437908}},
         {{3.061553, 3.036490, 3.012019, 2.989118, 2.967388, 2.948847, 2.934333,
           2.924700, 2.923178, 2.915519, 2.881814, 2.854090, 2.834418, 2.816404,
           2.798921, 2.782122, 2.767027, 2.751594, 2.737878, 2.724292, 2.710909,
           2.698179, 2.686081, 2.675679, 2.665313, 2.655031, 2.645480, 2.636554,
           2.628197, 2.621890, 2.615206, 2.614428, 2.610478, 2.606836, 2.602185,
           2.598312, 2.594298, 2.591737, 2.588383, 2.584613, 2.581245, 2.577802,
           2.574452, 2.571694, 2.568476, 2.564559, 2.558300, 2.552289, 2.546700,
           2.542010},
          {3.884578, 3.804245, 3.728791, 3.658470, 3.590761, 3.527076, 3.469351,
           3.414643, 3.363433, 3.315888, 3.271710, 3.227442, 3.185217, 3.146585,
           3.156667, 3.132923, 3.102915, 3.072952, 3.044789, 3.017276, 2.990250,
           2.965017, 2.940911, 2.918078, 2.895195, 2.872266, 2.850105, 2.828636,
           2.807979, 2.789845, 2.772016, 2.754043, 2.737008, 2.722191, 2.706547,
           2.692635, 2.678427, 2.665891, 2.652881, 2.639606, 2.627207, 2.619532,
           2.607680, 2.597339, 2.585845, 2.574074, 2.560154, 2.546929, 2.535954,
           2.525621}}},
        {{{3.963243, 3.921244, 3.877759, 3.835477, 3.794079, 3.753653, 3.714006,
           3.677074, 3.639309, 3.602296, 3.566678, 3.532010, 3.498814, 3.468127,
           3.437684, 3.408015, 3.379524, 3.351878, 3.325391, 3.299709, 3.274996,
           3.251447, 3.229445, 3.207188, 3.185891, 3.165441, 3.145803, 3.127000,
           3.108790, 3.091252, 3.074344, 3.058100, 3.042483, 3.027566, 3.018036,
           3.005078, 2.991023, 2.977133, 2.963040, 2.948730, 2.934310, 2.919696,
           2.904989, 2.890424, 2.875846, 2.861484, 2.848025, 2.835610, 2.825465,
           2.815384},
          {4.961175, 4.849466, 4.742702, 4.644207, 4.549133, 4.460473, 4.379025,
           4.300543, 4.227004, 4.159186, 4.092240, 4.027386, 3.965252, 3.908298,
           3.851723, 3.798209, 3.746979, 3.698822, 3.652887, 3.608001, 3.565362,
           3.524553, 3.486133, 3.450727, 3.414119, 3.378112, 3.342593, 3.307832,
           3.273906, 3.241957, 3.210141, 3.179088, 3.149084, 3.120384, 3.092576,
           3.065710, 3.040424, 3.019947, 3.000295, 2.977538, 2.955159, 2.933251,
           2.911900, 2.891511, 2.871813, 2.852958, 2.835502, 2.820708, 2.806308,
           2.793628}},
         {{3.947226, 3.905366, 3.862860, 3.821409, 3.780782, 3.741614, 3.703933,
           3.668158, 3.632652, 3.596749, 3.558329, 3.522373, 3.489556, 3.459183,
           3.428650, 3.399324, 3.371178, 3.343914, 3.317806, 3.292567, 3.268195,
           3.245803, 3.223318, 3.201622, 3.180762, 3.160664, 3.141566, 3.123221,
           3.105587, 3.088819, 3.072633, 3.057052, 3.042870, 3.028738, 3.046966,
           3.039350, 3.029001, 3.017943, 3.006412, 2.994561, 2.982503, 2.970290,
           2.957939, 2.945786, 2.933746, 2.922002, 2.911033, 2.901379, 2.893153,
           2.886431},
          {5.181078, 5.053255, 4.935239, 4.821327, 4.713850, 4.615222, 4.521183,
           4.435953, 4.356242, 4.281108, 4.208328, 4.139490, 4.071450, 4.006889,
           3.949113, 3.895012, 3.840937, 3.789021, 3.739946, 3.694153, 3.649504,
           3.606266, 3.565433, 3.526016, 3.487887, 3.451268, 3.414324, 3.378513,
           3.343726, 3.310410, 3.278027, 3.247741, 3.217747, 3.188822, 3.160704,
           3.133941, 3.107871, 3.082400, 3.057387, 3.033878, 3.044935, 3.027711,
           3.007453, 2.986845, 2.966309, 2.946151, 2.926804, 2.908517, 2.891434,
           2.875974}},
         {{3.952949, 3.915367, 3.874004, 3.833262, 3.794412, 3.760847, 3.731918,
           3.711305, 3.703718, 3.686605, 3.629612, 3.582939, 3.549699, 3.518056,
           3.487844, 3.458689, 3.430585, 3.403576, 3.379210, 3.354849, 3.332043,
           3.308855, 3.286557, 3.266868, 3.247160, 3.227815, 3.209474, 3.192127,
           3.175663, 3.162353, 3.147888, 3.135066, 3.124116, 3.116081, 3.113997,
           3.107665, 3.100484, 3.094886, 3.088875, 3.082667, 3.077162, 3.071589,
           3.065865, 3.061142, 3.056012, 3.050097, 3.040816, 3.031863, 3.023282,
           3.014883},
          {5.453577, 5.306674, 5.165731, 5.035391, 4.915187, 4.800257, 4.694454,
           4.596546, 4.506002, 4.424022, 4.344282, 4.267021, 4.192785, 4.124213,
           4.057073, 3.993878, 3.987625, 3.968097, 3.926264, 3.881055, 3.835404,
           3.792245, 3.749083, 3.707822, 3.666637, 3.625395, 3.586313, 3.549067,
           3.512247, 3.479585, 3.445791, 3.413299, 3.382315, 3.354842, 3.326854,
           3.300419, 3.273601, 3.249490, 3.225140, 3.200693, 3.177815, 3.155670,
           3.135848, 3.122102, 3.103593, 3.084697, 3.062947, 3.042407, 3.023224,
           3.005520}}},
        {{{5.332237, 5.252422, 5.176270, 5.099944, 5.024804, 4.950876, 4.880040,
           4.812655, 4.745814, 4.681350, 4.619301, 4.560735, 4.504272, 4.448309,
           4.394135, 4.342133, 4.291501, 4.244716, 4.197707, 4.153056, 4.110244,
           4.069545, 4.030450, 3.993187, 3.958991, 3.924348, 3.890960, 3.858750,
           3.827737, 3.797772, 3.768813, 3.740859, 3.713329, 3.687766, 3.661102,
           3.634858, 3.613373, 3.596079, 3.573334, 3.550262, 3.527468, 3.504724,
           3.482096, 3.460871, 3.439494, 3.418133, 3.398185, 3.379792, 3.363066,
           3.348042},
          {6.887825, 6.700755, 6.528673, 6.360853, 6.206993, 6.062860, 5.929007,
           5.805584, 5.687195, 5.575574, 5.471315, 5.367963, 5.268732, 5.175989,
           5.085087, 4.997659, 4.912989, 4.835077, 4.758755, 4.686452, 4.617547,
           4.553268, 4.490592, 4.427541, 4.364951, 4.303293, 4.244677, 4.185495,
           4.128638, 4.073777, 4.020962, 3.971642, 3.922889, 3.875339, 3.829557,
           3.785338, 3.742550, 3.700932, 3.659947, 3.618387, 3.590841, 3.553612,
           3.516913, 3.481271, 3.448123, 3.415068, 3.384651, 3.357200, 3.332629,
           3.311001}},
         {{5.301787, 5.223605, 5.149220, 5.073630, 5.000585, 4.928554, 4.860880,
           4.795176, 4.732550, 4.669551, 4.603474, 4.543509, 4.485962, 4.430466,
           4.376824, 4.325024, 4.275822, 4.228858, 4.182933, 4.139287, 4.097296,
           4.057139, 4.018841, 3.983484, 3.948797, 3.914807, 3.882050, 3.850615,
           3.820311, 3.791657, 3.763616, 3.736648, 3.710824, 3.686718, 3.661755,
           3.637799, 3.640866, 3.644836, 3.628702, 3.610601, 3.591968, 3.573263,
           3.554271, 3.535783, 3.518534, 3.501328, 3.484804, 3.470062, 3.457284,
           3.446442},
          {7.366721, 7.146870, 6.943455, 6.747112, 6.566770, 6.393679, 6.235644,
           6.091048, 5.956943, 5.834494, 5.712929, 5.593802, 5.482491, 5.375658,
           5.274184, 5.179640, 5.087656, 5.005568, 4.922326, 4.845148, 4.769352,
           4.697566, 4.628118, 4.561700, 4.495528, 4.429895, 4.365640, 4.303223,
           4.244765, 4.186805, 4.131578, 4.078658, 4.027918, 3.981285, 3.935077,
           3.890188, 3.846365, 3.803481, 3.761108, 3.719380, 3.680021, 3.639877,
           3.649659, 3.622728, 3.590810, 3.558543, 3.527219, 3.498676, 3.470627,
           3.444785}},
         {{5.233711, 5.153230, 5.086330, 5.059089, 4.999098, 4.941745, 4.888474,
           4.850076, 4.831636, 4.799559, 4.707187, 4.627309, 4.567442, 4.511919,
           4.458379, 4.408018, 4.358661, 4.311286, 4.268546, 4.226299, 4.185003,
           4.145554, 4.109186, 4.075380, 4.041436, 4.007828, 3.975930, 3.945632,
           3.916914, 3.892637, 3.866880, 3.843098, 3.822896, 3.805275, 3.785841,
           3.770124, 3.754662, 3.752462, 3.744810, 3.734294, 3.724609, 3.714609,
           3.704756, 3.696752, 3.688583, 3.679503, 3.665569, 3.652207, 3.639250,
           3.627143},
          {7.942730, 7.687386, 7.447769, 7.218889, 7.005107, 6.801490, 6.615831,
           6.441824, 6.280572, 6.137457, 5.997934, 5.865253, 5.735564, 5.611943,
           5.496617, 5.387488, 5.283999, 5.187838, 5.095329, 5.121119, 5.068930,
           5.002828, 4.929880, 4.859319, 4.787847, 4.718404, 4.649667, 4.582204,
           4.516814, 4.457705, 4.399185, 4.341078, 4.286655, 4.237864, 4.188142,
           4.142804, 4.098244, 4.056486, 4.014406, 3.972852, 3.934303, 3.897126,
           3.862835, 3.833180, 3.804357, 3.775885, 3.740772, 3.708056, 3.677931,
           3.650122}}},
        {{{8.401947, 8.211117, 8.027100, 7.852568, 7.683209, 7.518432, 7.363271,
           7.211601, 7.065869, 6.926213, 6.794730, 6.665242, 6.541359, 6.426737,
           6.312518, 6.202239, 6.095939, 5.994989, 5.899684, 5.804997, 5.714474,
           5.628462, 5.547804, 5.469048, 5.393435, 5.321359, 5.252632, 5.188499,
           5.126059, 5.065664, 5.007811, 4.952157, 4.898162, 4.845588, 4.795735,
           4.747136, 4.699119, 4.652035, 4.608655, 4.575512, 4.535636, 4.495003,
           4.456741, 4.417512, 4.379360, 4.342533, 4.308299, 4.276626, 4.247548,
           4.221137},
          {10.552383, 10.274506, 10.007625, 9.746089, 9.491828, 9.251395,
           9.022887,  8.804224,  8.602201,  8.413066, 8.225582, 8.040896,
           7.864597,  7.695309,  7.531270,  7.377511, 7.228919, 7.086351,
           6.949905,  6.821261,  6.690182,  6.561249, 6.438714, 6.315873,
           6.196006,  6.079333,  5.968471,  5.859429, 5.752104, 5.648527,
           5.550339,  5.454098,  5.360698,  5.271266, 5.187393, 5.104890,
           5.022881,  4.943058,  4.865397,  4.790971, 4.718009, 4.646402,
           4.588117,  4.531832,  4.472932,  4.415182, 4.360200, 4.309682,
           4.263652,  4.222094}},
         {{8.302710, 8.118970, 7.942606, 7.776238, 7.610564, 7.451873, 7.300362,
           7.154326, 7.018766, 6.885594, 6.746066, 6.614958, 6.495083, 6.379976,
           6.267669, 6.159072, 6.055318, 5.957823, 5.862217, 5.769572, 5.680999,
           5.596420, 5.517683, 5.440073, 5.366218, 5.295733, 5.228501, 5.166871,
           5.105369, 5.046932, 4.991321, 4.937814, 4.886530, 4.837402, 4.791657,
           4.745840, 4.701375, 4.658746, 4.628109, 4.635546, 4.611162, 4.580165,
           4.548262, 4.516930, 4.484898, 4.453891, 4.424925, 4.398918, 4.376056,
           4.356314},
          {11.368895, 11.045041, 10.739027, 10.452224, 10.175244, 9.911582,
           9.654765,  9.410627,  9.185317,  8.969703,  8.755164,  8.547045,
           8.342146,  8.143971,  7.954707,  7.778437,  7.605689,  7.442811,
           7.288506,  7.141714,  6.997471,  6.856931,  6.715916,  6.579174,
           6.450753,  6.323446,  6.200211,  6.082039,  5.970727,  5.862440,
           5.756569,  5.655168,  5.559561,  5.466739,  5.376318,  5.288581,
           5.204440,  5.122324,  5.041224,  4.962938,  4.887659,  4.815583,
           4.747007,  4.680149,  4.630487,  4.623515,  4.575653,  4.527766,
           4.479176,  4.433532}},
         {{8.084633, 7.900844, 7.726687, 7.556478, 7.396181, 7.241481, 7.214029,
           7.151809, 7.101615, 7.020673, 6.842171, 6.690162, 6.566919, 6.451853,
           6.340123, 6.232998, 6.132971, 6.033870, 5.942196, 5.852910, 5.766885,
           5.684641, 5.606282, 5.535598, 5.466105, 5.399402, 5.337311, 5.276068,
           5.217920, 5.167805, 5.117042, 5.070244, 5.027101, 4.991119, 4.954320,
           4.921954, 4.890309, 4.864832, 4.839622, 4.816555, 4.799851, 4.785259,
           4.766347, 4.749851, 4.733206, 4.715694, 4.693037, 4.669675, 4.646818,
           4.624454},
          {12.309409, 11.934985, 11.576079, 11.241654, 10.922738, 10.629198,
           10.352194, 10.089666, 9.840175,  9.599366,  9.360677,  9.127362,
           8.894522,  8.668673,  8.456125,  8.247776,  8.048481,  7.860834,
           7.678633,  7.500480,  7.332388,  7.232142,  7.223988,  7.109046,
           6.969390,  6.826451,  6.690698,  6.556401,  6.427193,  6.309788,
           6.194630,  6.083587,  5.977247,  5.878475,  5.778949,  5.685575,
           5.594967,  5.512121,  5.430759,  5.353733,  5.279559,  5.209142,
           5.143398,  5.086201,  5.034203,  4.980710,  4.918157,  4.859756,
           4.805475,  4.755211}}},
        {{{20.374414, 19.578024, 18.823790, 18.119316, 17.449755, 16.816992,
           16.220382, 15.653053, 15.122837, 14.615683, 14.135446, 13.691329,
           13.270693, 12.873914, 12.496364, 12.149904, 11.823901, 11.516691,
           11.217921, 10.923479, 10.640210, 10.378156, 10.130770, 9.895415,
           9.670343,  9.456963,  9.254198,  9.060538,  8.874370,  8.697471,
           8.527544,  8.365131,  8.210179,  8.061936,  7.924580,  7.788175,
           7.657104,  7.531582,  7.410853,  7.294983,  7.185055,  7.080089,
           6.983981,  6.909539,  6.821122,  6.732039,  6.648114,  6.569540,
           6.499305,  6.431607},
          {20.314161, 19.702157, 19.114995, 18.553991, 18.044959, 17.555300,
           17.086435, 16.661551, 16.249704, 15.850385, 15.470493, 15.093128,
           14.719202, 14.348634, 14.005214, 13.673778, 13.351591, 13.039674,
           12.740960, 12.451150, 12.184339, 11.926251, 11.678581, 11.438863,
           11.202822, 10.967192, 10.731134, 10.501963, 10.279885, 10.057158,
           9.833521,  9.610582,  9.390722,  9.172590,  8.960033,  8.751469,
           8.549364,  8.353224,  8.163787,  7.984700,  7.810630,  7.641976,
           7.480522,  7.326065,  7.179741,  7.040573,  6.940477,  6.835783,
           6.727456,  6.628687}},
         {{19.657515, 18.945890, 18.271541, 17.630407, 17.011078, 16.429992,
           15.864024, 15.331934, 14.820624, 14.314573, 13.861157, 13.465386,
           13.066240, 12.686340, 12.328990, 11.996624, 11.682034, 11.380822,
           11.085453, 10.797130, 10.526498, 10.273340, 10.032565, 9.804687,
           9.587438,  9.380177,  9.182424,  8.995682,  8.816576,  8.646262,
           8.483075,  8.327629,  8.178696,  8.040054,  7.907661,  7.778791,
           7.655518,  7.538092,  7.425999,  7.318270,  7.216925,  7.121466,
           7.037940,  7.034766,  6.988226,  6.914877,  6.841422,  6.775467,
           6.715185,  6.661874},
          {22.197518, 21.474565, 20.798025, 20.148702, 19.534063, 18.961140,
           18.412434, 17.901738, 17.418547, 16.959625, 16.516982, 16.077730,
           15.651190, 15.242042, 14.849768, 14.466646, 14.104330, 13.773088,
           13.464735, 13.149319, 12.846271, 12.551976, 12.277280, 12.016293,
           11.763478, 11.513793, 11.263566, 11.011362, 10.762081, 10.525577,
           10.297852, 10.071614, 9.845874,  9.624835,  9.406208,  9.191332,
           8.983120,  8.780655,  8.585024,  8.395267,  8.213435,  8.040507,
           7.873825,  7.711908,  7.557402,  7.410232,  7.272324,  7.148298,
           7.060930,  7.039121}},
         {{18.181204, 17.565790, 16.974039, 16.416663, 15.874115, 15.366251,
           14.883810, 14.410025, 13.969541, 14.348924, 14.025072, 13.565028,
           13.176099, 12.820225, 12.476073, 12.146628, 11.825432, 11.512027,
           11.222330, 10.950553, 10.689429, 10.442308, 10.207246, 9.993351,
           9.785302,  9.585170,  9.395351,  9.215748,  9.046325,  8.897145,
           8.745686,  8.601668,  8.466918,  8.353342,  8.232742,  8.125543,
           8.020367,  7.929407,  7.840244,  7.754989,  7.678858,  7.610097,
           7.547751,  7.497327,  7.453661,  7.405596,  7.340072,  7.277370,
           7.216925,  7.159046},
          {24.378776, 23.517460, 22.712491, 21.947646, 21.214920, 20.538326,
           19.892552, 19.278340, 18.713314, 18.194261, 17.683639, 17.174775,
           16.698678, 16.234383, 15.784549, 15.359955, 14.960202, 14.568373,
           14.194019, 13.967401, 14.167131, 13.934682, 13.628296, 13.315231,
           13.002592, 12.690747, 12.388487, 12.093467, 11.802837, 11.533456,
           11.259270, 11.004374, 10.762117, 10.543090, 10.320069, 10.110759,
           9.898730,  9.700997,  9.503051,  9.305502,  9.118334,  8.938182,
           8.766658,  8.614174,  8.474190,  8.332299,  8.172815,  8.023913,
           7.885380,  7.755908}}}},
       {{{{2.186199, 2.193139, 2.183936, 2.194919, 2.188356, 2.192296, 2.182882,
           2.180726, 2.187478, 2.185643, 2.184786, 2.175690, 2.174270, 2.173319,
           2.166363, 2.162494, 2.158685, 2.158351, 2.150850, 2.142270, 2.135361,
           2.127835, 2.123211, 2.123250, 2.123267, 2.123267, 2.123267, 2.123267,
           2.123267, 2.123327, 2.123327, 2.123327, 2.123335, 2.123388, 2.123388,
           2.123434, 2.123453, 2.123529, 2.123579, 2.123629, 2.123709, 2.123799,
           2.123914, 2.124103, 2.124301, 2.124519, 2.124884, 2.125389, 2.126043,
           2.126838},
          {2.266303, 2.262784, 2.251689, 2.251068, 2.257689, 2.248801, 2.251105,
           2.252458, 2.244573, 2.241552, 2.253289, 2.232553, 2.245522, 2.227311,
           2.237117, 2.236540, 2.219821, 2.223348, 2.217815, 2.213649, 2.209477,
           2.198373, 2.185864, 2.174703, 2.156518, 2.148883, 2.133132, 2.123267,
           2.123268, 2.123327, 2.123327, 2.123327, 2.123333, 2.123388, 2.123388,
           2.123431, 2.123453, 2.123527, 2.123579, 2.123627, 2.123709, 2.123801,
           2.123915, 2.124103, 2.124300, 2.124520, 2.124884, 2.125390, 2.126040,
           2.126838}},
         {{2.185934, 2.189414, 2.190272, 2.187945, 2.185724, 2.182073, 2.187050,
           2.183356, 2.181351, 2.182454, 2.181389, 2.177857, 2.176293, 2.168205,
           2.166788, 2.163005, 2.164222, 2.163679, 2.154565, 2.150369, 2.143540,
           2.133633, 2.133633, 2.133665, 2.133676, 2.133676, 2.133676, 2.133676,
           2.133676, 2.133722, 2.133722, 2.133722, 2.133727, 2.133769, 2.133769,
           2.133798, 2.133816, 2.133871, 2.133910, 2.133947, 2.134007, 2.134076,
           2.134161, 2.134299, 2.134450, 2.134615, 2.134885, 2.135263, 2.135750,
           2.136344},
          {2.268147, 2.265719, 2.264640, 2.264246, 2.252999, 2.247761, 2.248705,
           2.250243, 2.243390, 2.242175, 2.247214, 2.245048, 2.242665, 2.239908,
           2.235890, 2.226471, 2.228653, 2.223689, 2.212201, 2.211335, 2.198626,
           2.194879, 2.184810, 2.169831, 2.165792, 2.154907, 2.139672, 2.133676,
           2.133677, 2.133722, 2.133722, 2.133722, 2.133727, 2.133769, 2.133769,
           2.133799, 2.133816, 2.133871, 2.133910, 2.133947, 2.134008, 2.134077,
           2.134162, 2.134300, 2.134448, 2.134612, 2.134883, 2.135261, 2.135750,
           2.136344}},
         {{2.194646, 2.198546, 2.200159, 2.192175, 2.191986, 2.196011, 2.195110,
           2.195655, 2.200685, 2.206079, 2.202146, 2.194033, 2.191933, 2.184097,
           2.183846, 2.182112, 2.179994, 2.175949, 2.169842, 2.165412, 2.158125,
           2.150525, 2.150525, 2.150836, 2.150964, 2.150964, 2.150964, 2.150964,
           2.150968, 2.151408, 2.151408, 2.151408, 2.151454, 2.151866, 2.151866,
           2.152198, 2.152339, 2.152905, 2.153308, 2.153656, 2.154265, 2.154945,
           2.155799, 2.157203, 2.158679, 2.160036, 2.160296, 2.160660, 2.161121,
           2.161693},
          {2.270163, 2.262184, 2.257929, 2.262989, 2.261557, 2.251200, 2.256362,
           2.255883, 2.251482, 2.255213, 2.258053, 2.253598, 2.252985, 2.254226,
           2.241064, 2.244546, 2.238599, 2.233263, 2.230435, 2.217918, 2.220409,
           2.215369, 2.202219, 2.191778, 2.180323, 2.169470, 2.157010, 2.150964,
           2.150966, 2.151408, 2.151408, 2.151408, 2.151453, 2.151866, 2.151866,
           2.152196, 2.152339, 2.152903, 2.153308, 2.153653, 2.154263, 2.154945,
           2.155800, 2.157206, 2.158680, 2.160034, 2.160296, 2.160660, 2.161124,
           2.161698}}},
        {{{2.183201, 2.185112, 2.185914, 2.186195, 2.187662, 2.189505, 2.183215,
           2.185464, 2.190325, 2.183436, 2.180370, 2.184865, 2.177161, 2.176087,
           2.186099, 2.180662, 2.179526, 2.171176, 2.177566, 2.173817, 2.165574,
           2.161334, 2.164507, 2.159175, 2.151206, 2.144481, 2.140111, 2.134004,
           2.130476, 2.130539, 2.130539, 2.130539, 2.130541, 2.130600, 2.130600,
           2.130645, 2.130665, 2.130740, 2.130796, 2.130841, 2.130925, 2.131021,
           2.131134, 2.131327, 2.131529, 2.131749, 2.132123, 2.132639, 2.133302,
           2.134117},
          {2.187120, 2.188284, 2.188595, 2.188395, 2.189543, 2.188611, 2.198055,
           2.188171, 2.193478, 2.202846, 2.191430, 2.189497, 2.200691, 2.187449,
           2.186202, 2.188004, 2.187125, 2.185157, 2.183444, 2.182623, 2.178757,
           2.174009, 2.168685, 2.165392, 2.162391, 2.156921, 2.149458, 2.144467,
           2.136607, 2.130539, 2.130539, 2.130539, 2.130540, 2.130600, 2.130600,
           2.130644, 2.130665, 2.130740, 2.130796, 2.130845, 2.130928, 2.131020,
           2.131136, 2.131328, 2.131527, 2.131751, 2.132123, 2.132639, 2.133303,
           2.134118}},
         {{2.184931, 2.182490, 2.183525, 2.183858, 2.184850, 2.184300, 2.181814,
           2.184095, 2.185521, 2.185534, 2.183264, 2.182967, 2.179124, 2.181541,
           2.178680, 2.176603, 2.175789, 2.174231, 2.176669, 2.176456, 2.176745,
           2.172132, 2.166896, 2.164627, 2.161342, 2.154588, 2.150241, 2.143849,
           2.141073, 2.141120, 2.141120, 2.141120, 2.141124, 2.141162, 2.141162,
           2.141195, 2.141209, 2.141268, 2.141308, 2.141345, 2.141405, 2.141474,
           2.141558, 2.141701, 2.141849, 2.142014, 2.142287, 2.142671, 2.143161,
           2.143758},
          {2.187041, 2.189336, 2.186698, 2.187698, 2.190603, 2.190444, 2.193816,
           2.189655, 2.184780, 2.194898, 2.189497, 2.188580, 2.188413, 2.190367,
           2.187142, 2.185600, 2.185029, 2.181539, 2.181337, 2.175560, 2.175387,
           2.175464, 2.174803, 2.172960, 2.168411, 2.162381, 2.154553, 2.150946,
           2.142392, 2.141120, 2.141120, 2.141120, 2.141122, 2.141162, 2.141162,
           2.141198, 2.141209, 2.141270, 2.141308, 2.141343, 2.141406, 2.141472,
           2.141559, 2.141702, 2.141850, 2.142016, 2.142288, 2.142670, 2.143159,
           2.143765}},
         {{2.193565, 2.194761, 2.196471, 2.192929, 2.194616, 2.195221, 2.193113,
           2.196935, 2.204165, 2.205711, 2.200347, 2.200506, 2.194489, 2.196869,
           2.192736, 2.194471, 2.195395, 2.190498, 2.191732, 2.188954, 2.189433,
           2.186008, 2.181275, 2.177729, 2.173468, 2.170180, 2.164907, 2.158482,
           2.158482, 2.158933, 2.158933, 2.158933, 2.158972, 2.159393, 2.159393,
           2.159724, 2.159865, 2.160437, 2.160839, 2.161191, 2.161805, 2.162490,
           2.163351, 2.164767, 2.166252, 2.167614, 2.167871, 2.168231, 2.168689,
           2.169252},
          {2.197095, 2.199735, 2.196542, 2.196774, 2.198553, 2.203306, 2.210903,
           2.211418, 2.215353, 2.221103, 2.214661, 2.211092, 2.212425, 2.207165,
           2.208051, 2.205283, 2.202833, 2.201099, 2.200220, 2.199729, 2.194436,
           2.191272, 2.191837, 2.190085, 2.184993, 2.179086, 2.175283, 2.167799,
           2.163626, 2.158933, 2.158933, 2.158933, 2.158973, 2.159393, 2.159393,
           2.159725, 2.159862, 2.160434, 2.160839, 2.161192, 2.161803, 2.162491,
           2.163350, 2.164764, 2.166250, 2.167615, 2.167870, 2.168231, 2.168688,
           2.169248}}},
        {{{2.165368, 2.165477, 2.166467, 2.167162, 2.166683, 2.167368, 2.167753,
           2.166668, 2.170212, 2.172198, 2.172791, 2.173548, 2.170318, 2.174536,
           2.172716, 2.172357, 2.172716, 2.169465, 2.169151, 2.167871, 2.166942,
           2.161807, 2.164761, 2.163249, 2.161795, 2.155276, 2.155934, 2.152112,
           2.152112, 2.152178, 2.152178, 2.152178, 2.152185, 2.152244, 2.152244,
           2.152289, 2.152311, 2.152391, 2.152452, 2.152502, 2.152590, 2.152688,
           2.152811, 2.153010, 2.153224, 2.153460, 2.153851, 2.154400, 2.155101,
           2.155956},
          {2.200101, 2.199527, 2.199584, 2.198452, 2.200057, 2.198037, 2.200455,
           2.199430, 2.199953, 2.202440, 2.205062, 2.201971, 2.197931, 2.201127,
           2.205470, 2.193674, 2.203450, 2.202744, 2.194610, 2.202348, 2.191019,
           2.198436, 2.197638, 2.195198, 2.189845, 2.192600, 2.185330, 2.180764,
           2.180304, 2.177124, 2.175256, 2.166432, 2.166553, 2.157253, 2.152244,
           2.152293, 2.152311, 2.152394, 2.152452, 2.152500, 2.152589, 2.152687,
           2.152811, 2.153010, 2.153224, 2.153461, 2.153854, 2.154398, 2.155099,
           2.155963}},
         {{2.174948, 2.175840, 2.175736, 2.177392, 2.176954, 2.174237, 2.175797,
           2.178880, 2.179673, 2.178529, 2.180522, 2.184315, 2.181427, 2.179967,
           2.179783, 2.182306, 2.178730, 2.179108, 2.179952, 2.175691, 2.175681,
           2.174463, 2.171770, 2.173440, 2.168688, 2.167719, 2.163275, 2.163275,
           2.163275, 2.163318, 2.163318, 2.163318, 2.163323, 2.163365, 2.163365,
           2.163400, 2.163413, 2.163474, 2.163513, 2.163552, 2.163612, 2.163682,
           2.163769, 2.163913, 2.164068, 2.164234, 2.164515, 2.164903, 2.165403,
           2.166016},
          {2.207767, 2.205751, 2.207534, 2.207010, 2.207529, 2.207765, 2.207365,
           2.207939, 2.208977, 2.209494, 2.210444, 2.210110, 2.209588, 2.208138,
           2.208384, 2.209801, 2.211737, 2.208969, 2.205928, 2.209166, 2.208317,
           2.205325, 2.204082, 2.207746, 2.200194, 2.195670, 2.196512, 2.201849,
           2.191277, 2.195585, 2.182579, 2.180974, 2.179659, 2.170753, 2.173316,
           2.163401, 2.163413, 2.163475, 2.163513, 2.163551, 2.163613, 2.163683,
           2.163771, 2.163915, 2.164065, 2.164237, 2.164515, 2.164905, 2.165406,
           2.166014}},
         {{2.183700, 2.184557, 2.184151, 2.183441, 2.182673, 2.181754, 2.184959,
           2.190677, 2.195779, 2.200789, 2.199856, 2.197497, 2.200566, 2.198608,
           2.196300, 2.200088, 2.196058, 2.196538, 2.194132, 2.196573, 2.195220,
           2.192229, 2.193779, 2.190748, 2.191020, 2.187514, 2.187061, 2.181048,
           2.181050, 2.181507, 2.181507, 2.181507, 2.181549, 2.181977, 2.181977,
           2.182316, 2.182458, 2.183043, 2.183453, 2.183813, 2.184441, 2.185140,
           2.186019, 2.187464, 2.188980, 2.190372, 2.190616, 2.190957, 2.191396,
           2.191935},
          {2.214798, 2.212706, 2.214425, 2.213428, 2.208946, 2.215648, 2.211451,
           2.220133, 2.224531, 2.229612, 2.222880, 2.221177, 2.217119, 2.220195,
           2.229363, 2.225449, 2.220132, 2.224421, 2.222477, 2.223240, 2.222276,
           2.221139, 2.219822, 2.214171, 2.216926, 2.219538, 2.212793, 2.210366,
           2.205042, 2.205721, 2.201904, 2.199050, 2.191585, 2.190996, 2.186784,
           2.182317, 2.182456, 2.183045, 2.183453, 2.183815, 2.184442, 2.185140,
           2.186017, 2.187464, 2.188981, 2.190372, 2.190615, 2.190956, 2.191395,
           2.191929}}},
        {{{2.209600, 2.208396, 2.210652, 2.209253, 2.207240, 2.209000, 2.207820,
           2.207536, 2.210434, 2.212272, 2.212265, 2.213841, 2.215181, 2.214482,
           2.210699, 2.214060, 2.213599, 2.219686, 2.212965, 2.224804, 2.221249,
           2.210834, 2.210104, 2.210140, 2.219534, 2.212276, 2.205956, 2.203670,
           2.215702, 2.199753, 2.198207, 2.198864, 2.197410, 2.190359, 2.188311,
           2.188364, 2.188388, 2.188477, 2.188537, 2.188594, 2.188689, 2.188795,
           2.188928, 2.189150, 2.189382, 2.189641, 2.190068, 2.190659, 2.191425,
           2.192365},
          {2.306924, 2.297222, 2.289295, 2.283098, 2.278620, 2.275912, 2.275049,
           2.278024, 2.275710, 2.270629, 2.276548, 2.274133, 2.272745, 2.271071,
           2.269254, 2.266517, 2.269627, 2.268807, 2.270094, 2.269844, 2.274265,
           2.272629, 2.271211, 2.264835, 2.261679, 2.268816, 2.263327, 2.260764,
           2.262441, 2.259565, 2.259216, 2.257389, 2.248190, 2.252268, 2.243966,
           2.240583, 2.241579, 2.239470, 2.228155, 2.226672, 2.225005, 2.221163,
           2.217429, 2.215705, 2.214267, 2.213296, 2.208258, 2.203859, 2.204938,
           2.190900}},
         {{2.218782, 2.217831, 2.217181, 2.215800, 2.217946, 2.220416, 2.219315,
           2.216619, 2.216107, 2.218010, 2.220899, 2.221544, 2.216973, 2.219293,
           2.224001, 2.225605, 2.226204, 2.230961, 2.224951, 2.225610, 2.224960,
           2.222464, 2.222354, 2.222649, 2.222619, 2.223193, 2.216217, 2.219030,
           2.212226, 2.213639, 2.214627, 2.203166, 2.205530, 2.200367, 2.200367,
           2.200405, 2.200420, 2.200480, 2.200521, 2.200559, 2.200623, 2.200697,
           2.200787, 2.200934, 2.201091, 2.201268, 2.201558, 2.201958, 2.202474,
           2.203112},
          {2.303771, 2.294418, 2.286811, 2.280936, 2.276808, 2.274480, 2.272474,
           2.277170, 2.270682, 2.275208, 2.268007, 2.277247, 2.272648, 2.275051,
           2.268288, 2.274352, 2.271339, 2.271436, 2.270559, 2.268369, 2.272441,
           2.279953, 2.266222, 2.274667, 2.271347, 2.271713, 2.264464, 2.265737,
           2.265342, 2.267163, 2.265217, 2.265883, 2.265881, 2.259395, 2.260835,
           2.263305, 2.252759, 2.246240, 2.242876, 2.247625, 2.235586, 2.243264,
           2.236712, 2.226973, 2.230629, 2.227184, 2.227374, 2.224609, 2.225279,
           2.216686}},
         {{2.228264, 2.226658, 2.226337, 2.225132, 2.226516, 2.227795, 2.230981,
           2.232740, 2.236280, 2.239592, 2.240710, 2.240403, 2.236761, 2.235380,
           2.234784, 2.240505, 2.242874, 2.241731, 2.243740, 2.245535, 2.241018,
           2.239822, 2.243208, 2.237952, 2.244752, 2.239323, 2.235749, 2.233246,
           2.234445, 2.233063, 2.231509, 2.226631, 2.227692, 2.228743, 2.222286,
           2.219965, 2.220112, 2.220717, 2.221145, 2.221515, 2.222163, 2.222885,
           2.223793, 2.225289, 2.226859, 2.228295, 2.228519, 2.228833, 2.229238,
           2.229734},
          {2.312814, 2.303384, 2.295772, 2.290324, 2.286924, 2.285965, 2.286129,
           2.293075, 2.293627, 2.294328, 2.296689, 2.297997, 2.296746, 2.295317,
           2.290494, 2.291990, 2.293126, 2.288901, 2.291281, 2.303288, 2.298767,
           2.300272, 2.293404, 2.302857, 2.301453, 2.302645, 2.293985, 2.294359,
           2.289679, 2.296638, 2.290773, 2.287069, 2.278533, 2.284367, 2.279462,
           2.270430, 2.277054, 2.271706, 2.272773, 2.275295, 2.268594, 2.262213,
           2.254277, 2.251212, 2.258799, 2.249113, 2.249490, 2.255167, 2.248941,
           2.242837}}},
        {{{2.354823, 2.345089, 2.334061, 2.323071, 2.314583, 2.307107, 2.301060,
           2.296677, 2.294080, 2.292673, 2.296067, 2.293791, 2.292355, 2.295343,
           2.294802, 2.293979, 2.296744, 2.293761, 2.292505, 2.292900, 2.296556,
           2.297634, 2.293879, 2.293982, 2.298331, 2.297163, 2.308332, 2.312439,
           2.309665, 2.305985, 2.305413, 2.303411, 2.307690, 2.301760, 2.302571,
           2.301571, 2.298011, 2.298637, 2.294963, 2.296443, 2.296042, 2.289629,
           2.289120, 2.287639, 2.285929, 2.287306, 2.283461, 2.282573, 2.277898,
           2.259987},
          {2.556627, 2.535965, 2.514783, 2.502316, 2.484255, 2.462146, 2.440978,
           2.422675, 2.407130, 2.394255, 2.383965, 2.377251, 2.372083, 2.367022,
           2.370781, 2.365026, 2.360636, 2.363381, 2.356634, 2.353569, 2.346137,
           2.344596, 2.336069, 2.342255, 2.336543, 2.332678, 2.329542, 2.325602,
           2.317160, 2.314699, 2.307600, 2.311076, 2.308059, 2.306394, 2.296807,
           2.305602, 2.303024, 2.302604, 2.299384, 2.295104, 2.296395, 2.295958,
           2.291431, 2.288401, 2.281110, 2.277825, 2.268301, 2.265680, 2.259684,
           2.256677}},
         {{2.348128, 2.338042, 2.329412, 2.323437, 2.323761, 2.319037, 2.313865,
           2.309808, 2.307206, 2.305821, 2.303869, 2.303776, 2.305612, 2.303705,
           2.302725, 2.305025, 2.303960, 2.306360, 2.309845, 2.308454, 2.306496,
           2.309601, 2.313829, 2.313761, 2.313210, 2.314745, 2.312669, 2.315542,
           2.314153, 2.321059, 2.306667, 2.309746, 2.318115, 2.313904, 2.302700,
           2.300097, 2.305378, 2.309305, 2.303091, 2.297122, 2.295789, 2.295605,
           2.293332, 2.292625, 2.296929, 2.298420, 2.290925, 2.297957, 2.294960,
           2.291253},
          {2.551511, 2.532209, 2.512382, 2.500961, 2.484136, 2.463445, 2.442749,
           2.424845, 2.409964, 2.397518, 2.386676, 2.379428, 2.374414, 2.369900,
           2.374597, 2.369731, 2.365270, 2.368379, 2.363420, 2.365437, 2.361688,
           2.351227, 2.350523, 2.351045, 2.349296, 2.344709, 2.340246, 2.338156,
           2.344370, 2.335919, 2.331616, 2.334761, 2.332584, 2.322635, 2.324129,
           2.326325, 2.322507, 2.322059, 2.323783, 2.324586, 2.319916, 2.322332,
           2.321203, 2.316322, 2.298868, 2.296722, 2.286017, 2.288136, 2.284536,
           2.287369}},
         {{2.359003, 2.350433, 2.338963, 2.333293, 2.328562, 2.323221, 2.319431,
           2.318178, 2.320097, 2.321759, 2.319711, 2.318126, 2.317089, 2.320249,
           2.319293, 2.318177, 2.320967, 2.319236, 2.325258, 2.323788, 2.320881,
           2.322369, 2.320510, 2.327476, 2.324944, 2.326361, 2.327412, 2.325551,
           2.333480, 2.326128, 2.326805, 2.325012, 2.325912, 2.324660, 2.325173,
           2.320000, 2.321629, 2.319814, 2.318257, 2.320350, 2.318130, 2.320537,
           2.317354, 2.323404, 2.321679, 2.320705, 2.315083, 2.313930, 2.314221,
           2.302759},
          {2.546552, 2.528753, 2.513039, 2.509179, 2.496395, 2.479099, 2.461471,
           2.447421, 2.438806, 2.430326, 2.414711, 2.404058, 2.399032, 2.395338,
           2.391604, 2.396495, 2.392135, 2.395982, 2.391979, 2.389023, 2.393022,
           2.385642, 2.388461, 2.388000, 2.386079, 2.381728, 2.379427, 2.366966,
           2.379271, 2.380803, 2.360679, 2.368480, 2.373716, 2.357217, 2.367141,
           2.366757, 2.363997, 2.361166, 2.362913, 2.352257, 2.350849, 2.360306,
           2.346369, 2.339740, 2.343113, 2.335853, 2.328398, 2.326430, 2.314782,
           2.306303}}},
        {{{2.579857, 2.573102, 2.570268, 2.563292, 2.556156, 2.548825, 2.541385,
           2.533793, 2.526067, 2.519555, 2.515876, 2.508840, 2.501073, 2.493192,
           2.485159, 2.476872, 2.468323, 2.459418, 2.452420, 2.454201, 2.450677,
           2.445703, 2.447501, 2.443023, 2.443537, 2.439113, 2.437765, 2.435192,
           2.433698, 2.432806, 2.430426, 2.428639, 2.425919, 2.423717, 2.420841,
           2.418204, 2.415225, 2.412032, 2.408526, 2.404755, 2.400630, 2.396498,
           2.391977, 2.387464, 2.382655, 2.378005, 2.373469, 2.369172, 2.365293,
           2.362245},
          {2.973035, 2.956479, 2.927234, 2.905531, 2.879876, 2.853296, 2.832820,
           2.807174, 2.784671, 2.762452, 2.740581, 2.720838, 2.698749, 2.679104,
           2.659878, 2.640962, 2.623525, 2.607259, 2.590749, 2.574872, 2.560737,
           2.547246, 2.533709, 2.521347, 2.509741, 2.498482, 2.488312, 2.478161,
           2.468861, 2.460206, 2.451880, 2.445164, 2.437687, 2.432758, 2.426815,
           2.420591, 2.414488, 2.408533, 2.402651, 2.396363, 2.390844, 2.383990,
           2.379067, 2.372827, 2.367715, 2.362225, 2.357195, 2.352532, 2.348332,
           2.344923}},
         {{2.579104, 2.573358, 2.566496, 2.559483, 2.552319, 2.545538, 2.542825,
           2.535884, 2.529072, 2.521714, 2.513144, 2.504808, 2.496753, 2.488598,
           2.480251, 2.471650, 2.462732, 2.454003, 2.448292, 2.452103, 2.446677,
           2.447984, 2.445047, 2.445552, 2.440822, 2.438211, 2.437558, 2.436223,
           2.441112, 2.442047, 2.441391, 2.439813, 2.438193, 2.435812, 2.433914,
           2.431551, 2.429197, 2.426261, 2.423204, 2.419974, 2.416362, 2.412805,
           2.408849, 2.405040, 2.401216, 2.397400, 2.394004, 2.390888, 2.388452,
           2.386206},
          {2.965602, 2.949101, 2.920858, 2.900099, 2.875459, 2.849904, 2.830600,
           2.806093, 2.784802, 2.763812, 2.742739, 2.724630, 2.703944, 2.684836,
           2.668320, 2.650015, 2.634397, 2.618938, 2.603541, 2.589196, 2.575362,
           2.562662, 2.550163, 2.538628, 2.527394, 2.516865, 2.507234, 2.497807,
           2.488629, 2.480553, 2.472312, 2.465014, 2.457746, 2.452045, 2.445666,
           2.439247, 2.442086, 2.440140, 2.435486, 2.429876, 2.424637, 2.418387,
           2.412783, 2.406294, 2.400601, 2.394549, 2.388850, 2.383201, 2.378322,
           2.373995}},
         {{2.586321, 2.578973, 2.573937, 2.568762, 2.562052, 2.556353, 2.551616,
           2.549213, 2.550077, 2.547878, 2.534468, 2.526731, 2.520308, 2.512414,
           2.504405, 2.496207, 2.487789, 2.480110, 2.474750, 2.468685, 2.472773,
           2.468001, 2.469797, 2.465617, 2.464864, 2.464722, 2.464680, 2.464082,
           2.461847, 2.461962, 2.459800, 2.458541, 2.456417, 2.455747, 2.453296,
           2.452182, 2.449703, 2.448130, 2.446371, 2.444402, 2.442452, 2.440649,
           2.438678, 2.437848, 2.436739, 2.435534, 2.433077, 2.430733, 2.428836,
           2.426891},
          {2.956800, 2.941043, 2.914026, 2.894378, 2.870874, 2.846551, 2.828463,
           2.805623, 2.787444, 2.765701, 2.744828, 2.729321, 2.709670, 2.693029,
           2.692898, 2.684246, 2.670689, 2.657492, 2.644238, 2.630938, 2.618764,
           2.606949, 2.595328, 2.584781, 2.574762, 2.564862, 2.555584, 2.546679,
           2.538527, 2.531013, 2.523523, 2.516086, 2.509377, 2.503224, 2.496903,
           2.491218, 2.485279, 2.480934, 2.475477, 2.469531, 2.467109, 2.461570,
           2.456264, 2.451109, 2.445710, 2.440232, 2.433754, 2.428199, 2.423333,
           2.418856}}},
        {{{2.823365, 2.812559, 2.800678, 2.791067, 2.780563, 2.770386, 2.760504,
           2.750463, 2.740588, 2.730921, 2.721325, 2.711843, 2.702894, 2.693840,
           2.684987, 2.676443, 2.667990, 2.660174, 2.652320, 2.644654, 2.637437,
           2.630241, 2.623242, 2.616464, 2.610230, 2.604010, 2.598039, 2.592236,
           2.587033, 2.581789, 2.576783, 2.571907, 2.569786, 2.565918, 2.561677,
           2.557329, 2.552932, 2.548899, 2.545109, 2.539936, 2.534652, 2.529289,
           2.523632, 2.517955, 2.512233, 2.506323, 2.500689, 2.495599, 2.490838,
           2.486498},
          {3.443479, 3.392964, 3.344886, 3.297726, 3.254369, 3.211130, 3.169008,
           3.129790, 3.091923, 3.056687, 3.021955, 2.987360, 2.956445, 2.926017,
           2.896681, 2.869705, 2.843983, 2.820551, 2.797461, 2.775644, 2.755914,
           2.736679, 2.718565, 2.701791, 2.685948, 2.670781, 2.656509, 2.643461,
           2.630793, 2.618871, 2.607613, 2.597287, 2.587187, 2.577645, 2.570339,
           2.563415, 2.555394, 2.548288, 2.541190, 2.533348, 2.525945, 2.518424,
           2.510952, 2.503649, 2.496518, 2.489289, 2.483076, 2.477175, 2.471831,
           2.467368}},
         {{2.818565, 2.807093, 2.797027, 2.786936, 2.776948, 2.767089, 2.757359,
           2.748245, 2.739164, 2.729896, 2.719275, 2.709729, 2.700477, 2.691437,
           2.682746, 2.674126, 2.666034, 2.657982, 2.650186, 2.642813, 2.635466,
           2.628347, 2.621473, 2.615134, 2.608795, 2.602738, 2.596878, 2.591559,
           2.586286, 2.581321, 2.576473, 2.576792, 2.581141, 2.578810, 2.575670,
           2.572153, 2.568541, 2.565208, 2.561940, 2.557534, 2.553101, 2.548426,
           2.543432, 2.538747, 2.533915, 2.529065, 2.524853, 2.520929, 2.517570,
           2.514978},
          {3.448078, 3.396048, 3.348399, 3.302237, 3.258826, 3.215906, 3.174341,
           3.135345, 3.097841, 3.064180, 3.029953, 2.996810, 2.966809, 2.937206,
           2.908897, 2.882836, 2.859471, 2.836467, 2.815032, 2.794837, 2.775117,
           2.756623, 2.739210, 2.723583, 2.708174, 2.693678, 2.679915, 2.667576,
           2.655394, 2.644032, 2.633013, 2.623094, 2.613229, 2.603942, 2.594815,
           2.586620, 2.578242, 2.580801, 2.578147, 2.571837, 2.565345, 2.558550,
           2.551176, 2.543611, 2.535866, 2.528440, 2.521032, 2.514167, 2.507971,
           2.502212}},
         {{2.826913, 2.815570, 2.804362, 2.794918, 2.785405, 2.777145, 2.770613,
           2.766823, 2.767690, 2.764890, 2.747812, 2.733875, 2.724452, 2.715483,
           2.706755, 2.698214, 2.690166, 2.682173, 2.674933, 2.667753, 2.660625,
           2.653845, 2.647218, 2.641542, 2.635978, 2.630285, 2.624912, 2.619832,
           2.615158, 2.611493, 2.608434, 2.607513, 2.604532, 2.601807, 2.598536,
           2.595592, 2.592837, 2.590430, 2.588789, 2.586503, 2.584045, 2.581570,
           2.579594, 2.578090, 2.576727, 2.575138, 2.572105, 2.568968, 2.566038,
           2.563212},
          {3.453300, 3.401005, 3.350615, 3.303561, 3.258230, 3.217858, 3.177333,
           3.140093, 3.104048, 3.069943, 3.037518, 3.007150, 2.977049, 2.948582,
           2.922511, 2.897005, 2.892049, 2.880395, 2.864322, 2.845968, 2.827879,
           2.810389, 2.794841, 2.780032, 2.765670, 2.751956, 2.739391, 2.727087,
           2.715479, 2.705453, 2.695086, 2.684965, 2.675320, 2.667063, 2.658237,
           2.650271, 2.642171, 2.635142, 2.627784, 2.620332, 2.613904, 2.609369,
           2.602218, 2.595493, 2.590236, 2.583006, 2.575310, 2.567536, 2.560535,
           2.554266}}},
        {{{3.163117, 3.144541, 3.126677, 3.109431, 3.092878, 3.076859, 3.061464,
           3.046232, 3.031354, 3.016681, 3.002399, 2.989004, 2.976226, 2.962824,
           2.949841, 2.937195, 2.924955, 2.913076, 2.901610, 2.890520, 2.879799,
           2.869382, 2.859360, 2.849761, 2.840597, 2.831847, 2.823534, 2.815626,
           2.808063, 2.800887, 2.794023, 2.787452, 2.781269, 2.778268, 2.772828,
           2.767260, 2.761557, 2.755743, 2.749649, 2.743287, 2.736724, 2.729902,
           2.722887, 2.715731, 2.708408, 2.701031, 2.695320, 2.688681, 2.682603,
           2.677138},
          {3.920574, 3.854868, 3.791554, 3.731952, 3.672589, 3.615500, 3.560871,
           3.508919, 3.461599, 3.415649, 3.372205, 3.330793, 3.291540, 3.254322,
           3.220387, 3.186466, 3.154284, 3.123726, 3.094679, 3.067110, 3.040904,
           3.016025, 2.992489, 2.971468, 2.950108, 2.929872, 2.910701, 2.892560,
           2.875414, 2.859206, 2.843957, 2.829516, 2.815857, 2.802897, 2.790583,
           2.780329, 2.771680, 2.761400, 2.751221, 2.741130, 2.731228, 2.721453,
           2.711791, 2.702411, 2.694533, 2.685534, 2.677169, 2.669557, 2.662749,
           2.656770}},
         {{3.156051, 3.138125, 3.120732, 3.103968, 3.087782, 3.072284, 3.057413,
           3.042979, 3.029170, 3.014950, 2.999488, 2.986278, 2.972631, 2.959272,
           2.946342, 2.933741, 2.921559, 2.909736, 2.898425, 2.887407, 2.876796,
           2.866482, 2.856638, 2.847306, 2.838397, 2.829869, 2.821807, 2.814096,
           2.806819, 2.799999, 2.793397, 2.787112, 2.789056, 2.792520, 2.788955,
           2.784623, 2.779910, 2.774995, 2.769823, 2.764381, 2.758726, 2.752860,
           2.746809, 2.740762, 2.734694, 2.728730, 2.723177, 2.718536, 2.715255,
           2.711636},
          {3.950857, 3.883671, 3.818721, 3.757926, 3.698195, 3.640388, 3.585126,
           3.532722, 3.483982, 3.438384, 3.394590, 3.353224, 3.313920, 3.276838,
           3.242399, 3.209925, 3.178804, 3.150010, 3.121788, 3.094913, 3.069356,
           3.045066, 3.022040, 3.000292, 2.981099, 2.961379, 2.942731, 2.925050,
           2.908286, 2.892562, 2.877441, 2.862877, 2.849066, 2.836122, 2.823723,
           2.811935, 2.800616, 2.789780, 2.793390, 2.788341, 2.779857, 2.770680,
           2.761124, 2.751487, 2.741820, 2.732184, 2.722889, 2.715439, 2.707181,
           2.699602}},
         {{3.162679, 3.145367, 3.128160, 3.111783, 3.096360, 3.083072, 3.071825,
           3.064560, 3.062660, 3.056724, 3.034721, 3.015079, 3.000778, 2.987548,
           2.974618, 2.962097, 2.949945, 2.938205, 2.927583, 2.916985, 2.906470,
           2.896496, 2.887014, 2.878826, 2.870661, 2.862691, 2.855112, 2.848002,
           2.841304, 2.836046, 2.830150, 2.826276, 2.824831, 2.821365, 2.816979,
           2.813319, 2.809604, 2.806679, 2.803499, 2.800177, 2.797170, 2.794365,
           2.791812, 2.790063, 2.788320, 2.786289, 2.782375, 2.778451, 2.774599,
           2.770917},
          {3.985659, 3.916384, 3.849951, 3.786424, 3.727083, 3.668473, 3.612610,
           3.559569, 3.509470, 3.464201, 3.419947, 3.378166, 3.338889, 3.301961,
           3.267087, 3.235480, 3.204464, 3.192319, 3.184363, 3.162377, 3.139215,
           3.116439, 3.094517, 3.074393, 3.054749, 3.036302, 3.019173, 3.001910,
           2.985596, 2.971130, 2.956048, 2.941570, 2.927720, 2.915352, 2.902546,
           2.891116, 2.879740, 2.869854, 2.859847, 2.849903, 2.840690, 2.833335,
           2.825799, 2.817430, 2.808911, 2.800150, 2.789911, 2.780216, 2.771100,
           2.762620}}},
        {{{3.921286, 3.890308, 3.859252, 3.829443, 3.800351, 3.772466, 3.745525,
           3.719598, 3.694018, 3.669256, 3.645293, 3.622534, 3.601759, 3.580102,
           3.559142, 3.538845, 3.519179, 3.500066, 3.481613, 3.463658, 3.446315,
           3.429382, 3.413018, 3.397053, 3.381639, 3.366668, 3.352282, 3.338356,
           3.324956, 3.313625, 3.301131, 3.289065, 3.277346, 3.266004, 3.254914,
           3.244099, 3.238037, 3.228529, 3.218226, 3.207623, 3.196832, 3.185860,
           3.174730, 3.163561, 3.152280, 3.140977, 3.130166, 3.119983, 3.110540,
           3.101952},
          {4.997916, 4.904506, 4.812706, 4.721931, 4.633067, 4.549591, 4.467970,
           4.391549, 4.320678, 4.254774, 4.194552, 4.134948, 4.078389, 4.024831,
           3.973824, 3.925383, 3.879401, 3.833861, 3.789841, 3.747796, 3.707612,
           3.669584, 3.634055, 3.602635, 3.571274, 3.541606, 3.513519, 3.486847,
           3.461366, 3.437134, 3.413594, 3.390380, 3.367709, 3.345665, 3.324258,
           3.304873, 3.284564, 3.264870, 3.245733, 3.232874, 3.215882, 3.198862,
           3.182137, 3.166061, 3.150441, 3.135354, 3.121238, 3.108266, 3.096459,
           3.085859}},
         {{3.909499, 3.878550, 3.848641, 3.819697, 3.791446, 3.764333, 3.738278,
           3.713428, 3.689437, 3.665463, 3.639912, 3.617341, 3.595518, 3.574093,
           3.553271, 3.533122, 3.513523, 3.494616, 3.476339, 3.458541, 3.441304,
           3.424545, 3.408271, 3.392652, 3.377485, 3.362772, 3.348628, 3.334995,
           3.322862, 3.310983, 3.298854, 3.287121, 3.275865, 3.265085, 3.254505,
           3.247563, 3.257823, 3.251534, 3.243456, 3.234687, 3.225565, 3.216187,
           3.206645, 3.197172, 3.187758, 3.178537, 3.169921, 3.162129, 3.155329,
           3.149504},
          {5.102890, 5.002483, 4.906627, 4.813103, 4.720892, 4.631210, 4.547308,
           4.465893, 4.390294, 4.320362, 4.254409, 4.194079, 4.134950, 4.079197,
           4.026584, 3.976576, 3.928950, 3.884487, 3.841850, 3.800732, 3.760375,
           3.721626, 3.684712, 3.650507, 3.619575, 3.590193, 3.561772, 3.534909,
           3.509369, 3.485301, 3.461606, 3.438439, 3.415808, 3.393866, 3.372430,
           3.351759, 3.331614, 3.313737, 3.294743, 3.276148, 3.258066, 3.256322,
           3.250483, 3.236003, 3.220736, 3.205365, 3.190440, 3.176239, 3.162858,
           3.150317}},
         {{3.884251, 3.857210, 3.848123, 3.824260, 3.798752, 3.775439, 3.755106,
           3.740746, 3.734897, 3.723386, 3.688063, 3.657557, 3.634727, 3.613434,
           3.592802, 3.572728, 3.553307, 3.534395, 3.517196, 3.499907, 3.482721,
           3.466089, 3.450098, 3.435774, 3.421397, 3.407202, 3.393551, 3.381663,
           3.369551, 3.359153, 3.347687, 3.336900, 3.326809, 3.318490, 3.309186,
           3.306167, 3.301292, 3.295842, 3.289958, 3.283987, 3.278857, 3.274073,
           3.269785, 3.266711, 3.263738, 3.260214, 3.254095, 3.248030, 3.242104,
           3.236411},
          {5.226453, 5.118365, 5.015019, 4.915971, 4.821157, 4.727581, 4.636998,
           4.552834, 4.471887, 4.396743, 4.326193, 4.259736, 4.199509, 4.140856,
           4.085903, 4.034129, 3.985008, 3.938201, 3.895429, 3.860985, 3.863873,
           3.832305, 3.797934, 3.764873, 3.732753, 3.701990, 3.674887, 3.647514,
           3.621751, 3.598670, 3.574481, 3.550936, 3.528120, 3.507371, 3.485702,
           3.465801, 3.445899, 3.427969, 3.410026, 3.393071, 3.377311, 3.361138,
           3.345705, 3.332329, 3.320605, 3.306442, 3.289889, 3.274179, 3.259398,
           3.245499}}},
        {{{4.962738, 4.911516, 4.862862, 4.816542, 4.774704, 4.732318, 4.691743,
           4.651907, 4.612457, 4.573150, 4.534150, 4.495505, 4.458022, 4.422000,
           4.385536, 4.350769, 4.317279, 4.285145, 4.254209, 4.224593, 4.195948,
           4.168389, 4.141739, 4.116014, 4.091034, 4.068872, 4.045353, 4.022631,
           4.000572, 3.979342, 3.958762, 3.938853, 3.919521, 3.900698, 3.882303,
           3.864418, 3.846888, 3.829361, 3.814735, 3.802505, 3.786102, 3.769368,
           3.753813, 3.737478, 3.720572, 3.703765, 3.687748, 3.672635, 3.658567,
           3.645720},
          {6.458068, 6.320339, 6.188077, 6.064575, 5.945797, 5.829021, 5.716211,
           5.607044, 5.505021, 5.405293, 5.311122, 5.222862, 5.139986, 5.060512,
           4.985484, 4.915330, 4.848764, 4.787405, 4.727950, 4.670390, 4.614866,
           4.560817, 4.508486, 4.458088, 4.410299, 4.363892, 4.320130, 4.278883,
           4.240037, 4.203136, 4.167063, 4.131514, 4.096540, 4.064215, 4.030409,
           3.997341, 3.965113, 3.933638, 3.902982, 3.873042, 3.843948, 3.816374,
           3.797652, 3.772572, 3.749715, 3.725683, 3.703191, 3.682467, 3.663582,
           3.646538}},
         {{4.942719, 4.893389, 4.846335, 4.801979, 4.760398, 4.719042, 4.679802,
           4.641559, 4.604176, 4.565428, 4.523921, 4.483796, 4.447468, 4.410241,
           4.374427, 4.339952, 4.306966, 4.275239, 4.244907, 4.215482, 4.187264,
           4.160026, 4.133691, 4.108410, 4.084717, 4.061917, 4.038733, 4.016424,
           3.994784, 3.974118, 3.953956, 3.934506, 3.915664, 3.897652, 3.879896,
           3.862872, 3.846114, 3.829877, 3.836845, 3.834101, 3.821499, 3.807778,
           3.794416, 3.781170, 3.767019, 3.753169, 3.740178, 3.728367, 3.717944,
           3.708867},
          {6.719470, 6.557362, 6.407608, 6.265928, 6.132738, 6.009502, 5.888915,
           5.772418, 5.661965, 5.557465, 5.453663, 5.353425, 5.260024, 5.175673,
           5.093554, 5.016843, 4.944904, 4.877534, 4.813698, 4.755779, 4.701449,
           4.647306, 4.594338, 4.542877, 4.492869, 4.446375, 4.399870, 4.356248,
           4.315201, 4.276713, 4.238921, 4.201942, 4.165944, 4.131088, 4.097012,
           4.065969, 4.033600, 4.002300, 3.971828, 3.942040, 3.913143, 3.884982,
           3.857467, 3.830875, 3.838829, 3.819278, 3.798031, 3.777901, 3.757338,
           3.737968}},
         {{4.905578, 4.856044, 4.808839, 4.765264, 4.734909, 4.721336, 4.695187,
           4.673796, 4.662396, 4.642422, 4.584887, 4.536683, 4.497529, 4.461148,
           4.425987, 4.392338, 4.359838, 4.328594, 4.300021, 4.271740, 4.243780,
           4.216847, 4.190897, 4.168689, 4.145713, 4.122295, 4.099695, 4.077860,
           4.056931, 4.038917, 4.019507, 4.000903, 3.983459, 3.968542, 3.952392,
           3.938621, 3.925169, 3.914701, 3.910174, 3.902364, 3.894006, 3.886113,
           3.878999, 3.873760, 3.868983, 3.864791, 3.855486, 3.846375, 3.837524,
           3.828987},
          {7.045955, 6.859033, 6.681398, 6.514144, 6.362647, 6.219544, 6.086681,
           5.964229, 5.844928, 5.731663, 5.619559, 5.513093, 5.407057, 5.308799,
           5.219193, 5.135758, 5.056739, 4.982698, 4.913903, 4.849119, 4.789637,
           4.738965, 4.749000, 4.712419, 4.667436, 4.620950, 4.574449, 4.531321,
           4.487451, 4.448304, 4.407690, 4.368329, 4.330411, 4.295761, 4.260241,
           4.227544, 4.195061, 4.167741, 4.138267, 4.109421, 4.082579, 4.056471,
           4.031599, 4.009952, 3.989125, 3.968361, 3.942823, 3.918678, 3.895992,
           3.875221}}},
        {{{6.959991, 6.859443, 6.767117, 6.677979, 6.592335, 6.511007, 6.432474,
           6.355941, 6.284704, 6.211471, 6.140073, 6.070577, 6.002570, 5.936151,
           5.871456, 5.809893, 5.751490, 5.692013, 5.634306, 5.578009, 5.523386,
           5.470082, 5.418440, 5.368143, 5.319456, 5.275219, 5.230181, 5.186949,
           5.145727, 5.106064, 5.068014, 5.031395, 4.995973, 4.961510, 4.928202,
           4.895707, 4.864066, 4.834119, 4.803738, 4.772639, 4.744016, 4.722327,
           4.694228, 4.666287, 4.638602, 4.611445, 4.585622, 4.561207, 4.538388,
           4.517438},
          {9.343450, 9.094670, 8.859876, 8.640117, 8.432293, 8.234292, 8.050783,
           7.879535, 7.715650, 7.561117, 7.412740, 7.268921, 7.126249, 6.990939,
           6.862896, 6.744167, 6.627993, 6.516597, 6.412086, 6.316010, 6.223692,
           6.134948, 6.050249, 5.969244, 5.891852, 5.818292, 5.750842, 5.682709,
           5.617301, 5.551794, 5.486488, 5.421482, 5.357116, 5.295571, 5.235192,
           5.175505, 5.117901, 5.062306, 5.008479, 4.956322, 4.905815, 4.856783,
           4.810556, 4.762221, 4.727253, 4.684446, 4.642596, 4.603318, 4.566962,
           4.533682}},
         {{6.929890, 6.831022, 6.740907, 6.651874, 6.566866, 6.486182, 6.407998,
           6.332551, 6.261740, 6.191711, 6.117821, 6.046788, 5.978922, 5.912951,
           5.848845, 5.789546, 5.729317, 5.670558, 5.613422, 5.557558, 5.503254,
           5.450605, 5.399345, 5.349886, 5.303467, 5.258516, 5.214377, 5.172141,
           5.131596, 5.093116, 5.055836, 5.019875, 4.985351, 4.952128, 4.919668,
           4.888532, 4.858028, 4.830848, 4.801562, 4.772765, 4.764155, 4.762350,
           4.740556, 4.717754, 4.694856, 4.672437, 4.651295, 4.631995, 4.614844,
           4.599734},
          {9.998226, 9.700943, 9.423200, 9.163590, 8.915946, 8.685542, 8.473898,
           8.272935, 8.087346, 7.917257, 7.747725, 7.581904, 7.422607, 7.273135,
           7.125981, 6.987203, 6.856384, 6.735804, 6.618102, 6.506017, 6.401565,
           6.306005, 6.218501, 6.132053, 6.048978, 5.969465, 5.893236, 5.821437,
           5.754459, 5.685344, 5.616887, 5.548902, 5.482012, 5.416889, 5.352702,
           5.293128, 5.233577, 5.176333, 5.121306, 5.068321, 5.017319, 4.967722,
           4.918914, 4.871388, 4.827323, 4.781873, 4.775983, 4.745532, 4.710554,
           4.676386}},
         {{6.883829, 6.780647, 6.681990, 6.586427, 6.496063, 6.411232, 6.332220,
           6.297340, 6.296762, 6.264266, 6.177242, 6.095692, 6.028247, 5.963817,
           5.904237, 5.842555, 5.782519, 5.723747, 5.668839, 5.614213, 5.560341,
           5.507966, 5.457427, 5.412877, 5.367451, 5.322699, 5.279875, 5.238880,
           5.199754, 5.164977, 5.128849, 5.094269, 5.061530, 5.032583, 5.002415,
           4.975656, 4.951628, 4.930618, 4.909388, 4.889103, 4.873416, 4.864165,
           4.850663, 4.839447, 4.828713, 4.817352, 4.801813, 4.786948, 4.772681,
           4.759092},
          {10.813837, 10.456602, 10.122275, 9.811401, 9.514703, 9.244502,
           8.988982,  8.752253,  8.538658,  8.336739, 8.143385, 7.958070,
           7.776127,  7.602568,  7.436505,  7.282677, 7.131387, 6.989285,
           6.856141,  6.734160,  6.615468,  6.502722, 6.398153, 6.303016,
           6.303202,  6.242720,  6.170287,  6.096175, 6.022005, 5.952315,
           5.882286,  5.810397,  5.740249,  5.674330, 5.606944, 5.543970,
           5.481336,  5.426239,  5.370707,  5.316666, 5.266405, 5.218067,
           5.171980,  5.130866,  5.091561,  5.052832, 5.009148, 4.969041,
           4.929755,  4.893010}}},
        {{{13.619808, 13.333210, 13.051239, 12.776222, 12.507832, 12.245829,
           11.995727, 11.753959, 11.514906, 11.283765, 11.060997, 10.847584,
           10.641758, 10.451045, 10.262137, 10.079405, 9.904556,  9.736280,
           9.574299,  9.417456,  9.271892,  9.125338,  8.983215,  8.846280,
           8.714579,  8.587793,  8.465422,  8.350489,  8.240701,  8.132710,
           8.029161,  7.929314,  7.833345,  7.741154,  7.652243,  7.568850,
           7.487030,  7.405312,  7.325642,  7.248407,  7.173284,  7.100055,
           7.028790,  6.959707,  6.908063,  6.852918,  6.796272,  6.740002,
           6.687043,  6.637476},
          {17.023178, 16.587282, 16.146413, 15.716627, 15.313235, 14.935164,
           14.578000, 14.251796, 13.950023, 13.665694, 13.380526, 13.092303,
           12.808487, 12.532148, 12.263044, 12.006815, 11.761296, 11.519980,
           11.288567, 11.066883, 10.854943, 10.651480, 10.463358, 10.277642,
           10.093792, 9.913269,  9.735366,  9.560994,  9.392387,  9.230983,
           9.068343,  8.911486,  8.758408,  8.611258,  8.468098,  8.333456,
           8.202244,  8.074103,  7.950588,  7.828766,  7.708682,  7.591400,
           7.480044,  7.367800,  7.259290,  7.154478,  7.053635,  6.958499,
           6.897247,  6.827157}},
         {{13.480532, 13.199702, 12.924371, 12.657368, 12.394979, 12.139100,
           11.896585, 11.652442, 11.411946, 11.175859, 10.951252, 10.739858,
           10.540739, 10.356916, 10.176097, 9.997932,  9.826595,  9.662166,
           9.504456,  9.354764,  9.207778,  9.063534,  8.924767,  8.790783,
           8.661476,  8.537152,  8.417686,  8.307254,  8.197149,  8.091746,
           7.990159,  7.892736,  7.799107,  7.709717,  7.623533,  7.545070,
           7.464700,  7.387142,  7.312549,  7.240093,  7.170801,  7.103651,
           7.039544,  6.979083,  6.965581,  6.938540,  6.897344,  6.853067,
           6.811643,  6.773948},
          {18.389574, 17.868128, 17.379752, 16.919954, 16.480883, 16.042002,
           15.623852, 15.243041, 14.888220, 14.557189, 14.240072, 13.926427,
           13.618521, 13.313774, 13.014845, 12.724313, 12.440925, 12.166572,
           11.909261, 11.656657, 11.413136, 11.180060, 10.957387, 10.742463,
           10.534916, 10.340772, 10.144957, 9.951462,  9.762970,  9.581154,
           9.406669,  9.240872,  9.075545,  8.917995,  8.764943,  8.619165,
           8.478013,  8.346718,  8.217575,  8.089138,  7.964049,  7.841836,
           7.723092,  7.609408,  7.503346,  7.397573,  7.297695,  7.204091,
           7.116498,  7.035261}},
         {{13.203329, 12.904131, 12.617064, 12.338443, 12.071996, 11.821350,
           11.571990, 11.332102, 11.101702, 10.882220, 10.671557, 10.475270,
           10.478742, 10.342613, 10.176987, 10.009588, 9.848459,  9.691306,
           9.548787,  9.402045,  9.257363,  9.117622,  8.982202,  8.856357,
           8.732197,  8.611428,  8.500031,  8.388422,  8.281074,  8.184131,
           8.086244,  7.992982,  7.904427,  7.824159,  7.748633,  7.678088,
           7.609661,  7.550085,  7.492398,  7.438112,  7.390446,  7.346785,
           7.308065,  7.277974,  7.250214,  7.222161,  7.182753,  7.145511,
           7.113407,  7.080288},
          {20.008668, 19.386847, 18.793876, 18.239456, 17.728495, 17.253798,
           16.810030, 16.386557, 15.974009, 15.584423, 15.216711, 14.849587,
           14.496031, 14.161395, 13.838792, 13.522618, 13.210886, 12.906528,
           12.612024, 12.325363, 12.051220, 11.792285, 11.535697, 11.285874,
           11.042868, 10.807854, 10.582432, 10.469400, 10.402180, 10.229573,
           10.038414, 9.853840,  9.675114,  9.511977,  9.343008,  9.186404,
           9.033071,  8.890591,  8.748925,  8.612979,  8.484463,  8.359190,
           8.240255,  8.132650,  8.031082,  7.932514,  7.831777,  7.733956,
           7.641643,  7.554973}}}}};

// RB: Move the rest of this file to stellar.h.

// Baraffe stellar evolution grid
double fdBaraffe(int, double, double, int, int *);

/* @endcond */

// Data grid for Proxima Centauri from the Y^2 isochrones
// obtained from www.astro.yale.edu/demarque/fs255_grid.tar.gz
// [Fe/H] = +0.3, mixing length parameter alpha_MLT = 1.0
static double const PROXIMACEN_MARR[PROXIMACEN_MLEN] = {0.1, 0.15};
static double const PROXIMACEN_AARR[PROXIMACEN_ALEN] = {
      -3.500000, -3.490782, -3.481563, -3.472345, -3.463126, -3.453908,
      -3.444689, -3.435471, -3.426253, -3.417034, -3.407816, -3.398597,
      -3.389379, -3.380160, -3.370942, -3.361723, -3.352505, -3.343287,
      -3.334068, -3.324850, -3.315631, -3.306413, -3.297194, -3.287976,
      -3.278758, -3.269539, -3.260321, -3.251102, -3.241884, -3.232665,
      -3.223447, -3.214228, -3.205010, -3.195792, -3.186573, -3.177355,
      -3.168136, -3.158918, -3.149699, -3.140481, -3.131263, -3.122044,
      -3.112826, -3.103607, -3.094389, -3.085170, -3.075952, -3.066733,
      -3.057515, -3.048297, -3.039078, -3.029860, -3.020641, -3.011423,
      -3.002204, -2.992986, -2.983768, -2.974549, -2.965331, -2.956112,
      -2.946894, -2.937675, -2.928457, -2.919238, -2.910020, -2.900802,
      -2.891583, -2.882365, -2.873146, -2.863928, -2.854709, -2.845491,
      -2.836273, -2.827054, -2.817836, -2.808617, -2.799399, -2.790180,
      -2.780962, -2.771743, -2.762525, -2.753307, -2.744088, -2.734870,
      -2.725651, -2.716433, -2.707214, -2.697996, -2.688778, -2.679559,
      -2.670341, -2.661122, -2.651904, -2.642685, -2.633467, -2.624248,
      -2.615030, -2.605812, -2.596593, -2.587375, -2.578156, -2.568938,
      -2.559719, -2.550501, -2.541283, -2.532064, -2.522846, -2.513627,
      -2.504409, -2.495190, -2.485972, -2.476754, -2.467535, -2.458317,
      -2.449098, -2.439880, -2.430661, -2.421443, -2.412224, -2.403006,
      -2.393788, -2.384569, -2.375351, -2.366132, -2.356914, -2.347695,
      -2.338477, -2.329259, -2.320040, -2.310822, -2.301603, -2.292385,
      -2.283166, -2.273948, -2.264729, -2.255511, -2.246293, -2.237074,
      -2.227856, -2.218637, -2.209419, -2.200200, -2.190982, -2.181764,
      -2.172545, -2.163327, -2.154108, -2.144890, -2.135671, -2.126453,
      -2.117234, -2.108016, -2.098798, -2.089579, -2.080361, -2.071142,
      -2.061924, -2.052705, -2.043487, -2.034269, -2.025050, -2.015832,
      -2.006613, -1.997395, -1.988176, -1.978958, -1.969739, -1.960521,
      -1.951303, -1.942084, -1.932866, -1.923647, -1.914429, -1.905210,
      -1.895992, -1.886774, -1.877555, -1.868337, -1.859118, -1.849900,
      -1.840681, -1.831463, -1.822244, -1.813026, -1.803808, -1.794589,
      -1.785371, -1.776152, -1.766934, -1.757715, -1.748497, -1.739279,
      -1.730060, -1.720842, -1.711623, -1.702405, -1.693186, -1.683968,
      -1.674749, -1.665531, -1.656313, -1.647094, -1.637876, -1.628657,
      -1.619439, -1.610220, -1.601002, -1.591784, -1.582565, -1.573347,
      -1.564128, -1.554910, -1.545691, -1.536473, -1.527255, -1.518036,
      -1.508818, -1.499599, -1.490381, -1.481162, -1.471944, -1.462725,
      -1.453507, -1.444289, -1.435070, -1.425852, -1.416633, -1.407415,
      -1.398196, -1.388978, -1.379760, -1.370541, -1.361323, -1.352104,
      -1.342886, -1.333667, -1.324449, -1.315230, -1.306012, -1.296794,
      -1.287575, -1.278357, -1.269138, -1.259920, -1.250701, -1.241483,
      -1.232265, -1.223046, -1.213828, -1.204609, -1.195391, -1.186172,
      -1.176954, -1.167735, -1.158517, -1.149299, -1.140080, -1.130862,
      -1.121643, -1.112425, -1.103206, -1.093988, -1.084770, -1.075551,
      -1.066333, -1.057114, -1.047896, -1.038677, -1.029459, -1.020240,
      -1.011022, -1.001804, -0.992585, -0.983367, -0.974148, -0.964930,
      -0.955711, -0.946493, -0.937275, -0.928056, -0.918838, -0.909619,
      -0.900401, -0.891182, -0.881964, -0.872745, -0.863527, -0.854309,
      -0.845090, -0.835872, -0.826653, -0.817435, -0.808216, -0.798998,
      -0.789780, -0.780561, -0.771343, -0.762124, -0.752906, -0.743687,
      -0.734469, -0.725251, -0.716032, -0.706814, -0.697595, -0.688377,
      -0.679158, -0.669940, -0.660721, -0.651503, -0.642285, -0.633066,
      -0.623848, -0.614629, -0.605411, -0.596192, -0.586974, -0.577756,
      -0.568537, -0.559319, -0.550100, -0.540882, -0.531663, -0.522445,
      -0.513226, -0.504008, -0.494790, -0.485571, -0.476353, -0.467134,
      -0.457916, -0.448697, -0.439479, -0.430261, -0.421042, -0.411824,
      -0.402605, -0.393387, -0.384168, -0.374950, -0.365731, -0.356513,
      -0.347295, -0.338076, -0.328858, -0.319639, -0.310421, -0.301202,
      -0.291984, -0.282766, -0.273547, -0.264329, -0.255110, -0.245892,
      -0.236673, -0.227455, -0.218236, -0.209018, -0.199800, -0.190581,
      -0.181363, -0.172144, -0.162926, -0.153707, -0.144489, -0.135271,
      -0.126052, -0.116834, -0.107615, -0.098397, -0.089178, -0.079960,
      -0.070741, -0.061523, -0.052305, -0.043086, -0.033868, -0.024649,
      -0.015431, -0.006212, 0.003006,  0.012224,  0.021443,  0.030661,
      0.039880,  0.049098,  0.058317,  0.067535,  0.076754,  0.085972,
      0.095190,  0.104409,  0.113627,  0.122846,  0.132064,  0.141283,
      0.150501,  0.159719,  0.168938,  0.178156,  0.187375,  0.196593,
      0.205812,  0.215030,  0.224248,  0.233467,  0.242685,  0.251904,
      0.261122,  0.270341,  0.279559,  0.288778,  0.297996,  0.307214,
      0.316433,  0.325651,  0.334870,  0.344088,  0.353307,  0.362525,
      0.371743,  0.380962,  0.390180,  0.399399,  0.408617,  0.417836,
      0.427054,  0.436273,  0.445491,  0.454709,  0.463928,  0.473146,
      0.482365,  0.491583,  0.500802,  0.510020,  0.519238,  0.528457,
      0.537675,  0.546894,  0.556112,  0.565331,  0.574549,  0.583768,
      0.592986,  0.602204,  0.611423,  0.620641,  0.629860,  0.639078,
      0.648297,  0.657515,  0.666733,  0.675952,  0.685170,  0.694389,
      0.703607,  0.712826,  0.722044,  0.731263,  0.740481,  0.749699,
      0.758918,  0.768136,  0.777355,  0.786573,  0.795792,  0.805010,
      0.814228,  0.823447,  0.832665,  0.841884,  0.851102,  0.860321,
      0.869539,  0.878758,  0.887976,  0.897194,  0.906413,  0.915631,
      0.924850,  0.934068,  0.943287,  0.952505,  0.961723,  0.970942,
      0.980160,  0.989379,  0.998597,  1.007816,  1.017034,  1.026253,
      1.035471,  1.044689,  1.053908,  1.063126,  1.072345,  1.081563,
      1.090782,  1.100000};
static double const PROXIMACEN_LOGL_LO[PROXIMACEN_ALEN] = {
      -1.073914, -1.077724, -1.081564, -1.085433, -1.089330, -1.093251,
      -1.097194, -1.101160, -1.105152, -1.109166, -1.113202, -1.117257,
      -1.121334, -1.125441, -1.129590, -1.133794, -1.138063, -1.142378,
      -1.146710, -1.151054, -1.155417, -1.159800, -1.164204, -1.168629,
      -1.173076, -1.177532, -1.181992, -1.186468, -1.190965, -1.195481,
      -1.200014, -1.204568, -1.209129, -1.213686, -1.218282, -1.222955,
      -1.227686, -1.232452, -1.237235, -1.242032, -1.246845, -1.251675,
      -1.256520, -1.261374, -1.266238, -1.271120, -1.276018, -1.280927,
      -1.285851, -1.290788, -1.295721, -1.300646, -1.305580, -1.310568,
      -1.315635, -1.320741, -1.325864, -1.331005, -1.336156, -1.341312,
      -1.346482, -1.351664, -1.356848, -1.362043, -1.367252, -1.372456,
      -1.377658, -1.382870, -1.388092, -1.393321, -1.398551, -1.403810,
      -1.409123, -1.414479, -1.419857, -1.425235, -1.430619, -1.436009,
      -1.441398, -1.446796, -1.452204, -1.457600, -1.462996, -1.468415,
      -1.473838, -1.479260, -1.484691, -1.490122, -1.495553, -1.501001,
      -1.506491, -1.512024, -1.517559, -1.523100, -1.528652, -1.534184,
      -1.539716, -1.545279, -1.550852, -1.556420, -1.561988, -1.567558,
      -1.573130, -1.578703, -1.584281, -1.589857, -1.595424, -1.601041,
      -1.606718, -1.612393, -1.618060, -1.623728, -1.629407, -1.635092,
      -1.640777, -1.646456, -1.652139, -1.657826, -1.663509, -1.669187,
      -1.674866, -1.680553, -1.686233, -1.691909, -1.697617, -1.703360,
      -1.709123, -1.714890, -1.720658, -1.726426, -1.732192, -1.737958,
      -1.743721, -1.749483, -1.755245, -1.761011, -1.766777, -1.772534,
      -1.778287, -1.784066, -1.789854, -1.795646, -1.801478, -1.807340,
      -1.813210, -1.819077, -1.824947, -1.830821, -1.836691, -1.842560,
      -1.848424, -1.854274, -1.860143, -1.866038, -1.871928, -1.877814,
      -1.883701, -1.889591, -1.895475, -1.901361, -1.907276, -1.913214,
      -1.919162, -1.925106, -1.931046, -1.936987, -1.942938, -1.948902,
      -1.954870, -1.960832, -1.966791, -1.972750, -1.978708, -1.984662,
      -1.990612, -1.996562, -2.002511, -2.008449, -2.014382, -2.020356,
      -2.026354, -2.032347, -2.038342, -2.044341, -2.050342, -2.056345,
      -2.062345, -2.068341, -2.074343, -2.080349, -2.086353, -2.092356,
      -2.098372, -2.104409, -2.110460, -2.116518, -2.122577, -2.128631,
      -2.134692, -2.140760, -2.146826, -2.152896, -2.158972, -2.165044,
      -2.171104, -2.177170, -2.183273, -2.189393, -2.195511, -2.201624,
      -2.207738, -2.213858, -2.219981, -2.226097, -2.232211, -2.238330,
      -2.244430, -2.250511, -2.256587, -2.262656, -2.268720, -2.274784,
      -2.280847, -2.286926, -2.293032, -2.299141, -2.305248, -2.311356,
      -2.317457, -2.323569, -2.329683, -2.335712, -2.341612, -2.347407,
      -2.353187, -2.358970, -2.364749, -2.370548, -2.376365, -2.382190,
      -2.388009, -2.393833, -2.399670, -2.405512, -2.411358, -2.417112,
      -2.422567, -2.427876, -2.433223, -2.438580, -2.443963, -2.449351,
      -2.454681, -2.460204, -2.466006, -2.471817, -2.477599, -2.483401,
      -2.489233, -2.494904, -2.500292, -2.505438, -2.510509, -2.515597,
      -2.520697, -2.525818, -2.530939, -2.536049, -2.541285, -2.546776,
      -2.552529, -2.558329, -2.564122, -2.569915, -2.575505, -2.580747,
      -2.585660, -2.590506, -2.595373, -2.600241, -2.605118, -2.610003,
      -2.614885, -2.619770, -2.624951, -2.630574, -2.636244, -2.641836,
      -2.647411, -2.653009, -2.658276, -2.662956, -2.667509, -2.672107,
      -2.676677, -2.681212, -2.685742, -2.690283, -2.694732, -2.699244,
      -2.704131, -2.709247, -2.714318, -2.719295, -2.724276, -2.729130,
      -2.734132, -2.738373, -2.741822, -2.745385, -2.748828, -2.752314,
      -2.755613, -2.758940, -2.762277, -2.765489, -2.768634, -2.771686,
      -2.775235, -2.779041, -2.782560, -2.785954, -2.789336, -2.792485,
      -2.795594, -2.798630, -2.801479, -2.804171, -2.806850, -2.809443,
      -2.811865, -2.814128, -2.816365, -2.818637, -2.820878, -2.823024,
      -2.825023, -2.826878, -2.828611, -2.830246, -2.831804, -2.833275,
      -2.834637, -2.835878, -2.837015, -2.838065, -2.839040, -2.839939,
      -2.840762, -2.841511, -2.842194, -2.842823, -2.843410, -2.843964,
      -2.844489, -2.844981, -2.845437, -2.845855, -2.846231, -2.846565,
      -2.846858, -2.847115, -2.847341, -2.847541, -2.847718, -2.847879,
      -2.848025, -2.848162, -2.848288, -2.848405, -2.848513, -2.848611,
      -2.848701, -2.848783, -2.848857, -2.848924, -2.848984, -2.849037,
      -2.849084, -2.849126, -2.849162, -2.849194, -2.849221, -2.849245,
      -2.849265, -2.849282, -2.849297, -2.849310, -2.849321, -2.849330,
      -2.849338, -2.849344, -2.849349, -2.849352, -2.849354, -2.849355,
      -2.849356, -2.849355, -2.849353, -2.849351, -2.849348, -2.849344,
      -2.849340, -2.849336, -2.849331, -2.849325, -2.849319, -2.849313,
      -2.849306, -2.849299, -2.849292, -2.849284, -2.849276, -2.849268,
      -2.849259, -2.849251, -2.849242, -2.849233, -2.849223, -2.849213,
      -2.849203, -2.849193, -2.849182, -2.849171, -2.849159, -2.849147,
      -2.849134, -2.849120, -2.849106, -2.849092, -2.849076, -2.849060,
      -2.849044, -2.849027, -2.849009, -2.848990, -2.848972, -2.848952,
      -2.848933, -2.848912, -2.848892, -2.848871, -2.848850, -2.848828,
      -2.848807, -2.848785, -2.848763, -2.848740, -2.848718, -2.848695,
      -2.848673, -2.848650, -2.848626, -2.848603, -2.848579, -2.848555,
      -2.848530, -2.848505, -2.848479, -2.848453, -2.848426, -2.848398,
      -2.848370, -2.848342, -2.848312, -2.848282, -2.848251, -2.848219,
      -2.848187, -2.848153, -2.848119, -2.848084, -2.848048, -2.848011,
      -2.847973, -2.847934, -2.847894, -2.847853, -2.847811, -2.847769,
      -2.847725, -2.847680, -2.847634, -2.847586, -2.847538, -2.847489,
      -2.847438, -2.847386, -2.847333, -2.847279, -2.847224, -2.847167,
      -2.847110, -2.847051, -2.846991, -2.846930, -2.846867, -2.846804,
      -2.846739, -2.846672};
static double const PROXIMACEN_LOGR_LO[PROXIMACEN_ALEN] = {
      0.125120,  0.122800,  0.120459,  0.118098,  0.115722,  0.113330,
      0.110922,  0.108498,  0.106058,  0.103603,  0.101132,  0.098646,
      0.096145,  0.093629,  0.091099,  0.088555,  0.085995,  0.083420,
      0.080830,  0.078227,  0.075611,  0.072981,  0.070339,  0.067682,
      0.065011,  0.062332,  0.059647,  0.056946,  0.054228,  0.051498,
      0.048757,  0.046002,  0.043236,  0.040465,  0.037688,  0.034899,
      0.032093,  0.029270,  0.026433,  0.023586,  0.020730,  0.017863,
      0.014987,  0.012100,  0.009205,  0.006299,  0.003384,  0.000460,
      -0.002475, -0.005420, -0.008363, -0.011305, -0.014258, -0.017223,
      -0.020199, -0.023190, -0.026195, -0.029209, -0.032231, -0.035259,
      -0.038295, -0.041337, -0.044386, -0.047443, -0.050507, -0.053571,
      -0.056636, -0.059705, -0.062781, -0.065864, -0.068954, -0.072049,
      -0.075152, -0.078268, -0.081397, -0.084532, -0.087672, -0.090816,
      -0.093964, -0.097120, -0.100280, -0.103432, -0.106581, -0.109741,
      -0.112907, -0.116077, -0.119252, -0.122432, -0.125615, -0.128798,
      -0.131993, -0.135203, -0.138419, -0.141639, -0.144862, -0.148080,
      -0.151295, -0.154516, -0.157742, -0.160971, -0.164203, -0.167439,
      -0.170678, -0.173919, -0.177165, -0.180413, -0.183654, -0.186906,
      -0.190175, -0.193451, -0.196718, -0.199976, -0.203242, -0.206515,
      -0.209789, -0.213064, -0.216342, -0.219623, -0.222904, -0.226188,
      -0.229475, -0.232763, -0.236054, -0.239346, -0.242631, -0.245916,
      -0.249208, -0.252504, -0.255801, -0.259099, -0.262398, -0.265699,
      -0.269000, -0.272303, -0.275608, -0.278914, -0.282223, -0.285529,
      -0.288827, -0.292130, -0.295439, -0.298750, -0.302055, -0.305364,
      -0.308682, -0.312002, -0.315322, -0.318642, -0.321963, -0.325286,
      -0.328608, -0.331921, -0.335236, -0.338557, -0.341880, -0.345202,
      -0.348525, -0.351848, -0.355172, -0.358495, -0.361811, -0.365132,
      -0.368461, -0.371789, -0.375120, -0.378444, -0.381757, -0.385075,
      -0.388401, -0.391724, -0.395047, -0.398369, -0.401691, -0.405013,
      -0.408334, -0.411655, -0.414977, -0.418296, -0.421604, -0.424914,
      -0.428224, -0.431529, -0.434837, -0.438146, -0.441456, -0.444765,
      -0.448075, -0.451384, -0.454693, -0.458003, -0.461314, -0.464628,
      -0.467933, -0.471231, -0.474540, -0.477855, -0.481166, -0.484466,
      -0.487770, -0.491082, -0.494394, -0.497704, -0.501016, -0.504328,
      -0.507633, -0.510933, -0.514240, -0.517551, -0.520861, -0.524170,
      -0.527477, -0.530784, -0.534090, -0.537394, -0.540699, -0.544002,
      -0.547296, -0.550583, -0.553871, -0.557165, -0.560462, -0.563757,
      -0.567038, -0.570314, -0.573598, -0.576883, -0.580166, -0.583449,
      -0.586729, -0.590009, -0.593289, -0.596556, -0.599825, -0.603108,
      -0.606387, -0.609656, -0.612921, -0.616198, -0.619482, -0.622764,
      -0.626047, -0.629331, -0.632616, -0.635904, -0.639184, -0.642458,
      -0.645754, -0.649050, -0.652329, -0.655616, -0.658912, -0.662213,
      -0.665509, -0.668801, -0.672097, -0.675409, -0.678724, -0.682032,
      -0.685340, -0.688637, -0.691929, -0.695238, -0.698542, -0.701836,
      -0.705138, -0.708446, -0.711758, -0.715074, -0.718381, -0.721681,
      -0.724981, -0.728291, -0.731607, -0.734909, -0.738188, -0.741467,
      -0.744761, -0.748029, -0.751279, -0.754540, -0.757802, -0.761062,
      -0.764318, -0.767561, -0.770785, -0.774000, -0.777228, -0.780454,
      -0.783653, -0.786817, -0.789969, -0.793123, -0.796233, -0.799309,
      -0.802383, -0.805443, -0.808487, -0.811528, -0.814545, -0.817531,
      -0.820499, -0.823461, -0.826375, -0.829243, -0.832108, -0.834918,
      -0.837713, -0.840404, -0.843014, -0.845625, -0.848169, -0.850717,
      -0.853154, -0.855595, -0.858020, -0.860360, -0.862650, -0.864889,
      -0.867097, -0.869270, -0.871339, -0.873328, -0.875297, -0.877145,
      -0.878960, -0.880725, -0.882390, -0.883973, -0.885530, -0.887027,
      -0.888436, -0.889769, -0.891049, -0.892286, -0.893470, -0.894592,
      -0.895644, -0.896627, -0.897547, -0.898413, -0.899232, -0.900002,
      -0.900716, -0.901372, -0.901976, -0.902536, -0.903056, -0.903535,
      -0.903971, -0.904365, -0.904723, -0.905051, -0.905356, -0.905646,
      -0.905922, -0.906182, -0.906424, -0.906646, -0.906847, -0.907025,
      -0.907182, -0.907320, -0.907442, -0.907549, -0.907644, -0.907730,
      -0.907809, -0.907882, -0.907949, -0.908011, -0.908068, -0.908121,
      -0.908168, -0.908211, -0.908250, -0.908285, -0.908317, -0.908344,
      -0.908369, -0.908391, -0.908410, -0.908427, -0.908441, -0.908453,
      -0.908464, -0.908473, -0.908481, -0.908488, -0.908494, -0.908499,
      -0.908504, -0.908507, -0.908510, -0.908512, -0.908514, -0.908515,
      -0.908516, -0.908516, -0.908516, -0.908515, -0.908514, -0.908513,
      -0.908512, -0.908510, -0.908508, -0.908506, -0.908504, -0.908501,
      -0.908499, -0.908496, -0.908492, -0.908489, -0.908485, -0.908482,
      -0.908478, -0.908474, -0.908469, -0.908465, -0.908460, -0.908455,
      -0.908450, -0.908445, -0.908440, -0.908434, -0.908428, -0.908422,
      -0.908416, -0.908410, -0.908403, -0.908396, -0.908389, -0.908382,
      -0.908374, -0.908367, -0.908359, -0.908351, -0.908342, -0.908334,
      -0.908325, -0.908316, -0.908307, -0.908298, -0.908288, -0.908279,
      -0.908269, -0.908259, -0.908249, -0.908239, -0.908228, -0.908218,
      -0.908207, -0.908196, -0.908185, -0.908174, -0.908162, -0.908151,
      -0.908139, -0.908127, -0.908115, -0.908102, -0.908089, -0.908076,
      -0.908063, -0.908049, -0.908035, -0.908021, -0.908006, -0.907991,
      -0.907976, -0.907960, -0.907944, -0.907928, -0.907911, -0.907894,
      -0.907876, -0.907858, -0.907839, -0.907820, -0.907800, -0.907780,
      -0.907760, -0.907738, -0.907717, -0.907694, -0.907671, -0.907648,
      -0.907624, -0.907599, -0.907574, -0.907547, -0.907521, -0.907493,
      -0.907465, -0.907436, -0.907407, -0.907376, -0.907345, -0.907313,
      -0.907281, -0.907247};
static double const PROXIMACEN_LOGT_LO[PROXIMACEN_ALEN] = {
      3.430590, 3.430795, 3.431005, 3.431218, 3.431430, 3.431646, 3.431867,
      3.432091, 3.432312, 3.432532, 3.432757, 3.432987, 3.433219, 3.433451,
      3.433679, 3.433902, 3.434116, 3.434325, 3.434535, 3.434749, 3.434969,
      3.435190, 3.435407, 3.435625, 3.435850, 3.436076, 3.436301, 3.436533,
      3.436774, 3.437013, 3.437248, 3.437483, 3.437723, 3.437971, 3.438213,
      3.438439, 3.438659, 3.438880, 3.439100, 3.439323, 3.439552, 3.439781,
      3.440006, 3.440233, 3.440466, 3.440700, 3.440934, 3.441169, 3.441405,
      3.441641, 3.441877, 3.442115, 3.442361, 3.442602, 3.442824, 3.443040,
      3.443257, 3.443479, 3.443704, 3.443929, 3.444155, 3.444381, 3.444606,
      3.444836, 3.445071, 3.445301, 3.445530, 3.445765, 3.445998, 3.446227,
      3.446464, 3.446700, 3.446925, 3.447146, 3.447365, 3.447584, 3.447808,
      3.448037, 3.448259, 3.448481, 3.448710, 3.448941, 3.449169, 3.449391,
      3.449617, 3.449849, 3.450080, 3.450310, 3.450542, 3.450776, 3.451003,
      3.451221, 3.451444, 3.451672, 3.451900, 3.452122, 3.452342, 3.452563,
      3.452783, 3.453006, 3.453235, 3.453460, 3.453682, 3.453911, 3.454142,
      3.454371, 3.454596, 3.454815, 3.455031, 3.455256, 3.455476, 3.455685,
      3.455895, 3.456109, 3.456327, 3.456546, 3.456765, 3.456983, 3.457201,
      3.457423, 3.457650, 3.457870, 3.458091, 3.458318, 3.458536, 3.458745,
      3.458951, 3.459157, 3.459364, 3.459571, 3.459777, 3.459987, 3.460202,
      3.460410, 3.460619, 3.460834, 3.461049, 3.461261, 3.461469, 3.461675,
      3.461885, 3.462093, 3.462284, 3.462472, 3.462668, 3.462857, 3.463047,
      3.463243, 3.463438, 3.463634, 3.463829, 3.464018, 3.464206, 3.464394,
      3.464583, 3.464771, 3.464960, 3.465156, 3.465351, 3.465537, 3.465712,
      3.465891, 3.466073, 3.466247, 3.466429, 3.466608, 3.466769, 3.466938,
      3.467114, 3.467283, 3.467450, 3.467621, 3.467797, 3.467967, 3.468136,
      3.468312, 3.468489, 3.468663, 3.468831, 3.468995, 3.469154, 3.469308,
      3.469462, 3.469617, 3.469772, 3.469927, 3.470082, 3.470237, 3.470392,
      3.470547, 3.470702, 3.470853, 3.470997, 3.471138, 3.471279, 3.471421,
      3.471562, 3.471705, 3.471843, 3.471977, 3.472119, 3.472257, 3.472391,
      3.472533, 3.472674, 3.472806, 3.472927, 3.473050, 3.473180, 3.473308,
      3.473433, 3.473553, 3.473673, 3.473799, 3.473927, 3.474048, 3.474168,
      3.474290, 3.474418, 3.474545, 3.474674, 3.474809, 3.474936, 3.475053,
      3.475167, 3.475281, 3.475395, 3.475508, 3.475623, 3.475736, 3.475849,
      3.475974, 3.476133, 3.476330, 3.476522, 3.476707, 3.476894, 3.477087,
      3.477281, 3.477468, 3.477649, 3.477832, 3.478021, 3.478205, 3.478383,
      3.478584, 3.478862, 3.479179, 3.479487, 3.479791, 3.480092, 3.480395,
      3.480712, 3.480980, 3.481180, 3.481378, 3.481588, 3.481797, 3.481993,
      3.482221, 3.482520, 3.482891, 3.483273, 3.483643, 3.484022, 3.484398,
      3.484771, 3.485148, 3.485495, 3.485776, 3.485987, 3.486192, 3.486401,
      3.486599, 3.486840, 3.487171, 3.487590, 3.488016, 3.488428, 3.488837,
      3.489247, 3.489656, 3.490065, 3.490468, 3.490785, 3.490984, 3.491177,
      3.491391, 3.491597, 3.491782, 3.492042, 3.492446, 3.492864, 3.493257,
      3.493654, 3.494049, 3.494435, 3.494818, 3.495212, 3.495578, 3.495849,
      3.496049, 3.496237, 3.496429, 3.496612, 3.496808, 3.496951, 3.497233,
      3.497681, 3.498095, 3.498500, 3.498904, 3.499301, 3.499692, 3.500066,
      3.500438, 3.500794, 3.501153, 3.501371, 3.501504, 3.501657, 3.501801,
      3.501945, 3.502083, 3.502215, 3.502335, 3.502450, 3.502568, 3.502677,
      3.502779, 3.502880, 3.502980, 3.503060, 3.503111, 3.503144, 3.503169,
      3.503192, 3.503218, 3.503247, 3.503274, 3.503294, 3.503309, 3.503324,
      3.503341, 3.503359, 3.503377, 3.503393, 3.503408, 3.503420, 3.503429,
      3.503436, 3.503442, 3.503448, 3.503454, 3.503461, 3.503469, 3.503476,
      3.503483, 3.503490, 3.503495, 3.503501, 3.503505, 3.503509, 3.503512,
      3.503515, 3.503518, 3.503521, 3.503524, 3.503527, 3.503529, 3.503532,
      3.503534, 3.503536, 3.503538, 3.503539, 3.503540, 3.503541, 3.503541,
      3.503541, 3.503541, 3.503541, 3.503541, 3.503541, 3.503540, 3.503540,
      3.503540, 3.503540, 3.503540, 3.503541, 3.503541, 3.503542, 3.503543,
      3.503543, 3.503544, 3.503545, 3.503546, 3.503547, 3.503548, 3.503549,
      3.503550, 3.503550, 3.503551, 3.503551, 3.503551, 3.503552, 3.503552,
      3.503552, 3.503552, 3.503551, 3.503551, 3.503551, 3.503551, 3.503550,
      3.503550, 3.503550, 3.503550, 3.503550, 3.503549, 3.503549, 3.503549,
      3.503549, 3.503549, 3.503549, 3.503549, 3.503549, 3.503549, 3.503549,
      3.503549, 3.503549, 3.503550, 3.503550, 3.503550, 3.503551, 3.503551,
      3.503552, 3.503552, 3.503553, 3.503554, 3.503554, 3.503555, 3.503556,
      3.503556, 3.503557, 3.503558, 3.503558, 3.503559, 3.503560, 3.503560,
      3.503561, 3.503562, 3.503562, 3.503563, 3.503563, 3.503564, 3.503564,
      3.503565, 3.503566, 3.503566, 3.503567, 3.503567, 3.503568, 3.503568,
      3.503569, 3.503569, 3.503570, 3.503571, 3.503571, 3.503572, 3.503572,
      3.503573, 3.503574, 3.503574, 3.503575, 3.503575, 3.503576, 3.503576,
      3.503577, 3.503578, 3.503578, 3.503579, 3.503579, 3.503580, 3.503580,
      3.503580, 3.503581, 3.503581, 3.503581, 3.503582, 3.503582, 3.503582,
      3.503582, 3.503582, 3.503582, 3.503582, 3.503582, 3.503582, 3.503582,
      3.503582, 3.503581, 3.503581};
static double const PROXIMACEN_LOGL_HI[PROXIMACEN_ALEN] = {
      -0.825540, -0.829467, -0.833419, -0.837392, -0.841390, -0.845400,
      -0.849396, -0.853376, -0.857368, -0.861385, -0.865429, -0.869494,
      -0.873572, -0.877693, -0.881933, -0.886299, -0.890709, -0.895124,
      -0.899555, -0.904005, -0.908471, -0.912956, -0.917463, -0.921979,
      -0.926482, -0.930974, -0.935463, -0.939962, -0.944477, -0.949010,
      -0.953560, -0.958123, -0.962706, -0.967324, -0.972014, -0.976802,
      -0.981641, -0.986487, -0.991341, -0.996210, -1.001102, -1.005991,
      -1.010850, -1.015713, -1.020603, -1.025511, -1.030428, -1.035356,
      -1.040293, -1.045239, -1.050198, -1.055174, -1.060171, -1.065219,
      -1.070340, -1.075488, -1.080635, -1.085775, -1.090916, -1.096064,
      -1.101219, -1.106381, -1.111550, -1.116731, -1.121921, -1.127114,
      -1.132314, -1.137529, -1.142746, -1.147960, -1.153188, -1.158470,
      -1.163818, -1.169165, -1.174496, -1.179839, -1.185191, -1.190548,
      -1.195907, -1.201271, -1.206641, -1.212012, -1.217385, -1.222763,
      -1.228148, -1.233534, -1.238907, -1.244255, -1.249613, -1.255049,
      -1.260516, -1.265973, -1.271439, -1.276907, -1.282372, -1.287846,
      -1.293325, -1.298799, -1.304277, -1.309757, -1.315239, -1.320731,
      -1.326222, -1.331692, -1.337163, -1.342683, -1.348285, -1.353909,
      -1.359527, -1.365151, -1.370773, -1.376390, -1.382008, -1.387631,
      -1.393258, -1.398881, -1.404487, -1.410086, -1.415705, -1.421327,
      -1.426940, -1.432549, -1.438205, -1.443919, -1.449657, -1.455399,
      -1.461139, -1.466880, -1.472620, -1.478358, -1.484090, -1.489808,
      -1.495523, -1.501254, -1.506982, -1.512707, -1.518440, -1.524168,
      -1.529889, -1.535624, -1.541420, -1.547249, -1.553067, -1.558891,
      -1.564711, -1.570516, -1.576338, -1.582171, -1.587998, -1.593832,
      -1.599662, -1.605483, -1.611315, -1.617143, -1.622963, -1.628789,
      -1.634624, -1.640472, -1.646346, -1.652236, -1.658131, -1.664032,
      -1.669924, -1.675812, -1.681708, -1.687570, -1.693412, -1.699270,
      -1.705121, -1.710969, -1.716830, -1.722680, -1.728528, -1.734398,
      -1.740287, -1.746195, -1.752121, -1.758050, -1.763976, -1.769900,
      -1.775825, -1.781750, -1.787676, -1.793603, -1.799528, -1.805445,
      -1.811356, -1.817277, -1.823213, -1.829151, -1.835087, -1.841019,
      -1.846956, -1.852929, -1.858931, -1.864929, -1.870919, -1.876914,
      -1.882914, -1.888884, -1.894848, -1.900833, -1.906815, -1.912794,
      -1.918776, -1.924760, -1.930737, -1.936711, -1.942696, -1.948658,
      -1.954620, -1.960653, -1.966689, -1.972694, -1.978699, -1.984708,
      -1.990706, -1.996688, -2.002683, -2.008694, -2.014702, -2.020711,
      -2.026715, -2.032712, -2.038722, -2.044713, -2.050652, -2.056573,
      -2.062488, -2.068415, -2.074405, -2.080433, -2.086460, -2.092482,
      -2.098501, -2.104520, -2.110539, -2.116570, -2.122588, -2.128533,
      -2.134425, -2.140301, -2.146178, -2.152054, -2.157932, -2.163815,
      -2.169694, -2.175585, -2.181532, -2.187542, -2.193591, -2.199641,
      -2.205640, -2.211575, -2.217473, -2.223356, -2.229233, -2.235104,
      -2.240973, -2.246837, -2.252693, -2.258538, -2.264374, -2.270198,
      -2.276007, -2.281752, -2.287385, -2.293008, -2.298682, -2.304374,
      -2.310060, -2.315727, -2.321372, -2.326992, -2.332584, -2.338149,
      -2.343686, -2.349195, -2.354677, -2.360115, -2.365480, -2.370715,
      -2.375815, -2.380859, -2.385877, -2.390856, -2.395768, -2.400706,
      -2.405746, -2.410746, -2.415678, -2.420548, -2.425348, -2.430066,
      -2.434703, -2.439256, -2.443721, -2.448072, -2.452260, -2.456358,
      -2.460358, -2.464250, -2.468047, -2.471731, -2.475303, -2.478754,
      -2.482098, -2.485325, -2.488426, -2.491403, -2.494264, -2.497008,
      -2.499619, -2.502115, -2.504486, -2.506732, -2.508869, -2.510875,
      -2.512772, -2.514588, -2.516307, -2.517916, -2.519416, -2.520815,
      -2.522118, -2.523329, -2.524450, -2.525481, -2.526426, -2.527285,
      -2.528066, -2.528780, -2.529437, -2.530043, -2.530600, -2.531109,
      -2.531572, -2.531991, -2.532371, -2.532713, -2.533023, -2.533302,
      -2.533555, -2.533784, -2.533989, -2.534175, -2.534341, -2.534490,
      -2.534624, -2.534745, -2.534853, -2.534950, -2.535036, -2.535112,
      -2.535179, -2.535238, -2.535290, -2.535335, -2.535374, -2.535408,
      -2.535437, -2.535462, -2.535482, -2.535499, -2.535512, -2.535523,
      -2.535531, -2.535537, -2.535541, -2.535544, -2.535545, -2.535545,
      -2.535544, -2.535542, -2.535538, -2.535534, -2.535529, -2.535524,
      -2.535517, -2.535510, -2.535503, -2.535495, -2.535487, -2.535479,
      -2.535470, -2.535461, -2.535451, -2.535442, -2.535432, -2.535421,
      -2.535411, -2.535400, -2.535390, -2.535379, -2.535367, -2.535356,
      -2.535345, -2.535333, -2.535321, -2.535309, -2.535297, -2.535285,
      -2.535272, -2.535260, -2.535247, -2.535234, -2.535220, -2.535206,
      -2.535192, -2.535178, -2.535163, -2.535149, -2.535133, -2.535118,
      -2.535102, -2.535085, -2.535068, -2.535051, -2.535033, -2.535015,
      -2.534996, -2.534977, -2.534957, -2.534936, -2.534914, -2.534892,
      -2.534870, -2.534846, -2.534822, -2.534796, -2.534771, -2.534744,
      -2.534716, -2.534688, -2.534658, -2.534628, -2.534596, -2.534564,
      -2.534530, -2.534496, -2.534461, -2.534424, -2.534386, -2.534347,
      -2.534307, -2.534266, -2.534224, -2.534180, -2.534135, -2.534088,
      -2.534040, -2.533990, -2.533938, -2.533885, -2.533829, -2.533772,
      -2.533712, -2.533650, -2.533586, -2.533520, -2.533451, -2.533380,
      -2.533306, -2.533230, -2.533150, -2.533068, -2.532983, -2.532895,
      -2.532804, -2.532709, -2.532611, -2.532510, -2.532405, -2.532296,
      -2.532183, -2.532067, -2.531947, -2.531823, -2.531694, -2.531562,
      -2.531425, -2.531284, -2.531138, -2.530988, -2.530833, -2.530673,
      -2.530507, -2.530336, -2.530159, -2.529976, -2.529787, -2.529591,
      -2.529390, -2.529181, -2.528967, -2.528747, -2.528521, -2.528289,
      -2.528050, -2.527803};
static double const PROXIMACEN_LOGR_HI[PROXIMACEN_ALEN] = {
      0.228566,  0.226213,  0.223842,  0.221457,  0.219060,  0.216651,
      0.214228,  0.211789,  0.209337,  0.206871,  0.204389,  0.201891,
      0.199382,  0.196861,  0.194320,  0.191757,  0.189173,  0.186572,
      0.183959,  0.181336,  0.178700,  0.176052,  0.173389,  0.170717,
      0.168042,  0.165357,  0.162657,  0.159944,  0.157221,  0.154486,
      0.151739,  0.148977,  0.146205,  0.143425,  0.140633,  0.137820,
      0.134988,  0.132145,  0.129292,  0.126430,  0.123556,  0.120677,
      0.117804,  0.114923,  0.112026,  0.109118,  0.106202,  0.103277,
      0.100342,  0.097398,  0.094445,  0.091483,  0.088518,  0.085542,
      0.082545,  0.079534,  0.076516,  0.073501,  0.070484,  0.067459,
      0.064424,  0.061384,  0.058336,  0.055282,  0.052221,  0.049152,
      0.046076,  0.042995,  0.039905,  0.036810,  0.033717,  0.030611,
      0.027486,  0.024363,  0.021242,  0.018111,  0.014973,  0.011831,
      0.008683,  0.005530,  0.002372,  -0.000792, -0.003960, -0.007133,
      -0.010313, -0.013495, -0.016673, -0.019853, -0.023040, -0.026237,
      -0.029443, -0.032655, -0.035870, -0.039089, -0.042311, -0.045538,
      -0.048768, -0.052003, -0.055241, -0.058483, -0.061728, -0.064980,
      -0.068230, -0.071470, -0.074711, -0.077961, -0.081224, -0.084496,
      -0.087772, -0.091049, -0.094329, -0.097610, -0.100894, -0.104180,
      -0.107470, -0.110763, -0.114050, -0.117333, -0.120622, -0.123916,
      -0.127211, -0.130510, -0.133807, -0.137106, -0.140416, -0.143732,
      -0.147048, -0.150365, -0.153683, -0.157004, -0.160327, -0.163640,
      -0.166947, -0.170265, -0.173586, -0.176908, -0.180231, -0.183558,
      -0.186882, -0.190200, -0.193527, -0.196864, -0.200203, -0.203544,
      -0.206879, -0.210204, -0.213535, -0.216872, -0.220209, -0.223546,
      -0.226883, -0.230222, -0.233561, -0.236902, -0.240243, -0.243585,
      -0.246923, -0.250256, -0.253592, -0.256933, -0.260276, -0.263620,
      -0.266963, -0.270307, -0.273652, -0.276987, -0.280318, -0.283658,
      -0.286999, -0.290338, -0.293680, -0.297024, -0.300362, -0.303691,
      -0.307021, -0.310357, -0.313698, -0.317039, -0.320381, -0.323722,
      -0.327063, -0.330404, -0.333746, -0.337087, -0.340430, -0.343770,
      -0.347099, -0.350433, -0.353774, -0.357113, -0.360454, -0.363793,
      -0.367121, -0.370454, -0.373793, -0.377133, -0.380470, -0.383809,
      -0.387147, -0.390474, -0.393800, -0.397136, -0.400472, -0.403806,
      -0.407140, -0.410475, -0.413809, -0.417143, -0.420477, -0.423804,
      -0.427125, -0.430453, -0.433777, -0.437096, -0.440422, -0.443751,
      -0.447075, -0.450388, -0.453702, -0.457018, -0.460335, -0.463650,
      -0.466963, -0.470276, -0.473590, -0.476897, -0.480196, -0.483504,
      -0.486811, -0.490106, -0.493407, -0.496714, -0.500020, -0.503325,
      -0.506628, -0.509931, -0.513231, -0.516532, -0.519831, -0.523118,
      -0.526408, -0.529708, -0.533003, -0.536296, -0.539590, -0.542883,
      -0.546177, -0.549467, -0.552743, -0.556018, -0.559300, -0.562582,
      -0.565850, -0.569110, -0.572379, -0.575645, -0.578902, -0.582156,
      -0.585407, -0.588653, -0.591893, -0.595126, -0.598354, -0.601577,
      -0.604780, -0.607965, -0.611147, -0.614315, -0.617470, -0.620624,
      -0.623772, -0.626909, -0.630032, -0.633140, -0.636233, -0.639311,
      -0.642373, -0.645417, -0.648444, -0.651447, -0.654415, -0.657368,
      -0.660313, -0.663228, -0.666112, -0.668975, -0.671818, -0.674625,
      -0.677387, -0.680129, -0.682838, -0.685508, -0.688137, -0.690722,
      -0.693263, -0.695757, -0.698202, -0.700587, -0.702933, -0.705219,
      -0.707452, -0.709627, -0.711743, -0.713798, -0.715789, -0.717717,
      -0.719580, -0.721376, -0.723105, -0.724766, -0.726360, -0.727884,
      -0.729341, -0.730729, -0.732048, -0.733302, -0.734489, -0.735602,
      -0.736646, -0.737635, -0.738566, -0.739438, -0.740253, -0.741013,
      -0.741719, -0.742373, -0.742980, -0.743542, -0.744057, -0.744523,
      -0.744944, -0.745328, -0.745681, -0.746009, -0.746311, -0.746587,
      -0.746839, -0.747066, -0.747272, -0.747458, -0.747626, -0.747778,
      -0.747916, -0.748040, -0.748152, -0.748253, -0.748344, -0.748425,
      -0.748498, -0.748564, -0.748622, -0.748674, -0.748721, -0.748762,
      -0.748798, -0.748829, -0.748857, -0.748882, -0.748903, -0.748922,
      -0.748938, -0.748953, -0.748965, -0.748975, -0.748983, -0.748990,
      -0.748995, -0.748999, -0.749003, -0.749005, -0.749007, -0.749008,
      -0.749008, -0.749008, -0.749007, -0.749006, -0.749004, -0.749002,
      -0.748999, -0.748996, -0.748993, -0.748990, -0.748987, -0.748983,
      -0.748980, -0.748976, -0.748972, -0.748968, -0.748964, -0.748960,
      -0.748956, -0.748951, -0.748947, -0.748943, -0.748938, -0.748933,
      -0.748929, -0.748924, -0.748919, -0.748915, -0.748910, -0.748905,
      -0.748900, -0.748894, -0.748889, -0.748884, -0.748878, -0.748873,
      -0.748867, -0.748861, -0.748855, -0.748849, -0.748843, -0.748837,
      -0.748830, -0.748824, -0.748817, -0.748810, -0.748802, -0.748795,
      -0.748787, -0.748779, -0.748771, -0.748762, -0.748753, -0.748744,
      -0.748735, -0.748725, -0.748715, -0.748705, -0.748694, -0.748683,
      -0.748672, -0.748660, -0.748648, -0.748635, -0.748622, -0.748608,
      -0.748594, -0.748580, -0.748565, -0.748549, -0.748533, -0.748516,
      -0.748498, -0.748480, -0.748461, -0.748442, -0.748422, -0.748401,
      -0.748379, -0.748357, -0.748334, -0.748309, -0.748284, -0.748258,
      -0.748231, -0.748203, -0.748174, -0.748144, -0.748112, -0.748080,
      -0.748046, -0.748011, -0.747975, -0.747937, -0.747898, -0.747857,
      -0.747815, -0.747772, -0.747726, -0.747679, -0.747630, -0.747580,
      -0.747527, -0.747472, -0.747416, -0.747357, -0.747296, -0.747233,
      -0.747167, -0.747100, -0.747029, -0.746957, -0.746882, -0.746805,
      -0.746724, -0.746642, -0.746556, -0.746468, -0.746377, -0.746283,
      -0.746186, -0.746085, -0.745982, -0.745875, -0.745765, -0.745651,
      -0.745534, -0.745413};
static double const PROXIMACEN_LOGT_HI[PROXIMACEN_ALEN] = {
      3.440961, 3.441153, 3.441351, 3.441552, 3.441752, 3.441953, 3.442166,
      3.442392, 3.442621, 3.442849, 3.443077, 3.443308, 3.443543, 3.443776,
      3.443986, 3.444173, 3.444362, 3.444560, 3.444761, 3.444960, 3.445160,
      3.445362, 3.445568, 3.445778, 3.445990, 3.446209, 3.446436, 3.446666,
      3.446895, 3.447128, 3.447366, 3.447606, 3.447846, 3.448082, 3.448309,
      3.448520, 3.448726, 3.448937, 3.449150, 3.449362, 3.449573, 3.449788,
      3.450014, 3.450240, 3.450460, 3.450688, 3.450924, 3.451156, 3.451385,
      3.451615, 3.451851, 3.452090, 3.452328, 3.452557, 3.452774, 3.452988,
      3.453207, 3.453430, 3.453656, 3.453880, 3.454110, 3.454343, 3.454570,
      3.454799, 3.455034, 3.455269, 3.455505, 3.455746, 3.455990, 3.456234,
      3.456475, 3.456708, 3.456934, 3.457159, 3.457386, 3.457612, 3.457842,
      3.458077, 3.458312, 3.458548, 3.458783, 3.459019, 3.459256, 3.459500,
      3.459743, 3.459988, 3.460238, 3.460491, 3.460742, 3.460982, 3.461219,
      3.461459, 3.461699, 3.461941, 3.462189, 3.462437, 3.462685, 3.462933,
      3.463182, 3.463430, 3.463679, 3.463933, 3.464189, 3.464439, 3.464691,
      3.464938, 3.465169, 3.465399, 3.465631, 3.465861, 3.466098, 3.466337,
      3.466569, 3.466803, 3.467045, 3.467292, 3.467535, 3.467774, 3.468014,
      3.468253, 3.468494, 3.468745, 3.468979, 3.469196, 3.469415, 3.469642,
      3.469870, 3.470091, 3.470316, 3.470545, 3.470766, 3.470991, 3.471221,
      3.471450, 3.471678, 3.471907, 3.472137, 3.472367, 3.472595, 3.472818,
      3.473037, 3.473250, 3.473459, 3.473674, 3.473890, 3.474099, 3.474308,
      3.474518, 3.474727, 3.474943, 3.475159, 3.475368, 3.475577, 3.475790,
      3.476008, 3.476227, 3.476439, 3.476641, 3.476837, 3.477034, 3.477230,
      3.477426, 3.477627, 3.477831, 3.478027, 3.478228, 3.478433, 3.478635,
      3.478844, 3.479054, 3.479257, 3.479466, 3.479675, 3.479872, 3.480063,
      3.480253, 3.480443, 3.480633, 3.480824, 3.481014, 3.481205, 3.481395,
      3.481585, 3.481777, 3.481963, 3.482147, 3.482338, 3.482525, 3.482708,
      3.482898, 3.483086, 3.483268, 3.483446, 3.483617, 3.483788, 3.483965,
      3.484138, 3.484307, 3.484477, 3.484647, 3.484817, 3.484987, 3.485155,
      3.485328, 3.485505, 3.485677, 3.485845, 3.486016, 3.486192, 3.486368,
      3.486536, 3.486693, 3.486847, 3.487004, 3.487166, 3.487330, 3.487492,
      3.487648, 3.487803, 3.487959, 3.488115, 3.488270, 3.488426, 3.488582,
      3.488736, 3.488895, 3.489064, 3.489240, 3.489413, 3.489575, 3.489726,
      3.489873, 3.490022, 3.490172, 3.490317, 3.490459, 3.490607, 3.490753,
      3.490894, 3.491049, 3.491222, 3.491405, 3.491589, 3.491767, 3.491942,
      3.492116, 3.492295, 3.492467, 3.492612, 3.492748, 3.492881, 3.493007,
      3.493142, 3.493294, 3.493453, 3.493615, 3.493775, 3.493927, 3.494085,
      3.494244, 3.494397, 3.494553, 3.494712, 3.494865, 3.495016, 3.495177,
      3.495352, 3.495530, 3.495697, 3.495849, 3.495995, 3.496145, 3.496299,
      3.496452, 3.496599, 3.496745, 3.496891, 3.497036, 3.497181, 3.497324,
      3.497465, 3.497632, 3.497830, 3.498026, 3.498212, 3.498400, 3.498595,
      3.498763, 3.498884, 3.499005, 3.499124, 3.499245, 3.499362, 3.499469,
      3.499581, 3.499693, 3.499797, 3.499904, 3.500030, 3.500147, 3.500265,
      3.500373, 3.500489, 3.500597, 3.500693, 3.500796, 3.500892, 3.500982,
      3.501071, 3.501160, 3.501242, 3.501316, 3.501397, 3.501470, 3.501531,
      3.501589, 3.501652, 3.501708, 3.501757, 3.501799, 3.501836, 3.501869,
      3.501899, 3.501928, 3.501957, 3.501983, 3.502007, 3.502027, 3.502044,
      3.502061, 3.502077, 3.502093, 3.502107, 3.502121, 3.502133, 3.502144,
      3.502154, 3.502163, 3.502171, 3.502178, 3.502185, 3.502190, 3.502196,
      3.502201, 3.502205, 3.502209, 3.502213, 3.502217, 3.502220, 3.502223,
      3.502226, 3.502229, 3.502232, 3.502234, 3.502236, 3.502238, 3.502239,
      3.502240, 3.502241, 3.502241, 3.502241, 3.502241, 3.502241, 3.502241,
      3.502240, 3.502240, 3.502240, 3.502240, 3.502240, 3.502240, 3.502241,
      3.502242, 3.502242, 3.502243, 3.502244, 3.502245, 3.502246, 3.502247,
      3.502248, 3.502249, 3.502250, 3.502250, 3.502251, 3.502251, 3.502251,
      3.502251, 3.502251, 3.502251, 3.502251, 3.502251, 3.502250, 3.502250,
      3.502250, 3.502250, 3.502250, 3.502250, 3.502250, 3.502250, 3.502251,
      3.502251, 3.502252, 3.502252, 3.502253, 3.502253, 3.502254, 3.502255,
      3.502256, 3.502257, 3.502258, 3.502259, 3.502260, 3.502261, 3.502262,
      3.502263, 3.502264, 3.502266, 3.502267, 3.502268, 3.502269, 3.502271,
      3.502272, 3.502273, 3.502274, 3.502276, 3.502277, 3.502278, 3.502279,
      3.502280, 3.502281, 3.502282, 3.502283, 3.502284, 3.502285, 3.502286,
      3.502287, 3.502288, 3.502289, 3.502290, 3.502292, 3.502293, 3.502294,
      3.502295, 3.502297, 3.502298, 3.502300, 3.502301, 3.502303, 3.502305,
      3.502307, 3.502309, 3.502311, 3.502313, 3.502315, 3.502316, 3.502318,
      3.502320, 3.502322, 3.502324, 3.502325, 3.502327, 3.502328, 3.502329,
      3.502330, 3.502331, 3.502331, 3.502332, 3.502332, 3.502333, 3.502333,
      3.502334, 3.502335, 3.502336, 3.502337, 3.502338, 3.502340, 3.502342,
      3.502345, 3.502347, 3.502350, 3.502353, 3.502356, 3.502359, 3.502362,
      3.502364, 3.502366, 3.502368, 3.502369, 3.502370, 3.502370, 3.502369,
      3.502369, 3.502369, 3.502370, 3.502373, 3.502375, 3.502378, 3.502380,
      3.502381, 3.502381, 3.502380};


// Baraffe stellar evolution grid
static double const STELLAR_BAR_MARR[STELLAR_BAR_MLEN] = {
      0.0698, 0.07, 0.072, 0.075, 0.08, 0.09, 0.1, 0.11, 0.13,
      0.15,   0.17, 0.2,   0.3,   0.4,  0.5,  0.6, 0.7,  0.8,
      0.9,    1.0,  1.1,   1.2,   1.3,  1.4,  1.41};
static double const STELLAR_BAR_AARR[STELLAR_BAR_ALEN] = {
      0.00099813710097553117,
      0.001,
      0.0010186289902446876,
      0.0010376050197669117,
      0.0010569345535579883,
      0.0010766241770454933,
      0.0010966805983368686,
      0.0011171106505048242,
      0.0011379212939153151,
      0.0011591196185988858,
      0.0011807128466661903,
      0.001202708334768512,
      0.0012251135766041189,
      0.0012479362054713115,
      0.0012711839968690289,
      0.0012948648711459049,
      0.0013189868961986705,
      0.0013435582902208262,
      0.0013685874245025192,
      0.0013940828262825786,
      0.0014200531816536831,
      0.0014465073385216472,
      0.0014734543096198358,
      0.0015009032755797366,
      0.0015288635880587311,
      0.0015573447729261352,
      0.0015863565335085897,
      0.0016159087538959191,
      0.0016460115023085497,
      0.0016766750345276991,
      0.0017079097973894269,
      0.0017397264323438008,
      0.0017721357790803584,
      0.0018051488792211082,
      0.0018387769800823268,
      0.0018730315385064365,
      0.0019079242247652651,
      0.0019434669265360202,
      0.0019796717529513325,
      0.0020165510387247466,
      0.0020541173483530641,
      0.0020923834803969771,
      0.0021313624718414378,
      0.0021710676025372649,
      0.0022115123997254889,
      0.0022527106426459805,
      0.0022946763672319361,
      0.0023374238708918148,
      0.0023809677173803581,
      0.0024253227417603529,
      0.0024705040554568254,
      0.0025165270514053918,
      0.0025634074092965148,
      0.0026111611009174596,
      0.0026598043955937583,
      0.0027093538657320514,
      0.0027598263924661806,
      0.0028112391714084643,
      0.0028636097185081159,
      0.0029169558760187959,
      0.0029712958185773341,
      0.0030266480593956922,
      0.0030830314565682771,
      0.0031404652194967522,
      0.0031989689154345376,
      0.0032585624761532262,
      0.0033192662047331896,
      0.0033811007824806854,
      0.0034440872759738234,
      0.0035082471442397923,
      0.0035736022460657883,
      0.0036401748474461414,
      0.0037079876291681727,
      0.0037770636945393646,
      0.0038474265772585059,
      0.0039191002494334019,
      0.0039921091297480532,
      0.0040664780917818539,
      0.0041422324724838975,
      0.0042193980808050242,
      0.0042980012064907985,
      0.0043780686290381668,
      0.0044596276268190956,
      0.0045427059863740425,
      0.004627332011878693,
      0.0047135345347869067,
      0.0048013429236534498,
      0.00489078709413959,
      0.0049818975192051597,
      0.0050747052394904662,
      0.0051692418738915986,
      0.0052655396303327552,
      0.0053636313167392397,
      0.0054635503522148755,
      0.0055653307784276464,
      0.0056690072712074343,
      0.0057746151523598195,
      0.0058821904016999561,
      0.0059917696693106192,
      0.0061033902880286211,
      0.0062170902861638268,
      0.0063329084004551143,
      0.0064508840892676927,
      0.0065710575460362708,
      0.0066934697129586605,
      0.0068181622949444794,
      0.0069451777738236964,
      0.0070745594228198783,
      0.0072063513212930527,
      0.0073405983697572121,
      0.0074773463051775882,
      0.0076166417165528921,
      0.007758532060787836,
      0.0079030656788613481,
      0.0080502918122959816,
      0.008200260619934131,
      0.0083530231950267788,
      0.0085086315826405825,
      0.0086671387973892341,
      0.0088285988414951496,
      0.0089930667231876236,
      0.0091605984754437094,
      0.0093312511750782492,
      0.0095050829621895119,
      0.0096821530599670753,
      0.0098625217948687845,
      0.010046250617173407,
      0.010233402121916417,
      0.010424040070215563,
      0.010618229410993841,
      0.0108160363031071,
      0.011017528137883868,
      0.011222773562085077,
      0.011431842501291497,
      0.011644806183726861,
      0.011861737164524785,
      0.012082709350447765,
      0.012307798025066652,
      0.012537079874409203,
      0.01277063301308644,
      0.013008537010905713,
      0.013250872919979532,
      0.013497723302339417,
      0.01374917225806419,
      0.014005305453932197,
      0.014266210152607369,
      0.014531975242368954,
      0.014802691267395087,
      0.015078450458610511,
      0.01535934676510897,
      0.01564547588616096,
      0.015936935303817745,
      0.016233824316122782,
      0.016536244070941803,
      0.016844297600423153,
      0.01715808985610005,
      0.017477727744646808,
      0.01780332016430114,
      0.018134978041964952,
      0.018472814370996342,
      0.018816944249705557,
      0.019167484920568151,
      0.019524555810168608,
      0.019888278569888097,
      0.02025877711735017,
      0.020636177678638588,
      0.021020608831301583,
      0.021412201548157293,
      0.021811089241915188,
      0.022217407810628836,
      0.022631295683995283,
      0.023052893870517073,
      0.023482346005542751,
      0.023919798400202384,
      0.024365400091254649,
      0.02481930289186254,
      0.025281661443314993,
      0.025752633267711999,
      0.026232378821631222,
      0.026721061550794338,
      0.027218847945751783,
      0.027725907598604826,
      0.028242413260784343,
      0.028768540901905927,
      0.029304469769721429,
      0.029850382451187308,
      0.030406464934670672,
      0.030972906673314083,
      0.031549900649580867,
      0.032137643441002769,
      0.03273633528715246,
      0.033346180157863642,
      0.033967385822722081,
      0.034600163921851085,
      0.035244730038015842,
      0.035901303770070683,
      0.036570108807774893,
      0.037251373008002096,
      0.03794532847236938,
      0.038652211626312619,
      0.039372263299634794,
      0.040105728808554959,
      0.040852858039285615,
      0.041613905533167073,
      0.042389130573387786,
      0.043178797273320212,
      0.043983174666502235,
      0.044802536798294899,
      0.045637162819247588,
      0.046487337080202563,
      0.047353349229171164,
      0.04823549431001467,
      0.049134072862963618,
      0.050049391027009532,
      0.050981760644204246,
      0.05193149936590212,
      0.052898930760981511,
      0.053884384426082239,
      0.054888196097896719,
      0.0559107077675529,
      0.056952267797128218,
      0.058013231038333751,
      0.05909395895340968,
      0.06019481973827271,
      0.06131618844795772,
      0.062458447124396146,
      0.063621984926574851,
      0.064807198263119675,
      0.066014490927348859,
      0.067244274234842458,
      0.068496967163574438,
      0.069772996496655346,
      0.07107279696773415,
      0.072396811409108738,
      0.073745490902595495,
      0.075119294933209652,
      0.076518691545708215,
      0.077944157504049469,
      0.079396178453822797,
      0.080875249087704534,
      0.08238187331399606,
      0.083916564428301574,
      0.085479845288404091,
      0.087072248492399168,
      0.088694316560147091,
      0.090346602118105246,
      0.0920296680876041,
      0.093744087876629911,
      0.095490445575180771,
      0.097269336154261685,
      0.099081365668586663,
      0.10092715146305709,
      0.10280732238308647,
      0.10472251898884344,
      0.10667339377348571,
      0.10866061138545968,
      0.1106848488549412,
      0.11274679582449461,
      0.1148471547840289,
      0.11698664131013069,
      0.11916598430985588,
      0.12138592626906278,
      0.12364722350537151,
      0.12595064642583575,
      0.12829697978941476,
      0.13068702297433463,
      0.13312159025043077,
      0.13560151105656335,
      0.13812763028320094,
      0.14070080856026851,
      0.14332192255035739,
      0.14599186524739785,
      0.14871154628089536,
      0.15148189222583458,
      0.15430384691835647,
      0.15717837177731628,
      0.16010644613183178,
      0.16308906755493327,
      0.1661272522034293,
      0.16922203516410372,
      0.17237447080636198,
      0.17558563314144687,
      0.1788566161883462,
      0.18218853434651652,
      0.18558252277555171,
      0.18903973778192196,
      0.19256135721291967,
      0.19614858085794293,
      0.19980263085725486,
      0.20352475211835758,
      0.20731621274012291,
      0.21117830444482422,
      0.2151123430182165,
      0.21911966875781475,
      0.22320164692952324,
      0.22735966823277151,
      0.23159514927431521,
      0.23590953305086337,
      0.24030428944069671,
      0.24478091570444402,
      0.24934093699518778,
      0.25398590687807238,
      0.25871740785959213,
      0.26353705192673932,
      0.26844648109619623,
      0.27344736797375796,
      0.27854141632417662,
      0.28373036165162113,
      0.28901597179095084,
      0.29440004751000337,
      0.29988442312310276,
      0.30547096711599681,
      0.311161582782436,
      0.3169582088726115,
      0.32286282025367302,
      0.32887742858255103,
      0.33500408299131329,
      0.34124487078528892,
      0.34760191815419778,
      0.35407739089652696,
      0.36067349515740282,
      0.36739247818020743,
      0.37423662907219807,
      0.3812082795843888,
      0.38830980490596051,
      0.39554362447347013,
      0.40291220279513479,
      0.4104180502904709,
      0.4180637241455758,
      0.42585182918434145,
      0.43378501875589892,
      0.44186599563859413,
      0.45009751296080464,
      0.4584823751389096,
      0.46702343883273306,
      0.47572361391878848,
      0.48458586448164909,
      0.4936132098237912,
      0.50280872549424749,
      0.51217554433642365,
      0.5217168575554344,
      0.53143591580532357,
      0.54133603029653754,
      0.55142057392402966,
      0.56169298241638044,
      0.57215675550632461,
      0.582815458123084,
      0.59367272160691198,
      0.60473224494626432,
      0.61599779603801619,
      0.62747321297115743,
      0.63916240533439994,
      0.65106935554814549,
      0.6631981202212669,
      0.67555283153316392,
      0.68813769864156626,
      0.70095700911656178,
      0.71401513040133957,
      0.72731651130014541,
      0.74086568349395598,
      0.7546672630843887,
      0.76872595216637285,
      0.78304654043011834,
      0.79763390679292734,
      0.81249302106140475,
      0.8276289456246344,
      0.84304683717889672,
      0.85875194848451708,
      0.87474963015544149,
      0.89104533248215134,
      0.90764460728853569,
      0.92455310982335703,
      0.9417766006869519,
      0.95932094779382415,
      0.9771921283717997,
      0.99539623099842334,
      1.0139394576752918,
      1.0328281259410286,
      1.0520686710236229,
      1.0716676480328633,
      1.0916317341936148,
      1.111967731120699,
      1.1326825671361538,
      1.1537832996296609,
      1.1752771174629453,
      1.1971713434189657,
      1.219473436696737,
      1.2421909954526162,
      1.2653317593889417,
      1.2889036123908915,
      1.312914585212464,
      1.3373728582124951,
      1.3622867641416454,
      1.3876647909813071,
      1.4135155848353942,
      1.4398479528760066,
      1.4666708663439672,
      1.4939934636052563,
      1.5218250532643856,
      1.5501751173357687,
      1.5790533144741743,
      1.6084694832653552,
      1.6384336455779831,
      1.6689560099780232,
      1.7000469752067164,
      1.7317171337233528,
      1.7639772753140435,
      1.7968383907677192,
      1.8303116756206113,
      1.8644085339704852,
      1.8991405823619336,
      1.9345196537440443,
      1.9705578015017984,
      2.0072673035625681,
      2.0446606665791158,
      2.0827506301905143,
      2.1215501713624501,
      2.1610725088083766,
      2.2013311074930302,
      2.242339683219845,
      2.2841122073038234,
      2.3266629113314581,
      2.3700062920093279,
      2.4141571161030178,
      2.4591304254680439,
      2.5049415421745023,
      2.551606073727184,
      2.5991399183829329,
      2.6475592705670667,
      2.6968806263906924,
      2.7471207892708116,
      2.7982968756551156,
      2.8504263208534346,
      2.903526884977814,
      2.9576166589932535,
      3.0127140708811644,
      3.0688378919176422,
      3.1260072430687003,
      3.1842416015046497,
      3.243560807235808,
      3.3039850698718549,
      3.3655349755070909,
      3.4282314937339673,
      3.4920959847872677,
      3.5571502068213823,
      3.6234163233231462,
      3.6909169106627746,
      3.7596749657854636,
      3.8297139140462773,
      3.9010576171909892,
      3.9737303814856042,
      4.0477569659973183,
      4.1231625910297485,
      4.1999729467153015,
      4.2782142017676126,
      4.3579130123970256,
      4.4390965313921669,
      4.5217924173706976,
      4.606028844202398,
      4.6918345106077943,
      4.7792386499355954,
      4.8682710401222788,
      4.9589620138372119,
      5.0513424688167614,
      5.1454438783909247,
      5.2412983022060562,
      5.3389383971473503,
      5.4383974284647962,
      5.5397092811064006,
      5.6429084712625368,
      5.748030158125351,
      5.855110155867238,
      5.9641849458424598,
      6.0752916890160709,
      6.1884682386243828,
      6.3037531530712752,
      6.4211857090647584,
      6.5408059149982529,
      6.6626545245811499,
      6.7867730507232951,
      6.9132037796781276,
      7.0419897854492888,
      7.1731749444656128,
      7.3068039505294999,
      7.4429223300437579,
      7.5815764575221101,
      7.7228135713886426,
      7.8666817900715822,
      8.013230128396879,
      8.1625085142872198,
      8.3145678057720556,
      8.4694598083145767,
      8.6272372924614409,
      8.7879540118213111,
      8.9516647213782932,
      9.1184251961465623,
      9.2882922501724892,
      9.4613237558907599,
      9.6375786638410794,
      9.8171170227521838,
      10.0,
      10.018288297724782};
static const int STELLAR_BICUBIC_MATRIX[16][16] = {
      {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {-3, 3, 0, 0, -2, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2, -2, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, -3, 3, 0, 0, -2, -1, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 2, -2, 0, 0, 1, 1, 0, 0},
      {-3, 0, 3, 0, 0, 0, 0, 0, -2, 0, -1, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, -3, 0, 3, 0, 0, 0, 0, 0, -2, 0, -1, 0},
      {9, -9, -9, 9, 6, 3, -6, -3, 6, -6, 3, -3, 4, 2, 2, 1},
      {-6, 6, 6, -6, -3, -3, 3, 3, -4, 4, -2, 2, -2, -2, -1, -1},
      {2, 0, -2, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 2, 0, -2, 0, 0, 0, 0, 0, 1, 0, 1, 0},
      {-6, 6, 6, -6, -4, -2, 4, 2, -3, 3, -3, 3, -2, -1, -2, -1},
      {4, -4, -4, 4, 2, 2, -2, -2, 2, -2, 2, -2, 1, 1, 1, 1}};

// Baraffe+2015 log10 luminosity interpolation grid
static double const DATA_LOGL[STELLAR_BAR_MLEN][STELLAR_BAR_ALEN] = {
      -1.414042, -1.414200, -1.415501, -1.416179, -1.416948, -1.418311,
      -1.419831, -1.421171, -1.422398, -1.423563, -1.424556, -1.425607,
      -1.426942, -1.428276, -1.429080, -1.429793, -1.430530, -1.431328,
      -1.432126, -1.433302, -1.434647, -1.435622, -1.436041, -1.436609,
      -1.437611, -1.438628, -1.439700, -1.440796, -1.441959, -1.443209,
      -1.444413, -1.444988, -1.445934, -1.447353, -1.448821, -1.449801,
      -1.450836, -1.452428, -1.454098, -1.455070, -1.456655, -1.458620,
      -1.459939, -1.461693, -1.463032, -1.465081, -1.467415, -1.469071,
      -1.470965, -1.473373, -1.475502, -1.477643, -1.481076, -1.483636,
      -1.486453, -1.489495, -1.491966, -1.495887, -1.499545, -1.503483,
      -1.507801, -1.511877, -1.516329, -1.521973, -1.527561, -1.533326,
      -1.539711, -1.547037, -1.555546, -1.564115, -1.572884, -1.582386,
      -1.593148, -1.604622, -1.615938, -1.627298, -1.638798, -1.650559,
      -1.661597, -1.672361, -1.682919, -1.693302, -1.703568, -1.713642,
      -1.723566, -1.733375, -1.743083, -1.752920, -1.762197, -1.770877,
      -1.779452, -1.788715, -1.797534, -1.805739, -1.814186, -1.822528,
      -1.830433, -1.838757, -1.846596, -1.854766, -1.862674, -1.870044,
      -1.878331, -1.885718, -1.893858, -1.902291, -1.910219, -1.919083,
      -1.927145, -1.934967, -1.942881, -1.950897, -1.958586, -1.966161,
      -1.974030, -1.982182, -1.989616, -1.996987, -2.004307, -2.012009,
      -2.019800, -2.026974, -2.034087, -2.041152, -2.048560, -2.056152,
      -2.063085, -2.069973, -2.076825, -2.083862, -2.090917, -2.097894,
      -2.105422, -2.112064, -2.118663, -2.125267, -2.131837, -2.139018,
      -2.145564, -2.151888, -2.158677, -2.165904, -2.172252, -2.179137,
      -2.185534, -2.191849, -2.198189, -2.204500, -2.211247, -2.217958,
      -2.223821, -2.230098, -2.237071, -2.243081, -2.249810, -2.256411,
      -2.262557, -2.268628, -2.275100, -2.281333, -2.287552, -2.294351,
      -2.299989, -2.306785, -2.313145, -2.319182, -2.325626, -2.331593,
      -2.338068, -2.344332, -2.350246, -2.356814, -2.362738, -2.368885,
      -2.374931, -2.380886, -2.387103, -2.392917, -2.399254, -2.406084,
      -2.413190, -2.419605, -2.425966, -2.432308, -2.438565, -2.444846,
      -2.451135, -2.457406, -2.463669, -2.470471, -2.477248, -2.483464,
      -2.489669, -2.495860, -2.501974, -2.508358, -2.515218, -2.521533,
      -2.527652, -2.533734, -2.540083, -2.546838, -2.553028, -2.559746,
      -2.566178, -2.572496, -2.579174, -2.585283, -2.591859, -2.598206,
      -2.604470, -2.611004, -2.617020, -2.623513, -2.629817, -2.635961,
      -2.642412, -2.649378, -2.655267, -2.661666, -2.667871, -2.673930,
      -2.680293, -2.687155, -2.693576, -2.699977, -2.706715, -2.713016,
      -2.719009, -2.725447, -2.731832, -2.738110, -2.744914, -2.751208,
      -2.757928, -2.764382, -2.770982, -2.777761, -2.784156, -2.790416,
      -2.797130, -2.803800, -2.810201, -2.817230, -2.824188, -2.830830,
      -2.837483, -2.844124, -2.850648, -2.857802, -2.865043, -2.872334,
      -2.879400, -2.886057, -2.893991, -2.900700, -2.907533, -2.914819,
      -2.922371, -2.929456, -2.936637, -2.943381, -2.950131, -2.957377,
      -2.964619, -2.971978, -2.978650, -2.985313, -2.992505, -2.999737,
      -3.006988, -3.014205, -3.021429, -3.028542, -3.034909, -3.042038,
      -3.049238, -3.056824, -3.064760, -3.071974, -3.079199, -3.086358,
      -3.093531, -3.100757, -3.108365, -3.115897, -3.123190, -3.130288,
      -3.137539, -3.144795, -3.152526, -3.160307, -3.167506, -3.174663,
      -3.182322, -3.189959, -3.197021, -3.204112, -3.212098, -3.220128,
      -3.228155, -3.236084, -3.242824, -3.250597, -3.259365, -3.267154,
      -3.274879, -3.282524, -3.290122, -3.298458, -3.306216, -3.313979,
      -3.321944, -3.330127, -3.338485, -3.346470, -3.353981, -3.362182,
      -3.370404, -3.378494, -3.386956, -3.395456, -3.403645, -3.411584,
      -3.419767, -3.428250, -3.436014, -3.443724, -3.452231, -3.460774,
      -3.469213, -3.477596, -3.485961, -3.494199, -3.502454, -3.510608,
      -3.518701, -3.526764, -3.534804, -3.542814, -3.550802, -3.559182,
      -3.567811, -3.575729, -3.583615, -3.591489, -3.599685, -3.608279,
      -3.616028, -3.623755, -3.631956, -3.640335, -3.648041, -3.655718,
      -3.663896, -3.672066, -3.679662, -3.688093, -3.697074, -3.706851,
      -3.717647, -3.728243, -3.738800, -3.749499, -3.760671, -3.770965,
      -3.781042, -3.790993, -3.800691, -3.808990, -3.816316, -3.823619,
      -3.830896, -3.838129, -3.845399, -3.852675, -3.859953, -3.867227,
      -3.874507, -3.881893, -3.889286, -3.896638, -3.905273, -3.920080,
      -3.954381, -3.995088, -4.023928, -4.044514, -4.062519, -4.079737,
      -4.095006, -4.109984, -4.120168, -4.124849, -4.129680, -4.133780,
      -4.138568, -4.141574, -4.143039, -4.144940, -4.147904, -4.151094,
      -4.155384, -4.159805, -4.163336, -4.166994, -4.170863, -4.174841,
      -4.178962, -4.182983, -4.186494, -4.190571, -4.193984, -4.198243,
      -4.202212, -4.205960, -4.209384, -4.212602, -4.216561, -4.219665,
      -4.223999, -4.229257, -4.235006, -4.239855, -4.245280, -4.250010,
      -4.254365, -4.258450, -4.262578, -4.266054, -4.269478, -4.272997,
      -4.276403, -4.279093, -4.281914, -4.284786, -4.287634, -4.290024,
      -4.292043, -4.294003, -4.296100, -4.297663, -4.299007, -4.300194,
      -4.301630, -4.302844, -4.304359, -4.305575, -4.306407, -4.307484,
      -4.308316, -4.309175, -4.310048, -4.310600, -4.310740, -4.311661,
      -4.311642, -4.312172, -4.312700, -4.312820, -4.313579, -4.313800,
      -4.313800, -4.313800, -4.313800, -4.313800, -4.314587, -4.314900,
      -4.314900, -4.314900, -4.314900, -4.314900, -4.314900, -4.314900,
      -4.314900, -4.314900, -4.314900, -4.314900, -4.314900, -4.314900,
      -4.314900, -4.314900, -4.314900, -4.314900, -4.314900, -4.314900,
      -4.314900, -4.314900, -4.314900, -4.314900, -4.314900, -4.314900,
      -4.314900, -4.314900, -4.314900, -4.314900, -4.314900, -4.314900,
      -4.314900, -4.314900, NAN,       NAN,       -1.411800, -1.411950,
      -1.413222, -1.413979, -1.414750, -1.416062, -1.417541, -1.418884,
      -1.420053, -1.421177, -1.422171, -1.423215, -1.424509, -1.425802,
      -1.426638, -1.427390, -1.428152, -1.428954, -1.429755, -1.430907,
      -1.432211, -1.433185, -1.433654, -1.434264, -1.435270, -1.436295,
      -1.437371, -1.438475, -1.439621, -1.440815, -1.442017, -1.442662,
      -1.443650, -1.445015, -1.446456, -1.447473, -1.448490, -1.450093,
      -1.451774, -1.452764, -1.454356, -1.456264, -1.457629, -1.459347,
      -1.460767, -1.462827, -1.465121, -1.466788, -1.468732, -1.471144,
      -1.473283, -1.475450, -1.478812, -1.481399, -1.484223, -1.487312,
      -1.489850, -1.493731, -1.497398, -1.501388, -1.505728, -1.509835,
      -1.514404, -1.520037, -1.525658, -1.531506, -1.537963, -1.545303,
      -1.553796, -1.562414, -1.571277, -1.580862, -1.591681, -1.603126,
      -1.614489, -1.625898, -1.637410, -1.649103, -1.660120, -1.670874,
      -1.681466, -1.691898, -1.702157, -1.712225, -1.722142, -1.731918,
      -1.741549, -1.751347, -1.760646, -1.769377, -1.777978, -1.787163,
      -1.795980, -1.804176, -1.812603, -1.820960, -1.828864, -1.837184,
      -1.845017, -1.853126, -1.861022, -1.868450, -1.876652, -1.884097,
      -1.892212, -1.900591, -1.908604, -1.917386, -1.925428, -1.933247,
      -1.941144, -1.949137, -1.956846, -1.964431, -1.972295, -1.980448,
      -1.987880, -1.995250, -2.002569, -2.010265, -2.018061, -2.025234,
      -2.032348, -2.039413, -2.046817, -2.054415, -2.061348, -2.068236,
      -2.075088, -2.082121, -2.089180, -2.096136, -2.103589, -2.110294,
      -2.116928, -2.123532, -2.130103, -2.137271, -2.143815, -2.150153,
      -2.156912, -2.164068, -2.170483, -2.177317, -2.183756, -2.190111,
      -2.196449, -2.202760, -2.209502, -2.216181, -2.222079, -2.228353,
      -2.235256, -2.241316, -2.248007, -2.254568, -2.260713, -2.266835,
      -2.273274, -2.279535, -2.285790, -2.292522, -2.298240, -2.304968,
      -2.311296, -2.317330, -2.323778, -2.329747, -2.336219, -2.342456,
      -2.348397, -2.354966, -2.360856, -2.367038, -2.373090, -2.379042,
      -2.385264, -2.391079, -2.397414, -2.404209, -2.411252, -2.417665,
      -2.424028, -2.430370, -2.436696, -2.443010, -2.449300, -2.455576,
      -2.461841, -2.468593, -2.475321, -2.481537, -2.487742, -2.493934,
      -2.500106, -2.506509, -2.513305, -2.519604, -2.525723, -2.531837,
      -2.538226, -2.544921, -2.551101, -2.557760, -2.564193, -2.570541,
      -2.577161, -2.583262, -2.589840, -2.596191, -2.602457, -2.608993,
      -2.615013, -2.621508, -2.627816, -2.633963, -2.640417, -2.647298,
      -2.653279, -2.659681, -2.665888, -2.671952, -2.678318, -2.685096,
      -2.691507, -2.697914, -2.704652, -2.710955, -2.716956, -2.723403,
      -2.729811, -2.736092, -2.742851, -2.749128, -2.755856, -2.762308,
      -2.768889, -2.775658, -2.782015, -2.788337, -2.795029, -2.801677,
      -2.808094, -2.815070, -2.821978, -2.828663, -2.835333, -2.841955,
      -2.848456, -2.855620, -2.862792, -2.870013, -2.877058, -2.883749,
      -2.891547, -2.898283, -2.905155, -2.912414, -2.919866, -2.926967,
      -2.934155, -2.940947, -2.947743, -2.954987, -2.962219, -2.969500,
      -2.976200, -2.982914, -2.990104, -2.997335, -3.004519, -3.011710,
      -3.018932, -3.026057, -3.032502, -3.039639, -3.046839, -3.054401,
      -3.062299, -3.069485, -3.076728, -3.083890, -3.091033, -3.098276,
      -3.105866, -3.113383, -3.120682, -3.127804, -3.135061, -3.142320,
      -3.150009, -3.157743, -3.164945, -3.172105, -3.179720, -3.187311,
      -3.194379, -3.201469, -3.209461, -3.217398, -3.225331, -3.233172,
      -3.240018, -3.247799, -3.256481, -3.264224, -3.271906, -3.279548,
      -3.287150, -3.295418, -3.303222, -3.310989, -3.318909, -3.327026,
      -3.335334, -3.343314, -3.350830, -3.359015, -3.367154, -3.375243,
      -3.383665, -3.392121, -3.400231, -3.408133, -3.416320, -3.424796,
      -3.432558, -3.440266, -3.448692, -3.457147, -3.465546, -3.473892,
      -3.482184, -3.490416, -3.498587, -3.506710, -3.514792, -3.522843,
      -3.530867, -3.538856, -3.546821, -3.555137, -3.563676, -3.571564,
      -3.579419, -3.587261, -3.595392, -3.603881, -3.611645, -3.619382,
      -3.627503, -3.635783, -3.643448, -3.651084, -3.659173, -3.667304,
      -3.674892, -3.683201, -3.692005, -3.701531, -3.711992, -3.722319,
      -3.732559, -3.742911, -3.753690, -3.763720, -3.773582, -3.783366,
      -3.793018, -3.801404, -3.808897, -3.816353, -3.823795, -3.831225,
      -3.838637, -3.846030, -3.853410, -3.860767, -3.868102, -3.875421,
      -3.882713, -3.889981, -3.898390, -3.912394, -3.944133, -3.981666,
      -4.008407, -4.027708, -4.044622, -4.060816, -4.075239, -4.089346,
      -4.099164, -4.103962, -4.108849, -4.113036, -4.117928, -4.122586,
      -4.126976, -4.131447, -4.135801, -4.139640, -4.144367, -4.149145,
      -4.153068, -4.157066, -4.161139, -4.165289, -4.169515, -4.173616,
      -4.177206, -4.181255, -4.184667, -4.188855, -4.192678, -4.196305,
      -4.199640, -4.202715, -4.206515, -4.209396, -4.213363, -4.218192,
      -4.223480, -4.227952, -4.232903, -4.237268, -4.241241, -4.244985,
      -4.248798, -4.251958, -4.255132, -4.258361, -4.261485, -4.263994,
      -4.266558, -4.269169, -4.271826, -4.274022, -4.275864, -4.277730,
      -4.279636, -4.281058, -4.282279, -4.283384, -4.284751, -4.285858,
      -4.287236, -4.288341, -4.289113, -4.290160, -4.290924, -4.291704,
      -4.292498, -4.293000, -4.293127, -4.293965, -4.294000, -4.294520,
      -4.295000, -4.295109, -4.295799, -4.296000, -4.296000, -4.296000,
      -4.296000, -4.296000, -4.296715, -4.297000, -4.297000, -4.297000,
      -4.297000, -4.297000, -4.297000, -4.297000, -4.297000, -4.297000,
      -4.297000, -4.297000, -4.297000, -4.297000, -4.297000, -4.297000,
      -4.297000, -4.297000, -4.297000, -4.297000, -4.297000, -4.297000,
      -4.297000, -4.297000, -4.297000, -4.297000, -4.297000, -4.297000,
      -4.297000, -4.297000, -4.297000, -4.297000, -4.297000, -4.297000,
      -4.297000, NAN,       -1.389374, -1.389451, -1.390431, -1.391979,
      -1.392777, -1.393575, -1.394643, -1.396006, -1.396594, -1.397313,
      -1.398327, -1.399295, -1.400173, -1.401068, -1.402216, -1.403365,
      -1.404373, -1.405207, -1.406044, -1.406948, -1.407852, -1.408815,
      -1.409788, -1.410817, -1.411860, -1.412965, -1.414083, -1.415268,
      -1.416240, -1.416869, -1.418053, -1.419405, -1.420811, -1.421641,
      -1.422805, -1.424192, -1.425025, -1.426747, -1.428540, -1.429705,
      -1.431361, -1.432701, -1.434535, -1.435885, -1.438116, -1.440295,
      -1.442186, -1.443956, -1.446403, -1.448852, -1.451092, -1.453522,
      -1.456171, -1.459033, -1.461916, -1.465487, -1.468688, -1.472167,
      -1.475926, -1.480435, -1.484996, -1.489415, -1.495155, -1.500673,
      -1.506631, -1.513304, -1.520473, -1.527966, -1.536302, -1.545409,
      -1.555212, -1.565617, -1.577006, -1.588159, -1.599999, -1.611901,
      -1.623531, -1.634540, -1.645353, -1.656007, -1.666927, -1.677850,
      -1.688049, -1.698047, -1.707903, -1.717344, -1.726213, -1.735613,
      -1.745138, -1.754376, -1.763234, -1.771645, -1.780435, -1.788544,
      -1.796777, -1.805283, -1.813175, -1.821459, -1.829233, -1.836734,
      -1.844507, -1.852511, -1.859862, -1.867884, -1.875751, -1.883592,
      -1.892451, -1.900413, -1.908259, -1.916048, -1.923780, -1.931541,
      -1.939436, -1.947134, -1.954939, -1.963106, -1.970518, -1.977874,
      -1.985183, -1.992831, -2.000667, -2.007839, -2.014954, -2.022025,
      -2.029393, -2.037041, -2.043978, -2.050869, -2.057722, -2.064713,
      -2.071812, -2.078558, -2.085265, -2.092594, -2.099574, -2.106183,
      -2.112758, -2.119804, -2.126317, -2.132802, -2.139264, -2.145705,
      -2.152793, -2.159120, -2.165972, -2.172723, -2.179053, -2.185357,
      -2.192052, -2.198412, -2.204663, -2.210897, -2.217105, -2.223669,
      -2.229975, -2.236134, -2.242276, -2.248901, -2.255007, -2.261551,
      -2.268173, -2.274229, -2.280748, -2.286797, -2.292808, -2.298808,
      -2.305296, -2.311291, -2.317729, -2.323692, -2.329910, -2.336481,
      -2.342034, -2.348571, -2.354683, -2.360601, -2.366879, -2.372706,
      -2.379015, -2.385456, -2.391868, -2.398266, -2.404640, -2.410992,
      -2.417999, -2.424647, -2.430957, -2.437275, -2.443561, -2.449808,
      -2.456048, -2.462267, -2.468475, -2.474670, -2.481431, -2.488015,
      -2.494166, -2.500310, -2.506437, -2.512869, -2.519659, -2.525752,
      -2.531834, -2.537898, -2.544348, -2.550998, -2.557030, -2.563054,
      -2.569655, -2.576045, -2.582323, -2.588887, -2.594941, -2.601463,
      -2.607804, -2.613983, -2.620471, -2.626495, -2.633401, -2.639837,
      -2.646060, -2.652174, -2.658572, -2.664505, -2.670819, -2.677284,
      -2.684016, -2.690344, -2.696429, -2.702961, -2.709606, -2.715920,
      -2.722229, -2.728331, -2.735129, -2.741568, -2.747958, -2.754620,
      -2.760608, -2.767548, -2.774019, -2.780439, -2.787022, -2.793470,
      -2.799876, -2.806984, -2.813835, -2.820261, -2.826530, -2.833803,
      -2.840283, -2.846806, -2.853638, -2.860671, -2.867101, -2.874119,
      -2.881379, -2.888363, -2.894817, -2.902085, -2.909329, -2.916598,
      -2.923864, -2.931083, -2.938215, -2.944725, -2.951699, -2.958923,
      -2.966088, -2.973310, -2.979827, -2.986756, -2.993961, -3.001204,
      -3.008436, -3.015654, -3.022852, -3.030177, -3.037690, -3.044602,
      -3.052016, -3.059209, -3.066060, -3.073474, -3.080874, -3.088253,
      -3.095608, -3.102955, -3.110279, -3.117574, -3.124842, -3.132103,
      -3.139332, -3.146527, -3.153698, -3.160829, -3.167959, -3.175044,
      -3.183090, -3.190093, -3.197095, -3.204052, -3.211959, -3.219821,
      -3.227637, -3.234920, -3.242176, -3.249795, -3.257424, -3.265018,
      -3.273275, -3.281082, -3.288564, -3.296010, -3.303826, -3.311750,
      -3.319321, -3.327338, -3.334645, -3.342740, -3.350757, -3.358773,
      -3.366089, -3.373620, -3.381849, -3.390255, -3.397996, -3.405680,
      -3.413304, -3.420874, -3.428879, -3.436856, -3.444409, -3.452585,
      -3.459917, -3.467726, -3.475709, -3.483633, -3.491494, -3.499279,
      -3.507009, -3.514689, -3.522325, -3.529911, -3.537460, -3.544975,
      -3.552457, -3.559891, -3.567816, -3.575649, -3.582980, -3.590259,
      -3.597514, -3.604742, -3.611934, -3.619689, -3.627198, -3.634284,
      -3.641320, -3.648327, -3.655438, -3.663072, -3.670151, -3.677030,
      -3.683879, -3.691261, -3.698976, -3.707090, -3.716286, -3.725548,
      -3.734701, -3.743689, -3.752780, -3.762181, -3.771012, -3.779585,
      -3.787981, -3.796170, -3.804051, -3.810705, -3.816982, -3.823414,
      -3.829561, -3.835530, -3.841651, -3.847444, -3.853193, -3.859649,
      -3.865652, -3.871600, -3.877566, -3.882973, -3.889125, -3.895092,
      -3.900542, -3.905606, -3.911530, -3.932698, -3.966347, -3.996523,
      -4.014774, -4.025094, -4.034193, -4.042543, -4.050390, -4.057790,
      -4.063899, -4.069768, -4.075049, -4.079943, -4.084328, -4.088094,
      -4.091504, -4.094976, -4.097346, -4.099752, -4.102201, -4.103847,
      -4.106059, -4.106708, -4.107000, -4.107539, -4.108222, -4.108917,
      -4.109127, -4.109850, -4.110000, -4.110338, -4.111000, -4.111000,
      -4.111673, -4.112000, -4.112302, -4.113000, -4.113000, -4.113000,
      -4.113748, -4.114000, -4.114068, -4.115000, -4.115000, -4.115000,
      -4.115000, -4.115286, -4.115968, -4.116000, -4.116000, -4.116000,
      -4.116170, -4.116920, -4.117000, -4.117000, -4.117000, -4.117000,
      -4.117000, -4.117000, -4.117581, -4.118000, -4.118000, -4.118000,
      -4.118000, -4.118000, -4.118000, -4.118000, -4.118000, -4.118000,
      -4.118000, -4.118000, -4.118000, -4.118000, -4.118000, -4.118000,
      -4.118000, -4.118000, -4.118000, -4.118000, -4.118000, -4.118000,
      -4.118000, -4.118000, -4.118000, -4.118000, -4.118000, -4.118000,
      -4.118000, -4.118000, -4.118000, -4.118000, -4.118000, -4.118000,
      -4.118000, -4.118000, -4.118000, -4.118000, -4.118000, -4.118000,
      -4.118000, -4.118000, -4.118000, -4.118000, NAN,       NAN,
      -1.359870, -1.360026, -1.361292, -1.362081, -1.362883, -1.364393,
      -1.365437, -1.366115, -1.366713, -1.367534, -1.368564, -1.369514,
      -1.370406, -1.371267, -1.372068, -1.372868, -1.373364, -1.373800,
      -1.374512, -1.375455, -1.376214, -1.376721, -1.377489, -1.378574,
      -1.379703, -1.380861, -1.381541, -1.382331, -1.383634, -1.384991,
      -1.386195, -1.386922, -1.388369, -1.389474, -1.390588, -1.392295,
      -1.394037, -1.394968, -1.396880, -1.397956, -1.400019, -1.401270,
      -1.403305, -1.405067, -1.407645, -1.409364, -1.411619, -1.414133,
      -1.416481, -1.418445, -1.421257, -1.424259, -1.427037, -1.429999,
      -1.433453, -1.437570, -1.441254, -1.445067, -1.449218, -1.454129,
      -1.459466, -1.464776, -1.470583, -1.477118, -1.483877, -1.491251,
      -1.499623, -1.508750, -1.518563, -1.529187, -1.540030, -1.550943,
      -1.562200, -1.574033, -1.585408, -1.596648, -1.607906, -1.619316,
      -1.630059, -1.640396, -1.650594, -1.660634, -1.670871, -1.680610,
      -1.690223, -1.699509, -1.708154, -1.717636, -1.726722, -1.735336,
      -1.743604, -1.752607, -1.760562, -1.769002, -1.777228, -1.785236,
      -1.793500, -1.801067, -1.808521, -1.816382, -1.824229, -1.831492,
      -1.838696, -1.846397, -1.854177, -1.862096, -1.869952, -1.877742,
      -1.886193, -1.894151, -1.901777, -1.909436, -1.917234, -1.924849,
      -1.932277, -1.939659, -1.946988, -1.954266, -1.962003, -1.969700,
      -1.976859, -1.983971, -1.991041, -1.998072, -2.005295, -2.013020,
      -2.019425, -2.026333, -2.033915, -2.040911, -2.047665, -2.054385,
      -2.061061, -2.068231, -2.075326, -2.081917, -2.088185, -2.095438,
      -2.101985, -2.108451, -2.114894, -2.121305, -2.128192, -2.134550,
      -2.140888, -2.147342, -2.154341, -2.160286, -2.166542, -2.173202,
      -2.180007, -2.186211, -2.191912, -2.198742, -2.205222, -2.211323,
      -2.217203, -2.223988, -2.230177, -2.236005, -2.242235, -2.248858,
      -2.254886, -2.261335, -2.267396, -2.273392, -2.279392, -2.285850,
      -2.291792, -2.297721, -2.303711, -2.310042, -2.315934, -2.322345,
      -2.328494, -2.334369, -2.340863, -2.346338, -2.352799, -2.359241,
      -2.365671, -2.372072, -2.378458, -2.384825, -2.391165, -2.397503,
      -2.403842, -2.410138, -2.416405, -2.422662, -2.428896, -2.435146,
      -2.442327, -2.448516, -2.454693, -2.460860, -2.467010, -2.473147,
      -2.479276, -2.485388, -2.491593, -2.498585, -2.504665, -2.510737,
      -2.516793, -2.523013, -2.529880, -2.535907, -2.541924, -2.548418,
      -2.554923, -2.560904, -2.566875, -2.572837, -2.579259, -2.585721,
      -2.591807, -2.598311, -2.604458, -2.610778, -2.617225, -2.623198,
      -2.629634, -2.635892, -2.642083, -2.648398, -2.654916, -2.660523,
      -2.666759, -2.673115, -2.679894, -2.686692, -2.692872, -2.698673,
      -2.705312, -2.711770, -2.718098, -2.724369, -2.730624, -2.736938,
      -2.743397, -2.750162, -2.756977, -2.763475, -2.769885, -2.776309,
      -2.782792, -2.789265, -2.795647, -2.802016, -2.809123, -2.815840,
      -2.822278, -2.828814, -2.835139, -2.842309, -2.849068, -2.855482,
      -2.861961, -2.869148, -2.875700, -2.882101, -2.888799, -2.895915,
      -2.902383, -2.909511, -2.916714, -2.923931, -2.931096, -2.937912,
      -2.944989, -2.951707, -2.958345, -2.965445, -2.972889, -2.980017,
      -2.986656, -2.993620, -3.000855, -3.008337, -3.015461, -3.022389,
      -3.029715, -3.037149, -3.044564, -3.051463, -3.058353, -3.065720,
      -3.073060, -3.080379, -3.087691, -3.094691, -3.101225, -3.108458,
      -3.115677, -3.122859, -3.130011, -3.137131, -3.144225, -3.151297,
      -3.158329, -3.165327, -3.172515, -3.179935, -3.187105, -3.193955,
      -3.201164, -3.208713, -3.215917, -3.222960, -3.229837, -3.237381,
      -3.244870, -3.252307, -3.259692, -3.267012, -3.274328, -3.281581,
      -3.288780, -3.295927, -3.303020, -3.310317, -3.318052, -3.324998,
      -3.332747, -3.339924, -3.346783, -3.353944, -3.361456, -3.368954,
      -3.376353, -3.383679, -3.390944, -3.398146, -3.405286, -3.412355,
      -3.419346, -3.426750, -3.434142, -3.440947, -3.447663, -3.454797,
      -3.461912, -3.468440, -3.475533, -3.482623, -3.489584, -3.495818,
      -3.502430, -3.509225, -3.515929, -3.522562, -3.529147, -3.535654,
      -3.542105, -3.548517, -3.554846, -3.561130, -3.567367, -3.573520,
      -3.579639, -3.585689, -3.591679, -3.597633, -3.603492, -3.609323,
      -3.615080, -3.620788, -3.626445, -3.632025, -3.637576, -3.643031,
      -3.648440, -3.653452, -3.658512, -3.663689, -3.668958, -3.674302,
      -3.679260, -3.684193, -3.689238, -3.694380, -3.699515, -3.704188,
      -3.708940, -3.713872, -3.719509, -3.725786, -3.732090, -3.738022,
      -3.744100, -3.749745, -3.755246, -3.760724, -3.765615, -3.770608,
      -3.775691, -3.780864, -3.785777, -3.790249, -3.794803, -3.799353,
      -3.803133, -3.806985, -3.810909, -3.814908, -3.817985, -3.821064,
      -3.823178, -3.825496, -3.828524, -3.830757, -3.833032, -3.835348,
      -3.837707, -3.839110, -3.841561, -3.843528, -3.845596, -3.847183,
      -3.849411, -3.850753, -3.853120, -3.854515, -3.855933, -3.858075,
      -3.859857, -3.861354, -3.862880, -3.863720, -3.865026, -3.866640,
      -3.868142, -3.868978, -3.869833, -3.871407, -3.872589, -3.873491,
      -3.874275, -3.874897, -3.875798, -3.877031, -3.878000, -3.878357,
      -3.879725, -3.880000, -3.880605, -3.881328, -3.882093, -3.883000,
      -3.883000, -3.883691, -3.884000, -3.884292, -3.885000, -3.885000,
      -3.885807, -3.886000, -3.886229, -3.887000, -3.887000, -3.887000,
      -3.887458, -3.888000, -3.888000, -3.888000, -3.888000, -3.888491,
      -3.889000, -3.889000, -3.889000, -3.889000, -3.889000, -3.889000,
      -3.889000, -3.889000, -3.889000, -3.889000, -3.889000, -3.889000,
      -3.889000, -3.889000, -3.889000, -3.889000, -3.889000, -3.889000,
      -3.889000, -3.889000, -3.889000, -3.889000, -3.889000, -3.889000,
      -3.889000, -3.889000, -3.889000, -3.889000, -3.889000, -3.889000,
      -3.889000, -3.889000, -3.889000, NAN,       -1.310681, -1.310845,
      -1.312233, -1.313108, -1.314777, -1.315734, -1.316496, -1.317376,
      -1.318616, -1.319736, -1.320802, -1.321376, -1.321837, -1.322525,
      -1.323336, -1.324160, -1.325045, -1.325929, -1.326440, -1.326918,
      -1.327852, -1.328881, -1.329487, -1.330040, -1.330627, -1.331457,
      -1.332706, -1.333507, -1.334179, -1.334880, -1.335614, -1.336373,
      -1.337325, -1.338969, -1.339848, -1.340749, -1.341688, -1.343343,
      -1.344700, -1.346556, -1.348816, -1.350195, -1.352352, -1.354340,
      -1.356116, -1.358532, -1.361086, -1.363481, -1.365761, -1.368567,
      -1.371643, -1.375225, -1.378741, -1.381963, -1.385381, -1.389466,
      -1.394181, -1.398588, -1.404042, -1.409454, -1.415059, -1.421366,
      -1.428302, -1.435615, -1.444002, -1.453030, -1.462533, -1.472935,
      -1.484001, -1.495213, -1.506874, -1.518810, -1.530301, -1.541444,
      -1.552838, -1.564083, -1.574653, -1.585052, -1.595275, -1.605851,
      -1.616318, -1.626152, -1.635877, -1.644971, -1.654225, -1.663555,
      -1.672788, -1.682204, -1.690484, -1.699071, -1.707906, -1.716810,
      -1.724712, -1.732546, -1.740819, -1.748926, -1.756518, -1.764010,
      -1.771429, -1.779241, -1.787058, -1.794275, -1.801427, -1.808519,
      -1.816461, -1.824598, -1.832377, -1.840088, -1.847742, -1.855343,
      -1.862891, -1.870390, -1.877842, -1.885236, -1.892589, -1.899901,
      -1.907172, -1.914450, -1.922435, -1.929738, -1.936847, -1.943921,
      -1.950961, -1.957953, -1.964912, -1.972154, -1.979735, -1.986597,
      -1.993423, -2.000224, -2.007000, -2.013729, -2.020927, -2.027587,
      -2.034213, -2.040810, -2.047420, -2.054678, -2.061413, -2.067683,
      -2.074265, -2.081265, -2.087663, -2.094032, -2.100647, -2.107201,
      -2.113503, -2.119778, -2.126026, -2.132512, -2.138970, -2.145166,
      -2.151343, -2.157501, -2.163830, -2.170273, -2.176390, -2.182484,
      -2.188966, -2.195135, -2.201191, -2.207232, -2.213759, -2.219771,
      -2.225769, -2.231673, -2.237474, -2.244096, -2.250129, -2.256299,
      -2.262494, -2.268403, -2.274294, -2.280531, -2.286546, -2.292250,
      -2.298057, -2.304545, -2.311014, -2.317474, -2.323519, -2.329337,
      -2.335738, -2.342130, -2.348497, -2.354855, -2.361189, -2.367510,
      -2.373814, -2.380092, -2.386360, -2.392610, -2.398845, -2.405055,
      -2.411258, -2.417438, -2.423609, -2.429760, -2.435896, -2.442628,
      -2.449133, -2.455236, -2.461322, -2.467398, -2.473467, -2.479520,
      -2.485569, -2.491765, -2.498395, -2.504651, -2.510658, -2.516660,
      -2.522650, -2.528631, -2.535326, -2.541605, -2.547632, -2.553888,
      -2.560309, -2.566251, -2.572197, -2.578233, -2.584532, -2.590897,
      -2.597212, -2.603545, -2.609664, -2.615648, -2.622067, -2.628438,
      -2.634258, -2.640390, -2.646878, -2.653098, -2.658855, -2.665187,
      -2.671678, -2.678110, -2.684365, -2.690079, -2.696275, -2.702647,
      -2.709083, -2.715528, -2.722058, -2.728463, -2.734841, -2.740963,
      -2.746693, -2.753090, -2.759526, -2.765990, -2.772734, -2.779810,
      -2.786203, -2.792628, -2.799062, -2.805483, -2.811886, -2.818325,
      -2.825302, -2.831916, -2.838258, -2.844453, -2.851321, -2.858147,
      -2.864743, -2.871328, -2.877901, -2.884460, -2.890999, -2.897532,
      -2.904047, -2.910717, -2.917195, -2.923667, -2.930470, -2.937550,
      -2.943971, -2.950949, -2.957785, -2.964515, -2.971421, -2.978724,
      -2.986132, -2.993050, -2.999903, -3.006253, -3.013552, -3.020342,
      -3.027213, -3.034493, -3.041623, -3.048333, -3.055188, -3.062367,
      -3.069519, -3.076632, -3.083717, -3.090781, -3.097800, -3.104784,
      -3.111528, -3.117937, -3.124505, -3.131333, -3.138118, -3.144861,
      -3.151553, -3.158188, -3.164815, -3.171374, -3.177881, -3.184330,
      -3.190731, -3.197803, -3.204346, -3.210587, -3.217180, -3.223903,
      -3.229965, -3.235971, -3.242745, -3.249452, -3.255583, -3.261675,
      -3.268191, -3.274638, -3.281004, -3.287298, -3.293523, -3.299681,
      -3.305747, -3.311727, -3.317641, -3.323553, -3.330029, -3.335930,
      -3.341550, -3.347090, -3.353103, -3.358920, -3.364230, -3.369436,
      -3.374784, -3.380482, -3.385621, -3.390611, -3.396015, -3.401349,
      -3.406596, -3.411724, -3.416809, -3.421731, -3.426571, -3.430910,
      -3.435335, -3.439972, -3.444446, -3.448614, -3.452803, -3.457020,
      -3.461266, -3.465538, -3.469490, -3.473277, -3.477107, -3.480947,
      -3.484842, -3.488642, -3.492032, -3.495204, -3.498073, -3.501368,
      -3.504866, -3.507851, -3.510829, -3.513843, -3.516516, -3.518976,
      -3.521509, -3.524074, -3.526681, -3.529354, -3.531798, -3.533864,
      -3.535983, -3.538135, -3.540320, -3.542553, -3.544220, -3.545763,
      -3.547333, -3.548933, -3.550567, -3.552231, -3.553924, -3.555324,
      -3.556403, -3.558095, -3.559012, -3.560869, -3.561881, -3.562844,
      -3.563825, -3.564825, -3.565843, -3.566000, -3.566935, -3.568011,
      -3.569000, -3.569225, -3.570000, -3.570521, -3.571000, -3.571902,
      -3.572000, -3.572374, -3.573000, -3.573000, -3.574000, -3.574000,
      -3.574000, -3.575000, -3.575000, -3.575000, -3.575000, -3.576000,
      -3.576000, -3.576000, -3.576000, -3.576000, -3.576000, -3.576271,
      -3.576839, -3.577000, -3.577000, -3.577000, -3.577000, -3.577000,
      -3.577000, -3.577000, -3.577000, -3.577000, -3.577000, -3.577000,
      -3.577000, -3.577000, -3.577000, -3.577000, -3.577000, -3.577000,
      -3.577000, -3.577000, -3.577000, -3.577000, -3.577000, -3.577000,
      -3.577000, -3.577000, -3.577000, -3.577000, -3.577000, -3.577000,
      -3.577000, -3.577000, -3.577000, -3.577000, -3.577000, -3.577000,
      -3.577000, -3.577000, -3.577000, -3.577000, -3.576882, -3.576000,
      -3.576000, -3.576000, -3.576000, -3.576000, -3.576000, -3.576000,
      -3.576000, -3.576000, -3.576000, -3.576000, -3.576000, -3.576000,
      -3.576000, -3.576000, -3.576000, -3.576000, -3.576000, -3.576000,
      -3.576000, -3.576000, -3.575117, -3.575000, -3.575000, -3.575000,
      -3.575000, NAN,       -1.227261, -1.227332, -1.228071, -1.229287,
      -1.230433, -1.231479, -1.232228, -1.232681, -1.233249, -1.234093,
      -1.234938, -1.235848, -1.236764, -1.237366, -1.237860, -1.238758,
      -1.239816, -1.240467, -1.241067, -1.242271, -1.243251, -1.243890,
      -1.244560, -1.245249, -1.245964, -1.247432, -1.248495, -1.249304,
      -1.250146, -1.251050, -1.252891, -1.253907, -1.254913, -1.255965,
      -1.257141, -1.259233, -1.260454, -1.261736, -1.263088, -1.264522,
      -1.266063, -1.268641, -1.270348, -1.272165, -1.274141, -1.276192,
      -1.278733, -1.282734, -1.286123, -1.289740, -1.294115, -1.298939,
      -1.303393, -1.308379, -1.314385, -1.320820, -1.327275, -1.333876,
      -1.341738, -1.350280, -1.359419, -1.369034, -1.379000, -1.390118,
      -1.401259, -1.412704, -1.424324, -1.435420, -1.446348, -1.457102,
      -1.467976, -1.479123, -1.489365, -1.499431, -1.509351, -1.519134,
      -1.528766, -1.538660, -1.548011, -1.557246, -1.566283, -1.574733,
      -1.583643, -1.592717, -1.601512, -1.610002, -1.618057, -1.626535,
      -1.634488, -1.642598, -1.650809, -1.658420, -1.666860, -1.674299,
      -1.682028, -1.690016, -1.697282, -1.704531, -1.712274, -1.719747,
      -1.726782, -1.733762, -1.740689, -1.748371, -1.755787, -1.763124,
      -1.770642, -1.778098, -1.785495, -1.792837, -1.800126, -1.807358,
      -1.814534, -1.821665, -1.828755, -1.835805, -1.842810, -1.849776,
      -1.856712, -1.863440, -1.869985, -1.876827, -1.883640, -1.891132,
      -1.898174, -1.904906, -1.911615, -1.918300, -1.924960, -1.931422,
      -1.937708, -1.944443, -1.951740, -1.958406, -1.964925, -1.971377,
      -1.977394, -1.983850, -1.990975, -1.997686, -2.004077, -2.010444,
      -2.016290, -2.022609, -2.028909, -2.035322, -2.042270, -2.048179,
      -2.054386, -2.060577, -2.066837, -2.073404, -2.079544, -2.085660,
      -2.091763, -2.097852, -2.104416, -2.110466, -2.116496, -2.122347,
      -2.128348, -2.135002, -2.140967, -2.146602, -2.152371, -2.158307,
      -2.164229, -2.170603, -2.176542, -2.182164, -2.187813, -2.193761,
      -2.200273, -2.206021, -2.211739, -2.217573, -2.223396, -2.229680,
      -2.235514, -2.241565, -2.247981, -2.254389, -2.260773, -2.267149,
      -2.273051, -2.278846, -2.285171, -2.291486, -2.297788, -2.304071,
      -2.310345, -2.316608, -2.322858, -2.328480, -2.334314, -2.340521,
      -2.346722, -2.352908, -2.359079, -2.365242, -2.371388, -2.377528,
      -2.383650, -2.389759, -2.395855, -2.401943, -2.407388, -2.413464,
      -2.419732, -2.425543, -2.431328, -2.437454, -2.444144, -2.450095,
      -2.455685, -2.461799, -2.468106, -2.474297, -2.479990, -2.485889,
      -2.492221, -2.498615, -2.504755, -2.510438, -2.516712, -2.522996,
      -2.529049, -2.535057, -2.541371, -2.547030, -2.553240, -2.559197,
      -2.565126, -2.571575, -2.578009, -2.583699, -2.589794, -2.595963,
      -2.601637, -2.608021, -2.614444, -2.620388, -2.626293, -2.632704,
      -2.639132, -2.644838, -2.650965, -2.657350, -2.663462, -2.669029,
      -2.674848, -2.680823, -2.687011, -2.692901, -2.699209, -2.705512,
      -2.711663, -2.717300, -2.723423, -2.729402, -2.735344, -2.741745,
      -2.747833, -2.753669, -2.760078, -2.766217, -2.772252, -2.777900,
      -2.784486, -2.790724, -2.796637, -2.802925, -2.808927, -2.815274,
      -2.821790, -2.828280, -2.834764, -2.841222, -2.847657, -2.854063,
      -2.860462, -2.866830, -2.873167, -2.879478, -2.885771, -2.892027,
      -2.898246, -2.904445, -2.910616, -2.916749, -2.922850, -2.928907,
      -2.934946, -2.941610, -2.947918, -2.953856, -2.959751, -2.965627,
      -2.972052, -2.978277, -2.984043, -2.989772, -2.995658, -3.001772,
      -3.007721, -3.013501, -3.019808, -3.025324, -3.031643, -3.037053,
      -3.043082, -3.048874, -3.054494, -3.060230, -3.066138, -3.071972,
      -3.076897, -3.082377, -3.087962, -3.093260, -3.097914, -3.103217,
      -3.108454, -3.113613, -3.118687, -3.122976, -3.127512, -3.132310,
      -3.136979, -3.141565, -3.146072, -3.149815, -3.153750, -3.157955,
      -3.162037, -3.166068, -3.169553, -3.173051, -3.176580, -3.180139,
      -3.183716, -3.186783, -3.189819, -3.192887, -3.195985, -3.199081,
      -3.201613, -3.204163, -3.206735, -3.209345, -3.211976, -3.214636,
      -3.216751, -3.218794, -3.220858, -3.222957, -3.224718, -3.226220,
      -3.228397, -3.230069, -3.231558, -3.233064, -3.234598, -3.235967,
      -3.237240, -3.238366, -3.239434, -3.240427, -3.241435, -3.242465,
      -3.243512, -3.244578, -3.245333, -3.245885, -3.246903, -3.248022,
      -3.248611, -3.249000, -3.249000, -3.249432, -3.250060, -3.250703,
      -3.251000, -3.251021, -3.251701, -3.252000, -3.252094, -3.252812,
      -3.253000, -3.253000, -3.253042, -3.253816, -3.254000, -3.254000,
      -3.254000, -3.254000, -3.254000, -3.254000, -3.254000, -3.254532,
      -3.255000, -3.255000, -3.255000, -3.255000, -3.255000, -3.255000,
      -3.255000, -3.255000, -3.255000, -3.255000, -3.255000, -3.255000,
      -3.255000, -3.255000, -3.255000, -3.255000, -3.255000, -3.255000,
      -3.255000, -3.255000, -3.255000, -3.255000, -3.255000, -3.255000,
      -3.255000, -3.255000, -3.255000, -3.255000, -3.255000, -3.255000,
      -3.255000, -3.255000, -3.255000, -3.255000, -3.255000, -3.255000,
      -3.255000, -3.255000, -3.255000, -3.255000, -3.255000, -3.255000,
      -3.255000, -3.255000, -3.255000, -3.254022, -3.254000, -3.254000,
      -3.254000, -3.254000, -3.254000, -3.254000, -3.254000, -3.254000,
      -3.254000, -3.254000, -3.254000, -3.254000, -3.254000, -3.254000,
      -3.254000, -3.254000, -3.254000, -3.254000, -3.254000, -3.254000,
      -3.254000, -3.254000, -3.254000, -3.254000, -3.254000, -3.254000,
      -3.254000, -3.254000, -3.254000, -3.254000, -3.254000, -3.254000,
      -3.254000, -3.254000, -3.254000, -3.253230, -3.253000, -3.253000,
      -3.253000, -3.253000, -3.253000, -3.253000, -3.253000, -3.253000,
      -3.253000, -3.253000, -3.253000, -3.253000, -3.253000, -3.253000,
      -3.253000, -3.253000, -3.253000, -3.253000, NAN,       NAN,
      -1.172041, -1.172173, -1.173480, -1.174416, -1.175111, -1.175838,
      -1.176594, -1.177754, -1.179190, -1.180035, -1.180919, -1.181842,
      -1.182803, -1.184615, -1.185857, -1.186954, -1.188107, -1.189637,
      -1.191589, -1.192923, -1.194334, -1.195820, -1.197393, -1.200056,
      -1.201825, -1.203707, -1.205712, -1.207855, -1.210153, -1.212625,
      -1.215290, -1.218182, -1.221338, -1.225465, -1.230061, -1.234959,
      -1.240197, -1.246149, -1.252002, -1.257973, -1.264360, -1.271592,
      -1.279424, -1.287474, -1.295525, -1.303884, -1.312689, -1.321710,
      -1.330233, -1.339000, -1.347971, -1.356838, -1.365370, -1.373392,
      -1.381557, -1.389933, -1.398489, -1.406660, -1.414498, -1.422298,
      -1.430593, -1.438752, -1.446605, -1.454072, -1.461617, -1.469590,
      -1.477443, -1.484990, -1.492633, -1.500363, -1.507923, -1.514694,
      -1.522213, -1.529721, -1.536717, -1.543597, -1.550969, -1.558298,
      -1.564851, -1.571832, -1.578710, -1.585234, -1.592387, -1.598785,
      -1.605604, -1.612379, -1.618647, -1.624891, -1.631755, -1.638323,
      -1.644515, -1.650688, -1.656842, -1.662980, -1.669102, -1.675210,
      -1.681294, -1.688405, -1.695190, -1.701942, -1.708664, -1.715356,
      -1.722005, -1.728622, -1.735211, -1.741772, -1.747791, -1.754284,
      -1.760742, -1.767175, -1.773583, -1.779956, -1.785831, -1.792127,
      -1.798426, -1.804705, -1.810954, -1.816723, -1.822896, -1.829088,
      -1.835236, -1.840922, -1.847067, -1.853196, -1.859316, -1.865424,
      -1.872017, -1.878102, -1.884176, -1.890104, -1.895792, -1.901837,
      -1.907871, -1.913578, -1.919418, -1.925430, -1.931435, -1.937926,
      -1.943918, -1.949898, -1.955871, -1.961482, -1.968097, -1.974231,
      -1.980040, -1.985594, -1.991512, -1.997921, -2.003822, -2.009715,
      -2.015535, -2.020975, -2.026840, -2.032690, -2.038703, -2.044859,
      -2.050678, -2.055982, -2.061774, -2.067557, -2.073370, -2.079597,
      -2.085353, -2.090647, -2.096338, -2.102069, -2.107792, -2.114004,
      -2.119705, -2.125135, -2.130586, -2.136265, -2.141935, -2.147891,
      -2.153761, -2.159372, -2.164740, -2.171011, -2.176418, -2.182520,
      -2.188761, -2.194687, -2.200285, -2.206410, -2.212610, -2.218794,
      -2.224971, -2.230138, -2.236292, -2.242440, -2.248575, -2.254705,
      -2.260740, -2.266234, -2.272027, -2.278113, -2.284185, -2.289916,
      -2.295842, -2.301879, -2.307666, -2.313395, -2.319793, -2.325522,
      -2.331710, -2.337427, -2.343171, -2.349358, -2.354960, -2.361039,
      -2.366881, -2.372594, -2.378502, -2.385105, -2.390768, -2.396455,
      -2.402104, -2.407919, -2.414181, -2.419867, -2.425524, -2.431609,
      -2.437777, -2.443386, -2.448937, -2.455255, -2.461220, -2.466802,
      -2.472466, -2.478906, -2.484651, -2.490279, -2.496183, -2.502512,
      -2.508109, -2.514209, -2.520031, -2.525594, -2.531561, -2.537447,
      -2.543261, -2.549298, -2.555602, -2.561455, -2.567172, -2.572886,
      -2.578748, -2.585013, -2.590974, -2.596652, -2.602319, -2.607982,
      -2.614628, -2.620260, -2.625887, -2.632001, -2.638098, -2.643687,
      -2.649263, -2.655300, -2.661370, -2.667080, -2.673413, -2.678910,
      -2.684878, -2.690866, -2.696645, -2.702627, -2.708172, -2.714024,
      -2.719955, -2.725393, -2.731742, -2.737598, -2.743316, -2.748929,
      -2.754879, -2.760339, -2.766092, -2.772086, -2.777760, -2.783503,
      -2.789676, -2.795185, -2.800932, -2.806340, -2.812040, -2.818055,
      -2.823359, -2.828963, -2.834853, -2.840379, -2.845702, -2.851259,
      -2.856964, -2.862183, -2.867221, -2.872758, -2.878231, -2.883576,
      -2.888065, -2.893366, -2.898602, -2.903770, -2.908870, -2.913901,
      -2.918862, -2.923748, -2.928544, -2.933267, -2.937918, -2.942497,
      -2.946980, -2.951379, -2.955706, -2.959962, -2.964101, -2.968170,
      -2.972177, -2.976055, -2.979863, -2.983600, -2.986740, -2.989981,
      -2.993708, -2.997056, -3.000204, -3.003392, -3.006281, -3.008867,
      -3.011488, -3.014129, -3.016791, -3.019492, -3.022215, -3.024962,
      -3.027064, -3.029177, -3.031207, -3.032650, -3.034667, -3.036589,
      -3.038084, -3.039603, -3.041143, -3.042701, -3.044280, -3.045882,
      -3.047268, -3.048273, -3.048860, -3.049772, -3.050784, -3.051691,
      -3.052568, -3.053413, -3.054225, -3.055020, -3.055444, -3.055869,
      -3.056613, -3.057500, -3.058201, -3.058663, -3.059000, -3.059000,
      -3.059093, -3.059591, -3.060000, -3.060000, -3.060132, -3.060667,
      -3.061000, -3.061000, -3.061000, -3.061000, -3.061488, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.062000,
      -3.062000, -3.062000, -3.062000, -3.062000, -3.062000, -3.061292,
      -3.061000, -3.061000, -3.061000, -3.061000, -3.061000, -3.061000,
      -3.061000, -3.061000, -3.061000, -3.061000, -3.061000, -3.061000,
      -3.061000, -3.061000, -3.061000, -3.061000, -3.061000, -3.061000,
      -3.061000, -3.061000, -3.061000, -3.061000, -3.061000, -3.061000,
      -3.061000, -3.061000, -3.061000, -3.061000, -3.061000, -3.060204,
      -3.060000, -3.060000, -3.060000, -3.060000, -3.060000, -3.060000,
      -3.060000, -3.060000, NAN,       NAN,       -1.097436, -1.097515,
      -1.098617, -1.100133, -1.100990, -1.101888, -1.103644, -1.104796,
      -1.105813, -1.106875, -1.108971, -1.110153, -1.111378, -1.113322,
      -1.115007, -1.116433, -1.118860, -1.120518, -1.122194, -1.123972,
      -1.126865, -1.128880, -1.131031, -1.133338, -1.136622, -1.139492,
      -1.142373, -1.145504, -1.148369, -1.151655, -1.155489, -1.159200,
      -1.163411, -1.169682, -1.176371, -1.183190, -1.190194, -1.197171,
      -1.205733, -1.214270, -1.222549, -1.230969, -1.240509, -1.249463,
      -1.258029, -1.266422, -1.276145, -1.284482, -1.292918, -1.301436,
      -1.309767, -1.317828, -1.326045, -1.334271, -1.342432, -1.350391,
      -1.358134, -1.366040, -1.373959, -1.381400, -1.388640, -1.396350,
      -1.403766, -1.411275, -1.418683, -1.425465, -1.432987, -1.440462,
      -1.447352, -1.454288, -1.461459, -1.467960, -1.475220, -1.481843,
      -1.488640, -1.495597, -1.501928, -1.508998, -1.515484, -1.521722,
      -1.528052, -1.535009, -1.541276, -1.547412, -1.553528, -1.559623,
      -1.566696, -1.572753, -1.578791, -1.584815, -1.590826, -1.596823,
      -1.602806, -1.608779, -1.614526, -1.619737, -1.626848, -1.633521,
      -1.640164, -1.646785, -1.653385, -1.659962, -1.666511, -1.673383,
      -1.680048, -1.686534, -1.692999, -1.699433, -1.705847, -1.712105,
      -1.718109, -1.724459, -1.730776, -1.737073, -1.743204, -1.749103,
      -1.755336, -1.761540, -1.767724, -1.773793, -1.779531, -1.785657,
      -1.791756, -1.797526, -1.803404, -1.809453, -1.815483, -1.821003,
      -1.827001, -1.832989, -1.838968, -1.844431, -1.850389, -1.856338,
      -1.862139, -1.867712, -1.873642, -1.879561, -1.885201, -1.890882,
      -1.896872, -1.903179, -1.909070, -1.914956, -1.920634, -1.926216,
      -1.932091, -1.937961, -1.943514, -1.949188, -1.955160, -1.961392,
      -1.967237, -1.973073, -1.978695, -1.984229, -1.990046, -1.995852,
      -2.001359, -2.006946, -2.012816, -2.019011, -2.024783, -2.030544,
      -2.036155, -2.041542, -2.047281, -2.053452, -2.058533, -2.063936,
      -2.069644, -2.075842, -2.081533, -2.086862, -2.092397, -2.098192,
      -2.104236, -2.109893, -2.115536, -2.121303, -2.127546, -2.133777,
      -2.139995, -2.145577, -2.151392, -2.157571, -2.163744, -2.168905,
      -2.175055, -2.181351, -2.186935, -2.192644, -2.198928, -2.204694,
      -2.210743, -2.216665, -2.222424, -2.228283, -2.234383, -2.239942,
      -2.245590, -2.251444, -2.257496, -2.263630, -2.269530, -2.275239,
      -2.280882, -2.286699, -2.293007, -2.298779, -2.304364, -2.309963,
      -2.315834, -2.322211, -2.327805, -2.333482, -2.339563, -2.345713,
      -2.351337, -2.356940, -2.362550, -2.368611, -2.374771, -2.380389,
      -2.386007, -2.391613, -2.397686, -2.403758, -2.409121, -2.414652,
      -2.420319, -2.426292, -2.432417, -2.438209, -2.443998, -2.449775,
      -2.455547, -2.461312, -2.467071, -2.472819, -2.478565, -2.484301,
      -2.490033, -2.495759, -2.501476, -2.507191, -2.512895, -2.518592,
      -2.524280, -2.529963, -2.535634, -2.541301, -2.546956, -2.552662,
      -2.558856, -2.564859, -2.570468, -2.576068, -2.581657, -2.587231,
      -2.593262, -2.599330, -2.604853, -2.610357, -2.615851, -2.621320,
      -2.627042, -2.633015, -2.638605, -2.643989, -2.649774, -2.655653,
      -2.660993, -2.666299, -2.672062, -2.677743, -2.683118, -2.689085,
      -2.694201, -2.699564, -2.705359, -2.710653, -2.716354, -2.721288,
      -2.726449, -2.732072, -2.737074, -2.742675, -2.747448, -2.752580,
      -2.757712, -2.762330, -2.767642, -2.772356, -2.777350, -2.782173,
      -2.786800, -2.791770, -2.795983, -2.800946, -2.805200, -2.809660,
      -2.814171, -2.818077, -2.822641, -2.826604, -2.830531, -2.834859,
      -2.838386, -2.842190, -2.846205, -2.849474, -2.852677, -2.855821,
      -2.859602, -2.862959, -2.865537, -2.868678, -2.871765, -2.874412,
      -2.877089, -2.879785, -2.882510, -2.885268, -2.887787, -2.889894,
      -2.892033, -2.894192, -2.896249, -2.897723, -2.899217, -2.901093,
      -2.903257, -2.904808, -2.906382, -2.907488, -2.908593, -2.910115,
      -2.910946, -2.912328, -2.913320, -2.913929, -2.914924, -2.915972,
      -2.916883, -2.917789, -2.918303, -2.918698, -2.919084, -2.919444,
      -2.919804, -2.920345, -2.921101, -2.921858, -2.922000, -2.922000,
      -2.922121, -2.922537, -2.922952, -2.923383, -2.923817, -2.924000,
      -2.924000, -2.924000, -2.924000, -2.924000, -2.924000, -2.924100,
      -2.924608, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.925000, -2.925000,
      -2.925000, -2.925000, -2.925000, -2.925000, -2.924914, -2.924216,
      -2.924000, -2.924000, -2.924000, -2.924000, -2.924000, -2.924000,
      -2.924000, -2.924000, -2.924000, -2.924000, -2.924000, -2.924000,
      -2.924000, -2.924000, -2.924000, -2.924000, -2.924000, -2.924000,
      -2.924000, -2.924000, -2.924000, -2.924000, -2.924000, -2.924000,
      -2.924000, -2.924000, -2.924000, -2.924000, -2.924000, -2.924000,
      -2.924000, -2.924000, -2.924000, -2.924000, -2.924000, -2.924000,
      -2.924000, -2.924000, -2.923823, -2.923000, -2.923000, -2.923000,
      -2.923000, -2.923000, -2.923000, -2.923000, -2.923000, -2.923000,
      -2.923000, -2.923000, -2.923000, -2.923000, -2.923000, -2.923000,
      NAN,       NAN,       -0.961115, -0.961240, -0.962997, -0.964816,
      -0.966405, -0.968662, -0.970398, -0.972823, -0.975084, -0.977361,
      -0.980301, -0.982367, -0.985318, -0.988542, -0.991417, -0.994408,
      -0.997991, -1.001966, -1.005600, -1.009822, -1.014428, -1.018852,
      -1.023859, -1.029424, -1.035163, -1.040971, -1.046971, -1.053441,
      -1.060661, -1.068353, -1.076947, -1.086435, -1.095258, -1.104740,
      -1.113771, -1.122612, -1.131549, -1.140271, -1.148794, -1.157384,
      -1.166212, -1.174341, -1.182622, -1.190712, -1.198562, -1.206342,
      -1.214059, -1.221703, -1.229287, -1.236815, -1.244404, -1.252101,
      -1.259579, -1.266579, -1.273180, -1.280411, -1.287594, -1.294740,
      -1.301535, -1.308034, -1.315079, -1.321474, -1.328452, -1.334686,
      -1.341316, -1.347966, -1.354075, -1.360158, -1.366751, -1.373271,
      -1.379304, -1.385316, -1.391312, -1.397871, -1.404262, -1.410216,
      -1.416157, -1.422081, -1.427994, -1.433895, -1.439784, -1.445657,
      -1.451521, -1.457370, -1.463207, -1.468905, -1.473993, -1.479651,
      -1.485446, -1.490699, -1.496000, -1.501764, -1.507514, -1.513254,
      -1.518986, -1.524704, -1.530519, -1.537624, -1.544234, -1.550413,
      -1.556795, -1.563729, -1.570016, -1.576287, -1.582716, -1.589279,
      -1.595503, -1.601708, -1.607899, -1.614201, -1.620744, -1.626901,
      -1.633040, -1.638820, -1.644789, -1.650899, -1.656998, -1.663586,
      -1.669662, -1.675729, -1.681788, -1.687830, -1.693437, -1.699386,
      -1.705397, -1.711397, -1.717882, -1.723857, -1.729816, -1.735680,
      -1.741199, -1.747122, -1.752532, -1.758423, -1.764306, -1.770160,
      -1.775534, -1.781378, -1.787211, -1.792921, -1.798343, -1.804142,
      -1.809432, -1.815208, -1.820646, -1.825767, -1.831879, -1.836984,
      -1.842856, -1.848180, -1.853433, -1.859159, -1.864453, -1.870105,
      -1.875621, -1.881041, -1.886758, -1.891973, -1.897686, -1.903396,
      -1.909094, -1.914316, -1.920023, -1.925314, -1.930935, -1.936642,
      -1.942349, -1.947605, -1.953260, -1.958747, -1.964167, -1.969869,
      -1.975067, -1.980763, -1.986457, -1.992132, -1.997333, -2.003014,
      -2.009101, -2.015130, -2.021311, -2.027477, -2.033337, -2.038699,
      -2.045078, -2.050837, -2.056666, -2.062894, -2.068613, -2.074725,
      -2.080666, -2.086477, -2.092394, -2.098741, -2.104620, -2.110143,
      -2.115701, -2.121605, -2.128079, -2.133717, -2.139346, -2.144930,
      -2.150500, -2.156150, -2.162546, -2.168361, -2.173977, -2.179583,
      -2.185226, -2.190842, -2.196444, -2.202498, -2.208683, -2.214282,
      -2.219903, -2.225493, -2.230933, -2.236292, -2.242386, -2.248045,
      -2.253417, -2.259310, -2.265196, -2.270464, -2.275956, -2.281826,
      -2.287554, -2.292549, -2.298400, -2.304247, -2.309677, -2.314931,
      -2.320750, -2.326572, -2.332390, -2.337878, -2.343102, -2.348799,
      -2.354587, -2.360366, -2.365712, -2.370902, -2.376658, -2.382404,
      -2.388136, -2.393692, -2.398574, -2.404272, -2.409961, -2.415632,
      -2.420856, -2.425938, -2.431554, -2.437158, -2.442755, -2.448327,
      -2.453876, -2.459413, -2.464145, -2.469427, -2.474893, -2.480354,
      -2.485782, -2.491181, -2.496571, -2.501928, -2.507027, -2.511799,
      -2.516834, -2.522078, -2.527278, -2.532460, -2.537605, -2.543447,
      -2.548768, -2.553770, -2.558756, -2.563690, -2.568570, -2.573400,
      -2.578178, -2.583171, -2.588309, -2.593179, -2.597728, -2.602217,
      -2.606642, -2.611003, -2.615284, -2.619495, -2.623636, -2.627706,
      -2.631705, -2.635623, -2.640138, -2.644200, -2.647881, -2.651474,
      -2.654964, -2.658385, -2.661731, -2.664982, -2.667992, -2.670618,
      -2.673840, -2.676944, -2.679638, -2.682365, -2.684834, -2.686912,
      -2.689692, -2.692145, -2.694290, -2.696464, -2.698109, -2.699887,
      -2.702089, -2.703610, -2.705150, -2.706708, -2.708286, -2.709885,
      -2.711511, -2.712579, -2.713413, -2.714520, -2.716119, -2.716991,
      -2.717735, -2.718410, -2.719080, -2.720174, -2.721116, -2.721589,
      -2.722054, -2.722465, -2.722876, -2.723251, -2.723610, -2.723970,
      -2.724290, -2.724606, -2.724922, -2.725246, -2.725572, -2.725898,
      -2.726000, -2.726000, -2.726000, -2.726284, -2.726647, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727011,
      -2.727484, -2.727958, -2.728000, -2.728000, -2.728000, -2.728000,
      -2.728000, -2.728000, -2.728000, -2.728000, -2.728000, -2.728000,
      -2.728000, -2.728000, -2.728000, -2.728000, -2.728000, -2.728000,
      -2.728000, -2.728000, -2.728000, -2.728000, -2.728000, -2.728000,
      -2.728000, -2.728000, -2.728000, -2.728000, -2.728000, -2.728000,
      -2.728000, -2.728000, -2.728000, -2.728000, -2.727134, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.727000, -2.727000, -2.727000, -2.727000, -2.727000, -2.727000,
      -2.726224, -2.726000, -2.726000, -2.726000, -2.726000, -2.726000,
      -2.726000, -2.726000, -2.726000, -2.726000, -2.726000, -2.726000,
      -2.726000, -2.726000, -2.726000, -2.726000, -2.726000, -2.726000,
      -2.726000, -2.726000, -2.726000, -2.726000, -2.726000, -2.726000,
      -2.726000, -2.726000, -2.725412, -2.725000, -2.725000, -2.725000,
      -2.725000, -2.725000, -2.725000, -2.725000, NAN,       NAN,
      -0.846868, -0.847141, -0.849498, -0.852421, -0.855697, -0.859143,
      -0.862658, -0.866517, -0.870662, -0.875091, -0.879629, -0.883902,
      -0.889468, -0.894927, -0.900798, -0.907016, -0.913375, -0.920008,
      -0.927883, -0.935536, -0.943176, -0.951305, -0.959922, -0.968948,
      -0.977812, -0.986772, -0.996032, -1.004590, -1.013477, -1.022145,
      -1.030542, -1.039349, -1.048098, -1.056187, -1.064344, -1.072562,
      -1.080271, -1.088597, -1.096598, -1.103985, -1.111305, -1.119290,
      -1.126761, -1.133909, -1.141121, -1.148949, -1.156045, -1.162558,
      -1.170111, -1.177259, -1.184082, -1.190867, -1.197603, -1.204307,
      -1.210761, -1.216670, -1.223239, -1.230256, -1.236235, -1.242428,
      -1.248947, -1.255438, -1.261362, -1.267100, -1.273291, -1.279516,
      -1.285203, -1.291481, -1.297532, -1.303176, -1.308811, -1.314436,
      -1.320048, -1.325654, -1.331249, -1.337069, -1.343350, -1.348998,
      -1.354569, -1.359617, -1.364695, -1.370252, -1.375804, -1.381353,
      -1.386899, -1.392442, -1.397984, -1.402955, -1.408056, -1.413588,
      -1.418979, -1.423817, -1.429174, -1.434698, -1.440221, -1.445834,
      -1.452028, -1.458677, -1.464866, -1.471011, -1.477128, -1.482753,
      -1.488850, -1.494928, -1.501216, -1.507939, -1.513595, -1.519611,
      -1.525613, -1.531671, -1.538075, -1.544036, -1.549536, -1.556126,
      -1.562335, -1.568221, -1.573647, -1.579538, -1.585420, -1.591440,
      -1.597661, -1.603521, -1.609373, -1.614975, -1.620958, -1.627356,
      -1.632731, -1.638559, -1.644382, -1.650336, -1.656239, -1.661407,
      -1.667642, -1.673447, -1.679249, -1.685043, -1.690346, -1.696137,
      -1.701489, -1.707207, -1.712986, -1.718760, -1.724026, -1.729786,
      -1.735826, -1.741369, -1.746532, -1.752262, -1.757657, -1.763205,
      -1.768815, -1.774118, -1.779813, -1.784998, -1.790048, -1.795351,
      -1.801017, -1.806176, -1.811828, -1.817201, -1.822214, -1.827254,
      -1.832885, -1.838235, -1.843236, -1.848295, -1.854235, -1.859236,
      -1.864236, -1.869806, -1.875384, -1.880693, -1.885353, -1.890674,
      -1.895983, -1.901483, -1.907064, -1.912096, -1.917597, -1.924071,
      -1.929826, -1.935448, -1.941439, -1.947713, -1.953488, -1.959621,
      -1.965636, -1.971328, -1.976934, -1.983247, -1.989071, -1.994748,
      -2.000629, -2.006962, -2.012620, -2.018233, -2.024266, -2.030491,
      -2.036079, -2.041727, -2.047333, -2.053356, -2.059551, -2.065171,
      -2.070794, -2.076405, -2.082230, -2.088162, -2.093791, -2.099708,
      -2.105630, -2.111342, -2.116269, -2.122211, -2.128149, -2.133767,
      -2.139095, -2.144905, -2.150222, -2.155695, -2.161575, -2.167449,
      -2.172452, -2.178175, -2.184024, -2.189497, -2.194709, -2.200543,
      -2.206369, -2.211667, -2.217001, -2.222624, -2.227842, -2.233393,
      -2.239173, -2.244951, -2.249715, -2.255470, -2.261215, -2.266363,
      -2.271676, -2.277394, -2.283087, -2.288204, -2.293463, -2.299125,
      -2.304493, -2.309564, -2.315024, -2.320157, -2.325209, -2.330774,
      -2.336319, -2.341841, -2.346924, -2.351945, -2.357398, -2.362838,
      -2.367403, -2.372627, -2.377995, -2.383255, -2.387611, -2.392862,
      -2.398102, -2.403291, -2.408153, -2.412545, -2.417605, -2.422643,
      -2.427295, -2.431546, -2.436414, -2.441224, -2.446015, -2.450725,
      -2.455301, -2.458974, -2.463488, -2.467942, -2.472329, -2.476619,
      -2.480861, -2.484855, -2.488173, -2.492186, -2.496119, -2.499990,
      -2.503794, -2.507490, -2.510467, -2.513604, -2.517056, -2.520377,
      -2.523359, -2.526022, -2.529387, -2.532419, -2.535148, -2.537913,
      -2.540275, -2.542383, -2.544516, -2.546673, -2.548851, -2.551050,
      -2.553283, -2.555026, -2.556548, -2.558086, -2.559649, -2.561231,
      -2.562834, -2.564457, -2.566051, -2.566886, -2.567735, -2.569190,
      -2.570468, -2.571296, -2.572039, -2.572681, -2.573278, -2.573830,
      -2.574659, -2.575613, -2.576246, -2.576660, -2.577065, -2.577427,
      -2.577789, -2.578133, -2.578452, -2.578770, -2.579079, -2.579361,
      -2.579644, -2.579926, -2.580000, -2.580000, -2.580000, -2.580000,
      -2.580000, -2.580000, -2.580075, -2.580410, -2.580745, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.581000, -2.581000, -2.581000, -2.581000,
      -2.581000, -2.581000, -2.580000, -2.580000, -2.580000, -2.580000,
      -2.580000, -2.580000, -2.580000, -2.580000, -2.580000, -2.580000,
      -2.580000, -2.580000, -2.580000, -2.580000, -2.580000, -2.580000,
      -2.580000, -2.580000, -2.580000, -2.580000, -2.580000, -2.580000,
      -2.580000, -2.580000, -2.580000, -2.580000, -2.580000, -2.580000,
      -2.580000, -2.580000, -2.580000, -2.580000, -2.580000, -2.580000,
      -2.580000, -2.579893, -2.579064, -2.579000, -2.579000, -2.579000,
      -2.579000, -2.579000, -2.579000, -2.579000, -2.579000, -2.579000,
      -2.579000, -2.579000, -2.579000, -2.579000, -2.579000, -2.579000,
      -2.579000, -2.578397, -2.578000, -2.578000, -2.578000, -2.578000,
      -2.578000, -2.578000, -2.578000, -2.578000, -2.578000, -2.578000,
      -2.578000, -2.577834, -2.577011, NAN,       -0.725382, -0.725697,
      -0.728303, -0.731548, -0.735234, -0.739368, -0.743609, -0.748105,
      -0.753042, -0.758385, -0.763877, -0.769344, -0.775605, -0.782661,
      -0.790146, -0.797836, -0.805674, -0.814558, -0.824178, -0.833915,
      -0.843622, -0.853387, -0.863640, -0.873345, -0.883207, -0.893366,
      -0.903250, -0.912957, -0.922120, -0.931048, -0.940378, -0.949598,
      -0.958698, -0.967676, -0.976547, -0.985085, -0.993178, -1.001559,
      -1.010045, -1.018447, -1.026694, -1.034109, -1.042171, -1.049740,
      -1.057300, -1.065145, -1.072266, -1.080029, -1.087340, -1.094568,
      -1.102241, -1.109113, -1.115946, -1.123047, -1.130450, -1.136719,
      -1.143408, -1.150053, -1.157183, -1.164248, -1.170802, -1.176614,
      -1.183233, -1.189765, -1.196199, -1.202543, -1.208215, -1.214844,
      -1.220513, -1.226529, -1.232749, -1.238344, -1.243929, -1.250183,
      -1.256052, -1.261599, -1.267134, -1.272659, -1.278818, -1.284676,
      -1.290175, -1.295663, -1.301147, -1.306622, -1.311832, -1.316616,
      -1.322024, -1.327483, -1.332941, -1.338047, -1.342850, -1.348266,
      -1.353037, -1.358211, -1.363666, -1.369121, -1.374576, -1.381272,
      -1.387452, -1.393354, -1.399442, -1.405515, -1.411077, -1.417123,
      -1.423159, -1.429206, -1.435695, -1.441694, -1.447680, -1.453474,
      -1.459119, -1.465074, -1.471011, -1.477268, -1.483357, -1.489265,
      -1.495160, -1.501495, -1.507417, -1.513280, -1.519133, -1.524475,
      -1.530305, -1.536476, -1.542438, -1.548242, -1.554036, -1.559399,
      -1.565100, -1.571079, -1.576785, -1.581905, -1.587642, -1.593655,
      -1.599633, -1.605332, -1.610608, -1.616337, -1.622066, -1.627882,
      -1.634017, -1.639741, -1.644966, -1.650682, -1.656668, -1.662618,
      -1.668334, -1.674050, -1.679458, -1.684978, -1.690614, -1.695900,
      -1.701607, -1.707312, -1.712902, -1.718213, -1.723911, -1.729102,
      -1.734791, -1.740202, -1.745656, -1.751296, -1.756335, -1.761579,
      -1.767183, -1.772364, -1.777102, -1.782996, -1.788091, -1.793384,
      -1.798535, -1.803680, -1.809037, -1.814638, -1.819383, -1.824639,
      -1.829956, -1.834900, -1.839821, -1.845419, -1.851527, -1.857590,
      -1.863301, -1.868808, -1.874541, -1.880131, -1.885724, -1.891558,
      -1.897958, -1.903569, -1.909215, -1.914849, -1.920450, -1.926044,
      -1.932491, -1.938297, -1.943902, -1.949509, -1.955144, -1.961259,
      -1.967146, -1.972821, -1.978258, -1.984197, -1.989782, -1.995274,
      -2.001317, -2.007316, -2.012737, -2.018395, -2.024065, -2.029473,
      -2.035367, -2.040749, -2.046363, -2.052325, -2.058282, -2.063227,
      -2.069158, -2.075084, -2.080704, -2.086034, -2.091824, -2.097133,
      -2.102579, -2.108442, -2.113963, -2.119224, -2.124972, -2.130211,
      -2.135604, -2.141161, -2.146369, -2.151964, -2.157721, -2.163378,
      -2.168205, -2.173729, -2.178862, -2.184317, -2.189989, -2.195655,
      -2.200301, -2.205925, -2.211235, -2.216139, -2.221442, -2.226447,
      -2.231816, -2.237327, -2.242831, -2.247302, -2.252675, -2.257548,
      -2.262567, -2.267933, -2.273036, -2.277809, -2.282844, -2.287464,
      -2.292268, -2.297436, -2.302562, -2.306911, -2.311672, -2.316276,
      -2.320731, -2.325486, -2.330321, -2.334482, -2.338826, -2.343226,
      -2.347367, -2.351637, -2.356113, -2.360522, -2.364860, -2.368293,
      -2.372298, -2.376403, -2.380146, -2.383388, -2.387247, -2.391014,
      -2.394363, -2.397319, -2.400819, -2.404231, -2.407475, -2.410152,
      -2.412852, -2.415570, -2.418316, -2.421090, -2.423664, -2.425781,
      -2.427925, -2.430089, -2.432273, -2.434486, -2.436726, -2.438659,
      -2.440185, -2.441732, -2.443301, -2.444888, -2.446496, -2.448062,
      -2.448889, -2.450457, -2.451579, -2.452442, -2.453318, -2.454174,
      -2.454928, -2.455585, -2.456396, -2.457506, -2.458266, -2.458746,
      -2.459196, -2.459612, -2.460025, -2.460389, -2.460753, -2.461103,
      -2.461423, -2.461743, -2.462056, -2.462340, -2.462624, -2.462908,
      -2.463000, -2.463000, -2.463000, -2.463000, -2.463270, -2.463557,
      -2.463843, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.464000, -2.464000, -2.464000, -2.464000,
      -2.464000, -2.464000, -2.463423, -2.463000, -2.463000, -2.463000,
      -2.463000, -2.463000, -2.463000, -2.463000, -2.463000, -2.463000,
      -2.463000, -2.463000, -2.463000, -2.463000, -2.463000, -2.463000,
      -2.463000, -2.463000, -2.463000, -2.463000, -2.463000, -2.463000,
      -2.463000, -2.463000, -2.463000, -2.463000, -2.463000, -2.463000,
      -2.463000, -2.463000, -2.463000, -2.462166, -2.462000, -2.462000,
      -2.462000, -2.462000, -2.462000, -2.462000, -2.462000, -2.462000,
      -2.462000, -2.462000, -2.462000, -2.462000, -2.462000, -2.462000,
      -2.462000, -2.461311, -2.461000, -2.461000, -2.461000, -2.461000,
      -2.461000, -2.461000, -2.461000, -2.461000, -2.460973, -2.460158,
      -2.460000, -2.460000, -2.460000, -2.460000, -2.460000, -2.460000,
      -2.460000, -2.459738, -2.459000, -2.459000, -2.459000, -2.459000,
      -2.459000, -2.459000, -2.458151, -2.458000, -2.458000, -2.458000,
      NAN,       NAN,       -0.657168, -0.657953, -0.665728, -0.673905,
      -0.682336, -0.691062, -0.700114, -0.708808, -0.717483, -0.726200,
      -0.735208, -0.743256, -0.752036, -0.760821, -0.769147, -0.777225,
      -0.785724, -0.794203, -0.802153, -0.809958, -0.818232, -0.826434,
      -0.834556, -0.842052, -0.849606, -0.857536, -0.865398, -0.873184,
      -0.880910, -0.888578, -0.896191, -0.903727, -0.910687, -0.917645,
      -0.925009, -0.932332, -0.939606, -0.946815, -0.953983, -0.961115,
      -0.968185, -0.975206, -0.981496, -0.988175, -0.995104, -1.001986,
      -1.008847, -1.014997, -1.021458, -1.028233, -1.034771, -1.040816,
      -1.047374, -1.054047, -1.060322, -1.066292, -1.072237, -1.078524,
      -1.084590, -1.090666, -1.096884, -1.102761, -1.108619, -1.114744,
      -1.120803, -1.126630, -1.132442, -1.138607, -1.144032, -1.149466,
      -1.155589, -1.161352, -1.167101, -1.172640, -1.178075, -1.183802,
      -1.189017, -1.194725, -1.200045, -1.205136, -1.211170, -1.216227,
      -1.221283, -1.226877, -1.232386, -1.237434, -1.242481, -1.248092,
      -1.253570, -1.258614, -1.263656, -1.268785, -1.274457, -1.279779,
      -1.285287, -1.291047, -1.297318, -1.302867, -1.308354, -1.314088,
      -1.320095, -1.325706, -1.331791, -1.337411, -1.342785, -1.348098,
      -1.353687, -1.359269, -1.365057, -1.370928, -1.376504, -1.381576,
      -1.387146, -1.393015, -1.398779, -1.404306, -1.409405, -1.414965,
      -1.420914, -1.426579, -1.432134, -1.437685, -1.442876, -1.448284,
      -1.453739, -1.458876, -1.464419, -1.470398, -1.475996, -1.481361,
      -1.486564, -1.492096, -1.498111, -1.503022, -1.508415, -1.514196,
      -1.519718, -1.524735, -1.530251, -1.535766, -1.541278, -1.546288,
      -1.551796, -1.557560, -1.562942, -1.567835, -1.573817, -1.579319,
      -1.584508, -1.589824, -1.595291, -1.600331, -1.605836, -1.611019,
      -1.616173, -1.621481, -1.626821, -1.631983, -1.637372, -1.643024,
      -1.648617, -1.653880, -1.658853, -1.664694, -1.669477, -1.674724,
      -1.679957, -1.684920, -1.690487, -1.695418, -1.700827, -1.706124,
      -1.711102, -1.716708, -1.721566, -1.726417, -1.731623, -1.737050,
      -1.741873, -1.747471, -1.753085, -1.758708, -1.764530, -1.770952,
      -1.776572, -1.782185, -1.787790, -1.793392, -1.799008, -1.804625,
      -1.810397, -1.816231, -1.822018, -1.827513, -1.833098, -1.838757,
      -1.843679, -1.849081, -1.854754, -1.860367, -1.865753, -1.871634,
      -1.877010, -1.882649, -1.887891, -1.893117, -1.898533, -1.904336,
      -1.909689, -1.915377, -1.920884, -1.926251, -1.932062, -1.937392,
      -1.943022, -1.948540, -1.953857, -1.959667, -1.964970, -1.970521,
      -1.976061, -1.981349, -1.987132, -1.992404, -1.997859, -2.003431,
      -2.008684, -2.014364, -2.020160, -2.025378, -2.030769, -2.036288,
      -2.041475, -2.047055, -2.052301, -2.057497, -2.063191, -2.068679,
      -2.073771, -2.079164, -2.084394, -2.089432, -2.094938, -2.100430,
      -2.105388, -2.110485, -2.115748, -2.120639, -2.125789, -2.130532,
      -2.135497, -2.140813, -2.145273, -2.150329, -2.155058, -2.159714,
      -2.164799, -2.169868, -2.173913, -2.178356, -2.182808, -2.187659,
      -2.192476, -2.196389, -2.200938, -2.205200, -2.209335, -2.213681,
      -2.217705, -2.221516, -2.225665, -2.229077, -2.233234, -2.236651,
      -2.240318, -2.243996, -2.247098, -2.250841, -2.254002, -2.257087,
      -2.260584, -2.263962, -2.267061, -2.269754, -2.272478, -2.275221,
      -2.277984, -2.280088, -2.282608, -2.285341, -2.287503, -2.289689,
      -2.291598, -2.293127, -2.295390, -2.297119, -2.298662, -2.300224,
      -2.301807, -2.303414, -2.304521, -2.305690, -2.307181, -2.308056,
      -2.309778, -2.310763, -2.311651, -2.312462, -2.313189, -2.313839,
      -2.314420, -2.314979, -2.315464, -2.315947, -2.316372, -2.316791,
      -2.317184, -2.317549, -2.317915, -2.318247, -2.318569, -2.318891,
      -2.319189, -2.319474, -2.319759, -2.320000, -2.320000, -2.320000,
      -2.320000, -2.320057, -2.320314, -2.320570, -2.320826, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.321000, -2.321000, -2.321000, -2.321000, -2.321000, -2.321000,
      -2.320715, -2.320000, -2.320000, -2.320000, -2.320000, -2.320000,
      -2.320000, -2.320000, -2.320000, -2.320000, -2.320000, -2.320000,
      -2.320000, -2.320000, -2.320000, -2.320000, -2.320000, -2.320000,
      -2.320000, -2.320000, -2.320000, -2.320000, -2.320000, -2.320000,
      -2.320000, -2.320000, -2.319000, -2.319000, -2.319000, -2.319000,
      -2.319000, -2.319000, -2.319000, -2.319000, -2.319000, -2.319000,
      -2.319000, -2.319000, -2.319000, -2.318037, -2.318000, -2.318000,
      -2.318000, -2.318000, -2.318000, -2.318000, -2.318000, -2.318000,
      -2.317841, -2.317212, -2.317000, -2.317000, -2.317000, -2.317000,
      -2.317000, -2.317000, -2.316469, -2.316000, -2.316000, -2.316000,
      -2.316000, -2.316000, -2.315638, -2.315000, -2.315000, -2.315000,
      -2.315000, -2.314663, -2.314000, -2.314000, -2.314000, -2.314000,
      -2.313622, -2.313000, -2.313000, -2.313000, -2.312400, -2.312000,
      -2.312000, -2.312000, -2.311276, -2.311000, -2.311000, -2.310797,
      -2.310000, -2.310000, -2.310000, -2.309576, -2.309000, -2.308946,
      -2.308179, -2.308000, -2.307604, -2.307000, NAN,       NAN,
      -0.478041, -0.478794, -0.486187, -0.493083, -0.499665, -0.506218,
      -0.512738, -0.518883, -0.525208, -0.531652, -0.538077, -0.544461,
      -0.551255, -0.557659, -0.563968, -0.570255, -0.576507, -0.583158,
      -0.589442, -0.595622, -0.601782, -0.607906, -0.614353, -0.620598,
      -0.626847, -0.633559, -0.639247, -0.645907, -0.652194, -0.658176,
      -0.664597, -0.670520, -0.676915, -0.683464, -0.689986, -0.696498,
      -0.701990, -0.708464, -0.714924, -0.721373, -0.727803, -0.734222,
      -0.740628, -0.747015, -0.753393, -0.759758, -0.766110, -0.772444,
      -0.778771, -0.785079, -0.791380, -0.797660, -0.803933, -0.810186,
      -0.816433, -0.822658, -0.828871, -0.835072, -0.841777, -0.848423,
      -0.854570, -0.860706, -0.866826, -0.872921, -0.879207, -0.885880,
      -0.892124, -0.898152, -0.904172, -0.910170, -0.916155, -0.922127,
      -0.928108, -0.934043, -0.939972, -0.945882, -0.951783, -0.957670,
      -0.963539, -0.969404, -0.975252, -0.981092, -0.986923, -0.992740,
      -0.998553, -1.004354, -1.010146, -1.015934, -1.021712, -1.027480,
      -1.033245, -1.039000, -1.044749, -1.050494, -1.056230, -1.061963,
      -1.067691, -1.073411, -1.079130, -1.084842, -1.090549, -1.096253,
      -1.101954, -1.107650, -1.113345, -1.119035, -1.124721, -1.130405,
      -1.135468, -1.140761, -1.146435, -1.152105, -1.157771, -1.163435,
      -1.168676, -1.173752, -1.179406, -1.185058, -1.190235, -1.195353,
      -1.200996, -1.206473, -1.211549, -1.216916, -1.222557, -1.228199,
      -1.232874, -1.238465, -1.243815, -1.249095, -1.254280, -1.259628,
      -1.264846, -1.269760, -1.274883, -1.280136, -1.285401, -1.290463,
      -1.295488, -1.300750, -1.305790, -1.310613, -1.315383, -1.320218,
      -1.325192, -1.330517, -1.335797, -1.340667, -1.345433, -1.350371,
      -1.355168, -1.360282, -1.365833, -1.370623, -1.375467, -1.380265,
      -1.385074, -1.389912, -1.394691, -1.399513, -1.404322, -1.409132,
      -1.413949, -1.418760, -1.423572, -1.428384, -1.433198, -1.437994,
      -1.442712, -1.447590, -1.452660, -1.457906, -1.462441, -1.467055,
      -1.472145, -1.477481, -1.482288, -1.487044, -1.491806, -1.496570,
      -1.502175, -1.508111, -1.513095, -1.519047, -1.524995, -1.530540,
      -1.535883, -1.541720, -1.547050, -1.552640, -1.558548, -1.564454,
      -1.569351, -1.575240, -1.581125, -1.586702, -1.591989, -1.597742,
      -1.603598, -1.609161, -1.614295, -1.620135, -1.625969, -1.630837,
      -1.636616, -1.642434, -1.648034, -1.653296, -1.658920, -1.664237,
      -1.669494, -1.675143, -1.680335, -1.685801, -1.691554, -1.696865,
      -1.702033, -1.707681, -1.712820, -1.718172, -1.723573, -1.728684,
      -1.734202, -1.739849, -1.744927, -1.750093, -1.755517, -1.760532,
      -1.765253, -1.770346, -1.775461, -1.780401, -1.785792, -1.790196,
      -1.795044, -1.799983, -1.805054, -1.809452, -1.814188, -1.819118,
      -1.824314, -1.828969, -1.833063, -1.837664, -1.842159, -1.846691,
      -1.851362, -1.855774, -1.859913, -1.864192, -1.868553, -1.872640,
      -1.877020, -1.880675, -1.884403, -1.888870, -1.892418, -1.896613,
      -1.900310, -1.903677, -1.907239, -1.911251, -1.914443, -1.917570,
      -1.921047, -1.924647, -1.927587, -1.930444, -1.933550, -1.936973,
      -1.939685, -1.942408, -1.945113, -1.947195, -1.949728, -1.952414,
      -1.954556, -1.956724, -1.958608, -1.960122, -1.962365, -1.964089,
      -1.965618, -1.967165, -1.968735, -1.970326, -1.971469, -1.972571,
      -1.974113, -1.974952, -1.975806, -1.976671, -1.977549, -1.978441,
      -1.979346, -1.980222, -1.981001, -1.981667, -1.982286, -1.982858,
      -1.983371, -1.983865, -1.984311, -1.984739, -1.985146, -1.985520,
      -1.985894, -1.986000, -1.986000, -1.986000, -1.986195, -1.986485,
      -1.986776, -1.987000, -1.987000, -1.987000, -1.987000, -1.987085,
      -1.987318, -1.987551, -1.987784, -1.988000, -1.988000, -1.988000,
      -1.988000, -1.988000, -1.988000, -1.988000, -1.988000, -1.988000,
      -1.988000, -1.988000, -1.988000, -1.988000, -1.988000, -1.988000,
      -1.988000, -1.988000, -1.988000, -1.988000, -1.988000, -1.988000,
      -1.988000, -1.988000, -1.988000, -1.988000, -1.988000, -1.988000,
      -1.988000, -1.988000, -1.987773, -1.987403, -1.987032, -1.987000,
      -1.987000, -1.987000, -1.987000, -1.987000, -1.987000, -1.987000,
      -1.987000, -1.987000, -1.987000, -1.987000, -1.987000, -1.987000,
      -1.987000, -1.987000, -1.986974, -1.986455, -1.986000, -1.986000,
      -1.986000, -1.986000, -1.986000, -1.986000, -1.986000, -1.986000,
      -1.986000, -1.985757, -1.985128, -1.985000, -1.985000, -1.985000,
      -1.985000, -1.985000, -1.985000, -1.985000, -1.984645, -1.984000,
      -1.984000, -1.984000, -1.984000, -1.984000, -1.983971, -1.983139,
      -1.983000, -1.983000, -1.983000, -1.983000, -1.982746, -1.982000,
      -1.982000, -1.982000, -1.982000, -1.981929, -1.981000, -1.981000,
      -1.981000, -1.980741, -1.980000, -1.980000, -1.980000, -1.980000,
      -1.979054, -1.979000, -1.979000, -1.978379, -1.978000, -1.978000,
      -1.978000, -1.977154, -1.977000, -1.977000, -1.976000, -1.976000,
      -1.976000, -1.975555, -1.975000, -1.975000, -1.974886, -1.974000,
      -1.974000, -1.973000, -1.973000, -1.973000, -1.972000, -1.972000,
      -1.972000, -1.971000, -1.971000, -1.970370, -1.970000, -1.970000,
      -1.969000, -1.969000, -1.968102, -1.968000, -1.968000, -1.967000,
      -1.967000, -1.966000, -1.966000, -1.965677, -1.965000, -1.965000,
      -1.964370, -1.964000, -1.963739, -1.962878, -1.962187, -1.962000,
      -1.961518, -1.961000, -1.960798, -1.960044, -1.960000, -1.959486,
      -1.959000, -1.958874, -1.958046, -1.958000, -1.957469, -1.956712,
      -1.956000, -1.956000, -1.955234, -1.955000, -1.954662, -1.954000,
      -1.954000, -1.953268, -1.953000, -1.952596, -1.951781, -1.951053,
      -1.951000, -1.950385, -1.950000, -1.949830, -1.949032, -1.949000,
      -1.948385, -1.948000, -1.947846, -1.947000, -1.947000, -1.946426,
      -1.946000, -1.945823, -1.945000, NAN,       -0.315314, -0.315973,
      -0.321928, -0.327968, -0.334435, -0.340894, -0.347329, -0.353204,
      -0.359159, -0.365555, -0.371931, -0.378293, -0.384174, -0.389975,
      -0.396294, -0.402603, -0.408898, -0.415174, -0.421444, -0.427696,
      -0.433941, -0.440164, -0.446381, -0.452581, -0.458775, -0.464950,
      -0.471112, -0.477264, -0.483397, -0.489525, -0.495634, -0.501728,
      -0.507814, -0.513878, -0.519936, -0.525974, -0.532000, -0.538014,
      -0.544008, -0.549995, -0.555963, -0.561920, -0.567866, -0.574270,
      -0.580711, -0.586613, -0.592505, -0.598387, -0.604253, -0.610116,
      -0.615963, -0.621799, -0.627629, -0.633447, -0.639254, -0.645058,
      -0.650935, -0.657297, -0.663414, -0.669183, -0.674949, -0.680709,
      -0.687007, -0.693211, -0.698955, -0.704693, -0.710429, -0.716160,
      -0.722375, -0.728612, -0.734466, -0.740754, -0.746763, -0.752474,
      -0.758598, -0.764873, -0.770642, -0.776905, -0.782968, -0.788919,
      -0.795170, -0.801013, -0.806683, -0.812350, -0.818017, -0.823682,
      -0.829346, -0.835009, -0.840670, -0.846327, -0.851983, -0.857698,
      -0.863908, -0.869924, -0.875561, -0.881194, -0.886821, -0.892443,
      -0.898055, -0.903665, -0.909392, -0.915544, -0.921448, -0.927028,
      -0.932600, -0.938169, -0.943729, -0.949316, -0.954996, -0.960867,
      -0.966532, -0.971958, -0.977703, -0.983474, -0.989353, -0.995101,
      -1.000700, -1.006237, -1.011104, -1.016919, -1.022618, -1.028034,
      -1.033567, -1.039193, -1.044716, -1.050256, -1.055874, -1.061503,
      -1.066278, -1.071895, -1.077551, -1.083152, -1.088760, -1.094322,
      -1.099257, -1.104607, -1.110211, -1.115674, -1.120516, -1.126054,
      -1.131667, -1.136955, -1.141894, -1.147505, -1.153111, -1.158191,
      -1.163335, -1.168809, -1.173625, -1.178825, -1.183984, -1.189097,
      -1.194435, -1.199532, -1.204728, -1.209978, -1.214902, -1.220143,
      -1.225622, -1.230698, -1.235637, -1.240091, -1.245532, -1.250488,
      -1.255605, -1.260908, -1.265874, -1.270527, -1.275684, -1.280791,
      -1.285769, -1.290222, -1.294732, -1.299719, -1.304896, -1.310195,
      -1.315186, -1.319825, -1.324263, -1.329627, -1.335175, -1.340572,
      -1.345248, -1.350775, -1.356298, -1.361530, -1.366488, -1.371822,
      -1.377315, -1.382318, -1.387278, -1.392596, -1.397204, -1.402652,
      -1.408095, -1.413272, -1.418151, -1.423358, -1.428006, -1.433140,
      -1.438520, -1.442908, -1.448210, -1.453007, -1.457889, -1.462956,
      -1.467488, -1.472782, -1.477663, -1.482382, -1.487420, -1.492198,
      -1.496647, -1.501621, -1.506146, -1.510818, -1.515596, -1.519884,
      -1.525123, -1.529356, -1.533734, -1.538809, -1.543150, -1.547879,
      -1.552424, -1.556607, -1.560774, -1.564928, -1.569070, -1.573190,
      -1.577290, -1.581657, -1.586221, -1.590452, -1.594453, -1.598113,
      -1.601563, -1.605270, -1.609140, -1.612973, -1.616766, -1.620519,
      -1.624221, -1.627876, -1.631519, -1.635072, -1.637714, -1.641091,
      -1.644510, -1.647877, -1.650892, -1.653449, -1.656645, -1.659773,
      -1.662841, -1.665639, -1.667851, -1.670663, -1.673520, -1.676287,
      -1.678457, -1.681199, -1.683860, -1.686089, -1.688347, -1.690627,
      -1.692619, -1.694252, -1.696605, -1.698985, -1.700781, -1.702242,
      -1.704575, -1.706587, -1.708096, -1.709700, -1.711444, -1.713117,
      -1.714204, -1.715542, -1.717560, -1.718647, -1.720077, -1.721485,
      -1.722444, -1.723418, -1.724406, -1.725408, -1.726426, -1.727459,
      -1.728508, -1.729574, -1.730000, -1.730757, -1.731875, -1.732000,
      -1.732169, -1.733159, -1.733704, -1.734000, -1.734000, -1.734000,
      -1.734000, -1.734175, -1.734713, -1.735000, -1.735000, -1.734870,
      -1.734465, -1.734060, -1.734000, -1.734000, -1.733991, -1.733679,
      -1.733367, -1.733054, -1.733000, -1.733000, -1.733000, -1.732946,
      -1.732699, -1.732451, -1.732203, -1.731958, -1.731728, -1.731498,
      -1.731267, -1.731037, -1.730791, -1.730542, -1.730293, -1.730044,
      -1.729780, -1.729513, -1.729245, -1.728977, -1.728691, -1.728405,
      -1.728119, -1.727823, -1.727518, -1.727214, -1.726904, -1.726582,
      -1.726259, -1.726000, -1.726000, -1.726000, -1.725904, -1.725544,
      -1.725185, -1.724816, -1.724438, -1.724060, -1.724000, -1.724000,
      -1.723867, -1.723452, -1.723037, -1.723000, -1.723000, -1.722727,
      -1.722275, -1.722000, -1.722000, -1.721870, -1.721381, -1.721000,
      -1.721000, -1.720869, -1.720343, -1.720000, -1.720000, -1.720000,
      -1.720000, -1.719574, -1.719000, -1.719000, -1.719000, -1.719000,
      -1.719000, -1.718898, -1.718243, -1.718000, -1.718000, -1.718000,
      -1.718000, -1.718000, -1.718000, -1.717313, -1.717000, -1.717000,
      -1.717000, -1.717000, -1.717000, -1.716549, -1.716000, -1.716000,
      -1.716000, -1.716000, -1.716000, -1.716000, -1.716000, -1.716000,
      -1.715338, -1.715000, -1.715000, -1.715000, -1.715000, -1.715000,
      -1.715000, -1.715000, -1.714803, -1.714000, -1.714000, -1.714000,
      -1.714000, -1.714000, -1.714000, -1.714000, -1.714000, -1.713564,
      -1.713000, -1.713000, -1.713000, -1.713000, -1.713000, -1.713000,
      -1.713000, -1.712000, -1.712000, -1.712000, -1.712000, -1.712000,
      -1.712000, -1.711000, -1.711000, -1.711000, -1.711000, -1.711000,
      -1.711000, -1.710000, -1.710000, -1.710000, -1.710000, -1.710000,
      -1.710000, -1.709000, -1.709000, -1.709000, -1.709000, -1.708415,
      -1.708000, -1.708000, -1.708000, -1.708000, -1.707299, -1.707000,
      -1.707000, -1.707000, -1.707000, -1.706337, -1.706000, -1.706000,
      -1.706000, -1.706000, -1.705374, -1.705000, -1.705000, -1.705000,
      -1.705000, -1.705000, -1.704441, -1.704000, -1.704000, -1.704000,
      -1.704000, -1.703513, -1.703000, -1.703000, -1.703000, -1.703000,
      -1.702434, -1.702000, -1.702000, -1.701955, -1.701211, -1.701000,
      -1.701000, -1.701000, -1.700933, -1.700118, -1.700000, -1.700000,
      -1.700000, -1.700000, -1.699144, -1.699000, -1.699000, -1.698739,
      NAN,       NAN,       -0.177925, -0.178544, -0.185202, -0.191773,
      -0.197870, -0.203950, -0.210019, -0.216080, -0.222124, -0.228163,
      -0.234187, -0.240198, -0.246200, -0.252186, -0.258169, -0.264134,
      -0.270088, -0.276036, -0.281967, -0.287893, -0.294806, -0.300707,
      -0.306603, -0.312485, -0.318364, -0.324231, -0.330089, -0.335942,
      -0.341785, -0.347983, -0.354398, -0.360276, -0.366092, -0.371906,
      -0.377710, -0.383512, -0.389309, -0.395608, -0.401886, -0.407666,
      -0.413439, -0.419209, -0.425768, -0.431730, -0.437485, -0.443352,
      -0.449665, -0.455703, -0.461426, -0.467147, -0.472859, -0.479562,
      -0.485256, -0.490947, -0.496887, -0.503128, -0.508962, -0.515541,
      -0.521260, -0.526900, -0.532681, -0.538861, -0.544759, -0.551032,
      -0.556951, -0.562538, -0.569113, -0.574677, -0.580486, -0.586788,
      -0.592561, -0.598648, -0.604390, -0.610297, -0.616362, -0.621924,
      -0.627461, -0.633497, -0.639393, -0.644877, -0.650125, -0.655062,
      -0.660335, -0.665825, -0.671320, -0.676815, -0.682314, -0.687818,
      -0.693059, -0.697938, -0.703616, -0.709139, -0.714971, -0.720017,
      -0.725289, -0.730763, -0.736360, -0.742223, -0.747825, -0.753411,
      -0.758950, -0.764494, -0.770213, -0.776407, -0.782336, -0.787954,
      -0.793528, -0.799479, -0.805048, -0.810526, -0.816570, -0.822817,
      -0.828464, -0.834055, -0.839717, -0.845372, -0.851269, -0.857558,
      -0.863196, -0.868814, -0.874427, -0.880320, -0.886622, -0.892261,
      -0.897893, -0.903609, -0.909367, -0.915093, -0.921102, -0.927194,
      -0.932885, -0.938311, -0.943731, -0.949141, -0.954996, -0.960936,
      -0.966331, -0.971784, -0.977694, -0.983456, -0.988818, -0.994386,
      -1.000267, -1.005856, -1.011188, -1.016510, -1.021828, -1.027138,
      -1.032442, -1.038207, -1.044024, -1.049304, -1.054575, -1.059837,
      -1.065096, -1.070345, -1.075585, -1.080816, -1.086043, -1.091259,
      -1.096464, -1.101663, -1.106854, -1.112033, -1.117203, -1.122359,
      -1.127509, -1.132649, -1.137779, -1.142934, -1.148096, -1.152427,
      -1.157494, -1.162726, -1.167490, -1.172255, -1.177441, -1.181782,
      -1.186590, -1.191560, -1.196548, -1.201100, -1.205972, -1.211175,
      -1.215906, -1.220279, -1.225589, -1.230081, -1.234513, -1.238950,
      -1.243408, -1.247883, -1.252370, -1.256870, -1.261391, -1.265917,
      -1.270346, -1.274020, -1.278590, -1.282627, -1.286420, -1.290448,
      -1.294986, -1.299042, -1.302942, -1.306431, -1.309925, -1.314232,
      -1.317929, -1.321442, -1.324957, -1.328476, -1.331998, -1.335522,
      -1.338539, -1.341572, -1.345097, -1.348621, -1.351429, -1.354217,
      -1.357175, -1.360453, -1.362785, -1.365669, -1.368431, -1.370739,
      -1.373043, -1.375413, -1.378256, -1.380436, -1.382118, -1.384354,
      -1.386576, -1.388778, -1.390466, -1.392064, -1.393272, -1.395295,
      -1.396867, -1.398287, -1.399314, -1.400327, -1.401324, -1.402306,
      -1.403282, -1.404236, -1.405174, -1.406095, -1.406999, -1.407881,
      -1.408388, -1.408837, -1.409586, -1.410000, -1.410455, -1.411410,
      -1.412000, -1.412281, -1.413048, -1.413815, -1.414662, -1.415643,
      -1.416701, -1.417774, -1.418863, -1.420458, -1.422086, -1.423332,
      -1.425060, -1.426534, -1.428075, -1.429860, -1.432172, -1.434012,
      -1.436152, -1.438663, -1.441095, -1.442759, -1.445354, -1.447489,
      -1.449319, -1.450915, -1.452776, -1.454708, -1.455702, -1.456901,
      -1.458267, -1.459476, -1.460295, -1.460816, -1.461291, -1.461742,
      -1.462000, -1.462000, -1.462000, -1.462303, -1.462647, -1.462991,
      -1.463000, -1.463000, -1.463000, -1.462816, -1.462546, -1.462276,
      -1.462006, -1.462000, -1.462000, -1.462000, -1.462000, -1.462000,
      -1.462000, -1.462000, -1.462000, -1.462000, -1.462000, -1.462000,
      -1.462000, -1.462000, -1.462000, -1.462000, -1.462000, -1.462000,
      -1.462000, -1.461888, -1.461659, -1.461430, -1.461201, -1.461000,
      -1.461000, -1.461000, -1.461000, -1.461000, -1.461000, -1.461000,
      -1.461000, -1.461000, -1.461000, -1.461000, -1.461000, -1.461000,
      -1.461000, -1.461000, -1.460835, -1.460513, -1.460192, -1.460000,
      -1.460000, -1.460000, -1.460000, -1.460000, -1.460000, -1.460000,
      -1.460000, -1.460000, -1.460000, -1.460000, -1.459798, -1.459384,
      -1.459000, -1.459000, -1.459000, -1.459000, -1.459000, -1.459000,
      -1.459000, -1.459000, -1.459000, -1.459000, -1.459000, -1.458800,
      -1.458275, -1.458000, -1.458000, -1.458000, -1.458000, -1.458000,
      -1.458000, -1.458000, -1.457718, -1.457102, -1.457000, -1.457000,
      -1.457000, -1.457000, -1.457000, -1.457000, -1.456435, -1.456000,
      -1.456000, -1.456000, -1.456000, -1.456000, -1.456000, -1.456000,
      -1.456000, -1.455481, -1.455000, -1.455000, -1.455000, -1.455000,
      -1.455000, -1.455000, -1.455000, -1.454252, -1.454000, -1.454000,
      -1.454000, -1.454000, -1.454000, -1.454000, -1.453990, -1.453000,
      -1.453000, -1.453000, -1.453000, -1.453000, -1.453000, -1.453000,
      -1.452452, -1.452000, -1.452000, -1.452000, -1.452000, -1.452000,
      -1.452000, -1.451000, -1.451000, -1.451000, -1.451000, -1.451000,
      -1.451000, -1.450613, -1.450000, -1.450000, -1.450000, -1.450000,
      -1.450000, -1.449272, -1.449000, -1.449000, -1.449000, -1.449000,
      -1.449000, -1.448000, -1.448000, -1.448000, -1.448000, -1.448000,
      -1.447065, -1.447000, -1.447000, -1.447000, -1.447000, -1.446000,
      -1.446000, -1.446000, -1.446000, -1.445406, -1.445000, -1.445000,
      -1.445000, -1.445000, -1.444474, -1.444000, -1.444000, -1.444000,
      -1.443357, -1.443000, -1.443000, -1.442961, -1.442131, -1.442000,
      -1.442000, -1.441637, -1.441000, -1.441000, -1.441000, -1.440499,
      -1.440000, -1.440000, -1.439951, -1.439137, -1.439000, -1.438665,
      -1.438000, -1.438000, -1.438000, -1.437442, -1.437000, -1.436720,
      -1.436000, -1.436000, -1.435442, -1.435000, -1.434773, -1.434023,
      -1.434000, -1.433332, -1.433000, -1.432729, NAN,       NAN,
      -0.062834, -0.063422, -0.069246, -0.075068, -0.081172, -0.087566,
      -0.093507, -0.099308, -0.105109, -0.111092, -0.117457, -0.123474,
      -0.129253, -0.135023, -0.140790, -0.146702, -0.153031, -0.159049,
      -0.164787, -0.170522, -0.177245, -0.182962, -0.188674, -0.194376,
      -0.200376, -0.206632, -0.212437, -0.218107, -0.223929, -0.230430,
      -0.236078, -0.241719, -0.248026, -0.253978, -0.259599, -0.265255,
      -0.271817, -0.277413, -0.283407, -0.289589, -0.295167, -0.301489,
      -0.307308, -0.313352, -0.319422, -0.324972, -0.330517, -0.336472,
      -0.342598, -0.348444, -0.354527, -0.360189, -0.366655, -0.372232,
      -0.377901, -0.384265, -0.389776, -0.395742, -0.401791, -0.407622,
      -0.413672, -0.419290, -0.425739, -0.431798, -0.437758, -0.443363,
      -0.449389, -0.455192, -0.461192, -0.467434, -0.473440, -0.479036,
      -0.485381, -0.490924, -0.496398, -0.502125, -0.507031, -0.512726,
      -0.518608, -0.524186, -0.529573, -0.535030, -0.540612, -0.545948,
      -0.551299, -0.556626, -0.561916, -0.567277, -0.573160, -0.578586,
      -0.583956, -0.589495, -0.595129, -0.600676, -0.606273, -0.611932,
      -0.617601, -0.623184, -0.628774, -0.634422, -0.640069, -0.645709,
      -0.651356, -0.656885, -0.662487, -0.668143, -0.673775, -0.679289,
      -0.684625, -0.690391, -0.696438, -0.701782, -0.707147, -0.713226,
      -0.718892, -0.724261, -0.729630, -0.734997, -0.740898, -0.746725,
      -0.752085, -0.757587, -0.763475, -0.769146, -0.774491, -0.780112,
      -0.785980, -0.791492, -0.796815, -0.802538, -0.808376, -0.813730,
      -0.819019, -0.824819, -0.830561, -0.835814, -0.841160, -0.846909,
      -0.852502, -0.858046, -0.863892, -0.869061, -0.874217, -0.879833,
      -0.885485, -0.890607, -0.895736, -0.901456, -0.907050, -0.912239,
      -0.917457, -0.922685, -0.927937, -0.933212, -0.938067, -0.942847,
      -0.948139, -0.952541, -0.957940, -0.963077, -0.967783, -0.972800,
      -0.977189, -0.982243, -0.987029, -0.991473, -0.995942, -1.000435,
      -1.004954, -1.009483, -1.014044, -1.018619, -1.023215, -1.026875,
      -1.030922, -1.035138, -1.039598, -1.043714, -1.047441, -1.051511,
      -1.055584, -1.059186, -1.062802, -1.066434, -1.069576, -1.072756,
      -1.076447, -1.080151, -1.082914, -1.086013, -1.089142, -1.092285,
      -1.094952, -1.098115, -1.101037, -1.103598, -1.106164, -1.108746,
      -1.111334, -1.113452, -1.115543, -1.117873, -1.119842, -1.121213,
      -1.123069, -1.124860, -1.126188, -1.127520, -1.128854, -1.129596,
      -1.130265, -1.130936, -1.131000, -1.131000, -1.131000, -1.131000,
      -1.131000, -1.131000, -1.130370, -1.129405, -1.128537, -1.127744,
      -1.126420, -1.126000, -1.124179, -1.122216, -1.120262, -1.117757,
      -1.115184, -1.113310, -1.111066, -1.108513, -1.105969, -1.103418,
      -1.100880, -1.098349, -1.095844, -1.093362, -1.090911, -1.089005,
      -1.086975, -1.085443, -1.084430, -1.083212, -1.083000, -1.083000,
      -1.083030, -1.084491, -1.085966, -1.088195, -1.090946, -1.094237,
      -1.098081, -1.102711, -1.107916, -1.113686, -1.119796, -1.125249,
      -1.131382, -1.137628, -1.143723, -1.149458, -1.154052, -1.158797,
      -1.162093, -1.164985, -1.167319, -1.168503, -1.169592, -1.170612,
      -1.171000, -1.171000, -1.171000, -1.171000, -1.171000, -1.170742,
      -1.170405, -1.170067, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.170000,
      -1.170000, -1.170000, -1.170000, -1.170000, -1.170000, -1.169981,
      -1.169725, -1.169468, -1.169211, -1.169000, -1.169000, -1.169000,
      -1.169000, -1.169000, -1.169000, -1.169000, -1.168956, -1.168643,
      -1.168331, -1.168019, -1.168000, -1.168000, -1.168000, -1.167680,
      -1.167331, -1.167000, -1.167000, -1.167000, -1.167000, -1.167000,
      -1.167000, -1.167000, -1.167000, -1.166886, -1.166463, -1.166040,
      -1.166000, -1.166000, -1.166000, -1.166000, -1.166000, -1.166000,
      -1.166000, -1.166000, -1.166000, -1.166000, -1.165775, -1.165241,
      -1.165000, -1.165000, -1.165000, -1.165000, -1.165000, -1.165000,
      -1.165000, -1.165000, -1.164971, -1.164327, -1.164000, -1.164000,
      -1.164000, -1.164000, -1.164000, -1.164000, -1.163487, -1.163000,
      -1.163000, -1.163000, -1.163000, -1.163000, -1.163000, -1.162982,
      -1.162135, -1.162000, -1.162000, -1.162000, -1.162000, -1.162000,
      -1.161714, -1.161000, -1.161000, -1.161000, -1.161000, -1.161000,
      -1.160659, -1.160000, -1.160000, -1.160000, -1.160000, -1.160000,
      -1.159000, -1.159000, -1.159000, -1.159000, -1.158952, -1.158000,
      -1.158000, -1.158000, -1.158000, -1.158000, -1.157000, -1.157000,
      -1.157000, -1.157000, -1.156000, -1.156000, -1.156000, -1.156000,
      -1.155000, -1.155000, -1.155000, -1.155000, -1.154000, -1.154000,
      -1.154000, -1.153062, -1.153000, -1.153000, -1.153000, -1.152000,
      -1.152000, -1.152000, -1.151000, -1.151000, -1.151000, -1.150000,
      -1.150000, -1.149683, -1.149000, -1.149000, -1.148410, -1.148000,
      -1.148000, -1.147000, -1.147000, -1.147000, -1.146000, -1.146000,
      -1.145479, -1.144751, -1.144011, -1.144000, -1.143232, -1.143000,
      -1.142648, -1.142000, -1.142000, -1.141160, -1.140438, -1.140000,
      -1.139824, -1.139097, -1.138194, -1.138000, -1.137630, -1.136848,
      -1.136051, -1.136000, -1.135408, -1.134564, -1.133754, -1.133027,
      -1.133000, -1.132358, -1.131588, -1.130804, -1.130006, -1.129189,
      -1.128359, -1.127582, -1.126820, -1.125952, -1.125185, -1.124403,
      -1.123609, -1.122801, NAN,       NAN,       0.039290,  0.038599,
      0.032810,  0.027121,  0.021437,  0.015756,  0.010077,  0.004400,
      -0.001701, -0.007942, -0.013618, -0.019286, -0.024955, -0.030623,
      -0.036615, -0.042843, -0.048611, -0.054270, -0.059927, -0.065583,
      -0.071459, -0.077675, -0.083536, -0.089182, -0.094827, -0.101466,
      -0.107103, -0.112738, -0.118368, -0.124293, -0.130477, -0.136234,
      -0.141846, -0.147547, -0.154057, -0.159655, -0.165247, -0.171401,
      -0.177413, -0.182989, -0.188559, -0.195121, -0.200677, -0.206230,
      -0.212130, -0.218313, -0.223848, -0.229377, -0.235287, -0.241357,
      -0.246928, -0.253433, -0.258932, -0.264429, -0.270304, -0.276405,
      -0.281886, -0.288038, -0.293838, -0.299309, -0.305731, -0.311251,
      -0.317010, -0.322673, -0.328596, -0.334659, -0.340279, -0.346136,
      -0.352196, -0.358351, -0.364123, -0.369361, -0.375497, -0.380705,
      -0.386431, -0.392180, -0.397691, -0.403438, -0.408495, -0.414452,
      -0.420194, -0.425435, -0.431667, -0.437382, -0.442636, -0.448414,
      -0.454530, -0.459961, -0.465633, -0.471294, -0.476898, -0.482426,
      -0.487876, -0.493999, -0.499490, -0.504740, -0.510480, -0.516016,
      -0.522060, -0.527739, -0.533091, -0.538432, -0.544717, -0.550082,
      -0.555398, -0.560700, -0.566092, -0.571900, -0.577548, -0.582810,
      -0.588265, -0.594029, -0.599532, -0.604751, -0.609955, -0.615152,
      -0.621007, -0.626512, -0.631673, -0.636817, -0.641957, -0.647898,
      -0.653192, -0.658308, -0.663453, -0.668601, -0.673778, -0.678975,
      -0.684206, -0.689439, -0.694706, -0.699995, -0.705310, -0.710308,
      -0.714990, -0.720137, -0.725000, -0.730199, -0.734921, -0.739722,
      -0.744665, -0.749156, -0.754569, -0.759038, -0.763517, -0.768029,
      -0.772557, -0.777104, -0.781686, -0.786276, -0.790897, -0.795353,
      -0.799439, -0.803905, -0.808176, -0.812364, -0.816994, -0.821191,
      -0.825061, -0.828701, -0.832351, -0.836031, -0.839731, -0.843447,
      -0.847195, -0.850953, -0.854452, -0.857622, -0.860819, -0.863821,
      -0.866761, -0.870004, -0.872625, -0.875255, -0.877432, -0.879574,
      -0.881947, -0.883972, -0.886016, -0.887713, -0.889090, -0.890479,
      -0.891874, -0.893281, -0.894348, -0.895117, -0.896276, -0.896996,
      -0.897000, -0.897000, -0.897000, -0.897000, -0.896829, -0.896081,
      -0.895328, -0.894141, -0.892619, -0.891088, -0.889548, -0.887502,
      -0.885173, -0.882835, -0.880487, -0.877509, -0.874359, -0.870998,
      -0.867035, -0.863058, -0.859073, -0.854696, -0.849894, -0.845072,
      -0.839401, -0.833650, -0.828290, -0.822968, -0.817041, -0.811083,
      -0.805941, -0.800104, -0.794673, -0.789553, -0.784707, -0.780465,
      -0.776565, -0.773336, -0.770690, -0.767996, -0.767159, -0.767000,
      -0.767948, -0.769669, -0.773711, -0.778884, -0.785010, -0.792858,
      -0.801998, -0.811553, -0.821892, -0.832172, -0.841623, -0.850762,
      -0.858046, -0.864163, -0.869793, -0.874345, -0.877086, -0.879176,
      -0.880939, -0.881489, -0.881996, -0.881564, -0.881125, -0.880726,
      -0.880343, -0.879965, -0.879629, -0.879292, -0.879000, -0.879000,
      -0.879000, -0.879000, -0.879000, -0.879000, -0.879000, -0.879000,
      -0.879000, -0.879000, -0.879000, -0.879000, -0.879000, -0.879000,
      -0.879000, -0.879000, -0.879000, -0.879000, -0.879000, -0.879000,
      -0.879000, -0.879000, -0.879000, -0.879000, -0.879000, -0.879000,
      -0.879000, -0.878928, -0.878761, -0.878594, -0.878427, -0.878261,
      -0.878094, -0.878000, -0.878000, -0.878000, -0.878000, -0.878000,
      -0.878000, -0.877997, -0.877850, -0.877703, -0.877556, -0.877409,
      -0.877263, -0.877116, -0.876965, -0.876800, -0.876636, -0.876471,
      -0.876306, -0.876141, -0.876000, -0.876000, -0.876000, -0.876000,
      -0.876000, -0.876000, -0.875861, -0.875659, -0.875458, -0.875256,
      -0.875054, -0.875000, -0.875000, -0.875000, -0.875000, -0.875000,
      -0.875000, -0.875000, -0.875000, -0.875000, -0.874742, -0.874485,
      -0.874228, -0.874000, -0.874000, -0.874000, -0.874000, -0.874000,
      -0.874000, -0.874000, -0.874000, -0.874000, -0.874000, -0.874000,
      -0.873707, -0.873376, -0.873045, -0.873000, -0.873000, -0.873000,
      -0.873000, -0.873000, -0.873000, -0.873000, -0.873000, -0.872718,
      -0.872313, -0.872000, -0.872000, -0.872000, -0.872000, -0.872000,
      -0.871723, -0.871263, -0.871000, -0.871000, -0.871000, -0.871000,
      -0.871000, -0.871000, -0.870793, -0.870259, -0.870000, -0.870000,
      -0.870000, -0.870000, -0.869437, -0.869000, -0.869000, -0.869000,
      -0.869000, -0.869000, -0.868691, -0.868028, -0.868000, -0.868000,
      -0.868000, -0.868000, -0.867505, -0.867000, -0.867000, -0.867000,
      -0.867000, -0.866644, -0.866000, -0.866000, -0.866000, -0.865290,
      -0.865000, -0.865000, -0.865000, -0.864677, -0.864000, -0.864000,
      -0.864000, -0.863788, -0.863000, -0.863000, -0.863000, -0.862600,
      -0.862000, -0.862000, -0.861251, -0.861000, -0.861000, -0.860710,
      -0.860000, -0.860000, -0.859970, -0.859000, -0.859000, -0.859000,
      -0.858000, -0.858000, -0.857833, -0.857000, -0.857000, -0.856414,
      -0.856000, -0.856000, -0.855000, -0.855000, -0.854000, -0.854000,
      -0.853103, -0.853000, -0.852592, -0.852000, -0.851951, -0.851000,
      -0.851000, -0.850000, -0.850000, -0.849000, -0.849000, -0.848000,
      -0.847962, -0.847000, -0.846581, -0.846000, -0.845037, -0.845000,
      -0.844000, -0.843000, -0.843000, -0.842000, -0.841350, -0.841000,
      -0.840000, -0.839375, -0.839000, -0.838000, -0.837000, -0.837000,
      -0.836000, -0.835000, -0.834000, -0.833000, -0.832652, -0.831839,
      -0.831011, -0.830165, -0.829441, -0.828354, -0.826827, -0.826100,
      -0.825199, -0.824402, -0.823267, -0.821851, -0.821054, -0.820240,
      -0.818823, -0.817568, -0.816757, -0.816030, -0.814288, -0.813361,
      -0.812181, -0.810613, -0.809017, -0.808192, -0.806723, -0.805169,
      -0.803823, -0.802908, -0.801374, -0.799812, -0.798223, -0.796606,
      NAN,       NAN,       0.131344,  0.130772,  0.125111,  0.119458,
      0.113806,  0.108160,  0.102521,  0.096887,  0.091255,  0.085390,
      0.079203,  0.073382,  0.067764,  0.062149,  0.056535,  0.050924,
      0.045316,  0.039251,  0.033105,  0.027503,  0.021902,  0.016301,
      0.010701,  0.005101,  -0.000798, -0.007096, -0.012693, -0.018291,
      -0.023887, -0.029483, -0.036078, -0.041673, -0.047265, -0.052857,
      -0.058692, -0.064841, -0.070624, -0.076208, -0.082269, -0.088372,
      -0.093949, -0.099524, -0.105098, -0.111200, -0.117231, -0.122793,
      -0.128352, -0.134395, -0.140456, -0.146000, -0.151693, -0.157781,
      -0.163893, -0.169782, -0.175615, -0.181651, -0.187610, -0.193053,
      -0.198795, -0.204753, -0.210804, -0.216854, -0.222521, -0.228246,
      -0.234052, -0.240019, -0.246015, -0.251726, -0.257312, -0.262809,
      -0.268511, -0.274555, -0.280058, -0.285646, -0.291422, -0.297089,
      -0.302439, -0.307898, -0.313473, -0.319039, -0.324589, -0.330490,
      -0.335862, -0.341233, -0.346594, -0.352146, -0.358033, -0.363651,
      -0.368991, -0.374323, -0.379651, -0.385765, -0.391281, -0.396582,
      -0.401880, -0.407197, -0.413439, -0.418700, -0.423958, -0.429199,
      -0.434429, -0.439808, -0.445530, -0.451040, -0.456216, -0.461388,
      -0.466564, -0.471756, -0.477567, -0.482895, -0.487624, -0.492749,
      -0.498050, -0.503383, -0.508721, -0.513090, -0.518436, -0.523318,
      -0.528342, -0.533446, -0.538072, -0.543036, -0.547683, -0.552921,
      -0.557619, -0.562107, -0.566627, -0.571175, -0.575734, -0.580322,
      -0.584572, -0.588623, -0.593198, -0.597304, -0.601623, -0.606070,
      -0.610234, -0.613932, -0.618144, -0.622181, -0.625840, -0.629514,
      -0.633211, -0.636940, -0.640681, -0.644047, -0.647203, -0.650389,
      -0.653905, -0.657316, -0.660445, -0.663063, -0.665692, -0.668932,
      -0.671505, -0.673703, -0.676409, -0.679139, -0.681417, -0.683327,
      -0.685087, -0.687129, -0.688547, -0.689976, -0.691413, -0.692433,
      -0.693164, -0.693901, -0.694646, -0.695000, -0.695000, -0.695000,
      -0.694313, -0.694000, -0.693253, -0.692463, -0.691667, -0.690592,
      -0.688161, -0.685704, -0.683022, -0.680252, -0.676936, -0.673539,
      -0.670114, -0.665990, -0.660971, -0.656497, -0.651184, -0.645813,
      -0.640425, -0.634988, -0.629430, -0.622993, -0.616798, -0.610439,
      -0.603651, -0.596515, -0.589486, -0.582617, -0.575287, -0.567123,
      -0.559305, -0.551354, -0.543539, -0.536009, -0.528560, -0.521093,
      -0.515059, -0.508926, -0.503451, -0.498756, -0.494993, -0.492664,
      -0.492000, -0.493455, -0.497166, -0.502800, -0.510716, -0.520891,
      -0.532183, -0.543924, -0.556261, -0.568126, -0.578224, -0.587387,
      -0.595077, -0.600822, -0.605342, -0.608884, -0.611288, -0.612729,
      -0.613764, -0.614346, -0.614794, -0.615000, -0.615000, -0.615000,
      -0.614665, -0.614322, -0.614000, -0.614000, -0.614000, -0.614000,
      -0.614000, -0.614000, -0.614000, -0.613994, -0.613753, -0.613512,
      -0.613271, -0.613030, -0.613000, -0.613000, -0.613000, -0.613000,
      -0.612945, -0.612747, -0.612549, -0.612351, -0.612152, -0.612000,
      -0.612000, -0.612000, -0.612000, -0.612000, -0.612000, -0.611876,
      -0.611707, -0.611539, -0.611370, -0.611202, -0.611033, -0.610873,
      -0.610716, -0.610559, -0.610401, -0.610244, -0.610087, -0.609934,
      -0.609786, -0.609638, -0.609490, -0.609342, -0.609194, -0.609046,
      -0.609000, -0.609000, -0.609000, -0.609000, -0.609000, -0.609000,
      -0.609000, -0.608924, -0.608789, -0.608653, -0.608518, -0.608383,
      -0.608247, -0.608112, -0.608000, -0.608000, -0.608000, -0.608000,
      -0.608000, -0.608000, -0.608000, -0.607884, -0.607712, -0.607540,
      -0.607368, -0.607195, -0.607023, -0.607000, -0.607000, -0.607000,
      -0.607000, -0.607000, -0.606869, -0.606660, -0.606451, -0.606242,
      -0.606032, -0.606000, -0.606000, -0.606000, -0.606000, -0.606000,
      -0.606000, -0.606000, -0.606000, -0.605896, -0.605632, -0.605367,
      -0.605102, -0.605000, -0.605000, -0.605000, -0.605000, -0.605000,
      -0.605000, -0.605000, -0.604755, -0.604435, -0.604115, -0.604000,
      -0.604000, -0.604000, -0.603755, -0.603398, -0.603041, -0.603000,
      -0.603000, -0.602913, -0.602519, -0.602125, -0.602000, -0.602000,
      -0.602000, -0.602000, -0.602000, -0.601580, -0.601130, -0.601000,
      -0.601000, -0.600722, -0.600236, -0.600000, -0.600000, -0.600000,
      -0.600000, -0.599663, -0.599122, -0.599000, -0.599000, -0.598428,
      -0.598000, -0.598000, -0.597639, -0.597024, -0.597000, -0.597000,
      -0.597000, -0.596430, -0.596000, -0.596000, -0.595357, -0.595000,
      -0.595000, -0.595000, -0.594408, -0.594000, -0.593852, -0.593052,
      -0.593000, -0.593000, -0.592556, -0.592000, -0.591817, -0.591000,
      -0.591000, -0.590083, -0.590000, -0.590000, -0.589193, -0.589000,
      -0.588175, -0.588000, -0.588000, -0.587007, -0.586911, -0.586000,
      -0.586000, -0.585497, -0.585000, -0.584116, -0.584000, -0.583644,
      -0.583000, -0.582080, -0.582000, -0.581418, -0.581000, -0.580000,
      -0.580000, -0.579000, -0.578320, -0.577820, -0.577000, -0.576736,
      -0.576000, -0.575000, -0.574891, -0.574000, -0.573000, -0.572769,
      -0.571998, -0.571000, -0.570000, -0.569483, -0.568577, -0.568000,
      -0.567000, -0.566000, -0.565587, -0.564496, -0.564000, -0.563000,
      -0.561987, -0.561000, -0.560000, -0.559000, -0.558000, -0.557000,
      -0.556000, -0.555000, -0.553756, -0.553000, -0.551488, -0.550000,
      -0.549022, -0.548000, -0.547000, -0.545435, -0.544000, -0.543000,
      -0.541359, -0.540000, -0.539000, -0.537000, -0.536000, -0.534070,
      -0.532541, -0.531243, -0.529773, -0.527965, -0.526478, -0.524450,
      -0.522430, -0.520833, -0.518838, -0.516800, -0.514985, -0.513299,
      -0.511373, -0.508791, -0.506559, -0.504286, -0.501975, -0.499749,
      -0.498150, -0.495037, -0.492286, -0.490167, -0.488049, -0.485050,
      -0.482361, -0.479285, -0.476106, -0.472870, NAN,       NAN,
      0.213560,  0.212987,  0.206578,  0.200652,  0.194990,  0.189336,
      0.183690,  0.178046,  0.172409,  0.166779,  0.161151,  0.155529,
      0.149915,  0.144302,  0.138694,  0.132900,  0.126742,  0.120896,
      0.115306,  0.109718,  0.104131,  0.098549,  0.092970,  0.087393,
      0.081819,  0.076246,  0.070674,  0.065105,  0.059091,  0.052970,
      0.047403,  0.041837,  0.036272,  0.030709,  0.025144,  0.019578,
      0.014012,  0.007536,  0.001880,  -0.003688, -0.009255, -0.014823,
      -0.020392, -0.025961, -0.031893, -0.038034, -0.043829, -0.049220,
      -0.054787, -0.060674, -0.066419, -0.071972, -0.078084, -0.083599,
      -0.089225, -0.095494, -0.101072, -0.107046, -0.112726, -0.118332,
      -0.124010, -0.130027, -0.135789, -0.141152, -0.146794, -0.152730,
      -0.158198, -0.163656, -0.169509, -0.175492, -0.180965, -0.186382,
      -0.191783, -0.197182, -0.202880, -0.208938, -0.214298, -0.219650,
      -0.224987, -0.230309, -0.235624, -0.241712, -0.247214, -0.252484,
      -0.257751, -0.263002, -0.268236, -0.273455, -0.278676, -0.283912,
      -0.289169, -0.294448, -0.299748, -0.305060, -0.310398, -0.315486,
      -0.320328, -0.325541, -0.330966, -0.335979, -0.340895, -0.345340,
      -0.350313, -0.355438, -0.360603, -0.365340, -0.370392, -0.375080,
      -0.379621, -0.384168, -0.388746, -0.393348, -0.397971, -0.402610,
      -0.407284, -0.411967, -0.415600, -0.419752, -0.424191, -0.428615,
      -0.432824, -0.436565, -0.440833, -0.444814, -0.448516, -0.452229,
      -0.455974, -0.459740, -0.463524, -0.466786, -0.469982, -0.473200,
      -0.476924, -0.479564, -0.482490, -0.485794, -0.488377, -0.490392,
      -0.492898, -0.495621, -0.498030, -0.500070, -0.501467, -0.503745,
      -0.505294, -0.506726, -0.508167, -0.509624, -0.510547, -0.511288,
      -0.512000, -0.512292, -0.513000, -0.513000, -0.513000, -0.512613,
      -0.511821, -0.511021, -0.510211, -0.508785, -0.507132, -0.505462,
      -0.503699, -0.501418, -0.499013, -0.496399, -0.493675, -0.490112,
      -0.486507, -0.482853, -0.478973, -0.474317, -0.469583, -0.464493,
      -0.459039, -0.453203, -0.447000, -0.440394, -0.432444, -0.424929,
      -0.417314, -0.409502, -0.400750, -0.391779, -0.383163, -0.374482,
      -0.365351, -0.355356, -0.345072, -0.334524, -0.324172, -0.314363,
      -0.304849, -0.294750, -0.285134, -0.275747, -0.266976, -0.258979,
      -0.253307, -0.249578, -0.248000, -0.248646, -0.252448, -0.259184,
      -0.269075, -0.280043, -0.292506, -0.305942, -0.318585, -0.329700,
      -0.340000, -0.348589, -0.355848, -0.360951, -0.365472, -0.369414,
      -0.371563, -0.373308, -0.374237, -0.375072, -0.375476, -0.375880,
      -0.376000, -0.376000, -0.376000, -0.375727, -0.375416, -0.375104,
      -0.374816, -0.374539, -0.374263, -0.374000, -0.374000, -0.374000,
      -0.374000, -0.373998, -0.373775, -0.373552, -0.373329, -0.373107,
      -0.373000, -0.373000, -0.373000, -0.373000, -0.373000, -0.373000,
      -0.373000, -0.373000, -0.373000, -0.373000, -0.372964, -0.372793,
      -0.372621, -0.372449, -0.372277, -0.372106, -0.372000, -0.372000,
      -0.372000, -0.372000, -0.372000, -0.372000, -0.371980, -0.371830,
      -0.371679, -0.371529, -0.371379, -0.371229, -0.371079, -0.371000,
      -0.371000, -0.371000, -0.371000, -0.371000, -0.371000, -0.371000,
      -0.370941, -0.370806, -0.370671, -0.370535, -0.370400, -0.370265,
      -0.370130, -0.369995, -0.369865, -0.369735, -0.369606, -0.369476,
      -0.369347, -0.369217, -0.369087, -0.368957, -0.368826, -0.368695,
      -0.368564, -0.368432, -0.368301, -0.368170, -0.368038, -0.367894,
      -0.367744, -0.367594, -0.367445, -0.367295, -0.367145, -0.366995,
      -0.366827, -0.366658, -0.366490, -0.366322, -0.366154, -0.365984,
      -0.365797, -0.365610, -0.365424, -0.365237, -0.365050, -0.365000,
      -0.365000, -0.365000, -0.365000, -0.365000, -0.364808, -0.364584,
      -0.364361, -0.364137, -0.364000, -0.364000, -0.364000, -0.364000,
      -0.363933, -0.363672, -0.363412, -0.363151, -0.362883, -0.362604,
      -0.362325, -0.362046, -0.362000, -0.362000, -0.362000, -0.361850,
      -0.361534, -0.361218, -0.361000, -0.361000, -0.361000, -0.360887,
      -0.360534, -0.360181, -0.359819, -0.359448, -0.359076, -0.359000,
      -0.359000, -0.358907, -0.358498, -0.358090, -0.358000, -0.358000,
      -0.357806, -0.357361, -0.356912, -0.356448, -0.356000, -0.356000,
      -0.356000, -0.355520, -0.355019, -0.354501, -0.354000, -0.354000,
      -0.353903, -0.353347, -0.353000, -0.353000, -0.352623, -0.352031,
      -0.351420, -0.351000, -0.351000, -0.350530, -0.350000, -0.350000,
      -0.349532, -0.348843, -0.348139, -0.348000, -0.347691, -0.347000,
      -0.347000, -0.346417, -0.345631, -0.345000, -0.345000, -0.344185,
      -0.343337, -0.343000, -0.342597, -0.342000, -0.341792, -0.340864,
      -0.340000, -0.339956, -0.339000, -0.338975, -0.337957, -0.337000,
      -0.336863, -0.335787, -0.335000, -0.334574, -0.334000, -0.333278,
      -0.332099, -0.331897, -0.330672, -0.330000, -0.329155, -0.328000,
      -0.327542, -0.327000, -0.325832, -0.325000, -0.324020, -0.323000,
      -0.322101, -0.321000, -0.320073, -0.319516, -0.318000, -0.317316,
      -0.316000, -0.314996, -0.314000, -0.312549, -0.311778, -0.310000,
      -0.309137, -0.308000, -0.306357, -0.305415, -0.304000, -0.302421,
      -0.301368, -0.300277, -0.298148, -0.296978, -0.295767, -0.293515,
      -0.292222, -0.290886, -0.289000, -0.287083, -0.285613, -0.284099,
      -0.282000, -0.279927, -0.278268, -0.276000, -0.274000, -0.271994,
      -0.270131, -0.268000, -0.265246, -0.263221, -0.261140, -0.259000,
      -0.256000, -0.253547, -0.251230, -0.248000, -0.245408, -0.242903,
      -0.240000, -0.236693, -0.233735, -0.230758, -0.227354, -0.223556,
      -0.220365, -0.217108, -0.213243, -0.209421, -0.205731, -0.201364,
      -0.197723, -0.193353, -0.188729, -0.184026, -0.179234, -0.174337,
      -0.169354, -0.163828, -0.157845, -0.151848, -0.145708, -0.139460,
      -0.133105, -0.126468, -0.119060, NAN,       0.286858,  0.286281,
      0.280579,  0.274880,  0.269187,  0.263500,  0.257819,  0.252141,
      0.246473,  0.240807,  0.235147,  0.228496,  0.222850,  0.217207,
      0.211573,  0.205943,  0.200319,  0.194700,  0.189089,  0.183479,
      0.177876,  0.172279,  0.166684,  0.161094,  0.155510,  0.149930,
      0.144353,  0.138780,  0.133051,  0.126642,  0.121078,  0.115515,
      0.109955,  0.104398,  0.098842,  0.093290,  0.087738,  0.082190,
      0.076643,  0.071099,  0.065585,  0.060107,  0.054442,  0.048709,
      0.043059,  0.037555,  0.031930,  0.026199,  0.020317,  0.014720,
      0.009444,  0.003822,  -0.001764, -0.007281, -0.013225, -0.019149,
      -0.024316, -0.030030, -0.035757, -0.041290, -0.046744, -0.052190,
      -0.057626, -0.063048, -0.068469, -0.073877, -0.079397, -0.085318,
      -0.091029, -0.096389, -0.101733, -0.107069, -0.112392, -0.117698,
      -0.123007, -0.128318, -0.133655, -0.139012, -0.143862, -0.148795,
      -0.154098, -0.159001, -0.164128, -0.169616, -0.174819, -0.179795,
      -0.184372, -0.189793, -0.194324, -0.198818, -0.203510, -0.208621,
      -0.213442, -0.218024, -0.222635, -0.227253, -0.231903, -0.236577,
      -0.240699, -0.244988, -0.249722, -0.253426, -0.257622, -0.262091,
      -0.266570, -0.270821, -0.274518, -0.278211, -0.281919, -0.285657,
      -0.289405, -0.293186, -0.296986, -0.300344, -0.303655, -0.307277,
      -0.310523, -0.314234, -0.316871, -0.319906, -0.323191, -0.325885,
      -0.328594, -0.330746, -0.333078, -0.335846, -0.337482, -0.339594,
      -0.341726, -0.343582, -0.345022, -0.346481, -0.347948, -0.349432,
      -0.350464, -0.351218, -0.351979, -0.352749, -0.353526, -0.354000,
      -0.353898, -0.353098, -0.353000, -0.352470, -0.351643, -0.350615,
      -0.348963, -0.348107, -0.345726, -0.343733, -0.341612, -0.339230,
      -0.335718, -0.332785, -0.328660, -0.324148, -0.319696, -0.314860,
      -0.309627, -0.304020, -0.297985, -0.291878, -0.284669, -0.277681,
      -0.270405, -0.263101, -0.255556, -0.247264, -0.237892, -0.227633,
      -0.217207, -0.206903, -0.196229, -0.185116, -0.173010, -0.160820,
      -0.148304, -0.134949, -0.121866, -0.108306, -0.094769, -0.081458,
      -0.068720, -0.056437, -0.045495, -0.035956, -0.028916, -0.023968,
      -0.023044, -0.025844, -0.031548, -0.039898, -0.051125, -0.063865,
      -0.077378, -0.091001, -0.104226, -0.116693, -0.127831, -0.137078,
      -0.145290, -0.151126, -0.156262, -0.160720, -0.163715, -0.166122,
      -0.167531, -0.168367, -0.169089, -0.169454, -0.169819, -0.169838,
      -0.169517, -0.169196, -0.169000, -0.169000, -0.169000, -0.169000,
      -0.169000, -0.169000, -0.169000, -0.169000, -0.168785, -0.168556,
      -0.168328, -0.168099, -0.167882, -0.167675, -0.167468, -0.167260,
      -0.167053, -0.166859, -0.166669, -0.166479, -0.166290, -0.166100,
      -0.165917, -0.165742, -0.165567, -0.165392, -0.165217, -0.165042,
      -0.164753, -0.164428, -0.164103, -0.163777, -0.163452, -0.163127,
      -0.162814, -0.162509, -0.162204, -0.161899, -0.161594, -0.161289,
      -0.160978, -0.160546, -0.160114, -0.159682, -0.159251, -0.158819,
      -0.158387, -0.157958, -0.157547, -0.157137, -0.156726, -0.156316,
      -0.155905, -0.155495, -0.155085, -0.154688, -0.154296, -0.153903,
      -0.153510, -0.153118, -0.152725, -0.152332, -0.151961, -0.151709,
      -0.151458, -0.151206, -0.150954, -0.150702, -0.150450, -0.150198,
      -0.149948, -0.149704, -0.149461, -0.149217, -0.148973, -0.148730,
      -0.148486, -0.148242, -0.147999, -0.147865, -0.147731, -0.147596,
      -0.147462, -0.147328, -0.147194, -0.147059, -0.146915, -0.146762,
      -0.146609, -0.146456, -0.146303, -0.146151, -0.145998, -0.145826,
      -0.145655, -0.145484, -0.145313, -0.145141, -0.144967, -0.144777,
      -0.144587, -0.144398, -0.144208, -0.144018, -0.143812, -0.143603,
      -0.143395, -0.143187, -0.142977, -0.142750, -0.142524, -0.142297,
      -0.142070, -0.141831, -0.141586, -0.141340, -0.141095, -0.140839,
      -0.140575, -0.140312, -0.140048, -0.139769, -0.139487, -0.139205,
      -0.139000, -0.139000, -0.139000, -0.139000, -0.138698, -0.138379,
      -0.138060, -0.137727, -0.137389, -0.137052, -0.136698, -0.136343,
      -0.135986, -0.135612, -0.135237, -0.134856, -0.134463, -0.134070,
      -0.133662, -0.133251, -0.132832, -0.132403, -0.131972, -0.131524,
      -0.131075, -0.130612, -0.130145, -0.130000, -0.130000, -0.129684,
      -0.129180, -0.128665, -0.128143, -0.127607, -0.127067, -0.126510,
      -0.125949, -0.125372, -0.124788, -0.124192, -0.123583, -0.122968,
      -0.122335, -0.122000, -0.122000, -0.121374, -0.120696, -0.120008,
      -0.119301, -0.118584, -0.117855, -0.117112, -0.116353, -0.115581,
      -0.114795, -0.113996, -0.113179, -0.112347, -0.111501, -0.111000,
      -0.110761, -0.109867, -0.108956, -0.108028, -0.107082, -0.106118,
      -0.105137, -0.104138, -0.103120, -0.102083, -0.101027, -0.099951,
      -0.098854, -0.097737, -0.096600, -0.095441, -0.094262, -0.093061,
      -0.091836, -0.090588, -0.089317, -0.089000, -0.087705, -0.086362,
      -0.084996, -0.083601, -0.082183, -0.080736, -0.079264, -0.077764,
      -0.076236, -0.074679, -0.073095, -0.070958, -0.068834, -0.067159,
      -0.065451, -0.063713, -0.061942, -0.060138, -0.058300, -0.056428,
      -0.054521, -0.052157, -0.049600, -0.047585, -0.045531, -0.043440,
      -0.040620, -0.038141, -0.035931, -0.033358, -0.030386, -0.028050,
      -0.024670, -0.022246, -0.019554, -0.016262, -0.013399, -0.010091,
      -0.006432, -0.003448, 0.000033,  0.003844,  0.007706,  0.011621,
      0.015591,  0.019616,  0.023697,  0.027835,  0.033032,  0.037289,
      0.042214,  0.046986,  0.052428,  0.057868,  0.063506,  0.069144,
      0.074699,  0.080250,  0.085941,  0.091625,  0.098012,  0.104524,
      0.110632,  0.116896,  0.123288,  0.128387,  0.133721,  0.140225,
      0.147862,  0.156484,  0.165275,  NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       0.353274,  0.352693,  0.347457,  0.342213,
      0.336484,  0.330758,  0.325036,  0.319321,  0.313174,  0.306907,
      0.301212,  0.295521,  0.289833,  0.284153,  0.278481,  0.272812,
      0.267149,  0.261495,  0.255843,  0.250200,  0.244563,  0.238929,
      0.233303,  0.227685,  0.222069,  0.216463,  0.210864,  0.205267,
      0.199680,  0.194031,  0.188325,  0.182997,  0.176988,  0.171562,
      0.166274,  0.160822,  0.154854,  0.149158,  0.143680,  0.138334,
      0.132937,  0.127456,  0.121899,  0.116580,  0.111118,  0.105678,
      0.100327,  0.094651,  0.089215,  0.083516,  0.077996,  0.072976,
      0.067676,  0.062330,  0.056946,  0.051570,  0.046201,  0.040843,
      0.035499,  0.030156,  0.024804,  0.019489,  0.014633,  0.009633,
      0.004203,  -0.001026, -0.005949, -0.011209, -0.016356, -0.021332,
      -0.026812, -0.031353, -0.036403, -0.041298, -0.046274, -0.051364,
      -0.055922, -0.060499, -0.065619, -0.070721, -0.075366, -0.080030,
      -0.084709, -0.088993, -0.093140, -0.097892, -0.102668, -0.106844,
      -0.110874, -0.115121, -0.119870, -0.124156, -0.128107, -0.131824,
      -0.135734, -0.140308, -0.144082, -0.147875, -0.151083, -0.154540,
      -0.158401, -0.162240, -0.165504, -0.168425, -0.171589, -0.174910,
      -0.178256, -0.181099, -0.183817, -0.186552, -0.189303, -0.192079,
      -0.193936, -0.196020, -0.198150, -0.200294, -0.202460, -0.204094,
      -0.205563, -0.207044, -0.208537, -0.210021, -0.210782, -0.211551,
      -0.212325, -0.213000, -0.213000, -0.213000, -0.213000, -0.213000,
      -0.212856, -0.212022, -0.211178, -0.209649, -0.207922, -0.205761,
      -0.203203, -0.200814, -0.198195, -0.195435, -0.192178, -0.188408,
      -0.184577, -0.179866, -0.175200, -0.170449, -0.165272, -0.159800,
      -0.153653, -0.147294, -0.140508, -0.133116, -0.124259, -0.115212,
      -0.105815, -0.096118, -0.086063, -0.075708, -0.064137, -0.051974,
      -0.039442, -0.026304, -0.012181, 0.001899,  0.016560,  0.031753,
      0.047472,  0.063172,  0.079758,  0.095389,  0.111140,  0.126806,
      0.141672,  0.155046,  0.166736,  0.175730,  0.181247,  0.183523,
      0.180389,  0.173197,  0.162874,  0.149798,  0.133843,  0.116306,
      0.098837,  0.081856,  0.065752,  0.051188,  0.040803,  0.031935,
      0.026281,  0.021926,  0.018322,  0.016257,  0.016000,  0.016093,
      0.016603,  0.017097,  0.017538,  0.017979,  0.018367,  0.018751,
      0.019119,  0.019457,  0.019794,  0.020348,  0.021242,  0.022137,
      0.023018,  0.023549,  0.024079,  0.024610,  0.025189,  0.025903,
      0.026617,  0.027331,  0.028040,  0.028686,  0.029331,  0.029977,
      0.030623,  0.031326,  0.032111,  0.032896,  0.033680,  0.034465,
      0.035230,  0.035951,  0.036673,  0.037395,  0.038116,  0.038838,
      0.039518,  0.040187,  0.040856,  0.041525,  0.042194,  0.042862,
      0.043372,  0.043841,  0.044310,  0.044778,  0.045247,  0.045716,
      0.046173,  0.046615,  0.047056,  0.047497,  0.047938,  0.048379,
      0.048820,  0.049083,  0.049222,  0.049362,  0.049501,  0.049640,
      0.049780,  0.049919,  0.050112,  0.050378,  0.050644,  0.050910,
      0.051176,  0.051443,  0.051709,  0.051975,  0.052116,  0.052243,
      0.052371,  0.052499,  0.052627,  0.052754,  0.052882,  0.053019,
      0.053266,  0.053513,  0.053759,  0.054006,  0.054253,  0.054499,
      0.054746,  0.054993,  0.055116,  0.055236,  0.055355,  0.055475,
      0.055595,  0.055714,  0.055834,  0.055954,  0.056079,  0.056207,
      0.056336,  0.056464,  0.056592,  0.056721,  0.056849,  0.056978,
      0.057243,  0.057536,  0.057830,  0.058124,  0.058418,  0.058711,
      0.059003,  0.059168,  0.059334,  0.059499,  0.059664,  0.059830,
      0.059995,  0.060178,  0.060362,  0.060546,  0.060730,  0.060914,
      0.061108,  0.061310,  0.061512,  0.061714,  0.061917,  0.062260,
      0.062702,  0.063143,  0.063585,  0.064014,  0.064253,  0.064493,
      0.064732,  0.064971,  0.065227,  0.065484,  0.065742,  0.066000,
      0.066276,  0.066552,  0.066828,  0.067223,  0.067813,  0.068402,
      0.068991,  0.069308,  0.069621,  0.069935,  0.070262,  0.070594,
      0.070925,  0.071271,  0.071621,  0.071971,  0.072338,  0.072707,
      0.073079,  0.073466,  0.073853,  0.074252,  0.074657,  0.075130,
      0.075978,  0.076826,  0.077351,  0.077794,  0.078246,  0.078707,
      0.079174,  0.079654,  0.080138,  0.080636,  0.081138,  0.081655,
      0.082354,  0.083423,  0.084255,  0.084808,  0.085373,  0.085944,
      0.086533,  0.087127,  0.087735,  0.088708,  0.089962,  0.090626,
      0.091279,  0.091943,  0.092623,  0.093628,  0.095015,  0.095734,
      0.096465,  0.097208,  0.097964,  0.099476,  0.100525,  0.101325,
      0.102140,  0.102970,  0.104635,  0.105680,  0.106559,  0.107907,
      0.109365,  0.110293,  0.111476,  0.113201,  0.114182,  0.115363,
      0.117199,  0.118237,  0.119587,  0.121370,  0.122466,  0.124164,
      0.125719,  0.126877,  0.129056,  0.130519,  0.132484,  0.133731,
      0.136000,  0.137591,  0.139614,  0.141911,  0.143324,  0.145435,
      0.147272,  0.149583,  0.152054,  0.154111,  0.156165,  0.158640,
      0.161225,  0.163679,  0.165971,  0.168321,  0.171734,  0.174213,
      0.176755,  0.179362,  0.183018,  0.185781,  0.188596,  0.191740,
      0.195438,  0.198734,  0.202574,  0.205756,  0.209016,  0.213356,
      0.216778,  0.221281,  0.224866,  0.228539,  0.232649,  0.237145,
      0.241085,  0.246057,  0.250239,  0.254458,  0.258773,  0.262595,
      0.266353,  0.269163,  0.271024,  0.273940,  0.276819,  0.278000,
      0.281000,  0.285000,  0.286097,  0.287468,  0.292000,  0.294000,
      0.297000,  0.305334,  NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      0.415639,  0.415054,  0.409267,  0.403490,  0.397714,  0.391947,
      0.386189,  0.380434,  0.374685,  0.368945,  0.363208,  0.357477,
      0.351758,  0.346041,  0.340335,  0.334635,  0.328941,  0.323256,
      0.317582,  0.311911,  0.306252,  0.300602,  0.294957,  0.289324,
      0.283950,  0.278757,  0.272492,  0.267199,  0.261297,  0.255587,
      0.250094,  0.244556,  0.239009,  0.233859,  0.228440,  0.222594,
      0.217174,  0.211573,  0.206160,  0.200647,  0.195260,  0.190164,
      0.184533,  0.179145,  0.173986,  0.168674,  0.163025,  0.157762,
      0.152947,  0.148016,  0.142585,  0.137134,  0.132527,  0.127639,
      0.122645,  0.117194,  0.112184,  0.107368,  0.102355,  0.097049,
      0.091967,  0.087322,  0.082757,  0.077982,  0.072588,  0.067966,
      0.063330,  0.058675,  0.053989,  0.049294,  0.044746,  0.040591,
      0.036053,  0.031253,  0.026583,  0.022775,  0.018518,  0.013850,
      0.009439,  0.005249,  0.001277,  -0.003086, -0.006976, -0.010765,
      -0.014568, -0.018391, -0.022198, -0.025428, -0.029004, -0.032929,
      -0.036724, -0.040032, -0.042886, -0.046212, -0.049268, -0.051979,
      -0.054895, -0.058336, -0.060684, -0.063037, -0.065847, -0.068513,
      -0.070655, -0.072816, -0.074662, -0.076184, -0.078399, -0.080086,
      -0.081588, -0.082552, -0.083316, -0.084086, -0.084865, -0.085653,
      -0.086000, -0.086000, -0.086000, -0.086000, -0.086000, -0.085454,
      -0.084214, -0.082666, -0.081508, -0.080008, -0.078230, -0.076144,
      -0.073409, -0.070595, -0.067466, -0.064476, -0.061108, -0.057214,
      -0.053248, -0.047945, -0.042662, -0.036957, -0.030596, -0.023136,
      -0.015873, -0.007936, 0.000308,  0.008753,  0.018257,  0.028119,
      0.038342,  0.049383,  0.061333,  0.073557,  0.087225,  0.101482,
      0.116036,  0.130802,  0.146282,  0.162485,  0.179740,  0.197574,
      0.215653,  0.233920,  0.253348,  0.272930,  0.292518,  0.311455,
      0.329126,  0.345271,  0.357940,  0.366490,  0.371533,  0.372618,
      0.367358,  0.356328,  0.340258,  0.320555,  0.298406,  0.274462,
      0.251395,  0.230838,  0.212519,  0.199737,  0.190157,  0.185723,
      0.181368,  0.181000,  0.181788,  0.183120,  0.184802,  0.186557,
      0.188493,  0.190373,  0.192054,  0.193735,  0.194928,  0.196029,
      0.197192,  0.198806,  0.200420,  0.202018,  0.202876,  0.203734,
      0.204592,  0.205669,  0.206945,  0.208220,  0.209496,  0.210833,
      0.212211,  0.213588,  0.214965,  0.216259,  0.217299,  0.218340,
      0.219381,  0.220422,  0.221423,  0.222375,  0.223327,  0.224278,
      0.225230,  0.226134,  0.226836,  0.227538,  0.228240,  0.228942,
      0.229644,  0.230161,  0.230487,  0.230814,  0.231140,  0.231466,
      0.231792,  0.232167,  0.232625,  0.233084,  0.233542,  0.234001,
      0.234459,  0.234918,  0.235118,  0.235263,  0.235407,  0.235551,
      0.235695,  0.235839,  0.235984,  0.236122,  0.236259,  0.236396,
      0.236533,  0.236670,  0.236807,  0.236944,  0.237155,  0.237417,
      0.237679,  0.237942,  0.238204,  0.238466,  0.238728,  0.238991,
      0.239122,  0.239248,  0.239374,  0.239500,  0.239626,  0.239752,
      0.239878,  0.240004,  0.240126,  0.240248,  0.240370,  0.240492,
      0.240614,  0.240736,  0.240858,  0.240980,  0.241198,  0.241435,
      0.241672,  0.241910,  0.242147,  0.242384,  0.242621,  0.242858,
      0.243046,  0.243162,  0.243278,  0.243393,  0.243509,  0.243624,
      0.243740,  0.243856,  0.243971,  0.244196,  0.244457,  0.244717,
      0.244978,  0.245239,  0.245499,  0.245760,  0.246012,  0.246161,
      0.246309,  0.246458,  0.246607,  0.246756,  0.246905,  0.247120,
      0.247455,  0.247789,  0.248124,  0.248458,  0.248793,  0.249142,
      0.249513,  0.249885,  0.250256,  0.250628,  0.250999,  0.251204,
      0.251408,  0.251612,  0.251816,  0.252023,  0.252245,  0.252468,
      0.252691,  0.252913,  0.253295,  0.253777,  0.254259,  0.254742,
      0.255121,  0.255380,  0.255640,  0.255899,  0.256341,  0.256897,
      0.257453,  0.258005,  0.258302,  0.258598,  0.258895,  0.259406,
      0.260036,  0.260666,  0.261157,  0.261490,  0.261824,  0.262166,
      0.262518,  0.262870,  0.263467,  0.264208,  0.264949,  0.265362,
      0.265751,  0.266293,  0.267108,  0.267922,  0.268385,  0.268811,
      0.269494,  0.270382,  0.271141,  0.271604,  0.272138,  0.273101,
      0.274033,  0.274532,  0.275066,  0.276103,  0.277072,  0.277608,
      0.278300,  0.279410,  0.280269,  0.280842,  0.281858,  0.283021,
      0.283632,  0.284500,  0.285757,  0.286522,  0.287349,  0.288680,
      0.289520,  0.290419,  0.291825,  0.293263,  0.294361,  0.295207,
      0.296723,  0.297635,  0.298842,  0.300442,  0.302035,  0.302867,
      0.304429,  0.306154,  0.307455,  0.308699,  0.310522,  0.312189,
      0.313268,  0.315193,  0.317155,  0.319077,  0.320191,  0.322265,
      0.324189,  0.325531,  0.327724,  0.329957,  0.331616,  0.333547,
      0.335905,  0.338311,  0.340380,  0.342255,  0.344794,  0.347383,
      0.349511,  0.351705,  0.354441,  0.357229,  0.359064,  0.361958,
      0.364901,  0.367903,  0.370957,  0.374072,  0.377242,  0.379736,
      0.382763,  0.385113,  0.388528,  0.391006,  0.393548,  0.396077,
      0.399745,  0.401788,  0.405032,  0.411041,  0.412020,  0.414000,
      0.415089,  0.417347,  0.419000,  0.420936,  0.426000,  0.427000,
      0.428000,  0.429000,  0.430000,  0.432000,  0.435000,  0.440000,
      0.453000,  NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       0.471918,  0.471328,
      0.465497,  0.460110,  0.454862,  0.449056,  0.443261,  0.437472,
      0.431695,  0.425922,  0.420161,  0.414413,  0.408675,  0.402941,
      0.397499,  0.392359,  0.386809,  0.381125,  0.375444,  0.369776,
      0.364154,  0.358603,  0.352748,  0.347038,  0.341640,  0.335858,
      0.330378,  0.325225,  0.319812,  0.314342,  0.308873,  0.303171,
      0.297996,  0.292837,  0.287255,  0.281618,  0.276283,  0.271379,
      0.265710,  0.260439,  0.255446,  0.250124,  0.244936,  0.239911,
      0.234892,  0.229365,  0.224303,  0.219427,  0.214367,  0.209337,
      0.204769,  0.199228,  0.194567,  0.189945,  0.185294,  0.180627,
      0.175943,  0.171236,  0.166499,  0.161754,  0.157230,  0.153035,
      0.148368,  0.144078,  0.139657,  0.135040,  0.130741,  0.126412,
      0.122203,  0.118195,  0.113804,  0.109905,  0.106093,  0.102269,
      0.098416,  0.094550,  0.090655,  0.087496,  0.083801,  0.080034,
      0.076713,  0.073872,  0.070502,  0.067490,  0.064648,  0.061014,
      0.058249,  0.055458,  0.053327,  0.050822,  0.048230,  0.046080,
      0.043907,  0.041719,  0.039679,  0.038195,  0.036702,  0.035195,
      0.033673,  0.032139,  0.030796,  0.030015,  0.029225,  0.029000,
      0.029000,  0.029000,  0.029000,  0.029000,  0.029683,  0.030532,
      0.031391,  0.032348,  0.033522,  0.035062,  0.037299,  0.039692,
      0.041909,  0.045038,  0.047985,  0.051508,  0.055840,  0.060495,
      0.065581,  0.071061,  0.077081,  0.083424,  0.089904,  0.096475,
      0.103881,  0.112351,  0.121415,  0.130194,  0.140216,  0.151368,
      0.162519,  0.174350,  0.187060,  0.200955,  0.214945,  0.229542,
      0.245384,  0.261944,  0.278394,  0.297112,  0.316692,  0.336534,
      0.357337,  0.379435,  0.401702,  0.424671,  0.447252,  0.468725,
      0.488811,  0.507089,  0.521802,  0.533281,  0.540698,  0.543307,
      0.539507,  0.527084,  0.507762,  0.483198,  0.455006,  0.424541,
      0.396380,  0.373271,  0.354180,  0.341940,  0.336700,  0.333188,
      0.334257,  0.336589,  0.340288,  0.344097,  0.347524,  0.350682,
      0.353639,  0.356154,  0.358291,  0.360449,  0.362688,  0.364927,
      0.367195,  0.369491,  0.371786,  0.373957,  0.375987,  0.378018,
      0.380046,  0.381949,  0.383852,  0.385755,  0.387565,  0.389198,
      0.390830,  0.392463,  0.393913,  0.395273,  0.396633,  0.397993,
      0.398727,  0.399458,  0.400189,  0.400919,  0.401391,  0.401831,
      0.402271,  0.402710,  0.403000,  0.403000,  0.403000,  0.403000,
      0.403000,  0.403063,  0.403247,  0.403431,  0.403614,  0.403798,
      0.403981,  0.404153,  0.404323,  0.404493,  0.404663,  0.404832,
      0.405004,  0.405321,  0.405638,  0.405955,  0.406272,  0.406589,
      0.406906,  0.407105,  0.407254,  0.407403,  0.407552,  0.407701,
      0.407850,  0.407999,  0.408140,  0.408281,  0.408422,  0.408563,
      0.408704,  0.408845,  0.408986,  0.409121,  0.409255,  0.409389,
      0.409524,  0.409658,  0.409792,  0.409927,  0.410059,  0.410188,
      0.410316,  0.410445,  0.410574,  0.410703,  0.410832,  0.410961,
      0.411086,  0.411211,  0.411335,  0.411459,  0.411583,  0.411708,
      0.411832,  0.411956,  0.412078,  0.412198,  0.412319,  0.412439,
      0.412560,  0.412680,  0.412800,  0.412921,  0.413040,  0.413157,
      0.413275,  0.413392,  0.413509,  0.413626,  0.413743,  0.413861,
      0.413978,  0.414191,  0.414426,  0.414660,  0.414895,  0.415130,
      0.415365,  0.415600,  0.415835,  0.416081,  0.416353,  0.416625,
      0.416897,  0.417169,  0.417441,  0.417712,  0.417984,  0.418146,
      0.418300,  0.418454,  0.418609,  0.418763,  0.418918,  0.419162,
      0.419507,  0.419853,  0.420199,  0.420545,  0.420891,  0.421262,
      0.421645,  0.422027,  0.422410,  0.422793,  0.423096,  0.423306,
      0.423516,  0.423726,  0.423936,  0.424317,  0.424773,  0.425230,
      0.425687,  0.426077,  0.426324,  0.426571,  0.426818,  0.427139,
      0.427669,  0.428200,  0.428730,  0.429279,  0.429847,  0.430414,
      0.430982,  0.431585,  0.432189,  0.432793,  0.433211,  0.433532,
      0.433852,  0.434183,  0.434522,  0.434861,  0.435422,  0.436138,
      0.436853,  0.437299,  0.437675,  0.438106,  0.438895,  0.439684,
      0.440495,  0.441321,  0.442154,  0.443017,  0.443879,  0.444387,
      0.444837,  0.445597,  0.446534,  0.447489,  0.448463,  0.449453,
      0.450463,  0.451491,  0.452538,  0.453303,  0.453845,  0.454802,
      0.455923,  0.457079,  0.458244,  0.459439,  0.460654,  0.461886,
      0.462576,  0.463433,  0.464739,  0.466074,  0.467214,  0.467904,
      0.469219,  0.470653,  0.472109,  0.473600,  0.474558,  0.475660,
      0.477229,  0.478830,  0.480465,  0.482128,  0.483821,  0.485544,
      0.487299,  0.489043,  0.489955,  0.491767,  0.493659,  0.494793,
      0.496548,  0.498547,  0.500583,  0.502657,  0.504768,  0.506919,
      0.509167,  0.512519,  0.515933,  0.518939,  0.521948,  0.525551,
      0.528149,  0.530646,  0.533187,  0.535773,  0.538205,  0.539549,
      0.540915,  0.542309,  0.543000,  0.545417,  0.547813,  0.549000,
      0.549856,  0.553302,  0.557971,  0.559000,  0.560000,  0.561037,
      0.562000,  0.563000,  0.564000,  0.565000,  0.566000,  0.567167,
      0.570000,  0.576941,  0.594840,  NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       0.525256,  0.524664,  0.518803,  0.512948,
      0.507399,  0.502151,  0.496459,  0.490658,  0.484860,  0.479080,
      0.473312,  0.468244,  0.462813,  0.457089,  0.451369,  0.445792,
      0.440622,  0.434821,  0.429278,  0.424082,  0.418713,  0.412791,
      0.407540,  0.401951,  0.396379,  0.390970,  0.385625,  0.380428,
      0.375331,  0.370002,  0.364416,  0.359239,  0.353689,  0.348607,
      0.343286,  0.338047,  0.333170,  0.328397,  0.323286,  0.317831,
      0.313101,  0.308416,  0.303190,  0.298179,  0.293491,  0.288791,
      0.284068,  0.279312,  0.274549,  0.269755,  0.264946,  0.260116,
      0.255848,  0.251383,  0.246792,  0.242487,  0.238657,  0.234300,
      0.230225,  0.226026,  0.221665,  0.217850,  0.214021,  0.209688,
      0.205527,  0.202387,  0.198469,  0.194520,  0.191038,  0.187562,
      0.183626,  0.180258,  0.177300,  0.173962,  0.170826,  0.168067,
      0.165283,  0.162484,  0.159659,  0.157113,  0.154968,  0.152801,
      0.150164,  0.147920,  0.146135,  0.144647,  0.143145,  0.141628,
      0.140099,  0.138557,  0.137003,  0.136215,  0.135422,  0.135000,
      0.134814,  0.134000,  0.134000,  0.134663,  0.135000,  0.135358,
      0.136220,  0.137183,  0.138947,  0.140742,  0.142547,  0.144569,
      0.147355,  0.150241,  0.154075,  0.157945,  0.161887,  0.165870,
      0.170909,  0.175743,  0.181372,  0.187777,  0.194264,  0.200876,
      0.208875,  0.217285,  0.225905,  0.234831,  0.245058,  0.255699,
      0.267358,  0.279422,  0.291888,  0.305439,  0.319967,  0.335339,
      0.351593,  0.368983,  0.387729,  0.408057,  0.428886,  0.451162,
      0.474334,  0.497805,  0.522503,  0.546523,  0.570206,  0.593528,
      0.616266,  0.637299,  0.655818,  0.672056,  0.684888,  0.693502,
      0.696663,  0.690008,  0.673879,  0.649337,  0.617028,  0.581462,
      0.547412,  0.518406,  0.494486,  0.481205,  0.475679,  0.475379,
      0.477360,  0.481168,  0.486106,  0.491442,  0.496102,  0.500516,
      0.503975,  0.507455,  0.511074,  0.514694,  0.518815,  0.522983,
      0.528144,  0.533487,  0.539105,  0.545161,  0.551865,  0.557397,
      0.562636,  0.566736,  0.569033,  0.569527,  0.568146,  0.566179,
      0.564107,  0.562138,  0.560718,  0.559957,  0.559301,  0.558694,
      0.558130,  0.558000,  0.558000,  0.558000,  0.558000,  0.558000,
      0.558000,  0.558000,  0.558000,  0.558000,  0.558000,  0.558000,
      0.558000,  0.558000,  0.558000,  0.558000,  0.558000,  0.558000,
      0.558000,  0.558000,  0.558000,  0.558000,  0.558000,  0.558175,
      0.558384,  0.558593,  0.558802,  0.559000,  0.559000,  0.559000,
      0.559000,  0.559000,  0.559000,  0.559144,  0.559320,  0.559496,
      0.559672,  0.559848,  0.560022,  0.560186,  0.560350,  0.560513,
      0.560677,  0.560840,  0.561004,  0.561157,  0.561310,  0.561463,
      0.561616,  0.561770,  0.561923,  0.562072,  0.562216,  0.562361,
      0.562505,  0.562650,  0.562795,  0.562939,  0.563000,  0.563000,
      0.563000,  0.563000,  0.563000,  0.563000,  0.563000,  0.563039,
      0.563170,  0.563302,  0.563433,  0.563564,  0.563696,  0.563827,
      0.563958,  0.564086,  0.564213,  0.564339,  0.564465,  0.564591,
      0.564718,  0.564844,  0.564970,  0.565187,  0.565431,  0.565675,
      0.565920,  0.566164,  0.566408,  0.566653,  0.566897,  0.567137,
      0.567374,  0.567612,  0.567849,  0.568086,  0.568324,  0.568561,
      0.568798,  0.569037,  0.569283,  0.569528,  0.569774,  0.570020,
      0.570266,  0.570512,  0.570758,  0.571004,  0.571287,  0.571570,
      0.571853,  0.572136,  0.572419,  0.572701,  0.572984,  0.573302,
      0.573622,  0.573942,  0.574261,  0.574581,  0.574901,  0.575246,
      0.575603,  0.575960,  0.576317,  0.576673,  0.577033,  0.577427,
      0.577821,  0.578214,  0.578608,  0.579001,  0.579216,  0.579432,
      0.579647,  0.579862,  0.580168,  0.580636,  0.581104,  0.581571,
      0.582042,  0.582546,  0.583051,  0.583555,  0.584064,  0.584606,
      0.585147,  0.585689,  0.586246,  0.586824,  0.587402,  0.587981,
      0.588297,  0.588605,  0.588913,  0.589467,  0.590119,  0.590772,
      0.591448,  0.592137,  0.592826,  0.593543,  0.594269,  0.594995,
      0.595758,  0.596521,  0.597298,  0.598098,  0.598897,  0.599730,
      0.600566,  0.601421,  0.602295,  0.603088,  0.603543,  0.603999,
      0.604945,  0.605892,  0.606872,  0.607857,  0.608873,  0.609894,
      0.610949,  0.612008,  0.613103,  0.614205,  0.615337,  0.616485,
      0.617654,  0.618849,  0.620028,  0.620650,  0.621559,  0.622839,
      0.624151,  0.625480,  0.626834,  0.628830,  0.630627,  0.632056,
      0.633521,  0.635010,  0.636524,  0.638095,  0.640458,  0.642242,
      0.644311,  0.646802,  0.649226,  0.650948,  0.653560,  0.656246,
      0.658321,  0.660178,  0.662069,  0.663995,  0.665479,  0.666956,
      0.668496,  0.670067,  0.671589,  0.672665,  0.673912,  0.678434,
      0.680635,  0.681884,  0.683231,  0.684741,  0.686000,  0.687000,
      0.688000,  0.689000,  0.689000,  0.690000,  0.690585,  0.691000,
      0.692000,  0.693507,  0.696074,  0.702000,  0.719000,  NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      0.530590,  0.529997,  0.524133,  0.518231,  0.512653,  0.507460,
      0.501779,  0.495976,  0.490177,  0.484396,  0.478627,  0.473627,
      0.468227,  0.462503,  0.456756,  0.451135,  0.446003,  0.440191,
      0.434662,  0.429512,  0.424169,  0.418209,  0.413019,  0.407443,
      0.401853,  0.396482,  0.391149,  0.385949,  0.380883,  0.375568,
      0.369970,  0.364846,  0.359258,  0.354184,  0.348889,  0.343690,
      0.338859,  0.334099,  0.329043,  0.323571,  0.318866,  0.314245,
      0.309015,  0.304006,  0.299351,  0.294734,  0.290044,  0.285301,
      0.280567,  0.275797,  0.270963,  0.266205,  0.261976,  0.257526,
      0.252942,  0.248673,  0.244929,  0.240606,  0.236598,  0.232453,
      0.228109,  0.224331,  0.220586,  0.216249,  0.212114,  0.209122,
      0.205242,  0.201331,  0.197922,  0.194499,  0.190609,  0.187293,
      0.184420,  0.181131,  0.178067,  0.175419,  0.172746,  0.169982,
      0.167245,  0.164821,  0.162794,  0.160694,  0.158130,  0.155963,
      0.154284,  0.153010,  0.151634,  0.150245,  0.148776,  0.147331,
      0.145881,  0.145229,  0.144574,  0.144328,  0.144327,  0.143581,
      0.143730,  0.144609,  0.145133,  0.145680,  0.146762,  0.147900,
      0.149919,  0.151916,  0.153901,  0.156126,  0.159191,  0.162365,
      0.166514,  0.170686,  0.174937,  0.179222,  0.184648,  0.189811,
      0.195779,  0.202585,  0.209499,  0.216460,  0.224964,  0.233863,
      0.242911,  0.252264,  0.263006,  0.274162,  0.286386,  0.299021,
      0.312086,  0.326335,  0.341575,  0.357637,  0.374610,  0.392862,
      0.412480,  0.433725,  0.455523,  0.478844,  0.503062,  0.527490,
      0.553258,  0.578221,  0.602688,  0.626686,  0.650053,  0.671317,
      0.689730,  0.705609,  0.717643,  0.724908,  0.726160,  0.716542,
      0.696542,  0.667398,  0.629850,  0.588900,  0.549973,  0.516918,
      0.489865,  0.474995,  0.469296,  0.470208,  0.474320,  0.480965,
      0.489216,  0.498132,  0.506075,  0.513241,  0.518955,  0.524006,
      0.528512,  0.532845,  0.537271,  0.541622,  0.546929,  0.552426,
      0.558263,  0.564608,  0.571687,  0.577522,  0.583070,  0.587365,
      0.589667,  0.589987,  0.588241,  0.585848,  0.583339,  0.580957,
      0.579191,  0.578151,  0.577226,  0.576369,  0.575558,  0.575224,
      0.575044,  0.574880,  0.574717,  0.574554,  0.574409,  0.574273,
      0.574137,  0.574001,  0.573927,  0.573854,  0.573781,  0.573708,
      0.573661,  0.573617,  0.573573,  0.573529,  0.573500,  0.573500,
      0.573500,  0.573500,  0.573500,  0.573686,  0.573898,  0.574109,
      0.574321,  0.574520,  0.574502,  0.574485,  0.574468,  0.574451,
      0.574434,  0.574575,  0.574751,  0.574913,  0.575075,  0.575237,
      0.575397,  0.575546,  0.575694,  0.575854,  0.576019,  0.576184,
      0.576349,  0.576502,  0.576656,  0.576810,  0.576964,  0.577119,
      0.577273,  0.577423,  0.577568,  0.577713,  0.577857,  0.578003,
      0.578149,  0.578294,  0.578348,  0.578334,  0.578321,  0.578307,
      0.578294,  0.578281,  0.578268,  0.578298,  0.578430,  0.578562,
      0.578693,  0.578825,  0.578957,  0.579089,  0.579221,  0.579349,
      0.579475,  0.579602,  0.579728,  0.579855,  0.579982,  0.580109,
      0.580236,  0.580462,  0.580718,  0.580975,  0.581232,  0.581488,
      0.581745,  0.582002,  0.582259,  0.582512,  0.582761,  0.583010,
      0.583260,  0.583509,  0.583758,  0.583998,  0.584235,  0.584474,
      0.584721,  0.584968,  0.585215,  0.585462,  0.585709,  0.585955,
      0.586198,  0.586442,  0.586726,  0.587010,  0.587294,  0.587578,
      0.587862,  0.588157,  0.588453,  0.588787,  0.589123,  0.589459,
      0.589796,  0.590123,  0.590440,  0.590786,  0.591143,  0.591501,
      0.591859,  0.592215,  0.592572,  0.592967,  0.593362,  0.593757,
      0.594159,  0.594570,  0.594786,  0.595002,  0.595218,  0.595417,
      0.595708,  0.596177,  0.596645,  0.597121,  0.597614,  0.598144,
      0.598674,  0.599197,  0.599704,  0.600246,  0.600789,  0.601330,
      0.601886,  0.602465,  0.603045,  0.603620,  0.603908,  0.604186,
      0.604483,  0.605061,  0.605746,  0.606431,  0.607141,  0.607865,
      0.608567,  0.609284,  0.610011,  0.610765,  0.611566,  0.612362,
      0.613138,  0.613939,  0.614738,  0.615570,  0.616408,  0.617262,
      0.618136,  0.618958,  0.619414,  0.619839,  0.620786,  0.621732,
      0.622713,  0.623697,  0.624714,  0.625735,  0.626790,  0.627878,
      0.629029,  0.630145,  0.631279,  0.632425,  0.633595,  0.634790,
      0.635966,  0.636526,  0.637457,  0.638779,  0.640092,  0.641421,
      0.642796,  0.644923,  0.646768,  0.648197,  0.649662,  0.651151,
      0.652720,  0.654339,  0.656781,  0.658583,  0.660695,  0.663270,
      0.665766,  0.667489,  0.670186,  0.672966,  0.675158,  0.677019,
      0.678910,  0.680916,  0.682372,  0.683797,  0.685288,  0.686808,
      0.688272,  0.689240,  0.690386,  0.695026,  0.697106,  0.698179,
      0.699359,  0.700660,  0.701785,  0.702635,  0.703481,  0.704323,
      0.704079,  0.705045,  0.705552,  0.705869,  0.706900,  0.708316,
      0.710900,  0.717300,  0.735914,  NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN,       NAN,       NAN,
      NAN,       NAN,       NAN,       NAN};

// Baraffe+2015 radius interpolation grid
static double const DATA_RADIUS[STELLAR_BAR_MLEN][STELLAR_BAR_ALEN] = {
      0.780655, 0.780423, 0.778376, 0.776942, 0.776230, 0.774947, 0.773469,
      0.772129, 0.770961, 0.769837, 0.768844, 0.767872, 0.767011, 0.766152,
      0.765420, 0.764707, 0.763970, 0.763172, 0.762374, 0.761530, 0.760664,
      0.759774, 0.758840, 0.757891, 0.756889, 0.755986, 0.755454, 0.754704,
      0.753541, 0.752291, 0.751087, 0.750492, 0.749476, 0.748120, 0.747439,
      0.746198, 0.744932, 0.744136, 0.743378, 0.741690, 0.740590, 0.738980,
      0.737661, 0.735907, 0.734468, 0.733424, 0.731185, 0.729529, 0.727575,
      0.726076, 0.723704, 0.721931, 0.719512, 0.716964, 0.714781, 0.712105,
      0.709634, 0.706534, 0.703082, 0.699935, 0.696866, 0.692717, 0.689152,
      0.685143, 0.680662, 0.675645, 0.670091, 0.664394, 0.658552, 0.651793,
      0.644591, 0.636914, 0.628377, 0.620235, 0.612190, 0.603839, 0.595284,
      0.587556, 0.579279, 0.571519, 0.564180, 0.556952, 0.549845, 0.543651,
      0.537047, 0.530476, 0.524618, 0.518057, 0.512380, 0.506749, 0.501017,
      0.494998, 0.489571, 0.484490, 0.479284, 0.474095, 0.469313, 0.464831,
      0.460243, 0.455504, 0.450996, 0.446723, 0.442153, 0.437514, 0.433193,
      0.429041, 0.425566, 0.421200, 0.417536, 0.414115, 0.410389, 0.406352,
      0.402864, 0.399517, 0.395805, 0.391741, 0.388437, 0.385161, 0.381908,
      0.378715, 0.375567, 0.372378, 0.369140, 0.366638, 0.363956, 0.360855,
      0.357773, 0.354712, 0.351667, 0.349102, 0.346683, 0.343629, 0.340648,
      0.338456, 0.335799, 0.332991, 0.330874, 0.327976, 0.325077, 0.322771,
      0.320640, 0.318554, 0.315634, 0.313706, 0.310965, 0.308480, 0.306437,
      0.304333, 0.302195, 0.299991, 0.297560, 0.295480, 0.293409, 0.290923,
      0.288784, 0.286730, 0.284681, 0.282591, 0.280913, 0.279074, 0.277048,
      0.275029, 0.273611, 0.271529, 0.269485, 0.267521, 0.266091, 0.264102,
      0.262072, 0.260289, 0.258637, 0.256601, 0.254673, 0.252983, 0.251223,
      0.249383, 0.248109, 0.246882, 0.245064, 0.243324, 0.241713, 0.239789,
      0.238374, 0.236408, 0.234920, 0.233691, 0.232290, 0.230345, 0.228897,
      0.227678, 0.226430, 0.224931, 0.223069, 0.221708, 0.220474, 0.219239,
      0.218034, 0.215854, 0.214550, 0.213327, 0.212108, 0.210891, 0.209676,
      0.208466, 0.207296, 0.206152, 0.204948, 0.202649, 0.201453, 0.200259,
      0.199069, 0.197881, 0.196696, 0.195516, 0.194337, 0.193162, 0.191989,
      0.190819, 0.189652, 0.188488, 0.187326, 0.186167, 0.185010, 0.184790,
      0.183804, 0.182654, 0.181506, 0.180361, 0.179218, 0.178042, 0.176878,
      0.175781, 0.174695, 0.174105, 0.173020, 0.172121, 0.171088, 0.169877,
      0.168792, 0.168001, 0.167304, 0.166536, 0.165365, 0.163916, 0.162933,
      0.162174, 0.161390, 0.160560, 0.159750, 0.158955, 0.158158, 0.157357,
      0.156576, 0.155744, 0.154108, 0.153142, 0.152341, 0.151534, 0.150703,
      0.149916, 0.149118, 0.148308, 0.147497, 0.146691, 0.145887, 0.145077,
      0.144275, 0.143476, 0.142812, 0.142892, 0.142176, 0.141377, 0.140575,
      0.139768, 0.138962, 0.138162, 0.137309, 0.136479, 0.135719, 0.134863,
      0.134855, 0.134339, 0.133551, 0.132747, 0.132069, 0.131242, 0.130528,
      0.129980, 0.128948, 0.128880, 0.128428, 0.127912, 0.126899, 0.125792,
      0.125330, 0.124849, 0.124840, 0.123900, 0.123837, 0.122796, 0.121801,
      0.121874, 0.120899, 0.120900, 0.119959, 0.119449, 0.118970, 0.118000,
      0.117880, 0.117116, 0.116870, 0.116609, 0.116143, 0.115458, 0.114900,
      0.114616, 0.113900, 0.113793, 0.112900, 0.112900, 0.112499, 0.112054,
      0.111324, 0.110900, 0.110573, 0.109900, 0.109843, 0.108986, 0.108900,
      0.108289, 0.107900, 0.107614, 0.106900, 0.106900, 0.106142, 0.105900,
      0.105520, 0.104900, 0.104900, 0.104109, 0.103900, 0.103927, 0.103816,
      0.102948, 0.102900, 0.102351, 0.101900, 0.101803, 0.101002, 0.100900,
      0.100900, 0.100777, 0.099961, 0.099900, 0.099383, 0.098861, 0.098900,
      0.098903, 0.098423, 0.097900, 0.097900, 0.097900, 0.097436, 0.096900,
      0.096900, 0.096900, 0.096565, 0.095892, 0.095900, 0.095925, 0.095901,
      0.095165, 0.094900, 0.094900, 0.094943, 0.094497, 0.093900, 0.093900,
      0.093900, 0.093758, 0.093039, 0.092874, 0.092900, 0.092900, 0.092900,
      0.092900, 0.092913, 0.092357, 0.091900, 0.091900, 0.091900, 0.091900,
      0.091900, 0.091944, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000,
      0.091682, 0.090900, 0.090900, 0.090935, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.090394, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900, 0.089900,
      0.089900, 0.089900, 0.089900, NAN,      NAN,      0.782800, 0.782575,
      0.780556, 0.779042, 0.778250, 0.776938, 0.775459, 0.774116, 0.772947,
      0.771823, 0.770829, 0.769856, 0.768994, 0.768132, 0.767362, 0.766610,
      0.765848, 0.765046, 0.764245, 0.763396, 0.762526, 0.761629, 0.760692,
      0.759736, 0.758730, 0.757852, 0.757315, 0.756525, 0.755379, 0.754185,
      0.752983, 0.752338, 0.751350, 0.749992, 0.749272, 0.748054, 0.746755,
      0.745953, 0.745113, 0.743472, 0.742322, 0.740736, 0.739371, 0.737653,
      0.736233, 0.735086, 0.732879, 0.731212, 0.729268, 0.727712, 0.725359,
      0.723550, 0.721094, 0.718601, 0.716389, 0.713688, 0.711150, 0.708015,
      0.704602, 0.701408, 0.698272, 0.694165, 0.690477, 0.686402, 0.681864,
      0.676807, 0.671251, 0.665542, 0.659543, 0.652752, 0.645500, 0.637752,
      0.629251, 0.621053, 0.612937, 0.604576, 0.596060, 0.588265, 0.579984,
      0.572241, 0.564908, 0.557686, 0.550583, 0.544323, 0.537748, 0.531204,
      0.525277, 0.518787, 0.513064, 0.507415, 0.501681, 0.495669, 0.490243,
      0.485140, 0.479936, 0.474775, 0.469996, 0.465516, 0.460864, 0.456152,
      0.451654, 0.447321, 0.442749, 0.438189, 0.433894, 0.429737, 0.426176,
      0.421807, 0.418143, 0.414668, 0.410928, 0.406932, 0.403402, 0.400031,
      0.396353, 0.392356, 0.389053, 0.385778, 0.382525, 0.379294, 0.376084,
      0.372896, 0.369734, 0.367196, 0.364473, 0.361371, 0.358290, 0.355228,
      0.352183, 0.349576, 0.347142, 0.344146, 0.341164, 0.338902, 0.336254,
      0.333489, 0.331299, 0.328492, 0.325593, 0.323282, 0.321126, 0.318970,
      0.316119, 0.314137, 0.311442, 0.308963, 0.306850, 0.304747, 0.302649,
      0.300410, 0.297974, 0.295894, 0.293823, 0.291342, 0.289198, 0.287144,
      0.285096, 0.283055, 0.281345, 0.279488, 0.277463, 0.275443, 0.273928,
      0.271910, 0.269901, 0.267934, 0.266407, 0.264418, 0.262434, 0.260636,
      0.258981, 0.257010, 0.255048, 0.253392, 0.251637, 0.249792, 0.248497,
      0.247205, 0.245376, 0.243632, 0.242024, 0.240101, 0.238692, 0.236789,
      0.235261, 0.233998, 0.232610, 0.230727, 0.229232, 0.227983, 0.226736,
      0.225239, 0.223377, 0.222013, 0.220779, 0.219544, 0.218308, 0.216158,
      0.214855, 0.213633, 0.212413, 0.211196, 0.209982, 0.208771, 0.207561,
      0.206356, 0.205153, 0.202952, 0.201756, 0.200562, 0.199372, 0.198183,
      0.196998, 0.195817, 0.194637, 0.193461, 0.192288, 0.191117, 0.189949,
      0.188785, 0.187622, 0.186463, 0.185306, 0.185000, 0.183999, 0.182848,
      0.181700, 0.180554, 0.179409, 0.178266, 0.177125, 0.175985, 0.174915,
      0.174287, 0.173229, 0.172336, 0.171293, 0.170062, 0.168998, 0.168208,
      0.167497, 0.166721, 0.165581, 0.164186, 0.163146, 0.162371, 0.161583,
      0.160756, 0.159949, 0.159153, 0.158356, 0.157554, 0.156743, 0.155850,
      0.154291, 0.153340, 0.152538, 0.151732, 0.150904, 0.150115, 0.149316,
      0.148507, 0.147695, 0.146890, 0.146087, 0.145278, 0.144475, 0.143676,
      0.143000, 0.143000, 0.142276, 0.141477, 0.140674, 0.139868, 0.139062,
      0.138262, 0.137462, 0.136660, 0.135856, 0.135057, 0.135000, 0.134457,
      0.133663, 0.132884, 0.132194, 0.131402, 0.130693, 0.130100, 0.129134,
      0.129000, 0.128533, 0.128017, 0.127008, 0.125993, 0.125485, 0.125000,
      0.124946, 0.124000, 0.123942, 0.122950, 0.122000, 0.121975, 0.121000,
      0.121000, 0.120058, 0.119549, 0.119068, 0.118181, 0.118000, 0.117287,
      0.117000, 0.116707, 0.116241, 0.115553, 0.115000, 0.114711, 0.114000,
      0.113887, 0.113000, 0.113000, 0.112597, 0.112152, 0.111419, 0.111000,
      0.110668, 0.110000, 0.109938, 0.109082, 0.109000, 0.108385, 0.108000,
      0.107711, 0.107000, 0.107000, 0.106241, 0.106000, 0.105621, 0.105000,
      0.105000, 0.104214, 0.104000, 0.104000, 0.103832, 0.103044, 0.103000,
      0.102474, 0.102000, 0.101912, 0.101135, 0.101000, 0.101000, 0.100822,
      0.100055, 0.100000, 0.099530, 0.099000, 0.099000, 0.099000, 0.098500,
      0.098000, 0.098000, 0.098000, 0.097531, 0.097000, 0.097000, 0.097000,
      0.096673, 0.096000, 0.096000, 0.096000, 0.095918, 0.095241, 0.095000,
      0.095000, 0.095000, 0.094543, 0.094000, 0.094000, 0.094000, 0.093871,
      0.093208, 0.093000, 0.093000, 0.093000, 0.093000, 0.093000, 0.093000,
      0.092429, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000,
      0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.091711, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.090449, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000, 0.090000,
      0.090000, 0.090000, NAN,      0.804251, 0.804098, 0.802354, 0.800032,
      0.798447, 0.796851, 0.795357, 0.793988, 0.792812, 0.791687, 0.790673,
      0.789705, 0.788827, 0.787932, 0.786784, 0.785635, 0.784627, 0.783793,
      0.782956, 0.782052, 0.781148, 0.780185, 0.779212, 0.778183, 0.777140,
      0.776518, 0.775917, 0.774732, 0.773760, 0.773131, 0.771947, 0.770798,
      0.770095, 0.768718, 0.767598, 0.766616, 0.764987, 0.764127, 0.762460,
      0.761295, 0.759639, 0.758299, 0.756465, 0.755115, 0.753884, 0.751705,
      0.749814, 0.748044, 0.746194, 0.744074, 0.741908, 0.739739, 0.736915,
      0.734967, 0.732467, 0.729513, 0.726312, 0.722833, 0.719805, 0.716137,
      0.712337, 0.708644, 0.703724, 0.698994, 0.693888, 0.688430, 0.682854,
      0.677026, 0.669453, 0.662339, 0.654590, 0.646130, 0.637995, 0.629232,
      0.620401, 0.611943, 0.603823, 0.595353, 0.587036, 0.579457, 0.572189,
      0.565027, 0.557966, 0.551044, 0.544752, 0.538492, 0.531869, 0.526084,
      0.519908, 0.514077, 0.508324, 0.502372, 0.496963, 0.491637, 0.486457,
      0.481573, 0.476829, 0.472368, 0.467075, 0.462633, 0.458227, 0.453301,
      0.448711, 0.444941, 0.440903, 0.436704, 0.432274, 0.427882, 0.424218,
      0.420196, 0.416320, 0.412730, 0.408782, 0.405163, 0.401824, 0.398508,
      0.395214, 0.391945, 0.388696, 0.385085, 0.381259, 0.378072, 0.375682,
      0.372767, 0.369643, 0.366537, 0.363454, 0.360392, 0.357346, 0.354315,
      0.351729, 0.349308, 0.346327, 0.343362, 0.340809, 0.338474, 0.335552,
      0.333643, 0.330748, 0.328399, 0.325994, 0.323131, 0.320962, 0.318440,
      0.316208, 0.313790, 0.310982, 0.308881, 0.307190, 0.304594, 0.302112,
      0.300034, 0.297965, 0.295532, 0.293342, 0.291289, 0.289241, 0.287700,
      0.285664, 0.283635, 0.281609, 0.279590, 0.277101, 0.275712, 0.274064,
      0.272064, 0.269568, 0.267570, 0.266054, 0.264103, 0.262418, 0.261104,
      0.258793, 0.257486, 0.255772, 0.253880, 0.252374, 0.250431, 0.248496,
      0.246709, 0.245140, 0.243220, 0.241872, 0.240602, 0.238667, 0.237071,
      0.235809, 0.234545, 0.232576, 0.231038, 0.229790, 0.228320, 0.226457,
      0.225066, 0.223831, 0.222597, 0.221050, 0.219207, 0.217913, 0.216689,
      0.215468, 0.214250, 0.213033, 0.211820, 0.210217, 0.208400, 0.207194,
      0.205990, 0.204790, 0.203591, 0.202396, 0.201202, 0.200012, 0.198825,
      0.197639, 0.196458, 0.195278, 0.194101, 0.192927, 0.191757, 0.190588,
      0.189423, 0.188260, 0.187099, 0.185942, 0.184786, 0.183633, 0.182483,
      0.181314, 0.180504, 0.179599, 0.178020, 0.177110, 0.176112, 0.175319,
      0.174490, 0.173343, 0.171911, 0.171056, 0.170272, 0.169426, 0.168570,
      0.167747, 0.166882, 0.165281, 0.164335, 0.163521, 0.162717, 0.161941,
      0.161133, 0.160340, 0.159524, 0.158414, 0.156916, 0.156112, 0.155320,
      0.154513, 0.153705, 0.152909, 0.152102, 0.151297, 0.150489, 0.149682,
      0.148880, 0.148087, 0.147284, 0.146478, 0.145675, 0.144879, 0.144077,
      0.143272, 0.142472, 0.141671, 0.140866, 0.140063, 0.139261, 0.139000,
      0.138471, 0.137218, 0.137000, 0.136453, 0.135632, 0.134781, 0.134252,
      0.133447, 0.133000, 0.132342, 0.131292, 0.131000, 0.130204, 0.129583,
      0.129064, 0.128095, 0.128000, 0.127043, 0.126512, 0.126003, 0.125000,
      0.124994, 0.124493, 0.123986, 0.122994, 0.122005, 0.122000, 0.121045,
      0.120539, 0.120055, 0.120000, 0.119197, 0.119000, 0.118303, 0.117682,
      0.117215, 0.116499, 0.116000, 0.115656, 0.115000, 0.114833, 0.114000,
      0.114000, 0.113569, 0.113124, 0.112364, 0.112000, 0.111615, 0.111000,
      0.110889, 0.110036, 0.110000, 0.109347, 0.109000, 0.108683, 0.108000,
      0.108000, 0.107231, 0.107000, 0.106629, 0.106000, 0.106000, 0.105272,
      0.105000, 0.104731, 0.104000, 0.104000, 0.104000, 0.103702, 0.103000,
      0.103000, 0.102471, 0.102000, 0.102000, 0.101274, 0.101000, 0.101000,
      0.101000, 0.100392, 0.100000, 0.099972, 0.099268, 0.099000, 0.099000,
      0.099000, 0.098485, 0.098000, 0.098000, 0.098000, 0.097752, 0.097076,
      0.097000, 0.096747, 0.096093, 0.096000, 0.096000, 0.096000, 0.095571,
      0.095000, 0.095000, 0.095000, 0.095000, 0.095000, 0.094902, 0.094259,
      0.094000, 0.094000, 0.094000, 0.094000, 0.093867, 0.093150, 0.093000,
      0.093000, 0.093000, 0.093000, 0.093000, 0.092558, 0.092000, 0.092000,
      0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.091645,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, NAN,
      NAN,      0.830695, 0.830460, 0.828416, 0.826838, 0.825234, 0.823607,
      0.822563, 0.821655, 0.819862, 0.818466, 0.817436, 0.816486, 0.815594,
      0.814599, 0.813399, 0.812198, 0.811271, 0.810399, 0.809488, 0.808545,
      0.807572, 0.806558, 0.805511, 0.804426, 0.803648, 0.803070, 0.801918,
      0.800669, 0.799366, 0.798504, 0.797610, 0.796155, 0.795316, 0.794052,
      0.792412, 0.790853, 0.789926, 0.788065, 0.787060, 0.785088, 0.783981,
      0.782730, 0.780695, 0.778933, 0.777177, 0.774818, 0.772762, 0.770867,
      0.768519, 0.766555, 0.763743, 0.761741, 0.758963, 0.756001, 0.752838,
      0.749430, 0.745746, 0.741933, 0.738426, 0.734318, 0.729778, 0.725224,
      0.719417, 0.714412, 0.708123, 0.701399, 0.694672, 0.687205, 0.679031,
      0.670567, 0.662131, 0.653478, 0.643854, 0.635176, 0.626394, 0.617921,
      0.609790, 0.601631, 0.594113, 0.586618, 0.578897, 0.571946, 0.565086,
      0.558347, 0.551692, 0.545327, 0.539564, 0.533243, 0.527633, 0.521748,
      0.515664, 0.510595, 0.505292, 0.499665, 0.494358, 0.489393, 0.484308,
      0.479583, 0.474924, 0.470563, 0.466232, 0.461693, 0.457190, 0.452448,
      0.448032, 0.444391, 0.440026, 0.435699, 0.432123, 0.428138, 0.424321,
      0.420782, 0.416883, 0.413289, 0.409735, 0.405707, 0.402450, 0.399215,
      0.395999, 0.392500, 0.388618, 0.385457, 0.382315, 0.379190, 0.376082,
      0.372990, 0.369922, 0.366900, 0.364626, 0.361817, 0.358816, 0.355829,
      0.352862, 0.350431, 0.347966, 0.345037, 0.342344, 0.340168, 0.337340,
      0.334466, 0.331702, 0.329565, 0.326936, 0.324817, 0.322272, 0.319597,
      0.317498, 0.314905, 0.312819, 0.310739, 0.308664, 0.306596, 0.304044,
      0.301977, 0.299926, 0.297838, 0.295339, 0.293304, 0.291274, 0.288998,
      0.286730, 0.284714, 0.282705, 0.281133, 0.279201, 0.277203, 0.275203,
      0.273717, 0.271736, 0.269760, 0.267858, 0.266319, 0.264355, 0.262396,
      0.260627, 0.258989, 0.257041, 0.255599, 0.253660, 0.251728, 0.249866,
      0.248378, 0.246463, 0.245035, 0.243767, 0.242249, 0.240348, 0.238958,
      0.237079, 0.235468, 0.234221, 0.232976, 0.231602, 0.229745, 0.228261,
      0.227028, 0.225697, 0.223856, 0.222345, 0.221122, 0.219901, 0.218525,
      0.216701, 0.215253, 0.214041, 0.212831, 0.211624, 0.210419, 0.209215,
      0.208015, 0.206815, 0.205429, 0.203637, 0.202233, 0.201044, 0.199856,
      0.198671, 0.197489, 0.196308, 0.195131, 0.193955, 0.192782, 0.191612,
      0.190422, 0.189546, 0.188651, 0.187021, 0.186079, 0.185040, 0.184236,
      0.183021, 0.181718, 0.180766, 0.180041, 0.178422, 0.177404, 0.176613,
      0.175804, 0.174844, 0.173266, 0.172325, 0.171538, 0.170753, 0.169881,
      0.168279, 0.167336, 0.166526, 0.165717, 0.164919, 0.164123, 0.162639,
      0.161520, 0.160715, 0.159898, 0.159108, 0.158299, 0.157491, 0.156690,
      0.155893, 0.155095, 0.154288, 0.153487, 0.152689, 0.151886, 0.151077,
      0.149553, 0.148476, 0.147674, 0.146878, 0.146121, 0.145335, 0.144537,
      0.143739, 0.142966, 0.142393, 0.141664, 0.140796, 0.140260, 0.139449,
      0.139000, 0.138314, 0.137248, 0.137000, 0.136122, 0.136000, 0.135006,
      0.134000, 0.133897, 0.132924, 0.132401, 0.131758, 0.130718, 0.129841,
      0.129324, 0.129000, 0.128592, 0.127570, 0.126776, 0.126270, 0.126000,
      0.125524, 0.125000, 0.124531, 0.123542, 0.123000, 0.122578, 0.122000,
      0.121819, 0.121339, 0.120720, 0.119770, 0.119000, 0.118891, 0.118000,
      0.118000, 0.117562, 0.117105, 0.116302, 0.116000, 0.115509, 0.115000,
      0.114743, 0.114000, 0.114000, 0.113139, 0.113000, 0.112402, 0.111781,
      0.111364, 0.110894, 0.110072, 0.110000, 0.109451, 0.109000, 0.109000,
      0.109000, 0.108295, 0.108000, 0.107762, 0.107006, 0.107000, 0.106520,
      0.106000, 0.106000, 0.105347, 0.105000, 0.104935, 0.104242, 0.104000,
      0.103878, 0.103207, 0.103000, 0.103000, 0.103000, 0.102590, 0.102000,
      0.102000, 0.101687, 0.101063, 0.101000, 0.100836, 0.100231, 0.100000,
      0.100000, 0.100000, 0.099868, 0.099292, 0.099000, 0.099000, 0.098597,
      0.098042, 0.098000, 0.098000, 0.098000, 0.097832, 0.097257, 0.097000,
      0.097000, 0.097000, 0.096851, 0.096220, 0.096000, 0.096000, 0.096000,
      0.096000, 0.095891, 0.095186, 0.095000, 0.095000, 0.095000, 0.095000,
      0.095000, 0.095000, 0.094879, 0.094064, 0.094000, 0.094000, 0.094000,
      0.094000, 0.094000, 0.094000, 0.094000, 0.094000, 0.094000, 0.093023,
      0.093000, 0.093000, 0.093000, 0.093000, 0.093000, 0.093000, 0.093000,
      0.093000, 0.093000, 0.093000, 0.093000, 0.093000, 0.093000, 0.093000,
      0.092909, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000,
      0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000,
      0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000,
      0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000,
      0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.092000, 0.091708,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000, 0.091000,
      0.091000, 0.091000, 0.091000, 0.091000, 0.091000, NAN,      0.878978,
      0.878733, 0.876534, 0.874892, 0.873223, 0.871532, 0.870007, 0.868436,
      0.866576, 0.865264, 0.864198, 0.862872, 0.861488, 0.860475, 0.859664,
      0.858760, 0.857433, 0.856107, 0.855121, 0.854164, 0.853148, 0.852119,
      0.851026, 0.849920, 0.848745, 0.847543, 0.846294, 0.845493, 0.844642,
      0.843240, 0.841773, 0.840253, 0.838675, 0.837031, 0.836152, 0.834503,
      0.832623, 0.831329, 0.829600, 0.827444, 0.826092, 0.823902, 0.822296,
      0.820330, 0.817942, 0.815935, 0.812914, 0.810519, 0.808239, 0.805433,
      0.802485, 0.799620, 0.796259, 0.793037, 0.789619, 0.785534, 0.780819,
      0.776412, 0.771678, 0.766546, 0.761235, 0.755786, 0.749620, 0.742385,
      0.735089, 0.727336, 0.718857, 0.710055, 0.700571, 0.690960, 0.681301,
      0.671686, 0.662223, 0.652855, 0.644116, 0.635860, 0.627728, 0.619729,
      0.611865, 0.604107, 0.596448, 0.588895, 0.582162, 0.575522, 0.568998,
      0.562539, 0.556147, 0.549847, 0.543637, 0.537879, 0.532396, 0.526460,
      0.521192, 0.515590, 0.510323, 0.505382, 0.500321, 0.495619, 0.490178,
      0.485640, 0.481206, 0.475828, 0.471358, 0.466925, 0.462770, 0.458668,
      0.454346, 0.450062, 0.445810, 0.442070, 0.438394, 0.434228, 0.430088,
      0.426784, 0.422895, 0.419266, 0.416035, 0.412775, 0.408783, 0.405450,
      0.402290, 0.398933, 0.395022, 0.391910, 0.388817, 0.385738, 0.382673,
      0.379624, 0.376590, 0.373567, 0.370556, 0.367565, 0.364691, 0.362471,
      0.359683, 0.356751, 0.353832, 0.350929, 0.348529, 0.346158, 0.343294,
      0.340578, 0.338446, 0.335764, 0.332941, 0.330600, 0.328332, 0.325543,
      0.323325, 0.320995, 0.318677, 0.316611, 0.314552, 0.312000, 0.309634,
      0.307545, 0.304870, 0.302839, 0.301207, 0.298718, 0.296270, 0.294256,
      0.292747, 0.290743, 0.288744, 0.286664, 0.284258, 0.282271, 0.280290,
      0.278540, 0.276835, 0.274866, 0.272902, 0.271294, 0.269485, 0.267375,
      0.265389, 0.264091, 0.262196, 0.260505, 0.258827, 0.256899, 0.254979,
      0.253374, 0.251651, 0.249829, 0.248343, 0.246447, 0.245037, 0.243172,
      0.241528, 0.240278, 0.239031, 0.237184, 0.235548, 0.234312, 0.233078,
      0.230848, 0.229621, 0.228395, 0.226760, 0.224953, 0.223736, 0.222520,
      0.221307, 0.220096, 0.218772, 0.216679, 0.215474, 0.214270, 0.213068,
      0.211868, 0.210670, 0.209474, 0.208279, 0.206619, 0.204874, 0.203790,
      0.203077, 0.201500, 0.200067, 0.198846, 0.198058, 0.196315, 0.195349,
      0.194494, 0.193239, 0.191794, 0.190983, 0.189390, 0.188392, 0.187576,
      0.186531, 0.184986, 0.184164, 0.183352, 0.182081, 0.180736, 0.179948,
      0.179132, 0.177681, 0.176544, 0.175740, 0.174868, 0.173235, 0.172317,
      0.171520, 0.170720, 0.169827, 0.168227, 0.167309, 0.166501, 0.165696,
      0.164899, 0.164100, 0.163296, 0.161985, 0.160690, 0.159889, 0.159084,
      0.158300, 0.157608, 0.156963, 0.156078, 0.154851, 0.154000, 0.153543,
      0.152445, 0.151350, 0.150257, 0.149583, 0.149039, 0.148000, 0.147881,
      0.146801, 0.145722, 0.145000, 0.144575, 0.143505, 0.143000, 0.142369,
      0.141305, 0.140243, 0.140000, 0.139124, 0.138535, 0.138007, 0.136964,
      0.135958, 0.135435, 0.134827, 0.134000, 0.133875, 0.133359, 0.132687,
      0.131662, 0.131000, 0.130624, 0.129612, 0.129000, 0.128600, 0.128000,
      0.127611, 0.126625, 0.126000, 0.125667, 0.125000, 0.124734, 0.123889,
      0.123415, 0.122884, 0.122000, 0.122000, 0.121096, 0.121000, 0.120275,
      0.119690, 0.119244, 0.119000, 0.118728, 0.117862, 0.117004, 0.117000,
      0.116319, 0.116000, 0.115666, 0.115000, 0.115000, 0.114250, 0.114000,
      0.113685, 0.113000, 0.113000, 0.112409, 0.112000, 0.111939, 0.111219,
      0.111000, 0.110806, 0.110114, 0.110000, 0.109760, 0.109096, 0.109000,
      0.108802, 0.108169, 0.108000, 0.107932, 0.107332, 0.107000, 0.107000,
      0.106586, 0.106021, 0.106000, 0.105929, 0.105386, 0.105000, 0.105000,
      0.105000, 0.105000, 0.104775, 0.104247, 0.104000, 0.104000, 0.103644,
      0.103103, 0.103000, 0.103000, 0.103000, 0.103000, 0.103000, 0.102759,
      0.102185, 0.102000, 0.102000, 0.102000, 0.102000, 0.102000, 0.101621,
      0.101006, 0.101000, 0.101000, 0.101000, 0.101000, 0.101000, 0.101000,
      0.101000, 0.100622, 0.100000, 0.100000, 0.100000, 0.100000, 0.100000,
      0.100000, 0.100000, 0.100000, 0.100000, 0.100000, 0.100000, 0.100000,
      0.100000, 0.100000, 0.100000, 0.099156, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000, 0.099000,
      0.099000, 0.099000, 0.099000, NAN,      0.960478, 0.960335, 0.958929,
      0.957713, 0.956351, 0.954781, 0.953317, 0.951958, 0.950751, 0.949907,
      0.949062, 0.947727, 0.946354, 0.945267, 0.944280, 0.943242, 0.942184,
      0.940600, 0.938933, 0.937729, 0.936497, 0.935220, 0.933881, 0.932502,
      0.931072, 0.929568, 0.928505, 0.927392, 0.925707, 0.923950, 0.922109,
      0.920185, 0.918174, 0.916069, 0.914859, 0.912534, 0.910093, 0.908264,
      0.905824, 0.902956, 0.900937, 0.897718, 0.894652, 0.891835, 0.889098,
      0.885616, 0.881414, 0.878213, 0.874252, 0.869826, 0.865238, 0.860904,
      0.855709, 0.849892, 0.844163, 0.838180, 0.831565, 0.824140, 0.816262,
      0.807720, 0.798581, 0.788966, 0.779000, 0.768676, 0.758331, 0.747677,
      0.736821, 0.726419, 0.716937, 0.707367, 0.697876, 0.689049, 0.680383,
      0.671866, 0.663472, 0.655358, 0.647949, 0.640265, 0.633069, 0.625965,
      0.618877, 0.611667, 0.604813, 0.598522, 0.591991, 0.585498, 0.579576,
      0.573706, 0.567884, 0.562125, 0.556393, 0.550720, 0.545093, 0.539526,
      0.534318, 0.529488, 0.524199, 0.519632, 0.514272, 0.509283, 0.504886,
      0.500428, 0.495233, 0.490905, 0.486439, 0.481931, 0.477754, 0.473612,
      0.469503, 0.465424, 0.461375, 0.457357, 0.454096, 0.450408, 0.446470,
      0.442553, 0.438661, 0.435433, 0.431938, 0.428280, 0.424786, 0.421188,
      0.418160, 0.414434, 0.411145, 0.408153, 0.405171, 0.401500, 0.398240,
      0.395289, 0.392352, 0.389423, 0.386504, 0.383597, 0.380700, 0.377811,
      0.374936, 0.372067, 0.369210, 0.366362, 0.363522, 0.360692, 0.357871,
      0.355063, 0.352697, 0.350471, 0.347692, 0.344940, 0.342871, 0.340410,
      0.337665, 0.335199, 0.333152, 0.330484, 0.328079, 0.326049, 0.323528,
      0.321511, 0.319002, 0.316663, 0.314661, 0.311999, 0.310011, 0.307699,
      0.305543, 0.303564, 0.301590, 0.299159, 0.297153, 0.295459, 0.293729,
      0.291772, 0.289818, 0.287489, 0.285420, 0.283476, 0.281535, 0.280064,
      0.278162, 0.275974, 0.273806, 0.271883, 0.269968, 0.268370, 0.266650,
      0.264831, 0.263349, 0.261454, 0.260042, 0.258179, 0.256296, 0.254418,
      0.253028, 0.251173, 0.249537, 0.248296, 0.247056, 0.244818, 0.243584,
      0.241703, 0.240122, 0.238894, 0.237505, 0.235672, 0.234229, 0.233011,
      0.231653, 0.230134, 0.228362, 0.226864, 0.225667, 0.224585, 0.222673,
      0.220986, 0.220188, 0.218800, 0.217224, 0.215815, 0.215001, 0.213317,
      0.211695, 0.210548, 0.209499, 0.207945, 0.207161, 0.205751, 0.204129,
      0.202743, 0.201894, 0.200277, 0.199345, 0.198087, 0.196734, 0.195856,
      0.194248, 0.193329, 0.192526, 0.191439, 0.189920, 0.189122, 0.187639,
      0.186516, 0.185713, 0.184824, 0.183217, 0.182309, 0.181119, 0.179831,
      0.179067, 0.178162, 0.176923, 0.175669, 0.174543, 0.174000, 0.173298,
      0.172180, 0.171061, 0.169946, 0.168832, 0.167720, 0.167000, 0.166501,
      0.165394, 0.164266, 0.163154, 0.162526, 0.161950, 0.160850, 0.159752,
      0.158655, 0.158000, 0.157468, 0.156377, 0.155288, 0.154202, 0.153560,
      0.153020, 0.151963, 0.150945, 0.150411, 0.149756, 0.148695, 0.147819,
      0.147294, 0.146538, 0.145495, 0.144729, 0.144213, 0.143397, 0.143000,
      0.142358, 0.141349, 0.140342, 0.140000, 0.139347, 0.138357, 0.137687,
      0.137198, 0.136421, 0.136000, 0.135493, 0.134538, 0.133796, 0.133325,
      0.133000, 0.132786, 0.131865, 0.131000, 0.131000, 0.130158, 0.129274,
      0.128702, 0.128270, 0.128000, 0.127837, 0.127004, 0.126184, 0.126000,
      0.125577, 0.125000, 0.125000, 0.124255, 0.124000, 0.123770, 0.123045,
      0.122337, 0.122000, 0.121956, 0.121289, 0.121000, 0.120988, 0.120364,
      0.120000, 0.120000, 0.119566, 0.119000, 0.119000, 0.118825, 0.118237,
      0.118000, 0.118000, 0.117443, 0.117000, 0.117000, 0.117000, 0.116980,
      0.116347, 0.116000, 0.116000, 0.116000, 0.115756, 0.115091, 0.115000,
      0.115000, 0.115000, 0.115000, 0.115000, 0.114927, 0.114201, 0.114000,
      0.114000, 0.114000, 0.114000, 0.114000, 0.114000, 0.114000, 0.114000,
      0.114000, 0.113782, 0.113268, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000, 0.113000,
      NAN,      NAN,      1.003959, 1.003827, 1.002520, 1.001169, 0.999778,
      0.998325, 0.996812, 0.995246, 0.993621, 0.991931, 0.990161, 0.988317,
      0.987197, 0.985385, 0.983287, 0.981092, 0.978785, 0.976363, 0.974411,
      0.972155, 0.969332, 0.967180, 0.964213, 0.960944, 0.958349, 0.955293,
      0.951575, 0.948290, 0.944694, 0.940750, 0.936420, 0.931773, 0.927827,
      0.922842, 0.917429, 0.912041, 0.906803, 0.900851, 0.894283, 0.887033,
      0.879640, 0.872408, 0.864576, 0.855778, 0.847475, 0.838763, 0.829311,
      0.820290, 0.811081, 0.802417, 0.794029, 0.785162, 0.776630, 0.768608,
      0.760871, 0.753067, 0.744549, 0.737006, 0.729502, 0.722435, 0.715695,
      0.708518, 0.701873, 0.694928, 0.688187, 0.681697, 0.674779, 0.668393,
      0.662174, 0.656162, 0.650193, 0.644267, 0.638390, 0.633186, 0.627748,
      0.621980, 0.616354, 0.611468, 0.605881, 0.600779, 0.595629, 0.590178,
      0.585409, 0.580661, 0.575930, 0.570918, 0.565515, 0.560832, 0.556163,
      0.551923, 0.547864, 0.543234, 0.538619, 0.534015, 0.529686, 0.525843,
      0.521279, 0.516775, 0.513006, 0.509255, 0.505224, 0.500802, 0.497108,
      0.493432, 0.489772, 0.486127, 0.482005, 0.478398, 0.474810, 0.471236,
      0.467676, 0.464135, 0.460127, 0.457277, 0.454097, 0.450608, 0.447137,
      0.444139, 0.440725, 0.437284, 0.433882, 0.430932, 0.427518, 0.424691,
      0.421714, 0.418320, 0.415548, 0.412844, 0.409680, 0.406448, 0.403759,
      0.401072, 0.398391, 0.395066, 0.392036, 0.389364, 0.386696, 0.384029,
      0.381370, 0.378712, 0.376057, 0.373407, 0.370761, 0.368119, 0.365480,
      0.362847, 0.360217, 0.358590, 0.355968, 0.353349, 0.350732, 0.348342,
      0.346387, 0.343916, 0.341319, 0.339047, 0.337107, 0.334564, 0.331989,
      0.329814, 0.327852, 0.325468, 0.323549, 0.321177, 0.319221, 0.317080,
      0.314537, 0.312998, 0.310464, 0.308466, 0.306406, 0.303912, 0.302022,
      0.300422, 0.298746, 0.296814, 0.294478, 0.292597, 0.290291, 0.288344,
      0.286648, 0.285104, 0.283238, 0.281377, 0.279356, 0.277162, 0.275309,
      0.273959, 0.272113, 0.270512, 0.268927, 0.267089, 0.264836, 0.263615,
      0.261789, 0.260177, 0.258931, 0.256781, 0.254902, 0.253520, 0.252238,
      0.250601, 0.248773, 0.247137, 0.245572, 0.244021, 0.242690, 0.241612,
      0.240011, 0.238490, 0.236891, 0.235259, 0.233625, 0.231985, 0.231176,
      0.229727, 0.228113, 0.226520, 0.224974, 0.224162, 0.222707, 0.221098,
      0.219492, 0.217945, 0.217152, 0.215686, 0.214540, 0.213485, 0.211883,
      0.210273, 0.209336, 0.208063, 0.206727, 0.205854, 0.204254, 0.203349,
      0.202242, 0.200852, 0.199360, 0.198444, 0.197548, 0.196400, 0.195254,
      0.194109, 0.192966, 0.191823, 0.190682, 0.189543, 0.188405, 0.187270,
      0.186136, 0.185004, 0.183874, 0.182748, 0.181623, 0.180500, 0.179380,
      0.178263, 0.177147, 0.176518, 0.175926, 0.174820, 0.173717, 0.172618,
      0.171520, 0.170427, 0.169337, 0.168250, 0.168000, 0.167086, 0.166008,
      0.164935, 0.163933, 0.163400, 0.162737, 0.161678, 0.160624, 0.159575,
      0.158529, 0.158000, 0.157448, 0.156416, 0.155387, 0.154363, 0.154000,
      0.153332, 0.152327, 0.151324, 0.151000, 0.150340, 0.149358, 0.148693,
      0.148209, 0.147457, 0.146506, 0.145563, 0.145000, 0.144707, 0.143795,
      0.143000, 0.142989, 0.142106, 0.141233, 0.141000, 0.140522, 0.139683,
      0.139000, 0.139000, 0.138243, 0.137455, 0.137000, 0.136917, 0.136170,
      0.135437, 0.135000, 0.135000, 0.134317, 0.133638, 0.133000, 0.133000,
      0.132689, 0.132067, 0.131452, 0.131000, 0.131000, 0.130589, 0.130000,
      0.130000, 0.129680, 0.129033, 0.129000, 0.129000, 0.129000, 0.128377,
      0.128000, 0.128000, 0.128000, 0.127608, 0.127000, 0.127000, 0.127000,
      0.127000, 0.126958, 0.126198, 0.126000, 0.126000, 0.126000, 0.126000,
      0.126000, 0.126000, 0.126000, 0.125614, 0.125108, 0.125000, 0.125000,
      0.125000, 0.125000, 0.125000, 0.125000, 0.125000, 0.125000, 0.125000,
      0.125000, 0.125000, 0.125000, 0.125000, 0.125000, 0.125000, 0.125000,
      0.125000, 0.124868, 0.124333, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000, 0.124000,
      0.124000, 0.124000, 0.124000, 0.124000, 0.124000, NAN,      NAN,
      1.080128, 1.079971, 1.078383, 1.076735, 1.075020, 1.073225, 1.071356,
      1.069407, 1.067374, 1.065250, 1.063029, 1.060694, 1.058245, 1.055678,
      1.052986, 1.050134, 1.047140, 1.044482, 1.041612, 1.038056, 1.034270,
      1.031241, 1.026969, 1.023324, 1.019378, 1.015181, 1.010347, 1.004868,
      0.999855, 0.994518, 0.988617, 0.982680, 0.975943, 0.968608, 0.961144,
      0.953536, 0.945541, 0.937013, 0.928267, 0.918675, 0.909096, 0.899867,
      0.889532, 0.879832, 0.870584, 0.861578, 0.851844, 0.842865, 0.834082,
      0.825564, 0.817233, 0.809172, 0.800955, 0.792729, 0.785061, 0.777609,
      0.769866, 0.762631, 0.755592, 0.748600, 0.741653, 0.734754, 0.727908,
      0.721089, 0.714317, 0.707638, 0.701788, 0.695256, 0.689192, 0.683077,
      0.676723, 0.671035, 0.665384, 0.659763, 0.654169, 0.648603, 0.643063,
      0.637557, 0.632076, 0.626618, 0.621298, 0.616660, 0.611384, 0.606441,
      0.601663, 0.596330, 0.591728, 0.587185, 0.581933, 0.577139, 0.572631,
      0.568132, 0.563646, 0.559166, 0.554692, 0.550225, 0.545768, 0.541320,
      0.537576, 0.533477, 0.529077, 0.524692, 0.520938, 0.516970, 0.513195,
      0.509311, 0.505000, 0.501426, 0.497863, 0.494171, 0.490273, 0.486745,
      0.483235, 0.479737, 0.476097, 0.472276, 0.468813, 0.465367, 0.461931,
      0.458603, 0.455594, 0.452190, 0.448802, 0.445737, 0.442554, 0.439193,
      0.435843, 0.432999, 0.430333, 0.427340, 0.424018, 0.421205, 0.417895,
      0.415072, 0.412431, 0.409493, 0.406199, 0.403528, 0.400900, 0.398275,
      0.395477, 0.392031, 0.389413, 0.386798, 0.384183, 0.381571, 0.378959,
      0.376350, 0.373743, 0.371139, 0.368536, 0.365937, 0.363339, 0.360745,
      0.358576, 0.356565, 0.353980, 0.351399, 0.348821, 0.346685, 0.344674,
      0.342106, 0.339541, 0.337485, 0.335423, 0.332870, 0.330320, 0.328774,
      0.326233, 0.324021, 0.322119, 0.319719, 0.317822, 0.315569, 0.313534,
      0.311523, 0.309255, 0.307369, 0.305488, 0.303479, 0.301236, 0.299367,
      0.297002, 0.295141, 0.293522, 0.291572, 0.289102, 0.287728, 0.285884,
      0.284330, 0.282314, 0.280207, 0.278321, 0.277061, 0.275086, 0.273239,
      0.271525, 0.269679, 0.268154, 0.266588, 0.264974, 0.263302, 0.261439,
      0.259139, 0.258210, 0.256183, 0.254177, 0.252575, 0.250999, 0.250174,
      0.248753, 0.247153, 0.245541, 0.243940, 0.242342, 0.240719, 0.239109,
      0.237510, 0.235904, 0.234303, 0.233350, 0.232097, 0.230494, 0.228889,
      0.227284, 0.226341, 0.225078, 0.223498, 0.221970, 0.220587, 0.219447,
      0.218268, 0.216917, 0.215758, 0.214600, 0.213168, 0.211436, 0.210138,
      0.208986, 0.207836, 0.206687, 0.205540, 0.204393, 0.203248, 0.202105,
      0.200962, 0.199821, 0.198682, 0.197544, 0.196407, 0.195273, 0.194140,
      0.193009, 0.191880, 0.190754, 0.189628, 0.188506, 0.187386, 0.186269,
      0.185154, 0.184043, 0.182934, 0.181829, 0.180729, 0.179630, 0.178536,
      0.177447, 0.176361, 0.176000, 0.175202, 0.174132, 0.173063, 0.172001,
      0.170946, 0.169898, 0.169000, 0.168814, 0.167783, 0.166760, 0.165739,
      0.164728, 0.163725, 0.162865, 0.162371, 0.161759, 0.160786, 0.159821,
      0.159000, 0.158919, 0.157985, 0.157052, 0.156134, 0.155226, 0.155000,
      0.154442, 0.153565, 0.152700, 0.151846, 0.151003, 0.151000, 0.150360,
      0.149557, 0.149000, 0.148987, 0.148227, 0.147479, 0.146745, 0.146024,
      0.146000, 0.145635, 0.144959, 0.144305, 0.144000, 0.144000, 0.143400,
      0.142760, 0.142116, 0.142000, 0.141809, 0.141147, 0.141000, 0.140804,
      0.140123, 0.140000, 0.139738, 0.139035, 0.139000, 0.139000, 0.138876,
      0.138139, 0.138000, 0.138000, 0.138000, 0.138000, 0.137309, 0.137000,
      0.137000, 0.137000, 0.137000, 0.137000, 0.137000, 0.137000, 0.136538,
      0.136014, 0.136000, 0.136000, 0.136000, 0.136000, 0.136000, 0.136000,
      0.136000, 0.136000, 0.136000, 0.136000, 0.136000, 0.136000, 0.136000,
      0.136000, 0.136000, 0.135617, 0.135183, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000, 0.135000,
      0.135000, 0.135000, 0.135000, NAN,      NAN,      1.215770, 1.215520,
      1.213003, 1.210369, 1.207595, 1.204675, 1.201602, 1.198354, 1.194916,
      1.191278, 1.187398, 1.183266, 1.178910, 1.174188, 1.169639, 1.165029,
      1.160013, 1.154447, 1.148450, 1.142237, 1.135859, 1.129222, 1.121712,
      1.114313, 1.106482, 1.097903, 1.088665, 1.079769, 1.069379, 1.059041,
      1.048724, 1.038173, 1.026944, 1.016532, 1.005655, 0.995087, 0.985373,
      0.975404, 0.965363, 0.955616, 0.946296, 0.936563, 0.927690, 0.918875,
      0.910154, 0.901508, 0.892941, 0.885297, 0.877015, 0.868651, 0.860596,
      0.852899, 0.845421, 0.838098, 0.830820, 0.823589, 0.816406, 0.809260,
      0.802149, 0.795173, 0.788536, 0.781690, 0.775487, 0.769314, 0.762684,
      0.756034, 0.749925, 0.743842, 0.737777, 0.731729, 0.725696, 0.719684,
      0.713688, 0.708278, 0.702738, 0.696784, 0.690863, 0.685679, 0.680506,
      0.675105, 0.669216, 0.664550, 0.659419, 0.654302, 0.649194, 0.644095,
      0.639007, 0.633930, 0.628554, 0.623830, 0.619250, 0.614206, 0.609176,
      0.604560, 0.600260, 0.595634, 0.590689, 0.586425, 0.582177, 0.577940,
      0.573723, 0.569433, 0.564543, 0.560142, 0.555971, 0.552178, 0.548664,
      0.544528, 0.540400, 0.536279, 0.532170, 0.528066, 0.524645, 0.520885,
      0.517339, 0.513734, 0.509668, 0.505674, 0.502299, 0.498928, 0.495562,
      0.492205, 0.488422, 0.485008, 0.481668, 0.478335, 0.474510, 0.471191,
      0.467880, 0.464660, 0.461779, 0.458488, 0.455705, 0.452432, 0.449164,
      0.445920, 0.443148, 0.439901, 0.436661, 0.433540, 0.430698, 0.427476,
      0.424808, 0.422241, 0.419347, 0.416146, 0.413451, 0.410260, 0.407458,
      0.404910, 0.402363, 0.399818, 0.397274, 0.394164, 0.391189, 0.388648,
      0.386108, 0.383567, 0.381029, 0.378491, 0.375953, 0.373415, 0.370879,
      0.368343, 0.365807, 0.363270, 0.360734, 0.358198, 0.356247, 0.354127,
      0.351592, 0.349058, 0.346526, 0.343994, 0.341848, 0.339934, 0.337408,
      0.334882, 0.333180, 0.331076, 0.328581, 0.326071, 0.323888, 0.322100,
      0.319721, 0.317904, 0.316000, 0.313745, 0.312111, 0.309853, 0.307953,
      0.305939, 0.303652, 0.302065, 0.299591, 0.297816, 0.296228, 0.294398,
      0.291977, 0.290366, 0.288758, 0.286744, 0.284572, 0.282963, 0.281363,
      0.279754, 0.277724, 0.275548, 0.273935, 0.272331, 0.270730, 0.269125,
      0.267519, 0.265919, 0.264313, 0.262074, 0.260267, 0.258935, 0.257581,
      0.255858, 0.253917, 0.252607, 0.250841, 0.249384, 0.248209, 0.246552,
      0.244862, 0.243535, 0.241780, 0.240351, 0.239183, 0.238015, 0.235850,
      0.234686, 0.233522, 0.232041, 0.230299, 0.229040, 0.227824, 0.226090,
      0.224572, 0.223420, 0.222268, 0.221119, 0.219973, 0.218654, 0.216685,
      0.215546, 0.214408, 0.213274, 0.212143, 0.211014, 0.209889, 0.208768,
      0.207298, 0.205535, 0.204425, 0.203317, 0.202214, 0.201115, 0.200021,
      0.198929, 0.197844, 0.196764, 0.195686, 0.194614, 0.193550, 0.192489,
      0.191433, 0.190384, 0.189344, 0.188308, 0.187279, 0.186259, 0.185246,
      0.184246, 0.183249, 0.182262, 0.182000, 0.181320, 0.180364, 0.179423,
      0.178489, 0.177564, 0.176654, 0.175757, 0.174872, 0.173999, 0.173143,
      0.173000, 0.172473, 0.171659, 0.170859, 0.170075, 0.169310, 0.168560,
      0.168000, 0.168000, 0.167407, 0.166723, 0.166054, 0.165404, 0.164752,
      0.164096, 0.164000, 0.163764, 0.163091, 0.162409, 0.162000, 0.162000,
      0.161327, 0.161000, 0.160903, 0.160179, 0.160000, 0.160000, 0.159955,
      0.159195, 0.159000, 0.159000, 0.158857, 0.158057, 0.158000, 0.158000,
      0.158000, 0.157740, 0.157000, 0.157000, 0.157000, 0.157000, 0.157000,
      0.157000, 0.157000, 0.157000, 0.156946, 0.156535, 0.156124, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000, 0.156000,
      0.156000, NAN,      NAN,      1.328699, 1.328288, 1.324003, 1.319368,
      1.314455, 1.309171, 1.303546, 1.298077, 1.291741, 1.285212, 1.278868,
      1.271177, 1.263970, 1.256110, 1.247303, 1.237975, 1.228437, 1.218488,
      1.208052, 1.197448, 1.186554, 1.175221, 1.163469, 1.151884, 1.140075,
      1.128474, 1.117361, 1.106213, 1.095227, 1.084385, 1.073589, 1.063816,
      1.054192, 1.044084, 1.034221, 1.024869, 1.015645, 1.006763, 0.997825,
      0.988797, 0.979849, 0.971710, 0.963181, 0.954767, 0.946879, 0.939051,
      0.930612, 0.922553, 0.914889, 0.907741, 0.900687, 0.893148, 0.885663,
      0.878214, 0.871018, 0.864372, 0.857761, 0.850744, 0.844111, 0.837572,
      0.830948, 0.823736, 0.817638, 0.811375, 0.804709, 0.798484, 0.792797,
      0.786519, 0.780468, 0.774302, 0.768189, 0.762564, 0.756952, 0.751346,
      0.745220, 0.739161, 0.733578, 0.728627, 0.723431, 0.717866, 0.712305,
      0.706748, 0.701196, 0.696235, 0.691101, 0.685738, 0.680889, 0.675479,
      0.670458, 0.665861, 0.661021, 0.656183, 0.650847, 0.646014, 0.641181,
      0.636351, 0.631534, 0.627194, 0.622437, 0.617992, 0.613872, 0.608831,
      0.604767, 0.600167, 0.595670, 0.591637, 0.587537, 0.582858, 0.578591,
      0.574597, 0.570617, 0.566643, 0.562678, 0.558724, 0.554777, 0.550834,
      0.546902, 0.542975, 0.539054, 0.535424, 0.532226, 0.528319, 0.524418,
      0.521013, 0.517625, 0.513733, 0.510038, 0.506801, 0.503079, 0.499466,
      0.496601, 0.493371, 0.489643, 0.486418, 0.483195, 0.479968, 0.476252,
      0.473035, 0.470256, 0.467107, 0.463897, 0.460689, 0.456985, 0.453785,
      0.450870, 0.447895, 0.444764, 0.442217, 0.439007, 0.436131, 0.433389,
      0.430059, 0.427528, 0.424501, 0.421476, 0.418955, 0.416437, 0.412922,
      0.410410, 0.407900, 0.405393, 0.402859, 0.399730, 0.396882, 0.394382,
      0.391882, 0.389382, 0.386882, 0.384382, 0.381864, 0.379209, 0.376175,
      0.373388, 0.371130, 0.368509, 0.365936, 0.363574, 0.361530, 0.359173,
      0.356723, 0.354360, 0.351951, 0.349460, 0.347108, 0.344648, 0.342267,
      0.340247, 0.338431, 0.336028, 0.333657, 0.331255, 0.329215, 0.327389,
      0.325014, 0.322591, 0.320790, 0.318775, 0.316574, 0.314966, 0.312546,
      0.310762, 0.308742, 0.306557, 0.304927, 0.302517, 0.300741, 0.298789,
      0.296613, 0.294736, 0.292941, 0.291123, 0.289219, 0.287919, 0.286137,
      0.284140, 0.282078, 0.280302, 0.279019, 0.277259, 0.275661, 0.274228,
      0.272465, 0.270274, 0.268948, 0.267193, 0.265626, 0.264187, 0.262437,
      0.260252, 0.258963, 0.257600, 0.255639, 0.254480, 0.252643, 0.251165,
      0.250010, 0.247857, 0.246706, 0.245557, 0.243819, 0.242265, 0.241121,
      0.238981, 0.237844, 0.236707, 0.235575, 0.234169, 0.232479, 0.231195,
      0.229614, 0.227958, 0.226845, 0.225736, 0.224632, 0.223519, 0.222411,
      0.220981, 0.219349, 0.218149, 0.217075, 0.216001, 0.214936, 0.213817,
      0.212241, 0.210780, 0.209742, 0.208712, 0.207691, 0.206679, 0.205671,
      0.204676, 0.203691, 0.202717, 0.201755, 0.200797, 0.199855, 0.198925,
      0.198006, 0.197102, 0.196212, 0.195334, 0.194476, 0.193628, 0.192786,
      0.191965, 0.191163, 0.190376, 0.189602, 0.188841, 0.188102, 0.187383,
      0.186679, 0.185989, 0.185325, 0.184660, 0.184000, 0.184000, 0.183645,
      0.182963, 0.182272, 0.181575, 0.181000, 0.181000, 0.180442, 0.180000,
      0.179983, 0.179239, 0.179000, 0.178726, 0.178000, 0.178000, 0.178000,
      0.177583, 0.177000, 0.177000, 0.177000, 0.177000, 0.176405, 0.176000,
      0.176000, 0.176000, 0.176000, 0.176000, 0.176000, 0.175670, 0.175194,
      0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000,
      0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000,
      0.175000, 0.175000, 0.175000, 0.175000, 0.174925, 0.174590, 0.174255,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000, 0.174000,
      0.174000, 0.174000, 0.174000, 0.174463, 0.175000, 0.175000, 0.175000,
      0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000,
      0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000,
      0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000,
      0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000,
      0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000, 0.175000,
      NAN,      1.466082, 1.465530, 1.459626, 1.453923, 1.448149, 1.441537,
      1.434086, 1.427342, 1.419264, 1.410681, 1.401782, 1.392441, 1.382266,
      1.371823, 1.360551, 1.348462, 1.335958, 1.322229, 1.308414, 1.294621,
      1.280567, 1.266117, 1.252203, 1.238595, 1.225057, 1.211524, 1.198683,
      1.186388, 1.174507, 1.162737, 1.150946, 1.139882, 1.128962, 1.118188,
      1.107544, 1.097034, 1.086628, 1.076329, 1.066551, 1.057308, 1.048075,
      1.038189, 1.029112, 1.020206, 1.011371, 1.002727, 0.994816, 0.986269,
      0.977695, 0.969432, 0.961733, 0.954096, 0.945955, 0.937953, 0.930437,
      0.922979, 0.915547, 0.908164, 0.900817, 0.893752, 0.887198, 0.879983,
      0.872767, 0.866235, 0.859801, 0.853321, 0.846231, 0.840125, 0.833798,
      0.827471, 0.821157, 0.814862, 0.808089, 0.801817, 0.795948, 0.790326,
      0.784100, 0.778341, 0.772182, 0.766324, 0.760825, 0.755337, 0.749853,
      0.744378, 0.738906, 0.733439, 0.727976, 0.722517, 0.717059, 0.711604,
      0.706150, 0.700734, 0.695963, 0.690789, 0.685501, 0.680879, 0.675424,
      0.670610, 0.665548, 0.660613, 0.655879, 0.651155, 0.646940, 0.642238,
      0.637543, 0.632835, 0.627870, 0.623870, 0.619360, 0.614895, 0.610574,
      0.605951, 0.601993, 0.597385, 0.593096, 0.589157, 0.585227, 0.581303,
      0.576787, 0.572480, 0.568578, 0.564683, 0.560797, 0.556915, 0.553041,
      0.549172, 0.545309, 0.541451, 0.538167, 0.534753, 0.530911, 0.527071,
      0.523238, 0.519938, 0.516578, 0.512751, 0.509107, 0.505924, 0.502741,
      0.499471, 0.495880, 0.492700, 0.489025, 0.485844, 0.482399, 0.478990,
      0.475814, 0.472639, 0.469771, 0.466790, 0.463540, 0.459944, 0.456774,
      0.454084, 0.451323, 0.447770, 0.444605, 0.441943, 0.438783, 0.435899,
      0.433375, 0.430815, 0.427666, 0.424711, 0.421610, 0.418649, 0.415959,
      0.413431, 0.410519, 0.407808, 0.404980, 0.402160, 0.399555, 0.397155,
      0.394809, 0.391920, 0.389022, 0.386550, 0.384089, 0.381678, 0.379302,
      0.376890, 0.374257, 0.371110, 0.368625, 0.366230, 0.363833, 0.361416,
      0.359016, 0.356613, 0.354796, 0.352779, 0.350379, 0.347983, 0.345566,
      0.343158, 0.340756, 0.338569, 0.336946, 0.334653, 0.332445, 0.330317,
      0.328278, 0.326268, 0.324109, 0.321690, 0.319273, 0.317860, 0.315450,
      0.313521, 0.311638, 0.309234, 0.307878, 0.306084, 0.304055, 0.302003,
      0.300215, 0.297932, 0.296153, 0.294583, 0.292798, 0.290429, 0.289035,
      0.287289, 0.285368, 0.283512, 0.282012, 0.280259, 0.278011, 0.276842,
      0.275358, 0.273280, 0.271544, 0.270207, 0.268584, 0.266905, 0.265518,
      0.263424, 0.261713, 0.260337, 0.258803, 0.257104, 0.255910, 0.254222,
      0.252691, 0.251572, 0.250186, 0.248518, 0.247237, 0.245702, 0.244051,
      0.242940, 0.241775, 0.240151, 0.238687, 0.237613, 0.236321, 0.234730,
      0.233431, 0.232384, 0.231020, 0.229469, 0.228288, 0.227272, 0.226266,
      0.225272, 0.224282, 0.222605, 0.221336, 0.220380, 0.219435, 0.218505,
      0.217585, 0.216673, 0.215777, 0.214791, 0.213056, 0.212177, 0.211340,
      0.210519, 0.209713, 0.208922, 0.208151, 0.207397, 0.206659, 0.205936,
      0.205236, 0.204554, 0.203881, 0.203212, 0.202537, 0.201858, 0.201171,
      0.201000, 0.200779, 0.200073, 0.199358, 0.199000, 0.198909, 0.198171,
      0.197425, 0.197000, 0.196907, 0.196134, 0.196000, 0.196000, 0.195752,
      0.195000, 0.195000, 0.195000, 0.194421, 0.194000, 0.194000, 0.194000,
      0.194000, 0.194000, 0.193802, 0.193247, 0.193000, 0.193000, 0.193000,
      0.193000, 0.193000, 0.193000, 0.193000, 0.192897, 0.192577, 0.192257,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000, 0.192000,
      0.192000, 0.192000, 0.192000, 0.192000, 0.192231, 0.193000, 0.193000,
      0.193000, 0.193000, 0.193000, 0.193000, 0.193000, 0.193000, 0.193000,
      0.193000, 0.193000, 0.193000, 0.193000, 0.193000, 0.193000, 0.193000,
      0.193000, 0.193000, 0.193000, 0.193000, NAN,      NAN,      1.504698,
      1.503284, 1.489290, 1.475262, 1.461269, 1.446989, 1.432236, 1.418470,
      1.404349, 1.390319, 1.377653, 1.364239, 1.350947, 1.338451, 1.326279,
      1.314163, 1.302187, 1.290316, 1.278540, 1.267155, 1.256275, 1.244793,
      1.234178, 1.223703, 1.213312, 1.203004, 1.192783, 1.183379, 1.173617,
      1.163906, 1.154752, 1.144955, 1.135750, 1.126825, 1.117989, 1.109201,
      1.100472, 1.092504, 1.084220, 1.075662, 1.067497, 1.059748, 1.052060,
      1.043991, 1.035896, 1.028716, 1.021168, 1.013670, 1.006196, 0.998767,
      0.991835, 0.984447, 0.977626, 0.970543, 0.963309, 0.956676, 0.950070,
      0.943476, 0.936901, 0.930001, 0.923116, 0.917239, 0.910757, 0.904256,
      0.897774, 0.891300, 0.884842, 0.879114, 0.872960, 0.866827, 0.861123,
      0.854720, 0.848899, 0.842950, 0.836925, 0.831198, 0.825983, 0.820275,
      0.814574, 0.808891, 0.803684, 0.797994, 0.791896, 0.786665, 0.781614,
      0.776262, 0.770584, 0.765474, 0.760430, 0.755060, 0.749387, 0.744302,
      0.739261, 0.734221, 0.729189, 0.724267, 0.719877, 0.715133, 0.710130,
      0.705130, 0.700138, 0.695150, 0.691125, 0.686190, 0.681662, 0.677246,
      0.672279, 0.667902, 0.663355, 0.658722, 0.654386, 0.650552, 0.646220,
      0.641588, 0.637061, 0.632771, 0.628907, 0.624583, 0.620651, 0.616439,
      0.612244, 0.608543, 0.604124, 0.600144, 0.596446, 0.592541, 0.588230,
      0.584361, 0.580670, 0.576979, 0.573291, 0.569603, 0.565903, 0.561606,
      0.557551, 0.554225, 0.551157, 0.547510, 0.543832, 0.540156, 0.536481,
      0.532808, 0.529136, 0.525976, 0.522793, 0.519124, 0.515546, 0.512489,
      0.509119, 0.505450, 0.501855, 0.499113, 0.495442, 0.492491, 0.489616,
      0.486346, 0.482959, 0.480010, 0.476419, 0.473129, 0.469933, 0.467060,
      0.464513, 0.461175, 0.458016, 0.454872, 0.452021, 0.449474, 0.446293,
      0.443055, 0.440360, 0.437584, 0.434370, 0.431167, 0.428717, 0.425722,
      0.422876, 0.420300, 0.417084, 0.414655, 0.412249, 0.409453, 0.406426,
      0.404018, 0.401612, 0.398608, 0.395804, 0.393403, 0.390997, 0.388589,
      0.385802, 0.383044, 0.380548, 0.378105, 0.375623, 0.373108, 0.370698,
      0.368297, 0.366449, 0.364504, 0.362110, 0.359718, 0.357329, 0.354941,
      0.352555, 0.350170, 0.347787, 0.345555, 0.343770, 0.341649, 0.339274,
      0.336950, 0.335528, 0.333159, 0.330791, 0.328820, 0.327048, 0.324778,
      0.323010, 0.320991, 0.318980, 0.317217, 0.314956, 0.313199, 0.311256,
      0.309190, 0.307439, 0.305254, 0.302947, 0.301207, 0.299646, 0.298237,
      0.296508, 0.294378, 0.292566, 0.290801, 0.288762, 0.287440, 0.285743,
      0.283734, 0.281869, 0.280189, 0.279012, 0.276904, 0.275803, 0.274406,
      0.272417, 0.270787, 0.269442, 0.267734, 0.266301, 0.265237, 0.263363,
      0.262134, 0.260647, 0.259095, 0.258040, 0.257026, 0.255044, 0.254032,
      0.253043, 0.251136, 0.250105, 0.249153, 0.247425, 0.246289, 0.245370,
      0.244464, 0.243147, 0.241697, 0.240834, 0.239985, 0.239153, 0.237674,
      0.236536, 0.235751, 0.234980, 0.234232, 0.233500, 0.232565, 0.231166,
      0.230408, 0.229735, 0.229061, 0.228381, 0.227695, 0.227004, 0.226304,
      0.226000, 0.225886, 0.225166, 0.224437, 0.223701, 0.223000, 0.223000,
      0.222440, 0.222000, 0.221888, 0.221096, 0.221000, 0.221000, 0.220655,
      0.220000, 0.220000, 0.220000, 0.219237, 0.219000, 0.219000, 0.219000,
      0.219000, 0.218580, 0.218021, 0.218000, 0.218000, 0.218000, 0.218000,
      0.217816, 0.217451, 0.217085, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217000,
      0.217000, 0.217000, 0.217000, 0.217000, 0.217000, 0.217159, 0.217788,
      0.218000, 0.218000, 0.218000, 0.218000, 0.218000, 0.218000, 0.218000,
      0.218000, 0.218000, 0.218000, 0.218000, 0.218000, 0.218000, 0.218000,
      0.218000, 0.218000, 0.218000, 0.218000, 0.218000, 0.218000, 0.218000,
      0.218000, 0.218000, 0.218000, 0.218000, 0.218771, 0.219000, 0.219000,
      0.219000, 0.219000, 0.219000, 0.219000, 0.219000, 0.219000, 0.219000,
      0.219000, 0.219000, 0.219000, 0.219000, 0.219000, 0.219000, 0.219602,
      0.220000, 0.220000, NAN,      NAN,      1.637630, 1.636349, 1.623063,
      1.610288, 1.597856, 1.585478, 1.573161, 1.561235, 1.549163, 1.536990,
      1.524864, 1.513515, 1.502193, 1.490940, 1.479724, 1.468574, 1.458155,
      1.447347, 1.436326, 1.425629, 1.415363, 1.405156, 1.395306, 1.385337,
      1.375230, 1.365162, 1.355629, 1.345640, 1.336031, 1.326753, 1.317515,
      1.308302, 1.299118, 1.289951, 1.280819, 1.271703, 1.263615, 1.254550,
      1.245899, 1.237477, 1.228656, 1.220312, 1.212446, 1.204181, 1.195889,
      1.187615, 1.179857, 1.171623, 1.163675, 1.156105, 1.148544, 1.141008,
      1.133480, 1.125977, 1.118481, 1.111010, 1.103555, 1.096113, 1.089223,
      1.082293, 1.075173, 1.068423, 1.061209, 1.054487, 1.047793, 1.041120,
      1.034051, 1.027333, 1.020828, 1.014613, 1.008029, 1.001873, 0.994892,
      0.988957, 0.983028, 0.976830, 0.970339, 0.964330, 0.958461, 0.952596,
      0.946748, 0.940908, 0.935077, 0.929260, 0.923447, 0.917646, 0.911854,
      0.906066, 0.900288, 0.894968, 0.889755, 0.884100, 0.878926, 0.873506,
      0.868093, 0.862933, 0.857309, 0.852130, 0.846870, 0.841527, 0.836706,
      0.831572, 0.826046, 0.820815, 0.815724, 0.811172, 0.806451, 0.801336,
      0.796532, 0.791616, 0.786509, 0.781906, 0.776806, 0.771852, 0.767324,
      0.762799, 0.758275, 0.753754, 0.749235, 0.744718, 0.740203, 0.735690,
      0.731179, 0.726667, 0.722155, 0.717641, 0.713126, 0.708638, 0.704477,
      0.700205, 0.695767, 0.691310, 0.687123, 0.683033, 0.678672, 0.674576,
      0.670833, 0.666614, 0.662427, 0.657928, 0.653675, 0.649656, 0.645681,
      0.641652, 0.637507, 0.633488, 0.629802, 0.626444, 0.622472, 0.618357,
      0.614360, 0.610370, 0.606445, 0.603251, 0.599278, 0.595279, 0.591617,
      0.588240, 0.584257, 0.580239, 0.576785, 0.573224, 0.569367, 0.566160,
      0.562190, 0.558744, 0.555168, 0.551338, 0.548192, 0.544721, 0.541338,
      0.538059, 0.534194, 0.530459, 0.527253, 0.524288, 0.520784, 0.517347,
      0.514372, 0.511072, 0.507495, 0.503944, 0.501343, 0.497772, 0.494802,
      0.491640, 0.488078, 0.485600, 0.482639, 0.479416, 0.476226, 0.473273,
      0.469824, 0.466880, 0.464150, 0.461499, 0.458562, 0.455155, 0.452761,
      0.450129, 0.446852, 0.443933, 0.441412, 0.439081, 0.436192, 0.433426,
      0.430870, 0.427947, 0.425432, 0.422646, 0.419803, 0.417492, 0.415185,
      0.412319, 0.409579, 0.407282, 0.404987, 0.402622, 0.399767, 0.397131,
      0.394856, 0.392585, 0.390319, 0.388060, 0.385810, 0.383563, 0.381326,
      0.379097, 0.376874, 0.374661, 0.372462, 0.370266, 0.368083, 0.365913,
      0.363758, 0.361607, 0.359473, 0.357516, 0.355937, 0.354153, 0.352074,
      0.350015, 0.347969, 0.345934, 0.344460, 0.342924, 0.340950, 0.338989,
      0.337044, 0.335562, 0.334223, 0.332344, 0.330617, 0.329247, 0.327839,
      0.326052, 0.324291, 0.323277, 0.321845, 0.320161, 0.318752, 0.317874,
      0.316278, 0.314715, 0.313591, 0.312676, 0.311206, 0.310389, 0.309380,
      0.308011, 0.307329, 0.306296, 0.304962, 0.304268, 0.303568, 0.302862,
      0.302148, 0.301425, 0.300696, 0.299959, 0.299212, 0.298456, 0.297691,
      0.297000, 0.297000, 0.296337, 0.295531, 0.295000, 0.294887, 0.294048,
      0.294000, 0.294000, 0.293451, 0.293000, 0.293000, 0.292778, 0.292000,
      0.292000, 0.292000, 0.292000, 0.291629, 0.291135, 0.291000, 0.291000,
      0.291000, 0.291000, 0.291000, 0.290765, 0.290437, 0.290108, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290000,
      0.290000, 0.290000, 0.290000, 0.290000, 0.290000, 0.290143, 0.290698,
      0.291000, 0.291000, 0.291000, 0.291000, 0.291000, 0.291000, 0.291000,
      0.291000, 0.291000, 0.291000, 0.291000, 0.291000, 0.291000, 0.291000,
      0.291000, 0.291000, 0.291000, 0.291000, 0.291000, 0.291000, 0.291000,
      0.291000, 0.291000, 0.291570, 0.292000, 0.292000, 0.292000, 0.292000,
      0.292000, 0.292000, 0.292000, 0.292000, 0.292000, 0.292000, 0.292000,
      0.292000, 0.292000, 0.292472, 0.293000, 0.293000, 0.293000, 0.293000,
      0.293000, 0.293000, 0.293000, 0.293000, 0.293000, 0.293000, 0.294000,
      0.294000, 0.294000, 0.294000, 0.294000, 0.294000, 0.294000, 0.294000,
      0.294000, 0.294730, 0.295000, 0.295000, 0.295000, 0.295000, 0.295000,
      0.295000, 0.295000, 0.295781, 0.296000, 0.296000, 0.296000, 0.296000,
      0.296000, 0.296000, 0.296000, 0.297000, 0.297000, 0.297000, 0.297000,
      0.297000, 0.297000, 0.297323, 0.298000, 0.298000, 0.298000, 0.298000,
      0.298000, 0.298122, 0.298813, 0.299000, 0.299000, 0.299000, 0.299000,
      0.299000, 0.299000, 0.299514, 0.300000, 0.300000, 0.300000, 0.300000,
      0.300000, 0.300288, 0.301000, 0.301000, 0.301000, 0.301000, 0.301000,
      0.301120, 0.301918, 0.302000, 0.302000, 0.302000, 0.302000, 0.302000,
      0.302000, 0.302615, 0.303000, 0.303000, 0.303000, 0.303000, 0.303000,
      0.303396, 0.304000, 0.304000, 0.304000, 0.304000, 0.304000, 0.304000,
      0.304000, NAN,      1.756203, 1.754951, 1.743143, 1.731064, 1.718817,
      1.707191, 1.695608, 1.684041, 1.672515, 1.661001, 1.649525, 1.638073,
      1.627115, 1.616245, 1.605100, 1.594375, 1.583183, 1.572504, 1.562089,
      1.551717, 1.541100, 1.531137, 1.521190, 1.511270, 1.501360, 1.491480,
      1.481621, 1.471778, 1.461965, 1.452265, 1.443386, 1.433636, 1.424661,
      1.415195, 1.405690, 1.397039, 1.388000, 1.379380, 1.370489, 1.361607,
      1.353055, 1.344120, 1.335588, 1.327292, 1.319004, 1.310741, 1.302494,
      1.294258, 1.286045, 1.277861, 1.270652, 1.262482, 1.254320, 1.246863,
      1.239469, 1.231918, 1.223895, 1.216376, 1.208621, 1.201180, 1.194261,
      1.186407, 1.179447, 1.172304, 1.164654, 1.157768, 1.150885, 1.144008,
      1.137136, 1.130266, 1.123401, 1.116541, 1.109684, 1.102831, 1.095984,
      1.089139, 1.083299, 1.076468, 1.069638, 1.063081, 1.056830, 1.050987,
      1.044381, 1.038115, 1.031983, 1.026150, 1.019920, 1.013389, 1.007330,
      1.001241, 0.995019, 0.989366, 0.983720, 0.978076, 0.972183, 0.965987,
      0.960179, 0.954557, 0.948945, 0.943335, 0.937734, 0.932142, 0.926552,
      0.920972, 0.915760, 0.910748, 0.905271, 0.899684, 0.894503, 0.889133,
      0.883784, 0.879037, 0.873297, 0.868271, 0.863269, 0.858342, 0.853300,
      0.847763, 0.842896, 0.838311, 0.833382, 0.828257, 0.823514, 0.818691,
      0.813957, 0.809209, 0.804394, 0.799497, 0.794722, 0.789804, 0.784957,
      0.780463, 0.776349, 0.771581, 0.766743, 0.762424, 0.758105, 0.753326,
      0.748484, 0.744390, 0.739857, 0.735371, 0.731362, 0.727354, 0.722619,
      0.718341, 0.714332, 0.710191, 0.705375, 0.701554, 0.697389, 0.692935,
      0.689377, 0.685034, 0.680963, 0.677241, 0.672865, 0.669114, 0.665102,
      0.661124, 0.657283, 0.652945, 0.649586, 0.645732, 0.641716, 0.637516,
      0.633653, 0.630105, 0.626053, 0.622472, 0.619154, 0.614834, 0.611431,
      0.607552, 0.603863, 0.600292, 0.596411, 0.592881, 0.589553, 0.586224,
      0.582860, 0.578571, 0.575251, 0.571935, 0.568621, 0.565313, 0.562008,
      0.558707, 0.555411, 0.552122, 0.548833, 0.545553, 0.542277, 0.539009,
      0.535743, 0.532486, 0.529233, 0.525985, 0.523497, 0.520516, 0.517288,
      0.514069, 0.510883, 0.508219, 0.505466, 0.502283, 0.499256, 0.496609,
      0.493753, 0.491047, 0.488387, 0.485201, 0.482676, 0.479827, 0.477427,
      0.474879, 0.472225, 0.469087, 0.466923, 0.464278, 0.461707, 0.459595,
      0.457361, 0.454734, 0.452288, 0.449795, 0.447113, 0.445036, 0.442965,
      0.440905, 0.438282, 0.435819, 0.433790, 0.431774, 0.429773, 0.427793,
      0.425821, 0.423865, 0.421930, 0.420014, 0.418117, 0.416241, 0.414389,
      0.412562, 0.410740, 0.408952, 0.407190, 0.405455, 0.403745, 0.402062,
      0.400405, 0.398776, 0.397177, 0.395614, 0.394080, 0.392574, 0.391100,
      0.390334, 0.389240, 0.387809, 0.386362, 0.384900, 0.383713, 0.382941,
      0.381436, 0.380458, 0.379691, 0.378832, 0.377263, 0.376338, 0.375609,
      0.374940, 0.374356, 0.373609, 0.372602, 0.371825, 0.371389, 0.370883,
      0.369796, 0.368819, 0.368147, 0.368000, 0.367462, 0.367000, 0.366556,
      0.366000, 0.366000, 0.365592, 0.365000, 0.365000, 0.364492, 0.364000,
      0.364000, 0.364000, 0.364000, 0.363000, 0.363000, 0.363000, 0.363000,
      0.363000, 0.363000, 0.363000, 0.363000, 0.363000, 0.363000, 0.363000,
      0.363000, 0.363130, 0.363535, 0.363940, 0.364000, 0.364000, 0.364000,
      0.364000, 0.364000, 0.364000, 0.364229, 0.364506, 0.364783, 0.365000,
      0.365000, 0.365000, 0.365000, 0.365042, 0.365272, 0.365502, 0.365733,
      0.365963, 0.366000, 0.366000, 0.366000, 0.366000, 0.366220, 0.366487,
      0.366755, 0.367000, 0.367000, 0.367000, 0.367000, 0.367177, 0.367482,
      0.367786, 0.368000, 0.368000, 0.368000, 0.368068, 0.368409, 0.368750,
      0.369000, 0.369000, 0.369000, 0.369000, 0.369000, 0.369000, 0.369334,
      0.369731, 0.370000, 0.370000, 0.370000, 0.370000, 0.370000, 0.370000,
      0.370000, 0.370184, 0.370655, 0.371000, 0.371000, 0.371000, 0.371000,
      0.371000, 0.371000, 0.371000, 0.371000, 0.371287, 0.371850, 0.372000,
      0.372000, 0.372000, 0.372000, 0.372000, 0.372000, 0.372000, 0.372000,
      0.372000, 0.372100, 0.372792, 0.373000, 0.373000, 0.373000, 0.373000,
      0.373000, 0.373000, 0.373000, 0.373000, 0.373000, 0.373000, 0.373000,
      0.373000, 0.373000, 0.373000, 0.373000, 0.373771, 0.374000, 0.374000,
      0.374000, 0.374000, 0.374000, 0.374000, 0.374000, 0.374000, 0.374000,
      0.374000, 0.374000, 0.374000, 0.374000, 0.374000, 0.374000, 0.374000,
      0.374000, 0.374000, 0.374000, 0.374436, 0.375000, 0.375000, 0.375000,
      0.375000, 0.375000, 0.375000, 0.375000, 0.375000, 0.375000, 0.375000,
      0.375000, 0.375000, 0.375000, 0.375000, 0.375000, 0.375000, 0.375000,
      0.375370, 0.376000, 0.376000, 0.376000, 0.376000, 0.376000, 0.376000,
      0.376000, 0.376000, 0.376000, 0.376000, 0.376000, 0.376000, 0.376378,
      0.377000, 0.377000, 0.377000, 0.377000, 0.377000, 0.377000, 0.377000,
      0.377000, 0.377000, 0.377000, 0.377000, 0.377098, 0.377855, 0.378000,
      0.378000, 0.378000, 0.378000, 0.378000, 0.378000, 0.378000, 0.378000,
      0.378000, 0.378000, 0.378000, 0.378000, 0.378319, 0.379000, 0.379000,
      0.379000, 0.379000, 0.379000, 0.379000, 0.379000, 0.379000, 0.379000,
      0.379000, 0.379000, 0.379000, 0.379000, 0.379612, 0.380000, 0.380000,
      0.380000, 0.380000, 0.380000, 0.380000, 0.380000, NAN,      NAN,
      1.878557, 1.877257, 1.864663, 1.852454, 1.840260, 1.828196, 1.816665,
      1.804841, 1.793165, 1.781673, 1.769746, 1.758324, 1.747240, 1.736046,
      1.724697, 1.713845, 1.702534, 1.691536, 1.680859, 1.670193, 1.659549,
      1.648927, 1.638315, 1.628076, 1.618081, 1.607607, 1.597649, 1.587893,
      1.577987, 1.567845, 1.557930, 1.548530, 1.538643, 1.528951, 1.519663,
      1.510381, 1.501106, 1.491843, 1.482583, 1.473335, 1.464341, 1.455687,
      1.446642, 1.437432, 1.428321, 1.420156, 1.411547, 1.402676, 1.393860,
      1.385280, 1.377197, 1.369157, 1.360615, 1.352274, 1.344325, 1.336383,
      1.328453, 1.319612, 1.312162, 1.304739, 1.296861, 1.288996, 1.281138,
      1.273296, 1.265864, 1.258601, 1.250853, 1.243620, 1.236267, 1.228854,
      1.221973, 1.214779, 1.207254, 1.199922, 1.192754, 1.186091, 1.179455,
      1.172321, 1.165328, 1.158748, 1.152167, 1.145583, 1.138331, 1.131410,
      1.124816, 1.118222, 1.111654, 1.105600, 1.099412, 1.092675, 1.086057,
      1.079841, 1.073176, 1.067179, 1.061099, 1.054892, 1.048589, 1.041888,
      1.036175, 1.030021, 1.023628, 1.017506, 1.011787, 1.005593, 0.999664,
      0.994046, 0.988472, 0.982453, 0.976087, 0.970474, 0.964876, 0.959183,
      0.953536, 0.947945, 0.942283, 0.936628, 0.931014, 0.925522, 0.920689,
      0.915186, 0.909573, 0.904260, 0.899378, 0.893739, 0.888806, 0.883842,
      0.878785, 0.873695, 0.868411, 0.863245, 0.858403, 0.853520, 0.848269,
      0.843687, 0.838912, 0.834053, 0.828735, 0.824359, 0.819523, 0.814835,
      0.810546, 0.806048, 0.801236, 0.796715, 0.792450, 0.787841, 0.783055,
      0.778689, 0.774447, 0.770213, 0.765976, 0.760757, 0.756540, 0.752331,
      0.748123, 0.743924, 0.739732, 0.735547, 0.731365, 0.727644, 0.724028,
      0.719870, 0.715717, 0.711574, 0.707438, 0.703649, 0.700043, 0.696081,
      0.691976, 0.688247, 0.684633, 0.680573, 0.676405, 0.673164, 0.669508,
      0.665796, 0.661669, 0.658391, 0.654652, 0.651064, 0.647839, 0.644367,
      0.640578, 0.637095, 0.633821, 0.630541, 0.627247, 0.623939, 0.620615,
      0.617288, 0.613944, 0.610588, 0.607222, 0.604565, 0.601457, 0.598063,
      0.595654, 0.592363, 0.589506, 0.586373, 0.583271, 0.580394, 0.577010,
      0.574690, 0.572058, 0.568808, 0.565896, 0.563384, 0.561047, 0.558131,
      0.555362, 0.553016, 0.550668, 0.548319, 0.545461, 0.542619, 0.540268,
      0.537919, 0.535571, 0.533227, 0.530884, 0.528660, 0.526911, 0.524888,
      0.522569, 0.520696, 0.518957, 0.516669, 0.514396, 0.512564, 0.510882,
      0.508646, 0.507212, 0.505222, 0.503534, 0.501872, 0.499864, 0.498705,
      0.497133, 0.495713, 0.494372, 0.492673, 0.491514, 0.490040, 0.488718,
      0.487527, 0.485826, 0.484905, 0.484001, 0.482237, 0.481223, 0.480325,
      0.479414, 0.477972, 0.476545, 0.475590, 0.474620, 0.473719, 0.472952,
      0.472185, 0.471338, 0.470357, 0.469299, 0.468226, 0.467137, 0.466514,
      0.465914, 0.464778, 0.463626, 0.463000, 0.462284, 0.461094, 0.460885,
      0.459659, 0.459000, 0.458584, 0.457905, 0.456810, 0.456162, 0.455504,
      0.455000, 0.454695, 0.454075, 0.454000, 0.454000, 0.453550, 0.453000,
      0.453000, 0.453000, 0.453000, 0.453000, 0.453000, 0.453000, 0.453000,
      0.453000, 0.453000, 0.453000, 0.453000, 0.453296, 0.453600, 0.453903,
      0.454000, 0.454000, 0.454000, 0.454000, 0.454236, 0.454478, 0.454720,
      0.454962, 0.455000, 0.455000, 0.455000, 0.455000, 0.455052, 0.455251,
      0.455450, 0.455649, 0.455848, 0.456000, 0.456000, 0.456000, 0.456000,
      0.456000, 0.456112, 0.456341, 0.456570, 0.456799, 0.457000, 0.457000,
      0.457000, 0.457000, 0.457000, 0.457000, 0.457000, 0.457000, 0.457000,
      0.457000, 0.457000, 0.457000, 0.457247, 0.457550, 0.457853, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458007, 0.458402, 0.458798, 0.459000,
      0.459000, 0.459000, 0.459000, 0.459000, 0.459000, 0.459000, 0.459000,
      0.459000, 0.459000, 0.459000, 0.459000, 0.459000, 0.459200, 0.459725,
      0.460000, 0.460000, 0.460000, 0.460000, 0.460000, 0.460000, 0.460000,
      0.460000, 0.460000, 0.460000, 0.460000, 0.460000, 0.460000, 0.460000,
      0.460000, 0.460565, 0.461000, 0.461000, 0.461000, 0.461000, 0.461000,
      0.461000, 0.461000, 0.461000, 0.461000, 0.461000, 0.461000, 0.461000,
      0.461000, 0.461910, 0.462000, 0.462000, 0.462000, 0.462000, 0.462000,
      0.462000, 0.462000, 0.462000, 0.462000, 0.462000, 0.462000, 0.462000,
      0.462000, 0.462604, 0.463000, 0.463000, 0.463000, 0.463000, 0.463000,
      0.463000, 0.463000, 0.463000, 0.463000, 0.463000, 0.463000, 0.463000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.466000, 0.466000, 0.466000,
      0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000,
      0.466526, 0.467000, 0.467000, 0.467000, 0.467000, 0.467000, 0.467000,
      0.467000, 0.467000, 0.467000, 0.467576, 0.468000, 0.468000, 0.468000,
      0.468000, 0.468000, 0.468000, 0.468000, 0.468049, 0.468863, 0.469000,
      0.469000, 0.469000, 0.469000, 0.469000, 0.469000, 0.469000, 0.469280,
      0.470000, 0.470000, 0.470000, 0.470000, 0.470000, 0.470000, 0.470867,
      0.471000, 0.471000, 0.471000, NAN,      NAN,      1.982333, 1.981071,
      1.968259, 1.955864, 1.943656, 1.930868, 1.918987, 1.906852, 1.894783,
      1.883197, 1.871623, 1.860052, 1.847644, 1.836457, 1.825420, 1.813905,
      1.802399, 1.790903, 1.779805, 1.768909, 1.758534, 1.747673, 1.736986,
      1.726248, 1.715475, 1.705238, 1.695013, 1.684297, 1.673606, 1.663712,
      1.654260, 1.644106, 1.633962, 1.624237, 1.614682, 1.604617, 1.594611,
      1.585098, 1.575791, 1.566599, 1.557116, 1.548015, 1.538246, 1.529215,
      1.520325, 1.511445, 1.502573, 1.493704, 1.484843, 1.475991, 1.467143,
      1.458297, 1.449460, 1.441152, 1.432799, 1.424103, 1.415837, 1.407801,
      1.399535, 1.391061, 1.382811, 1.374995, 1.367304, 1.359093, 1.350938,
      1.343265, 1.335596, 1.327493, 1.319276, 1.311629, 1.303986, 1.297157,
      1.289722, 1.282107, 1.274522, 1.267232, 1.259522, 1.252599, 1.245189,
      1.237904, 1.230902, 1.223675, 1.216499, 1.209638, 1.202601, 1.195624,
      1.188778, 1.181689, 1.175070, 1.168219, 1.161199, 1.154292, 1.147852,
      1.141514, 1.134507, 1.127649, 1.121170, 1.114790, 1.108402, 1.101947,
      1.095493, 1.089291, 1.083593, 1.077274, 1.070872, 1.064409, 1.058225,
      1.052639, 1.045969, 1.039986, 1.034508, 1.028495, 1.022223, 1.016478,
      1.010719, 1.004813, 0.999370, 0.994003, 0.988102, 0.982275, 0.976915,
      0.971413, 0.965525, 0.959854, 0.954509, 0.949171, 0.943837, 0.938508,
      0.933185, 0.927875, 0.922567, 0.918216, 0.912983, 0.907710, 0.902439,
      0.897186, 0.892051, 0.887347, 0.882498, 0.877636, 0.873108, 0.868146,
      0.863505, 0.858639, 0.854012, 0.849654, 0.845038, 0.840120, 0.835560,
      0.831408, 0.826889, 0.822184, 0.817850, 0.813631, 0.809385, 0.805136,
      0.800861, 0.796567, 0.792248, 0.788192, 0.784573, 0.780425, 0.776584,
      0.772406, 0.768349, 0.764461, 0.761044, 0.756620, 0.752665, 0.748888,
      0.745467, 0.742036, 0.738589, 0.734166, 0.730638, 0.727146, 0.723630,
      0.720102, 0.716559, 0.713492, 0.710416, 0.707012, 0.703998, 0.700566,
      0.697424, 0.694244, 0.690794, 0.687707, 0.684129, 0.681589, 0.678858,
      0.675715, 0.673048, 0.669885, 0.666963, 0.664402, 0.661795, 0.658567,
      0.655666, 0.653064, 0.650457, 0.648127, 0.646158, 0.643573, 0.640931,
      0.638710, 0.636623, 0.633959, 0.632146, 0.629616, 0.627204, 0.625193,
      0.623680, 0.621666, 0.619652, 0.617758, 0.616126, 0.614117, 0.612741,
      0.610810, 0.609074, 0.607744, 0.606420, 0.604199, 0.602786, 0.601477,
      0.600174, 0.598879, 0.597592, 0.596310, 0.595033, 0.593757, 0.592485,
      0.591209, 0.589940, 0.588674, 0.587422, 0.586181, 0.583941, 0.582670,
      0.581488, 0.580164, 0.578645, 0.577212, 0.575816, 0.574104, 0.571970,
      0.570509, 0.569034, 0.566805, 0.565027, 0.563258, 0.560973, 0.559430,
      0.557322, 0.555328, 0.553773, 0.552215, 0.550655, 0.549093, 0.547569,
      0.546136, 0.545990, 0.545041, 0.544227, 0.544000, 0.544000, 0.544000,
      0.544000, 0.544000, 0.544000, 0.544000, 0.544138, 0.544523, 0.544908,
      0.545258, 0.545595, 0.545933, 0.546000, 0.546000, 0.546000, 0.546120,
      0.546386, 0.546652, 0.546917, 0.547000, 0.547000, 0.547000, 0.547000,
      0.547106, 0.547321, 0.547537, 0.547752, 0.547968, 0.548000, 0.548000,
      0.548000, 0.548000, 0.548000, 0.548137, 0.548318, 0.548498, 0.548679,
      0.548859, 0.549000, 0.549000, 0.549000, 0.549000, 0.549000, 0.549000,
      0.549045, 0.549228, 0.549411, 0.549594, 0.549777, 0.549960, 0.550000,
      0.550000, 0.550000, 0.550000, 0.550000, 0.550000, 0.550000, 0.550000,
      0.550000, 0.550064, 0.550302, 0.550540, 0.550779, 0.551000, 0.551000,
      0.551000, 0.551000, 0.551000, 0.551000, 0.551000, 0.551000, 0.551160,
      0.551454, 0.551748, 0.552000, 0.552000, 0.552000, 0.552000, 0.552000,
      0.552000, 0.552000, 0.552000, 0.552000, 0.552000, 0.552000, 0.552000,
      0.552128, 0.552514, 0.552900, 0.553000, 0.553000, 0.553000, 0.553000,
      0.553000, 0.553000, 0.553000, 0.553000, 0.553000, 0.553000, 0.553000,
      0.553186, 0.553683, 0.554000, 0.554000, 0.554000, 0.554000, 0.554000,
      0.554000, 0.554000, 0.554000, 0.554000, 0.554000, 0.554000, 0.554000,
      0.554029, 0.554673, 0.555000, 0.555000, 0.555000, 0.555000, 0.555000,
      0.555000, 0.555000, 0.555000, 0.555000, 0.555000, 0.555000, 0.555000,
      0.555188, 0.556000, 0.556000, 0.556000, 0.556000, 0.556000, 0.556000,
      0.556000, 0.556000, 0.556000, 0.556000, 0.556000, 0.556000, 0.556285,
      0.557000, 0.557000, 0.557000, 0.557000, 0.557000, 0.557000, 0.557000,
      0.557000, 0.557000, 0.557000, 0.557000, 0.557343, 0.558000, 0.558000,
      0.558000, 0.558000, 0.558000, 0.558000, 0.558000, 0.558000, 0.558000,
      0.558000, 0.558273, 0.559000, 0.559000, 0.559000, 0.559000, 0.559000,
      0.559000, 0.559000, 0.559000, 0.559000, 0.559845, 0.560000, 0.560000,
      0.560000, 0.560000, 0.560000, 0.560000, 0.560000, 0.560000, 0.560688,
      0.561000, 0.561000, 0.561000, 0.561000, 0.561000, 0.561000, 0.561000,
      0.561000, 0.562000, 0.562000, 0.562000, 0.562000, 0.562000, 0.562000,
      0.562000, 0.562994, 0.563000, 0.563000, 0.563000, 0.563000, 0.563000,
      0.563000, 0.563562, 0.564000, 0.564000, 0.564000, 0.564000, 0.564601,
      0.565000, 0.565000, 0.565000, 0.565000, 0.565000, 0.565436, 0.566000,
      0.566000, 0.566000, 0.566000, 0.566412, 0.567000, 0.567000, 0.567000,
      0.567000, 0.567418, 0.568000, 0.568000, 0.568000, 0.568000, 0.568391,
      0.569000, NAN,      NAN,      2.098579, 2.097199, 2.084062, 2.071066,
      2.058561, 2.046064, 2.032985, 2.020081, 2.007598, 1.995116, 1.982703,
      1.970799, 1.958699, 1.946755, 1.934770, 1.922313, 1.910778, 1.898934,
      1.887145, 1.875835, 1.864529, 1.853227, 1.841928, 1.830636, 1.819347,
      1.808068, 1.796794, 1.785872, 1.775264, 1.764313, 1.753631, 1.743379,
      1.732793, 1.722180, 1.711886, 1.701021, 1.690955, 1.680899, 1.670857,
      1.660819, 1.650794, 1.640793, 1.631350, 1.621786, 1.611805, 1.602068,
      1.592659, 1.583597, 1.574375, 1.564994, 1.555129, 1.546263, 1.536916,
      1.527713, 1.518928, 1.510152, 1.501382, 1.492616, 1.483859, 1.475106,
      1.466359, 1.457624, 1.448985, 1.440491, 1.432055, 1.423511, 1.415241,
      1.407188, 1.399319, 1.391017, 1.382664, 1.374535, 1.366646, 1.358943,
      1.350908, 1.342885, 1.335013, 1.326804, 1.319258, 1.311435, 1.303708,
      1.296206, 1.288416, 1.281080, 1.273546, 1.266159, 1.259032, 1.252050,
      1.244757, 1.237479, 1.230274, 1.223166, 1.216159, 1.209251, 1.202201,
      1.195095, 1.188080, 1.181381, 1.174940, 1.168139, 1.161182, 1.154682,
      1.148283, 1.141901, 1.135522, 1.129159, 1.122809, 1.116473, 1.110143,
      1.103828, 1.097529, 1.091241, 1.084962, 1.079249, 1.073454, 1.067218,
      1.061661, 1.055785, 1.049593, 1.043801, 1.038148, 1.032919, 1.026970,
      1.021692, 1.016102, 1.010439, 1.005222, 0.999030, 0.993794, 0.988561,
      0.983294, 0.978005, 0.972690, 0.967692, 0.963010, 0.957863, 0.953000,
      0.947801, 0.942349, 0.937692, 0.933297, 0.927875, 0.923359, 0.918332,
      0.913483, 0.908971, 0.904443, 0.899896, 0.895314, 0.890724, 0.886327,
      0.882453, 0.877784, 0.873095, 0.868824, 0.864727, 0.861006, 0.856809,
      0.852939, 0.848848, 0.844649, 0.840797, 0.836481, 0.832553, 0.828805,
      0.825047, 0.821548, 0.818378, 0.814617, 0.810769, 0.807239, 0.803995,
      0.800718, 0.797117, 0.793613, 0.790426, 0.787422, 0.784047, 0.780641,
      0.777574, 0.774821, 0.771803, 0.768315, 0.765438, 0.762609, 0.759766,
      0.757173, 0.755012, 0.752110, 0.749641, 0.747249, 0.744293, 0.742487,
      0.740243, 0.737983, 0.735711, 0.733428, 0.731132, 0.728822, 0.727001,
      0.725173, 0.722890, 0.721325, 0.719755, 0.717769, 0.715599, 0.714014,
      0.712423, 0.710829, 0.709232, 0.707631, 0.706024, 0.704400, 0.702757,
      0.701097, 0.699419, 0.697726, 0.696024, 0.694314, 0.692601, 0.690891,
      0.688777, 0.686354, 0.684232, 0.681923, 0.679558, 0.677690, 0.674988,
      0.672476, 0.668958, 0.666104, 0.663331, 0.659467, 0.655587, 0.652281,
      0.648841, 0.645001, 0.641179, 0.637403, 0.634499, 0.631618, 0.628873,
      0.627488, 0.625959, 0.624701, 0.623914, 0.623229, 0.623000, 0.623000,
      0.623000, 0.623000, 0.623436, 0.623875, 0.624274, 0.624657, 0.625035,
      0.625371, 0.625708, 0.626000, 0.626000, 0.626000, 0.626000, 0.626201,
      0.626466, 0.626730, 0.626994, 0.627000, 0.627000, 0.627000, 0.627000,
      0.627164, 0.627378, 0.627593, 0.627807, 0.628020, 0.628215, 0.628411,
      0.628607, 0.628802, 0.628998, 0.629000, 0.629000, 0.629000, 0.629000,
      0.629000, 0.629072, 0.629239, 0.629406, 0.629573, 0.629739, 0.629906,
      0.630000, 0.630000, 0.630000, 0.630000, 0.630000, 0.630000, 0.630003,
      0.630150, 0.630297, 0.630444, 0.630591, 0.630737, 0.630884, 0.631000,
      0.631000, 0.631000, 0.631000, 0.631000, 0.631000, 0.631026, 0.631210,
      0.631393, 0.631576, 0.631760, 0.631943, 0.632000, 0.632000, 0.632000,
      0.632000, 0.632000, 0.632161, 0.632381, 0.632602, 0.632822, 0.633000,
      0.633000, 0.633000, 0.633000, 0.633000, 0.633000, 0.633000, 0.633000,
      0.633031, 0.633307, 0.633582, 0.633858, 0.634000, 0.634000, 0.634000,
      0.634000, 0.634000, 0.634000, 0.634000, 0.634000, 0.634000, 0.634000,
      0.634302, 0.634651, 0.635000, 0.635000, 0.635000, 0.635000, 0.635000,
      0.635000, 0.635000, 0.635000, 0.635096, 0.635519, 0.635943, 0.636000,
      0.636000, 0.636000, 0.636000, 0.636000, 0.636000, 0.636000, 0.636000,
      0.636000, 0.636000, 0.636207, 0.636741, 0.637000, 0.637000, 0.637000,
      0.637000, 0.637000, 0.637000, 0.637000, 0.637000, 0.637000, 0.637000,
      0.637309, 0.637972, 0.638000, 0.638000, 0.638000, 0.638000, 0.638000,
      0.638000, 0.638000, 0.638000, 0.638000, 0.638000, 0.638170, 0.639000,
      0.639000, 0.639000, 0.639000, 0.639000, 0.639000, 0.639000, 0.639000,
      0.639000, 0.639213, 0.640000, 0.640000, 0.640000, 0.640000, 0.640000,
      0.640000, 0.640000, 0.640000, 0.640000, 0.641000, 0.641000, 0.641000,
      0.641000, 0.641000, 0.641000, 0.641000, 0.641000, 0.641355, 0.642000,
      0.642000, 0.642000, 0.642000, 0.642000, 0.642000, 0.642000, 0.643000,
      0.643000, 0.643000, 0.643000, 0.643000, 0.643000, 0.643000, 0.644000,
      0.644000, 0.644000, 0.644000, 0.644000, 0.644000, 0.644764, 0.645000,
      0.645000, 0.645000, 0.645000, 0.645000, 0.646000, 0.646000, 0.646000,
      0.646000, 0.646000, 0.647000, 0.647000, 0.647000, 0.647000, 0.647000,
      0.648000, 0.648000, 0.648000, 0.648000, 0.648970, 0.649000, 0.649000,
      0.649000, 0.650000, 0.650000, 0.650000, 0.650000, 0.650835, 0.651000,
      0.651000, 0.651173, 0.651900, 0.652000, 0.652598, 0.653000, 0.653000,
      0.653000, 0.653760, 0.654000, 0.654432, 0.655000, 0.655000, 0.655856,
      0.656000, 0.656410, 0.657000, 0.657000, 0.657808, 0.658000, 0.658416,
      0.659000, 0.659046, 0.659813, 0.660594, 0.661000, 0.661197, NAN,
      NAN,      2.208361, 2.206930, 2.193066, 2.179498, 2.165934, 2.152385,
      2.139849, 2.126330, 2.113161, 2.100543, 2.087607, 2.074317, 2.061680,
      2.048842, 2.035977, 2.023634, 2.011295, 1.998961, 1.986632, 1.974307,
      1.961984, 1.950601, 1.938341, 1.926201, 1.914704, 1.902809, 1.891613,
      1.879760, 1.868226, 1.857035, 1.845844, 1.834655, 1.823470, 1.812287,
      1.801351, 1.790730, 1.779752, 1.768584, 1.757899, 1.747331, 1.737196,
      1.726604, 1.716224, 1.706200, 1.695861, 1.685293, 1.675166, 1.665172,
      1.655180, 1.645199, 1.635384, 1.625974, 1.616371, 1.606854, 1.597462,
      1.587914, 1.577954, 1.568623, 1.559465, 1.550103, 1.540950, 1.532212,
      1.522798, 1.514132, 1.505422, 1.496472, 1.487404, 1.478430, 1.470033,
      1.461319, 1.452547, 1.444207, 1.436346, 1.427700, 1.419112, 1.410866,
      1.402841, 1.394652, 1.386290, 1.378346, 1.370575, 1.363266, 1.355207,
      1.347274, 1.339768, 1.332073, 1.324046, 1.316289, 1.308813, 1.301348,
      1.293889, 1.287235, 1.280006, 1.272586, 1.265168, 1.257803, 1.251385,
      1.244020, 1.236851, 1.230322, 1.223286, 1.216863, 1.210101, 1.203144,
      1.196341, 1.189895, 1.183166, 1.176693, 1.170433, 1.164140, 1.157834,
      1.151502, 1.145140, 1.139179, 1.133306, 1.126892, 1.120467, 1.114500,
      1.108658, 1.103192, 1.097467, 1.091400, 1.085646, 1.080079, 1.074477,
      1.068866, 1.063294, 1.058178, 1.052832, 1.047678, 1.042326, 1.036539,
      1.031746, 1.026466, 1.021377, 1.016635, 1.011876, 1.007091, 1.001672,
      0.996819, 0.992546, 0.987649, 0.982753, 0.978403, 0.973759, 0.969135,
      0.964716, 0.960255, 0.956095, 0.951757, 0.947333, 0.943140, 0.938540,
      0.934482, 0.930484, 0.926445, 0.922489, 0.919077, 0.915166, 0.911346,
      0.907826, 0.903677, 0.900132, 0.897048, 0.893761, 0.890268, 0.887181,
      0.883494, 0.880415, 0.877413, 0.874387, 0.871335, 0.868252, 0.865150,
      0.862014, 0.859389, 0.857000, 0.854592, 0.852161, 0.849704, 0.847022,
      0.844252, 0.841702, 0.839770, 0.837586, 0.835330, 0.833383, 0.830832,
      0.829061, 0.827271, 0.825475, 0.823494, 0.820837, 0.818999, 0.817760,
      0.816268, 0.814359, 0.812413, 0.810425, 0.808923, 0.807661, 0.806215,
      0.804083, 0.802610, 0.800725, 0.798503, 0.796268, 0.794028, 0.791772,
      0.789642, 0.786972, 0.784005, 0.780988, 0.777107, 0.773167, 0.768454,
      0.762834, 0.757200, 0.750828, 0.744491, 0.738236, 0.732126, 0.726455,
      0.720835, 0.716511, 0.712806, 0.708978, 0.707337, 0.705886, 0.704779,
      0.704178, 0.703636, 0.703118, 0.703346, 0.703794, 0.704000, 0.704000,
      0.704000, 0.704335, 0.704678, 0.705000, 0.705000, 0.705000, 0.705000,
      0.705201, 0.705470, 0.705739, 0.706006, 0.706247, 0.706488, 0.706729,
      0.706970, 0.707000, 0.707000, 0.707000, 0.707000, 0.707055, 0.707253,
      0.707451, 0.707649, 0.707848, 0.708042, 0.708224, 0.708406, 0.708588,
      0.708770, 0.708952, 0.709000, 0.709000, 0.709000, 0.709000, 0.709000,
      0.709000, 0.709127, 0.709284, 0.709441, 0.709599, 0.709756, 0.709913,
      0.710066, 0.710214, 0.710362, 0.710510, 0.710658, 0.710806, 0.710954,
      0.711000, 0.711000, 0.711000, 0.711000, 0.711000, 0.711000, 0.711000,
      0.711076, 0.711211, 0.711347, 0.711482, 0.711617, 0.711753, 0.711888,
      0.712000, 0.712000, 0.712000, 0.712000, 0.712000, 0.712000, 0.712000,
      0.712116, 0.712288, 0.712460, 0.712632, 0.712805, 0.712977, 0.713000,
      0.713000, 0.713000, 0.713000, 0.713000, 0.713131, 0.713340, 0.713549,
      0.713758, 0.713968, 0.714000, 0.714000, 0.714000, 0.714000, 0.714112,
      0.714358, 0.714604, 0.714850, 0.715000, 0.715000, 0.715000, 0.715000,
      0.715000, 0.715000, 0.715000, 0.715024, 0.715326, 0.715628, 0.715929,
      0.716000, 0.716000, 0.716000, 0.716000, 0.716000, 0.716000, 0.716000,
      0.716000, 0.716000, 0.716332, 0.716708, 0.717000, 0.717000, 0.717000,
      0.717000, 0.717000, 0.717000, 0.717000, 0.717000, 0.717420, 0.717870,
      0.718000, 0.718000, 0.718000, 0.718000, 0.718000, 0.718000, 0.718279,
      0.718802, 0.719000, 0.719000, 0.719000, 0.719000, 0.719000, 0.719000,
      0.719000, 0.719361, 0.719976, 0.720000, 0.720000, 0.720000, 0.720000,
      0.720000, 0.720000, 0.720643, 0.721000, 0.721000, 0.721000, 0.721000,
      0.721000, 0.721148, 0.721948, 0.722000, 0.722000, 0.722000, 0.722000,
      0.722000, 0.722077, 0.722988, 0.723000, 0.723000, 0.723000, 0.723000,
      0.723807, 0.724000, 0.724000, 0.724000, 0.724000, 0.724089, 0.725000,
      0.725000, 0.725000, 0.725000, 0.725884, 0.726000, 0.726000, 0.726000,
      0.726000, 0.727000, 0.727000, 0.727000, 0.727000, 0.727762, 0.728000,
      0.728000, 0.728000, 0.728708, 0.729000, 0.729000, 0.729465, 0.730000,
      0.730000, 0.730000, 0.731000, 0.731000, 0.731000, 0.731645, 0.732000,
      0.732000, 0.733000, 0.733000, 0.733000, 0.734000, 0.734000, 0.734000,
      0.735000, 0.735000, 0.736000, 0.736000, 0.736000, 0.737000, 0.737000,
      0.738000, 0.738000, 0.739000, 0.739000, 0.740000, 0.740000, 0.740978,
      0.741000, 0.742000, 0.742000, 0.743000, 0.743000, 0.744000, 0.744779,
      0.745000, 0.746000, 0.746000, 0.747000, 0.747459, 0.748379, 0.749113,
      0.750018, 0.750761, 0.751517, 0.752285, 0.753083, 0.754054, 0.754733,
      0.755508, 0.756351, 0.757209, 0.758070, 0.758814, 0.759571, 0.760684,
      0.762126, 0.762925, 0.764482, 0.765571, 0.766416, 0.767476, 0.768975,
      0.769880, 0.771357, 0.772947, 0.774565, NAN,      NAN,      2.316144,
      2.314768, 2.300420, 2.286130, 2.271974, 2.257341, 2.243225, 2.229310,
      2.215782, 2.202270, 2.188763, 2.175270, 2.161795, 2.148324, 2.134866,
      2.121609, 2.108732, 2.096171, 2.083503, 2.070651, 2.057315, 2.045007,
      2.032735, 2.020143, 2.007202, 1.994941, 1.983348, 1.971432, 1.959183,
      1.946940, 1.935686, 1.923442, 1.911545, 1.899959, 1.888287, 1.877156,
      1.866025, 1.853983, 1.842759, 1.831625, 1.820490, 1.809353, 1.798695,
      1.788078, 1.777276, 1.766668, 1.755342, 1.745140, 1.734783, 1.724247,
      1.714033, 1.703380, 1.693193, 1.682802, 1.672663, 1.663083, 1.652723,
      1.643071, 1.633169, 1.623447, 1.613983, 1.603956, 1.594351, 1.584841,
      1.575488, 1.566359, 1.557064, 1.548150, 1.539032, 1.529785, 1.521056,
      1.512389, 1.503747, 1.494873, 1.485494, 1.477093, 1.469053, 1.460760,
      1.452221, 1.443706, 1.435201, 1.427503, 1.419257, 1.411274, 1.403373,
      1.394998, 1.387646, 1.379818, 1.372054, 1.364540, 1.356364, 1.348973,
      1.341553, 1.334117, 1.326643, 1.319409, 1.312416, 1.305043, 1.298241,
      1.291252, 1.284152, 1.277509, 1.270325, 1.263275, 1.256595, 1.249880,
      1.243144, 1.237150, 1.230569, 1.223790, 1.217881, 1.211146, 1.204790,
      1.198737, 1.192484, 1.186045, 1.180143, 1.174212, 1.168262, 1.162638,
      1.157226, 1.151274, 1.145786, 1.139976, 1.134225, 1.128695, 1.123539,
      1.117890, 1.112634, 1.107542, 1.102429, 1.097280, 1.092101, 1.087513,
      1.082616, 1.077330, 1.073119, 1.068418, 1.063678, 1.058758, 1.053940,
      1.049756, 1.044864, 1.040382, 1.036117, 1.031823, 1.027582, 1.023939,
      1.019716, 1.015558, 1.011858, 1.008539, 1.004732, 1.000881, 0.996991,
      0.993452, 0.990104, 0.986103, 0.982843, 0.979570, 0.976698, 0.973924,
      0.970624, 0.967772, 0.965013, 0.962198, 0.958837, 0.957056, 0.954380,
      0.951902, 0.949384, 0.946727, 0.944843, 0.942935, 0.941013, 0.939068,
      0.937400, 0.936079, 0.934111, 0.932714, 0.931330, 0.929938, 0.928792,
      0.927296, 0.926378, 0.925589, 0.924779, 0.923946, 0.923089, 0.922408,
      0.921652, 0.920736, 0.919846, 0.919125, 0.917790, 0.916149, 0.914175,
      0.911986, 0.908205, 0.903894, 0.898655, 0.892277, 0.884230, 0.875998,
      0.865925, 0.855965, 0.846120, 0.836043, 0.827910, 0.820724, 0.814333,
      0.809562, 0.805515, 0.802599, 0.800016, 0.797896, 0.797359, 0.796846,
      0.796381, 0.796000, 0.796000, 0.796000, 0.796000, 0.796000, 0.796000,
      0.796273, 0.796584, 0.796896, 0.797000, 0.797000, 0.797000, 0.797012,
      0.797260, 0.797507, 0.797754, 0.798002, 0.798225, 0.798448, 0.798671,
      0.798893, 0.799000, 0.799000, 0.799000, 0.799000, 0.799000, 0.799110,
      0.799295, 0.799481, 0.799667, 0.799853, 0.800036, 0.800207, 0.800379,
      0.800551, 0.800723, 0.800894, 0.801000, 0.801000, 0.801000, 0.801000,
      0.801000, 0.801000, 0.801020, 0.801170, 0.801321, 0.801471, 0.801621,
      0.801771, 0.801921, 0.802068, 0.802210, 0.802352, 0.802494, 0.802636,
      0.802778, 0.802920, 0.803000, 0.803000, 0.803000, 0.803000, 0.803000,
      0.803000, 0.803000, 0.803005, 0.803135, 0.803265, 0.803394, 0.803524,
      0.803653, 0.803783, 0.803913, 0.804043, 0.804174, 0.804305, 0.804436,
      0.804568, 0.804699, 0.804830, 0.804962, 0.805106, 0.805256, 0.805406,
      0.805555, 0.805705, 0.805855, 0.806000, 0.806000, 0.806000, 0.806000,
      0.806000, 0.806000, 0.806016, 0.806203, 0.806390, 0.806576, 0.806763,
      0.806950, 0.807150, 0.807355, 0.807560, 0.807766, 0.807971, 0.808000,
      0.808000, 0.808000, 0.808000, 0.808094, 0.808336, 0.808578, 0.808820,
      0.809000, 0.809000, 0.809000, 0.809000, 0.809000, 0.809000, 0.809000,
      0.809000, 0.809249, 0.809546, 0.809844, 0.810000, 0.810000, 0.810000,
      0.810104, 0.810438, 0.810773, 0.811000, 0.811000, 0.811000, 0.811000,
      0.811000, 0.811000, 0.811310, 0.811699, 0.812000, 0.812000, 0.812000,
      0.812333, 0.812759, 0.813000, 0.813000, 0.813000, 0.813000, 0.813016,
      0.813498, 0.813980, 0.814000, 0.814000, 0.814499, 0.815000, 0.815000,
      0.815000, 0.815000, 0.815216, 0.815790, 0.816000, 0.816000, 0.816000,
      0.816197, 0.816827, 0.817000, 0.817000, 0.817000, 0.817468, 0.818000,
      0.818000, 0.818000, 0.818309, 0.819000, 0.819000, 0.819000, 0.819369,
      0.820000, 0.820000, 0.820000, 0.820663, 0.821000, 0.821000, 0.821297,
      0.822000, 0.822000, 0.822081, 0.823000, 0.823025, 0.824000, 0.824000,
      0.824080, 0.825000, 0.825000, 0.825310, 0.826000, 0.826000, 0.826722,
      0.827000, 0.827103, 0.828000, 0.828000, 0.828845, 0.829000, 0.829458,
      0.830000, 0.830168, 0.831000, 0.831980, 0.832000, 0.832899, 0.833000,
      0.833927, 0.834000, 0.835000, 0.835000, 0.836000, 0.837000, 0.837000,
      0.838000, 0.838222, 0.839025, 0.840000, 0.840735, 0.841000, 0.842000,
      0.843000, 0.843579, 0.844000, 0.845000, 0.846000, 0.847000, 0.848000,
      0.848485, 0.849778, 0.850114, 0.851494, 0.852000, 0.853387, 0.854000,
      0.855464, 0.856073, 0.857732, 0.859000, 0.860000, 0.861006, 0.862000,
      0.863784, 0.865000, 0.866779, 0.867860, 0.869000, 0.871000, 0.872453,
      0.873770, 0.875149, 0.877000, 0.879000, 0.880669, 0.882307, 0.884510,
      0.886682, 0.888658, 0.890333, 0.892727, 0.895169, 0.897654, 0.900184,
      0.902635, 0.904818, 0.907458, 0.910431, 0.913514, 0.916649, 0.919844,
      0.923109, 0.927039, 0.930812, 0.934616, 0.939114, 0.943719, 0.948405,
      0.953171, 0.958192, 0.963953, NAN,      2.421203, 2.419588, 2.404507,
      2.389688, 2.374885, 2.360100, 2.345329, 2.330568, 2.315829, 2.301098,
      2.287354, 2.272740, 2.258625, 2.244696, 2.230933, 2.216858, 2.203165,
      2.189680, 2.176212, 2.162749, 2.149415, 2.136541, 2.123442, 2.110026,
      2.096973, 2.084138, 2.071776, 2.058995, 2.046260, 2.034013, 2.021771,
      2.009534, 1.997302, 1.985076, 1.972853, 1.960638, 1.948449, 1.936799,
      1.925015, 1.913197, 1.901901, 1.890337, 1.878885, 1.867418, 1.856119,
      1.845110, 1.833860, 1.822397, 1.811160, 1.800439, 1.789888, 1.778812,
      1.768439, 1.757439, 1.746958, 1.736703, 1.726368, 1.715569, 1.705038,
      1.695079, 1.685261, 1.675457, 1.665674, 1.655913, 1.646156, 1.636422,
      1.626713, 1.617025, 1.607754, 1.598700, 1.589353, 1.580282, 1.570894,
      1.562083, 1.553188, 1.544159, 1.535552, 1.526978, 1.518356, 1.509728,
      1.501048, 1.492331, 1.484021, 1.475814, 1.467615, 1.459874, 1.452100,
      1.443331, 1.435434, 1.427568, 1.419819, 1.412437, 1.404726, 1.397464,
      1.390094, 1.382621, 1.375532, 1.367937, 1.360602, 1.353518, 1.346417,
      1.339330, 1.332737, 1.325863, 1.318962, 1.312281, 1.306051, 1.299279,
      1.292802, 1.286130, 1.279325, 1.273221, 1.267521, 1.261311, 1.254908,
      1.248701, 1.242858, 1.237473, 1.231539, 1.225570, 1.219617, 1.214230,
      1.208813, 1.203345, 1.197845, 1.192307, 1.187714, 1.182083, 1.176973,
      1.171965, 1.166934, 1.162318, 1.157184, 1.152705, 1.148218, 1.143693,
      1.139129, 1.134507, 1.130370, 1.126448, 1.122490, 1.118488, 1.114439,
      1.110348, 1.106571, 1.103229, 1.099851, 1.096429, 1.092967, 1.089466,
      1.086265, 1.083288, 1.080539, 1.077785, 1.075664, 1.073370, 1.070818,
      1.067947, 1.065985, 1.064007, 1.062330, 1.060965, 1.058934, 1.057536,
      1.056344, 1.055510, 1.054778, 1.054024, 1.054000, 1.054000, 1.054000,
      1.054000, 1.054000, 1.054000, 1.054692, 1.055511, 1.056276, 1.057066,
      1.057883, 1.058731, 1.059000, 1.059000, 1.058866, 1.058143, 1.056785,
      1.054203, 1.049916, 1.044711, 1.036396, 1.026045, 1.013813, 1.000378,
      0.986521, 0.971658, 0.958232, 0.945823, 0.934507, 0.924874, 0.916486,
      0.910041, 0.904839, 0.901597, 0.898869, 0.896640, 0.895314, 0.894351,
      0.894000, 0.894000, 0.894000, 0.894000, 0.894000, 0.894162, 0.894483,
      0.894804, 0.895111, 0.895396, 0.895681, 0.895965, 0.896223, 0.896477,
      0.896731, 0.896985, 0.897430, 0.897888, 0.898345, 0.898802, 0.899118,
      0.899325, 0.899532, 0.899740, 0.899947, 0.900000, 0.900000, 0.900000,
      0.900000, 0.900000, 0.900083, 0.900258, 0.900433, 0.900608, 0.900783,
      0.900958, 0.901123, 0.901286, 0.901449, 0.901611, 0.901774, 0.901937,
      0.902093, 0.902246, 0.902398, 0.902551, 0.902703, 0.902855, 0.903007,
      0.903151, 0.903295, 0.903439, 0.903583, 0.903727, 0.903871, 0.904014,
      0.904151, 0.904288, 0.904425, 0.904561, 0.904698, 0.904835, 0.904972,
      0.905104, 0.905235, 0.905366, 0.905497, 0.905627, 0.905758, 0.905889,
      0.906019, 0.906145, 0.906271, 0.906397, 0.906523, 0.906649, 0.906775,
      0.906901, 0.907000, 0.907000, 0.907000, 0.907000, 0.907000, 0.907000,
      0.907000, 0.907000, 0.907001, 0.907135, 0.907269, 0.907404, 0.907538,
      0.907672, 0.907806, 0.907941, 0.908085, 0.908238, 0.908391, 0.908544,
      0.908697, 0.908849, 0.909000, 0.909000, 0.909000, 0.909000, 0.909000,
      0.909000, 0.909033, 0.909223, 0.909413, 0.909602, 0.909792, 0.909982,
      0.910188, 0.910397, 0.910605, 0.910813, 0.911000, 0.911000, 0.911000,
      0.911000, 0.911000, 0.911169, 0.911414, 0.911660, 0.911905, 0.912161,
      0.912425, 0.912688, 0.912952, 0.913000, 0.913000, 0.913000, 0.913082,
      0.913382, 0.913683, 0.913984, 0.914302, 0.914621, 0.914940, 0.915000,
      0.915000, 0.915000, 0.915302, 0.915657, 0.916014, 0.916388, 0.916763,
      0.917000, 0.917000, 0.917000, 0.917338, 0.917749, 0.918168, 0.918597,
      0.919000, 0.919000, 0.919000, 0.919388, 0.919855, 0.920000, 0.920000,
      0.920316, 0.920820, 0.921335, 0.921857, 0.922000, 0.922000, 0.922490,
      0.923000, 0.923000, 0.923212, 0.923808, 0.924417, 0.925000, 0.925000,
      0.925306, 0.925957, 0.926000, 0.926304, 0.926992, 0.927699, 0.928000,
      0.928145, 0.928888, 0.929647, 0.930000, 0.930205, 0.931000, 0.931000,
      0.931653, 0.932499, 0.933000, 0.933239, 0.934133, 0.935000, 0.935000,
      0.935918, 0.936882, 0.937000, 0.937862, 0.938000, 0.938917, 0.939973,
      0.940049, 0.941146, 0.942000, 0.942400, 0.943559, 0.944000, 0.944939,
      0.946164, 0.947000, 0.947683, 0.948975, 0.949295, 0.950638, 0.951004,
      0.952399, 0.953817, 0.954264, 0.955736, 0.957000, 0.957764, 0.959000,
      0.959905, 0.961521, 0.962166, 0.963841, 0.965549, 0.966287, 0.968058,
      0.969862, 0.970700, 0.972572, 0.974000, 0.975422, 0.977400, 0.979000,
      0.980469, 0.982560, 0.984690, 0.986859, 0.988069, 0.990321, 0.992614,
      0.994950, 0.997330, 0.999754, 1.002223, 1.004738, 1.007300, 1.009909,
      1.012568, 1.015276, 1.018067, 1.021844, 1.024706, 1.028621, 1.031591,
      1.035616, 1.039697, 1.043835, 1.048032, 1.052289, 1.057214, 1.061971,
      1.066856, 1.072868, 1.078012, 1.084144, 1.089699, 1.096250, 1.102941,
      1.109703, 1.116889, 1.124214, 1.132188, 1.140620, 1.149025, 1.157037,
      1.165869, 1.175547, 1.186617, 1.199294, 1.212879, NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      2.523568, 2.521942, 2.505867, 2.489798, 2.473906,
      2.458447, 2.443294, 2.427699, 2.412594, 2.397758, 2.382952, 2.368154,
      2.353366, 2.338598, 2.323850, 2.309110, 2.295359, 2.280688, 2.266223,
      2.252500, 2.238407, 2.224629, 2.211127, 2.197644, 2.184165, 2.170711,
      2.157273, 2.143842, 2.130431, 2.117172, 2.104048, 2.091994, 2.079225,
      2.066405, 2.053526, 2.040644, 2.028670, 2.015856, 2.003697, 1.991622,
      1.979617, 1.967969, 1.955948, 1.943687, 1.931681, 1.920024, 1.908654,
      1.897302, 1.885747, 1.874529, 1.863325, 1.851952, 1.841351, 1.830659,
      1.819893, 1.809139, 1.798402, 1.787687, 1.776997, 1.766312, 1.756609,
      1.745979, 1.736265, 1.726266, 1.715765, 1.705949, 1.696102, 1.686223,
      1.676661, 1.667263, 1.658302, 1.648373, 1.639395, 1.630079, 1.620361,
      1.611567, 1.603022, 1.594002, 1.585301, 1.576559, 1.567439, 1.558694,
      1.550259, 1.542014, 1.533755, 1.525439, 1.517082, 1.509312, 1.501663,
      1.493789, 1.485384, 1.477424, 1.469785, 1.462352, 1.454898, 1.447384,
      1.439837, 1.433209, 1.425601, 1.418434, 1.411999, 1.404521, 1.397992,
      1.391437, 1.384822, 1.378179, 1.371487, 1.364778, 1.358661, 1.352508,
      1.346319, 1.340071, 1.334257, 1.327941, 1.321934, 1.316215, 1.310440,
      1.304671, 1.299529, 1.294345, 1.289120, 1.283852, 1.278523, 1.273696,
      1.269047, 1.264356, 1.259612, 1.254815, 1.250475, 1.246077, 1.241278,
      1.237112, 1.233712, 1.229622, 1.225844, 1.222348, 1.218805, 1.215221,
      1.212195, 1.208913, 1.205785, 1.203272, 1.200433, 1.198346, 1.195829,
      1.193380, 1.191568, 1.190200, 1.189109, 1.187732, 1.186651, 1.185926,
      1.185188, 1.185000, 1.185000, 1.185000, 1.185246, 1.185893, 1.187517,
      1.188694, 1.190510, 1.192783, 1.195096, 1.196753, 1.199392, 1.202192,
      1.205094, 1.208234, 1.211552, 1.215188, 1.218437, 1.221433, 1.224297,
      1.225783, 1.225351, 1.222202, 1.215572, 1.205732, 1.190850, 1.173352,
      1.153530, 1.132164, 1.110330, 1.088617, 1.069368, 1.051856, 1.036389,
      1.024010, 1.015774, 1.009096, 1.005369, 1.002756, 1.000774, 1.000086,
      1.000517, 1.001093, 1.001603, 1.002194, 1.003076, 1.003959, 1.004733,
      1.005502, 1.006119, 1.006457, 1.006794, 1.007232, 1.007828, 1.008424,
      1.009018, 1.009549, 1.010079, 1.010610, 1.011126, 1.011602, 1.012078,
      1.012554, 1.013013, 1.013229, 1.013444, 1.013659, 1.013874, 1.014081,
      1.014278, 1.014474, 1.014670, 1.014866, 1.015115, 1.015476, 1.015837,
      1.016197, 1.016558, 1.016919, 1.017130, 1.017297, 1.017464, 1.017631,
      1.017798, 1.017966, 1.018124, 1.018280, 1.018437, 1.018593, 1.018749,
      1.018905, 1.019058, 1.019205, 1.019352, 1.019499, 1.019646, 1.019793,
      1.019940, 1.020000, 1.020000, 1.020000, 1.020000, 1.020000, 1.020000,
      1.020000, 1.020056, 1.020189, 1.020322, 1.020455, 1.020588, 1.020721,
      1.020854, 1.020987, 1.021116, 1.021243, 1.021371, 1.021499, 1.021627,
      1.021754, 1.021882, 1.022010, 1.022133, 1.022256, 1.022380, 1.022503,
      1.022626, 1.022750, 1.022873, 1.022996, 1.023116, 1.023236, 1.023355,
      1.023475, 1.023595, 1.023714, 1.023834, 1.023954, 1.024079, 1.024207,
      1.024336, 1.024464, 1.024592, 1.024721, 1.024849, 1.024978, 1.025121,
      1.025268, 1.025415, 1.025562, 1.025709, 1.025856, 1.026003, 1.026168,
      1.026334, 1.026499, 1.026664, 1.026830, 1.026995, 1.027357, 1.027725,
      1.028092, 1.028460, 1.028828, 1.029108, 1.029310, 1.029512, 1.029714,
      1.029917, 1.030130, 1.030351, 1.030572, 1.030792, 1.031014, 1.031253,
      1.031493, 1.031732, 1.031971, 1.032227, 1.032484, 1.032742, 1.033000,
      1.033276, 1.033552, 1.033828, 1.034112, 1.034406, 1.034701, 1.034996,
      1.035617, 1.036243, 1.036869, 1.037262, 1.037594, 1.037925, 1.038271,
      1.038621, 1.038971, 1.039338, 1.039707, 1.040079, 1.040466, 1.040853,
      1.041252, 1.041657, 1.042065, 1.042489, 1.042913, 1.043351, 1.043794,
      1.044492, 1.045414, 1.046174, 1.046654, 1.047138, 1.047636, 1.048138,
      1.048655, 1.049177, 1.049711, 1.050255, 1.050808, 1.051373, 1.051944,
      1.053065, 1.054127, 1.054735, 1.055354, 1.055981, 1.056626, 1.057279,
      1.057943, 1.058623, 1.059628, 1.061015, 1.061734, 1.062465, 1.063208,
      1.063964, 1.064738, 1.066049, 1.067325, 1.068140, 1.068970, 1.069817,
      1.071361, 1.072559, 1.073454, 1.074729, 1.076293, 1.077238, 1.078201,
      1.079364, 1.081181, 1.082199, 1.083473, 1.085293, 1.086370, 1.087931,
      1.089582, 1.090719, 1.092754, 1.094113, 1.096259, 1.097484, 1.099462,
      1.101000, 1.103295, 1.105228, 1.106956, 1.109324, 1.111435, 1.113272,
      1.115583, 1.118054, 1.120111, 1.123082, 1.125279, 1.127450, 1.130679,
      1.132971, 1.136160, 1.138734, 1.142213, 1.144755, 1.148362, 1.151037,
      1.154781, 1.158596, 1.162481, 1.166438, 1.170469, 1.174574, 1.178756,
      1.183016, 1.187356, 1.191778, 1.197281, 1.201866, 1.207539, 1.212299,
      1.218218, 1.224627, 1.230170, 1.236858, 1.243458, 1.249773, 1.256785,
      1.263706, 1.269488, 1.276073, 1.283819, 1.290728, 1.298991, 1.307745,
      1.317492, 1.326290, 1.335532, 1.347412, 1.358293, 1.371237, 1.386395,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      2.622552, 2.620855, 2.603802, 2.587171, 2.570999, 2.554851, 2.538730,
      2.522614, 2.506518, 2.490751, 2.475261, 2.459336, 2.443845, 2.428506,
      2.413671, 2.398851, 2.384047, 2.368517, 2.353956, 2.339769, 2.325056,
      2.310366, 2.296097, 2.282041, 2.267867, 2.254018, 2.239695, 2.226084,
      2.212250, 2.198468, 2.184847, 2.171695, 2.158421, 2.145352, 2.132241,
      2.119025, 2.106112, 2.093309, 2.080936, 2.068337, 2.055650, 2.043210,
      2.031142, 2.018902, 2.006968, 1.995016, 1.982833, 1.971138, 1.959867,
      1.948733, 1.937328, 1.925494, 1.914317, 1.903309, 1.892290, 1.881298,
      1.870722, 1.859921, 1.849239, 1.838770, 1.828041, 1.817725, 1.807453,
      1.797167, 1.786822, 1.776679, 1.766827, 1.756519, 1.746978, 1.737588,
      1.728133, 1.718637, 1.709105, 1.699506, 1.690026, 1.681374, 1.672252,
      1.663487, 1.654369, 1.645249, 1.636554, 1.627827, 1.620048, 1.611215,
      1.602603, 1.594320, 1.586524, 1.578772, 1.570992, 1.563142, 1.555262,
      1.547323, 1.539343, 1.531933, 1.524831, 1.518052, 1.510209, 1.503327,
      1.496388, 1.489408, 1.482383, 1.476291, 1.469151, 1.462552, 1.456353,
      1.450447, 1.443803, 1.437657, 1.431648, 1.426138, 1.420790, 1.415309,
      1.409076, 1.404084, 1.398870, 1.393505, 1.388645, 1.383726, 1.378752,
      1.374270, 1.370035, 1.365830, 1.361778, 1.357521, 1.353460, 1.349859,
      1.346212, 1.342595, 1.339466, 1.336476, 1.333739, 1.331142, 1.329124,
      1.327104, 1.325065, 1.323324, 1.321942, 1.320914, 1.320187, 1.319449,
      1.319000, 1.319063, 1.319855, 1.320677, 1.322057, 1.323828, 1.325765,
      1.327922, 1.330939, 1.334634, 1.338775, 1.343118, 1.347761, 1.353162,
      1.359615, 1.366823, 1.374826, 1.383921, 1.393247, 1.402619, 1.412259,
      1.420711, 1.426711, 1.430380, 1.428870, 1.421514, 1.409674, 1.393344,
      1.370551, 1.343082, 1.312285, 1.279832, 1.246934, 1.215895, 1.189020,
      1.166475, 1.147549, 1.135566, 1.127134, 1.124149, 1.121246, 1.121698,
      1.123183, 1.125160, 1.127403, 1.129557, 1.131493, 1.133373, 1.135054,
      1.136735, 1.137619, 1.138353, 1.139115, 1.140084, 1.141052, 1.142006,
      1.142292, 1.142578, 1.142864, 1.143267, 1.143778, 1.144288, 1.144798,
      1.145278, 1.145737, 1.146196, 1.146655, 1.147103, 1.147520, 1.147936,
      1.148352, 1.148769, 1.149085, 1.149275, 1.149465, 1.149656, 1.149846,
      1.150034, 1.150209, 1.150385, 1.150560, 1.150736, 1.150911, 1.151000,
      1.151000, 1.151000, 1.151000, 1.151000, 1.151000, 1.151056, 1.151208,
      1.151361, 1.151514, 1.151667, 1.151820, 1.151973, 1.152000, 1.152000,
      1.152000, 1.152000, 1.152000, 1.152000, 1.152000, 1.152000, 1.152000,
      1.152000, 1.152000, 1.152000, 1.152000, 1.152000, 1.152078, 1.152209,
      1.152340, 1.152471, 1.152602, 1.152733, 1.152864, 1.152995, 1.153122,
      1.153248, 1.153374, 1.153500, 1.153626, 1.153752, 1.153878, 1.154004,
      1.154126, 1.154248, 1.154370, 1.154492, 1.154614, 1.154736, 1.154858,
      1.154980, 1.155198, 1.155435, 1.155672, 1.155910, 1.156147, 1.156384,
      1.156621, 1.156858, 1.157046, 1.157162, 1.157278, 1.157393, 1.157509,
      1.157624, 1.157740, 1.157856, 1.157971, 1.158196, 1.158457, 1.158717,
      1.158978, 1.159239, 1.159499, 1.159760, 1.160024, 1.160321, 1.160619,
      1.160916, 1.161214, 1.161512, 1.161809, 1.162120, 1.162455, 1.162789,
      1.163124, 1.163458, 1.163793, 1.164071, 1.164257, 1.164442, 1.164628,
      1.164814, 1.165000, 1.165408, 1.165816, 1.166224, 1.166633, 1.167045,
      1.167491, 1.167936, 1.168381, 1.168827, 1.169295, 1.169777, 1.170259,
      1.170742, 1.171121, 1.171380, 1.171640, 1.171899, 1.172341, 1.172897,
      1.173453, 1.174010, 1.174603, 1.175196, 1.175789, 1.176406, 1.177036,
      1.177666, 1.178314, 1.178981, 1.179648, 1.180332, 1.181036, 1.181740,
      1.182234, 1.182604, 1.182974, 1.183724, 1.184502, 1.185293, 1.186108,
      1.186922, 1.187770, 1.188622, 1.189494, 1.190382, 1.191282, 1.192208,
      1.193138, 1.194101, 1.195065, 1.196065, 1.197033, 1.197551, 1.198144,
      1.199217, 1.200300, 1.201410, 1.202537, 1.203684, 1.204858, 1.206043,
      1.207264, 1.208500, 1.209757, 1.211045, 1.212349, 1.213680, 1.215040,
      1.216419, 1.217825, 1.219263, 1.220723, 1.222207, 1.223723, 1.225269,
      1.227262, 1.229442, 1.231070, 1.232734, 1.234429, 1.236154, 1.237910,
      1.239699, 1.241782, 1.244377, 1.246268, 1.248193, 1.250233, 1.253154,
      1.255191, 1.257398, 1.260378, 1.262531, 1.265086, 1.267957, 1.270231,
      1.273320, 1.275905, 1.279311, 1.282141, 1.285883, 1.288794, 1.292383,
      1.295532, 1.299557, 1.302661, 1.306843, 1.311096, 1.315437, 1.319852,
      1.324355, 1.328935, 1.333609, 1.338363, 1.343209, 1.348144, 1.353169,
      1.359055, 1.364508, 1.369822, 1.376232, 1.382660, 1.388363, 1.395790,
      1.405124, 1.411080, 1.418142, 1.426354, 1.435388, 1.442910, 1.452743,
      1.464426, 1.473947, 1.484531, 1.496508, 1.508963, 1.521133, 1.534866,
      1.550057, 1.568000, NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      2.718753, 2.716985,
      2.699491, 2.682465, 2.665585, 2.648262, 2.631457, 2.614921, 2.598415,
      2.581673, 2.565450, 2.549356, 2.533289, 2.517234, 2.501497, 2.486077,
      2.470266, 2.454837, 2.439554, 2.424573, 2.409076, 2.394275, 2.379369,
      2.364767, 2.350601, 2.336003, 2.321618, 2.307430, 2.293006, 2.278689,
      2.264947, 2.251406, 2.237655, 2.223898, 2.210439, 2.197215, 2.184088,
      2.171010, 2.157435, 2.144597, 2.132202, 2.119192, 2.106339, 2.094004,
      2.081761, 2.069519, 2.057707, 2.046068, 2.034208, 2.022342, 2.010924,
      1.999456, 1.987918, 1.976363, 1.964949, 1.953864, 1.943121, 1.932530,
      1.921311, 1.910197, 1.899701, 1.889513, 1.878827, 1.868474, 1.858313,
      1.848520, 1.838693, 1.828798, 1.818875, 1.809133, 1.799723, 1.789762,
      1.780232, 1.770962, 1.761970, 1.752951, 1.743862, 1.735489, 1.726535,
      1.717490, 1.708854, 1.700693, 1.692604, 1.684470, 1.676296, 1.668042,
      1.659747, 1.652010, 1.644636, 1.637011, 1.629435, 1.622267, 1.615024,
      1.607732, 1.600393, 1.593376, 1.586658, 1.579878, 1.573692, 1.567554,
      1.561365, 1.555121, 1.548801, 1.542995, 1.537361, 1.531672, 1.525937,
      1.520948, 1.515905, 1.510807, 1.506044, 1.501607, 1.496912, 1.492876,
      1.488836, 1.484615, 1.481091, 1.477962, 1.475015, 1.472119, 1.469377,
      1.466802, 1.464782, 1.462727, 1.461093, 1.459763, 1.458788, 1.458058,
      1.458000, 1.458000, 1.458439, 1.460032, 1.461703, 1.463441, 1.465880,
      1.469162, 1.472850, 1.476710, 1.482330, 1.488341, 1.495265, 1.503366,
      1.513225, 1.523921, 1.536373, 1.549500, 1.564037, 1.579994, 1.595409,
      1.610781, 1.624645, 1.636027, 1.644489, 1.647668, 1.645155, 1.636672,
      1.621849, 1.600594, 1.570956, 1.532655, 1.489523, 1.443670, 1.397608,
      1.354958, 1.319753, 1.292574, 1.271180, 1.258940, 1.255313, 1.253375,
      1.255342, 1.258453, 1.262288, 1.266097, 1.269524, 1.272402, 1.274866,
      1.276923, 1.278633, 1.280150, 1.280896, 1.281642, 1.282512, 1.283496,
      1.284480, 1.285273, 1.285854, 1.286434, 1.287007, 1.287278, 1.287550,
      1.287822, 1.288094, 1.288366, 1.288638, 1.288910, 1.289000, 1.289000,
      1.289000, 1.289000, 1.288515, 1.288028, 1.287541, 1.287054, 1.286609,
      1.286169, 1.285729, 1.285290, 1.284932, 1.284732, 1.284531, 1.284331,
      1.284131, 1.283937, 1.283753, 1.283569, 1.283386, 1.283202, 1.283019,
      1.282847, 1.282677, 1.282507, 1.282337, 1.282168, 1.282002, 1.282161,
      1.282319, 1.282478, 1.282636, 1.282795, 1.282953, 1.283000, 1.283000,
      1.283000, 1.283000, 1.283000, 1.283000, 1.283000, 1.283000, 1.283000,
      1.283000, 1.283000, 1.283000, 1.283000, 1.283000, 1.283121, 1.283255,
      1.283389, 1.283524, 1.283658, 1.283792, 1.283927, 1.284059, 1.284188,
      1.284316, 1.284445, 1.284574, 1.284703, 1.284832, 1.284961, 1.285086,
      1.285211, 1.285335, 1.285459, 1.285583, 1.285708, 1.285832, 1.285956,
      1.286078, 1.286198, 1.286319, 1.286439, 1.286560, 1.286680, 1.286800,
      1.286921, 1.287080, 1.287315, 1.287549, 1.287784, 1.288018, 1.288252,
      1.288487, 1.288721, 1.288956, 1.289191, 1.289426, 1.289660, 1.289895,
      1.290130, 1.290365, 1.290600, 1.290835, 1.291122, 1.291529, 1.291937,
      1.292345, 1.292753, 1.293161, 1.293569, 1.293977, 1.294437, 1.294900,
      1.295363, 1.295827, 1.296290, 1.296753, 1.297242, 1.297761, 1.298280,
      1.298799, 1.299317, 1.299836, 1.300262, 1.300645, 1.301027, 1.301410,
      1.301793, 1.302289, 1.302919, 1.303548, 1.304178, 1.304807, 1.305475,
      1.306160, 1.306845, 1.307530, 1.308155, 1.308649, 1.309142, 1.309636,
      1.310208, 1.311004, 1.311800, 1.312596, 1.313419, 1.314270, 1.315121,
      1.315972, 1.316877, 1.317784, 1.318690, 1.319633, 1.320595, 1.321557,
      1.322366, 1.323044, 1.323722, 1.324634, 1.325706, 1.326779, 1.327896,
      1.329024, 1.330159, 1.331343, 1.332526, 1.333743, 1.334982, 1.336231,
      1.337525, 1.338819, 1.340161, 1.341511, 1.342896, 1.344301, 1.345733,
      1.347194, 1.348679, 1.350195, 1.351737, 1.353308, 1.354910, 1.356536,
      1.358202, 1.359884, 1.361618, 1.363367, 1.365159, 1.366981, 1.368829,
      1.371304, 1.373650, 1.375608, 1.377611, 1.379643, 1.381712, 1.383829,
      1.386305, 1.389164, 1.391399, 1.393675, 1.396320, 1.399344, 1.401746,
      1.404929, 1.407692, 1.410641, 1.413816, 1.416598, 1.420130, 1.422864,
      1.426534, 1.430318, 1.433378, 1.437096, 1.441094, 1.445166, 1.449313,
      1.453537, 1.457838, 1.463223, 1.467866, 1.473244, 1.478348, 1.484247,
      1.489402, 1.495373, 1.501615, 1.507967, 1.514432, 1.521027, 1.527743,
      1.535491, 1.542856, 1.550643, 1.560251, 1.569065, 1.577883, 1.587138,
      1.599604, 1.610882, 1.620421, 1.632362, 1.644186, 1.657804, 1.670936,
      1.685528, 1.699883, 1.714641, 1.730167, 1.745505, 1.762941, 1.784000,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      2.813769, 2.811957, 2.793788, 2.775843,
      2.758041, 2.739964, 2.723286, 2.705973, 2.688580, 2.671432, 2.654704,
      2.637666, 2.621277, 2.604757, 2.588233, 2.572241, 2.556156, 2.540370,
      2.524835, 2.509245, 2.493569, 2.477775, 2.462619, 2.447204, 2.432042,
      2.417169, 2.402468, 2.387999, 2.373517, 2.359004, 2.344776, 2.331043,
      2.316772, 2.302554, 2.288981, 2.275466, 2.261717, 2.248593, 2.235365,
      2.222032, 2.209277, 2.196572, 2.183797, 2.170992, 2.158414, 2.146077,
      2.134169, 2.121695, 2.109373, 2.097387, 2.085364, 2.074261, 2.062729,
      2.051361, 2.040007, 2.028322, 2.017167, 2.006586, 1.995676, 1.984777,
      1.974052, 1.963241, 1.952722, 1.942430, 1.932344, 1.922700, 1.912251,
      1.902300, 1.892634, 1.883311, 1.873954, 1.864521, 1.855049, 1.846101,
      1.837185, 1.828219, 1.819671, 1.810572, 1.801978, 1.793749, 1.785874,
      1.777205, 1.769409, 1.761375, 1.753677, 1.746233, 1.738724, 1.731139,
      1.723944, 1.717008, 1.710015, 1.703723, 1.696800, 1.689974, 1.683696,
      1.677986, 1.672203, 1.666362, 1.660460, 1.654852, 1.649902, 1.645452,
      1.640158, 1.635936, 1.631633, 1.627241, 1.623526, 1.619799, 1.616604,
      1.613541, 1.610742, 1.608087, 1.606036, 1.603967, 1.602438, 1.601580,
      1.600860, 1.600125, 1.600625, 1.601390, 1.602346, 1.603969, 1.606515,
      1.609900, 1.613950, 1.618426, 1.623830, 1.630258, 1.638804, 1.648257,
      1.658728, 1.670797, 1.685164, 1.701286, 1.718944, 1.738274, 1.758845,
      1.780690, 1.801859, 1.820815, 1.837490, 1.851264, 1.862089, 1.869100,
      1.869312, 1.863466, 1.849900, 1.827733, 1.794534, 1.749047, 1.696324,
      1.638664, 1.578272, 1.521690, 1.472841, 1.435456, 1.407151, 1.392305,
      1.387469, 1.388413, 1.391880, 1.397223, 1.402843, 1.407512, 1.411590,
      1.415295, 1.417272, 1.419171, 1.420528, 1.421885, 1.423431, 1.424994,
      1.426286, 1.427711, 1.429368, 1.431193, 1.432000, 1.432000, 1.431364,
      1.429506, 1.425902, 1.421107, 1.415585, 1.410144, 1.405214, 1.401708,
      1.399153, 1.396914, 1.395601, 1.394389, 1.393261, 1.392626, 1.392139,
      1.391698, 1.391275, 1.390871, 1.390502, 1.390133, 1.390000, 1.390000,
      1.390000, 1.389840, 1.389552, 1.389265, 1.389000, 1.389000, 1.389000,
      1.389000, 1.389000, 1.389000, 1.389000, 1.389000, 1.389000, 1.389000,
      1.389000, 1.389000, 1.389000, 1.389000, 1.389000, 1.389000, 1.389000,
      1.389000, 1.389000, 1.389144, 1.389320, 1.389496, 1.389672, 1.389848,
      1.390000, 1.390000, 1.390000, 1.390000, 1.390000, 1.390000, 1.390004,
      1.390157, 1.390310, 1.390463, 1.390616, 1.390770, 1.390923, 1.391072,
      1.391216, 1.391361, 1.391505, 1.391650, 1.391795, 1.391939, 1.392079,
      1.392217, 1.392354, 1.392491, 1.392629, 1.392766, 1.392903, 1.393039,
      1.393170, 1.393302, 1.393433, 1.393564, 1.393696, 1.393827, 1.393958,
      1.394172, 1.394425, 1.394678, 1.394930, 1.395183, 1.395435, 1.395688,
      1.395941, 1.396280, 1.396647, 1.397013, 1.397380, 1.397746, 1.398112,
      1.398479, 1.398845, 1.399206, 1.399562, 1.399918, 1.400273, 1.400629,
      1.400985, 1.401341, 1.401697, 1.402055, 1.402424, 1.402793, 1.403162,
      1.403530, 1.403899, 1.404268, 1.404637, 1.405009, 1.405574, 1.406140,
      1.406706, 1.407271, 1.407837, 1.408403, 1.408968, 1.409604, 1.410244,
      1.410883, 1.411523, 1.412162, 1.412802, 1.413493, 1.414206, 1.414920,
      1.415633, 1.416347, 1.417083, 1.418067, 1.419052, 1.420036, 1.421020,
      1.422004, 1.422865, 1.423726, 1.424588, 1.425449, 1.426337, 1.427272,
      1.428207, 1.429142, 1.430105, 1.431366, 1.432627, 1.433888, 1.435128,
      1.436211, 1.437294, 1.438377, 1.439614, 1.441060, 1.442506, 1.443952,
      1.445190, 1.446420, 1.447651, 1.449168, 1.450799, 1.452429, 1.453896,
      1.455274, 1.456653, 1.458358, 1.460173, 1.461988, 1.463895, 1.465802,
      1.467595, 1.469195, 1.470795, 1.472824, 1.474916, 1.477053, 1.479237,
      1.481439, 1.483716, 1.485993, 1.488361, 1.490730, 1.493181, 1.495643,
      1.498183, 1.500736, 1.503373, 1.506019, 1.508757, 1.511512, 1.514343,
      1.517212, 1.520134, 1.523546, 1.527141, 1.530249, 1.533397, 1.536597,
      1.540452, 1.544441, 1.548502, 1.552050, 1.555881, 1.560169, 1.564562,
      1.569030, 1.573571, 1.578190, 1.582916, 1.587726, 1.593059, 1.598605,
      1.603790, 1.609819, 1.615121, 1.621241, 1.628284, 1.635123, 1.642276,
      1.649981, 1.658309, 1.667304, 1.676467, 1.685799, 1.695894, 1.705986,
      1.718119, 1.728737, 1.740084, 1.750076, 1.763305, 1.776931, 1.791586,
      1.806672, 1.822111, 1.838574, 1.854262, 1.871006, 1.888341, 1.904560,
      1.921961, 1.940015, 1.958074, 1.976922, 1.998000, NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      2.823270, 2.821455, 2.803218, 2.785181, 2.767286, 2.749134,
      2.732469, 2.715078, 2.697597, 2.680408, 2.663630, 2.646497, 2.630076,
      2.613509, 2.596906, 2.580857, 2.564745, 2.548924, 2.533363, 2.517713,
      2.502018, 2.486125, 2.470944, 2.455447, 2.440187, 2.425285, 2.410553,
      2.396056, 2.381568, 2.367036, 2.352759, 2.339007, 2.324683, 2.310420,
      2.296835, 2.283291, 2.269480, 2.256351, 2.243158, 2.229776, 2.216985,
      2.204310, 2.191543, 2.178690, 2.166079, 2.153733, 2.141815, 2.129258,
      2.116889, 2.104891, 2.092808, 2.081742, 2.070210, 2.058860, 2.047513,
      2.035768, 2.024572, 2.013991, 2.003112, 1.992235, 1.981487, 1.970614,
      1.960112, 1.949825, 1.939747, 1.930118, 1.919607, 1.909650, 1.900010,
      1.890729, 1.881377, 1.871997, 1.862531, 1.853615, 1.844706, 1.835746,
      1.827252, 1.818080, 1.809523, 1.801375, 1.793576, 1.784856, 1.777090,
      1.769065, 1.761415, 1.754052, 1.746622, 1.739052, 1.731875, 1.725008,
      1.718073, 1.711869, 1.704978, 1.698198, 1.692027, 1.686447, 1.680758,
      1.675011, 1.669137, 1.663582, 1.658755, 1.654485, 1.649294, 1.645230,
      1.641060, 1.636798, 1.633285, 1.629684, 1.626674, 1.623815, 1.621211,
      1.618735, 1.616949, 1.615077, 1.613798, 1.613277, 1.612836, 1.612341,
      1.613186, 1.614317, 1.615643, 1.617686, 1.620688, 1.624617, 1.629236,
      1.634292, 1.640334, 1.647478, 1.656884, 1.667283, 1.678757, 1.691873,
      1.707511, 1.725071, 1.744251, 1.765185, 1.787445, 1.811088, 1.833812,
      1.854062, 1.871712, 1.886054, 1.896975, 1.903617, 1.902606, 1.894862,
      1.878486, 1.852507, 1.814446, 1.762874, 1.703492, 1.638928, 1.571650,
      1.509092, 1.455609, 1.415334, 1.385681, 1.371476, 1.369121, 1.373989,
      1.382116, 1.392579, 1.403366, 1.412767, 1.420773, 1.427567, 1.431881,
      1.435194, 1.437049, 1.438736, 1.440240, 1.441648, 1.442686, 1.443872,
      1.445353, 1.447073, 1.447713, 1.447508, 1.446638, 1.444441, 1.440403,
      1.435053, 1.428892, 1.422809, 1.417287, 1.413351, 1.410482, 1.407962,
      1.406461, 1.405100, 1.403832, 1.403106, 1.402543, 1.402031, 1.401539,
      1.401067, 1.400652, 1.400246, 1.400100, 1.400100, 1.400148, 1.400021,
      1.399753, 1.399486, 1.399239, 1.399283, 1.399327, 1.399371, 1.399407,
      1.399427, 1.399447, 1.399467, 1.399487, 1.399506, 1.399525, 1.399543,
      1.399561, 1.399580, 1.399598, 1.399615, 1.399632, 1.399649, 1.399666,
      1.399841, 1.400052, 1.400229, 1.400407, 1.400585, 1.400736, 1.400721,
      1.400705, 1.400700, 1.400700, 1.400700, 1.400704, 1.400873, 1.401041,
      1.401210, 1.401378, 1.401547, 1.401715, 1.401879, 1.402038, 1.402197,
      1.402356, 1.402503, 1.402649, 1.402794, 1.402935, 1.403073, 1.403210,
      1.403348, 1.403486, 1.403624, 1.403762, 1.403898, 1.404030, 1.404162,
      1.404293, 1.404425, 1.404557, 1.404689, 1.404821, 1.405044, 1.405309,
      1.405575, 1.405840, 1.406106, 1.406371, 1.406637, 1.406903, 1.407265,
      1.407656, 1.408047, 1.408438, 1.408829, 1.409216, 1.409595, 1.409975,
      1.410348, 1.410716, 1.411084, 1.411452, 1.411820, 1.412188, 1.412556,
      1.412924, 1.413295, 1.413677, 1.414059, 1.414441, 1.414823, 1.415206,
      1.415583, 1.415948, 1.416316, 1.416897, 1.417479, 1.418060, 1.418642,
      1.419223, 1.419799, 1.420375, 1.421028, 1.421685, 1.422342, 1.423000,
      1.423654, 1.424306, 1.425014, 1.425747, 1.426480, 1.427213, 1.427955,
      1.428727, 1.429771, 1.430816, 1.431860, 1.432893, 1.433912, 1.434797,
      1.435681, 1.436566, 1.437446, 1.438355, 1.439315, 1.440275, 1.441241,
      1.442250, 1.443588, 1.444926, 1.446256, 1.447541, 1.448652, 1.449764,
      1.450873, 1.452149, 1.453654, 1.455159, 1.456659, 1.457930, 1.459193,
      1.460453, 1.462025, 1.463723, 1.465436, 1.466981, 1.468429, 1.469854,
      1.471623, 1.473512, 1.475397, 1.477382, 1.479366, 1.481220, 1.482862,
      1.484500, 1.486608, 1.488785, 1.491006, 1.493279, 1.495567, 1.497937,
      1.500303, 1.502768, 1.505230, 1.507780, 1.510339, 1.512981, 1.515636,
      1.518379, 1.521130, 1.523979, 1.526843, 1.529789, 1.532771, 1.535811,
      1.539385, 1.543157, 1.546391, 1.549606, 1.552891, 1.556936, 1.561124,
      1.565388, 1.569084, 1.573086, 1.577555, 1.582102, 1.586793, 1.591561,
      1.596377, 1.601273, 1.606324, 1.611872, 1.617696, 1.623104, 1.629420,
      1.634973, 1.641352, 1.648826, 1.655982, 1.663472, 1.671642, 1.680430,
      1.689925, 1.699597, 1.709448, 1.720130, 1.730801, 1.743609, 1.754824,
      1.766768, 1.777248, 1.791211, 1.805683, 1.821207, 1.837178, 1.853525,
      1.870988, 1.887585, 1.905332, 1.923626, 1.940731, 1.959093, 1.977991,
      1.996975, 2.016826, 2.039086, NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN};

// Baraffe+2015 log10 teff interpolation grid
static double const DATA_LOGT[STELLAR_BAR_MLEN][STELLAR_BAR_ALEN] = {
      3.461784, 3.461807, 3.462033, 3.462271, 3.462255, 3.462240, 3.462273,
      3.462373, 3.462365, 3.462351, 3.462343, 3.462336, 3.462329, 3.462323,
      3.462323, 3.462323, 3.462336, 3.462402, 3.462468, 3.462488, 3.462488,
      3.462488, 3.462488, 3.462488, 3.462488, 3.462481, 3.462473, 3.462473,
      3.462473, 3.462473, 3.462473, 3.462473, 3.462526, 3.462638, 3.462638,
      3.462638, 3.462637, 3.462625, 3.462623, 3.462623, 3.462623, 3.462623,
      3.462623, 3.462733, 3.462786, 3.462772, 3.462772, 3.462772, 3.462772,
      3.462772, 3.462878, 3.462922, 3.462922, 3.462922, 3.462922, 3.462920,
      3.463048, 3.463071, 3.463071, 3.463063, 3.463080, 3.463215, 3.463208,
      3.463324, 3.463363, 3.463407, 3.463517, 3.463505, 3.463622, 3.463750,
      3.463812, 3.463834, 3.463949, 3.464071, 3.464193, 3.464306, 3.464414,
      3.464529, 3.464641, 3.464761, 3.464939, 3.465166, 3.465283, 3.465503,
      3.465660, 3.465805, 3.466025, 3.466187, 3.466403, 3.466614, 3.466810,
      3.466975, 3.467176, 3.467451, 3.467665, 3.467847, 3.467990, 3.468179,
      3.468438, 3.468629, 3.468786, 3.468968, 3.469233, 3.469389, 3.469418,
      3.469410, 3.469410, 3.469410, 3.469410, 3.469410, 3.469410, 3.469410,
      3.469410, 3.469410, 3.469410, 3.469384, 3.469250, 3.469248, 3.469256,
      3.469262, 3.469262, 3.469258, 3.469130, 3.469100, 3.469100, 3.469102,
      3.469113, 3.469115, 3.469041, 3.468953, 3.468953, 3.468953, 3.468953,
      3.468953, 3.468959, 3.468967, 3.468967, 3.468866, 3.468805, 3.468805,
      3.468805, 3.468805, 3.468805, 3.468805, 3.468805, 3.468805, 3.468797,
      3.468694, 3.468658, 3.468658, 3.468658, 3.468652, 3.468540, 3.468495,
      3.468503, 3.468510, 3.468510, 3.468405, 3.468348, 3.468354, 3.468333,
      3.468224, 3.468200, 3.468200, 3.468145, 3.468052, 3.468022, 3.467924,
      3.467904, 3.467904, 3.467853, 3.467756, 3.467650, 3.467594, 3.467526,
      3.467446, 3.467408, 3.467312, 3.467217, 3.467094, 3.466904, 3.466714,
      3.466542, 3.466457, 3.466302, 3.466110, 3.465923, 3.465737, 3.465551,
      3.465365, 3.465179, 3.464994, 3.464810, 3.464625, 3.464341, 3.464093,
      3.463909, 3.463725, 3.463542, 3.463304, 3.463028, 3.462846, 3.462663,
      3.462382, 3.462143, 3.461919, 3.461639, 3.461281, 3.461111, 3.460865,
      3.460546, 3.460278, 3.460100, 3.459736, 3.459429, 3.459211, 3.458945,
      3.458614, 3.458250, 3.457986, 3.457756, 3.457505, 3.457145, 3.456793,
      3.456519, 3.456246, 3.455895, 3.455538, 3.455186, 3.454836, 3.454485,
      3.454138, 3.453792, 3.453520, 3.453152, 3.452770, 3.452392, 3.452012,
      3.451523, 3.451158, 3.450752, 3.450360, 3.450016, 3.449591, 3.449130,
      3.448666, 3.448246, 3.447827, 3.447314, 3.446821, 3.446205, 3.445641,
      3.445092, 3.444439, 3.443806, 3.443285, 3.442709, 3.442069, 3.441433,
      3.440951, 3.440331, 3.439688, 3.439095, 3.438569, 3.437932, 3.437290,
      3.436645, 3.436002, 3.435366, 3.434725, 3.434216, 3.433625, 3.432977,
      3.432320, 3.431673, 3.430911, 3.430224, 3.429518, 3.428757, 3.428123,
      3.427350, 3.426572, 3.425789, 3.424999, 3.424200, 3.423496, 3.422771,
      3.421992, 3.421135, 3.420428, 3.419583, 3.418723, 3.418053, 3.417206,
      3.416346, 3.415501, 3.414658, 3.413802, 3.412954, 3.411937, 3.410928,
      3.410095, 3.409088, 3.408099, 3.407098, 3.406115, 3.405134, 3.404139,
      3.403160, 3.402052, 3.401027, 3.400008, 3.398960, 3.397867, 3.396727,
      3.395595, 3.394476, 3.393357, 3.392230, 3.391121, 3.389939, 3.388749,
      3.387621, 3.386436, 3.385180, 3.383971, 3.382875, 3.381625, 3.380394,
      3.379168, 3.377847, 3.376512, 3.375299, 3.374091, 3.372735, 3.371487,
      3.370220, 3.368872, 3.367512, 3.366313, 3.365006, 3.363656, 3.362332,
      3.361150, 3.359805, 3.358450, 3.357103, 3.355774, 3.354582, 3.353256,
      3.351911, 3.350566, 3.349212, 3.347857, 3.346510, 3.345163, 3.343814,
      3.342463, 3.340784, 3.338830, 3.336664, 3.334674, 3.332590, 3.330458,
      3.328336, 3.326227, 3.324142, 3.322104, 3.320395, 3.318881, 3.317507,
      3.316132, 3.314850, 3.313587, 3.312198, 3.310810, 3.309419, 3.308018,
      3.306603, 3.305193, 3.303901, 3.302505, 3.300777, 3.297699, 3.289461,
      3.279494, 3.272534, 3.267677, 3.263449, 3.259360, 3.255501, 3.251984,
      3.249710, 3.248758, 3.247778, 3.246773, 3.245748, 3.245225, 3.244989,
      3.244608, 3.244058, 3.243289, 3.242487, 3.241651, 3.241037, 3.240109,
      3.239165, 3.238450, 3.237695, 3.236920, 3.236116, 3.235287, 3.234411,
      3.233527, 3.232854, 3.232096, 3.231246, 3.230417, 3.229741, 3.228733,
      3.227812, 3.226767, 3.225580, 3.224236, 3.223257, 3.222024, 3.221035,
      3.220133, 3.219045, 3.218158, 3.217403, 3.216718, 3.216021, 3.215297,
      3.214556, 3.214031, 3.213522, 3.212999, 3.212466, 3.211946, 3.211391,
      3.210978, 3.210622, 3.210362, 3.210179, 3.209894, 3.209491, 3.209168,
      3.208942, 3.208683, 3.208707, 3.208501, 3.208268, 3.208121, 3.208084,
      3.207838, 3.207828, 3.207675, 3.207535, 3.207535, 3.207535, 3.207338,
      3.207241, 3.207241, 3.207241, 3.207265, 3.207265, 3.207265, 3.207265,
      3.207265, 3.207265, 3.207034, 3.206972, 3.206972, 3.206972, 3.206972,
      3.206972, 3.206972, 3.206972, 3.206972, 3.206972, 3.206972, 3.206972,
      3.206972, 3.206972, 3.206972, 3.206972, 3.206972, 3.206972, 3.206972,
      3.206972, 3.206972, 3.206956, 3.206947, 3.206947, 3.206947, 3.206947,
      3.206947, 3.206947, 3.206947, NAN,      NAN,      3.461768, 3.461791,
      3.462015, 3.462242, 3.462248, 3.462248, 3.462289, 3.462389, 3.462398,
      3.462398, 3.462398, 3.462398, 3.462398, 3.462398, 3.462398, 3.462398,
      3.462409, 3.462469, 3.462529, 3.462548, 3.462548, 3.462548, 3.462548,
      3.462548, 3.462548, 3.462548, 3.462548, 3.462548, 3.462548, 3.462548,
      3.462548, 3.462548, 3.462596, 3.462697, 3.462697, 3.462697, 3.462697,
      3.462697, 3.462697, 3.462697, 3.462697, 3.462697, 3.462697, 3.462798,
      3.462847, 3.462847, 3.462847, 3.462847, 3.462847, 3.462847, 3.462943,
      3.462997, 3.462997, 3.462997, 3.462997, 3.462997, 3.463124, 3.463146,
      3.463146, 3.463146, 3.463168, 3.463291, 3.463296, 3.463403, 3.463445,
      3.463492, 3.463594, 3.463594, 3.463711, 3.463831, 3.463893, 3.463923,
      3.464039, 3.464161, 3.464282, 3.464395, 3.464504, 3.464620, 3.464743,
      3.464866, 3.465039, 3.465256, 3.465373, 3.465594, 3.465750, 3.465895,
      3.466116, 3.466282, 3.466495, 3.466704, 3.466900, 3.467065, 3.467266,
      3.467541, 3.467758, 3.467940, 3.468094, 3.468283, 3.468533, 3.468727,
      3.468889, 3.469072, 3.469336, 3.469492, 3.469527, 3.469527, 3.469527,
      3.469527, 3.469527, 3.469527, 3.469527, 3.469527, 3.469527, 3.469527,
      3.469527, 3.469504, 3.469382, 3.469380, 3.469380, 3.469380, 3.469380,
      3.469376, 3.469260, 3.469233, 3.469233, 3.469233, 3.469233, 3.469233,
      3.469166, 3.469085, 3.469085, 3.469085, 3.469085, 3.469085, 3.469085,
      3.469085, 3.469085, 3.468993, 3.468938, 3.468938, 3.468938, 3.468938,
      3.468938, 3.468938, 3.468938, 3.468938, 3.468930, 3.468827, 3.468790,
      3.468790, 3.468790, 3.468785, 3.468683, 3.468643, 3.468643, 3.468643,
      3.468643, 3.468547, 3.468495, 3.468495, 3.468469, 3.468369, 3.468347,
      3.468347, 3.468293, 3.468200, 3.468170, 3.468072, 3.468052, 3.468052,
      3.468002, 3.467904, 3.467808, 3.467756, 3.467689, 3.467608, 3.467571,
      3.467475, 3.467380, 3.467258, 3.467068, 3.466878, 3.466704, 3.466610,
      3.466461, 3.466274, 3.466087, 3.465901, 3.465715, 3.465529, 3.465344,
      3.465159, 3.464974, 3.464789, 3.464515, 3.464272, 3.464088, 3.463905,
      3.463722, 3.463485, 3.463208, 3.463026, 3.462843, 3.462571, 3.462332,
      3.462103, 3.461833, 3.461491, 3.461312, 3.461067, 3.460756, 3.460488,
      3.460296, 3.459940, 3.459639, 3.459421, 3.459156, 3.458824, 3.458475,
      3.458211, 3.457976, 3.457719, 3.457369, 3.457019, 3.456745, 3.456472,
      3.456123, 3.455774, 3.455426, 3.455078, 3.454731, 3.454383, 3.454043,
      3.453756, 3.453388, 3.453014, 3.452643, 3.452266, 3.451786, 3.451422,
      3.451017, 3.450625, 3.450274, 3.449844, 3.449392, 3.448935, 3.448513,
      3.448091, 3.447592, 3.447098, 3.446503, 3.445949, 3.445404, 3.444767,
      3.444158, 3.443632, 3.443058, 3.442426, 3.441792, 3.441296, 3.440686,
      3.440048, 3.439456, 3.438930, 3.438294, 3.437654, 3.437017, 3.436382,
      3.435747, 3.435107, 3.434585, 3.433991, 3.433349, 3.432703, 3.432057,
      3.431295, 3.430609, 3.429907, 3.429151, 3.428505, 3.427732, 3.426956,
      3.426182, 3.425402, 3.424613, 3.423902, 3.423171, 3.422392, 3.421550,
      3.420845, 3.420009, 3.419158, 3.418472, 3.417626, 3.416782, 3.415937,
      3.415095, 3.414256, 3.413409, 3.412409, 3.411416, 3.410584, 3.409593,
      3.408605, 3.407620, 3.406638, 3.405657, 3.404678, 3.403701, 3.402606,
      3.401587, 3.400569, 3.399522, 3.398438, 3.397315, 3.396195, 3.395077,
      3.393963, 3.392852, 3.391744, 3.390571, 3.389391, 3.388266, 3.387091,
      3.385846, 3.384640, 3.383547, 3.382315, 3.381087, 3.379864, 3.378562,
      3.377249, 3.376039, 3.374834, 3.373495, 3.372254, 3.370991, 3.369650,
      3.368309, 3.367116, 3.365817, 3.364478, 3.363171, 3.361982, 3.360654,
      3.359316, 3.357977, 3.356656, 3.355467, 3.354157, 3.352821, 3.351486,
      3.350150, 3.348815, 3.347479, 3.346142, 3.344804, 3.343465, 3.341827,
      3.339940, 3.337860, 3.335941, 3.333937, 3.331888, 3.329849, 3.327823,
      3.325805, 3.323806, 3.322097, 3.320560, 3.319144, 3.317730, 3.316405,
      3.315106, 3.313687, 3.312266, 3.310845, 3.309423, 3.308000, 3.306604,
      3.305323, 3.303944, 3.302269, 3.299370, 3.291777, 3.282618, 3.276194,
      3.271668, 3.267723, 3.263905, 3.260296, 3.257008, 3.254837, 3.253870,
      3.252883, 3.251872, 3.250841, 3.249915, 3.248959, 3.247934, 3.247039,
      3.246126, 3.245193, 3.244250, 3.243517, 3.242525, 3.241521, 3.240745,
      3.239953, 3.239145, 3.238321, 3.237479, 3.236621, 3.235745, 3.235077,
      3.234326, 3.233514, 3.232705, 3.232061, 3.231130, 3.230286, 3.229324,
      3.228236, 3.227013, 3.226102, 3.224981, 3.224068, 3.223240, 3.222249,
      3.221425, 3.220736, 3.220103, 3.219457, 3.218800, 3.218126, 3.217630,
      3.217163, 3.216688, 3.216202, 3.215709, 3.215205, 3.214829, 3.214505,
      3.214263, 3.214082, 3.213821, 3.213455, 3.213161, 3.212956, 3.212720,
      3.212720, 3.212533, 3.212321, 3.212188, 3.212154, 3.211931, 3.211921,
      3.211783, 3.211654, 3.211654, 3.211654, 3.211476, 3.211388, 3.211388,
      3.211388, 3.211388, 3.211388, 3.211388, 3.211388, 3.211388, 3.211388,
      3.211177, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121,
      3.211121, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121,
      3.211121, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121,
      3.211121, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121, 3.211121,
      3.211121, 3.211121, NAN,      3.461611, 3.461634, 3.461831, 3.461947,
      3.462181, 3.462334, 3.462446, 3.462550, 3.462725, 3.462870, 3.462946,
      3.463019, 3.463084, 3.463146, 3.463146, 3.463146, 3.463146, 3.463146,
      3.463146, 3.463146, 3.463146, 3.463146, 3.463146, 3.463146, 3.463146,
      3.463218, 3.463296, 3.463296, 3.463296, 3.463296, 3.463296, 3.463296,
      3.463296, 3.463296, 3.463296, 3.463296, 3.463297, 3.463426, 3.463445,
      3.463445, 3.463445, 3.463445, 3.463445, 3.463445, 3.463454, 3.463594,
      3.463594, 3.463594, 3.463594, 3.463594, 3.463594, 3.463744, 3.463744,
      3.463744, 3.463744, 3.463762, 3.463881, 3.463893, 3.463893, 3.463978,
      3.464042, 3.464051, 3.464173, 3.464191, 3.464269, 3.464340, 3.464365,
      3.464489, 3.464602, 3.464639, 3.464698, 3.464815, 3.464936, 3.465055,
      3.465175, 3.465289, 3.465400, 3.465526, 3.465768, 3.465920, 3.466040,
      3.466158, 3.466275, 3.466504, 3.466650, 3.466802, 3.467021, 3.467235,
      3.467413, 3.467605, 3.467797, 3.467964, 3.468164, 3.468441, 3.468688,
      3.468874, 3.469135, 3.469323, 3.469477, 3.469702, 3.469924, 3.470105,
      3.470369, 3.470523, 3.470622, 3.470704, 3.470704, 3.470704, 3.470704,
      3.470704, 3.470704, 3.470704, 3.470704, 3.470704, 3.470704, 3.470704,
      3.470704, 3.470704, 3.470620, 3.470557, 3.470557, 3.470557, 3.470557,
      3.470557, 3.470557, 3.470540, 3.470427, 3.470410, 3.470410, 3.470410,
      3.470410, 3.470410, 3.470410, 3.470410, 3.470352, 3.470263, 3.470263,
      3.470263, 3.470263, 3.470263, 3.470263, 3.470263, 3.470263, 3.470263,
      3.470263, 3.470263, 3.470263, 3.470160, 3.470116, 3.470116, 3.470116,
      3.470116, 3.470116, 3.470116, 3.470035, 3.469969, 3.469969, 3.469969,
      3.469969, 3.469902, 3.469822, 3.469822, 3.469822, 3.469822, 3.469776,
      3.469678, 3.469654, 3.469555, 3.469527, 3.469527, 3.469485, 3.469388,
      3.469380, 3.469380, 3.469320, 3.469233, 3.469202, 3.469106, 3.469011,
      3.468895, 3.468706, 3.468517, 3.468328, 3.468141, 3.468052, 3.467914,
      3.467728, 3.467541, 3.467355, 3.467170, 3.466985, 3.466800, 3.466616,
      3.466433, 3.466249, 3.466066, 3.465883, 3.465701, 3.465519, 3.465290,
      3.465006, 3.464825, 3.464644, 3.464463, 3.464224, 3.463953, 3.463773,
      3.463592, 3.463323, 3.463085, 3.462861, 3.462593, 3.462252, 3.461984,
      3.461744, 3.461526, 3.461261, 3.460928, 3.460731, 3.460467, 3.460172,
      3.459865, 3.459602, 3.459272, 3.459002, 3.458724, 3.458404, 3.458143,
      3.457826, 3.457505, 3.457185, 3.456826, 3.456551, 3.456113, 3.455752,
      3.455451, 3.455152, 3.454804, 3.454413, 3.454054, 3.453666, 3.453274,
      3.452858, 3.452373, 3.452005, 3.451634, 3.451185, 3.450734, 3.450367,
      3.449869, 3.449481, 3.449030, 3.448525, 3.448049, 3.447675, 3.447096,
      3.446545, 3.445997, 3.445378, 3.444748, 3.444230, 3.443645, 3.443064,
      3.442542, 3.441921, 3.441290, 3.440738, 3.440181, 3.439553, 3.438919,
      3.438282, 3.437649, 3.437066, 3.436533, 3.435894, 3.435137, 3.434458,
      3.433793, 3.433092, 3.432327, 3.431552, 3.430805, 3.430115, 3.429431,
      3.428748, 3.427966, 3.427167, 3.426400, 3.425694, 3.425015, 3.424267,
      3.423504, 3.422668, 3.421824, 3.421139, 3.420305, 3.419465, 3.418794,
      3.417959, 3.417128, 3.416296, 3.415469, 3.414643, 3.413657, 3.412834,
      3.411865, 3.410888, 3.410060, 3.409118, 3.408153, 3.407189, 3.406176,
      3.405139, 3.404148, 3.403197, 3.402191, 3.401089, 3.400019, 3.399067,
      3.397975, 3.396889, 3.395803, 3.394722, 3.393646, 3.392507, 3.391331,
      3.390268, 3.389210, 3.388013, 3.386816, 3.385710, 3.384615, 3.383433,
      3.382257, 3.381088, 3.379924, 3.378699, 3.377432, 3.376286, 3.375147,
      3.373921, 3.372700, 3.371568, 3.370305, 3.369147, 3.367974, 3.366719,
      3.365468, 3.364320, 3.363163, 3.361920, 3.360681, 3.359529, 3.358397,
      3.357164, 3.355935, 3.354707, 3.353482, 3.352261, 3.351042, 3.349825,
      3.348611, 3.347400, 3.346191, 3.344983, 3.343777, 3.342426, 3.340822,
      3.339113, 3.337348, 3.335517, 3.333709, 3.331950, 3.330300, 3.328579,
      3.326830, 3.325107, 3.323472, 3.321970, 3.320720, 3.319546, 3.318340,
      3.317192, 3.316081, 3.314938, 3.313864, 3.312799, 3.311584, 3.310465,
      3.309358, 3.308247, 3.307256, 3.306106, 3.304996, 3.303942, 3.302867,
      3.301768, 3.296814, 3.288659, 3.281193, 3.276852, 3.274502, 3.272260,
      3.270244, 3.268322, 3.266681, 3.265077, 3.263691, 3.262536, 3.261398,
      3.260370, 3.259404, 3.258716, 3.257924, 3.257314, 3.256627, 3.256188,
      3.255588, 3.255258, 3.255102, 3.255031, 3.254901, 3.254790, 3.254790,
      3.254548, 3.254548, 3.254406, 3.254306, 3.254282, 3.254094, 3.254064,
      3.253948, 3.253822, 3.253822, 3.253822, 3.253614, 3.253580, 3.253580,
      3.253564, 3.253338, 3.253338, 3.253338, 3.253338, 3.253269, 3.253103,
      3.253096, 3.253096, 3.253096, 3.253096, 3.253096, 3.252853, 3.252853,
      3.252853, 3.252853, 3.252853, 3.252853, 3.252853, 3.252853, 3.252853,
      3.252853, 3.252853, 3.252853, 3.252853, 3.252853, 3.252851, 3.252610,
      3.252610, 3.252610, 3.252610, 3.252610, 3.252610, 3.252610, 3.252610,
      3.252610, 3.252610, 3.252610, 3.252610, 3.252610, 3.252610, 3.252610,
      3.252610, 3.252610, 3.252610, 3.252610, 3.252610, 3.252610, 3.252610,
      3.252610, 3.252610, 3.252610, 3.252610, 3.252610, 3.252769, 3.252853,
      3.252853, 3.252853, 3.252853, 3.252853, 3.252853, 3.252853, NAN,
      NAN,      3.461864, 3.461876, 3.462036, 3.462260, 3.462380, 3.462502,
      3.462613, 3.462732, 3.462911, 3.463037, 3.463114, 3.463185, 3.463251,
      3.463336, 3.463455, 3.463575, 3.463649, 3.463714, 3.463782, 3.463852,
      3.463957, 3.464108, 3.464191, 3.464191, 3.464191, 3.464191, 3.464191,
      3.464191, 3.464191, 3.464191, 3.464220, 3.464329, 3.464340, 3.464340,
      3.464340, 3.464340, 3.464340, 3.464340, 3.464340, 3.464483, 3.464490,
      3.464490, 3.464490, 3.464490, 3.464490, 3.464517, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464677, 3.464788, 3.464788, 3.464788, 3.464831,
      3.464936, 3.464944, 3.465058, 3.465085, 3.465152, 3.465234, 3.465251,
      3.465374, 3.465383, 3.465463, 3.465580, 3.465680, 3.465680, 3.465799,
      3.465916, 3.466035, 3.466188, 3.466425, 3.466542, 3.466660, 3.466766,
      3.466859, 3.467086, 3.467244, 3.467362, 3.467478, 3.467593, 3.467754,
      3.467975, 3.468123, 3.468261, 3.468474, 3.468637, 3.468841, 3.469044,
      3.469243, 3.469402, 3.469695, 3.469904, 3.470125, 3.470359, 3.470546,
      3.470817, 3.471091, 3.471289, 3.471483, 3.471750, 3.472014, 3.472108,
      3.472207, 3.472318, 3.472318, 3.472318, 3.472318, 3.472401, 3.472464,
      3.472464, 3.472464, 3.472464, 3.472464, 3.472464, 3.472464, 3.472464,
      3.472464, 3.472464, 3.472464, 3.472464, 3.472366, 3.472318, 3.472318,
      3.472318, 3.472318, 3.472318, 3.472318, 3.472318, 3.472318, 3.472311,
      3.472203, 3.472171, 3.472171, 3.472171, 3.472171, 3.472171, 3.472171,
      3.472171, 3.472171, 3.472171, 3.472171, 3.472171, 3.472171, 3.472171,
      3.472171, 3.472166, 3.472065, 3.472025, 3.472025, 3.472025, 3.472025,
      3.472025, 3.472025, 3.472025, 3.472025, 3.472025, 3.472025, 3.472025,
      3.471963, 3.471878, 3.471878, 3.471878, 3.471878, 3.471839, 3.471742,
      3.471732, 3.471732, 3.471732, 3.471711, 3.471585, 3.471585, 3.471507,
      3.471438, 3.471438, 3.471438, 3.471399, 3.471210, 3.471083, 3.470978,
      3.470790, 3.470603, 3.470557, 3.470376, 3.470190, 3.470003, 3.469820,
      3.469728, 3.469596, 3.469413, 3.469229, 3.469046, 3.468864, 3.468681,
      3.468499, 3.468317, 3.468136, 3.467955, 3.467774, 3.467593, 3.467413,
      3.467233, 3.467053, 3.466874, 3.466669, 3.466367, 3.466188, 3.466009,
      3.465758, 3.465504, 3.465298, 3.465031, 3.464822, 3.464645, 3.464447,
      3.464118, 3.463853, 3.463640, 3.463400, 3.463139, 3.462948, 3.462685,
      3.462461, 3.462194, 3.461896, 3.461652, 3.461372, 3.461060, 3.460703,
      3.460450, 3.460211, 3.459889, 3.459562, 3.459186, 3.458911, 3.458613,
      3.458247, 3.457858, 3.457619, 3.457273, 3.456914, 3.456557, 3.456187,
      3.455822, 3.455455, 3.455013, 3.454563, 3.454198, 3.453833, 3.453361,
      3.452945, 3.452575, 3.452184, 3.451699, 3.451202, 3.450784, 3.450366,
      3.449875, 3.449383, 3.448994, 3.448544, 3.448050, 3.447552, 3.447051,
      3.446441, 3.445819, 3.445246, 3.444730, 3.444139, 3.443525, 3.442973,
      3.442432, 3.441815, 3.441184, 3.440540, 3.439896, 3.439304, 3.438668,
      3.437992, 3.437315, 3.436638, 3.435831, 3.435125, 3.434449, 3.433773,
      3.433089, 3.432262, 3.431576, 3.430818, 3.430080, 3.429409, 3.428713,
      3.427959, 3.427238, 3.426504, 3.425743, 3.425042, 3.424297, 3.423515,
      3.422692, 3.421949, 3.421218, 3.420402, 3.419589, 3.418780, 3.417973,
      3.417171, 3.416372, 3.415574, 3.414743, 3.413825, 3.413023, 3.412086,
      3.411153, 3.410226, 3.409299, 3.408378, 3.407461, 3.406549, 3.405642,
      3.404696, 3.403671, 3.402777, 3.401740, 3.400806, 3.399814, 3.398795,
      3.397780, 3.396767, 3.395764, 3.394767, 3.393777, 3.392732, 3.391640,
      3.390670, 3.389584, 3.388576, 3.387584, 3.386507, 3.385441, 3.384382,
      3.383330, 3.382287, 3.381254, 3.380229, 3.379211, 3.378205, 3.377207,
      3.376193, 3.375089, 3.374078, 3.373090, 3.372011, 3.371015, 3.370066,
      3.369127, 3.368134, 3.367089, 3.366158, 3.365211, 3.364190, 3.363246,
      3.362349, 3.361360, 3.360367, 3.359385, 3.358462, 3.357600, 3.356671,
      3.355716, 3.354774, 3.353841, 3.352983, 3.352116, 3.351226, 3.350319,
      3.349398, 3.348552, 3.347711, 3.346849, 3.345969, 3.345091, 3.344304,
      3.343503, 3.342645, 3.341531, 3.340393, 3.339234, 3.338054, 3.336839,
      3.335709, 3.334605, 3.333503, 3.332516, 3.331506, 3.330476, 3.329522,
      3.328556, 3.327563, 3.326786, 3.325835, 3.324872, 3.324080, 3.323271,
      3.322647, 3.322016, 3.321578, 3.320940, 3.320527, 3.320245, 3.319781,
      3.319308, 3.318825, 3.318511, 3.318052, 3.317737, 3.317326, 3.316893,
      3.316580, 3.316304, 3.316022, 3.315525, 3.315232, 3.314934, 3.314629,
      3.314319, 3.314041, 3.313880, 3.313564, 3.313337, 3.313099, 3.312782,
      3.312605, 3.312424, 3.312240, 3.312053, 3.311862, 3.311637, 3.311374,
      3.311330, 3.311111, 3.310906, 3.310868, 3.310722, 3.310515, 3.310352,
      3.310268, 3.310249, 3.310056, 3.310056, 3.309909, 3.309740, 3.309630,
      3.309630, 3.309630, 3.309458, 3.309417, 3.309368, 3.309204, 3.309204,
      3.309204, 3.309204, 3.309151, 3.308991, 3.308991, 3.308991, 3.308991,
      3.308991, 3.308991, 3.308991, 3.308991, 3.308882, 3.308778, 3.308778,
      3.308778, 3.308778, 3.308778, 3.308778, 3.308778, 3.308778, 3.308778,
      3.308778, 3.308778, 3.308778, 3.308778, 3.308778, 3.308778, 3.308778,
      3.308778, 3.308778, 3.308778, 3.308778, 3.308873, 3.308991, 3.308991,
      3.308991, 3.308991, 3.308991, 3.308991, 3.308991, NAN,      3.462000,
      3.462012, 3.462133, 3.462264, 3.462514, 3.462658, 3.462772, 3.462875,
      3.462968, 3.463052, 3.463131, 3.463258, 3.463396, 3.463484, 3.463545,
      3.463606, 3.463672, 3.463738, 3.463875, 3.464018, 3.464106, 3.464182,
      3.464264, 3.464346, 3.464434, 3.464524, 3.464617, 3.464714, 3.464814,
      3.464919, 3.465028, 3.465141, 3.465258, 3.465381, 3.465509, 3.465643,
      3.465783, 3.465829, 3.465933, 3.465977, 3.465977, 3.465977, 3.465977,
      3.465977, 3.466126, 3.466126, 3.466126, 3.466126, 3.466274, 3.466274,
      3.466293, 3.466400, 3.466423, 3.466511, 3.466571, 3.466571, 3.466576,
      3.466707, 3.466826, 3.466868, 3.466911, 3.467038, 3.467152, 3.467273,
      3.467393, 3.467514, 3.467637, 3.467755, 3.467872, 3.468077, 3.468257,
      3.468396, 3.468596, 3.468739, 3.468862, 3.468984, 3.469123, 3.469359,
      3.469485, 3.469600, 3.469750, 3.469971, 3.470081, 3.470226, 3.470427,
      3.470598, 3.470702, 3.470856, 3.471059, 3.471259, 3.471468, 3.471751,
      3.471944, 3.472108, 3.472328, 3.472607, 3.472885, 3.473031, 3.473284,
      3.473551, 3.473817, 3.474081, 3.474341, 3.474599, 3.474720, 3.474799,
      3.474799, 3.474913, 3.474944, 3.474944, 3.474991, 3.475090, 3.475090,
      3.475090, 3.475180, 3.475235, 3.475235, 3.475235, 3.475235, 3.475235,
      3.475235, 3.475235, 3.475235, 3.475235, 3.475235, 3.475235, 3.475235,
      3.475235, 3.475235, 3.475235, 3.475235, 3.475235, 3.475235, 3.475235,
      3.475235, 3.475235, 3.475235, 3.475235, 3.475235, 3.475235, 3.475173,
      3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090,
      3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090,
      3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090,
      3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090, 3.475090,
      3.475090, 3.475090, 3.475083, 3.474988, 3.474944, 3.474944, 3.474944,
      3.474944, 3.474944, 3.474769, 3.474653, 3.474539, 3.474357, 3.474264,
      3.474125, 3.473939, 3.473779, 3.473715, 3.473530, 3.473346, 3.473252,
      3.473126, 3.472943, 3.472761, 3.472668, 3.472544, 3.472363, 3.472183,
      3.472002, 3.471823, 3.471643, 3.471524, 3.471431, 3.471253, 3.471075,
      3.470896, 3.470719, 3.470541, 3.470363, 3.470186, 3.469901, 3.469685,
      3.469508, 3.469332, 3.469155, 3.468979, 3.468734, 3.468476, 3.468285,
      3.468074, 3.467830, 3.467618, 3.467414, 3.467181, 3.466914, 3.466674,
      3.466421, 3.466161, 3.465916, 3.465678, 3.465441, 3.465202, 3.464959,
      3.464718, 3.464485, 3.464240, 3.463998, 3.463756, 3.463476, 3.463131,
      3.462886, 3.462650, 3.462343, 3.462020, 3.461769, 3.461401, 3.461143,
      3.460832, 3.460471, 3.460107, 3.459745, 3.459382, 3.459016, 3.458651,
      3.458290, 3.457927, 3.457562, 3.457120, 3.456680, 3.456316, 3.455949,
      3.455484, 3.455063, 3.454676, 3.454271, 3.453796, 3.453307, 3.452837,
      3.452417, 3.451947, 3.451444, 3.451006, 3.450587, 3.450091, 3.449577,
      3.449077, 3.448577, 3.448079, 3.447547, 3.446966, 3.446430, 3.445834,
      3.445225, 3.444646, 3.444002, 3.443341, 3.442753, 3.442174, 3.441515,
      3.440856, 3.440197, 3.439539, 3.438882, 3.438206, 3.437470, 3.436759,
      3.436106, 3.435398, 3.434648, 3.434001, 3.433292, 3.432552, 3.431848,
      3.431113, 3.430478, 3.429786, 3.429052, 3.428376, 3.427638, 3.426999,
      3.426305, 3.425602, 3.424838, 3.424078, 3.423472, 3.422739, 3.421994,
      3.421257, 3.420523, 3.419792, 3.419070, 3.418353, 3.417641, 3.416797,
      3.416073, 3.415381, 3.414638, 3.413820, 3.413010, 3.412208, 3.411413,
      3.410626, 3.409847, 3.409077, 3.408317, 3.407508, 3.406648, 3.405912,
      3.405184, 3.404430, 3.403600, 3.402880, 3.402144, 3.401344, 3.400558,
      3.399814, 3.399156, 3.398431, 3.397684, 3.396952, 3.396229, 3.395517,
      3.394819, 3.394127, 3.393363, 3.392609, 3.391940, 3.391281, 3.390644,
      3.390004, 3.389359, 3.388709, 3.388053, 3.387443, 3.386873, 3.386296,
      3.385717, 3.385130, 3.384541, 3.384040, 3.383540, 3.383032, 3.382520,
      3.382004, 3.381481, 3.380955, 3.380522, 3.380091, 3.379654, 3.379212,
      3.378766, 3.378305, 3.377888, 3.377532, 3.377167, 3.376796, 3.376419,
      3.376153, 3.375822, 3.375442, 3.375169, 3.374891, 3.374608, 3.374321,
      3.374027, 3.373871, 3.373605, 3.373379, 3.373161, 3.372875, 3.372710,
      3.372543, 3.372372, 3.372197, 3.372020, 3.371839, 3.371654, 3.371622,
      3.371460, 3.371265, 3.371251, 3.371048, 3.370883, 3.370883, 3.370787,
      3.370698, 3.370531, 3.370513, 3.370444, 3.370328, 3.370328, 3.370280,
      3.370143, 3.370143, 3.370143, 3.370143, 3.369958, 3.369958, 3.369958,
      3.369958, 3.369958, 3.369958, 3.369820, 3.369772, 3.369772, 3.369772,
      3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772,
      3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772,
      3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772,
      3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772,
      3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772, 3.369772,
      3.369772, 3.369772, 3.369772, 3.369843, 3.369958, 3.369958, 3.369958,
      3.369958, 3.369958, 3.369958, 3.369958, 3.369958, 3.369958, 3.369958,
      3.369958, 3.369958, 3.369958, 3.369958, 3.369958, 3.369958, 3.369958,
      3.369958, 3.369958, 3.369958, 3.370002, 3.370141, 3.370143, 3.370143,
      3.370143, 3.370143, 3.370143, NAN,      3.463484, 3.463495, 3.463600,
      3.463690, 3.463776, 3.463854, 3.463927, 3.463995, 3.464061, 3.464124,
      3.464187, 3.464255, 3.464323, 3.464395, 3.464469, 3.464546, 3.464625,
      3.464708, 3.464793, 3.464882, 3.464974, 3.465069, 3.465169, 3.465271,
      3.465377, 3.465489, 3.465605, 3.465725, 3.465851, 3.465981, 3.466118,
      3.466126, 3.466261, 3.466418, 3.466582, 3.466754, 3.466935, 3.467125,
      3.467325, 3.467537, 3.467613, 3.467851, 3.468103, 3.468372, 3.468663,
      3.468952, 3.469107, 3.469225, 3.469233, 3.469297, 3.469432, 3.469527,
      3.469527, 3.469635, 3.469755, 3.469874, 3.469993, 3.470114, 3.470230,
      3.470347, 3.470519, 3.470754, 3.470998, 3.471115, 3.471318, 3.471550,
      3.471774, 3.471977, 3.472213, 3.472386, 3.472547, 3.472769, 3.472884,
      3.473092, 3.473315, 3.473511, 3.473619, 3.473773, 3.473981, 3.474188,
      3.474385, 3.474546, 3.474746, 3.474907, 3.475090, 3.475272, 3.475427,
      3.475616, 3.475870, 3.476087, 3.476266, 3.476410, 3.476682, 3.476863,
      3.477074, 3.477302, 3.477457, 3.477712, 3.477884, 3.478088, 3.478342,
      3.478594, 3.478844, 3.478855, 3.478919, 3.478999, 3.478999, 3.479097,
      3.479215, 3.479287, 3.479305, 3.479421, 3.479431, 3.479506, 3.479575,
      3.479588, 3.479700, 3.479719, 3.479779, 3.479863, 3.479927, 3.480007,
      3.480007, 3.480007, 3.480074, 3.480151, 3.480151, 3.480151, 3.480214,
      3.480294, 3.480294, 3.480294, 3.480294, 3.480294, 3.480294, 3.480294,
      3.480294, 3.480294, 3.480395, 3.480438, 3.480438, 3.480438, 3.480438,
      3.480438, 3.480438, 3.480438, 3.480438, 3.480438, 3.480438, 3.480438,
      3.480438, 3.480438, 3.480438, 3.480528, 3.480582, 3.480582, 3.480582,
      3.480582, 3.480582, 3.480582, 3.480582, 3.480582, 3.480582, 3.480582,
      3.480652, 3.480725, 3.480725, 3.480725, 3.480725, 3.480725, 3.480725,
      3.480736, 3.480830, 3.480869, 3.480869, 3.480869, 3.480869, 3.480869,
      3.480781, 3.480725, 3.480668, 3.480571, 3.480387, 3.480294, 3.480230,
      3.480126, 3.479976, 3.479885, 3.479725, 3.479632, 3.479542, 3.479451,
      3.479291, 3.479200, 3.479077, 3.478948, 3.478859, 3.478698, 3.478609,
      3.478473, 3.478295, 3.478133, 3.478086, 3.477909, 3.477789, 3.477701,
      3.477530, 3.477420, 3.477292, 3.477102, 3.476928, 3.476802, 3.476663,
      3.476538, 3.476306, 3.476165, 3.475994, 3.475789, 3.475671, 3.475427,
      3.475286, 3.475104, 3.474908, 3.474783, 3.474555, 3.474417, 3.474235,
      3.473996, 3.473771, 3.473653, 3.473442, 3.473208, 3.472971, 3.472735,
      3.472500, 3.472267, 3.472032, 3.471837, 3.471708, 3.471474, 3.471239,
      3.471003, 3.470767, 3.470532, 3.470295, 3.470060, 3.469775, 3.469478,
      3.469253, 3.468986, 3.468779, 3.468569, 3.468320, 3.468028, 3.467800,
      3.467513, 3.467182, 3.467000, 3.466670, 3.466381, 3.466158, 3.465829,
      3.465500, 3.465274, 3.464982, 3.464654, 3.464325, 3.463997, 3.463670,
      3.463342, 3.463015, 3.462688, 3.462361, 3.461928, 3.461559, 3.461168,
      3.460762, 3.460280, 3.459803, 3.459401, 3.458980, 3.458546, 3.458172,
      3.457694, 3.457218, 3.456743, 3.456312, 3.455920, 3.455483, 3.455016,
      3.454551, 3.454089, 3.453628, 3.453169, 3.452712, 3.452159, 3.451652,
      3.451201, 3.450751, 3.450305, 3.449782, 3.449264, 3.448794, 3.448285,
      3.447800, 3.447335, 3.446764, 3.446198, 3.445636, 3.445079, 3.444528,
      3.443984, 3.443444, 3.442907, 3.442352, 3.441698, 3.441182, 3.440673,
      3.440170, 3.439640, 3.439028, 3.438545, 3.438071, 3.437568, 3.436993,
      3.436536, 3.436036, 3.435498, 3.435072, 3.434596, 3.434082, 3.433683,
      3.433249, 3.432768, 3.432370, 3.431998, 3.431531, 3.431118, 3.430834,
      3.430490, 3.430106, 3.429716, 3.429350, 3.429052, 3.428752, 3.428449,
      3.428141, 3.427831, 3.427518, 3.427296, 3.427042, 3.426718, 3.426484,
      3.426262, 3.426038, 3.425811, 3.425580, 3.425359, 3.425241, 3.425034,
      3.424836, 3.424708, 3.424457, 3.424312, 3.424189, 3.424005, 3.423865,
      3.423784, 3.423666, 3.423497, 3.423368, 3.423280, 3.423191, 3.423101,
      3.423082, 3.423078, 3.422982, 3.422884, 3.422785, 3.422754, 3.422744,
      3.422639, 3.422590, 3.422586, 3.422475, 3.422426, 3.422426, 3.422426,
      3.422426, 3.422379, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422215, 3.422097, 3.422097, 3.422097, 3.422097, 3.422097,
      3.422097, 3.422097, 3.422097, 3.422097, 3.422097, 3.422097, 3.422097,
      3.422097, 3.422097, 3.422097, 3.422097, 3.422097, 3.422097, 3.422097,
      3.422228, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261, 3.422261,
      3.422261, 3.422261, 3.422261, 3.422321, 3.422426, 3.422426, 3.422426,
      3.422426, 3.422426, 3.422426, 3.422426, 3.422426, 3.422426, 3.422426,
      3.422426, 3.422426, 3.422426, 3.422426, 3.422426, 3.422426, 3.422426,
      3.422426, 3.422426, 3.422426, 3.422426, 3.422426, 3.422426, 3.422532,
      3.422590, 3.422590, 3.422590, 3.422590, 3.422590, 3.422590, 3.422590,
      NAN,      NAN,      3.467611, 3.467621, 3.467718, 3.467756, 3.467772,
      3.467880, 3.467992, 3.468108, 3.468228, 3.468347, 3.468347, 3.468472,
      3.468614, 3.468762, 3.468790, 3.468931, 3.469101, 3.469280, 3.469380,
      3.469516, 3.469724, 3.469822, 3.470027, 3.470272, 3.470410, 3.470661,
      3.470851, 3.471124, 3.471438, 3.471677, 3.472025, 3.472324, 3.472440,
      3.472565, 3.472610, 3.472651, 3.472761, 3.472885, 3.472903, 3.472975,
      3.473092, 3.473218, 3.473333, 3.473559, 3.473714, 3.473830, 3.473969,
      3.474209, 3.474416, 3.474568, 3.474717, 3.474985, 3.475176, 3.475333,
      3.475505, 3.475686, 3.475833, 3.476106, 3.476315, 3.476450, 3.476625,
      3.476871, 3.477046, 3.477195, 3.477324, 3.477452, 3.477571, 3.477655,
      3.477758, 3.477882, 3.478006, 3.478128, 3.478365, 3.478514, 3.478634,
      3.478752, 3.478870, 3.478988, 3.479210, 3.479365, 3.479480, 3.479615,
      3.479843, 3.479967, 3.480154, 3.480337, 3.480461, 3.480686, 3.480909,
      3.481132, 3.481354, 3.481575, 3.481795, 3.482015, 3.482234, 3.482449,
      3.482557, 3.482594, 3.482702, 3.482731, 3.482731, 3.482736, 3.482842,
      3.482874, 3.482909, 3.483013, 3.483016, 3.483016, 3.483108, 3.483210,
      3.483302, 3.483302, 3.483439, 3.483445, 3.483499, 3.483598, 3.483697,
      3.483730, 3.483823, 3.483921, 3.484015, 3.484045, 3.484143, 3.484157,
      3.484194, 3.484291, 3.484316, 3.484412, 3.484442, 3.484481, 3.484585,
      3.484585, 3.484677, 3.484727, 3.484727, 3.484749, 3.484844, 3.484869,
      3.484869, 3.484915, 3.485009, 3.485011, 3.485011, 3.485011, 3.485011,
      3.485052, 3.485146, 3.485153, 3.485153, 3.485153, 3.485153, 3.485185,
      3.485277, 3.485295, 3.485295, 3.485295, 3.485295, 3.485311, 3.485402,
      3.485437, 3.485437, 3.485437, 3.485437, 3.485567, 3.485579, 3.485579,
      3.485579, 3.485580, 3.485669, 3.485721, 3.485721, 3.485726, 3.485815,
      3.485863, 3.485863, 3.485863, 3.485863, 3.485863, 3.485742, 3.485643,
      3.485579, 3.485537, 3.485449, 3.485361, 3.485295, 3.485256, 3.485168,
      3.485011, 3.485011, 3.484942, 3.484818, 3.484731, 3.484703, 3.484530,
      3.484442, 3.484384, 3.484297, 3.484195, 3.484079, 3.483947, 3.483818,
      3.483730, 3.483714, 3.483597, 3.483485, 3.483375, 3.483258, 3.483132,
      3.483017, 3.482909, 3.482794, 3.482678, 3.482561, 3.482440, 3.482209,
      3.482068, 3.481952, 3.481838, 3.481725, 3.481609, 3.481493, 3.481378,
      3.481263, 3.481140, 3.480912, 3.480775, 3.480659, 3.480508, 3.480286,
      3.480170, 3.480055, 3.479872, 3.479680, 3.479565, 3.479450, 3.479244,
      3.479034, 3.478844, 3.478737, 3.478550, 3.478357, 3.478191, 3.478026,
      3.477731, 3.477550, 3.477385, 3.477197, 3.476950, 3.476745, 3.476581,
      3.476354, 3.476107, 3.475934, 3.475689, 3.475471, 3.475235, 3.475000,
      3.474837, 3.474551, 3.474297, 3.474060, 3.473872, 3.473592, 3.473431,
      3.473201, 3.472965, 3.472709, 3.472391, 3.472195, 3.471970, 3.471733,
      3.471419, 3.471115, 3.470881, 3.470627, 3.470316, 3.470006, 3.469697,
      3.469389, 3.469081, 3.468775, 3.468470, 3.468166, 3.467863, 3.467562,
      3.467262, 3.466964, 3.466667, 3.466372, 3.465980, 3.465638, 3.465303,
      3.464943, 3.464626, 3.464342, 3.463996, 3.463633, 3.463336, 3.462995,
      3.462663, 3.462393, 3.461996, 3.461603, 3.461216, 3.460832, 3.460502,
      3.460231, 3.459864, 3.459502, 3.459146, 3.458795, 3.458449, 3.458110,
      3.457778, 3.457450, 3.457128, 3.456814, 3.456505, 3.456201, 3.455906,
      3.455616, 3.455332, 3.455051, 3.454770, 3.454486, 3.454199, 3.453917,
      3.453722, 3.453477, 3.453181, 3.452975, 3.452773, 3.452569, 3.452362,
      3.452153, 3.451943, 3.451729, 3.451573, 3.451448, 3.451226, 3.451087,
      3.450928, 3.450704, 3.450588, 3.450469, 3.450349, 3.450228, 3.450104,
      3.449997, 3.449899, 3.449808, 3.449727, 3.449649, 3.449579, 3.449512,
      3.449447, 3.449384, 3.449321, 3.449255, 3.449190, 3.449122, 3.449054,
      3.449015, 3.449015, 3.448995, 3.448921, 3.448861, 3.448861, 3.448861,
      3.448861, 3.448861, 3.448861, 3.448828, 3.448743, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706, 3.448706,
      3.448706, 3.448706, 3.448733, 3.448861, 3.448861, 3.448861, 3.448861,
      3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861,
      3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861,
      3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861,
      3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861, 3.448861,
      3.448976, 3.449015, 3.449015, 3.449015, 3.449015, 3.449015, 3.449015,
      3.449015, 3.449015, 3.449015, 3.449015, 3.449015, 3.449015, 3.449015,
      3.449015, 3.449015, 3.449015, 3.449015, 3.449015, NAN,      NAN,
      3.470475, 3.470486, 3.470557, 3.470577, 3.470703, 3.470704, 3.470825,
      3.470968, 3.470998, 3.471127, 3.471145, 3.471314, 3.471438, 3.471535,
      3.471586, 3.471795, 3.471878, 3.472025, 3.472200, 3.472318, 3.472590,
      3.472739, 3.472907, 3.473098, 3.473341, 3.473535, 3.473820, 3.474162,
      3.474475, 3.474834, 3.475220, 3.475543, 3.475910, 3.476065, 3.476107,
      3.476146, 3.476252, 3.476252, 3.476354, 3.476473, 3.476593, 3.476711,
      3.476826, 3.477036, 3.477177, 3.477272, 3.477412, 3.477505, 3.477618,
      3.477755, 3.477906, 3.478085, 3.478230, 3.478362, 3.478493, 3.478642,
      3.478814, 3.478952, 3.479079, 3.479204, 3.479330, 3.479461, 3.479625,
      3.479824, 3.480037, 3.480158, 3.480278, 3.480501, 3.480660, 3.480778,
      3.480921, 3.481154, 3.481271, 3.481473, 3.481644, 3.481786, 3.482013,
      3.482127, 3.482319, 3.482542, 3.482764, 3.482985, 3.483111, 3.483281,
      3.483499, 3.483716, 3.483933, 3.484149, 3.484467, 3.484720, 3.484934,
      3.485147, 3.485463, 3.485714, 3.485956, 3.486273, 3.486318, 3.486423,
      3.486430, 3.486430, 3.486430, 3.486430, 3.486430, 3.486526, 3.486572,
      3.486572, 3.486572, 3.486673, 3.486714, 3.486714, 3.486763, 3.486855,
      3.486855, 3.486919, 3.486997, 3.487046, 3.487138, 3.487138, 3.487197,
      3.487308, 3.487421, 3.487421, 3.487512, 3.487652, 3.487704, 3.487727,
      3.487821, 3.487986, 3.488080, 3.488174, 3.488268, 3.488291, 3.488384,
      3.488478, 3.488551, 3.488593, 3.488686, 3.488692, 3.488769, 3.488893,
      3.488974, 3.488974, 3.488974, 3.489051, 3.489114, 3.489165, 3.489255,
      3.489255, 3.489344, 3.489396, 3.489396, 3.489433, 3.489525, 3.489537,
      3.489596, 3.489677, 3.489677, 3.489677, 3.489760, 3.489818, 3.489818,
      3.489849, 3.489939, 3.489958, 3.489958, 3.489998, 3.490088, 3.490099,
      3.490099, 3.490145, 3.490234, 3.490253, 3.490341, 3.490380, 3.490380,
      3.490413, 3.490485, 3.490397, 3.490380, 3.490343, 3.490204, 3.490116,
      3.490099, 3.490012, 3.489958, 3.489908, 3.489822, 3.489805, 3.489719,
      3.489583, 3.489537, 3.489499, 3.489411, 3.489396, 3.489306, 3.489202,
      3.489114, 3.489092, 3.488984, 3.488874, 3.488833, 3.488783, 3.488665,
      3.488557, 3.488439, 3.488410, 3.488352, 3.488239, 3.488127, 3.488011,
      3.487898, 3.487785, 3.487671, 3.487563, 3.487563, 3.487472, 3.487358,
      3.487245, 3.487132, 3.487018, 3.486905, 3.486791, 3.486678, 3.486564,
      3.486451, 3.486337, 3.486223, 3.486111, 3.486001, 3.485805, 3.485721,
      3.485618, 3.485432, 3.485349, 3.485239, 3.485074, 3.484910, 3.484869,
      3.484725, 3.484561, 3.484398, 3.484234, 3.484071, 3.483908, 3.483809,
      3.483724, 3.483562, 3.483399, 3.483237, 3.483075, 3.482861, 3.482618,
      3.482446, 3.482285, 3.482123, 3.481962, 3.481802, 3.481641, 3.481376,
      3.481178, 3.481019, 3.480850, 3.480557, 3.480399, 3.480241, 3.480017,
      3.479784, 3.479627, 3.479368, 3.479172, 3.478956, 3.478724, 3.478567,
      3.478266, 3.478039, 3.477823, 3.477646, 3.477364, 3.477141, 3.476901,
      3.476647, 3.476462, 3.476193, 3.475978, 3.475746, 3.475464, 3.475209,
      3.475051, 3.474775, 3.474503, 3.474232, 3.473964, 3.473812, 3.473583,
      3.473324, 3.473068, 3.472815, 3.472565, 3.472319, 3.472076, 3.471837,
      3.471602, 3.471370, 3.471141, 3.470918, 3.470698, 3.470482, 3.470270,
      3.469965, 3.469715, 3.469515, 3.469323, 3.469134, 3.468948, 3.468761,
      3.468572, 3.468382, 3.468189, 3.467995, 3.467800, 3.467602, 3.467431,
      3.467330, 3.467144, 3.466977, 3.466873, 3.466767, 3.466660, 3.466553,
      3.466443, 3.466332, 3.466220, 3.466107, 3.465992, 3.465875, 3.465756,
      3.465680, 3.465663, 3.465540, 3.465433, 3.465383, 3.465383, 3.465314,
      3.465236, 3.465168, 3.465101, 3.465040, 3.464981, 3.464936, 3.464936,
      3.464936, 3.464911, 3.464854, 3.464798, 3.464788, 3.464788, 3.464788,
      3.464788, 3.464788, 3.464788, 3.464788, 3.464749, 3.464681, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639, 3.464639,
      3.464639, 3.464698, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788,
      3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788,
      3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788,
      3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788,
      3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788, 3.464788,
      3.464788, 3.464788, 3.464788, 3.464788, 3.464792, 3.464914, 3.464936,
      3.464936, 3.464936, 3.464936, 3.464936, 3.464936, 3.464936, 3.464936,
      3.464936, 3.464936, 3.464936, 3.464936, 3.464936, 3.464936, 3.464936,
      3.464936, 3.464936, 3.464936, NAN,      NAN,      3.478855, 3.478855,
      3.478927, 3.478999, 3.478999, 3.478999, 3.479143, 3.479143, 3.479221,
      3.479287, 3.479302, 3.479401, 3.479543, 3.479575, 3.479643, 3.479760,
      3.479863, 3.479977, 3.480100, 3.480218, 3.480329, 3.480435, 3.480554,
      3.480763, 3.480934, 3.481093, 3.481334, 3.481566, 3.481798, 3.481949,
      3.482016, 3.481997, 3.481882, 3.481872, 3.481872, 3.481872, 3.481878,
      3.481967, 3.482016, 3.482016, 3.482051, 3.482159, 3.482159, 3.482218,
      3.482342, 3.482445, 3.482446, 3.482567, 3.482688, 3.482807, 3.482908,
      3.482987, 3.483162, 3.483348, 3.483463, 3.483578, 3.483692, 3.483880,
      3.484060, 3.484187, 3.484409, 3.484608, 3.484718, 3.484929, 3.485148,
      3.485365, 3.485581, 3.485689, 3.485871, 3.486086, 3.486300, 3.486512,
      3.486801, 3.487078, 3.487289, 3.487499, 3.487712, 3.488026, 3.488268,
      3.488476, 3.488684, 3.489032, 3.489238, 3.489538, 3.489846, 3.490153,
      3.490460, 3.490683, 3.490972, 3.491304, 3.491607, 3.491909, 3.492211,
      3.492512, 3.492795, 3.492947, 3.493040, 3.493040, 3.493040, 3.493089,
      3.493179, 3.493179, 3.493179, 3.493179, 3.493179, 3.493230, 3.493319,
      3.493319, 3.493319, 3.493319, 3.493361, 3.493457, 3.493458, 3.493458,
      3.493458, 3.493487, 3.493582, 3.493597, 3.493597, 3.493597, 3.493610,
      3.493703, 3.493737, 3.493737, 3.493774, 3.493867, 3.493890, 3.493982,
      3.494015, 3.494039, 3.494155, 3.494155, 3.494302, 3.494393, 3.494433,
      3.494438, 3.494596, 3.494686, 3.494776, 3.494850, 3.494886, 3.494976,
      3.495135, 3.495224, 3.495313, 3.495402, 3.495436, 3.495613, 3.495683,
      3.495756, 3.495844, 3.495932, 3.496080, 3.496178, 3.496238, 3.496284,
      3.496372, 3.496529, 3.496617, 3.496705, 3.496795, 3.496950, 3.497038,
      3.497068, 3.497144, 3.497231, 3.497319, 3.497344, 3.497425, 3.497543,
      3.497669, 3.497757, 3.497775, 3.497862, 3.497950, 3.498035, 3.498035,
      3.498035, 3.498035, 3.498035, 3.497969, 3.497854, 3.497759, 3.497759,
      3.497654, 3.497621, 3.497575, 3.497483, 3.497483, 3.497384, 3.497344,
      3.497295, 3.497206, 3.497206, 3.497095, 3.497068, 3.497015, 3.496902,
      3.496791, 3.496791, 3.496705, 3.496653, 3.496623, 3.496515, 3.496515,
      3.496428, 3.496376, 3.496345, 3.496238, 3.496238, 3.496150, 3.496099,
      3.496066, 3.495960, 3.495960, 3.495871, 3.495822, 3.495817, 3.495723,
      3.495630, 3.495544, 3.495526, 3.495444, 3.495406, 3.495296, 3.495267,
      3.495248, 3.495106, 3.495025, 3.494989, 3.494932, 3.494851, 3.494701,
      3.494620, 3.494572, 3.494528, 3.494447, 3.494299, 3.494286, 3.494205,
      3.494095, 3.493975, 3.493895, 3.493753, 3.493665, 3.493597, 3.493554,
      3.493395, 3.493319, 3.493217, 3.493119, 3.493041, 3.492892, 3.492814,
      3.492711, 3.492621, 3.492540, 3.492386, 3.492341, 3.492217, 3.492133,
      3.492052, 3.491900, 3.491749, 3.491642, 3.491588, 3.491439, 3.491290,
      3.491142, 3.490995, 3.490849, 3.490704, 3.490559, 3.490416, 3.490274,
      3.490134, 3.489993, 3.489751, 3.489577, 3.489441, 3.489306, 3.489133,
      3.488936, 3.488771, 3.488643, 3.488482, 3.488250, 3.488127, 3.488007,
      3.487789, 3.487629, 3.487514, 3.487381, 3.487160, 3.487041, 3.486935,
      3.486805, 3.486602, 3.486488, 3.486391, 3.486296, 3.486120, 3.485970,
      3.485877, 3.485783, 3.485688, 3.485592, 3.485412, 3.485256, 3.485158,
      3.485058, 3.485011, 3.484997, 3.484894, 3.484790, 3.484685, 3.484585,
      3.484585, 3.484503, 3.484442, 3.484422, 3.484308, 3.484300, 3.484217,
      3.484157, 3.484120, 3.484015, 3.484015, 3.484015, 3.483957, 3.483872,
      3.483872, 3.483872, 3.483872, 3.483865, 3.483806, 3.483748, 3.483730,
      3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483695, 3.483648,
      3.483602, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587, 3.483587,
      3.483722, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730,
      3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730,
      3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730,
      3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730,
      3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730,
      3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483730,
      3.483730, 3.483730, 3.483730, 3.483730, 3.483730, 3.483829, 3.483872,
      3.483872, 3.483872, 3.483872, 3.483872, 3.483872, 3.483872, 3.483872,
      3.483872, 3.483872, 3.483872, 3.483872, 3.483872, 3.483872, 3.483872,
      3.483872, 3.483872, 3.483872, 3.483872, 3.483872, 3.483872, 3.483933,
      3.484015, NAN,      NAN,      3.488269, 3.488269, 3.488269, 3.488269,
      3.488269, 3.488269, 3.488269, 3.488269, 3.488344, 3.488410, 3.488410,
      3.488410, 3.488410, 3.488410, 3.488410, 3.488410, 3.488416, 3.488533,
      3.488551, 3.488551, 3.488595, 3.488692, 3.488692, 3.488794, 3.488758,
      3.488656, 3.488569, 3.488551, 3.488459, 3.488358, 3.488273, 3.488269,
      3.488168, 3.488127, 3.488127, 3.488127, 3.488116, 3.488037, 3.487986,
      3.487986, 3.487986, 3.487986, 3.487986, 3.488032, 3.488127, 3.488127,
      3.488222, 3.488269, 3.488269, 3.488335, 3.488442, 3.488549, 3.488654,
      3.488759, 3.488864, 3.488968, 3.489071, 3.489234, 3.489347, 3.489503,
      3.489692, 3.489794, 3.489971, 3.490173, 3.490373, 3.490573, 3.490773,
      3.490972, 3.491170, 3.491368, 3.491565, 3.491762, 3.491958, 3.492247,
      3.492490, 3.492717, 3.493009, 3.493214, 3.493498, 3.493741, 3.493965,
      3.494255, 3.494544, 3.494793, 3.495053, 3.495342, 3.495630, 3.495918,
      3.496241, 3.496562, 3.496869, 3.497251, 3.497495, 3.497876, 3.498090,
      3.498185, 3.498280, 3.498311, 3.498400, 3.498448, 3.498448, 3.498475,
      3.498569, 3.498586, 3.498617, 3.498709, 3.498724, 3.498724, 3.498779,
      3.498862, 3.498862, 3.498862, 3.498999, 3.498999, 3.498999, 3.499007,
      3.499137, 3.499137, 3.499205, 3.499275, 3.499275, 3.499275, 3.499357,
      3.499412, 3.499412, 3.499412, 3.499439, 3.499528, 3.499617, 3.499687,
      3.499687, 3.499687, 3.499824, 3.499824, 3.499874, 3.499962, 3.499982,
      3.500071, 3.500099, 3.500179, 3.500236, 3.500236, 3.500374, 3.500462,
      3.500511, 3.500569, 3.500656, 3.500743, 3.500876, 3.500986, 3.501059,
      3.501124, 3.501297, 3.501402, 3.501506, 3.501643, 3.501729, 3.501883,
      3.501969, 3.502092, 3.502209, 3.502298, 3.502469, 3.502640, 3.502755,
      3.502845, 3.503015, 3.503186, 3.503246, 3.503391, 3.503545, 3.503649,
      3.503773, 3.503918, 3.504096, 3.504247, 3.504364, 3.504471, 3.504471,
      3.504471, 3.504471, 3.504423, 3.504335, 3.504335, 3.504335, 3.504256,
      3.504199, 3.504199, 3.504199, 3.504093, 3.504063, 3.504009, 3.503927,
      3.503927, 3.503818, 3.503791, 3.503735, 3.503655, 3.503653, 3.503543,
      3.503518, 3.503461, 3.503382, 3.503379, 3.503269, 3.503246, 3.503191,
      3.503092, 3.503007, 3.502973, 3.502933, 3.502847, 3.502833, 3.502752,
      3.502700, 3.502658, 3.502577, 3.502564, 3.502484, 3.502427, 3.502427,
      3.502427, 3.502309, 3.502288, 3.502208, 3.502154, 3.502117, 3.502037,
      3.502017, 3.501946, 3.501880, 3.501856, 3.501777, 3.501744, 3.501687,
      3.501608, 3.501607, 3.501587, 3.501508, 3.501470, 3.501420, 3.501341,
      3.501332, 3.501254, 3.501156, 3.501059, 3.501021, 3.500944, 3.500922,
      3.500859, 3.500779, 3.500648, 3.500630, 3.500554, 3.500445, 3.500374,
      3.500327, 3.500252, 3.500120, 3.500036, 3.499962, 3.499953, 3.499816,
      3.499744, 3.499687, 3.499652, 3.499550, 3.499528, 3.499459, 3.499367,
      3.499230, 3.499137, 3.499098, 3.498966, 3.498862, 3.498842, 3.498714,
      3.498587, 3.498463, 3.498448, 3.498357, 3.498238, 3.498121, 3.498005,
      3.497897, 3.497897, 3.497811, 3.497704, 3.497599, 3.497497, 3.497397,
      3.497300, 3.497206, 3.497206, 3.497159, 3.497067, 3.496974, 3.496881,
      3.496786, 3.496691, 3.496653, 3.496635, 3.496537, 3.496515, 3.496475,
      3.496376, 3.496376, 3.496305, 3.496238, 3.496232, 3.496123, 3.496099,
      3.496041, 3.495960, 3.495960, 3.495960, 3.495960, 3.495878, 3.495822,
      3.495822, 3.495822, 3.495822, 3.495783, 3.495707, 3.495683, 3.495683,
      3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495665, 3.495620,
      3.495576, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544, 3.495544,
      3.495544, 3.495544, 3.495544, 3.495596, 3.495683, 3.495683, 3.495683,
      3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683,
      3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683,
      3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683,
      3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683,
      3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683,
      3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495683, 3.495755,
      3.495822, 3.495822, 3.495822, 3.495822, 3.495822, 3.495822, 3.495822,
      3.495822, 3.495822, 3.495822, 3.495822, 3.495822, 3.495822, 3.495822,
      3.495822, 3.495822, 3.495822, 3.495822, 3.495822, 3.495822, 3.495822,
      3.495859, 3.495960, 3.495960, 3.495960, 3.495960, 3.495960, 3.495960,
      NAN,      3.497206, 3.497206, 3.497312, 3.497344, 3.497344, 3.497344,
      3.497344, 3.497344, 3.497344, 3.497344, 3.497344, 3.497344, 3.497344,
      3.497344, 3.497281, 3.497206, 3.497206, 3.497206, 3.497112, 3.497068,
      3.497026, 3.496916, 3.496815, 3.496707, 3.496605, 3.496509, 3.496374,
      3.496284, 3.496174, 3.496043, 3.495898, 3.495727, 3.495590, 3.495466,
      3.495342, 3.495236, 3.495156, 3.495051, 3.494989, 3.494955, 3.494839,
      3.494725, 3.494711, 3.494711, 3.494665, 3.494572, 3.494572, 3.494572,
      3.494572, 3.494572, 3.494572, 3.494572, 3.494572, 3.494572, 3.494588,
      3.494711, 3.494717, 3.494820, 3.494922, 3.494989, 3.494989, 3.495086,
      3.495245, 3.495356, 3.495455, 3.495554, 3.495652, 3.495817, 3.496013,
      3.496153, 3.496263, 3.496457, 3.496650, 3.496843, 3.497035, 3.497237,
      3.497524, 3.497747, 3.497937, 3.498173, 3.498454, 3.498643, 3.498941,
      3.499238, 3.499520, 3.499802, 3.500015, 3.500296, 3.500644, 3.500970,
      3.501254, 3.501628, 3.502001, 3.502318, 3.502609, 3.502873, 3.502966,
      3.503059, 3.503109, 3.503175, 3.503246, 3.503246, 3.503383, 3.503475,
      3.503518, 3.503524, 3.503655, 3.503655, 3.503725, 3.503791, 3.503838,
      3.503928, 3.504018, 3.504063, 3.504063, 3.504082, 3.504171, 3.504199,
      3.504281, 3.504369, 3.504458, 3.504471, 3.504471, 3.504565, 3.504607,
      3.504626, 3.504713, 3.504743, 3.504820, 3.504878, 3.504926, 3.505013,
      3.505014, 3.505086, 3.505150, 3.505161, 3.505286, 3.505286, 3.505347,
      3.505421, 3.505452, 3.505538, 3.505557, 3.505557, 3.505629, 3.505693,
      3.505698, 3.505784, 3.505828, 3.505888, 3.505984, 3.506099, 3.506099,
      3.506164, 3.506265, 3.506403, 3.506489, 3.506507, 3.506592, 3.506715,
      3.506830, 3.506916, 3.507001, 3.507133, 3.507262, 3.507362, 3.507454,
      3.507605, 3.507703, 3.507918, 3.508059, 3.508163, 3.508300, 3.508516,
      3.508628, 3.508804, 3.509021, 3.509223, 3.509333, 3.509337, 3.509337,
      3.509337, 3.509337, 3.509337, 3.509337, 3.509337, 3.509337, 3.509337,
      3.509337, 3.509337, 3.509337, 3.509337, 3.509337, 3.509337, 3.509337,
      3.509337, 3.509281, 3.509203, 3.509203, 3.509203, 3.509133, 3.509068,
      3.509068, 3.509068, 3.509008, 3.508934, 3.508934, 3.508929, 3.508848,
      3.508799, 3.508753, 3.508672, 3.508659, 3.508578, 3.508530, 3.508530,
      3.508530, 3.508395, 3.508395, 3.508339, 3.508260, 3.508260, 3.508260,
      3.508183, 3.508126, 3.508060, 3.507991, 3.507991, 3.507991, 3.507913,
      3.507856, 3.507823, 3.507745, 3.507721, 3.507657, 3.507586, 3.507586,
      3.507560, 3.507483, 3.507451, 3.507451, 3.507451, 3.507316, 3.507316,
      3.507316, 3.507258, 3.507181, 3.507181, 3.507078, 3.507046, 3.507046,
      3.507038, 3.506911, 3.506911, 3.506911, 3.506858, 3.506776, 3.506776,
      3.506699, 3.506640, 3.506596, 3.506526, 3.506505, 3.506505, 3.506406,
      3.506370, 3.506370, 3.506275, 3.506234, 3.506150, 3.506099, 3.506065,
      3.506003, 3.505964, 3.505933, 3.505814, 3.505696, 3.505693, 3.505603,
      3.505557, 3.505518, 3.505411, 3.505306, 3.505286, 3.505239, 3.505150,
      3.505150, 3.505089, 3.504998, 3.504907, 3.504878, 3.504859, 3.504766,
      3.504743, 3.504713, 3.504617, 3.504607, 3.504557, 3.504471, 3.504471,
      3.504471, 3.504426, 3.504335, 3.504335, 3.504335, 3.504275, 3.504199,
      3.504199, 3.504199, 3.504199, 3.504120, 3.504063, 3.504063, 3.504063,
      3.504063, 3.504063, 3.504063, 3.504063, 3.504027, 3.503961, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927, 3.503927,
      3.503927, 3.503927, 3.503933, 3.504063, 3.504063, 3.504063, 3.504063,
      3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063,
      3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063,
      3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063,
      3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063, 3.504063,
      3.504063, 3.504063, 3.504063, 3.504063, 3.504138, 3.504199, 3.504199,
      3.504199, 3.504199, 3.504199, 3.504199, 3.504199, 3.504199, 3.504199,
      3.504199, 3.504199, 3.504199, 3.504199, 3.504199, 3.504199, 3.504199,
      3.504199, 3.504207, 3.504318, 3.504335, 3.504335, 3.504335, 3.504335,
      3.504335, 3.504335, 3.504335, 3.504335, NAN,      NAN,      3.508664,
      3.508664, 3.508664, 3.508664, 3.508664, 3.508627, 3.508530, 3.508530,
      3.508467, 3.508395, 3.508392, 3.508271, 3.508152, 3.508034, 3.507941,
      3.507863, 3.507856, 3.507772, 3.507659, 3.507546, 3.507451, 3.507451,
      3.507349, 3.507240, 3.507132, 3.507025, 3.506919, 3.506814, 3.506643,
      3.506470, 3.506367, 3.506265, 3.506164, 3.506027, 3.505896, 3.505796,
      3.505698, 3.505600, 3.505503, 3.505421, 3.505378, 3.505283, 3.505188,
      3.505093, 3.505014, 3.504974, 3.504881, 3.504878, 3.504831, 3.504743,
      3.504743, 3.504743, 3.504670, 3.504607, 3.504607, 3.504607, 3.504607,
      3.504607, 3.504607, 3.504607, 3.504607, 3.504607, 3.504692, 3.504743,
      3.504800, 3.504878, 3.504878, 3.504976, 3.505083, 3.505150, 3.505189,
      3.505276, 3.505439, 3.505557, 3.505650, 3.505822, 3.505859, 3.506030,
      3.506150, 3.506238, 3.506409, 3.506580, 3.506751, 3.506922, 3.507093,
      3.507263, 3.507434, 3.507604, 3.507774, 3.507988, 3.508243, 3.508407,
      3.508492, 3.508623, 3.508664, 3.508664, 3.508664, 3.508761, 3.508799,
      3.508828, 3.508934, 3.508934, 3.508934, 3.509014, 3.509068, 3.509114,
      3.509198, 3.509281, 3.509337, 3.509381, 3.509464, 3.509480, 3.509563,
      3.509606, 3.509662, 3.509750, 3.509874, 3.509874, 3.509979, 3.510077,
      3.510160, 3.510242, 3.510277, 3.510341, 3.510411, 3.510439, 3.510521,
      3.510545, 3.510619, 3.510725, 3.510851, 3.510933, 3.510949, 3.511031,
      3.511081, 3.511128, 3.511211, 3.511349, 3.511349, 3.511390, 3.511472,
      3.511620, 3.511702, 3.511750, 3.511799, 3.511881, 3.512029, 3.512111,
      3.512151, 3.512207, 3.512294, 3.512437, 3.512519, 3.512551, 3.512631,
      3.512740, 3.512852, 3.512951, 3.513070, 3.513180, 3.513286, 3.513393,
      3.513500, 3.513611, 3.513716, 3.513821, 3.513927, 3.514051, 3.514263,
      3.514380, 3.514488, 3.514642, 3.514834, 3.514940, 3.515047, 3.515230,
      3.515443, 3.515632, 3.515739, 3.515845, 3.515874, 3.515874, 3.515849,
      3.515742, 3.515741, 3.515741, 3.515741, 3.515741, 3.515741, 3.515741,
      3.515741, 3.515741, 3.515741, 3.515741, 3.515787, 3.515870, 3.515874,
      3.515874, 3.515874, 3.515874, 3.515874, 3.515874, 3.515874, 3.515874,
      3.515874, 3.515874, 3.515874, 3.515874, 3.515874, 3.515874, 3.515874,
      3.515874, 3.515874, 3.515874, 3.515874, 3.515874, 3.515874, 3.515874,
      3.515874, 3.515874, 3.515829, 3.515751, 3.515741, 3.515741, 3.515741,
      3.515706, 3.515628, 3.515609, 3.515607, 3.515530, 3.515476, 3.515476,
      3.515476, 3.515369, 3.515344, 3.515344, 3.515344, 3.515319, 3.515244,
      3.515211, 3.515161, 3.515087, 3.515079, 3.515072, 3.514999, 3.514946,
      3.514946, 3.514946, 3.514946, 3.514862, 3.514813, 3.514813, 3.514813,
      3.514698, 3.514681, 3.514681, 3.514681, 3.514681, 3.514551, 3.514548,
      3.514548, 3.514548, 3.514429, 3.514415, 3.514415, 3.514415, 3.514415,
      3.514415, 3.514358, 3.514282, 3.514282, 3.514282, 3.514282, 3.514282,
      3.514220, 3.514149, 3.514149, 3.514149, 3.514149, 3.514120, 3.514027,
      3.514016, 3.513981, 3.513891, 3.513883, 3.513883, 3.513883, 3.513791,
      3.513750, 3.513750, 3.513750, 3.513675, 3.513617, 3.513617, 3.513617,
      3.513617, 3.513573, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484,
      3.513484, 3.513484, 3.513484, 3.513382, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513326,
      3.513288, 3.513250, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218,
      3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218,
      3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218,
      3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218,
      3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218,
      3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513218,
      3.513218, 3.513218, 3.513218, 3.513218, 3.513218, 3.513253, 3.513324,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351, 3.513351,
      3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484,
      3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484,
      3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484,
      3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513484, 3.513507,
      3.513617, 3.513617, 3.513617, 3.513617, 3.513617, 3.513617, 3.513617,
      3.513617, 3.513617, 3.513617, 3.513617, 3.513617, 3.513617, 3.513617,
      3.513617, 3.513617, 3.513713, 3.513750, 3.513750, 3.513750, 3.513750,
      3.513750, 3.513750, 3.513750, 3.513750, 3.513750, 3.513750, 3.513750,
      3.513750, 3.513777, NAN,      NAN,      3.535065, 3.535046, 3.534859,
      3.534787, 3.534811, 3.534903, 3.534995, 3.535041, 3.535114, 3.535205,
      3.535295, 3.535385, 3.535421, 3.535500, 3.535547, 3.535547, 3.535547,
      3.535547, 3.535624, 3.535674, 3.535674, 3.535674, 3.535674, 3.535674,
      3.535674, 3.535674, 3.535674, 3.535674, 3.535629, 3.535547, 3.535547,
      3.535547, 3.535485, 3.535421, 3.535383, 3.535300, 3.535167, 3.535167,
      3.535118, 3.535031, 3.534891, 3.534810, 3.534670, 3.534584, 3.534503,
      3.534423, 3.534277, 3.534116, 3.533956, 3.533847, 3.533762, 3.533603,
      3.533443, 3.533284, 3.533125, 3.532988, 3.532909, 3.532674, 3.532560,
      3.532463, 3.532274, 3.532039, 3.531866, 3.531711, 3.531555, 3.531400,
      3.531351, 3.531219, 3.531066, 3.530912, 3.530759, 3.530712, 3.530581,
      3.530429, 3.530277, 3.530163, 3.530087, 3.529952, 3.529801, 3.529687,
      3.529629, 3.529479, 3.529430, 3.529309, 3.529174, 3.529139, 3.528990,
      3.528917, 3.528821, 3.528731, 3.528653, 3.528518, 3.528444, 3.528338,
      3.528232, 3.528158, 3.528024, 3.527947, 3.527858, 3.527759, 3.527726,
      3.527652, 3.527528, 3.527441, 3.527372, 3.527345, 3.527221, 3.527148,
      3.527114, 3.527066, 3.526992, 3.526984, 3.526911, 3.526856, 3.526856,
      3.526820, 3.526747, 3.526727, 3.526727, 3.526727, 3.526727, 3.526727,
      3.526727, 3.526727, 3.526727, 3.526727, 3.526727, 3.526727, 3.526727,
      3.526766, 3.526856, 3.526856, 3.526856, 3.526958, 3.526985, 3.527031,
      3.527144, 3.527243, 3.527243, 3.527331, 3.527432, 3.527536, 3.527638,
      3.527742, 3.527849, 3.527952, 3.528055, 3.528159, 3.528262, 3.528368,
      3.528470, 3.528573, 3.528695, 3.528901, 3.529012, 3.529185, 3.529346,
      3.529469, 3.529674, 3.529783, 3.529957, 3.530117, 3.530240, 3.530446,
      3.530651, 3.530856, 3.531015, 3.531138, 3.531339, 3.531519, 3.531692,
      3.531859, 3.532024, 3.532248, 3.532461, 3.532612, 3.532764, 3.532932,
      3.533159, 3.533345, 3.533338, 3.533264, 3.533264, 3.533264, 3.533264,
      3.533213, 3.533138, 3.533136, 3.533136, 3.533136, 3.533091, 3.533016,
      3.533009, 3.533009, 3.532955, 3.532882, 3.532882, 3.532882, 3.532882,
      3.532845, 3.532754, 3.532754, 3.532754, 3.532754, 3.532683, 3.532627,
      3.532627, 3.532627, 3.532527, 3.532500, 3.532500, 3.532500, 3.532500,
      3.532428, 3.532372, 3.532372, 3.532372, 3.532372, 3.532372, 3.532317,
      3.532245, 3.532245, 3.532245, 3.532245, 3.532245, 3.532245, 3.532245,
      3.532245, 3.532245, 3.532159, 3.532117, 3.532117, 3.532239, 3.532245,
      3.532245, 3.532245, 3.532245, 3.532245, 3.532245, 3.532245, 3.532245,
      3.532276, 3.532341, 3.532372, 3.532372, 3.532372, 3.532406, 3.532468,
      3.532500, 3.532500, 3.532549, 3.532627, 3.532643, 3.532701, 3.532754,
      3.532754, 3.532754, 3.532754, 3.532828, 3.532882, 3.532882, 3.532882,
      3.532882, 3.532964, 3.533009, 3.533009, 3.533009, 3.533009, 3.533009,
      3.533009, 3.533094, 3.533136, 3.533136, 3.533136, 3.533136, 3.533136,
      3.533136, 3.533209, 3.533264, 3.533264, 3.533264, 3.533264, 3.533264,
      3.533264, 3.533264, 3.533264, 3.533264, 3.533264, 3.533264, 3.533264,
      3.533366, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391, 3.533391,
      3.533405, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518, 3.533518,
      3.533518, NAN,      3.560405, 3.560397, 3.560385, 3.560338, 3.560265,
      3.560265, 3.560190, 3.560146, 3.560096, 3.560026, 3.560003, 3.559927,
      3.559851, 3.559764, 3.559640, 3.559564, 3.559548, 3.559474, 3.559369,
      3.559264, 3.559189, 3.559114, 3.559011, 3.558906, 3.558831, 3.558686,
      3.558538, 3.558469, 3.558363, 3.558216, 3.558089, 3.558015, 3.557896,
      3.557751, 3.557605, 3.557483, 3.557411, 3.557290, 3.557146, 3.557002,
      3.556858, 3.556714, 3.556571, 3.556428, 3.556285, 3.556162, 3.556091,
      3.555979, 3.555838, 3.555699, 3.555676, 3.555535, 3.555394, 3.555253,
      3.555112, 3.554972, 3.554832, 3.554692, 3.554552, 3.554412, 3.554272,
      3.554133, 3.553993, 3.553854, 3.553714, 3.553575, 3.553436, 3.553195,
      3.553036, 3.552897, 3.552741, 3.552533, 3.552358, 3.552134, 3.551958,
      3.551819, 3.551551, 3.551342, 3.551158, 3.550987, 3.550778, 3.550521,
      3.550358, 3.550285, 3.550203, 3.550065, 3.549926, 3.549787, 3.549739,
      3.549633, 3.549494, 3.549356, 3.549217, 3.549079, 3.548940, 3.548802,
      3.548664, 3.548526, 3.548388, 3.548250, 3.548113, 3.547975, 3.547838,
      3.547700, 3.547563, 3.547426, 3.547289, 3.547137, 3.546974, 3.546845,
      3.546750, 3.546667, 3.546465, 3.546327, 3.546219, 3.546118, 3.546019,
      3.545920, 3.545820, 3.545726, 3.545624, 3.545519, 3.545421, 3.545321,
      3.545224, 3.545126, 3.545026, 3.544925, 3.544827, 3.544725, 3.544625,
      3.544564, 3.544551, 3.544452, 3.544352, 3.544316, 3.544277, 3.544178,
      3.544078, 3.544068, 3.544068, 3.544028, 3.543944, 3.543944, 3.543853,
      3.543820, 3.543820, 3.543820, 3.543820, 3.543820, 3.543773, 3.543696,
      3.543696, 3.543766, 3.543820, 3.543820, 3.543820, 3.543820, 3.543820,
      3.543871, 3.543939, 3.543944, 3.543951, 3.544020, 3.544108, 3.544192,
      3.544192, 3.544270, 3.544316, 3.544368, 3.544437, 3.544564, 3.544584,
      3.544722, 3.544812, 3.544872, 3.545010, 3.545060, 3.545161, 3.545183,
      3.545183, 3.545166, 3.545060, 3.545060, 3.545060, 3.545060, 3.545060,
      3.545060, 3.545060, 3.545060, 3.545060, 3.545060, 3.545060, 3.545060,
      3.545060, 3.545091, 3.545158, 3.545183, 3.545183, 3.545183, 3.545183,
      3.545183, 3.545186, 3.545252, 3.545307, 3.545307, 3.545307, 3.545307,
      3.545359, 3.545431, 3.545431, 3.545431, 3.545431, 3.545520, 3.545555,
      3.545555, 3.545555, 3.545675, 3.545678, 3.545678, 3.545696, 3.545802,
      3.545818, 3.545882, 3.545925, 3.545925, 3.545953, 3.546017, 3.546113,
      3.546172, 3.546172, 3.546209, 3.546271, 3.546372, 3.546419, 3.546457,
      3.546517, 3.546613, 3.546732, 3.546789, 3.546844, 3.546913, 3.546950,
      3.547063, 3.547159, 3.547162, 3.547271, 3.547377, 3.547483, 3.547529,
      3.547565, 3.547665, 3.547764, 3.547860, 3.547954, 3.548047, 3.548138,
      3.548144, 3.548190, 3.548278, 3.548367, 3.548457, 3.548512, 3.548516,
      3.548608, 3.548702, 3.548758, 3.548768, 3.548864, 3.548881, 3.548881,
      3.548888, 3.548960, 3.549003, 3.549003, 3.549003, 3.549003, 3.549003,
      3.549003, 3.549003, 3.549003, 3.548924, 3.548881, 3.548881, 3.548881,
      3.548829, 3.548758, 3.548758, 3.548706, 3.548635, 3.548573, 3.548512,
      3.548512, 3.548419, 3.548389, 3.548267, 3.548267, 3.548247, 3.548180,
      3.548095, 3.548021, 3.548021, 3.547950, 3.547876, 3.547810, 3.547775,
      3.547775, 3.547759, 3.547709, 3.547659, 3.547615, 3.547571, 3.547529,
      3.547529, 3.547529, 3.547529, 3.547500, 3.547466, 3.547432, 3.547399,
      3.547368, 3.547338, 3.547307, 3.547282, 3.547282, 3.547282, 3.547282,
      3.547282, 3.547257, 3.547226, 3.547195, 3.547165, 3.547132, 3.547099,
      3.547066, 3.547036, 3.547036, 3.547036, 3.547036, 3.547014, 3.546977,
      3.546939, 3.546913, 3.546913, 3.546913, 3.546904, 3.546862, 3.546820,
      3.546789, 3.546789, 3.546789, 3.546789, 3.546789, 3.546789, 3.546789,
      3.546789, 3.546773, 3.546722, 3.546671, 3.546666, 3.546666, 3.546666,
      3.546666, 3.546666, 3.546666, 3.546666, 3.546666, 3.546652, 3.546590,
      3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543,
      3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543,
      3.546543, 3.546530, 3.546445, 3.546419, 3.546419, 3.546419, 3.546419,
      3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419,
      3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419,
      3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419,
      3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419,
      3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419, 3.546419,
      3.546427, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543,
      3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543,
      3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543,
      3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543, 3.546543,
      3.546543, 3.546543, 3.546543, 3.546629, 3.546666, 3.546666, 3.546666,
      3.546666, 3.546666, 3.546666, 3.546666, 3.546666, 3.546666, 3.546666,
      3.546666, 3.546666, 3.546666, 3.546666, 3.546666, 3.546666, 3.546666,
      3.546666, 3.546666, 3.546666, 3.546726, 3.546789, 3.546789, 3.546789,
      3.546789, 3.546789, 3.546789, 3.546789, 3.546789, 3.546789, 3.546789,
      3.546789, 3.546789, 3.546798, 3.546898, 3.546913, 3.546913, 3.546913,
      3.546913, 3.546913, 3.546913, 3.546913, 3.546913, NAN,      NAN,
      3.580105, 3.580091, 3.579951, 3.579812, 3.579672, 3.579544, 3.579475,
      3.579370, 3.579279, 3.579208, 3.579070, 3.578932, 3.578868, 3.578772,
      3.578635, 3.578512, 3.578443, 3.578340, 3.578204, 3.578068, 3.577942,
      3.577874, 3.577777, 3.577641, 3.577506, 3.577372, 3.577237, 3.577102,
      3.576968, 3.576834, 3.576699, 3.576680, 3.576547, 3.576413, 3.576248,
      3.576047, 3.575896, 3.575763, 3.575629, 3.575496, 3.575362, 3.575229,
      3.575096, 3.574908, 3.574714, 3.574567, 3.574368, 3.574200, 3.574068,
      3.573935, 3.573690, 3.573548, 3.573350, 3.573175, 3.573012, 3.572815,
      3.572663, 3.572532, 3.572340, 3.572132, 3.571888, 3.571692, 3.571528,
      3.571398, 3.571221, 3.571026, 3.570772, 3.570578, 3.570397, 3.570226,
      3.569994, 3.569800, 3.569646, 3.569425, 3.569167, 3.569024, 3.568778,
      3.568650, 3.568521, 3.568436, 3.568409, 3.568344, 3.568241, 3.568202,
      3.568151, 3.568086, 3.567963, 3.567899, 3.567818, 3.567732, 3.567694,
      3.567612, 3.567514, 3.567402, 3.567307, 3.567221, 3.567132, 3.566935,
      3.566827, 3.566724, 3.566623, 3.566505, 3.566316, 3.566225, 3.566056,
      3.565916, 3.565796, 3.565604, 3.565510, 3.565341, 3.565152, 3.564960,
      3.564777, 3.564682, 3.564507, 3.564315, 3.564125, 3.563937, 3.563746,
      3.563555, 3.563365, 3.563209, 3.563104, 3.562912, 3.562721, 3.562542,
      3.562358, 3.562130, 3.561933, 3.561764, 3.561594, 3.561400, 3.561251,
      3.561122, 3.560939, 3.560745, 3.560616, 3.560479, 3.560286, 3.560111,
      3.559983, 3.559855, 3.559727, 3.559599, 3.559471, 3.559344, 3.559216,
      3.558990, 3.558842, 3.558772, 3.558708, 3.558581, 3.558455, 3.558328,
      3.558202, 3.558076, 3.557969, 3.557906, 3.557819, 3.557748, 3.557688,
      3.557563, 3.557507, 3.557434, 3.557310, 3.557226, 3.557164, 3.557146,
      3.557146, 3.557099, 3.557037, 3.556923, 3.556905, 3.556905, 3.556845,
      3.556785, 3.556785, 3.556785, 3.556785, 3.556747, 3.556664, 3.556636,
      3.556571, 3.556544, 3.556544, 3.556544, 3.556544, 3.556544, 3.556544,
      3.556544, 3.556544, 3.556544, 3.556544, 3.556630, 3.556664, 3.556664,
      3.556678, 3.556800, 3.556869, 3.556905, 3.556947, 3.557016, 3.557146,
      3.557216, 3.557304, 3.557416, 3.557486, 3.557604, 3.557745, 3.557817,
      3.557906, 3.558047, 3.558188, 3.558329, 3.558470, 3.558634, 3.558872,
      3.559013, 3.559154, 3.559348, 3.559555, 3.559708, 3.559917, 3.560159,
      3.560351, 3.560593, 3.560865, 3.561021, 3.561293, 3.561563, 3.561831,
      3.562097, 3.562361, 3.562623, 3.562940, 3.563259, 3.563530, 3.563895,
      3.564206, 3.564531, 3.564859, 3.565137, 3.565490, 3.565838, 3.566183,
      3.566549, 3.566970, 3.567295, 3.567614, 3.567925, 3.568286, 3.568655,
      3.568994, 3.569320, 3.569651, 3.569938, 3.570209, 3.570525, 3.570793,
      3.571061, 3.571281, 3.571434, 3.571639, 3.571799, 3.571926, 3.571998,
      3.572058, 3.572058, 3.572058, 3.572058, 3.572016, 3.571947, 3.571812,
      3.571669, 3.571525, 3.571331, 3.571104, 3.570827, 3.570600, 3.570369,
      3.570118, 3.569852, 3.569634, 3.569408, 3.569210, 3.569034, 3.568858,
      3.568646, 3.568484, 3.568362, 3.568251, 3.568145, 3.568045, 3.567953,
      3.567860, 3.567778, 3.567697, 3.567616, 3.567580, 3.567544, 3.567508,
      3.567454, 3.567390, 3.567327, 3.567263, 3.567234, 3.567205, 3.567177,
      3.567149, 3.567101, 3.567049, 3.566998, 3.566947, 3.566903, 3.566879,
      3.566856, 3.566832, 3.566809, 3.566779, 3.566730, 3.566680, 3.566630,
      3.566581, 3.566542, 3.566515, 3.566488, 3.566461, 3.566434, 3.566405,
      3.566375, 3.566346, 3.566317, 3.566286, 3.566254, 3.566223, 3.566191,
      3.566157, 3.566124, 3.566090, 3.566084, 3.566084, 3.566084, 3.566084,
      3.566084, 3.566084, 3.566068, 3.566028, 3.565987, 3.565966, 3.565966,
      3.565966, 3.565966, 3.565966, 3.565966, 3.565966, 3.565966, 3.565942,
      3.565893, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848,
      3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848,
      3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848,
      3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848,
      3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848,
      3.565848, 3.565848, 3.565767, 3.565730, 3.565730, 3.565730, 3.565730,
      3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730,
      3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730,
      3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730,
      3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730,
      3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730, 3.565730,
      3.565730, 3.565730, 3.565730, 3.565730, 3.565848, 3.565848, 3.565848,
      3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848, 3.565848,
      3.565848, 3.565848, 3.565848, 3.565848, 3.565966, 3.565966, 3.565966,
      3.565966, 3.565966, 3.565966, 3.565966, 3.565966, 3.565966, 3.565966,
      3.566028, 3.566084, 3.566084, 3.566084, 3.566084, 3.566084, 3.566084,
      3.566088, 3.566186, 3.566202, 3.566202, 3.566202, 3.566202, 3.566202,
      3.566290, 3.566320, 3.566320, 3.566320, 3.566325, 3.566421, 3.566437,
      3.566437, 3.566437, 3.566444, 3.566532, 3.566555, 3.566555, 3.566555,
      3.566563, 3.566646, 3.566673, 3.566673, 3.566700, 3.566788, 3.566791,
      3.566870, 3.566909, 3.566909, NAN,      NAN,      3.597260, 3.597247,
      3.597119, 3.597037, 3.596973, 3.596846, 3.596718, 3.596590, 3.596463,
      3.596335, 3.596208, 3.596081, 3.595953, 3.595826, 3.595699, 3.595555,
      3.595365, 3.595208, 3.595082, 3.594955, 3.594829, 3.594703, 3.594540,
      3.594340, 3.594214, 3.594088, 3.593963, 3.593781, 3.593601, 3.593444,
      3.593240, 3.593115, 3.592990, 3.592821, 3.592634, 3.592500, 3.592270,
      3.592145, 3.591977, 3.591757, 3.591571, 3.591427, 3.591207, 3.591013,
      3.590832, 3.590698, 3.590512, 3.590304, 3.590115, 3.589956, 3.589771,
      3.589539, 3.589397, 3.589157, 3.588927, 3.588775, 3.588528, 3.588333,
      3.588147, 3.587937, 3.587752, 3.587518, 3.587272, 3.587084, 3.586891,
      3.586644, 3.586397, 3.586200, 3.586017, 3.585771, 3.585525, 3.585279,
      3.585033, 3.584898, 3.584774, 3.584652, 3.584573, 3.584481, 3.584376,
      3.584280, 3.584188, 3.584072, 3.583891, 3.583799, 3.583703, 3.583610,
      3.583522, 3.583418, 3.583251, 3.583131, 3.583037, 3.582923, 3.582743,
      3.582653, 3.582562, 3.582423, 3.582265, 3.582171, 3.581989, 3.581878,
      3.581736, 3.581581, 3.581483, 3.581303, 3.581121, 3.580988, 3.580867,
      3.580689, 3.580537, 3.580383, 3.580228, 3.580075, 3.579868, 3.579703,
      3.579580, 3.579457, 3.579229, 3.579097, 3.578975, 3.578835, 3.578614,
      3.578475, 3.578291, 3.578131, 3.578009, 3.577854, 3.577670, 3.577446,
      3.577289, 3.577120, 3.576937, 3.576808, 3.576571, 3.576389, 3.576213,
      3.576073, 3.575844, 3.575663, 3.575500, 3.575340, 3.575144, 3.575001,
      3.574822, 3.574671, 3.574495, 3.574288, 3.574110, 3.573965, 3.573845,
      3.573724, 3.573563, 3.573382, 3.573244, 3.573122, 3.572999, 3.572876,
      3.572751, 3.572633, 3.572570, 3.572492, 3.572365, 3.572264, 3.572201,
      3.572099, 3.572058, 3.572058, 3.571943, 3.571935, 3.571870, 3.571825,
      3.571825, 3.571825, 3.571825, 3.571825, 3.571825, 3.571825, 3.571825,
      3.571825, 3.571920, 3.572000, 3.572081, 3.572197, 3.572267, 3.572385,
      3.572527, 3.572669, 3.572811, 3.572955, 3.573099, 3.573244, 3.573415,
      3.573634, 3.573797, 3.574016, 3.574267, 3.574517, 3.574745, 3.575043,
      3.575342, 3.575642, 3.575943, 3.576278, 3.576657, 3.577036, 3.577385,
      3.577788, 3.578235, 3.578616, 3.579047, 3.579574, 3.580110, 3.580645,
      3.581364, 3.581975, 3.582636, 3.583308, 3.583982, 3.584739, 3.585423,
      3.586201, 3.586995, 3.587756, 3.588573, 3.589436, 3.590315, 3.591249,
      3.592263, 3.593387, 3.594454, 3.595553, 3.596624, 3.597689, 3.598803,
      3.599919, 3.600967, 3.602052, 3.603242, 3.604384, 3.605568, 3.606593,
      3.607573, 3.608467, 3.609333, 3.610121, 3.610801, 3.611414, 3.611945,
      3.612358, 3.612671, 3.612755, 3.612784, 3.612705, 3.612457, 3.611966,
      3.611413, 3.610764, 3.610006, 3.609175, 3.608269, 3.607300, 3.606318,
      3.605393, 3.604542, 3.603824, 3.603124, 3.602551, 3.602082, 3.601760,
      3.601497, 3.601275, 3.601101, 3.600957, 3.600834, 3.600750, 3.600666,
      3.600618, 3.600581, 3.600545, 3.600511, 3.600479, 3.600446, 3.600415,
      3.600386, 3.600357, 3.600328, 3.600266, 3.600188, 3.600110, 3.600032,
      3.599969, 3.599922, 3.599875, 3.599828, 3.599781, 3.599738, 3.599695,
      3.599652, 3.599609, 3.599566, 3.599526, 3.599486, 3.599447, 3.599407,
      3.599368, 3.599325, 3.599270, 3.599215, 3.599160, 3.599106, 3.599051,
      3.599004, 3.598984, 3.598964, 3.598944, 3.598924, 3.598904, 3.598883,
      3.598861, 3.598839, 3.598817, 3.598795, 3.598771, 3.598747, 3.598723,
      3.598699, 3.598681, 3.598681, 3.598681, 3.598681, 3.598681, 3.598681,
      3.598681, 3.598681, 3.598676, 3.598646, 3.598615, 3.598585, 3.598572,
      3.598572, 3.598572, 3.598572, 3.598572, 3.598572, 3.598572, 3.598606,
      3.598642, 3.598678, 3.598681, 3.598681, 3.598681, 3.598681, 3.598681,
      3.598681, 3.598681, 3.598681, 3.598648, 3.598604, 3.598572, 3.598572,
      3.598572, 3.598572, 3.598572, 3.598572, 3.598572, 3.598572, 3.598572,
      3.598572, 3.598572, 3.598572, 3.598572, 3.598547, 3.598489, 3.598462,
      3.598462, 3.598462, 3.598462, 3.598462, 3.598462, 3.598462, 3.598462,
      3.598462, 3.598462, 3.598462, 3.598462, 3.598462, 3.598462, 3.598462,
      3.598462, 3.598462, 3.598462, 3.598462, 3.598462, 3.598525, 3.598572,
      3.598572, 3.598572, 3.598572, 3.598572, 3.598572, 3.598572, 3.598572,
      3.598572, 3.598572, 3.598572, 3.598572, 3.598597, 3.598681, 3.598681,
      3.598681, 3.598681, 3.598681, 3.598681, 3.598681, 3.598681, 3.598791,
      3.598791, 3.598791, 3.598791, 3.598791, 3.598791, 3.598863, 3.598900,
      3.598900, 3.598900, 3.598900, 3.598969, 3.599009, 3.599009, 3.599009,
      3.599009, 3.599119, 3.599119, 3.599119, 3.599119, 3.599219, 3.599228,
      3.599228, 3.599253, 3.599337, 3.599337, 3.599337, 3.599423, 3.599446,
      3.599446, 3.599538, 3.599556, 3.599556, 3.599665, 3.599665, 3.599665,
      3.599774, 3.599774, 3.599850, 3.599883, 3.599883, 3.599992, 3.599992,
      3.600101, 3.600101, 3.600171, 3.600210, 3.600233, 3.600319, 3.600346,
      3.600427, 3.600428, 3.600512, 3.600598, 3.600646, 3.600664, 3.600754,
      3.600847, 3.600925, 3.600973, 3.600992, 3.601071, 3.601169, 3.601256,
      3.601340, 3.601425, 3.601511, 3.601600, 3.601625, 3.601673, 3.601761,
      3.601840, 3.601936, 3.602021, 3.602150, 3.602298, 3.602385, 3.602474,
      3.602564, 3.602648, 3.602750, 3.602933, 3.603016, 3.603101, 3.603229,
      3.603383, NAN,      NAN,      3.610470, 3.610458, 3.610337, 3.610234,
      3.610201, 3.610080, 3.609959, 3.609838, 3.609762, 3.609702, 3.609581,
      3.609460, 3.609339, 3.609218, 3.609097, 3.608976, 3.608855, 3.608734,
      3.608613, 3.608492, 3.608371, 3.608250, 3.608129, 3.607964, 3.607782,
      3.607659, 3.607539, 3.607418, 3.607297, 3.607144, 3.606963, 3.606828,
      3.606707, 3.606587, 3.606443, 3.606238, 3.606118, 3.605937, 3.605770,
      3.605650, 3.605530, 3.605302, 3.605183, 3.605036, 3.604835, 3.604691,
      3.604512, 3.604369, 3.604208, 3.604029, 3.603797, 3.603671, 3.603493,
      3.603330, 3.603169, 3.602958, 3.602780, 3.602638, 3.602436, 3.602199,
      3.602065, 3.601837, 3.601712, 3.601535, 3.601310, 3.601130, 3.600938,
      3.600779, 3.600593, 3.600359, 3.600177, 3.600096, 3.600000, 3.599821,
      3.599713, 3.599597, 3.599436, 3.599346, 3.599173, 3.598997, 3.598860,
      3.598751, 3.598581, 3.598487, 3.598329, 3.598150, 3.597976, 3.597806,
      3.597717, 3.597561, 3.597385, 3.597212, 3.597041, 3.596872, 3.596721,
      3.596579, 3.596439, 3.596311, 3.596118, 3.595923, 3.595747, 3.595619,
      3.595501, 3.595328, 3.595157, 3.595040, 3.594912, 3.594737, 3.594579,
      3.594423, 3.594236, 3.594121, 3.593949, 3.593779, 3.593664, 3.593549,
      3.593360, 3.593208, 3.593094, 3.592980, 3.592866, 3.592662, 3.592528,
      3.592414, 3.592250, 3.592079, 3.591959, 3.591844, 3.591728, 3.591611,
      3.591494, 3.591376, 3.591258, 3.591140, 3.591020, 3.590927, 3.590867,
      3.590771, 3.590649, 3.590573, 3.590512, 3.590396, 3.590388, 3.590326,
      3.590242, 3.590145, 3.590081, 3.590061, 3.590010, 3.589950, 3.589950,
      3.589950, 3.589950, 3.589950, 3.589950, 3.589950, 3.589950, 3.589950,
      3.589950, 3.590000, 3.590074, 3.590192, 3.590261, 3.590284, 3.590403,
      3.590543, 3.590684, 3.590825, 3.590967, 3.591110, 3.591333, 3.591621,
      3.591840, 3.592094, 3.592389, 3.592684, 3.593018, 3.593392, 3.593769,
      3.594187, 3.594643, 3.595129, 3.595668, 3.596210, 3.596755, 3.597308,
      3.597937, 3.598568, 3.599204, 3.599842, 3.600503, 3.601228, 3.602047,
      3.602939, 3.603836, 3.604782, 3.605787, 3.606857, 3.607930, 3.609006,
      3.610083, 3.611167, 3.612331, 3.613626, 3.615042, 3.616518, 3.617931,
      3.619426, 3.620919, 3.622451, 3.624081, 3.625902, 3.627815, 3.629617,
      3.631436, 3.633334, 3.635311, 3.637361, 3.639326, 3.641214, 3.643131,
      3.645065, 3.646940, 3.648701, 3.650421, 3.652051, 3.653565, 3.654952,
      3.656157, 3.657114, 3.657795, 3.658346, 3.658567, 3.658443, 3.658065,
      3.657356, 3.656353, 3.655193, 3.653840, 3.652406, 3.650994, 3.649601,
      3.648400, 3.647354, 3.646375, 3.645543, 3.644936, 3.644502, 3.644155,
      3.643998, 3.643848, 3.643761, 3.643675, 3.643650, 3.643650, 3.643647,
      3.643613, 3.643580, 3.643548, 3.643518, 3.643489, 3.643460, 3.643433,
      3.643407, 3.643381, 3.643355, 3.643285, 3.643215, 3.643145, 3.643074,
      3.643025, 3.642983, 3.642941, 3.642898, 3.642856, 3.642817, 3.642779,
      3.642740, 3.642701, 3.642663, 3.642645, 3.642627, 3.642609, 3.642591,
      3.642574, 3.642556, 3.642540, 3.642523, 3.642507, 3.642490, 3.642474,
      3.642458, 3.642442, 3.642427, 3.642411, 3.642396, 3.642381, 3.642365,
      3.642351, 3.642336, 3.642322, 3.642307, 3.642293, 3.642278, 3.642267,
      3.642267, 3.642267, 3.642267, 3.642267, 3.642267, 3.642267, 3.642267,
      3.642267, 3.642267, 3.642267, 3.642267, 3.642267, 3.642267, 3.642267,
      3.642267, 3.642267, 3.642267, 3.642267, 3.642267, 3.642267, 3.642262,
      3.642238, 3.642215, 3.642191, 3.642168, 3.642168, 3.642168, 3.642168,
      3.642165, 3.642137, 3.642110, 3.642083, 3.642069, 3.642069, 3.642069,
      3.642069, 3.642069, 3.642069, 3.642069, 3.642069, 3.642069, 3.642069,
      3.642069, 3.642069, 3.642069, 3.642032, 3.641996, 3.641970, 3.641970,
      3.641970, 3.641970, 3.641970, 3.641970, 3.641970, 3.641970, 3.641970,
      3.641970, 3.641997, 3.642043, 3.642069, 3.642069, 3.642069, 3.642069,
      3.642069, 3.642069, 3.642069, 3.642069, 3.642069, 3.642069, 3.642069,
      3.642069, 3.642124, 3.642168, 3.642168, 3.642168, 3.642168, 3.642168,
      3.642168, 3.642168, 3.642232, 3.642267, 3.642267, 3.642267, 3.642267,
      3.642290, 3.642365, 3.642366, 3.642366, 3.642401, 3.642465, 3.642465,
      3.642465, 3.642465, 3.642523, 3.642563, 3.642563, 3.642595, 3.642662,
      3.642662, 3.642683, 3.642761, 3.642761, 3.642761, 3.642793, 3.642860,
      3.642909, 3.642959, 3.642959, 3.643048, 3.643058, 3.643086, 3.643156,
      3.643156, 3.643255, 3.643255, 3.643319, 3.643354, 3.643389, 3.643453,
      3.643469, 3.643551, 3.643560, 3.643650, 3.643661, 3.643749, 3.643774,
      3.643847, 3.643946, 3.643965, 3.644044, 3.644107, 3.644143, 3.644242,
      3.644246, 3.644340, 3.644439, 3.644514, 3.644537, 3.644636, 3.644734,
      3.644823, 3.644836, 3.644951, 3.645029, 3.645127, 3.645222, 3.645324,
      3.645391, 3.645432, 3.645577, 3.645627, 3.645717, 3.645815, 3.645913,
      3.646011, 3.646147, 3.646227, 3.646311, 3.646499, 3.646594, 3.646694,
      3.646796, 3.646903, 3.647026, 3.647121, 3.647283, 3.647449, 3.647535,
      3.647642, 3.647808, 3.647950, 3.648126, 3.648281, 3.648432, 3.648570,
      3.648647, 3.648801, 3.648962, 3.649127, 3.649308, 3.649521, 3.649696,
      3.649848, 3.650038, 3.650248, 3.650403, 3.650562, 3.650785, 3.651011,
      3.651215, 3.651388, 3.651611, 3.651838, 3.652069, 3.652303, NAN,
      NAN,      3.622456, 3.622450, 3.622361, 3.622244, 3.622127, 3.622110,
      3.621997, 3.621880, 3.621799, 3.621750, 3.621633, 3.621517, 3.621400,
      3.621334, 3.621271, 3.621154, 3.621037, 3.620968, 3.620908, 3.620792,
      3.620675, 3.620559, 3.620442, 3.620326, 3.620209, 3.620114, 3.620056,
      3.619895, 3.619742, 3.619626, 3.619509, 3.619392, 3.619276, 3.619159,
      3.619042, 3.618925, 3.618809, 3.618692, 3.618525, 3.618354, 3.618228,
      3.618053, 3.617900, 3.617783, 3.617632, 3.617458, 3.617329, 3.617161,
      3.616991, 3.616853, 3.616653, 3.616537, 3.616354, 3.616166, 3.616007,
      3.615833, 3.615721, 3.615558, 3.615385, 3.615206, 3.615009, 3.614797,
      3.614598, 3.614442, 3.614262, 3.614079, 3.613917, 3.613744, 3.613596,
      3.613426, 3.613247, 3.613093, 3.613010, 3.612855, 3.612680, 3.612535,
      3.612428, 3.612283, 3.612175, 3.612062, 3.611944, 3.611814, 3.611643,
      3.611485, 3.611371, 3.611236, 3.611065, 3.610923, 3.610809, 3.610660,
      3.610490, 3.610363, 3.610249, 3.610088, 3.609918, 3.609804, 3.609687,
      3.609519, 3.609360, 3.609249, 3.609137, 3.609008, 3.608841, 3.608696,
      3.608586, 3.608449, 3.608283, 3.608146, 3.608034, 3.607922, 3.607809,
      3.607696, 3.607583, 3.607468, 3.607354, 3.607239, 3.607128, 3.607070,
      3.606997, 3.606919, 3.606869, 3.606751, 3.606704, 3.606619, 3.606596,
      3.606593, 3.606489, 3.606489, 3.606489, 3.606426, 3.606381, 3.606381,
      3.606384, 3.606447, 3.606489, 3.606489, 3.606489, 3.606594, 3.606660,
      3.606746, 3.606844, 3.606910, 3.607030, 3.607096, 3.607193, 3.607372,
      3.607571, 3.607708, 3.607845, 3.608033, 3.608253, 3.608504, 3.608715,
      3.608991, 3.609275, 3.609563, 3.609862, 3.610226, 3.610592, 3.610943,
      3.611278, 3.611658, 3.612110, 3.612616, 3.613127, 3.613679, 3.614314,
      3.615014, 3.615652, 3.616325, 3.617056, 3.617856, 3.618662, 3.619472,
      3.620366, 3.621270, 3.622179, 3.623150, 3.624405, 3.625670, 3.626964,
      3.628270, 3.629639, 3.631083, 3.632559, 3.634144, 3.635876, 3.637565,
      3.639422, 3.641260, 3.643107, 3.645000, 3.646979, 3.649045, 3.651090,
      3.653155, 3.655278, 3.657454, 3.659641, 3.661889, 3.664215, 3.666672,
      3.669128, 3.671607, 3.674060, 3.676459, 3.678838, 3.681212, 3.683451,
      3.685617, 3.687635, 3.689571, 3.691440, 3.693083, 3.694397, 3.695362,
      3.695963, 3.696174, 3.695912, 3.695289, 3.694413, 3.693351, 3.692027,
      3.690647, 3.689422, 3.688283, 3.687340, 3.686533, 3.685869, 3.685334,
      3.684957, 3.684682, 3.684449, 3.684334, 3.684254, 3.684198, 3.684163,
      3.684128, 3.684067, 3.684005, 3.683946, 3.683918, 3.683891, 3.683864,
      3.683839, 3.683815, 3.683791, 3.683766, 3.683723, 3.683679, 3.683636,
      3.683593, 3.683587, 3.683587, 3.683587, 3.683587, 3.683582, 3.683565,
      3.683547, 3.683529, 3.683511, 3.683494, 3.683477, 3.683461, 3.683444,
      3.683428, 3.683412, 3.683418, 3.683434, 3.683449, 3.683464, 3.683479,
      3.683494, 3.683497, 3.683497, 3.683497, 3.683497, 3.683497, 3.683497,
      3.683491, 3.683478, 3.683465, 3.683451, 3.683438, 3.683425, 3.683411,
      3.683407, 3.683407, 3.683407, 3.683407, 3.683407, 3.683407, 3.683407,
      3.683400, 3.683388, 3.683376, 3.683364, 3.683352, 3.683340, 3.683327,
      3.683317, 3.683317, 3.683317, 3.683317, 3.683317, 3.683317, 3.683317,
      3.683307, 3.683291, 3.683276, 3.683260, 3.683245, 3.683229, 3.683227,
      3.683227, 3.683227, 3.683227, 3.683227, 3.683215, 3.683197, 3.683178,
      3.683159, 3.683140, 3.683137, 3.683137, 3.683137, 3.683137, 3.683137,
      3.683137, 3.683137, 3.683137, 3.683137, 3.683137, 3.683137, 3.683137,
      3.683137, 3.683137, 3.683137, 3.683137, 3.683137, 3.683137, 3.683137,
      3.683137, 3.683137, 3.683137, 3.683137, 3.683137, 3.683137, 3.683137,
      3.683137, 3.683137, 3.683137, 3.683137, 3.683145, 3.683180, 3.683216,
      3.683227, 3.683227, 3.683227, 3.683227, 3.683227, 3.683265, 3.683306,
      3.683317, 3.683317, 3.683317, 3.683317, 3.683341, 3.683386, 3.683407,
      3.683407, 3.683438, 3.683486, 3.683497, 3.683497, 3.683549, 3.683587,
      3.683587, 3.683620, 3.683675, 3.683677, 3.683700, 3.683758, 3.683767,
      3.683767, 3.683767, 3.683825, 3.683857, 3.683865, 3.683932, 3.683947,
      3.683980, 3.684050, 3.684122, 3.684127, 3.684181, 3.684217, 3.684244,
      3.684307, 3.684314, 3.684395, 3.684396, 3.684474, 3.684486, 3.684559,
      3.684576, 3.684650, 3.684666, 3.684748, 3.684756, 3.684845, 3.684864,
      3.684966, 3.685025, 3.685086, 3.685114, 3.685204, 3.685236, 3.685294,
      3.685376, 3.685473, 3.685525, 3.685563, 3.685652, 3.685720, 3.685760,
      3.685892, 3.685937, 3.686010, 3.686100, 3.686176, 3.686231, 3.686368,
      3.686438, 3.686502, 3.686568, 3.686726, 3.686798, 3.686873, 3.686950,
      3.687083, 3.687172, 3.687261, 3.687383, 3.687477, 3.687574, 3.687675,
      3.687796, 3.687887, 3.687998, 3.688153, 3.688242, 3.688420, 3.688509,
      3.688613, 3.688776, 3.688886, 3.689042, 3.689177, 3.689328, 3.689484,
      3.689645, 3.689753, 3.689930, 3.690066, 3.690246, 3.690430, 3.690619,
      3.690728, 3.690925, 3.691130, 3.691341, 3.691516, 3.691712, 3.691907,
      3.692146, 3.692343, 3.692543, 3.692771, 3.693038, 3.693218, 3.693457,
      3.693730, 3.693995, 3.694221, 3.694460, 3.694787, 3.695019, 3.695281,
      3.695602, 3.695812, 3.696156, 3.696469, 3.696764, 3.697071, 3.697398,
      3.697710, 3.698048, 3.698394, 3.698745, NAN,      NAN,      3.632457,
      3.632457, 3.632383, 3.632312, 3.632255, 3.632147, 3.632090, 3.632013,
      3.631898, 3.631817, 3.631760, 3.631657, 3.631595, 3.631531, 3.631417,
      3.631323, 3.631266, 3.631178, 3.631102, 3.631045, 3.630939, 3.630830,
      3.630773, 3.630700, 3.630609, 3.630552, 3.630462, 3.630349, 3.630281,
      3.630224, 3.630110, 3.629997, 3.629919, 3.629872, 3.629759, 3.629646,
      3.629532, 3.629419, 3.629357, 3.629294, 3.629180, 3.629066, 3.628953,
      3.628839, 3.628725, 3.628611, 3.628495, 3.628373, 3.628250, 3.628124,
      3.627993, 3.627856, 3.627717, 3.627620, 3.627514, 3.627351, 3.627188,
      3.627087, 3.626931, 3.626777, 3.626648, 3.626545, 3.626348, 3.626223,
      3.626098, 3.625976, 3.625864, 3.625752, 3.625597, 3.625430, 3.625313,
      3.625202, 3.625090, 3.624955, 3.624765, 3.624655, 3.624544, 3.624434,
      3.624324, 3.624214, 3.624104, 3.623914, 3.623782, 3.623673, 3.623565,
      3.623456, 3.623348, 3.623240, 3.623132, 3.623024, 3.622915, 3.622805,
      3.622696, 3.622628, 3.622579, 3.622468, 3.622356, 3.622318, 3.622235,
      3.622122, 3.622009, 3.622003, 3.621945, 3.621873, 3.621799, 3.621772,
      3.621714, 3.621695, 3.621648, 3.621592, 3.621592, 3.621574, 3.621514,
      3.621488, 3.621530, 3.621591, 3.621592, 3.621592, 3.621592, 3.621630,
      3.621693, 3.621808, 3.621871, 3.621966, 3.622050, 3.622118, 3.622248,
      3.622378, 3.622508, 3.622698, 3.622876, 3.623009, 3.623244, 3.623448,
      3.623682, 3.623955, 3.624334, 3.624611, 3.624939, 3.625305, 3.625730,
      3.626156, 3.626587, 3.627019, 3.627522, 3.628035, 3.628559, 3.629154,
      3.629754, 3.630357, 3.630965, 3.631638, 3.632405, 3.633106, 3.633935,
      3.634844, 3.635800, 3.636764, 3.637737, 3.638778, 3.639849, 3.640954,
      3.642126, 3.643309, 3.644562, 3.645914, 3.647217, 3.648609, 3.650098,
      3.651629, 3.653183, 3.654798, 3.656434, 3.658197, 3.659951, 3.661792,
      3.663812, 3.665924, 3.668080, 3.670349, 3.672655, 3.674998, 3.677400,
      3.679850, 3.682301, 3.684750, 3.687215, 3.689751, 3.692555, 3.695391,
      3.698201, 3.700913, 3.703527, 3.706247, 3.708980, 3.711675, 3.714289,
      3.716846, 3.719125, 3.721115, 3.722834, 3.724242, 3.725118, 3.725607,
      3.725561, 3.725297, 3.724722, 3.723951, 3.723062, 3.722117, 3.721206,
      3.720394, 3.719655, 3.719111, 3.718629, 3.718201, 3.717933, 3.717702,
      3.717548, 3.717409, 3.717342, 3.717274, 3.717234, 3.717204, 3.717175,
      3.717171, 3.717171, 3.717171, 3.717156, 3.717133, 3.717110, 3.717088,
      3.717088, 3.717088, 3.717088, 3.717088, 3.717069, 3.717050, 3.717032,
      3.717013, 3.716996, 3.716979, 3.716962, 3.716945, 3.716928, 3.716912,
      3.716896, 3.716881, 3.716865, 3.716850, 3.716835, 3.716820, 3.716806,
      3.716792, 3.716777, 3.716763, 3.716749, 3.716736, 3.716723, 3.716709,
      3.716696, 3.716683, 3.716671, 3.716671, 3.716671, 3.716671, 3.716671,
      3.716671, 3.716671, 3.716665, 3.716653, 3.716642, 3.716630, 3.716618,
      3.716606, 3.716594, 3.716588, 3.716588, 3.716588, 3.716588, 3.716588,
      3.716588, 3.716588, 3.716588, 3.716588, 3.716588, 3.716588, 3.716588,
      3.716588, 3.716588, 3.716588, 3.716591, 3.716602, 3.716613, 3.716624,
      3.716635, 3.716646, 3.716657, 3.716668, 3.716671, 3.716671, 3.716671,
      3.716671, 3.716671, 3.716671, 3.716671, 3.716685, 3.716699, 3.716713,
      3.716728, 3.716742, 3.716754, 3.716754, 3.716754, 3.716754, 3.716754,
      3.716754, 3.716767, 3.716784, 3.716801, 3.716818, 3.716835, 3.716838,
      3.716838, 3.716838, 3.716838, 3.716838, 3.716838, 3.716838, 3.716838,
      3.716838, 3.716838, 3.716838, 3.716838, 3.716838, 3.716838, 3.716838,
      3.716838, 3.716858, 3.716883, 3.716908, 3.716921, 3.716921, 3.716921,
      3.716930, 3.716958, 3.716985, 3.717004, 3.717004, 3.717004, 3.717019,
      3.717050, 3.717081, 3.717088, 3.717088, 3.717096, 3.717130, 3.717164,
      3.717171, 3.717171, 3.717187, 3.717224, 3.717262, 3.717300, 3.717338,
      3.717338, 3.717338, 3.717378, 3.717419, 3.717462, 3.717504, 3.717504,
      3.717512, 3.717558, 3.717605, 3.717653, 3.717702, 3.717751, 3.717754,
      3.717770, 3.717822, 3.717876, 3.717920, 3.717920, 3.717959, 3.718016,
      3.718075, 3.718134, 3.718169, 3.718174, 3.718237, 3.718301, 3.718366,
      3.718419, 3.718419, 3.718486, 3.718557, 3.718628, 3.718701, 3.718751,
      3.718768, 3.718834, 3.718840, 3.718920, 3.719002, 3.719085, 3.719169,
      3.719248, 3.719260, 3.719349, 3.719440, 3.719532, 3.719580, 3.719640,
      3.719737, 3.719828, 3.719855, 3.719959, 3.720064, 3.720159, 3.720197,
      3.720308, 3.720407, 3.720454, 3.720571, 3.720691, 3.720812, 3.720903,
      3.720980, 3.721108, 3.721157, 3.721290, 3.721426, 3.721564, 3.721704,
      3.721811, 3.721911, 3.722058, 3.722129, 3.722283, 3.722440, 3.722600,
      3.722716, 3.722846, 3.722963, 3.723104, 3.723279, 3.723457, 3.723557,
      3.723742, 3.723930, 3.724121, 3.724316, 3.724433, 3.724553, 3.724759,
      3.724969, 3.725182, 3.725400, 3.725621, 3.725764, 3.725993, 3.726146,
      3.726384, 3.726626, 3.726872, 3.727123, 3.727297, 3.727557, 3.727822,
      3.728092, 3.728366, 3.728645, 3.728930, 3.729219, 3.729514, 3.729753,
      3.730084, 3.730431, 3.730746, 3.731068, 3.731397, 3.731730, 3.732070,
      3.732432, 3.732842, 3.733205, 3.733619, 3.734021, 3.734411, 3.734858,
      3.735315, 3.735779, 3.736251, 3.736727, 3.737206, 3.737755, 3.738268,
      3.738801, 3.739377, 3.739963, NAN,      3.641213, 3.641202, 3.641133,
      3.641075, 3.640978, 3.640948, 3.640835, 3.640779, 3.640709, 3.640597,
      3.640581, 3.640476, 3.640420, 3.640346, 3.640258, 3.640203, 3.640110,
      3.640084, 3.639986, 3.639885, 3.639862, 3.639751, 3.639686, 3.639628,
      3.639551, 3.639496, 3.639394, 3.639335, 3.639271, 3.639160, 3.639069,
      3.639013, 3.638927, 3.638852, 3.638797, 3.638695, 3.638684, 3.638573,
      3.638489, 3.638451, 3.638363, 3.638302, 3.638198, 3.638131, 3.638034,
      3.637883, 3.637803, 3.637721, 3.637637, 3.637552, 3.637464, 3.637373,
      3.637293, 3.637214, 3.637142, 3.637056, 3.636908, 3.636826, 3.636733,
      3.636622, 3.636513, 3.636488, 3.636395, 3.636286, 3.636177, 3.636069,
      3.635961, 3.635852, 3.635785, 3.635737, 3.635630, 3.635523, 3.635416,
      3.635309, 3.635242, 3.635189, 3.635088, 3.634981, 3.634926, 3.634880,
      3.634868, 3.634813, 3.634737, 3.634679, 3.634647, 3.634592, 3.634493,
      3.634477, 3.634477, 3.634457, 3.634376, 3.634376, 3.634376, 3.634376,
      3.634376, 3.634376, 3.634388, 3.634447, 3.634477, 3.634515, 3.634575,
      3.634584, 3.634645, 3.634731, 3.634816, 3.634878, 3.634998, 3.635122,
      3.635246, 3.635372, 3.635511, 3.635724, 3.635852, 3.636027, 3.636231,
      3.636464, 3.636660, 3.636912, 3.637177, 3.637442, 3.637709, 3.638000,
      3.638339, 3.638680, 3.639050, 3.639464, 3.639881, 3.640301, 3.640725,
      3.641180, 3.641679, 3.642160, 3.642740, 3.643327, 3.643917, 3.644533,
      3.645207, 3.645887, 3.646625, 3.647393, 3.648167, 3.648947, 3.649805,
      3.650731, 3.651658, 3.652585, 3.653645, 3.654716, 3.655796, 3.656947,
      3.658166, 3.659415, 3.660704, 3.662078, 3.663545, 3.665110, 3.666719,
      3.668433, 3.670196, 3.671998, 3.673883, 3.675812, 3.677794, 3.679828,
      3.681913, 3.684006, 3.686128, 3.688349, 3.690708, 3.693232, 3.695897,
      3.698601, 3.701219, 3.703911, 3.706674, 3.709607, 3.712645, 3.715753,
      3.718885, 3.722106, 3.725351, 3.728605, 3.731858, 3.735046, 3.738092,
      3.741032, 3.743727, 3.746157, 3.748096, 3.749564, 3.750649, 3.751391,
      3.751732, 3.751677, 3.751382, 3.750853, 3.750139, 3.749307, 3.748477,
      3.747673, 3.746910, 3.746203, 3.745612, 3.745134, 3.744755, 3.744417,
      3.744147, 3.743918, 3.743717, 3.743603, 3.743488, 3.743393, 3.743318,
      3.743242, 3.743179, 3.743134, 3.743089, 3.743045, 3.743004, 3.742964,
      3.742924, 3.742885, 3.742882, 3.742882, 3.742882, 3.742882, 3.742882,
      3.742882, 3.742882, 3.742882, 3.742882, 3.742882, 3.742882, 3.742882,
      3.742882, 3.742882, 3.742889, 3.742902, 3.742916, 3.742930, 3.742944,
      3.742957, 3.742990, 3.743028, 3.743066, 3.743105, 3.743143, 3.743181,
      3.743233, 3.743292, 3.743352, 3.743412, 3.743472, 3.743531, 3.743592,
      3.743659, 3.743727, 3.743795, 3.743862, 3.743930, 3.743998, 3.744066,
      3.744141, 3.744216, 3.744291, 3.744366, 3.744441, 3.744516, 3.744590,
      3.744663, 3.744734, 3.744806, 3.744877, 3.744949, 3.745021, 3.745092,
      3.745159, 3.745198, 3.745238, 3.745277, 3.745316, 3.745356, 3.745395,
      3.745434, 3.745469, 3.745488, 3.745507, 3.745526, 3.745545, 3.745564,
      3.745583, 3.745602, 3.745621, 3.745642, 3.745663, 3.745684, 3.745705,
      3.745726, 3.745747, 3.745768, 3.745784, 3.745796, 3.745808, 3.745820,
      3.745832, 3.745843, 3.745856, 3.745882, 3.745909, 3.745936, 3.745962,
      3.745989, 3.746011, 3.746011, 3.746011, 3.746011, 3.746011, 3.746011,
      3.746026, 3.746042, 3.746058, 3.746074, 3.746091, 3.746109, 3.746126,
      3.746144, 3.746161, 3.746180, 3.746199, 3.746218, 3.746237, 3.746257,
      3.746278, 3.746298, 3.746319, 3.746323, 3.746323, 3.746323, 3.746329,
      3.746353, 3.746376, 3.746399, 3.746424, 3.746449, 3.746474, 3.746500,
      3.746526, 3.746552, 3.746580, 3.746608, 3.746635, 3.746664, 3.746694,
      3.746723, 3.746754, 3.746784, 3.746816, 3.746848, 3.746881, 3.746914,
      3.746948, 3.746982, 3.747017, 3.747053, 3.747090, 3.747127, 3.747165,
      3.747203, 3.747242, 3.747282, 3.747323, 3.747365, 3.747407, 3.747450,
      3.747493, 3.747538, 3.747584, 3.747630, 3.747677, 3.747725, 3.747774,
      3.747848, 3.747949, 3.748004, 3.748056, 3.748110, 3.748165, 3.748220,
      3.748277, 3.748334, 3.748393, 3.748453, 3.748514, 3.748576, 3.748639,
      3.748704, 3.748769, 3.748836, 3.748904, 3.748983, 3.749121, 3.749193,
      3.749266, 3.749341, 3.749416, 3.749494, 3.749572, 3.749653, 3.749809,
      3.749821, 3.749979, 3.750066, 3.750153, 3.750243, 3.750334, 3.750427,
      3.750521, 3.750649, 3.750792, 3.750892, 3.750994, 3.751097, 3.751202,
      3.751310, 3.751419, 3.751607, 3.751721, 3.751836, 3.751953, 3.752073,
      3.752195, 3.752396, 3.752522, 3.752651, 3.752782, 3.752937, 3.753128,
      3.753266, 3.753407, 3.753550, 3.753733, 3.753922, 3.754073, 3.754227,
      3.754384, 3.754620, 3.754783, 3.754949, 3.755118, 3.755314, 3.755541,
      3.755719, 3.755900, 3.756085, 3.756349, 3.756540, 3.756735, 3.756933,
      3.757211, 3.757417, 3.757626, 3.757839, 3.758110, 3.758353, 3.758578,
      3.758807, 3.759040, 3.759278, 3.759595, 3.759841, 3.760091, 3.760346,
      3.760606, 3.760870, 3.761063, 3.761326, 3.761541, 3.761702, 3.761853,
      3.761940, 3.762060, 3.762182, 3.762228, 3.762228, 3.762187, 3.762023,
      3.761766, 3.761528, 3.761322, 3.761145, 3.760965, NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      3.648970, 3.648959, 3.648896, 3.648848, 3.648833,
      3.648777, 3.648692, 3.648653, 3.648610, 3.648553, 3.648450, 3.648394,
      3.648318, 3.648262, 3.648194, 3.648083, 3.648067, 3.648013, 3.647947,
      3.647854, 3.647799, 3.647714, 3.647676, 3.647592, 3.647482, 3.647470,
      3.647361, 3.647285, 3.647240, 3.647158, 3.647102, 3.647089, 3.647016,
      3.646935, 3.646860, 3.646793, 3.646698, 3.646698, 3.646633, 3.646548,
      3.646476, 3.646409, 3.646404, 3.646332, 3.646251, 3.646208, 3.646198,
      3.646119, 3.646042, 3.646011, 3.645979, 3.645902, 3.645844, 3.645815,
      3.645785, 3.645732, 3.645717, 3.645635, 3.645575, 3.645522, 3.645521,
      3.645515, 3.645462, 3.645422, 3.645387, 3.645324, 3.645324, 3.645261,
      3.645226, 3.645226, 3.645226, 3.645222, 3.645167, 3.645127, 3.645127,
      3.645127, 3.645127, 3.645127, 3.645127, 3.645127, 3.645144, 3.645201,
      3.645226, 3.645268, 3.645324, 3.645335, 3.645394, 3.645483, 3.645561,
      3.645622, 3.645729, 3.645789, 3.645884, 3.646006, 3.646128, 3.646250,
      3.646374, 3.646592, 3.646721, 3.646871, 3.647060, 3.647299, 3.647491,
      3.647718, 3.647976, 3.648236, 3.648497, 3.648762, 3.649093, 3.649426,
      3.649761, 3.650098, 3.650483, 3.650894, 3.651307, 3.651732, 3.652221,
      3.652665, 3.653161, 3.653712, 3.654287, 3.654866, 3.655451, 3.656094,
      3.656762, 3.657445, 3.658200, 3.658962, 3.659731, 3.660536, 3.661395,
      3.662262, 3.663294, 3.664339, 3.665394, 3.666497, 3.667675, 3.668900,
      3.670155, 3.671482, 3.672862, 3.674251, 3.675737, 3.677247, 3.678849,
      3.680478, 3.682129, 3.683794, 3.685556, 3.687383, 3.689291, 3.691309,
      3.693586, 3.695876, 3.698261, 3.700630, 3.703036, 3.705537, 3.708211,
      3.710986, 3.713835, 3.716773, 3.719845, 3.722987, 3.726240, 3.729524,
      3.732876, 3.736319, 3.739742, 3.743110, 3.746477, 3.749808, 3.753057,
      3.756180, 3.759143, 3.761934, 3.764485, 3.766779, 3.768696, 3.770189,
      3.771302, 3.772058, 3.772372, 3.772189, 3.771689, 3.770979, 3.770144,
      3.769230, 3.768329, 3.767510, 3.766907, 3.766389, 3.765957, 3.765700,
      3.765514, 3.765357, 3.765281, 3.765214, 3.765181, 3.765148, 3.765119,
      3.765091, 3.765072, 3.765072, 3.765072, 3.765090, 3.765134, 3.765178,
      3.765223, 3.765283, 3.765342, 3.765401, 3.765473, 3.765580, 3.765686,
      3.765792, 3.765898, 3.765994, 3.766090, 3.766186, 3.766282, 3.766393,
      3.766524, 3.766655, 3.766787, 3.766918, 3.767050, 3.767184, 3.767318,
      3.767452, 3.767585, 3.767719, 3.767845, 3.767969, 3.768093, 3.768217,
      3.768341, 3.768465, 3.768564, 3.768656, 3.768748, 3.768841, 3.768933,
      3.769026, 3.769107, 3.769173, 3.769238, 3.769303, 3.769368, 3.769433,
      3.769498, 3.769549, 3.769591, 3.769632, 3.769673, 3.769714, 3.769755,
      3.769796, 3.769833, 3.769862, 3.769892, 3.769921, 3.769950, 3.769980,
      3.770009, 3.770039, 3.770067, 3.770095, 3.770124, 3.770152, 3.770180,
      3.770208, 3.770237, 3.770263, 3.770273, 3.770282, 3.770291, 3.770300,
      3.770309, 3.770318, 3.770327, 3.770336, 3.770354, 3.770371, 3.770389,
      3.770406, 3.770424, 3.770442, 3.770459, 3.770477, 3.770490, 3.770499,
      3.770509, 3.770518, 3.770527, 3.770537, 3.770546, 3.770556, 3.770566,
      3.770577, 3.770588, 3.770599, 3.770610, 3.770621, 3.770631, 3.770644,
      3.770656, 3.770668, 3.770680, 3.770692, 3.770704, 3.770718, 3.770731,
      3.770745, 3.770759, 3.770772, 3.770778, 3.770778, 3.770778, 3.770778,
      3.770778, 3.770788, 3.770804, 3.770820, 3.770837, 3.770853, 3.770871,
      3.770888, 3.770906, 3.770923, 3.770942, 3.770961, 3.770980, 3.770999,
      3.771020, 3.771040, 3.771060, 3.771081, 3.771103, 3.771124, 3.771146,
      3.771146, 3.771146, 3.771146, 3.771166, 3.771190, 3.771214, 3.771240,
      3.771266, 3.771291, 3.771318, 3.771345, 3.771367, 3.771367, 3.771367,
      3.771385, 3.771415, 3.771445, 3.771476, 3.771508, 3.771514, 3.771514,
      3.771532, 3.771566, 3.771600, 3.771636, 3.771671, 3.771708, 3.771745,
      3.771783, 3.771808, 3.771808, 3.771827, 3.771867, 3.771909, 3.771951,
      3.771994, 3.772037, 3.772082, 3.772128, 3.772174, 3.772221, 3.772248,
      3.772248, 3.772294, 3.772345, 3.772396, 3.772449, 3.772503, 3.772557,
      3.772612, 3.772669, 3.772727, 3.772785, 3.772845, 3.772906, 3.772968,
      3.773031, 3.773096, 3.773161, 3.773228, 3.773274, 3.773309, 3.773435,
      3.773494, 3.773507, 3.773582, 3.773658, 3.773735, 3.773813, 3.773894,
      3.773975, 3.774058, 3.774143, 3.774229, 3.774317, 3.774406, 3.774497,
      3.774590, 3.774684, 3.774781, 3.774809, 3.774905, 3.775007, 3.775110,
      3.775216, 3.775323, 3.775432, 3.775538, 3.775584, 3.775700, 3.775817,
      3.775937, 3.776059, 3.776120, 3.776237, 3.776365, 3.776496, 3.776557,
      3.776693, 3.776832, 3.776900, 3.777044, 3.777137, 3.777266, 3.777354,
      3.777499, 3.777585, 3.777672, 3.777763, 3.777789, 3.777862, 3.777934,
      3.777934, 3.777934, 3.777934, 3.777934, 3.777862, 3.777789, 3.777601,
      3.777329, 3.776968, 3.776405, 3.775842, 3.775187, 3.774372, 3.773604,
      3.772836, 3.771780, 3.770474, 3.769701, 3.768394, 3.767258, 3.766863,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      3.656037, 3.656032, 3.656002, 3.655969, 3.655913, 3.655810, 3.655755,
      3.655715, 3.655670, 3.655589, 3.655534, 3.655523, 3.655472, 3.655408,
      3.655315, 3.655260, 3.655235, 3.655163, 3.655096, 3.655041, 3.654939,
      3.654885, 3.654850, 3.654799, 3.654721, 3.654666, 3.654651, 3.654578,
      3.654497, 3.654465, 3.654454, 3.654388, 3.654292, 3.654273, 3.654242,
      3.654169, 3.654080, 3.654080, 3.654014, 3.653984, 3.653956, 3.653888,
      3.653888, 3.653821, 3.653791, 3.653759, 3.653695, 3.653695, 3.653695,
      3.653666, 3.653614, 3.653598, 3.653598, 3.653598, 3.653598, 3.653598,
      3.653598, 3.653598, 3.653649, 3.653695, 3.653695, 3.653695, 3.653695,
      3.653714, 3.653791, 3.653816, 3.653872, 3.653888, 3.653936, 3.653984,
      3.654001, 3.654058, 3.654151, 3.654222, 3.654286, 3.654386, 3.654444,
      3.654541, 3.654659, 3.654778, 3.654897, 3.655017, 3.655138, 3.655319,
      3.655477, 3.655599, 3.655822, 3.656008, 3.656162, 3.656381, 3.656633,
      3.656886, 3.657141, 3.657397, 3.657655, 3.657914, 3.658192, 3.658520,
      3.658851, 3.659208, 3.659609, 3.659919, 3.660325, 3.660793, 3.661243,
      3.661664, 3.662153, 3.662696, 3.663263, 3.663834, 3.664439, 3.665091,
      3.665749, 3.666412, 3.667123, 3.667872, 3.668633, 3.669472, 3.670319,
      3.671224, 3.672164, 3.673129, 3.674141, 3.675222, 3.676328, 3.677423,
      3.678605, 3.679869, 3.681173, 3.682471, 3.683789, 3.685131, 3.686631,
      3.688133, 3.689700, 3.691446, 3.693306, 3.695278, 3.697245, 3.699284,
      3.701361, 3.703469, 3.705709, 3.708079, 3.710535, 3.713075, 3.715705,
      3.718404, 3.721235, 3.724166, 3.727182, 3.730264, 3.733408, 3.736615,
      3.739837, 3.743075, 3.746327, 3.749603, 3.752953, 3.756329, 3.759754,
      3.763197, 3.766640, 3.770117, 3.773567, 3.776837, 3.779886, 3.782646,
      3.785025, 3.786704, 3.787805, 3.788332, 3.788347, 3.787845, 3.787001,
      3.786016, 3.784935, 3.783909, 3.783060, 3.782535, 3.782015, 3.781770,
      3.781684, 3.781686, 3.781727, 3.781775, 3.781845, 3.781932, 3.782083,
      3.782234, 3.782368, 3.782500, 3.782638, 3.782823, 3.783008, 3.783192,
      3.783356, 3.783519, 3.783683, 3.783875, 3.784094, 3.784312, 3.784531,
      3.784766, 3.785011, 3.785256, 3.785501, 3.785741, 3.785963, 3.786184,
      3.786406, 3.786628, 3.786829, 3.787005, 3.787180, 3.787356, 3.787531,
      3.787699, 3.787836, 3.787972, 3.788109, 3.788245, 3.788382, 3.788497,
      3.788589, 3.788681, 3.788773, 3.788865, 3.788958, 3.789044, 3.789119,
      3.789195, 3.789270, 3.789346, 3.789421, 3.789497, 3.789552, 3.789603,
      3.789654, 3.789704, 3.789755, 3.789806, 3.789857, 3.789888, 3.789917,
      3.789946, 3.789975, 3.790004, 3.790033, 3.790062, 3.790090, 3.790118,
      3.790146, 3.790173, 3.790201, 3.790229, 3.790256, 3.790284, 3.790302,
      3.790320, 3.790338, 3.790356, 3.790373, 3.790391, 3.790409, 3.790427,
      3.790444, 3.790461, 3.790478, 3.790495, 3.790512, 3.790529, 3.790547,
      3.790564, 3.790574, 3.790582, 3.790590, 3.790599, 3.790607, 3.790615,
      3.790624, 3.790632, 3.790643, 3.790660, 3.790676, 3.790692, 3.790709,
      3.790725, 3.790741, 3.790757, 3.790774, 3.790784, 3.790794, 3.790803,
      3.790812, 3.790821, 3.790830, 3.790839, 3.790848, 3.790848, 3.790848,
      3.790848, 3.790848, 3.790848, 3.790848, 3.790852, 3.790864, 3.790876,
      3.790887, 3.790899, 3.790911, 3.790923, 3.790936, 3.790949, 3.790962,
      3.790975, 3.790988, 3.790988, 3.790988, 3.790988, 3.790988, 3.790988,
      3.790988, 3.790988, 3.790988, 3.790988, 3.790999, 3.791016, 3.791033,
      3.791050, 3.791059, 3.791059, 3.791059, 3.791059, 3.791059, 3.791059,
      3.791059, 3.791059, 3.791080, 3.791101, 3.791122, 3.791129, 3.791129,
      3.791129, 3.791129, 3.791129, 3.791129, 3.791129, 3.791129, 3.791129,
      3.791145, 3.791171, 3.791197, 3.791199, 3.791199, 3.791199, 3.791199,
      3.791199, 3.791226, 3.791256, 3.791269, 3.791269, 3.791269, 3.791269,
      3.791274, 3.791308, 3.791340, 3.791340, 3.791342, 3.791378, 3.791410,
      3.791410, 3.791410, 3.791410, 3.791429, 3.791469, 3.791480, 3.791482,
      3.791524, 3.791550, 3.791550, 3.791587, 3.791620, 3.791620, 3.791657,
      3.791691, 3.791691, 3.791735, 3.791761, 3.791768, 3.791821, 3.791831,
      3.791860, 3.791901, 3.791904, 3.791962, 3.791971, 3.792012, 3.792041,
      3.792066, 3.792111, 3.792125, 3.792181, 3.792188, 3.792252, 3.792257,
      3.792322, 3.792331, 3.792392, 3.792410, 3.792462, 3.792462, 3.792505,
      3.792532, 3.792532, 3.792602, 3.792602, 3.792646, 3.792672, 3.792672,
      3.792672, 3.792672, 3.792672, 3.792672, 3.792672, 3.792672, 3.792672,
      3.792672, 3.792672, 3.792602, 3.792532, 3.792480, 3.792365, 3.792244,
      3.792058, 3.791831, 3.791582, 3.791259, 3.791065, 3.790626, 3.790285,
      3.790414, 3.789854, 3.789072, 3.788208, 3.787479, 3.786585, 3.785633,
      3.785327, 3.784142, 3.782755, 3.781312, 3.779818, 3.778462, 3.777229,
      3.776411, 3.777095, NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      3.662408, 3.662402,
      3.662380, 3.662339, 3.662286, 3.662277, 3.662222, 3.662191, 3.662160,
      3.662105, 3.662096, 3.662002, 3.662002, 3.661963, 3.661881, 3.661827,
      3.661813, 3.661767, 3.661718, 3.661695, 3.661623, 3.661623, 3.661573,
      3.661498, 3.661434, 3.661421, 3.661356, 3.661339, 3.661290, 3.661245,
      3.661245, 3.661245, 3.661166, 3.661150, 3.661099, 3.661055, 3.661055,
      3.661055, 3.660992, 3.660960, 3.660960, 3.660960, 3.660960, 3.660960,
      3.660960, 3.660960, 3.660960, 3.660960, 3.660960, 3.660992, 3.661046,
      3.661055, 3.661055, 3.661080, 3.661150, 3.661150, 3.661222, 3.661245,
      3.661351, 3.661434, 3.661482, 3.661595, 3.661623, 3.661729, 3.661844,
      3.661934, 3.661992, 3.662099, 3.662216, 3.662334, 3.662453, 3.662666,
      3.662786, 3.662961, 3.663163, 3.663345, 3.663529, 3.663748, 3.663946,
      3.664150, 3.664400, 3.664652, 3.664905, 3.665195, 3.665509, 3.665790,
      3.666114, 3.666500, 3.666831, 3.667198, 3.667597, 3.667998, 3.668403,
      3.668850, 3.669343, 3.669858, 3.670343, 3.670832, 3.671386, 3.671994,
      3.672637, 3.673285, 3.673939, 3.674652, 3.675390, 3.676135, 3.676887,
      3.677722, 3.678626, 3.679505, 3.680434, 3.681434, 3.682388, 3.683398,
      3.684452, 3.685549, 3.686725, 3.687953, 3.689213, 3.690553, 3.692012,
      3.693532, 3.695134, 3.696810, 3.698507, 3.700228, 3.702010, 3.703833,
      3.705746, 3.707761, 3.709864, 3.712021, 3.714337, 3.716674, 3.719137,
      3.721691, 3.724299, 3.727012, 3.729809, 3.732617, 3.735473, 3.738387,
      3.741372, 3.744381, 3.747485, 3.750621, 3.753838, 3.757162, 3.760596,
      3.764201, 3.767897, 3.771739, 3.775729, 3.779818, 3.783862, 3.787813,
      3.791621, 3.795195, 3.798305, 3.800601, 3.802066, 3.802705, 3.802551,
      3.801769, 3.800567, 3.799249, 3.797947, 3.796930, 3.796367, 3.795910,
      3.795747, 3.795801, 3.796001, 3.796247, 3.796524, 3.796838, 3.797180,
      3.797496, 3.797792, 3.798106, 3.798493, 3.798879, 3.799276, 3.799683,
      3.800090, 3.800502, 3.800920, 3.801338, 3.801757, 3.802185, 3.802613,
      3.803041, 3.803462, 3.803870, 3.804278, 3.804686, 3.805044, 3.805377,
      3.805710, 3.806043, 3.806291, 3.806538, 3.806786, 3.807034, 3.807194,
      3.807342, 3.807491, 3.807640, 3.807770, 3.807865, 3.807960, 3.808054,
      3.808149, 3.808232, 3.808294, 3.808356, 3.808418, 3.808480, 3.808542,
      3.808600, 3.808657, 3.808715, 3.808772, 3.808829, 3.808886, 3.808929,
      3.808972, 3.809015, 3.809057, 3.809100, 3.809143, 3.809184, 3.809224,
      3.809264, 3.809304, 3.809344, 3.809384, 3.809425, 3.809453, 3.809482,
      3.809510, 3.809539, 3.809567, 3.809596, 3.809624, 3.809643, 3.809661,
      3.809679, 3.809698, 3.809716, 3.809734, 3.809752, 3.809766, 3.809774,
      3.809783, 3.809792, 3.809800, 3.809809, 3.809818, 3.809826, 3.809835,
      3.809843, 3.809851, 3.809860, 3.809868, 3.809877, 3.809885, 3.809893,
      3.809901, 3.809910, 3.809918, 3.809926, 3.809934, 3.809942, 3.809950,
      3.809958, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964,
      3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964,
      3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964,
      3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964,
      3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964, 3.809964,
      3.809964, 3.809964, 3.809964, 3.809946, 3.809920, 3.809894, 3.809869,
      3.809843, 3.809822, 3.809808, 3.809794, 3.809780, 3.809766, 3.809762,
      3.809762, 3.809762, 3.809762, 3.809756, 3.809740, 3.809723, 3.809707,
      3.809690, 3.809672, 3.809654, 3.809636, 3.809627, 3.809627, 3.809627,
      3.809627, 3.809607, 3.809587, 3.809567, 3.809546, 3.809524, 3.809502,
      3.809455, 3.809387, 3.809318, 3.809290, 3.809290, 3.809290, 3.809270,
      3.809245, 3.809219, 3.809193, 3.809166, 3.809156, 3.809156, 3.809150,
      3.809121, 3.809092, 3.809088, 3.809088, 3.809068, 3.809036, 3.809004,
      3.808971, 3.808953, 3.808953, 3.808937, 3.808901, 3.808865, 3.808829,
      3.808818, 3.808818, 3.808782, 3.808751, 3.808751, 3.808729, 3.808687,
      3.808645, 3.808601, 3.808557, 3.808549, 3.808505, 3.808365, 3.808305,
      3.808256, 3.808207, 3.808157, 3.808106, 3.808054, 3.808008, 3.808008,
      3.807959, 3.807903, 3.807818, 3.807720, 3.807660, 3.807600, 3.807538,
      3.807415, 3.807344, 3.807117, 3.806956, 3.806821, 3.806737, 3.806610,
      3.806527, 3.806454, 3.806384, 3.806384, 3.806384, 3.806384, 3.806339,
      3.806316, 3.806170, 3.806000, 3.805747, 3.805396, 3.804941, 3.804355,
      3.803702, 3.802925, 3.802101, 3.801386, 3.800670, 3.799766, 3.798691,
      3.797890, 3.797553, 3.796549, 3.795235, 3.793839, 3.792369, 3.790784,
      3.789129, 3.787521, 3.785925, 3.784379, 3.783081, 3.782612, 3.784535,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      3.668199, 3.668199, 3.668199, 3.668161,
      3.668106, 3.668106, 3.668106, 3.668056, 3.668013, 3.668013, 3.668013,
      3.668013, 3.667935, 3.667920, 3.667920, 3.667907, 3.667854, 3.667826,
      3.667826, 3.667826, 3.667813, 3.667752, 3.667733, 3.667733, 3.667733,
      3.667721, 3.667659, 3.667640, 3.667640, 3.667640, 3.667640, 3.667640,
      3.667640, 3.667640, 3.667640, 3.667640, 3.667640, 3.667640, 3.667668,
      3.667724, 3.667733, 3.667739, 3.667793, 3.667826, 3.667856, 3.667910,
      3.667920, 3.667974, 3.668047, 3.668132, 3.668188, 3.668290, 3.668348,
      3.668424, 3.668538, 3.668652, 3.668768, 3.668884, 3.669000, 3.669117,
      3.669239, 3.669417, 3.669564, 3.669728, 3.669918, 3.670135, 3.670317,
      3.670500, 3.670706, 3.670953, 3.671201, 3.671450, 3.671700, 3.671953,
      3.672233, 3.672552, 3.672919, 3.673242, 3.673621, 3.674013, 3.674408,
      3.674806, 3.675206, 3.675673, 3.676147, 3.676624, 3.677104, 3.677651,
      3.678209, 3.678791, 3.679427, 3.680069, 3.680716, 3.681368, 3.682042,
      3.682778, 3.683522, 3.684273, 3.685120, 3.686004, 3.686872, 3.687805,
      3.688747, 3.689755, 3.690825, 3.692023, 3.693190, 3.694511, 3.695798,
      3.697165, 3.698582, 3.699966, 3.701455, 3.702999, 3.704589, 3.706232,
      3.707916, 3.709725, 3.711616, 3.713579, 3.715615, 3.717714, 3.719909,
      3.722177, 3.724531, 3.726911, 3.729318, 3.731827, 3.734365, 3.736997,
      3.739695, 3.742447, 3.745228, 3.748118, 3.751159, 3.754305, 3.757521,
      3.760840, 3.764412, 3.768168, 3.772139, 3.776304, 3.780578, 3.785041,
      3.789697, 3.794462, 3.799298, 3.804120, 3.808732, 3.812550, 3.815227,
      3.816705, 3.816823, 3.815859, 3.814306, 3.812633, 3.811026, 3.809870,
      3.809219, 3.809021, 3.809021, 3.809270, 3.809705, 3.810243, 3.810782,
      3.811324, 3.811886, 3.812462, 3.813127, 3.813792, 3.814609, 3.815442,
      3.816468, 3.817553, 3.818745, 3.820089, 3.821478, 3.822917, 3.824350,
      3.825632, 3.826712, 3.827553, 3.828087, 3.828418, 3.828653, 3.828780,
      3.828890, 3.828985, 3.829027, 3.829086, 3.829158, 3.829223, 3.829286,
      3.829323, 3.829350, 3.829393, 3.829464, 3.829535, 3.829574, 3.829595,
      3.829616, 3.829646, 3.829683, 3.829720, 3.829755, 3.829772, 3.829788,
      3.829805, 3.829818, 3.829818, 3.829818, 3.829818, 3.829818, 3.829829,
      3.829843, 3.829856, 3.829870, 3.829884, 3.829908, 3.829933, 3.829957,
      3.829982, 3.830006, 3.830029, 3.830052, 3.830075, 3.830097, 3.830120,
      3.830142, 3.830163, 3.830184, 3.830205, 3.830226, 3.830247, 3.830268,
      3.830278, 3.830288, 3.830298, 3.830307, 3.830317, 3.830327, 3.830341,
      3.830360, 3.830378, 3.830397, 3.830415, 3.830434, 3.830453, 3.830450,
      3.830433, 3.830415, 3.830397, 3.830380, 3.830362, 3.830344, 3.830334,
      3.830343, 3.830351, 3.830360, 3.830368, 3.830377, 3.830385, 3.830393,
      3.830396, 3.830396, 3.830396, 3.830396, 3.830396, 3.830396, 3.830396,
      3.830396, 3.830396, 3.830396, 3.830396, 3.830396, 3.830396, 3.830396,
      3.830396, 3.830396, 3.830396, 3.830396, 3.830396, 3.830396, 3.830396,
      3.830396, 3.830396, 3.830396, 3.830395, 3.830387, 3.830379, 3.830371,
      3.830363, 3.830356, 3.830348, 3.830340, 3.830332, 3.830314, 3.830295,
      3.830277, 3.830259, 3.830241, 3.830223, 3.830205, 3.830194, 3.830184,
      3.830173, 3.830163, 3.830153, 3.830143, 3.830124, 3.830101, 3.830078,
      3.830055, 3.830032, 3.830008, 3.829970, 3.829932, 3.829894, 3.829856,
      3.829818, 3.829735, 3.829652, 3.829569, 3.829486, 3.829422, 3.829392,
      3.829361, 3.829331, 3.829300, 3.829251, 3.829202, 3.829154, 3.829107,
      3.829072, 3.829037, 3.829002, 3.828958, 3.828902, 3.828846, 3.828791,
      3.828674, 3.828555, 3.828436, 3.828357, 3.828294, 3.828231, 3.828165,
      3.828099, 3.828032, 3.827962, 3.827892, 3.827822, 3.827748, 3.827674,
      3.827599, 3.827521, 3.827444, 3.827363, 3.827282, 3.827199, 3.827114,
      3.827006, 3.826800, 3.826594, 3.826501, 3.826409, 3.826314, 3.826218,
      3.826120, 3.826020, 3.825887, 3.825750, 3.825643, 3.825529, 3.825382,
      3.825233, 3.825081, 3.824926, 3.824762, 3.824438, 3.824183, 3.824017,
      3.823846, 3.823657, 3.823436, 3.823250, 3.823066, 3.822878, 3.822639,
      3.822428, 3.822230, 3.822031, 3.821876, 3.821719, 3.821559, 3.821395,
      3.821229, 3.821060, 3.820831, 3.820547, 3.820204, 3.819799, 3.819293,
      3.818722, 3.818076, 3.817350, 3.816543, 3.815648, 3.814697, 3.813649,
      3.812400, 3.812262, 3.811502, 3.810594, 3.809275, 3.807897, 3.806425,
      3.804849, 3.803221, 3.801539, 3.799794, 3.797993, 3.796192, 3.794413,
      3.792699, 3.791058, 3.789716, 3.789157, 3.791010, NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      3.668779, 3.668779, 3.668781, 3.668743, 3.668688, 3.668689,
      3.668695, 3.668643, 3.668598, 3.668604, 3.668605, 3.668614, 3.668528,
      3.668515, 3.668524, 3.668515, 3.668458, 3.668432, 3.668437, 3.668440,
      3.668432, 3.668364, 3.668349, 3.668357, 3.668363, 3.668351, 3.668289,
      3.668270, 3.668275, 3.668279, 3.668279, 3.668279, 3.668287, 3.668289,
      3.668294, 3.668298, 3.668298, 3.668298, 3.668335, 3.668401, 3.668410,
      3.668417, 3.668477, 3.668513, 3.668545, 3.668605, 3.668616, 3.668676,
      3.668755, 3.668846, 3.668903, 3.669013, 3.669077, 3.669158, 3.669277,
      3.669403, 3.669522, 3.669647, 3.669765, 3.669885, 3.670015, 3.670199,
      3.670358, 3.670527, 3.670726, 3.670955, 3.671149, 3.671340, 3.671555,
      3.671815, 3.672075, 3.672328, 3.672592, 3.672852, 3.673140, 3.673473,
      3.673858, 3.674191, 3.674588, 3.674999, 3.675408, 3.675821, 3.676236,
      3.676721, 3.677211, 3.677707, 3.678203, 3.678767, 3.679347, 3.679950,
      3.680610, 3.681276, 3.681947, 3.682620, 3.683312, 3.684071, 3.684840,
      3.685617, 3.686494, 3.687406, 3.688295, 3.689257, 3.690228, 3.691265,
      3.692368, 3.693612, 3.694820, 3.696190, 3.697515, 3.698931, 3.700397,
      3.701819, 3.703362, 3.704959, 3.706602, 3.708301, 3.710035, 3.711902,
      3.713856, 3.715882, 3.717976, 3.720132, 3.722386, 3.724714, 3.727134,
      3.729579, 3.732049, 3.734626, 3.737226, 3.739920, 3.742678, 3.745490,
      3.748318, 3.751262, 3.754361, 3.757566, 3.760843, 3.764223, 3.767873,
      3.771723, 3.775805, 3.780096, 3.784498, 3.789107, 3.793918, 3.798846,
      3.803844, 3.808816, 3.813546, 3.817385, 3.819959, 3.821201, 3.820933,
      3.819463, 3.817351, 3.815115, 3.812966, 3.811337, 3.810310, 3.809863,
      3.809716, 3.809926, 3.810420, 3.811091, 3.811803, 3.812531, 3.813280,
      3.814015, 3.814803, 3.815580, 3.816496, 3.817406, 3.818515, 3.819683,
      3.820967, 3.822414, 3.823908, 3.825459, 3.827006, 3.828385, 3.829534,
      3.830420, 3.830969, 3.831292, 3.831509, 3.831608, 3.831686, 3.831749,
      3.831754, 3.831776, 3.831813, 3.831842, 3.831868, 3.831869, 3.831858,
      3.831864, 3.831906, 3.831951, 3.831961, 3.831951, 3.831949, 3.831957,
      3.831973, 3.831988, 3.832011, 3.832015, 3.832018, 3.832021, 3.832023,
      3.832014, 3.832004, 3.831995, 3.831985, 3.831989, 3.831998, 3.832006,
      3.832015, 3.832024, 3.832045, 3.832066, 3.832087, 3.832109, 3.832130,
      3.832149, 3.832169, 3.832189, 3.832210, 3.832230, 3.832251, 3.832270,
      3.832288, 3.832307, 3.832327, 3.832346, 3.832364, 3.832371, 3.832378,
      3.832385, 3.832393, 3.832401, 3.832409, 3.832421, 3.832439, 3.832457,
      3.832474, 3.832493, 3.832511, 3.832530, 3.832525, 3.832504, 3.832483,
      3.832462, 3.832441, 3.832421, 3.832401, 3.832389, 3.832397, 3.832406,
      3.832414, 3.832422, 3.832431, 3.832439, 3.832448, 3.832450, 3.832449,
      3.832448, 3.832447, 3.832446, 3.832446, 3.832445, 3.832444, 3.832443,
      3.832442, 3.832442, 3.832441, 3.832440, 3.832439, 3.832439, 3.832439,
      3.832439, 3.832439, 3.832439, 3.832439, 3.832439, 3.832439, 3.832439,
      3.832439, 3.832438, 3.832429, 3.832421, 3.832412, 3.832403, 3.832395,
      3.832386, 3.832377, 3.832369, 3.832349, 3.832329, 3.832309, 3.832289,
      3.832269, 3.832249, 3.832229, 3.832217, 3.832206, 3.832194, 3.832183,
      3.832172, 3.832160, 3.832140, 3.832114, 3.832089, 3.832064, 3.832041,
      3.832016, 3.831977, 3.831938, 3.831899, 3.831859, 3.831819, 3.831729,
      3.831639, 3.831549, 3.831458, 3.831388, 3.831355, 3.831321, 3.831289,
      3.831256, 3.831204, 3.831152, 3.831100, 3.831050, 3.831013, 3.830977,
      3.830939, 3.830891, 3.830830, 3.830768, 3.830709, 3.830582, 3.830454,
      3.830325, 3.830240, 3.830173, 3.830108, 3.830043, 3.829977, 3.829906,
      3.829830, 3.829752, 3.829677, 3.829598, 3.829520, 3.829439, 3.829357,
      3.829273, 3.829184, 3.829095, 3.829007, 3.828917, 3.828798, 3.828571,
      3.828346, 3.828248, 3.828150, 3.828048, 3.827945, 3.827836, 3.827729,
      3.827585, 3.827438, 3.827325, 3.827200, 3.827039, 3.826878, 3.826714,
      3.826544, 3.826365, 3.826013, 3.825737, 3.825558, 3.825374, 3.825167,
      3.824929, 3.824738, 3.824542, 3.824340, 3.824082, 3.823855, 3.823643,
      3.823429, 3.823263, 3.823090, 3.822919, 3.822745, 3.822570, 3.822394,
      3.822148, 3.821841, 3.821470, 3.821037, 3.820488, 3.819882, 3.819188,
      3.818403, 3.817523, 3.816552, 3.815514, 3.814368, 3.813002, 3.812850,
      3.812014, 3.811016, 3.809568, 3.808056, 3.806451, 3.804734, 3.802968,
      3.801153, 3.799279, 3.797357, 3.795441, 3.793561, 3.791758, 3.790025,
      3.788621, 3.788097, 3.790242, NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN};

// Baraffe+2015 radius of gyration interpolation grid
static double const DATA_RG[STELLAR_BAR_MLEN][STELLAR_BAR_ALEN] = {
      0.448100, 0.448108, 0.448182, 0.448258, 0.448335, 0.448354, 0.448377,
      0.448444, 0.448502, 0.448558, 0.448608, 0.448658, 0.448705, 0.448752,
      0.448787, 0.448821, 0.448856, 0.448896, 0.448937, 0.448983, 0.449031,
      0.449076, 0.449123, 0.449156, 0.449151, 0.449161, 0.449215, 0.449276,
      0.449337, 0.449396, 0.449455, 0.449448, 0.449477, 0.449551, 0.449626,
      0.449650, 0.449677, 0.449756, 0.449840, 0.449843, 0.449915, 0.450013,
      0.450050, 0.450115, 0.450140, 0.450238, 0.450344, 0.450383, 0.450521,
      0.450557, 0.450645, 0.450765, 0.450840, 0.451017, 0.451103, 0.451210,
      0.451325, 0.451488, 0.451599, 0.451716, 0.451862, 0.452022, 0.452206,
      0.452447, 0.452635, 0.452822, 0.453060, 0.453305, 0.453557, 0.453853,
      0.454175, 0.454501, 0.454800, 0.455215, 0.455557, 0.455935, 0.456357,
      0.456666, 0.457074, 0.457438, 0.457764, 0.458069, 0.458298, 0.458601,
      0.458907, 0.459210, 0.459514, 0.459729, 0.459941, 0.460202, 0.460473,
      0.460677, 0.460874, 0.461073, 0.461271, 0.461471, 0.461672, 0.461864,
      0.462055, 0.462181, 0.462324, 0.462516, 0.462700, 0.462882, 0.463014,
      0.463134, 0.463321, 0.463498, 0.463673, 0.463789, 0.463919, 0.464092,
      0.464197, 0.464329, 0.464463, 0.464563, 0.464728, 0.464811, 0.464955,
      0.465073, 0.465153, 0.465233, 0.465304, 0.465447, 0.465569, 0.465646,
      0.465723, 0.465800, 0.465876, 0.465927, 0.465971, 0.466052, 0.466126,
      0.466200, 0.466274, 0.466347, 0.466420, 0.466429, 0.466499, 0.466530,
      0.466560, 0.466630, 0.466623, 0.466715, 0.466730, 0.466765, 0.466830,
      0.466830, 0.466871, 0.466930, 0.466927, 0.466924, 0.467000, 0.467030,
      0.467030, 0.467059, 0.467127, 0.467130, 0.467130, 0.467130, 0.467129,
      0.467122, 0.467230, 0.467230, 0.467230, 0.467230, 0.467230, 0.467230,
      0.467230, 0.467224, 0.467220, 0.467220, 0.467292, 0.467330, 0.467330,
      0.467330, 0.467330, 0.467330, 0.467330, 0.467330, 0.467327, 0.467321,
      0.467320, 0.467320, 0.467320, 0.467320, 0.467334, 0.467403, 0.467430,
      0.467430, 0.467430, 0.467430, 0.467430, 0.467421, 0.467420, 0.467445,
      0.467513, 0.467530, 0.467530, 0.467530, 0.467530, 0.467530, 0.467531,
      0.467592, 0.467630, 0.467630, 0.467630, 0.467630, 0.467624, 0.467668,
      0.467730, 0.467730, 0.467730, 0.467730, 0.467726, 0.467750, 0.467814,
      0.467830, 0.467830, 0.467824, 0.467862, 0.467930, 0.467930, 0.467930,
      0.467925, 0.467920, 0.467937, 0.468000, 0.468030, 0.468030, 0.468028,
      0.468022, 0.468130, 0.468130, 0.468130, 0.468130, 0.468166, 0.468227,
      0.468230, 0.468230, 0.468230, 0.468230, 0.468251, 0.468320, 0.468330,
      0.468330, 0.468330, 0.468327, 0.468326, 0.468413, 0.468430, 0.468430,
      0.468430, 0.468437, 0.468515, 0.468530, 0.468530, 0.468530, 0.468530,
      0.468530, 0.468605, 0.468635, 0.468630, 0.468630, 0.468630, 0.468630,
      0.468630, 0.468630, 0.468630, 0.468630, 0.468710, 0.468735, 0.468730,
      0.468730, 0.468730, 0.468730, 0.468730, 0.468730, 0.468730, 0.468730,
      0.468730, 0.468730, 0.468730, 0.468730, 0.468730, 0.468730, 0.468700,
      0.468640, 0.468630, 0.468630, 0.468630, 0.468630, 0.468630, 0.468630,
      0.468630, 0.468630, 0.468624, 0.468530, 0.468530, 0.468530, 0.468530,
      0.468530, 0.468530, 0.468423, 0.468430, 0.468430, 0.468430, 0.468430,
      0.468430, 0.468359, 0.468330, 0.468330, 0.468330, 0.468286, 0.468230,
      0.468230, 0.468230, 0.468232, 0.468238, 0.468139, 0.468130, 0.468130,
      0.468072, 0.468030, 0.468030, 0.468008, 0.467921, 0.467930, 0.467930,
      0.467930, 0.467885, 0.467830, 0.467831, 0.467840, 0.467757, 0.467730,
      0.467734, 0.467720, 0.467631, 0.467630, 0.467635, 0.467599, 0.467530,
      0.467530, 0.467454, 0.467430, 0.467430, 0.467420, 0.467340, 0.467330,
      0.467285, 0.467227, 0.467230, 0.467230, 0.467230, 0.467211, 0.467131,
      0.467130, 0.467075, 0.467024, 0.467030, 0.467030, 0.466978, 0.466922,
      0.466929, 0.466930, 0.466896, 0.466829, 0.466830, 0.466830, 0.466821,
      0.466752, 0.466730, 0.466730, 0.466730, 0.466730, 0.466722, 0.466652,
      0.466630, 0.466630, 0.466630, 0.466580, 0.466524, 0.466530, 0.466530,
      0.466530, 0.466530, 0.466530, 0.466497, 0.466423, 0.466430, 0.466430,
      0.466430, 0.466430, 0.466430, 0.466431, 0.466439, 0.466440, 0.466440,
      0.466440, 0.466440, 0.466426, 0.466330, 0.466330, 0.466330, 0.466330,
      0.466330, 0.466330, 0.466330, 0.466330, 0.466330, 0.466330, 0.466330,
      0.466330, 0.466330, 0.466330, 0.466330, 0.466330, 0.466330, 0.466323,
      0.466250, 0.466220, 0.466222, 0.466229, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230, 0.466230,
      0.466230, 0.466230, 0.466230, NAN,      NAN,      0.448040, 0.448047,
      0.448122, 0.448198, 0.448275, 0.448300, 0.448327, 0.448394, 0.448453,
      0.448509, 0.448559, 0.448607, 0.448650, 0.448693, 0.448732, 0.448770,
      0.448808, 0.448848, 0.448888, 0.448930, 0.448974, 0.449019, 0.449065,
      0.449100, 0.449100, 0.449115, 0.449169, 0.449224, 0.449281, 0.449341,
      0.449400, 0.449400, 0.449432, 0.449501, 0.449573, 0.449600, 0.449624,
      0.449705, 0.449789, 0.449800, 0.449868, 0.449963, 0.450000, 0.450067,
      0.450100, 0.450191, 0.450300, 0.450339, 0.450473, 0.450514, 0.450600,
      0.450722, 0.450800, 0.450970, 0.451061, 0.451166, 0.451285, 0.451449,
      0.451560, 0.451680, 0.451829, 0.451993, 0.452176, 0.452416, 0.452605,
      0.452794, 0.453032, 0.453277, 0.453534, 0.453833, 0.454156, 0.454482,
      0.454791, 0.455197, 0.455543, 0.455921, 0.456338, 0.456649, 0.457051,
      0.457412, 0.457737, 0.458044, 0.458281, 0.458584, 0.458889, 0.459190,
      0.459486, 0.459708, 0.459923, 0.460179, 0.460444, 0.460650, 0.460853,
      0.461054, 0.461252, 0.461449, 0.461643, 0.461835, 0.462025, 0.462157,
      0.462301, 0.462486, 0.462670, 0.462852, 0.462989, 0.463113, 0.463291,
      0.463468, 0.463643, 0.463758, 0.463889, 0.464060, 0.464165, 0.464298,
      0.464433, 0.464532, 0.464697, 0.464781, 0.464924, 0.465043, 0.465123,
      0.465203, 0.465282, 0.465420, 0.465538, 0.465616, 0.465693, 0.465769,
      0.465845, 0.465900, 0.465946, 0.466021, 0.466096, 0.466170, 0.466244,
      0.466317, 0.466390, 0.466400, 0.466470, 0.466500, 0.466529, 0.466600,
      0.466600, 0.466686, 0.466700, 0.466735, 0.466800, 0.466800, 0.466845,
      0.466900, 0.466900, 0.466904, 0.466973, 0.467000, 0.467000, 0.467029,
      0.467097, 0.467100, 0.467100, 0.467100, 0.467100, 0.467100, 0.467200,
      0.467200, 0.467200, 0.467200, 0.467200, 0.467200, 0.467200, 0.467200,
      0.467200, 0.467200, 0.467265, 0.467300, 0.467300, 0.467300, 0.467300,
      0.467300, 0.467300, 0.467300, 0.467300, 0.467300, 0.467300, 0.467300,
      0.467300, 0.467300, 0.467313, 0.467376, 0.467400, 0.467400, 0.467400,
      0.467400, 0.467400, 0.467400, 0.467400, 0.467423, 0.467485, 0.467500,
      0.467500, 0.467500, 0.467500, 0.467500, 0.467501, 0.467561, 0.467600,
      0.467600, 0.467600, 0.467600, 0.467600, 0.467644, 0.467700, 0.467700,
      0.467700, 0.467700, 0.467700, 0.467727, 0.467786, 0.467800, 0.467800,
      0.467800, 0.467838, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900,
      0.467915, 0.467972, 0.468000, 0.468000, 0.468000, 0.468002, 0.468100,
      0.468100, 0.468100, 0.468100, 0.468135, 0.468197, 0.468200, 0.468200,
      0.468200, 0.468200, 0.468221, 0.468291, 0.468300, 0.468300, 0.468300,
      0.468300, 0.468305, 0.468385, 0.468400, 0.468400, 0.468400, 0.468407,
      0.468485, 0.468500, 0.468500, 0.468500, 0.468500, 0.468500, 0.468568,
      0.468600, 0.468600, 0.468600, 0.468600, 0.468600, 0.468600, 0.468600,
      0.468600, 0.468600, 0.468672, 0.468700, 0.468700, 0.468700, 0.468700,
      0.468700, 0.468700, 0.468700, 0.468700, 0.468700, 0.468700, 0.468700,
      0.468700, 0.468700, 0.468700, 0.468700, 0.468669, 0.468610, 0.468600,
      0.468600, 0.468600, 0.468600, 0.468600, 0.468600, 0.468600, 0.468600,
      0.468595, 0.468500, 0.468500, 0.468500, 0.468500, 0.468500, 0.468500,
      0.468403, 0.468400, 0.468400, 0.468400, 0.468400, 0.468400, 0.468329,
      0.468300, 0.468300, 0.468300, 0.468255, 0.468200, 0.468200, 0.468200,
      0.468200, 0.468198, 0.468108, 0.468100, 0.468100, 0.468042, 0.468000,
      0.468000, 0.467980, 0.467900, 0.467900, 0.467900, 0.467900, 0.467855,
      0.467800, 0.467800, 0.467800, 0.467724, 0.467700, 0.467700, 0.467682,
      0.467601, 0.467600, 0.467600, 0.467562, 0.467500, 0.467500, 0.467426,
      0.467400, 0.467400, 0.467391, 0.467314, 0.467300, 0.467259, 0.467200,
      0.467200, 0.467200, 0.467200, 0.467177, 0.467101, 0.467100, 0.467050,
      0.467000, 0.467000, 0.467000, 0.466953, 0.466900, 0.466900, 0.466900,
      0.466867, 0.466800, 0.466800, 0.466800, 0.466792, 0.466724, 0.466700,
      0.466700, 0.466700, 0.466700, 0.466687, 0.466620, 0.466600, 0.466600,
      0.466600, 0.466555, 0.466500, 0.466500, 0.466500, 0.466500, 0.466500,
      0.466500, 0.466470, 0.466400, 0.466400, 0.466400, 0.466400, 0.466400,
      0.466400, 0.466400, 0.466400, 0.466400, 0.466400, 0.466400, 0.466400,
      0.466387, 0.466300, 0.466300, 0.466300, 0.466300, 0.466300, 0.466300,
      0.466300, 0.466300, 0.466300, 0.466300, 0.466300, 0.466300, 0.466300,
      0.466300, 0.466300, 0.466300, 0.466300, 0.466293, 0.466227, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, NAN,      0.447437, 0.447445, 0.447522, 0.447599,
      0.447678, 0.447757, 0.447832, 0.447901, 0.447959, 0.448016, 0.448066,
      0.448100, 0.448100, 0.448105, 0.448181, 0.448258, 0.448319, 0.448360,
      0.448400, 0.448400, 0.448400, 0.448441, 0.448489, 0.448541, 0.448593,
      0.448648, 0.448700, 0.448700, 0.448724, 0.448787, 0.448853, 0.448920,
      0.448991, 0.449000, 0.449040, 0.449100, 0.449101, 0.449187, 0.449277,
      0.449370, 0.449400, 0.449470, 0.449500, 0.449588, 0.449700, 0.449729,
      0.449859, 0.449900, 0.450000, 0.450093, 0.450155, 0.450300, 0.450400,
      0.450503, 0.450638, 0.450724, 0.450884, 0.451058, 0.451173, 0.451315,
      0.451500, 0.451712, 0.451876, 0.452100, 0.452304, 0.452510, 0.452749,
      0.452999, 0.453302, 0.453633, 0.453960, 0.454293, 0.454700, 0.455019,
      0.455400, 0.455784, 0.456147, 0.456486, 0.456819, 0.457146, 0.457469,
      0.457788, 0.458102, 0.458409, 0.458712, 0.458984, 0.459207, 0.459496,
      0.459736, 0.459947, 0.460156, 0.460381, 0.460652, 0.460864, 0.461062,
      0.461229, 0.461350, 0.461541, 0.461731, 0.461918, 0.462069, 0.462192,
      0.462372, 0.462553, 0.462733, 0.462906, 0.462995, 0.463165, 0.463339,
      0.463456, 0.463584, 0.463736, 0.463849, 0.463992, 0.464129, 0.464225,
      0.464389, 0.464476, 0.464615, 0.464738, 0.464819, 0.464898, 0.465055,
      0.465156, 0.465234, 0.465312, 0.465389, 0.465465, 0.465541, 0.465634,
      0.465700, 0.465717, 0.465792, 0.465866, 0.465940, 0.466013, 0.466086,
      0.466109, 0.466181, 0.466200, 0.466225, 0.466297, 0.466368, 0.466400,
      0.466400, 0.466430, 0.466500, 0.466500, 0.466581, 0.466600, 0.466630,
      0.466699, 0.466700, 0.466700, 0.466700, 0.466724, 0.466792, 0.466800,
      0.466800, 0.466800, 0.466813, 0.466880, 0.466900, 0.466900, 0.466900,
      0.466900, 0.466900, 0.466900, 0.466900, 0.466963, 0.467000, 0.467000,
      0.467000, 0.467000, 0.467000, 0.467000, 0.467000, 0.467000, 0.467000,
      0.467000, 0.467029, 0.467093, 0.467100, 0.467100, 0.467100, 0.467100,
      0.467100, 0.467100, 0.467100, 0.467100, 0.467100, 0.467100, 0.467100,
      0.467193, 0.467200, 0.467200, 0.467200, 0.467200, 0.467200, 0.467200,
      0.467200, 0.467200, 0.467200, 0.467259, 0.467300, 0.467300, 0.467300,
      0.467300, 0.467360, 0.467400, 0.467400, 0.467400, 0.467400, 0.467400,
      0.467436, 0.467500, 0.467500, 0.467500, 0.467504, 0.467562, 0.467600,
      0.467600, 0.467600, 0.467600, 0.467653, 0.467700, 0.467700, 0.467700,
      0.467700, 0.467700, 0.467720, 0.467799, 0.467800, 0.467800, 0.467800,
      0.467800, 0.467833, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900,
      0.467925, 0.468000, 0.468000, 0.468000, 0.468000, 0.468028, 0.468100,
      0.468100, 0.468100, 0.468100, 0.468100, 0.468108, 0.468189, 0.468200,
      0.468200, 0.468200, 0.468200, 0.468200, 0.468200, 0.468251, 0.468300,
      0.468300, 0.468300, 0.468300, 0.468300, 0.468300, 0.468300, 0.468300,
      0.468300, 0.468353, 0.468400, 0.468400, 0.468400, 0.468400, 0.468400,
      0.468400, 0.468400, 0.468400, 0.468400, 0.468400, 0.468400, 0.468400,
      0.468400, 0.468400, 0.468367, 0.468315, 0.468300, 0.468300, 0.468300,
      0.468300, 0.468300, 0.468300, 0.468300, 0.468300, 0.468300, 0.468200,
      0.468200, 0.468200, 0.468200, 0.468200, 0.468200, 0.468200, 0.468105,
      0.468100, 0.468100, 0.468100, 0.468100, 0.468025, 0.468000, 0.468000,
      0.468000, 0.467950, 0.467900, 0.467900, 0.467900, 0.467883, 0.467800,
      0.467800, 0.467800, 0.467800, 0.467736, 0.467700, 0.467700, 0.467700,
      0.467689, 0.467604, 0.467600, 0.467600, 0.467551, 0.467500, 0.467486,
      0.467404, 0.467400, 0.467400, 0.467363, 0.467300, 0.467300, 0.467300,
      0.467250, 0.467200, 0.467200, 0.467200, 0.467145, 0.467100, 0.467100,
      0.467100, 0.467047, 0.467000, 0.467000, 0.466927, 0.466900, 0.466900,
      0.466900, 0.466839, 0.466800, 0.466800, 0.466800, 0.466757, 0.466700,
      0.466700, 0.466700, 0.466680, 0.466611, 0.466600, 0.466575, 0.466508,
      0.466500, 0.466500, 0.466500, 0.466445, 0.466400, 0.466400, 0.466400,
      0.466396, 0.466336, 0.466300, 0.466300, 0.466300, 0.466300, 0.466300,
      0.466260, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466167, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466091,
      0.466007, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000,
      0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000,
      0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000,
      0.466000, 0.466000, 0.466000, 0.466000, 0.466000, 0.465978, 0.465908,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, NAN,
      NAN,      0.446844, 0.446851, 0.446929, 0.447008, 0.447088, 0.447170,
      0.447200, 0.447211, 0.447271, 0.447327, 0.447378, 0.447426, 0.447470,
      0.447513, 0.447553, 0.447593, 0.447636, 0.447680, 0.447726, 0.447773,
      0.447821, 0.447872, 0.447900, 0.447900, 0.447935, 0.447993, 0.448054,
      0.448100, 0.448100, 0.448150, 0.448220, 0.448292, 0.448368, 0.448400,
      0.448429, 0.448500, 0.448504, 0.448597, 0.448694, 0.448700, 0.448802,
      0.448900, 0.448931, 0.449000, 0.449082, 0.449200, 0.449262, 0.449313,
      0.449474, 0.449545, 0.449626, 0.449713, 0.449852, 0.450000, 0.450129,
      0.450228, 0.450410, 0.450563, 0.450729, 0.450934, 0.451149, 0.451333,
      0.451582, 0.451829, 0.452063, 0.452330, 0.452644, 0.452893, 0.453219,
      0.453592, 0.453955, 0.454305, 0.454705, 0.455021, 0.455400, 0.455757,
      0.456070, 0.456395, 0.456717, 0.457035, 0.457349, 0.457658, 0.457897,
      0.458196, 0.458491, 0.458763, 0.458979, 0.459288, 0.459501, 0.459708,
      0.459913, 0.460120, 0.460385, 0.460611, 0.460753, 0.460898, 0.461088,
      0.461277, 0.461463, 0.461598, 0.461731, 0.461912, 0.462092, 0.462214,
      0.462348, 0.462524, 0.462699, 0.462786, 0.462944, 0.463114, 0.463242,
      0.463335, 0.463446, 0.463586, 0.463725, 0.463815, 0.463978, 0.464070,
      0.464200, 0.464330, 0.464410, 0.464489, 0.464634, 0.464745, 0.464823,
      0.464900, 0.464954, 0.465103, 0.465179, 0.465255, 0.465330, 0.465400,
      0.465400, 0.465452, 0.465526, 0.465599, 0.465700, 0.465700, 0.465766,
      0.465838, 0.465900, 0.465900, 0.465902, 0.465973, 0.466043, 0.466100,
      0.466100, 0.466103, 0.466173, 0.466200, 0.466211, 0.466280, 0.466300,
      0.466300, 0.466300, 0.466308, 0.466400, 0.466400, 0.466400, 0.466400,
      0.466442, 0.466500, 0.466500, 0.466500, 0.466500, 0.466500, 0.466500,
      0.466509, 0.466575, 0.466600, 0.466600, 0.466600, 0.466600, 0.466600,
      0.466600, 0.466600, 0.466600, 0.466613, 0.466678, 0.466700, 0.466700,
      0.466700, 0.466700, 0.466700, 0.466700, 0.466700, 0.466700, 0.466700,
      0.466700, 0.466700, 0.466700, 0.466700, 0.466713, 0.466775, 0.466800,
      0.466800, 0.466800, 0.466800, 0.466800, 0.466800, 0.466800, 0.466816,
      0.466877, 0.466900, 0.466900, 0.466900, 0.466900, 0.466900, 0.466900,
      0.466949, 0.467000, 0.467000, 0.467000, 0.467000, 0.467048, 0.467100,
      0.467100, 0.467100, 0.467100, 0.467143, 0.467200, 0.467200, 0.467200,
      0.467200, 0.467200, 0.467217, 0.467299, 0.467300, 0.467300, 0.467300,
      0.467300, 0.467328, 0.467400, 0.467400, 0.467400, 0.467400, 0.467400,
      0.467420, 0.467500, 0.467500, 0.467500, 0.467500, 0.467525, 0.467600,
      0.467600, 0.467600, 0.467600, 0.467600, 0.467608, 0.467688, 0.467700,
      0.467700, 0.467700, 0.467700, 0.467700, 0.467700, 0.467751, 0.467800,
      0.467800, 0.467800, 0.467800, 0.467800, 0.467800, 0.467811, 0.467892,
      0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900,
      0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900,
      0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900,
      0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900, 0.467900,
      0.467900, 0.467900, 0.467900, 0.467857, 0.467800, 0.467800, 0.467800,
      0.467800, 0.467800, 0.467777, 0.467727, 0.467700, 0.467700, 0.467700,
      0.467682, 0.467634, 0.467600, 0.467600, 0.467600, 0.467600, 0.467596,
      0.467504, 0.467500, 0.467500, 0.467430, 0.467400, 0.467400, 0.467400,
      0.467374, 0.467300, 0.467300, 0.467300, 0.467226, 0.467200, 0.467200,
      0.467200, 0.467189, 0.467107, 0.467100, 0.467100, 0.467065, 0.467000,
      0.467000, 0.466929, 0.466900, 0.466900, 0.466900, 0.466826, 0.466800,
      0.466800, 0.466800, 0.466735, 0.466700, 0.466700, 0.466700, 0.466656,
      0.466600, 0.466600, 0.466554, 0.466500, 0.466500, 0.466500, 0.466495,
      0.466432, 0.466400, 0.466400, 0.466345, 0.466300, 0.466300, 0.466300,
      0.466300, 0.466245, 0.466200, 0.466200, 0.466200, 0.466200, 0.466160,
      0.466104, 0.466100, 0.466100, 0.466100, 0.466083, 0.466026, 0.466000,
      0.466000, 0.466000, 0.465985, 0.465922, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465889, 0.465819, 0.465800, 0.465800, 0.465800, 0.465800,
      0.465800, 0.465768, 0.465700, 0.465700, 0.465700, 0.465700, 0.465700,
      0.465700, 0.465700, 0.465700, 0.465700, 0.465700, 0.465700, 0.465700,
      0.465602, 0.465600, 0.465600, 0.465600, 0.465600, 0.465600, 0.465600,
      0.465600, 0.465600, 0.465600, 0.465600, 0.465600, 0.465600, 0.465600,
      0.465600, 0.465600, 0.465600, 0.465500, 0.465500, 0.465500, 0.465500,
      0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500,
      0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500,
      0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500,
      0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500,
      0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500,
      0.465500, 0.465454, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, NAN,      0.445934,
      0.445942, 0.446023, 0.446100, 0.446100, 0.446173, 0.446250, 0.446319,
      0.446381, 0.446437, 0.446490, 0.446538, 0.446584, 0.446600, 0.446600,
      0.446608, 0.446652, 0.446696, 0.446744, 0.446792, 0.446843, 0.446894,
      0.446949, 0.447000, 0.447000, 0.447023, 0.447085, 0.447151, 0.447200,
      0.447200, 0.447261, 0.447337, 0.447400, 0.447400, 0.447485, 0.447575,
      0.447600, 0.447667, 0.447700, 0.447778, 0.447891, 0.447910, 0.448000,
      0.448067, 0.448200, 0.448253, 0.448309, 0.448474, 0.448525, 0.448619,
      0.448726, 0.448869, 0.448991, 0.449119, 0.449269, 0.449449, 0.449607,
      0.449784, 0.450016, 0.450199, 0.450388, 0.450646, 0.450877, 0.451194,
      0.451518, 0.451847, 0.452178, 0.452498, 0.452893, 0.453293, 0.453696,
      0.454083, 0.454421, 0.454760, 0.455095, 0.455426, 0.455751, 0.456071,
      0.456314, 0.456596, 0.456902, 0.457204, 0.457428, 0.457724, 0.457967,
      0.458209, 0.458493, 0.458705, 0.458912, 0.459156, 0.459420, 0.459620,
      0.459818, 0.460014, 0.460207, 0.460398, 0.460588, 0.460775, 0.460880,
      0.461044, 0.461226, 0.461403, 0.461493, 0.461663, 0.461792, 0.461913,
      0.462086, 0.462179, 0.462328, 0.462497, 0.462632, 0.462715, 0.462798,
      0.462961, 0.463062, 0.463187, 0.463324, 0.463404, 0.463484, 0.463628,
      0.463743, 0.463821, 0.463900, 0.464055, 0.464155, 0.464232, 0.464308,
      0.464384, 0.464460, 0.464536, 0.464611, 0.464686, 0.464800, 0.464800,
      0.464858, 0.464931, 0.465004, 0.465077, 0.465149, 0.465200, 0.465243,
      0.465300, 0.465300, 0.465356, 0.465400, 0.465447, 0.465500, 0.465500,
      0.465556, 0.465600, 0.465600, 0.465613, 0.465682, 0.465700, 0.465700,
      0.465700, 0.465704, 0.465772, 0.465800, 0.465800, 0.465800, 0.465800,
      0.465808, 0.465875, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465939, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000,
      0.466000, 0.466000, 0.466000, 0.466000, 0.466039, 0.466100, 0.466100,
      0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100,
      0.466100, 0.466100, 0.466197, 0.466200, 0.466200, 0.466200, 0.466200,
      0.466200, 0.466200, 0.466200, 0.466200, 0.466200, 0.466213, 0.466274,
      0.466300, 0.466300, 0.466300, 0.466300, 0.466300, 0.466300, 0.466347,
      0.466400, 0.466400, 0.466400, 0.466400, 0.466400, 0.466400, 0.466421,
      0.466492, 0.466500, 0.466500, 0.466515, 0.466594, 0.466600, 0.466600,
      0.466600, 0.466600, 0.466621, 0.466700, 0.466700, 0.466700, 0.466700,
      0.466700, 0.466701, 0.466784, 0.466800, 0.466800, 0.466800, 0.466805,
      0.466887, 0.466900, 0.466900, 0.466900, 0.466900, 0.466900, 0.466968,
      0.467000, 0.467000, 0.467000, 0.467000, 0.467000, 0.467050, 0.467100,
      0.467100, 0.467100, 0.467100, 0.467100, 0.467100, 0.467100, 0.467100,
      0.467170, 0.467200, 0.467200, 0.467200, 0.467200, 0.467200, 0.467200,
      0.467200, 0.467200, 0.467200, 0.467242, 0.467296, 0.467300, 0.467300,
      0.467300, 0.467300, 0.467300, 0.467300, 0.467300, 0.467300, 0.467300,
      0.467300, 0.467300, 0.467300, 0.467300, 0.467300, 0.467300, 0.467300,
      0.467300, 0.467300, 0.467300, 0.467279, 0.467200, 0.467200, 0.467200,
      0.467200, 0.467200, 0.467200, 0.467200, 0.467200, 0.467200, 0.467160,
      0.467100, 0.467100, 0.467100, 0.467100, 0.467100, 0.467073, 0.467000,
      0.467000, 0.467000, 0.466995, 0.466902, 0.466900, 0.466900, 0.466900,
      0.466869, 0.466824, 0.466800, 0.466800, 0.466786, 0.466700, 0.466700,
      0.466700, 0.466700, 0.466667, 0.466600, 0.466600, 0.466600, 0.466546,
      0.466500, 0.466500, 0.466500, 0.466441, 0.466400, 0.466394, 0.466322,
      0.466300, 0.466300, 0.466300, 0.466243, 0.466200, 0.466200, 0.466200,
      0.466180, 0.466117, 0.466100, 0.466100, 0.466100, 0.466074, 0.466016,
      0.466000, 0.466000, 0.466000, 0.465993, 0.465939, 0.465900, 0.465900,
      0.465882, 0.465830, 0.465800, 0.465800, 0.465800, 0.465800, 0.465800,
      0.465800, 0.465756, 0.465701, 0.465700, 0.465700, 0.465700, 0.465700,
      0.465700, 0.465661, 0.465602, 0.465600, 0.465600, 0.465600, 0.465600,
      0.465600, 0.465600, 0.465573, 0.465508, 0.465500, 0.465500, 0.465500,
      0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465500, 0.465483,
      0.465403, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400, 0.465400,
      0.465340, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, NAN,      0.444800, 0.444800, 0.444804,
      0.444864, 0.444922, 0.444974, 0.445023, 0.445068, 0.445112, 0.445155,
      0.445197, 0.445242, 0.445288, 0.445300, 0.445300, 0.445338, 0.445391,
      0.445447, 0.445500, 0.445500, 0.445525, 0.445589, 0.445656, 0.445700,
      0.445700, 0.445772, 0.445849, 0.445900, 0.445915, 0.446000, 0.446000,
      0.446091, 0.446191, 0.446200, 0.446300, 0.446323, 0.446445, 0.446500,
      0.446600, 0.446652, 0.446703, 0.446800, 0.446935, 0.447017, 0.447138,
      0.447210, 0.447329, 0.447489, 0.447625, 0.447743, 0.447870, 0.448031,
      0.448180, 0.448419, 0.448610, 0.448806, 0.449048, 0.449295, 0.449532,
      0.449828, 0.450147, 0.450485, 0.450900, 0.451218, 0.451595, 0.451990,
      0.452387, 0.452803, 0.453157, 0.453488, 0.453814, 0.454135, 0.454450,
      0.454759, 0.455065, 0.455349, 0.455572, 0.455889, 0.456108, 0.456321,
      0.456532, 0.456753, 0.457027, 0.457249, 0.457450, 0.457650, 0.457847,
      0.458043, 0.458237, 0.458429, 0.458620, 0.458809, 0.458997, 0.459182,
      0.459367, 0.459550, 0.459732, 0.459912, 0.460091, 0.460234, 0.460345,
      0.460519, 0.460692, 0.460782, 0.460934, 0.461101, 0.461185, 0.461336,
      0.461450, 0.461563, 0.461713, 0.461793, 0.461873, 0.462004, 0.462131,
      0.462209, 0.462287, 0.462364, 0.462441, 0.462536, 0.462644, 0.462720,
      0.462796, 0.462871, 0.462993, 0.463121, 0.463196, 0.463270, 0.463344,
      0.463400, 0.463441, 0.463514, 0.463587, 0.463660, 0.463732, 0.463800,
      0.463827, 0.463898, 0.463970, 0.464041, 0.464112, 0.464183, 0.464203,
      0.464273, 0.464300, 0.464313, 0.464383, 0.464402, 0.464471, 0.464500,
      0.464500, 0.464527, 0.464595, 0.464600, 0.464631, 0.464698, 0.464700,
      0.464700, 0.464700, 0.464700, 0.464733, 0.464800, 0.464800, 0.464800,
      0.464800, 0.464800, 0.464800, 0.464800, 0.464862, 0.464900, 0.464900,
      0.464900, 0.464900, 0.464900, 0.464900, 0.464900, 0.464900, 0.464900,
      0.464900, 0.464951, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465061, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465165, 0.465200, 0.465200, 0.465200, 0.465200, 0.465200, 0.465200,
      0.465200, 0.465200, 0.465200, 0.465257, 0.465300, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465339, 0.465400, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465405, 0.465484, 0.465500, 0.465500,
      0.465500, 0.465500, 0.465500, 0.465566, 0.465600, 0.465600, 0.465600,
      0.465600, 0.465667, 0.465700, 0.465700, 0.465700, 0.465700, 0.465700,
      0.465748, 0.465800, 0.465800, 0.465800, 0.465800, 0.465800, 0.465817,
      0.465893, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465941, 0.465997, 0.466000, 0.466000, 0.466000, 0.466000, 0.466000,
      0.466000, 0.466000, 0.466000, 0.466047, 0.466100, 0.466100, 0.466100,
      0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100,
      0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100,
      0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100,
      0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100, 0.466100,
      0.466100, 0.466100, 0.466045, 0.466000, 0.466000, 0.466000, 0.466000,
      0.466000, 0.466000, 0.465987, 0.465900, 0.465900, 0.465900, 0.465900,
      0.465900, 0.465900, 0.465868, 0.465800, 0.465800, 0.465800, 0.465800,
      0.465758, 0.465700, 0.465700, 0.465700, 0.465700, 0.465700, 0.465700,
      0.465634, 0.465600, 0.465600, 0.465600, 0.465600, 0.465599, 0.465536,
      0.465500, 0.465500, 0.465500, 0.465499, 0.465441, 0.465400, 0.465400,
      0.465400, 0.465400, 0.465400, 0.465384, 0.465322, 0.465300, 0.465300,
      0.465300, 0.465300, 0.465300, 0.465241, 0.465200, 0.465200, 0.465200,
      0.465200, 0.465200, 0.465200, 0.465200, 0.465200, 0.465200, 0.465200,
      0.465200, 0.465197, 0.465120, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465060, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.465000,
      NAN,      NAN,      0.444900, 0.444900, 0.444900, 0.444942, 0.445000,
      0.445000, 0.445059, 0.445138, 0.445200, 0.445203, 0.445292, 0.445300,
      0.445380, 0.445481, 0.445500, 0.445595, 0.445611, 0.445700, 0.445759,
      0.445800, 0.445900, 0.445982, 0.446100, 0.446106, 0.446283, 0.446371,
      0.446471, 0.446586, 0.446615, 0.446763, 0.446900, 0.447009, 0.447167,
      0.447339, 0.447502, 0.447656, 0.447806, 0.447976, 0.448214, 0.448449,
      0.448659, 0.448848, 0.449083, 0.449399, 0.449666, 0.449906, 0.450161,
      0.450489, 0.450774, 0.451067, 0.451344, 0.451571, 0.451899, 0.452135,
      0.452371, 0.452631, 0.452915, 0.453166, 0.453431, 0.453673, 0.453886,
      0.454154, 0.454396, 0.454602, 0.454780, 0.454986, 0.455233, 0.455407,
      0.455581, 0.455779, 0.456023, 0.456192, 0.456360, 0.456527, 0.456693,
      0.456858, 0.457022, 0.457184, 0.457273, 0.457407, 0.457568, 0.457727,
      0.457886, 0.458045, 0.458151, 0.458259, 0.458416, 0.458572, 0.458728,
      0.458883, 0.459019, 0.459096, 0.459246, 0.459399, 0.459526, 0.459605,
      0.459757, 0.459909, 0.460060, 0.460155, 0.460259, 0.460404, 0.460478,
      0.460603, 0.460725, 0.460797, 0.460920, 0.460992, 0.461128, 0.461235,
      0.461306, 0.461377, 0.461496, 0.461568, 0.461638, 0.461708, 0.461777,
      0.461893, 0.461966, 0.462034, 0.462100, 0.462121, 0.462190, 0.462258,
      0.462326, 0.462394, 0.462511, 0.462579, 0.462600, 0.462628, 0.462731,
      0.462798, 0.462800, 0.462864, 0.462949, 0.463016, 0.463083, 0.463100,
      0.463166, 0.463200, 0.463200, 0.463300, 0.463300, 0.463347, 0.463400,
      0.463429, 0.463495, 0.463510, 0.463576, 0.463600, 0.463613, 0.463700,
      0.463700, 0.463700, 0.463700, 0.463732, 0.463796, 0.463800, 0.463800,
      0.463840, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464085, 0.464100,
      0.464100, 0.464100, 0.464100, 0.464100, 0.464100, 0.464100, 0.464100,
      0.464100, 0.464100, 0.464100, 0.464136, 0.464197, 0.464200, 0.464200,
      0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464200,
      0.464200, 0.464211, 0.464293, 0.464300, 0.464300, 0.464300, 0.464300,
      0.464300, 0.464300, 0.464355, 0.464400, 0.464400, 0.464400, 0.464400,
      0.464400, 0.464400, 0.464424, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464566, 0.464600, 0.464600, 0.464600,
      0.464600, 0.464600, 0.464647, 0.464700, 0.464700, 0.464700, 0.464700,
      0.464700, 0.464700, 0.464700, 0.464756, 0.464800, 0.464800, 0.464800,
      0.464800, 0.464800, 0.464800, 0.464816, 0.464873, 0.464900, 0.464900,
      0.464900, 0.464900, 0.464906, 0.464963, 0.465000, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465007, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100, 0.465100,
      0.465100, 0.465100, 0.465100, 0.465100, 0.465037, 0.465000, 0.465000,
      0.465000, 0.465000, 0.465000, 0.465000, 0.465000, 0.464992, 0.464917,
      0.464900, 0.464900, 0.464900, 0.464900, 0.464900, 0.464900, 0.464900,
      0.464869, 0.464807, 0.464800, 0.464800, 0.464800, 0.464800, 0.464800,
      0.464800, 0.464800, 0.464800, 0.464738, 0.464700, 0.464700, 0.464700,
      0.464700, 0.464700, 0.464700, 0.464700, 0.464700, 0.464700, 0.464700,
      0.464700, 0.464696, 0.464620, 0.464600, 0.464600, 0.464600, 0.464600,
      0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600,
      0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600,
      0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600,
      0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600,
      0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600,
      0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464600, 0.464584,
      0.464508, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500, 0.464500,
      0.464500, 0.464500, 0.464500, 0.464500, 0.464500, NAN,      NAN,
      0.443900, 0.443900, 0.443931, 0.444013, 0.444099, 0.444100, 0.444182,
      0.444200, 0.444281, 0.444300, 0.444399, 0.444415, 0.444538, 0.444600,
      0.444700, 0.444743, 0.444800, 0.444900, 0.445000, 0.445097, 0.445186,
      0.445288, 0.445400, 0.445500, 0.445600, 0.445716, 0.445819, 0.445975,
      0.446118, 0.446283, 0.446460, 0.446612, 0.446865, 0.447042, 0.447246,
      0.447482, 0.447740, 0.447972, 0.448211, 0.448458, 0.448742, 0.449066,
      0.449384, 0.449612, 0.449855, 0.450113, 0.450404, 0.450661, 0.450931,
      0.451176, 0.451385, 0.451633, 0.451902, 0.452117, 0.452348, 0.452604,
      0.452843, 0.453034, 0.453210, 0.453427, 0.453659, 0.453842, 0.454070,
      0.454273, 0.454442, 0.454616, 0.454866, 0.455044, 0.455209, 0.455373,
      0.455536, 0.455699, 0.455941, 0.456061, 0.456181, 0.456340, 0.456498,
      0.456656, 0.456812, 0.456968, 0.457112, 0.457189, 0.457332, 0.457485,
      0.457619, 0.457695, 0.457842, 0.457994, 0.458145, 0.458248, 0.458346,
      0.458496, 0.458573, 0.458694, 0.458822, 0.458896, 0.459021, 0.459095,
      0.459237, 0.459384, 0.459515, 0.459588, 0.459661, 0.459768, 0.459912,
      0.460028, 0.460100, 0.460171, 0.460285, 0.460428, 0.460535, 0.460605,
      0.460675, 0.460745, 0.460830, 0.460934, 0.461004, 0.461073, 0.461141,
      0.461220, 0.461328, 0.461396, 0.461464, 0.461500, 0.461549, 0.461616,
      0.461683, 0.461800, 0.461800, 0.461833, 0.461900, 0.462000, 0.462000,
      0.462048, 0.462128, 0.462200, 0.462200, 0.462262, 0.462300, 0.462343,
      0.462400, 0.462424, 0.462490, 0.462500, 0.462541, 0.462600, 0.462601,
      0.462666, 0.462700, 0.462747, 0.462800, 0.462800, 0.462800, 0.462856,
      0.462900, 0.462900, 0.462901, 0.462965, 0.463000, 0.463044, 0.463100,
      0.463100, 0.463100, 0.463100, 0.463100, 0.463128, 0.463192, 0.463200,
      0.463200, 0.463200, 0.463200, 0.463200, 0.463200, 0.463200, 0.463200,
      0.463224, 0.463300, 0.463300, 0.463300, 0.463300, 0.463300, 0.463300,
      0.463300, 0.463300, 0.463300, 0.463300, 0.463300, 0.463300, 0.463300,
      0.463300, 0.463356, 0.463400, 0.463400, 0.463400, 0.463400, 0.463400,
      0.463400, 0.463400, 0.463400, 0.463400, 0.463400, 0.463400, 0.463419,
      0.463495, 0.463500, 0.463500, 0.463500, 0.463500, 0.463500, 0.463500,
      0.463562, 0.463600, 0.463600, 0.463600, 0.463600, 0.463600, 0.463600,
      0.463600, 0.463605, 0.463685, 0.463700, 0.463700, 0.463700, 0.463700,
      0.463700, 0.463700, 0.463746, 0.463800, 0.463800, 0.463800, 0.463800,
      0.463800, 0.463804, 0.463862, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463931, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464016, 0.464073, 0.464100, 0.464100, 0.464100,
      0.464100, 0.464100, 0.464100, 0.464100, 0.464125, 0.464181, 0.464200,
      0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464200,
      0.464228, 0.464282, 0.464300, 0.464300, 0.464300, 0.464300, 0.464300,
      0.464300, 0.464300, 0.464300, 0.464300, 0.464300, 0.464300, 0.464300,
      0.464300, 0.464300, 0.464300, 0.464300, 0.464300, 0.464300, 0.464300,
      0.464300, 0.464300, 0.464300, 0.464300, 0.464300, 0.464300, 0.464300,
      0.464300, 0.464300, 0.464300, 0.464300, 0.464300, 0.464218, 0.464200,
      0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464200,
      0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464200, 0.464140,
      0.464100, 0.464100, 0.464100, 0.464100, 0.464100, 0.464100, 0.464100,
      0.464100, 0.464100, 0.464100, 0.464100, 0.464100, 0.464100, 0.464088,
      0.464014, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.464000,
      0.464000, 0.464000, 0.464000, 0.464000, 0.464000, 0.463988, 0.463935,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900, 0.463900,
      0.463900, 0.463900, 0.463900, NAN,      NAN,      0.442800, 0.442800,
      0.442850, 0.442900, 0.443000, 0.443066, 0.443100, 0.443182, 0.443254,
      0.443336, 0.443410, 0.443479, 0.443577, 0.443727, 0.443847, 0.443956,
      0.444100, 0.444179, 0.444330, 0.444494, 0.444648, 0.444795, 0.444962,
      0.445126, 0.445290, 0.445484, 0.445736, 0.445979, 0.446148, 0.446414,
      0.446699, 0.446939, 0.447180, 0.447491, 0.447757, 0.448020, 0.448316,
      0.448565, 0.448826, 0.449102, 0.449374, 0.449638, 0.449875, 0.450082,
      0.450285, 0.450530, 0.450702, 0.450957, 0.451210, 0.451407, 0.451597,
      0.451817, 0.452002, 0.452164, 0.452339, 0.452580, 0.452747, 0.452905,
      0.453063, 0.453221, 0.453377, 0.453533, 0.453688, 0.453913, 0.454096,
      0.454249, 0.454402, 0.454554, 0.454653, 0.454757, 0.454908, 0.455058,
      0.455208, 0.455357, 0.455507, 0.455655, 0.455804, 0.455952, 0.456050,
      0.456147, 0.456295, 0.456441, 0.456588, 0.456667, 0.456780, 0.456926,
      0.457071, 0.457158, 0.457261, 0.457353, 0.457450, 0.457594, 0.457669,
      0.457741, 0.457825, 0.457934, 0.458010, 0.458152, 0.458247, 0.458300,
      0.458376, 0.458508, 0.458578, 0.458648, 0.458734, 0.458836, 0.458906,
      0.458975, 0.459043, 0.459124, 0.459230, 0.459299, 0.459367, 0.459470,
      0.459553, 0.459621, 0.459689, 0.459807, 0.459874, 0.459941, 0.460009,
      0.460076, 0.460100, 0.460160, 0.460227, 0.460293, 0.460410, 0.460476,
      0.460542, 0.460600, 0.460624, 0.460690, 0.460800, 0.460800, 0.460837,
      0.460904, 0.461000, 0.461000, 0.461047, 0.461100, 0.461126, 0.461190,
      0.461205, 0.461269, 0.461300, 0.461300, 0.461311, 0.461375, 0.461400,
      0.461452, 0.461500, 0.461500, 0.461500, 0.461557, 0.461600, 0.461600,
      0.461600, 0.461611, 0.461674, 0.461700, 0.461700, 0.461700, 0.461700,
      0.461783, 0.461800, 0.461800, 0.461800, 0.461800, 0.461858, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461938, 0.462000, 0.462000,
      0.462000, 0.462000, 0.462000, 0.462000, 0.462000, 0.462006, 0.462095,
      0.462100, 0.462100, 0.462100, 0.462100, 0.462100, 0.462100, 0.462100,
      0.462100, 0.462100, 0.462100, 0.462100, 0.462100, 0.462139, 0.462200,
      0.462200, 0.462200, 0.462200, 0.462200, 0.462200, 0.462200, 0.462200,
      0.462200, 0.462200, 0.462200, 0.462200, 0.462200, 0.462200, 0.462244,
      0.462300, 0.462300, 0.462300, 0.462300, 0.462300, 0.462300, 0.462300,
      0.462300, 0.462300, 0.462313, 0.462372, 0.462400, 0.462400, 0.462400,
      0.462400, 0.462415, 0.462474, 0.462500, 0.462500, 0.462500, 0.462500,
      0.462500, 0.462500, 0.462532, 0.462590, 0.462600, 0.462600, 0.462600,
      0.462600, 0.462600, 0.462600, 0.462688, 0.462700, 0.462700, 0.462700,
      0.462700, 0.462700, 0.462700, 0.462700, 0.462700, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462807, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462935, 0.463000, 0.463000, 0.463000, 0.463000,
      0.463000, 0.463000, 0.463000, 0.463000, 0.463000, 0.463000, 0.462956,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900, 0.462900,
      0.462900, 0.462900, 0.462871, 0.462839, 0.462808, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800, 0.462800,
      0.462800, NAN,      NAN,      0.442343, 0.442357, 0.442400, 0.442521,
      0.442642, 0.442723, 0.442793, 0.442941, 0.443053, 0.443170, 0.443325,
      0.443496, 0.443656, 0.443831, 0.443995, 0.444176, 0.444409, 0.444575,
      0.444806, 0.445001, 0.445195, 0.445436, 0.445671, 0.445917, 0.446159,
      0.446401, 0.446648, 0.446920, 0.447130, 0.447347, 0.447587, 0.447767,
      0.448013, 0.448215, 0.448430, 0.448657, 0.448834, 0.449056, 0.449287,
      0.449522, 0.449685, 0.449919, 0.450106, 0.450297, 0.450522, 0.450679,
      0.450901, 0.451089, 0.451242, 0.451442, 0.451646, 0.451797, 0.451947,
      0.452096, 0.452266, 0.452488, 0.452639, 0.452785, 0.452931, 0.453076,
      0.453221, 0.453365, 0.453509, 0.453652, 0.453795, 0.453938, 0.454080,
      0.454222, 0.454363, 0.454504, 0.454623, 0.454693, 0.454826, 0.454966,
      0.455106, 0.455223, 0.455293, 0.455425, 0.455564, 0.455652, 0.455742,
      0.455881, 0.455960, 0.456059, 0.456197, 0.456318, 0.456387, 0.456513,
      0.456600, 0.456690, 0.456814, 0.456883, 0.457002, 0.457071, 0.457140,
      0.457219, 0.457356, 0.457400, 0.457530, 0.457633, 0.457700, 0.457739,
      0.457874, 0.457955, 0.458022, 0.458089, 0.458207, 0.458273, 0.458340,
      0.458400, 0.458423, 0.458489, 0.458605, 0.458671, 0.458700, 0.458706,
      0.458818, 0.458884, 0.458949, 0.459000, 0.459030, 0.459095, 0.459160,
      0.459200, 0.459240, 0.459309, 0.459419, 0.459484, 0.459549, 0.459600,
      0.459628, 0.459693, 0.459707, 0.459772, 0.459836, 0.459900, 0.459915,
      0.459979, 0.460000, 0.460058, 0.460122, 0.460186, 0.460200, 0.460264,
      0.460300, 0.460342, 0.460400, 0.460400, 0.460466, 0.460500, 0.460500,
      0.460524, 0.460587, 0.460600, 0.460626, 0.460700, 0.460700, 0.460700,
      0.460700, 0.460700, 0.460740, 0.460800, 0.460800, 0.460800, 0.460800,
      0.460800, 0.460815, 0.460878, 0.460900, 0.460900, 0.460900, 0.460900,
      0.460900, 0.460900, 0.460900, 0.460935, 0.461000, 0.461000, 0.461000,
      0.461000, 0.461000, 0.461000, 0.461000, 0.461000, 0.461078, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461180, 0.461200, 0.461200, 0.461200, 0.461200, 0.461200,
      0.461200, 0.461200, 0.461200, 0.461200, 0.461200, 0.461200, 0.461200,
      0.461213, 0.461275, 0.461300, 0.461300, 0.461300, 0.461300, 0.461300,
      0.461300, 0.461300, 0.461300, 0.461300, 0.461300, 0.461300, 0.461326,
      0.461384, 0.461400, 0.461400, 0.461400, 0.461400, 0.461400, 0.461400,
      0.461400, 0.461400, 0.461420, 0.461500, 0.461500, 0.461500, 0.461500,
      0.461500, 0.461500, 0.461500, 0.461500, 0.461559, 0.461600, 0.461600,
      0.461600, 0.461600, 0.461600, 0.461600, 0.461628, 0.461684, 0.461700,
      0.461700, 0.461700, 0.461700, 0.461700, 0.461700, 0.461700, 0.461700,
      0.461734, 0.461788, 0.461800, 0.461800, 0.461800, 0.461800, 0.461800,
      0.461800, 0.461800, 0.461800, 0.461829, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900, 0.461900,
      NAN,      0.441485, 0.441492, 0.441577, 0.441702, 0.441831, 0.441927,
      0.442020, 0.442170, 0.442267, 0.442397, 0.442554, 0.442713, 0.442948,
      0.443126, 0.443338, 0.443585, 0.443818, 0.444061, 0.444304, 0.444548,
      0.444821, 0.445130, 0.445349, 0.445645, 0.445904, 0.446111, 0.446407,
      0.446666, 0.446892, 0.447081, 0.447290, 0.447536, 0.447734, 0.447970,
      0.448191, 0.448366, 0.448540, 0.448767, 0.448981, 0.449149, 0.449315,
      0.449480, 0.449643, 0.449858, 0.450066, 0.450225, 0.450384, 0.450541,
      0.450696, 0.450851, 0.451008, 0.451237, 0.451410, 0.451561, 0.451711,
      0.451860, 0.452009, 0.452157, 0.452355, 0.452550, 0.452696, 0.452840,
      0.452985, 0.453128, 0.453271, 0.453414, 0.453555, 0.453697, 0.453769,
      0.453878, 0.454019, 0.454159, 0.454298, 0.454437, 0.454538, 0.454615,
      0.454753, 0.454891, 0.455029, 0.455133, 0.455204, 0.455342, 0.455400,
      0.455516, 0.455626, 0.455695, 0.455813, 0.455881, 0.455999, 0.456100,
      0.456171, 0.456304, 0.456372, 0.456440, 0.456517, 0.456627, 0.456695,
      0.456763, 0.456861, 0.456948, 0.457016, 0.457083, 0.457201, 0.457268,
      0.457335, 0.457404, 0.457519, 0.457585, 0.457652, 0.457700, 0.457735,
      0.457801, 0.457867, 0.457965, 0.458048, 0.458114, 0.458180, 0.458200,
      0.458260, 0.458325, 0.458390, 0.458405, 0.458470, 0.458570, 0.458600,
      0.458614, 0.458678, 0.458700, 0.458757, 0.458800, 0.458835, 0.458899,
      0.458963, 0.459000, 0.459040, 0.459100, 0.459118, 0.459182, 0.459200,
      0.459218, 0.459300, 0.459300, 0.459399, 0.459400, 0.459453, 0.459500,
      0.459504, 0.459567, 0.459600, 0.459644, 0.459700, 0.459721, 0.459785,
      0.459800, 0.459800, 0.459825, 0.459888, 0.459901, 0.459964, 0.460000,
      0.460000, 0.460004, 0.460067, 0.460100, 0.460100, 0.460100, 0.460100,
      0.460114, 0.460187, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460237, 0.460300, 0.460300, 0.460300, 0.460300, 0.460300, 0.460300,
      0.460300, 0.460300, 0.460300, 0.460300, 0.460300, 0.460300, 0.460300,
      0.460300, 0.460300, 0.460322, 0.460400, 0.460400, 0.460400, 0.460400,
      0.460400, 0.460400, 0.460400, 0.460400, 0.460400, 0.460500, 0.460500,
      0.460500, 0.460500, 0.460500, 0.460500, 0.460500, 0.460500, 0.460500,
      0.460500, 0.460502, 0.460562, 0.460600, 0.460600, 0.460600, 0.460600,
      0.460600, 0.460600, 0.460600, 0.460600, 0.460600, 0.460600, 0.460658,
      0.460700, 0.460700, 0.460700, 0.460700, 0.460700, 0.460700, 0.460700,
      0.460700, 0.460700, 0.460700, 0.460740, 0.460797, 0.460800, 0.460800,
      0.460800, 0.460800, 0.460800, 0.460800, 0.460800, 0.460800, 0.460800,
      0.460900, 0.460900, 0.460900, 0.460900, 0.460900, 0.460900, 0.460900,
      0.460900, 0.460962, 0.461000, 0.461000, 0.461000, 0.461000, 0.461000,
      0.461000, 0.461000, 0.461000, 0.461000, 0.461000, 0.461000, 0.461025,
      0.461071, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100, 0.461100,
      0.461100, 0.461100, 0.461100, 0.461100, NAN,      NAN,      0.443105,
      0.443129, 0.443362, 0.443538, 0.443746, 0.443956, 0.444128, 0.444345,
      0.444539, 0.444724, 0.444905, 0.445083, 0.445341, 0.445536, 0.445747,
      0.445978, 0.446154, 0.446324, 0.446538, 0.446759, 0.446925, 0.447089,
      0.447327, 0.447512, 0.447708, 0.447931, 0.448088, 0.448244, 0.448447,
      0.448652, 0.448804, 0.448955, 0.449104, 0.449253, 0.449450, 0.449647,
      0.449792, 0.449936, 0.450080, 0.450222, 0.450364, 0.450502, 0.450572,
      0.450683, 0.450822, 0.450960, 0.451097, 0.451233, 0.451369, 0.451505,
      0.451620, 0.451687, 0.451807, 0.451941, 0.452074, 0.452206, 0.452339,
      0.452470, 0.452551, 0.452633, 0.452764, 0.452895, 0.453025, 0.453100,
      0.453185, 0.453314, 0.453443, 0.453572, 0.453650, 0.453729, 0.453858,
      0.453986, 0.454113, 0.454200, 0.454268, 0.454396, 0.454511, 0.454575,
      0.454676, 0.454803, 0.454915, 0.454978, 0.455082, 0.455154, 0.455235,
      0.455361, 0.455487, 0.455557, 0.455639, 0.455733, 0.455796, 0.455817,
      0.455943, 0.456000, 0.456048, 0.456121, 0.456246, 0.456300, 0.456348,
      0.456422, 0.456523, 0.456586, 0.456696, 0.456760, 0.456800, 0.456834,
      0.456897, 0.456959, 0.457041, 0.457133, 0.457194, 0.457206, 0.457268,
      0.457300, 0.457342, 0.457408, 0.457516, 0.457577, 0.457600, 0.457651,
      0.457713, 0.457774, 0.457800, 0.457848, 0.457900, 0.457921, 0.457982,
      0.458000, 0.458056, 0.458134, 0.458200, 0.458200, 0.458201, 0.458263,
      0.458300, 0.458336, 0.458397, 0.458408, 0.458469, 0.458500, 0.458500,
      0.458600, 0.458600, 0.458600, 0.458637, 0.458698, 0.458700, 0.458700,
      0.458763, 0.458800, 0.458800, 0.458815, 0.458876, 0.458900, 0.458900,
      0.458941, 0.459000, 0.459000, 0.459000, 0.459000, 0.459052, 0.459100,
      0.459100, 0.459100, 0.459100, 0.459153, 0.459200, 0.459200, 0.459200,
      0.459200, 0.459200, 0.459235, 0.459300, 0.459300, 0.459300, 0.459300,
      0.459300, 0.459317, 0.459398, 0.459400, 0.459400, 0.459400, 0.459400,
      0.459400, 0.459400, 0.459400, 0.459440, 0.459500, 0.459500, 0.459500,
      0.459500, 0.459500, 0.459500, 0.459500, 0.459500, 0.459500, 0.459500,
      0.459500, 0.459500, 0.459500, 0.459500, 0.459500, 0.459500, 0.459500,
      0.459500, 0.459500, 0.459500, 0.459500, 0.459500, 0.459500, 0.459500,
      0.459500, 0.459500, 0.459500, 0.459560, 0.459600, 0.459600, 0.459600,
      0.459600, 0.459600, 0.459600, 0.459600, 0.459600, 0.459600, 0.459600,
      0.459627, 0.459685, 0.459700, 0.459700, 0.459700, 0.459700, 0.459700,
      0.459700, 0.459700, 0.459748, 0.459800, 0.459800, 0.459800, 0.459800,
      0.459800, 0.459800, 0.459800, 0.459800, 0.459800, 0.459800, 0.459830,
      0.459900, 0.459900, 0.459900, 0.459900, 0.459900, 0.459900, 0.459900,
      0.459900, 0.459900, 0.459900, 0.459996, 0.460000, 0.460000, 0.460000,
      0.460000, 0.460000, 0.460000, 0.460000, 0.460000, 0.460000, 0.460000,
      0.460000, 0.460000, 0.460030, 0.460100, 0.460100, 0.460100, 0.460100,
      0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100,
      0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100,
      0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460104, 0.460180,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200, 0.460200,
      0.460200, 0.460162, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100,
      0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100,
      0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100, 0.460100,
      0.460100, 0.460100, NAN,      NAN,      0.446081, 0.446096, 0.446172,
      0.446291, 0.446437, 0.446583, 0.446664, 0.446772, 0.446916, 0.447059,
      0.447202, 0.447344, 0.447400, 0.447526, 0.447666, 0.447803, 0.447872,
      0.447983, 0.448121, 0.448229, 0.448298, 0.448431, 0.448567, 0.448651,
      0.448718, 0.448786, 0.448905, 0.449038, 0.449135, 0.449204, 0.449318,
      0.449384, 0.449498, 0.449600, 0.449660, 0.449790, 0.449910, 0.449975,
      0.450078, 0.450207, 0.450318, 0.450382, 0.450493, 0.450620, 0.450724,
      0.450788, 0.450902, 0.451029, 0.451155, 0.451241, 0.451308, 0.451433,
      0.451559, 0.451684, 0.451800, 0.451833, 0.451957, 0.452081, 0.452205,
      0.452328, 0.452426, 0.452487, 0.452597, 0.452718, 0.452820, 0.452881,
      0.452982, 0.453103, 0.453200, 0.453243, 0.453363, 0.453483, 0.453501,
      0.453560, 0.453639, 0.453729, 0.453788, 0.453893, 0.453955, 0.454028,
      0.454123, 0.454181, 0.454200, 0.454247, 0.454311, 0.454414, 0.454471,
      0.454500, 0.454574, 0.454645, 0.454700, 0.454710, 0.454767, 0.454850,
      0.454900, 0.454900, 0.454994, 0.455054, 0.455100, 0.455137, 0.455200,
      0.455200, 0.455279, 0.455346, 0.455400, 0.455421, 0.455500, 0.455500,
      0.455562, 0.455638, 0.455694, 0.455701, 0.455758, 0.455800, 0.455842,
      0.455928, 0.455984, 0.456000, 0.456047, 0.456100, 0.456120, 0.456200,
      0.456200, 0.456258, 0.456300, 0.456300, 0.456397, 0.456405, 0.456465,
      0.456530, 0.456600, 0.456600, 0.456600, 0.456679, 0.456700, 0.456736,
      0.456800, 0.456800, 0.456800, 0.456868, 0.456900, 0.456927, 0.457000,
      0.457000, 0.457000, 0.457050, 0.457100, 0.457100, 0.457100, 0.457173,
      0.457200, 0.457200, 0.457200, 0.457200, 0.457274, 0.457300, 0.457300,
      0.457300, 0.457300, 0.457375, 0.457400, 0.457400, 0.457400, 0.457400,
      0.457400, 0.457456, 0.457500, 0.457500, 0.457500, 0.457500, 0.457500,
      0.457500, 0.457500, 0.457551, 0.457600, 0.457600, 0.457600, 0.457600,
      0.457600, 0.457600, 0.457600, 0.457600, 0.457600, 0.457600, 0.457600,
      0.457600, 0.457600, 0.457600, 0.457600, 0.457600, 0.457600, 0.457600,
      0.457600, 0.457600, 0.457600, 0.457600, 0.457600, 0.457600, 0.457600,
      0.457600, 0.457603, 0.457608, 0.457615, 0.457626, 0.457637, 0.457655,
      0.457664, 0.457628, 0.457651, 0.457663, 0.457665, 0.457661, 0.457642,
      0.457617, 0.457651, 0.457669, 0.457669, 0.457650, 0.457610, 0.457585,
      0.457601, 0.457632, 0.457640, 0.457619, 0.457618, 0.457618, 0.457608,
      0.457567, 0.457546, 0.457546, 0.457564, 0.457590, 0.457565, 0.457562,
      0.457558, 0.457554, 0.457530, 0.457473, 0.457491, 0.457502, 0.457500,
      0.457489, 0.457466, 0.457455, 0.457443, 0.457432, 0.457431, 0.457430,
      0.457430, 0.457429, 0.457428, 0.457403, 0.457364, 0.457409, 0.457450,
      0.457451, 0.457451, 0.457452, 0.457455, 0.457457, 0.457459, 0.457466,
      0.457531, 0.457544, 0.457587, 0.457648, 0.457651, 0.457667, 0.457690,
      0.457720, 0.457754, 0.457783, 0.457806, 0.457822, 0.457841, 0.457864,
      0.457894, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457966, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000, 0.458000,
      0.458000, 0.457997, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900, 0.457900,
      0.457900, NAN,      0.447483, 0.447490, 0.447610, 0.447700, 0.447769,
      0.447898, 0.447963, 0.448055, 0.448142, 0.448200, 0.448239, 0.448366,
      0.448446, 0.448519, 0.448623, 0.448686, 0.448798, 0.448862, 0.448949,
      0.449037, 0.449099, 0.449162, 0.449248, 0.449336, 0.449398, 0.449509,
      0.449571, 0.449665, 0.449788, 0.449900, 0.449933, 0.450055, 0.450100,
      0.450198, 0.450300, 0.450339, 0.450460, 0.450500, 0.450600, 0.450700,
      0.450739, 0.450858, 0.450900, 0.450996, 0.451100, 0.451116, 0.451175,
      0.451268, 0.451343, 0.451402, 0.451510, 0.451568, 0.451600, 0.451669,
      0.451743, 0.451800, 0.451808, 0.451866, 0.451948, 0.452032, 0.452089,
      0.452100, 0.452155, 0.452224, 0.452300, 0.452354, 0.452469, 0.452500,
      0.452549, 0.452612, 0.452713, 0.452770, 0.452800, 0.452869, 0.452942,
      0.452999, 0.453106, 0.453163, 0.453239, 0.453327, 0.453383, 0.453400,
      0.453494, 0.453554, 0.453620, 0.453717, 0.453773, 0.453860, 0.453900,
      0.453943, 0.454000, 0.454106, 0.454163, 0.454238, 0.454326, 0.454382,
      0.454400, 0.454489, 0.454551, 0.454600, 0.454613, 0.454669, 0.454749,
      0.454800, 0.454836, 0.454892, 0.454900, 0.454959, 0.455000, 0.455055,
      0.455100, 0.455100, 0.455181, 0.455200, 0.455200, 0.455244, 0.455300,
      0.455304, 0.455385, 0.455400, 0.455400, 0.455429, 0.455500, 0.455500,
      0.455500, 0.455547, 0.455600, 0.455600, 0.455600, 0.455670, 0.455700,
      0.455700, 0.455700, 0.455700, 0.455771, 0.455800, 0.455800, 0.455811,
      0.455892, 0.455900, 0.455900, 0.455933, 0.456000, 0.456000, 0.456000,
      0.456053, 0.456100, 0.456100, 0.456100, 0.456100, 0.456137, 0.456200,
      0.456200, 0.456200, 0.456236, 0.456306, 0.456313, 0.456323, 0.456331,
      0.456351, 0.456376, 0.456415, 0.456363, 0.456391, 0.456428, 0.456473,
      0.456496, 0.456505, 0.456478, 0.456462, 0.456454, 0.456550, 0.456487,
      0.456518, 0.456537, 0.456531, 0.456519, 0.456516, 0.456515, 0.456479,
      0.456440, 0.456484, 0.456477, 0.456455, 0.456394, 0.456341, 0.456345,
      0.456293, 0.456236, 0.456214, 0.456192, 0.456090, 0.456077, 0.456046,
      0.455982, 0.455900, 0.455857, 0.455768, 0.455724, 0.455653, 0.455565,
      0.455447, 0.455388, 0.455307, 0.455218, 0.455093, 0.454980, 0.454855,
      0.454728, 0.454593, 0.454460, 0.454336, 0.454209, 0.454096, 0.453953,
      0.453772, 0.453585, 0.453479, 0.453292, 0.453121, 0.452940, 0.452715,
      0.452545, 0.452362, 0.452142, 0.451916, 0.451746, 0.451507, 0.451255,
      0.451038, 0.450834, 0.450585, 0.450323, 0.450061, 0.449800, 0.449539,
      0.449279, 0.449031, 0.448741, 0.448432, 0.448141, 0.447864, 0.447596,
      0.447305, 0.447017, 0.446754, 0.446527, 0.446191, 0.445930, 0.445674,
      0.445407, 0.445150, 0.444835, 0.444566, 0.444293, 0.444039, 0.443844,
      0.443605, 0.443368, 0.443141, 0.442952, 0.442741, 0.442541, 0.442369,
      0.442205, 0.442084, 0.441958, 0.441807, 0.441701, 0.441627, 0.441552,
      0.441479, 0.441479, 0.441470, 0.441445, 0.441439, 0.441462, 0.441493,
      0.441580, 0.441668, 0.441761, 0.441851, 0.441964, 0.442080, 0.442197,
      0.442354, 0.442544, 0.442737, 0.442884, 0.443018, 0.443219, 0.443424,
      0.443631, 0.443872, 0.444095, 0.444312, 0.444475, 0.444660, 0.444839,
      0.445038, 0.445247, 0.445424, 0.445595, 0.445753, 0.445883, 0.446015,
      0.446149, 0.446271, 0.446368, 0.446465, 0.446586, 0.446711, 0.446835,
      0.446923, 0.447011, 0.447098, 0.447176, 0.447252, 0.447328, 0.447403,
      0.447476, 0.447549, 0.447622, 0.447694, 0.447765, 0.447836, 0.447907,
      0.447978, 0.448039, 0.448097, 0.448155, 0.448214, 0.448284, 0.448357,
      0.448430, 0.448502, 0.448573, 0.448644, 0.448714, 0.448773, 0.448823,
      0.448873, 0.448926, 0.448985, 0.449045, 0.449107, 0.449185, 0.449262,
      0.449331, 0.449380, 0.449429, 0.449488, 0.449557, 0.449626, 0.449672,
      0.449713, 0.449747, 0.449767, 0.449787, 0.449842, 0.449900, 0.449924,
      0.449924, 0.449941, 0.449986, 0.450031, 0.450074, 0.450113, 0.450137,
      0.450160, 0.450185, 0.450217, 0.450266, 0.450289, 0.450289, 0.450343,
      0.450414, 0.450414, 0.450414, 0.450414, 0.450414, 0.450419, 0.450449,
      0.450460, 0.450469, 0.450530, 0.450549, 0.450559, 0.450592, 0.450595,
      0.450595, 0.450595, 0.450595, 0.450595, 0.450595, 0.450595, 0.450595,
      0.450595, 0.450595, 0.450613, 0.450614, 0.450614, 0.450614, 0.450614,
      0.450614, 0.450614, 0.450614, 0.450614, 0.450614, 0.450614, 0.450614,
      0.450611, 0.450569, 0.450569, 0.450569, 0.450569, 0.450569, 0.450569,
      0.450569, 0.450534, 0.450435, 0.450435, 0.450435, 0.450435, 0.450435,
      0.450435, 0.450411, 0.450390, 0.450390, 0.450390, 0.450390, 0.450390,
      0.450261, 0.450256, 0.450256, 0.450256, 0.450217, 0.450211, 0.450211,
      0.450211, 0.450211, 0.450211, 0.450184, 0.450076, 0.450076, 0.450060,
      0.450032, 0.450032, 0.450032, 0.450032, 0.450032, 0.450005, 0.449953,
      0.449897, 0.449897, 0.449897, 0.449897, 0.449872, 0.449853, 0.449853,
      0.449853, 0.449853, 0.449830, 0.449776, 0.449718, 0.449718, 0.449690,
      0.449673, 0.449655, 0.449583, 0.449546, 0.449539, 0.449539, 0.449524,
      0.449488, 0.449406, 0.449372, 0.449360, 0.449346, 0.449294, 0.449226,
      0.449226, 0.449201, 0.449167, 0.449121, 0.449045, 0.449012, 0.449002,
      0.448951, 0.448868, 0.448865, 0.448829, 0.448805, 0.448740, 0.448649,
      0.448615, 0.448613, 0.448525, 0.448479, 0.448467, NAN,      NAN,
      0.448419, 0.448431, 0.448500, 0.448538, 0.448599, 0.448709, 0.448770,
      0.448862, 0.448941, 0.449000, 0.449012, 0.449072, 0.449164, 0.449242,
      0.449300, 0.449311, 0.449371, 0.449461, 0.449540, 0.449599, 0.449608,
      0.449667, 0.449752, 0.449835, 0.449894, 0.449902, 0.449961, 0.450039,
      0.450100, 0.450136, 0.450195, 0.450303, 0.450361, 0.450400, 0.450427,
      0.450485, 0.450586, 0.450651, 0.450700, 0.450733, 0.450824, 0.450882,
      0.450900, 0.450995, 0.451100, 0.451112, 0.451170, 0.451254, 0.451334,
      0.451391, 0.451497, 0.451506, 0.451563, 0.451639, 0.451726, 0.451783,
      0.451879, 0.451900, 0.451953, 0.452018, 0.452115, 0.452171, 0.452200,
      0.452267, 0.452340, 0.452395, 0.452401, 0.452457, 0.452525, 0.452600,
      0.452623, 0.452679, 0.452700, 0.452739, 0.452794, 0.452898, 0.452904,
      0.452959, 0.453000, 0.453038, 0.453100, 0.453100, 0.453167, 0.453200,
      0.453243, 0.453298, 0.453303, 0.453358, 0.453400, 0.453439, 0.453500,
      0.453502, 0.453585, 0.453600, 0.453661, 0.453700, 0.453705, 0.453789,
      0.453869, 0.453900, 0.453942, 0.454000, 0.454003, 0.454080, 0.454100,
      0.454142, 0.454222, 0.454300, 0.454300, 0.454365, 0.454445, 0.454500,
      0.454507, 0.454587, 0.454600, 0.454648, 0.454700, 0.454708, 0.454789,
      0.454800, 0.454849, 0.454929, 0.455002, 0.455004, 0.455012, 0.455074,
      0.455150, 0.455170, 0.455196, 0.455234, 0.455261, 0.455263, 0.455314,
      0.455331, 0.455347, 0.455366, 0.455366, 0.455354, 0.455370, 0.455381,
      0.455401, 0.455430, 0.455418, 0.455412, 0.455413, 0.455421, 0.455434,
      0.455392, 0.455382, 0.455365, 0.455347, 0.455338, 0.455343, 0.455305,
      0.455265, 0.455226, 0.455217, 0.455197, 0.455182, 0.455139, 0.455062,
      0.455000, 0.454967, 0.454883, 0.454824, 0.454785, 0.454731, 0.454659,
      0.454522, 0.454424, 0.454345, 0.454233, 0.454127, 0.454041, 0.453935,
      0.453826, 0.453682, 0.453537, 0.453408, 0.453264, 0.453081, 0.452907,
      0.452753, 0.452568, 0.452372, 0.452182, 0.452007, 0.451826, 0.451563,
      0.451312, 0.451080, 0.450820, 0.450592, 0.450293, 0.450011, 0.449749,
      0.449359, 0.449071, 0.448738, 0.448400, 0.448051, 0.447698, 0.447310,
      0.446902, 0.446509, 0.446127, 0.445707, 0.445236, 0.444816, 0.444322,
      0.443828, 0.443341, 0.442850, 0.442293, 0.441786, 0.441272, 0.440670,
      0.440074, 0.439478, 0.438860, 0.438232, 0.437532, 0.436877, 0.436218,
      0.435506, 0.434783, 0.434067, 0.433364, 0.432588, 0.431813, 0.431024,
      0.430220, 0.429415, 0.428635, 0.427800, 0.426957, 0.426096, 0.425202,
      0.424363, 0.423496, 0.422593, 0.421722, 0.420790, 0.419914, 0.419061,
      0.418241, 0.417359, 0.416508, 0.415687, 0.414837, 0.414032, 0.413235,
      0.412485, 0.411764, 0.411047, 0.410368, 0.409740, 0.409154, 0.408619,
      0.408085, 0.407629, 0.407219, 0.406886, 0.406584, 0.406338, 0.406222,
      0.406130, 0.406146, 0.406228, 0.406312, 0.406491, 0.406734, 0.407080,
      0.407520, 0.407971, 0.408456, 0.408956, 0.409518, 0.410120, 0.410737,
      0.411347, 0.411920, 0.412494, 0.413080, 0.413566, 0.414055, 0.414538,
      0.414960, 0.415310, 0.415604, 0.415898, 0.416191, 0.416441, 0.416635,
      0.416829, 0.416992, 0.417151, 0.417309, 0.417438, 0.417566, 0.417695,
      0.417825, 0.417956, 0.418087, 0.418218, 0.418304, 0.418388, 0.418472,
      0.418556, 0.418647, 0.418739, 0.418831, 0.418923, 0.419014, 0.419100,
      0.419187, 0.419273, 0.419360, 0.419447, 0.419535, 0.419624, 0.419712,
      0.419801, 0.419893, 0.419989, 0.420086, 0.420182, 0.420274, 0.420335,
      0.420396, 0.420458, 0.420520, 0.420594, 0.420668, 0.420741, 0.420811,
      0.420870, 0.420930, 0.420989, 0.421035, 0.421078, 0.421121, 0.421153,
      0.421175, 0.421197, 0.421228, 0.421271, 0.421315, 0.421350, 0.421374,
      0.421398, 0.421442, 0.421495, 0.421548, 0.421548, 0.421548, 0.421548,
      0.421548, 0.421550, 0.421579, 0.421608, 0.421641, 0.421674, 0.421706,
      0.421738, 0.421743, 0.421710, 0.421689, 0.421689, 0.421703, 0.421738,
      0.421739, 0.421702, 0.421689, 0.421689, 0.421689, 0.421689, 0.421689,
      0.421689, 0.421689, 0.421689, 0.421689, 0.421689, 0.421656, 0.421621,
      0.421621, 0.421621, 0.421621, 0.421621, 0.421621, 0.421621, 0.421625,
      0.421634, 0.421634, 0.421680, 0.421701, 0.421728, 0.421775, 0.421775,
      0.421775, 0.421775, 0.421775, 0.421775, 0.421775, 0.421775, 0.421775,
      0.421775, 0.421775, 0.421775, 0.421775, 0.421775, 0.421775, 0.421775,
      0.421775, 0.421775, 0.421775, 0.421775, 0.421775, 0.421775, 0.421719,
      0.421708, 0.421708, 0.421708, 0.421708, 0.421708, 0.421700, 0.421641,
      0.421641, 0.421641, 0.421641, 0.421641, 0.421615, 0.421574, 0.421574,
      0.421574, 0.421500, 0.421500, 0.421433, 0.421433, 0.421433, 0.421410,
      0.421366, 0.421366, 0.421366, 0.421299, 0.421299, 0.421299, 0.421232,
      0.421163, 0.421145, 0.421091, 0.421091, 0.421078, 0.421024, 0.421024,
      0.420970, 0.420903, 0.420843, 0.420816, 0.420813, 0.420751, 0.420704,
      0.420643, 0.420583, 0.420541, 0.420541, 0.420498, 0.420446, 0.420375,
      0.420266, 0.420266, 0.420218, 0.420161, 0.420105, 0.420052, 0.419991,
      0.419941, 0.419891, 0.419810, 0.419700, 0.419650, 0.419650, 0.419529,
      0.419419, 0.419375, 0.419363, 0.419208, 0.419130, 0.419100, 0.419042,
      0.418888, 0.418841, 0.418747, 0.418647, 0.418576, 0.418525, 0.418401,
      0.418337, 0.418219, 0.418088, NAN,      NAN,      0.448900, 0.448904,
      0.448962, 0.449041, 0.449129, 0.449187, 0.449290, 0.449353, 0.449400,
      0.449419, 0.449477, 0.449569, 0.449600, 0.449650, 0.449716, 0.449815,
      0.449873, 0.449900, 0.449938, 0.449995, 0.450102, 0.450160, 0.450200,
      0.450248, 0.450331, 0.450388, 0.450400, 0.450451, 0.450500, 0.450529,
      0.450621, 0.450677, 0.450700, 0.450740, 0.450796, 0.450804, 0.450900,
      0.450900, 0.450940, 0.451026, 0.451082, 0.451100, 0.451100, 0.451149,
      0.451200, 0.451210, 0.451265, 0.451300, 0.451352, 0.451431, 0.451487,
      0.451500, 0.451594, 0.451600, 0.451615, 0.451713, 0.451768, 0.451800,
      0.451856, 0.451933, 0.451988, 0.452000, 0.452096, 0.452100, 0.452115,
      0.452212, 0.452267, 0.452300, 0.452353, 0.452431, 0.452486, 0.452500,
      0.452590, 0.452649, 0.452700, 0.452716, 0.452786, 0.452868, 0.452900,
      0.452946, 0.453000, 0.453015, 0.453094, 0.453100, 0.453155, 0.453200,
      0.453215, 0.453300, 0.453300, 0.453360, 0.453400, 0.453421, 0.453500,
      0.453500, 0.453561, 0.453600, 0.453601, 0.453606, 0.453691, 0.453713,
      0.453728, 0.453752, 0.453778, 0.453809, 0.453843, 0.453861, 0.453880,
      0.453932, 0.453981, 0.453988, 0.453970, 0.454027, 0.454050, 0.454070,
      0.454099, 0.454127, 0.454131, 0.454109, 0.454109, 0.454112, 0.454154,
      0.454223, 0.454193, 0.454204, 0.454237, 0.454234, 0.454237, 0.454191,
      0.454167, 0.454186, 0.454210, 0.454186, 0.454175, 0.454125, 0.454083,
      0.454102, 0.454056, 0.454034, 0.453985, 0.453910, 0.453870, 0.453774,
      0.453707, 0.453653, 0.453573, 0.453480, 0.453417, 0.453343, 0.453214,
      0.453113, 0.453001, 0.452865, 0.452757, 0.452574, 0.452439, 0.452317,
      0.452078, 0.451877, 0.451697, 0.451516, 0.451334, 0.451123, 0.450863,
      0.450652, 0.450434, 0.450195, 0.449881, 0.449639, 0.449324, 0.449014,
      0.448681, 0.448308, 0.447986, 0.447609, 0.447250, 0.446883, 0.446481,
      0.446073, 0.445654, 0.445226, 0.444792, 0.444319, 0.443817, 0.443322,
      0.442778, 0.442241, 0.441709, 0.441095, 0.440491, 0.439909, 0.439285,
      0.438589, 0.437884, 0.437172, 0.436477, 0.435773, 0.435040, 0.434220,
      0.433370, 0.432540, 0.431700, 0.430790, 0.429912, 0.428907, 0.427946,
      0.426978, 0.425953, 0.424873, 0.423805, 0.422700, 0.421557, 0.420364,
      0.419154, 0.417934, 0.416679, 0.415404, 0.414084, 0.412726, 0.411335,
      0.409985, 0.408549, 0.407062, 0.405618, 0.404111, 0.402619, 0.401104,
      0.399550, 0.397963, 0.396417, 0.394874, 0.393318, 0.391763, 0.390234,
      0.388733, 0.387203, 0.385703, 0.384315, 0.382928, 0.381560, 0.380326,
      0.379107, 0.377986, 0.376930, 0.376015, 0.375191, 0.374449, 0.373830,
      0.373453, 0.373188, 0.373099, 0.373185, 0.373426, 0.373797, 0.374444,
      0.375182, 0.376011, 0.376935, 0.377994, 0.379069, 0.380138, 0.381177,
      0.382161, 0.383081, 0.383876, 0.384604, 0.385214, 0.385721, 0.386073,
      0.386374, 0.386638, 0.386837, 0.386996, 0.387129, 0.387213, 0.387298,
      0.387341, 0.387371, 0.387401, 0.387460, 0.387525, 0.387590, 0.387662,
      0.387741, 0.387820, 0.387899, 0.387972, 0.388043, 0.388114, 0.388184,
      0.388267, 0.388361, 0.388455, 0.388550, 0.388644, 0.388708, 0.388766,
      0.388824, 0.388883, 0.388941, 0.389000, 0.389060, 0.389120, 0.389179,
      0.389239, 0.389296, 0.389345, 0.389395, 0.389445, 0.389494, 0.389544,
      0.389591, 0.389631, 0.389671, 0.389711, 0.389751, 0.389791, 0.389846,
      0.389906, 0.389965, 0.390025, 0.390085, 0.390096, 0.390096, 0.390096,
      0.390096, 0.390101, 0.390119, 0.390138, 0.390156, 0.390173, 0.390173,
      0.390173, 0.390173, 0.390177, 0.390194, 0.390211, 0.390228, 0.390236,
      0.390236, 0.390236, 0.390236, 0.390236, 0.390236, 0.390236, 0.390212,
      0.390186, 0.390161, 0.390158, 0.390158, 0.390158, 0.390158, 0.390158,
      0.390158, 0.390158, 0.390158, 0.390158, 0.390158, 0.390169, 0.390206,
      0.390244, 0.390247, 0.390247, 0.390247, 0.390247, 0.390247, 0.390247,
      0.390259, 0.390291, 0.390326, 0.390365, 0.390388, 0.390388, 0.390388,
      0.390388, 0.390388, 0.390388, 0.390388, 0.390388, 0.390388, 0.390388,
      0.390388, 0.390388, 0.390388, 0.390388, 0.390388, 0.390388, 0.390388,
      0.390388, 0.390388, 0.390368, 0.390311, 0.390311, 0.390311, 0.390311,
      0.390311, 0.390311, 0.390311, 0.390311, 0.390311, 0.390311, 0.390279,
      0.390234, 0.390234, 0.390234, 0.390234, 0.390234, 0.390215, 0.390157,
      0.390157, 0.390157, 0.390157, 0.390108, 0.390080, 0.390080, 0.390080,
      0.390056, 0.390003, 0.390003, 0.390003, 0.389976, 0.389926, 0.389926,
      0.389902, 0.389803, 0.389785, 0.389785, 0.389708, 0.389708, 0.389698,
      0.389631, 0.389631, 0.389563, 0.389554, 0.389538, 0.389477, 0.389477,
      0.389400, 0.389336, 0.389259, 0.389259, 0.389194, 0.389182, 0.389123,
      0.389105, 0.389041, 0.388964, 0.388887, 0.388887, 0.388810, 0.388810,
      0.388733, 0.388657, 0.388612, 0.388516, 0.388470, 0.388439, 0.388362,
      0.388285, 0.388285, 0.388144, 0.388067, 0.387990, 0.387950, 0.387878,
      0.387774, 0.387696, 0.387637, 0.387576, 0.387515, 0.387442, 0.387325,
      0.387260, 0.387161, 0.387048, 0.386940, 0.386884, 0.386814, 0.386669,
      0.386553, 0.386482, 0.386359, 0.386244, 0.386165, 0.386067, 0.385927,
      0.385769, 0.385697, 0.385547, 0.385437, 0.385362, 0.385240, 0.385062,
      0.384972, 0.384863, 0.384709, 0.384523, 0.384405, 0.384284, 0.384136,
      0.383973, NAN,      NAN,      0.449279, 0.449290, 0.449352, 0.449400,
      0.449416, 0.449472, 0.449500, 0.449572, 0.449643, 0.449699, 0.449706,
      0.449763, 0.449839, 0.449900, 0.449933, 0.449989, 0.450000, 0.450053,
      0.450119, 0.450200, 0.450222, 0.450279, 0.450300, 0.450384, 0.450497,
      0.450505, 0.450561, 0.450600, 0.450647, 0.450730, 0.450786, 0.450800,
      0.450848, 0.450900, 0.450921, 0.451017, 0.451072, 0.451100, 0.451134,
      0.451190, 0.451200, 0.451300, 0.451300, 0.451325, 0.451400, 0.451423,
      0.451478, 0.451500, 0.451539, 0.451594, 0.451600, 0.451604, 0.451659,
      0.451700, 0.451738, 0.451800, 0.451800, 0.451867, 0.451900, 0.451900,
      0.451996, 0.452003, 0.452060, 0.452100, 0.452100, 0.452128, 0.452200,
      0.452200, 0.452274, 0.452300, 0.452330, 0.452400, 0.452400, 0.452478,
      0.452501, 0.452533, 0.452606, 0.452615, 0.452628, 0.452695, 0.452757,
      0.452792, 0.452856, 0.452897, 0.452942, 0.452992, 0.453024, 0.453003,
      0.453052, 0.453126, 0.453158, 0.453175, 0.453275, 0.453277, 0.453300,
      0.453327, 0.453312, 0.453366, 0.453406, 0.453394, 0.453377, 0.453460,
      0.453440, 0.453432, 0.453425, 0.453433, 0.453428, 0.453417, 0.453408,
      0.453394, 0.453372, 0.453363, 0.453313, 0.453230, 0.453180, 0.453149,
      0.453115, 0.453094, 0.453020, 0.452961, 0.452915, 0.452778, 0.452715,
      0.452607, 0.452536, 0.452472, 0.452298, 0.452208, 0.452125, 0.452036,
      0.451892, 0.451754, 0.451558, 0.451381, 0.451237, 0.451068, 0.450894,
      0.450688, 0.450489, 0.450311, 0.450136, 0.449893, 0.449617, 0.449377,
      0.449144, 0.448896, 0.448613, 0.448317, 0.447982, 0.447646, 0.447404,
      0.447045, 0.446657, 0.446291, 0.445900, 0.445497, 0.445087, 0.444679,
      0.444229, 0.443757, 0.443281, 0.442741, 0.442209, 0.441734, 0.441195,
      0.440623, 0.440006, 0.439400, 0.438739, 0.438069, 0.437370, 0.436651,
      0.435933, 0.435180, 0.434405, 0.433614, 0.432769, 0.431945, 0.431035,
      0.430088, 0.429111, 0.428139, 0.427144, 0.426082, 0.425031, 0.423980,
      0.422753, 0.421612, 0.420391, 0.419171, 0.417918, 0.416572, 0.415239,
      0.413814, 0.412319, 0.410837, 0.409355, 0.407818, 0.406192, 0.404526,
      0.402842, 0.401152, 0.399356, 0.397545, 0.395693, 0.393802, 0.391896,
      0.389889, 0.387890, 0.385905, 0.383900, 0.381788, 0.379727, 0.377655,
      0.375546, 0.373428, 0.371350, 0.369287, 0.367292, 0.365280, 0.363257,
      0.361318, 0.359493, 0.357715, 0.356022, 0.354470, 0.353004, 0.351741,
      0.350795, 0.350055, 0.349576, 0.349258, 0.349416, 0.349896, 0.350524,
      0.351494, 0.352717, 0.353998, 0.355490, 0.356910, 0.358293, 0.359661,
      0.360834, 0.361829, 0.362711, 0.363376, 0.363844, 0.364207, 0.364517,
      0.364636, 0.364749, 0.364786, 0.364823, 0.364810, 0.364778, 0.364755,
      0.364801, 0.364848, 0.364891, 0.364916, 0.364941, 0.364966, 0.365000,
      0.365036, 0.365073, 0.365109, 0.365161, 0.365213, 0.365265, 0.365317,
      0.365366, 0.365414, 0.365461, 0.365509, 0.365556, 0.365599, 0.365643,
      0.365686, 0.365729, 0.365773, 0.365788, 0.365803, 0.365818, 0.365833,
      0.365848, 0.365856, 0.365856, 0.365856, 0.365856, 0.365856, 0.365856,
      0.365866, 0.365887, 0.365909, 0.365931, 0.365952, 0.365974, 0.365995,
      0.365995, 0.365995, 0.365995, 0.365995, 0.365995, 0.365995, 0.365995,
      0.365995, 0.365995, 0.365995, 0.365995, 0.365995, 0.365993, 0.365978,
      0.365962, 0.365947, 0.365932, 0.365917, 0.365912, 0.365912, 0.365912,
      0.365912, 0.365912, 0.365934, 0.365965, 0.365996, 0.366026, 0.366051,
      0.366051, 0.366051, 0.366051, 0.366051, 0.366051, 0.366051, 0.366051,
      0.366051, 0.366051, 0.366051, 0.366051, 0.366057, 0.366068, 0.366080,
      0.366091, 0.366091, 0.366091, 0.366091, 0.366108, 0.366127, 0.366145,
      0.366148, 0.366148, 0.366148, 0.366148, 0.366148, 0.366148, 0.366148,
      0.366148, 0.366148, 0.366148, 0.366148, 0.366148, 0.366148, 0.366148,
      0.366148, 0.366125, 0.366087, 0.366066, 0.366066, 0.366066, 0.366066,
      0.366066, 0.366066, 0.366049, 0.366005, 0.365983, 0.365983, 0.365983,
      0.365983, 0.365983, 0.365970, 0.365920, 0.365901, 0.365901, 0.365901,
      0.365876, 0.365821, 0.365819, 0.365819, 0.365815, 0.365756, 0.365737,
      0.365723, 0.365680, 0.365617, 0.365597, 0.365568, 0.365515, 0.365515,
      0.365515, 0.365457, 0.365433, 0.365393, 0.365351, 0.365351, 0.365329,
      0.365268, 0.365268, 0.365251, 0.365186, 0.365149, 0.365047, 0.365014,
      0.364965, 0.364965, 0.364903, 0.364883, 0.364800, 0.364800, 0.364758,
      0.364718, 0.364636, 0.364636, 0.364546, 0.364497, 0.364415, 0.364353,
      0.364332, 0.364282, 0.364243, 0.364168, 0.364159, 0.364086, 0.363947,
      0.363875, 0.363865, 0.363782, 0.363709, 0.363648, 0.363561, 0.363479,
      0.363397, 0.363321, 0.363315, 0.363233, 0.363151, 0.363069, 0.362929,
      0.362855, 0.362765, 0.362748, 0.362649, 0.362489, 0.362383, 0.362355,
      0.362242, 0.362134, 0.362052, 0.361912, 0.361830, 0.361727, 0.361667,
      0.361476, 0.361363, 0.361281, 0.361195, 0.360981, 0.360901, 0.360814,
      0.360650, 0.360510, 0.360421, 0.360250, 0.360114, 0.359974, 0.359823,
      0.359672, 0.359522, 0.359361, 0.359207, 0.359076, 0.358898, 0.358681,
      0.358550, 0.358372, 0.358174, 0.357976, 0.357791, 0.357611, 0.357414,
      0.357225, 0.357011, 0.356784, 0.356587, 0.356339, 0.356124, 0.355908,
      0.355677, 0.355412, 0.355160, 0.354919, 0.354646, 0.354361, NAN,
      NAN,      0.449567, 0.449572, 0.449600, 0.449635, 0.449692, 0.449700,
      0.449805, 0.449861, 0.449900, 0.449924, 0.449980, 0.450000, 0.450000,
      0.450049, 0.450100, 0.450111, 0.450167, 0.450246, 0.450300, 0.450335,
      0.450391, 0.450400, 0.450453, 0.450500, 0.450530, 0.450600, 0.450600,
      0.450666, 0.450739, 0.450795, 0.450800, 0.450807, 0.450863, 0.450937,
      0.451024, 0.451080, 0.451100, 0.451100, 0.451148, 0.451207, 0.451300,
      0.451300, 0.451342, 0.451400, 0.451432, 0.451488, 0.451587, 0.451600,
      0.451609, 0.451700, 0.451715, 0.451771, 0.451800, 0.451847, 0.451900,
      0.451900, 0.451909, 0.451987, 0.452003, 0.452011, 0.452052, 0.452130,
      0.452173, 0.452202, 0.452237, 0.452239, 0.452216, 0.452260, 0.452288,
      0.452301, 0.452321, 0.452362, 0.452349, 0.452432, 0.452474, 0.452458,
      0.452441, 0.452493, 0.452502, 0.452502, 0.452501, 0.452525, 0.452548,
      0.452558, 0.452531, 0.452529, 0.452527, 0.452524, 0.452540, 0.452566,
      0.452589, 0.452571, 0.452556, 0.452510, 0.452458, 0.452485, 0.452432,
      0.452410, 0.452370, 0.452276, 0.452268, 0.452247, 0.452177, 0.452118,
      0.452043, 0.451951, 0.451889, 0.451803, 0.451685, 0.451566, 0.451451,
      0.451332, 0.451208, 0.451088, 0.450970, 0.450787, 0.450594, 0.450435,
      0.450259, 0.450079, 0.449937, 0.449708, 0.449469, 0.449271, 0.449038,
      0.448789, 0.448479, 0.448196, 0.447945, 0.447695, 0.447353, 0.446988,
      0.446620, 0.446307, 0.445979, 0.445614, 0.445237, 0.444786, 0.444385,
      0.443952, 0.443462, 0.442988, 0.442459, 0.441987, 0.441468, 0.440904,
      0.440345, 0.439766, 0.439166, 0.438515, 0.437855, 0.437180, 0.436467,
      0.435724, 0.434994, 0.434229, 0.433424, 0.432591, 0.431751, 0.430890,
      0.429968, 0.429040, 0.428048, 0.427034, 0.425999, 0.424916, 0.423794,
      0.422593, 0.421460, 0.420259, 0.418976, 0.417643, 0.416343, 0.414949,
      0.413501, 0.412038, 0.410524, 0.408947, 0.407347, 0.405662, 0.403934,
      0.402190, 0.400388, 0.398556, 0.396633, 0.394631, 0.392602, 0.390547,
      0.388433, 0.386255, 0.384057, 0.381810, 0.379517, 0.377109, 0.374720,
      0.372274, 0.369724, 0.367146, 0.364571, 0.361894, 0.359200, 0.356524,
      0.353845, 0.351145, 0.348460, 0.345775, 0.343113, 0.340511, 0.338069,
      0.335652, 0.333422, 0.331364, 0.329496, 0.328152, 0.327234, 0.326771,
      0.326766, 0.327213, 0.328167, 0.329482, 0.331027, 0.332712, 0.334493,
      0.336227, 0.337781, 0.339237, 0.340470, 0.341338, 0.342008, 0.342556,
      0.342982, 0.343203, 0.343339, 0.343447, 0.343547, 0.343612, 0.343646,
      0.343680, 0.343708, 0.343736, 0.343762, 0.343762, 0.343762, 0.343762,
      0.343807, 0.343867, 0.343927, 0.343986, 0.343986, 0.343986, 0.343986,
      0.343986, 0.343986, 0.343986, 0.343986, 0.343986, 0.343990, 0.344008,
      0.344025, 0.344042, 0.344059, 0.344069, 0.344053, 0.344037, 0.344021,
      0.344006, 0.343990, 0.343975, 0.343960, 0.343945, 0.343931, 0.343916,
      0.343902, 0.343899, 0.343899, 0.343899, 0.343899, 0.343899, 0.343899,
      0.343896, 0.343891, 0.343885, 0.343880, 0.343874, 0.343869, 0.343863,
      0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862,
      0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862,
      0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862,
      0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862, 0.343862,
      0.343862, 0.343862, 0.343862, 0.343862, 0.343870, 0.343884, 0.343898,
      0.343912, 0.343926, 0.343928, 0.343928, 0.343928, 0.343928, 0.343928,
      0.343928, 0.343928, 0.343928, 0.343919, 0.343896, 0.343874, 0.343851,
      0.343842, 0.343842, 0.343842, 0.343842, 0.343842, 0.343842, 0.343842,
      0.343821, 0.343793, 0.343765, 0.343756, 0.343756, 0.343756, 0.343734,
      0.343704, 0.343673, 0.343669, 0.343669, 0.343662, 0.343628, 0.343594,
      0.343583, 0.343583, 0.343573, 0.343536, 0.343499, 0.343460, 0.343422,
      0.343410, 0.343410, 0.343386, 0.343344, 0.343311, 0.343285, 0.343249,
      0.343204, 0.343158, 0.343111, 0.343063, 0.343015, 0.343015, 0.343001,
      0.342950, 0.342897, 0.342844, 0.342842, 0.342821, 0.342764, 0.342707,
      0.342636, 0.342541, 0.342533, 0.342502, 0.342439, 0.342375, 0.342360,
      0.342329, 0.342261, 0.342192, 0.342122, 0.342102, 0.342063, 0.341974,
      0.341879, 0.341872, 0.341793, 0.341713, 0.341632, 0.341620, 0.341550,
      0.341464, 0.341335, 0.341311, 0.341232, 0.341139, 0.341052, 0.341035,
      0.340937, 0.340837, 0.340700, 0.340657, 0.340562, 0.340485, 0.340431,
      0.340319, 0.340260, 0.340115, 0.339976, 0.339942, 0.339819, 0.339694,
      0.339625, 0.339473, 0.339341, 0.339293, 0.339155, 0.339056, 0.338954,
      0.338765, 0.338660, 0.338553, 0.338400, 0.338264, 0.338121, 0.338004,
      0.337881, 0.337695, 0.337578, 0.337404, 0.337263, 0.337107, 0.336898,
      0.336797, 0.336607, 0.336448, 0.336250, 0.336125, 0.335915, 0.335729,
      0.335542, 0.335291, 0.335140, 0.334915, 0.334746, 0.334488, 0.334334,
      0.334054, 0.333852, 0.333603, 0.333371, 0.333150, 0.332889, 0.332650,
      0.332409, 0.332185, 0.331848, 0.331642, 0.331360, 0.331109, 0.330849,
      0.330490, 0.330196, 0.329922, 0.329618, 0.329247, 0.328940, 0.328646,
      0.328286, 0.327880, 0.327494, 0.327182, 0.326759, 0.326378, 0.325977,
      0.325543, 0.325158, 0.324701, 0.324230, 0.323778, 0.323331, 0.322840,
      0.322331, 0.321817, 0.321296, 0.320766, NAN,      NAN,      0.449800,
      0.449800, 0.449800, 0.449843, 0.449900, 0.450000, 0.450000, 0.450039,
      0.450100, 0.450132, 0.450188, 0.450200, 0.450251, 0.450300, 0.450300,
      0.450319, 0.450375, 0.450462, 0.450500, 0.450500, 0.450597, 0.450600,
      0.450600, 0.450632, 0.450700, 0.450700, 0.450767, 0.450800, 0.450845,
      0.450900, 0.450906, 0.450962, 0.451000, 0.451000, 0.451029, 0.451084,
      0.451100, 0.451191, 0.451200, 0.451214, 0.451300, 0.451301, 0.451350,
      0.451406, 0.451452, 0.451521, 0.451545, 0.451585, 0.451615, 0.451642,
      0.451702, 0.451769, 0.451814, 0.451814, 0.451818, 0.451870, 0.451949,
      0.451951, 0.451963, 0.451993, 0.452026, 0.452026, 0.452098, 0.452100,
      0.452099, 0.452105, 0.452116, 0.452151, 0.452148, 0.452129, 0.452212,
      0.452122, 0.452135, 0.452134, 0.452094, 0.452082, 0.452059, 0.452045,
      0.452046, 0.452023, 0.451979, 0.451925, 0.451885, 0.451823, 0.451756,
      0.451657, 0.451576, 0.451535, 0.451481, 0.451356, 0.451308, 0.451201,
      0.451095, 0.450995, 0.450882, 0.450740, 0.450602, 0.450467, 0.450353,
      0.450175, 0.449981, 0.449857, 0.449637, 0.449416, 0.449224, 0.449040,
      0.448807, 0.448634, 0.448390, 0.448121, 0.447856, 0.447633, 0.447319,
      0.447012, 0.446715, 0.446418, 0.446045, 0.445688, 0.445347, 0.444968,
      0.444555, 0.444182, 0.443754, 0.443284, 0.442813, 0.442361, 0.441893,
      0.441333, 0.440870, 0.440326, 0.439739, 0.439098, 0.438552, 0.437886,
      0.437215, 0.436540, 0.435806, 0.435110, 0.434390, 0.433628, 0.432769,
      0.431927, 0.431110, 0.430222, 0.429288, 0.428317, 0.427309, 0.426287,
      0.425254, 0.424200, 0.423111, 0.421924, 0.420734, 0.419441, 0.418155,
      0.416842, 0.415486, 0.414088, 0.412593, 0.411075, 0.409555, 0.407991,
      0.406314, 0.404553, 0.402784, 0.400989, 0.399147, 0.397181, 0.395196,
      0.393103, 0.390979, 0.388781, 0.386443, 0.384146, 0.381740, 0.379192,
      0.376603, 0.374011, 0.371357, 0.368525, 0.365651, 0.362733, 0.359658,
      0.356604, 0.353416, 0.350142, 0.346805, 0.343425, 0.339982, 0.336490,
      0.332980, 0.329465, 0.325960, 0.322363, 0.318908, 0.315553, 0.312400,
      0.309444, 0.306933, 0.304940, 0.303457, 0.302517, 0.302369, 0.302820,
      0.303910, 0.305314, 0.307042, 0.308894, 0.310773, 0.312531, 0.314143,
      0.315427, 0.316544, 0.317465, 0.318146, 0.318715, 0.319079, 0.319385,
      0.319573, 0.319741, 0.319816, 0.319892, 0.319947, 0.319994, 0.320041,
      0.320022, 0.319994, 0.319966, 0.319957, 0.319957, 0.319957, 0.319956,
      0.319933, 0.319911, 0.319889, 0.319866, 0.319866, 0.319866, 0.319866,
      0.319866, 0.319876, 0.319894, 0.319913, 0.319931, 0.319949, 0.319957,
      0.319957, 0.319957, 0.319957, 0.319957, 0.319958, 0.319966, 0.319973,
      0.319980, 0.319988, 0.319995, 0.320000, 0.320000, 0.320000, 0.320000,
      0.320000, 0.320000, 0.320000, 0.320000, 0.320000, 0.320000, 0.320000,
      0.320000, 0.320000, 0.319994, 0.319981, 0.319968, 0.319955, 0.319942,
      0.319929, 0.319917, 0.319909, 0.319909, 0.319909, 0.319909, 0.319909,
      0.319909, 0.319909, 0.319909, 0.319897, 0.319885, 0.319874, 0.319862,
      0.319850, 0.319839, 0.319827, 0.319811, 0.319788, 0.319764, 0.319740,
      0.319716, 0.319693, 0.319669, 0.319645, 0.319619, 0.319592, 0.319565,
      0.319538, 0.319511, 0.319484, 0.319457, 0.319442, 0.319427, 0.319412,
      0.319397, 0.319381, 0.319364, 0.319322, 0.319280, 0.319239, 0.319197,
      0.319155, 0.319130, 0.319112, 0.319093, 0.319075, 0.319056, 0.319054,
      0.319054, 0.319054, 0.319054, 0.319045, 0.319023, 0.319002, 0.318980,
      0.318957, 0.318934, 0.318910, 0.318887, 0.318863, 0.318837, 0.318812,
      0.318787, 0.318761, 0.318734, 0.318707, 0.318679, 0.318651, 0.318622,
      0.318593, 0.318563, 0.318533, 0.318512, 0.318512, 0.318512, 0.318496,
      0.318462, 0.318429, 0.318394, 0.318359, 0.318315, 0.318241, 0.318168,
      0.318121, 0.318083, 0.318035, 0.317976, 0.317920, 0.317878, 0.317836,
      0.317793, 0.317749, 0.317661, 0.317570, 0.317522, 0.317475, 0.317427,
      0.317378, 0.317328, 0.317268, 0.317191, 0.317129, 0.317076, 0.316968,
      0.316875, 0.316818, 0.316760, 0.316701, 0.316641, 0.316580, 0.316518,
      0.316454, 0.316312, 0.316190, 0.316124, 0.316055, 0.315986, 0.315881,
      0.315745, 0.315637, 0.315561, 0.315484, 0.315407, 0.315291, 0.315157,
      0.315075, 0.314985, 0.314863, 0.314772, 0.314598, 0.314508, 0.314412,
      0.314233, 0.314132, 0.313997, 0.313870, 0.313708, 0.313605, 0.313501,
      0.313356, 0.313153, 0.313043, 0.312879, 0.312727, 0.312554, 0.312358,
      0.312237, 0.312114, 0.311914, 0.311734, 0.311585, 0.311346, 0.311202,
      0.310980, 0.310790, 0.310641, 0.310411, 0.310232, 0.309977, 0.309759,
      0.309535, 0.309387, 0.309120, 0.308876, 0.308629, 0.308390, 0.308124,
      0.307895, 0.307578, 0.307355, 0.307167, 0.306841, 0.306556, 0.306223,
      0.305930, 0.305599, 0.305289, 0.304985, 0.304727, 0.304411, 0.304030,
      0.303681, 0.303325, 0.302934, 0.302532, 0.302192, 0.301805, 0.301456,
      0.300988, 0.300542, 0.300114, 0.299702, 0.299238, 0.298799, 0.298282,
      0.297750, 0.297277, 0.296800, 0.296252, 0.295641, 0.295150, 0.294582,
      0.294009, 0.293390, 0.292731, 0.292120, 0.291431, 0.290722, 0.290001,
      0.289255, 0.288470, 0.287704, 0.286959, 0.286134, 0.285238, 0.284409,
      0.283459, 0.282513, 0.281520, 0.280466, 0.279387, 0.278259, 0.277126,
      0.275930, 0.274661, 0.273401, NAN,      0.449900, 0.449900, 0.449944,
      0.450000, 0.450016, 0.450115, 0.450172, 0.450200, 0.450235, 0.450292,
      0.450300, 0.450400, 0.450400, 0.450436, 0.450500, 0.450500, 0.450574,
      0.450600, 0.450649, 0.450700, 0.450711, 0.450767, 0.450800, 0.450800,
      0.450835, 0.450891, 0.450900, 0.450952, 0.451001, 0.451004, 0.451031,
      0.451100, 0.451140, 0.451156, 0.451174, 0.451212, 0.451258, 0.451260,
      0.451280, 0.451317, 0.451317, 0.451354, 0.451361, 0.451410, 0.451452,
      0.451475, 0.451517, 0.451523, 0.451562, 0.451597, 0.451596, 0.451599,
      0.451616, 0.451649, 0.451642, 0.451621, 0.451610, 0.451612, 0.451613,
      0.451636, 0.451678, 0.451601, 0.451646, 0.451642, 0.451624, 0.451545,
      0.451559, 0.451525, 0.451490, 0.451414, 0.451325, 0.451267, 0.451258,
      0.451219, 0.451140, 0.451057, 0.450928, 0.450915, 0.450807, 0.450686,
      0.450513, 0.450416, 0.450306, 0.450174, 0.450021, 0.449855, 0.449685,
      0.449561, 0.449341, 0.449117, 0.448909, 0.448745, 0.448496, 0.448288,
      0.448056, 0.447799, 0.447513, 0.447222, 0.446954, 0.446656, 0.446326,
      0.445975, 0.445661, 0.445325, 0.444936, 0.444533, 0.444123, 0.443696,
      0.443260, 0.442832, 0.442405, 0.441917, 0.441415, 0.440904, 0.440375,
      0.439786, 0.439208, 0.438575, 0.438002, 0.437358, 0.436661, 0.435952,
      0.435234, 0.434538, 0.433803, 0.432998, 0.432169, 0.431308, 0.430438,
      0.429552, 0.428644, 0.427636, 0.426710, 0.425613, 0.424511, 0.423410,
      0.422303, 0.421110, 0.419895, 0.418633, 0.417309, 0.415953, 0.414584,
      0.413119, 0.411620, 0.410101, 0.408439, 0.406770, 0.405085, 0.403290,
      0.401412, 0.399505, 0.397567, 0.395561, 0.393450, 0.391234, 0.388976,
      0.386663, 0.384220, 0.381684, 0.379075, 0.376484, 0.373654, 0.370785,
      0.367852, 0.364765, 0.361599, 0.358370, 0.354901, 0.351382, 0.347838,
      0.344218, 0.340404, 0.336419, 0.332398, 0.328279, 0.324043, 0.319669,
      0.315369, 0.310972, 0.306535, 0.302136, 0.297891, 0.293747, 0.289879,
      0.286322, 0.283308, 0.280735, 0.279130, 0.278440, 0.278406, 0.278952,
      0.280122, 0.281727, 0.283639, 0.285680, 0.287710, 0.289632, 0.291452,
      0.293051, 0.294437, 0.295516, 0.296427, 0.297176, 0.297704, 0.298143,
      0.298462, 0.298711, 0.298917, 0.298988, 0.299059, 0.299131, 0.299202,
      0.299273, 0.299327, 0.299354, 0.299381, 0.299407, 0.299431, 0.299455,
      0.299479, 0.299502, 0.299464, 0.299421, 0.299378, 0.299335, 0.299306,
      0.299287, 0.299267, 0.299248, 0.299229, 0.299210, 0.299193, 0.299175,
      0.299157, 0.299140, 0.299123, 0.299106, 0.299090, 0.299073, 0.299057,
      0.299041, 0.298979, 0.298903, 0.298827, 0.298752, 0.298676, 0.298600,
      0.298523, 0.298447, 0.298370, 0.298293, 0.298217, 0.298140, 0.298062,
      0.297949, 0.297836, 0.297724, 0.297611, 0.297499, 0.297386, 0.297275,
      0.297179, 0.297083, 0.296988, 0.296892, 0.296796, 0.296700, 0.296604,
      0.296503, 0.296401, 0.296298, 0.296195, 0.296093, 0.295990, 0.295888,
      0.295791, 0.295728, 0.295665, 0.295601, 0.295538, 0.295475, 0.295411,
      0.295348, 0.295291, 0.295257, 0.295223, 0.295188, 0.295154, 0.295120,
      0.295086, 0.295052, 0.295018, 0.294980, 0.294942, 0.294905, 0.294867,
      0.294829, 0.294792, 0.294754, 0.294718, 0.294684, 0.294650, 0.294616,
      0.294582, 0.294548, 0.294514, 0.294482, 0.294450, 0.294418, 0.294386,
      0.294354, 0.294322, 0.294286, 0.294251, 0.294215, 0.294180, 0.294145,
      0.294124, 0.294104, 0.294085, 0.294065, 0.294046, 0.294024, 0.294003,
      0.293982, 0.293961, 0.293923, 0.293877, 0.293831, 0.293785, 0.293737,
      0.293688, 0.293639, 0.293590, 0.293551, 0.293515, 0.293478, 0.293436,
      0.293380, 0.293324, 0.293268, 0.293217, 0.293167, 0.293116, 0.293055,
      0.292992, 0.292929, 0.292863, 0.292796, 0.292730, 0.292682, 0.292634,
      0.292576, 0.292503, 0.292429, 0.292352, 0.292275, 0.292213, 0.292172,
      0.292129, 0.292046, 0.291962, 0.291875, 0.291787, 0.291686, 0.291578,
      0.291508, 0.291461, 0.291381, 0.291284, 0.291220, 0.291170, 0.291071,
      0.290967, 0.290858, 0.290749, 0.290637, 0.290509, 0.290376, 0.290317,
      0.290228, 0.290106, 0.289981, 0.289854, 0.289726, 0.289593, 0.289444,
      0.289301, 0.289231, 0.289100, 0.288955, 0.288808, 0.288659, 0.288506,
      0.288327, 0.288203, 0.288089, 0.287925, 0.287758, 0.287586, 0.287379,
      0.287201, 0.287021, 0.286835, 0.286641, 0.286443, 0.286241, 0.286036,
      0.285924, 0.285711, 0.285494, 0.285272, 0.285049, 0.284819, 0.284587,
      0.284350, 0.284107, 0.283860, 0.283609, 0.283352, 0.283091, 0.282826,
      0.282556, 0.282280, 0.282001, 0.281712, 0.281409, 0.281112, 0.280809,
      0.280415, 0.280137, 0.279772, 0.279446, 0.279114, 0.278748, 0.278337,
      0.277986, 0.277628, 0.277171, 0.276799, 0.276381, 0.275941, 0.275510,
      0.275055, 0.274592, 0.274135, 0.273616, 0.273092, 0.272623, 0.272106,
      0.271549, 0.270990, 0.270424, 0.269824, 0.269189, 0.268570, 0.267973,
      0.267305, 0.266614, 0.265908, 0.265187, 0.264450, 0.263698, 0.262929,
      0.262048, 0.261246, 0.260332, 0.259489, 0.258541, 0.257573, 0.256590,
      0.255585, 0.254498, 0.253456, 0.252395, 0.251318, 0.250222, 0.249210,
      0.248098, 0.247094, 0.246005, 0.244938, 0.243920, 0.242995, 0.242235,
      0.241627, 0.240885, 0.239918, 0.238714, 0.237426, NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      0.450000, 0.450000, 0.450050, 0.450100, 0.450115,
      0.450172, 0.450200, 0.450274, 0.450300, 0.450302, 0.450408, 0.450465,
      0.450500, 0.450500, 0.450535, 0.450592, 0.450601, 0.450660, 0.450709,
      0.450739, 0.450812, 0.450864, 0.450904, 0.450954, 0.451007, 0.451008,
      0.451020, 0.451053, 0.451100, 0.451118, 0.451153, 0.451145, 0.451165,
      0.451228, 0.451254, 0.451225, 0.451297, 0.451296, 0.451295, 0.451305,
      0.451326, 0.451354, 0.451327, 0.451319, 0.451310, 0.451305, 0.451310,
      0.451332, 0.451297, 0.451262, 0.451240, 0.451219, 0.451161, 0.451145,
      0.451137, 0.451103, 0.450991, 0.450943, 0.450890, 0.450832, 0.450747,
      0.450680, 0.450593, 0.450522, 0.450471, 0.450355, 0.450233, 0.450143,
      0.450046, 0.449934, 0.449804, 0.449666, 0.449512, 0.449353, 0.449188,
      0.449035, 0.448878, 0.448667, 0.448429, 0.448198, 0.447999, 0.447760,
      0.447550, 0.447270, 0.446957, 0.446699, 0.446401, 0.446093, 0.445775,
      0.445456, 0.445096, 0.444744, 0.444382, 0.443971, 0.443546, 0.443121,
      0.442703, 0.442289, 0.441823, 0.441336, 0.440822, 0.440253, 0.439667,
      0.439079, 0.438479, 0.437865, 0.437222, 0.436592, 0.435896, 0.435181,
      0.434451, 0.433712, 0.432906, 0.432084, 0.431246, 0.430382, 0.429456,
      0.428515, 0.427569, 0.426527, 0.425457, 0.424408, 0.423312, 0.422181,
      0.420997, 0.419746, 0.418416, 0.417054, 0.415709, 0.414319, 0.412828,
      0.411276, 0.409767, 0.408122, 0.406416, 0.404656, 0.402822, 0.400932,
      0.398992, 0.397005, 0.394908, 0.392712, 0.390518, 0.388150, 0.385740,
      0.383279, 0.380683, 0.378027, 0.375234, 0.372348, 0.369340, 0.366202,
      0.362996, 0.359647, 0.356201, 0.352610, 0.348854, 0.344979, 0.340973,
      0.336824, 0.332491, 0.328038, 0.323472, 0.318798, 0.313934, 0.308969,
      0.303860, 0.298642, 0.293370, 0.287967, 0.282503, 0.277021, 0.271628,
      0.266489, 0.261876, 0.257859, 0.254723, 0.252507, 0.251423, 0.251454,
      0.252289, 0.253845, 0.256226, 0.259188, 0.262368, 0.265516, 0.268532,
      0.271292, 0.273448, 0.275337, 0.276565, 0.277513, 0.278285, 0.278696,
      0.278848, 0.278944, 0.278946, 0.278930, 0.278847, 0.278763, 0.278724,
      0.278687, 0.278640, 0.278576, 0.278511, 0.278414, 0.278268, 0.278121,
      0.277976, 0.277844, 0.277713, 0.277582, 0.277430, 0.277220, 0.277010,
      0.276800, 0.276594, 0.276444, 0.276295, 0.276145, 0.275995, 0.275819,
      0.275606, 0.275392, 0.275179, 0.274965, 0.274744, 0.274505, 0.274266,
      0.274027, 0.273788, 0.273549, 0.273349, 0.273159, 0.272970, 0.272781,
      0.272591, 0.272402, 0.272235, 0.272074, 0.271913, 0.271752, 0.271591,
      0.271429, 0.271297, 0.271209, 0.271121, 0.271033, 0.270945, 0.270858,
      0.270770, 0.270689, 0.270614, 0.270538, 0.270463, 0.270388, 0.270312,
      0.270237, 0.270166, 0.270100, 0.270034, 0.269969, 0.269903, 0.269837,
      0.269771, 0.269706, 0.269665, 0.269626, 0.269588, 0.269550, 0.269511,
      0.269473, 0.269434, 0.269396, 0.269360, 0.269323, 0.269287, 0.269251,
      0.269214, 0.269178, 0.269141, 0.269105, 0.269070, 0.269034, 0.268999,
      0.268964, 0.268928, 0.268893, 0.268858, 0.268822, 0.268785, 0.268747,
      0.268710, 0.268672, 0.268634, 0.268596, 0.268558, 0.268520, 0.268489,
      0.268460, 0.268431, 0.268402, 0.268373, 0.268343, 0.268314, 0.268258,
      0.268202, 0.268146, 0.268091, 0.268035, 0.267979, 0.267925, 0.267870,
      0.267816, 0.267762, 0.267707, 0.267660, 0.267620, 0.267580, 0.267540,
      0.267500, 0.267445, 0.267380, 0.267316, 0.267251, 0.267187, 0.267140,
      0.267092, 0.267045, 0.266997, 0.266925, 0.266850, 0.266774, 0.266699,
      0.266644, 0.266589, 0.266534, 0.266467, 0.266381, 0.266295, 0.266208,
      0.266146, 0.266084, 0.266021, 0.265932, 0.265834, 0.265737, 0.265662,
      0.265592, 0.265523, 0.265451, 0.265378, 0.265305, 0.265229, 0.265154,
      0.265050, 0.264931, 0.264817, 0.264733, 0.264650, 0.264560, 0.264469,
      0.264354, 0.264219, 0.264098, 0.264003, 0.263894, 0.263748, 0.263613,
      0.263511, 0.263407, 0.263303, 0.263171, 0.263007, 0.262841, 0.262673,
      0.262551, 0.262421, 0.262243, 0.262095, 0.261970, 0.261782, 0.261616,
      0.261486, 0.261290, 0.261086, 0.260880, 0.260727, 0.260534, 0.260335,
      0.260185, 0.259961, 0.259729, 0.259492, 0.259265, 0.259100, 0.258854,
      0.258599, 0.258393, 0.258172, 0.257904, 0.257630, 0.257368, 0.257140,
      0.256852, 0.256556, 0.256255, 0.255971, 0.255734, 0.255415, 0.255091,
      0.254760, 0.254426, 0.254083, 0.253803, 0.253446, 0.253084, 0.252716,
      0.252340, 0.251957, 0.251568, 0.251170, 0.250757, 0.250327, 0.249906,
      0.249479, 0.249043, 0.248598, 0.248145, 0.247654, 0.247163, 0.246610,
      0.246122, 0.245621, 0.245096, 0.244484, 0.243959, 0.243322, 0.242756,
      0.242202, 0.241539, 0.240870, 0.240250, 0.239556, 0.238870, 0.238201,
      0.237469, 0.236727, 0.235975, 0.235275, 0.234596, 0.233807, 0.233102,
      0.232289, 0.231559, 0.230743, 0.229988, 0.229316, 0.228620, 0.228091,
      0.227662, 0.227411, 0.227463, 0.227376, 0.227442, 0.227822, 0.227928,
      0.227902, 0.228530, 0.229495, 0.229410, 0.230238, 0.230742, 0.229785,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      0.450064, 0.450069, 0.450100, 0.450135, 0.450193, 0.450202, 0.450263,
      0.450310, 0.450323, 0.450376, 0.450459, 0.450510, 0.450510, 0.450526,
      0.450610, 0.450622, 0.450671, 0.450699, 0.450726, 0.450762, 0.450845,
      0.450860, 0.450870, 0.450897, 0.450946, 0.450989, 0.450995, 0.450994,
      0.451007, 0.451031, 0.451063, 0.451106, 0.451117, 0.451107, 0.451102,
      0.451100, 0.451038, 0.451074, 0.451053, 0.451035, 0.451000, 0.450936,
      0.450954, 0.450914, 0.450877, 0.450818, 0.450725, 0.450661, 0.450587,
      0.450524, 0.450482, 0.450359, 0.450295, 0.450182, 0.450062, 0.449968,
      0.449846, 0.449708, 0.449561, 0.449435, 0.449312, 0.449103, 0.448939,
      0.448768, 0.448566, 0.448405, 0.448222, 0.447986, 0.447738, 0.447507,
      0.447317, 0.447036, 0.446816, 0.446526, 0.446205, 0.445877, 0.445537,
      0.445224, 0.444902, 0.444567, 0.444181, 0.443770, 0.443385, 0.443004,
      0.442565, 0.442044, 0.441623, 0.441121, 0.440641, 0.440100, 0.439528,
      0.438943, 0.438384, 0.437815, 0.437201, 0.436535, 0.435814, 0.435126,
      0.434412, 0.433665, 0.432875, 0.432035, 0.431159, 0.430343, 0.429479,
      0.428534, 0.427522, 0.426480, 0.425446, 0.424388, 0.423282, 0.422120,
      0.420957, 0.419681, 0.418418, 0.417131, 0.415727, 0.414283, 0.412850,
      0.411313, 0.409732, 0.408123, 0.406412, 0.404657, 0.402858, 0.401000,
      0.399033, 0.397040, 0.394875, 0.392691, 0.390434, 0.388117, 0.385715,
      0.383145, 0.380520, 0.377768, 0.374934, 0.372036, 0.368897, 0.365685,
      0.362389, 0.358987, 0.355387, 0.351655, 0.347745, 0.343692, 0.339499,
      0.335155, 0.330549, 0.325865, 0.321013, 0.315918, 0.310609, 0.305042,
      0.299340, 0.293287, 0.287015, 0.280626, 0.273871, 0.267047, 0.260218,
      0.253490, 0.247039, 0.241194, 0.236138, 0.232035, 0.228831, 0.226707,
      0.226035, 0.226886, 0.228963, 0.232012, 0.235798, 0.240279, 0.244696,
      0.248763, 0.252505, 0.255232, 0.257368, 0.258310, 0.259233, 0.259454,
      0.259398, 0.259191, 0.258802, 0.258435, 0.258120, 0.257791, 0.257416,
      0.257040, 0.256734, 0.256441, 0.256135, 0.255731, 0.255326, 0.254924,
      0.254640, 0.254355, 0.254071, 0.253745, 0.253380, 0.253016, 0.252651,
      0.252262, 0.251857, 0.251453, 0.251048, 0.250657, 0.250309, 0.249961,
      0.249613, 0.249265, 0.248949, 0.248672, 0.248396, 0.248119, 0.247843,
      0.247577, 0.247357, 0.247138, 0.246918, 0.246698, 0.246478, 0.246298,
      0.246158, 0.246018, 0.245879, 0.245739, 0.245599, 0.245468, 0.245353,
      0.245237, 0.245121, 0.245006, 0.244890, 0.244774, 0.244706, 0.244648,
      0.244591, 0.244533, 0.244475, 0.244417, 0.244360, 0.244302, 0.244245,
      0.244188, 0.244131, 0.244073, 0.244016, 0.243959, 0.243912, 0.243873,
      0.243834, 0.243794, 0.243755, 0.243716, 0.243676, 0.243637, 0.243599,
      0.243561, 0.243524, 0.243486, 0.243448, 0.243410, 0.243372, 0.243334,
      0.243295, 0.243256, 0.243217, 0.243178, 0.243139, 0.243100, 0.243061,
      0.243022, 0.242977, 0.242929, 0.242882, 0.242835, 0.242788, 0.242741,
      0.242694, 0.242647, 0.242604, 0.242570, 0.242535, 0.242501, 0.242466,
      0.242432, 0.242397, 0.242363, 0.242328, 0.242281, 0.242229, 0.242177,
      0.242125, 0.242073, 0.242021, 0.241969, 0.241918, 0.241877, 0.241835,
      0.241793, 0.241751, 0.241710, 0.241668, 0.241623, 0.241573, 0.241524,
      0.241474, 0.241424, 0.241374, 0.241322, 0.241267, 0.241211, 0.241156,
      0.241100, 0.241045, 0.240984, 0.240923, 0.240862, 0.240801, 0.240739,
      0.240673, 0.240606, 0.240540, 0.240473, 0.240418, 0.240370, 0.240322,
      0.240275, 0.240213, 0.240135, 0.240058, 0.239980, 0.239899, 0.239816,
      0.239733, 0.239650, 0.239567, 0.239484, 0.239401, 0.239311, 0.239217,
      0.239123, 0.239042, 0.238976, 0.238909, 0.238825, 0.238720, 0.238615,
      0.238506, 0.238395, 0.238285, 0.238205, 0.238128, 0.238035, 0.237913,
      0.237791, 0.237665, 0.237537, 0.237408, 0.237275, 0.237144, 0.237014,
      0.236881, 0.236738, 0.236597, 0.236497, 0.236394, 0.236239, 0.236083,
      0.235924, 0.235762, 0.235596, 0.235428, 0.235257, 0.235082, 0.234906,
      0.234733, 0.234554, 0.234366, 0.234174, 0.233979, 0.233781, 0.233578,
      0.233372, 0.233162, 0.232947, 0.232735, 0.232513, 0.232211, 0.231966,
      0.231731, 0.231492, 0.231250, 0.231001, 0.230748, 0.230490, 0.230227,
      0.229928, 0.229595, 0.229318, 0.229036, 0.228748, 0.228447, 0.228057,
      0.227753, 0.227444, 0.227113, 0.226711, 0.226384, 0.226003, 0.225611,
      0.225265, 0.224913, 0.224454, 0.224052, 0.223619, 0.223241, 0.222835,
      0.222360, 0.221960, 0.221551, 0.221074, 0.220612, 0.220134, 0.219642,
      0.219049, 0.218539, 0.218074, 0.217601, 0.217119, 0.216728, 0.216328,
      0.215895, 0.215550, 0.215227, 0.214950, 0.214495, 0.214386, 0.214014,
      0.212873, 0.212979, 0.213188, 0.213584, 0.213507, 0.213858, 0.214159,
      0.213468, 0.214099, 0.214921, 0.215713, 0.216582, 0.217325, 0.217684,
      0.217487, 0.214682, NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      0.450094, 0.450098,
      0.450144, 0.450202, 0.450260, 0.450234, 0.450275, 0.450329, 0.450359,
      0.450359, 0.450439, 0.450443, 0.450458, 0.450489, 0.450539, 0.450581,
      0.450591, 0.450600, 0.450610, 0.450620, 0.450664, 0.450702, 0.450705,
      0.450699, 0.450703, 0.450696, 0.450679, 0.450671, 0.450651, 0.450629,
      0.450621, 0.450623, 0.450641, 0.450606, 0.450567, 0.450508, 0.450427,
      0.450424, 0.450365, 0.450265, 0.450160, 0.450124, 0.450024, 0.449905,
      0.449792, 0.449754, 0.449630, 0.449495, 0.449347, 0.449217, 0.449090,
      0.448876, 0.448723, 0.448550, 0.448318, 0.448123, 0.447974, 0.447704,
      0.447474, 0.447258, 0.447031, 0.446751, 0.446450, 0.446198, 0.445918,
      0.445560, 0.445216, 0.444867, 0.444511, 0.444150, 0.443816, 0.443411,
      0.442970, 0.442521, 0.442079, 0.441634, 0.441098, 0.440575, 0.440071,
      0.439557, 0.439013, 0.438347, 0.437793, 0.437183, 0.436531, 0.435870,
      0.435162, 0.434422, 0.433673, 0.432911, 0.432123, 0.431290, 0.430387,
      0.429477, 0.428565, 0.427598, 0.426568, 0.425510, 0.424476, 0.423390,
      0.422247, 0.421064, 0.419836, 0.418586, 0.417259, 0.415867, 0.414517,
      0.413011, 0.411515, 0.409962, 0.408322, 0.406602, 0.404835, 0.403026,
      0.401173, 0.399236, 0.397160, 0.395016, 0.392831, 0.390574, 0.388204,
      0.385764, 0.383148, 0.380506, 0.377768, 0.374925, 0.371878, 0.368780,
      0.365537, 0.362158, 0.358618, 0.354890, 0.351101, 0.347041, 0.342892,
      0.338555, 0.334023, 0.329221, 0.324198, 0.318988, 0.313480, 0.307705,
      0.301631, 0.295226, 0.288498, 0.281481, 0.274100, 0.266562, 0.258794,
      0.251043, 0.243453, 0.236152, 0.229212, 0.222920, 0.217427, 0.212793,
      0.209209, 0.206808, 0.205923, 0.206982, 0.209617, 0.213557, 0.218498,
      0.223991, 0.229252, 0.233763, 0.237610, 0.240089, 0.241119, 0.241823,
      0.241686, 0.241167, 0.240419, 0.239660, 0.238979, 0.238280, 0.237568,
      0.236941, 0.236388, 0.235833, 0.235272, 0.234711, 0.234134, 0.233543,
      0.232951, 0.232360, 0.231769, 0.231179, 0.230590, 0.230051, 0.229513,
      0.228974, 0.228444, 0.227930, 0.227416, 0.226902, 0.226482, 0.226108,
      0.225734, 0.225360, 0.225105, 0.224850, 0.224595, 0.224341, 0.224212,
      0.224098, 0.223985, 0.223871, 0.223775, 0.223712, 0.223649, 0.223586,
      0.223523, 0.223461, 0.223403, 0.223345, 0.223287, 0.223229, 0.223171,
      0.223117, 0.223063, 0.223009, 0.222956, 0.222902, 0.222848, 0.222801,
      0.222753, 0.222706, 0.222658, 0.222610, 0.222563, 0.222516, 0.222469,
      0.222422, 0.222374, 0.222327, 0.222280, 0.222233, 0.222191, 0.222149,
      0.222106, 0.222064, 0.222022, 0.221979, 0.221937, 0.221909, 0.221882,
      0.221855, 0.221828, 0.221801, 0.221774, 0.221748, 0.221720, 0.221692,
      0.221664, 0.221637, 0.221609, 0.221581, 0.221553, 0.221525, 0.221499,
      0.221474, 0.221449, 0.221425, 0.221400, 0.221375, 0.221350, 0.221325,
      0.221301, 0.221277, 0.221253, 0.221229, 0.221205, 0.221181, 0.221157,
      0.221132, 0.221109, 0.221085, 0.221062, 0.221038, 0.221015, 0.220991,
      0.220968, 0.220945, 0.220921, 0.220879, 0.220832, 0.220785, 0.220738,
      0.220691, 0.220644, 0.220597, 0.220550, 0.220505, 0.220464, 0.220423,
      0.220383, 0.220342, 0.220301, 0.220260, 0.220220, 0.220174, 0.220127,
      0.220081, 0.220035, 0.219988, 0.219942, 0.219900, 0.219864, 0.219827,
      0.219790, 0.219754, 0.219717, 0.219666, 0.219609, 0.219551, 0.219494,
      0.219437, 0.219387, 0.219348, 0.219309, 0.219269, 0.219230, 0.219169,
      0.219097, 0.219026, 0.218955, 0.218883, 0.218809, 0.218735, 0.218661,
      0.218585, 0.218506, 0.218426, 0.218347, 0.218265, 0.218179, 0.218094,
      0.218009, 0.217919, 0.217828, 0.217738, 0.217665, 0.217601, 0.217537,
      0.217491, 0.217461, 0.217432, 0.217356, 0.217249, 0.217142, 0.217031,
      0.216918, 0.216804, 0.216686, 0.216568, 0.216443, 0.216314, 0.216185,
      0.216056, 0.215927, 0.215831, 0.215741, 0.215619, 0.215479, 0.215336,
      0.215190, 0.215041, 0.214890, 0.214736, 0.214579, 0.214419, 0.214256,
      0.214090, 0.213922, 0.213748, 0.213574, 0.213395, 0.213212, 0.213028,
      0.212838, 0.212646, 0.212450, 0.212250, 0.212090, 0.212021, 0.211829,
      0.211614, 0.211396, 0.211173, 0.210889, 0.210614, 0.210379, 0.210139,
      0.209894, 0.209645, 0.209391, 0.209133, 0.208870, 0.208606, 0.208424,
      0.208150, 0.207867, 0.207747, 0.207497, 0.207197, 0.206892, 0.206582,
      0.206265, 0.205943, 0.205503, 0.204960, 0.204463, 0.203884, 0.203360,
      0.202795, 0.202298, 0.201924, 0.201603, 0.201357, 0.201140, 0.201109,
      0.201181, 0.201245, 0.201412, 0.201265, 0.201211, 0.201392, 0.201775,
      0.201539, 0.200949, 0.201180, 0.201664, 0.202295, 0.202991, 0.203697,
      0.204575, 0.205435, 0.206278, 0.207029, 0.207423, 0.206638, 0.202093,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      0.450141, 0.450142, 0.450157, 0.450187,
      0.450238, 0.450281, 0.450290, 0.450288, 0.450291, 0.450315, 0.450338,
      0.450322, 0.450309, 0.450349, 0.450379, 0.450300, 0.450313, 0.450348,
      0.450346, 0.450325, 0.450314, 0.450315, 0.450263, 0.450234, 0.450191,
      0.450138, 0.450121, 0.450046, 0.449972, 0.449909, 0.449861, 0.449828,
      0.449699, 0.449608, 0.449526, 0.449451, 0.449365, 0.449197, 0.449071,
      0.448988, 0.448879, 0.448705, 0.448513, 0.448336, 0.448168, 0.447989,
      0.447791, 0.447601, 0.447376, 0.447117, 0.446907, 0.446619, 0.446370,
      0.446105, 0.445813, 0.445527, 0.445203, 0.444867, 0.444500, 0.444124,
      0.443757, 0.443355, 0.442960, 0.442501, 0.442032, 0.441597, 0.441105,
      0.440639, 0.440130, 0.439583, 0.439053, 0.438456, 0.437807, 0.437188,
      0.436547, 0.435868, 0.435134, 0.434433, 0.433704, 0.432940, 0.432135,
      0.431295, 0.430419, 0.429498, 0.428569, 0.427616, 0.426599, 0.425593,
      0.424532, 0.423425, 0.422306, 0.421113, 0.419887, 0.418603, 0.417265,
      0.415913, 0.414490, 0.412993, 0.411485, 0.409918, 0.408249, 0.406528,
      0.404721, 0.402915, 0.401048, 0.399025, 0.396959, 0.394846, 0.392610,
      0.390337, 0.387936, 0.385436, 0.382901, 0.380145, 0.377325, 0.374362,
      0.371305, 0.368128, 0.364817, 0.361348, 0.357718, 0.353967, 0.349929,
      0.345800, 0.341411, 0.336859, 0.332063, 0.327007, 0.321592, 0.315975,
      0.310034, 0.303716, 0.297082, 0.290001, 0.282618, 0.274892, 0.266940,
      0.258791, 0.250530, 0.242407, 0.234486, 0.226834, 0.219477, 0.212562,
      0.206313, 0.200853, 0.196405, 0.193069, 0.191255, 0.191389, 0.193400,
      0.197060, 0.202229, 0.208225, 0.214208, 0.219349, 0.223605, 0.226031,
      0.227090, 0.227101, 0.226656, 0.225813, 0.224787, 0.223770, 0.222831,
      0.221919, 0.221116, 0.220309, 0.219485, 0.218661, 0.217722, 0.216772,
      0.215599, 0.214413, 0.213197, 0.211915, 0.210602, 0.209350, 0.208321,
      0.207537, 0.207101, 0.206954, 0.207091, 0.207348, 0.207639, 0.207891,
      0.208062, 0.208210, 0.208275, 0.208336, 0.208392, 0.208405, 0.208405,
      0.208405, 0.208405, 0.208405, 0.208405, 0.208405, 0.208405, 0.208405,
      0.208405, 0.208389, 0.208360, 0.208331, 0.208305, 0.208305, 0.208305,
      0.208305, 0.208305, 0.208305, 0.208305, 0.208305, 0.208305, 0.208287,
      0.208266, 0.208245, 0.208224, 0.208204, 0.208184, 0.208165, 0.208146,
      0.208127, 0.208108, 0.208090, 0.208072, 0.208055, 0.208037, 0.208019,
      0.208002, 0.207986, 0.207969, 0.207953, 0.207936, 0.207920, 0.207903,
      0.207873, 0.207842, 0.207811, 0.207781, 0.207750, 0.207719, 0.207697,
      0.207682, 0.207668, 0.207653, 0.207639, 0.207624, 0.207610, 0.207604,
      0.207604, 0.207604, 0.207604, 0.207604, 0.207604, 0.207604, 0.207596,
      0.207570, 0.207544, 0.207517, 0.207491, 0.207465, 0.207439, 0.207412,
      0.207387, 0.207361, 0.207336, 0.207311, 0.207286, 0.207260, 0.207235,
      0.207210, 0.207176, 0.207139, 0.207102, 0.207066, 0.207029, 0.206992,
      0.206956, 0.206919, 0.206883, 0.206848, 0.206812, 0.206776, 0.206741,
      0.206705, 0.206670, 0.206634, 0.206598, 0.206561, 0.206524, 0.206488,
      0.206451, 0.206414, 0.206377, 0.206340, 0.206303, 0.206261, 0.206218,
      0.206176, 0.206133, 0.206091, 0.206049, 0.206006, 0.205958, 0.205910,
      0.205862, 0.205815, 0.205767, 0.205719, 0.205654, 0.205583, 0.205512,
      0.205440, 0.205369, 0.205299, 0.205240, 0.205181, 0.205122, 0.205063,
      0.205004, 0.204982, 0.204961, 0.204939, 0.204918, 0.204879, 0.204809,
      0.204738, 0.204668, 0.204596, 0.204495, 0.204394, 0.204293, 0.204195,
      0.204114, 0.204032, 0.203951, 0.203867, 0.203781, 0.203694, 0.203607,
      0.203575, 0.203544, 0.203513, 0.203411, 0.203281, 0.203150, 0.203038,
      0.202934, 0.202831, 0.202723, 0.202615, 0.202506, 0.202391, 0.202277,
      0.202160, 0.202040, 0.201920, 0.201796, 0.201670, 0.201542, 0.201411,
      0.201288, 0.201197, 0.201106, 0.200964, 0.200822, 0.200675, 0.200528,
      0.200375, 0.200222, 0.200064, 0.199905, 0.199741, 0.199576, 0.199406,
      0.199234, 0.199059, 0.198880, 0.198702, 0.198578, 0.198424, 0.198232,
      0.198036, 0.197836, 0.197634, 0.197426, 0.197215, 0.197001, 0.196782,
      0.196508, 0.196231, 0.195998, 0.195683, 0.195362, 0.195036, 0.194704,
      0.194366, 0.194022, 0.193670, 0.193312, 0.193014, 0.192736, 0.192507,
      0.192316, 0.192121, 0.191972, 0.191873, 0.191825, 0.191830, 0.191835,
      0.191935, 0.191277, 0.190985, 0.190955, 0.191092, 0.191259, 0.191549,
      0.191908, 0.192282, 0.192832, 0.193475, 0.194128, 0.194858, 0.195663,
      0.196493, 0.197139, 0.197349, 0.196622, 0.192740, NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      0.450146, 0.450146, 0.450158, 0.450186, 0.450236, 0.450286,
      0.450291, 0.450284, 0.450284, 0.450310, 0.450327, 0.450310, 0.450295,
      0.450336, 0.450363, 0.450272, 0.450285, 0.450323, 0.450320, 0.450295,
      0.450279, 0.450276, 0.450219, 0.450188, 0.450139, 0.450082, 0.450065,
      0.449983, 0.449904, 0.449837, 0.449785, 0.449749, 0.449605, 0.449509,
      0.449422, 0.449345, 0.449259, 0.449074, 0.448941, 0.448860, 0.448751,
      0.448563, 0.448362, 0.448179, 0.448006, 0.447813, 0.447607, 0.447411,
      0.447179, 0.446907, 0.446689, 0.446393, 0.446134, 0.445861, 0.445563,
      0.445268, 0.444926, 0.444584, 0.444203, 0.443811, 0.443430, 0.443015,
      0.442611, 0.442131, 0.441644, 0.441201, 0.440694, 0.440216, 0.439692,
      0.439126, 0.438577, 0.437961, 0.437291, 0.436654, 0.435994, 0.435291,
      0.434537, 0.433819, 0.433068, 0.432279, 0.431447, 0.430589, 0.429682,
      0.428729, 0.427773, 0.426790, 0.425742, 0.424710, 0.423618, 0.422476,
      0.421324, 0.420096, 0.418837, 0.417516, 0.416135, 0.414745, 0.413282,
      0.411742, 0.410186, 0.408571, 0.406849, 0.405075, 0.403209, 0.401348,
      0.399426, 0.397341, 0.395203, 0.393030, 0.390720, 0.388374, 0.385897,
      0.383319, 0.380708, 0.377857, 0.374940, 0.371874, 0.368719, 0.365440,
      0.362015, 0.358425, 0.354669, 0.350787, 0.346608, 0.342329, 0.337775,
      0.333052, 0.328081, 0.322830, 0.317197, 0.311357, 0.305176, 0.298599,
      0.291680, 0.284297, 0.276590, 0.268525, 0.260232, 0.251748, 0.243163,
      0.234749, 0.226586, 0.218747, 0.211261, 0.204296, 0.198095, 0.192791,
      0.188636, 0.185720, 0.184501, 0.185424, 0.188395, 0.193151, 0.199531,
      0.206756, 0.213886, 0.220005, 0.225044, 0.227954, 0.229206, 0.229113,
      0.228360, 0.227039, 0.225416, 0.223748, 0.222188, 0.220735, 0.219466,
      0.218331, 0.217322, 0.216344, 0.215325, 0.214333, 0.213117, 0.211888,
      0.210619, 0.209278, 0.207905, 0.206591, 0.205515, 0.204707, 0.204284,
      0.204178, 0.204387, 0.204729, 0.205108, 0.205445, 0.205691, 0.205913,
      0.206044, 0.206164, 0.206280, 0.206348, 0.206401, 0.206452, 0.206504,
      0.206555, 0.206597, 0.206635, 0.206672, 0.206709, 0.206735, 0.206742,
      0.206736, 0.206730, 0.206714, 0.206725, 0.206737, 0.206748, 0.206758,
      0.206764, 0.206770, 0.206776, 0.206783, 0.206770, 0.206753, 0.206735,
      0.206718, 0.206701, 0.206686, 0.206670, 0.206654, 0.206639, 0.206623,
      0.206609, 0.206595, 0.206580, 0.206565, 0.206551, 0.206536, 0.206523,
      0.206510, 0.206497, 0.206483, 0.206470, 0.206456, 0.206427, 0.206398,
      0.206369, 0.206340, 0.206310, 0.206281, 0.206260, 0.206248, 0.206237,
      0.206225, 0.206212, 0.206199, 0.206186, 0.206182, 0.206184, 0.206187,
      0.206190, 0.206192, 0.206195, 0.206198, 0.206192, 0.206166, 0.206140,
      0.206114, 0.206088, 0.206061, 0.206035, 0.206009, 0.205983, 0.205958,
      0.205932, 0.205907, 0.205882, 0.205856, 0.205831, 0.205805, 0.205770,
      0.205732, 0.205695, 0.205657, 0.205619, 0.205581, 0.205543, 0.205505,
      0.205468, 0.205431, 0.205394, 0.205357, 0.205320, 0.205284, 0.205249,
      0.205214, 0.205180, 0.205144, 0.205108, 0.205072, 0.205036, 0.205000,
      0.204964, 0.204928, 0.204891, 0.204848, 0.204806, 0.204763, 0.204721,
      0.204678, 0.204636, 0.204594, 0.204546, 0.204498, 0.204450, 0.204402,
      0.204353, 0.204304, 0.204237, 0.204162, 0.204088, 0.204013, 0.203939,
      0.203868, 0.203809, 0.203749, 0.203690, 0.203630, 0.203569, 0.203550,
      0.203530, 0.203510, 0.203493, 0.203457, 0.203387, 0.203317, 0.203247,
      0.203174, 0.203071, 0.202967, 0.202864, 0.202764, 0.202682, 0.202601,
      0.202520, 0.202436, 0.202349, 0.202262, 0.202176, 0.202149, 0.202125,
      0.202098, 0.201992, 0.201855, 0.201716, 0.201595, 0.201484, 0.201378,
      0.201271, 0.201162, 0.201053, 0.200939, 0.200824, 0.200708, 0.200588,
      0.200468, 0.200344, 0.200219, 0.200091, 0.199960, 0.199834, 0.199743,
      0.199655, 0.199513, 0.199371, 0.199224, 0.199076, 0.198924, 0.198771,
      0.198612, 0.198454, 0.198290, 0.198124, 0.197955, 0.197783, 0.197607,
      0.197428, 0.197251, 0.197133, 0.196983, 0.196791, 0.196594, 0.196395,
      0.196188, 0.195966, 0.195754, 0.195540, 0.195320, 0.195041, 0.194766,
      0.194536, 0.194213, 0.193884, 0.193550, 0.193210, 0.192864, 0.192510,
      0.192150, 0.191783, 0.191473, 0.191195, 0.190971, 0.190773, 0.190584,
      0.190449, 0.190371, 0.190350, 0.190386, 0.190424, 0.190578, 0.189909,
      0.189637, 0.189662, 0.189865, 0.190105, 0.190474, 0.190907, 0.191349,
      0.191980, 0.192708, 0.193430, 0.194226, 0.195105, 0.196001, 0.196726,
      0.196963, 0.196145, 0.191837, NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,      NAN,
      NAN,      NAN,      NAN,      NAN,      NAN,      NAN};
